------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
-----------------------------------------------------------------------------
-----------------------------------------------------------------------------

with Framework_Config; use Framework_Config;
with Events;           use Events;

package Editor_Config is

   pragma Elaborate_Body;

   -- Display all Editor configuration variables
   --
   procedure Put;

   -- Maximum number of time line displayed at simulation
   -- If the number of tasks/messages/buffers/resources in
   -- a project is greater that this constant, the user have
   -- to choose which object the simulator engine has to display
   --
   Max_Time_Line_To_Display : constant Natural := 30;

   -- Maximum number of time line which can de computed
   -- All these time line can not be displayed
   --
   Max_Time_Line : constant Natural :=
      Max_Tasks + Max_Messages + Max_Resources + Max_Buffers;

   ----------------------------------------------------------------------------
   ------
   -- options tuned by the user AT RUNTIME : then, these options are not
   --constant
   ----------------------------------------------------------------------------
   ------
   -- Each item of this boolean table store the
   -- user's choose on which event the simulator engine
   -- will generate
   --
   Generate_Events : Time_Unit_Event_Type_Boolean_Table := (others => True);

   -- should we do simulations on all processors ?
   --
   Schedule_All_Processeurs : Boolean := False;

   -- should we display the event table after simulation ?
   --
   Schedule_And_Display_Event_Table : Boolean := False;

   -- should we save the event table after simulation ?
   --
   Schedule_And_Export_Event_Table : Boolean := False;

   -- should we do simulations according to offset ?
   --
   Schedule_With_Offsets : Boolean := True;

   -- should we do simulations according to shared resources ?
   --
   Schedule_With_Resources : Boolean := True;

   -- should we do simulations according to task precedencies ?
   --
   Schedule_With_Precedencies : Boolean := True;

   -- should we use only one seed to all random values
   -- generated during simulations ?
   --
   Has_Global_Seed : Boolean := True;

   -- should we use seed with a predictable value to be able
   -- to do predictable simulation  ?
   --
   Is_Global_Predictable_Seed : Boolean := False;

   -- if Has_Global_Seed and Is_Global_Predictable_Seed
   -- are True, Global_Seed_Value store the seed
   --
   Global_Seed_Value : Integer := 0;

   type Data_From_Simulation is (
      Worst_Case,
      Best_Case,
      Average_Case,
      Plot_Case);
   type Data_From_Simulation_Table is
     array (Data_From_Simulation'Range) of Boolean;

   procedure Initialize
     (B     : in out Data_From_Simulation_Table;
      Value : in Boolean);

   -- should we compute, from a simulation worst/best/average/plot cases
   -- from response time of blocking time
   --
   Response_Time_From_Simulation : Data_From_Simulation_Table;
   Blocking_Time_From_Simulation : Data_From_Simulation_Table;

   -- should we compute these data during the simulation ?
   --
   Context_Switch_From_Simulation     : Boolean := True;
   Preemption_From_Simulation         : Boolean := True;
   Run_Event_Analyzer_From_Simulation : Boolean := True;

end Editor_Config;
