------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Call_Framework;
USE Call_Framework;
WITH Framework_Config;
USE Framework_Config;
WITH Systems;
USE Systems;
WITH Task_Set;
USE Task_Set;
WITH Processors;
USE Processors;
WITH Ada.Strings.Unbounded;
USE  Ada.Strings.Unbounded;
with Buffers;          use Buffers;
use Buffers.Buffer_Roles;
with Scheduling_Sequences; use Scheduling_Sequences;
use Scheduling_Sequences.Buffer_Size_Package;
use Scheduling_Sequences.Waiting_Time_Package;
WITH Indexed_Tables;


PACKAGE Call_Memory_Framework IS


   PACKAGE Buffer_Result_Package IS NEW Indexed_Tables(
      Buffer_Size_Table, Buffer_Ptr, Framework_Config.Max_Buffers, 0, Put,
      Initialize, Buffers.Put_Name);
   USE Buffer_Result_Package;

   SUBTYPE Buffer_Result_Table IS Buffer_Result_Package.
      Indexed_Table;
   SUBTYPE Buffer_Result_Range IS Buffer_Result_Package.
      Indexed_Table_Range;


   -- Last computed buffer size
   --
   Buff : Buffer_Result_Table;



   -- Raised if periodic producer/consumer tasks are expected
   --
   Task_Must_Be_Periodic : EXCEPTION;


   -- Check if producers/consumers are periodic
   --
   PROCEDURE Periodic_Buffer_Control (
         My_Tasks : IN     Tasks_Set;
         A_Buffer : IN     Buffer_Ptr);



   -- Compute global memory requirement (stack, buffer, text, ....)
   --
   PROCEDURE Compute_Memory_Requirement_Analysis (
         Sys                       : IN     System;
         Result                    : IN OUT Unbounded_String;
         Update_Address_Spaces_Set : IN     Boolean;
         Output                    : IN     Output_Format    := String_Output);


   -- Perform analysis on buffer requirement from a scheduling
   -- simulation
   --
   PROCEDURE Compute_Buffer_Scheduling_Simulation (
         Sys         : IN     System;
         Result      : IN OUT Unbounded_String;
         A_Processor : IN     Processor_Ptr;
         Output      : IN     Output_Format    := String_Output);


   -- Perform analysis on buffer requirement from analytical
   -- equation
   --
   PROCEDURE Compute_Buffer_Feasibility_Tests (
         Sys         : IN     System;
         Result      : IN OUT Unbounded_String;
         A_Processor : IN     Processor_Ptr;
         Output      : IN     Output_Format    := String_Output);




END Call_Memory_Framework;









