------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Framework_Config;          use Framework_Config;
with Translate;                 use Translate;
with Resource_Set;              use Resource_Set;
use Resource_Set.Generic_Resource_Set;
with Task_Set;                  use Task_Set;
use Task_Set.Generic_Task_Set;
with Scheduling_Sequences;      use Scheduling_Sequences;
use Scheduling_Sequences.Densities_Table_Package;
use Scheduling_Sequences.Density_Table_Package;
with Buffer_Set;                use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Double_Util;               use Double_Util;
with Unbounded_Strings;         use Unbounded_Strings;
with Message_Set;               use Message_Set;
use Message_Set.Generic_Message_Set;
with Call_Scheduling_Framework; use Call_Scheduling_Framework;
with Multiprocessor_Services;   use Multiprocessor_Services;
use Multiprocessor_Services.Scheduling_Result_Per_Processor;
with Xml_Tag;                   use Xml_Tag;

package body Call_Random_Framework is

   procedure Compute_Response_Time_Density
     (Sys         : in System;
      Result      : in out Unbounded_String;
      A_Processor : in Processor_Ptr;
      Output      : in Output_Format := String_Output)
   is

      Proba : Densities_Table;

   begin
      Result := To_Unbounded_String ("");

      if Output = Xml_Output then
         Set_Tag;
      else
         Set_Empty;
      end if;

      for I in  0 .. Call_Scheduling_Framework.Sched.Nb_Entries - 1 loop

         if Call_Scheduling_Framework.Sched.Entries (I).Item =
            A_Processor
         then

            Initialize (Proba);
            Compute_Response_Time_Distribution
              (Sys.Tasks,
               Call_Scheduling_Framework.Sched.Entries (I).Data.Result,
               Call_Scheduling_Framework.Sched.Entries (I).Item.Name,
               Proba);
            for I in  0 .. Proba.Nb_Entries - 1 loop
               Result :=
                 Result &
                 Lb_Minus &
                 Lb_Task (Current_Language) &
                 Unbounded_Sp &
                 Start_Task &
                 To_String (Proba.Entries (I).Item.Name) &
                 End_Task &
                 Lb_Colon &
                 Unbounded_Lf;
               for K in  0 .. Proba.Entries (I).Data.Nb_Entries - 1 loop
                  Result :=
                    Result &
                    To_Unbounded_String ("   ") &
                    To_Unbounded_String ("P(") &
                    Lb_Response_Time (Current_Language) &
                    Lb_Equal &
                    Start_Response_Time &
                    To_Unbounded_String
                       (Proba.Entries (I).Data.Entries (K).Response_Time'Img) &
                    End_Response_Time &
                    To_Unbounded_String (")=") &
                    Start_Probability &
                    Format (Proba.Entries (I).Data.Entries (K).Probability) &
                    End_Probability &
                    Unbounded_Lf;
               end loop;
            end loop;

         end if;

      end loop;

      Result := Result & End_Response_Time_Density;

   exception
      when Task_Set.Task_Must_Be_Periodic =>
         Result := Result &
                   Lb_Compute_Scheduling_Error_5 (Current_Language) &
                   Unbounded_Lf &
                   End_Response_Time_Density;
      when Task_Set.Task_Model_Error =>
         Result := Result &
                   Lb_Compute_Scheduling_Error_6 (Current_Language) &
                   Unbounded_Lf &
                   End_Response_Time_Density;

   end Compute_Response_Time_Density;

end Call_Random_Framework;
