------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Events;
USE Events;
WITH Systems;
USE Systems;
WITH Call_Framework;
USE Call_Framework;
WITH Processors;
USE Processors;
WITH Processor_Set;
USE Processor_Set;
USE Processor_Set.Generic_Processor_Set;
WITH Ada.Strings.Unbounded;
USE  Ada.Strings.Unbounded;
with Scheduler;                   use Scheduler;
use Scheduler.Double_Tasks_Parameters;
with Multiprocessor_Services; use Multiprocessor_Services;
use Multiprocessor_Services.Scheduling_Result_Per_Processor;
with Task_Set;           use Task_Set;
use Task_Set.Generic_Task_Set;



PACKAGE Call_Scheduling_Framework  IS


   PRAGMA Elaborate_Body;




   -- Last computed scheduling
   --
   Sched                  : Scheduling_Table_Ptr;
   Scheduling_Base_Period : Natural;





   PROCEDURE Compute_Simulation_Time_Line (
         Sys                        : IN OUT System;
         Result                     : IN OUT Unbounded_String;
         Period                     : IN     Natural;
         Schedule_With_Offsets      : IN     Boolean;
         Schedule_With_Precedencies : IN     Boolean;
         Schedule_With_Resources    : IN     Boolean;
         With_Task_Specific_Seed    : IN     Boolean                            := True;
         Global_Seed_Value          : IN     Integer                            := 0;
         Predictable_Global_Seed    : IN     Boolean                            := True;
         Event_To_Generate          : IN     Time_Unit_Event_Type_Boolean_Table;
         Output                     : IN     Output_Format                      := String_Output);


   PROCEDURE Compute_Simulation_Basics (
         Sys         : IN     System;
         Result      : IN OUT Unbounded_String;
         A_Processor : IN     Processor_Ptr;
         Output      : IN     Output_Format    := String_Output);


   PROCEDURE Compute_Simulation_Number_Of_Preemption (
         Sys         : IN     System;
         Result      : IN OUT Unbounded_String;
         A_Processor : IN     Processor_Ptr;
         Output      : IN     Output_Format    := String_Output);


   PROCEDURE Compute_Simulation_Number_Of_Context_Switch (
         Sys         : IN     System;
         Result      : IN OUT Unbounded_String;
         A_Processor : IN     Processor_Ptr;
         Output      : IN     Output_Format    := String_Output);


   PROCEDURE Compute_Simulation_Response_Time (
         Sys          : IN     System;
         Result       : IN OUT Unbounded_String;
         A_Processor  : IN     Processor_Ptr;
         Worst_Case   :        Boolean;
         Best_Case    :        Boolean;
         Average_Case :        Boolean;
         Output       : IN     Output_Format    := String_Output);


   PROCEDURE Run_User_Defined_Event_Handlers (
         Sys    : IN     System;
         Result : IN OUT Unbounded_String;
         Output : IN     Output_Format    := String_Output);


   PROCEDURE Compute_Feasibility_Basics (
         Sys         : IN OUT System;
         Result      : IN OUT Unbounded_String;
         A_Processor : IN     Processor_Ptr;
         Output      : IN     Output_Format    := String_Output);


   PROCEDURE Compute_Feasibility_Period (
         Sys         : IN     System;
         Result      : IN OUT Unbounded_String;
         A_Processor : IN     Processor_Ptr;
         Output      : IN     Output_Format    := String_Output);


   PROCEDURE Compute_Feasibility_Processor_Utilization_Factor (
         Sys         : IN     System;
         Result      : IN OUT Unbounded_String;
         A_Processor : IN     Processor_Ptr;
         Output      : IN     Output_Format    := String_Output);


   PROCEDURE Compute_Feasibility_Response_Time (
         Sys         : IN OUT System;
         Result      : IN OUT Unbounded_String;
         A_Processor : IN     Processor_Ptr;
         Output      : IN     Output_Format    := String_Output);


   PROCEDURE Set_Priorities (
         Sys         : IN OUT System;
         Result      : IN OUT Unbounded_String;
         A_Processor : IN     Processor_Ptr;
         A_Scheduler : IN     Scheduler_Type;
         Output      : IN     Output_Format    := String_Output);



   TYPE Partioning_Type IS
         (General_Task,
          Best_Fit,
          First_Fit,
          Next_Fit,
          Small_Task);


   PROCEDURE Partionning_Tasks (
         Sys             : IN OUT System;
         Result          : IN OUT Unbounded_String;
         A_Parition_Rule : IN     Partioning_Type;
         Output          : IN     Output_Format    := String_Output);



END Call_Scheduling_Framework;


