------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Text_IO;               use Text_IO;

package body Dependencies is

   procedure Initialize (My_Dep : in out Dependency_Ptr) is
   begin
      My_Dep := null;
   end Initialize;

   function Copy (My_Dep : in Dependency_Ptr) return Dependency_Ptr is
      Ptr : Dependency_Ptr;
   begin
      Ptr     := new Dependency (My_Dep.Depend_Type);
      Ptr.all := My_Dep.all;

      return Ptr;
   end Copy;

   procedure Put (My_Dep : in Dependency_Ptr) is
   begin
      if (My_Dep /= null) then
         Put (My_Dep.Depend_Type'Img & '/');
         case My_Dep.Depend_Type is

            when Precedence_Dependency =>
               Put
                 (To_String (My_Dep.Precedency_From_Task.Name) &
                  "=>" &
                  To_String (My_Dep.Precedency_To_Task.Name));

            when Buffer_Dependency =>
               if (My_Dep.Buffer_Orientation = From_Object_To_Task) then
                  Put
                    (To_String (My_Dep.Buffer_Dependency_Object.Name) &
                     "=>" &
                     To_String (My_Dep.Buffer_Dependent_Task.Name));
               else
                  Put
                    (To_String (My_Dep.Buffer_Dependent_Task.Name) &
                     "=>" &
                     To_String (My_Dep.Buffer_Dependency_Object.Name));
               end if;

            when Communication_Dependency =>
               if (My_Dep.Communication_Orientation =
                   From_Object_To_Task)
               then
                  Put
                    (To_String (My_Dep.Communication_Dependency_Object.Name) &
                     "=>" &
                     To_String (My_Dep.Communication_Dependent_Task.Name));
               else
                  Put
                    (To_String (My_Dep.Communication_Dependent_Task.Name) &
                     "=>" &
                     To_String (My_Dep.Communication_Dependency_Object.Name));
               end if;
         end case;
      end if;
   end Put;

end Dependencies;
