------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Tasks;
USE Tasks;
WITH Buffers;
USE Buffers;
WITH Messages;
USE Messages;
WITH Unchecked_Deallocation;



PACKAGE Dependencies IS


   TYPE Dependency_Type IS
         (Precedence_Dependency,
          Buffer_Dependency,
          Communication_Dependency);


   TYPE Orientation_Dependency_Type IS
         (From_Object_To_Task,
          From_Task_To_Object);



   TYPE Dependency
      (Depend_Type : Dependency_Type) IS
   RECORD
      CASE Depend_Type IS

         WHEN Precedence_Dependency =>
            Precedency_From_Task : Generic_Task_Ptr;
            Precedency_To_Task : Generic_Task_Ptr;

         WHEN Buffer_Dependency =>
            Buffer_Dependent_Task    : Generic_Task_Ptr;
            Buffer_Orientation       : Orientation_Dependency_Type;
            Buffer_Dependency_Object : Buffer_Ptr;

         WHEN Communication_Dependency =>
            Communication_Dependent_Task    : Generic_Task_Ptr;
            Communication_Orientation       : Orientation_Dependency_Type;
            Communication_Dependency_Object : Message_Ptr;

      END CASE;
   END RECORD;



   TYPE Dependency_Ptr IS ACCESS ALL Dependency;



   PROCEDURE Initialize (
         My_Dep : IN OUT Dependency_Ptr);

   PROCEDURE Put (
         My_Dep : IN     Dependency_Ptr);

   FUNCTION Copy (
         My_Dep : IN     Dependency_Ptr)
     RETURN Dependency_Ptr;


   PROCEDURE Free IS
   NEW Unchecked_Deallocation
      (Dependency, Dependency_Ptr);




END Dependencies;



