------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Processor_Set; use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Scheduler; use Scheduler;
use Scheduler.Double_Tasks_Parameters;
with Task_Set; use Task_Set;
use Task_Set.Generic_Task_Set;
WITH Task_Dependencies;
USE Task_Dependencies;
WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;


PACKAGE Dependency_Services IS



   -- Raised when tasks in a DAG have differents
   -- value of period
   --
   Dependencies_Period_Error : EXCEPTION;

   -- Raised when tasks in a DAG are of different type
   --
   Task_Model_Error : EXCEPTION;

   -- Raised when a method do not permit a cyclic graph
   --
   Cyclic_Graph_Error : EXCEPTION;




   -- Compute task parameters of a DAG organized tasks
   -- according to Chetto/Blazewicz method
   --
   PROCEDURE Set_Parameters_According_To_Chetto (
         My_Dependencies    : IN     Tasks_Dependencies_Ptr;
         My_Tasks           : IN OUT Tasks_Set;
         Compute_Deadlines  : IN     Boolean;
         Compute_Priorities : IN     Boolean);



   -- Compute end to end response time by the Holistic
   -- method
   --
   PROCEDURE Compute_End_To_End_Response_Time (
         My_Dependencies  : IN     Tasks_Dependencies_Ptr;
         My_Processors    : IN     Processors_Set;
         My_Tasks         : IN OUT Tasks_Set;
         One_Step         : IN     Boolean;
         Update_Tasks_Set : IN     Boolean;
         Msg              : IN OUT Unbounded_String;
         Rt               : IN OUT Response_Time_Table);



   -- Test is two tables of response time are equals
   --
   FUNCTION Is_Equal (
         Rtn  : IN     Response_Time_Table;
         Rtn1 : IN     Response_Time_Table)
     RETURN Boolean;



   -- Test is all deadlines are missed when we compute
   -- end to end response time
   -- In this case, we stop the computing
   --
   FUNCTION All_Deadlines_Missed (
         Rtn : IN     Response_Time_Table)
     RETURN Boolean;



   -- To schedule with precedencies
   -- all task of a dag must have the same priority
   -- If "Processor_Name" is a null string, do this check
   -- on all processor
   --
   PROCEDURE Dependencies_Periods_Control (
         My_Tasks       : IN     Tasks_Set;
         My_Deps        : IN     Tasks_Dependencies_Ptr;
         Processor_Name : IN     Unbounded_String);



END Dependency_Services;





