------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH framework_Config;
use framework_Config;
WITH Event_Analyzers;
USE Event_Analyzers;
WITH Sets;



PACKAGE Event_Analyzer_Set IS


   ----------------------------------------------------------
   -- Definition of a set
   ----------------------------------------------------------



   PACKAGE Generic_Event_Analyzer_Set IS NEW Sets (
      Max_Element => framework_Config.Max_Event_Analyzers,
      Element     => Event_Analyzer_Ptr,
      Free        => Free,
      Copy        => Copy,
      Put         => Put);


   USE Generic_Event_Analyzer_Set;

   TYPE Event_Analyzers_Set IS NEW Generic_Event_Analyzer_Set.Set WITH PRIVATE;


   SUBTYPE Event_Analyzers_Range IS Generic_Event_Analyzer_Set.
      Element_Range;
   SUBTYPE Event_Analyzers_Iterator IS   Generic_Event_Analyzer_Set.
      Iterator;




   ----------------------------------------------------------
   -- Exceptions defined in the set package
   ----------------------------------------------------------


   -- Raised when a given event analyzer is not in a event analyzer set
   --
   Event_Analyzer_Not_Found : EXCEPTION;

   -- Raised when parameters provided to Add_event_analyzer are wrong
   --
   Invalid_Parameter : EXCEPTION;



   ----------------------------------------------------------
   -- I/O operations
   ----------------------------------------------------------

   FUNCTION Export_Xml (
         My_Event_Analyzers : IN     Event_Analyzers_Set)
     RETURN Unbounded_String;

   FUNCTION Export_Aadl_Properties (
         My_Event_Analyzers : IN     Event_Analyzers_Set;
         Number_Of_Ht       : IN     Natural)
     RETURN Unbounded_String;



   ----------------------------------------------------------
   -- Procedure to proceed modification on a set
   ----------------------------------------------------------



   PROCEDURE Check_Event_Analyzer (
         My_Event_Analyzers : IN     Event_Analyzers_Set;
         File_Name          : IN     Unbounded_String);



   PROCEDURE Add_Event_Analyzer (
         My_Event_Analyzers : IN OUT Event_Analyzers_Set;
         File_Name          : IN     Unbounded_String);



   ------------------------------------------------------
   -- Read information from a set
   ------------------------------------------------------



   -- Get the task pointer of the task named "Name"
   --
   FUNCTION Search_Event_Analyzer (
         My_Event_Analyzers : IN     Event_Analyzers_Set;
         Name               : IN     Unbounded_String)
     RETURN Event_Analyzer_Ptr;



PRIVATE

   TYPE Event_Analyzers_Set IS NEW Generic_Event_Analyzer_Set.Set WITH NULL RECORD;




END Event_Analyzer_Set;
