------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Statements;
USE Statements;
WITH Expressions;
USE Expressions;
WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Unchecked_Deallocation;




PACKAGE Event_Analyzers IS


   TYPE Event_Analyzer IS TAGGED
      RECORD
         -- Event Analyzer behavior in string format
         --
         String_Behavior : Unbounded_String;

         -- file name which store the event Analyzer behavior
         --
         Behavior_File_Name : Unbounded_String;

         -- Statements for each scheduling
         -- section (pointers given by the parser)
         --
         Root_Statement_Pointer : Section_Pointers;

         -- Variables table for the parsed parametric file
         -- (given by the parser)
         --
         Variables_Table : Variables_Table_Type;


         -- Table of set statements given by the parser
         --
         Sets_Table : Sets_Table_Type;
      END RECORD;

   TYPE Event_Analyzer_Ptr IS ACCESS ALL Event_Analyzer'Class;


   PROCEDURE Put (
         My_Event_Analyzer : IN     Event_Analyzer_Ptr);




   PROCEDURE Free IS
   NEW Unchecked_Deallocation
      (Event_Analyzer'Class, Event_Analyzer_Ptr);


   FUNCTION Copy (
         My_Event_Analyzer : IN     Event_Analyzer_Ptr)
     RETURN Event_Analyzer_Ptr;


END Event_Analyzers;











