------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Text_IO;               use Text_IO;

package body Messages is

   procedure Initialize (A_Message : in out Generic_Message) is
   begin
      A_Message.Object_Type := Message_Object_Type;
   end Initialize;

   procedure Initialize (A_Message : in out Aperiodic_Message) is
   begin
      Initialize (Generic_Message (A_Message));
      A_Message.Message_Type := Aperiodic_Type;
   end Initialize;

   procedure Initialize (A_Message : in out Periodic_Message) is
   begin
      Initialize (Generic_Message (A_Message));
      A_Message.Message_Type := Periodic_Type;
   end Initialize;

   --------------------------------------------------

   procedure Put (My_Message : in Generic_Message) is
   begin
      Put ("Name : " & To_String (My_Message.Name));
      Put (" ; Type : " & My_Message.Message_Type'Img);
      Put (" ; Size : " & My_Message.Size'Img & " ; ");
      Put ("Deadline : " & My_Message.Deadline'Img & " ; ");
      Put (" ; x : " & My_Message.X'Img);
      Put (" ; y : " & My_Message.Y'Img);
      Put (" ; Response : " & My_Message.Response_Time'Img);
      Put (" ; Communication : " & My_Message.Communication_Time'Img);
   end Put;

   procedure Put_Name (A_Message : in Message_Ptr) is
   begin
      Put (To_String (A_Message.Name));
   end Put_Name;

   procedure Put (A_Message : in Message_Ptr) is
   begin
      Put (A_Message.all);
   end Put;

   procedure Put (My_Message : in Periodic_Message) is
   begin
      Put (Generic_Message (My_Message));
      Put ("Period : " & My_Message.Period'Img & " ; ");
      Put ("Jitter : " & My_Message.Jitter'Img & " ; ");
   end Put;

   function Copy (My_Message : in Message_Ptr) return Message_Ptr is
      New_Periodic_Message  : Periodic_Message_Ptr;
      New_Aperiodic_Message : Aperiodic_Message_Ptr;
      Ptr                   : Message_Ptr;

   begin
      if (My_Message.Message_Type = Periodic_Type) then
         New_Periodic_Message     := new Periodic_Message;
         New_Periodic_Message.all := Periodic_Message_Ptr (My_Message).all;
         Ptr                      := Message_Ptr (New_Periodic_Message);
      else
         New_Aperiodic_Message     := new Aperiodic_Message;
         New_Aperiodic_Message.all := Aperiodic_Message_Ptr (My_Message).all;
         Ptr                       := Message_Ptr (New_Aperiodic_Message);
      end if;

      return Ptr;
   end Copy;

end Messages;
