------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Parameters;
USE Parameters;
USE Parameters.User_Defined_Parameters_Table_Package;
WITH Objects;
USE Objects;
WITH Unchecked_Deallocation;



PACKAGE Messages IS


   TYPE Messages_Type IS
         (Periodic_Type,
          Aperiodic_Type,
          Generic_Type);

   --------------------------------------------------

   TYPE Generic_Message IS ABSTRACT NEW Generic_Object WITH
      RECORD
         Message_Type       : Messages_Type;
         Deadline           : Natural                       := 0;
         Size               : Natural                       := 0;
         X                  : Natural                       := 0;
         Y                  : Natural                       := 0;
         Param              : User_Defined_Parameters_Table;
         Response_Time      : Natural                       := 0;
         Communication_Time : Natural                       := 0;
      END RECORD;

   TYPE Message_Ptr IS ACCESS ALL Generic_Message'Class;


   PROCEDURE Initialize (
         A_Message : IN OUT Generic_Message);


   PROCEDURE Put_Name (
         A_Message : IN     Message_Ptr);

   PROCEDURE Put (
         A_Message : IN     Message_Ptr);

   PROCEDURE Put (
         My_Message : IN     Generic_Message);

   FUNCTION Copy (
         My_Message : IN     Message_Ptr)
     RETURN Message_Ptr;

   PROCEDURE Free IS
   NEW Unchecked_Deallocation
      (Generic_Message'Class, Message_Ptr);


   --------------------------------------------------

   TYPE Periodic_Message IS NEW Generic_Message WITH
      RECORD
         Period : Natural := 0;
         Jitter : Natural := 0;
      END RECORD;

   TYPE Periodic_Message_Ptr IS ACCESS ALL Periodic_Message'Class;

   PROCEDURE Initialize (
         A_Message : IN OUT Periodic_Message);

   PROCEDURE Put (
         My_Message : IN     Periodic_Message);

   PROCEDURE Free IS
   NEW Unchecked_Deallocation
      (Periodic_Message'Class, Periodic_Message_Ptr);


   --------------------------------------------------


   TYPE Aperiodic_Message IS NEW Generic_Message WITH NULL RECORD;

   TYPE Aperiodic_Message_Ptr IS ACCESS ALL Aperiodic_Message'Class;

   PROCEDURE Initialize (
         A_Message : IN OUT Aperiodic_Message);

   PROCEDURE Free IS
   NEW Unchecked_Deallocation
      (Aperiodic_Message'Class, Aperiodic_Message_Ptr);



END Messages;








