------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Framework_Config;
USE Framework_Config;
WITH Networks;
USE Networks;
WITH Sets;


PACKAGE Network_Set IS



   ----------------------------------------------------------
   -- Definition of a set
   ----------------------------------------------------------

   PACKAGE Generic_Network_Set IS NEW Sets(
      Max_Element => Framework_Config.Max_Networks,
      Element     => Network_Ptr,
      Free        => Free,
      Copy        => Copy,
      Put         => Put);

   USE Generic_Network_Set;

   TYPE Networks_Set IS NEW Generic_Network_Set.Set WITH PRIVATE;

   SUBTYPE Network_Range IS Generic_Network_Set.Element_Range;
   SUBTYPE Network_Iterator IS Generic_Network_Set.Iterator;




   ----------------------------------------------------------
   -- Exceptions defined in the set package
   ----------------------------------------------------------

   -- Raised when a given network is not in a network set
   --
   Network_Not_Found : EXCEPTION;

   -- Raised when 2 networks with the same name are added
   -- to a network set
   --
   Network_Already_Defined : EXCEPTION;

   -- Raised when parameters provided to Add_network are wrong
   --
   Invalid_Parameter : EXCEPTION;



   ----------------------------------------------------------
   -- I/O operations
   ----------------------------------------------------------
   FUNCTION Export_Aadl_Implementations (
         My_Networks : IN     Networks_Set)
     RETURN Unbounded_String;

   FUNCTION Export_Aadl_Declarations (
         My_Networks  : IN     Networks_Set;
         Number_Of_Ht : IN     Natural)
     RETURN Unbounded_String;

   FUNCTION Export_Xml (
         My_Networks : IN     Networks_Set)
     RETURN Unbounded_String;




   ----------------------------------------------------------
   -- Procedure to proceed modification on a set
   ----------------------------------------------------------


   PROCEDURE Update_Network (
         My_Networks  : IN OUT Networks_Set;
         Name         : IN     Unbounded_String;
         Network_Type : IN     Unbounded_String);


   PROCEDURE Add_Network (
         My_Networks  : IN OUT Networks_Set;
         Name         : IN     Unbounded_String;
         Network_Type : IN     Unbounded_String);



   ------------------------------------------------------
   -- Read information from a set
   ------------------------------------------------------



   FUNCTION Search_Network (
         My_Networks : IN     Networks_Set;
         Name        : IN     Unbounded_String)
     RETURN Network_Ptr;



PRIVATE

   TYPE Networks_Set IS NEW Generic_Network_Set.Set WITH NULL RECORD;

END Network_Set;


