%WITH Ada.strings.unbounded;
%USE Ada.strings.unbounded;
%WITH Ada.numerics.aux;
%USE Ada.numerics.aux;

%WITH Statements;
%USE Statements;
%USE Statements.Sets_Type_Package;
%WITH Expressions;
%USE Expressions;
%USE Expressions.Variables_Type_Package;
%WITH Simulations;
%USE Simulations;



%Token Double_Type_Token Array_Token 
%Token Integer_Type_Token Task_Token String_Type_Token
%Token Boolean_Type_Token Of_Token
%Token Last_Token First_Token
%Token Double_Token Integer_Token Identifier_Token True_Token False_Token String_Token
%Token In_Token For_Token Is_Token Function_Token Procedure_Token Set_Token While_Token Loop_Token 
%Token Task_Token Message_Token Processor_Token Buffer_Token Resource_Token Time_Unit_Token
%Token Tasks_Range_Token Messages_Range_Token Processors_Range_Token Buffers_Range_Token
%Token Resources_Range_Token Time_Units_Range_Token
%Token Return_Token Max_Token Min_Token Lcm_Token Put_Token Mod_Token Abs_Token
%Token Max_To_Index_Token Min_To_Index_Token
%Token Then_Token End_Token Else_Token If_Token End_If_Token End_Loop_Token
%Token Random_Type_Token Uniform_Token Exponential_Token 
%Token Get_Task_Index_Token Get_Buffer_Index_Token Get_Message_Index_Token
%Token Get_Resource_Index_Token
%Token Gather_Event_Analyzer_Token Display_Event_Analyzer_Token Start_Token Priority_Token Election_Token Activation_Token Check_Resource_Token Release_Resource_Token Allocate_Resource_Token

%Token '[' ']' '(' ')' '}' '{'
%right     Equal_Token
%left      '+' '-'
%left      '*' '/'
%right     Unary_Minus
%right     Unary_Plus
%nonassoc  Exp_Token
%Token ':' '|' ';' ',' 
%Token Inf_Token Sup_Token Inf_Equal_Token Sup_Equal_Token Not_Equal_Token Affectation_Token
%Token Concat_token Or_Token And_Token Not_Token

{

type Key_Type is (Cval, Ival, Noval);

type Yystype is record
  	String_Value 		     : Unbounded_String;
	Integer_Value 		     : Integer;
	Double_Value 		     : Double;
  	Expr		          	: Expression_Ptr;
  	Stats 	     		: Statement_Ptr;
  	Variable_Type_Value 	: Simulation_Type;
     Table_Type_Value        : Table_Types;
end record;

}

%%

Entry :  Start_Section Priority_Section Election_Section Activation_Section Check_Resource_Section Release_Resource_Section  Allocate_Resource_Section Gather_Event_Analyzer_Section Display_Event_Analyzer_Section; 


Start_Section : 
	{
		Current_Section:=Start;
	} 
	Start_Token Statements
     |;

Priority_Section : 
	{
		Current_Section:=Priority;
	} 
	Priority_Token Statements
     |;

Election_Section  : 
	{
		Current_Section:=Election;
	} 
	Election_Token Statements
     |;

Activation_Section  : 
	{
		Current_Section:=Activation;
	} 
	Activation_Token Statements
     |;

Check_Resource_Section  : 
	{
		Current_Section:=Check_Resource;
	} 
	Check_Resource_Token Statements
     |;

Release_Resource_Section  : 
	{
		Current_Section:=Release_Resource;
	} 
	Release_Resource_Token Statements
     |;

Allocate_Resource_Section : 
	{
		Current_Section:=Allocate_Resource;
	} 
	Allocate_Resource_Token Statements
     |;

Gather_Event_Analyzer_Section  : 
	{
          Current_Section:=Gather_Event_Analyzer;
	} 
     Gather_Event_Analyzer_Token Statements
     |;

Display_Event_Analyzer_Section : 
	{
          Current_Section:=Display_Event_Analyzer;
	} 
     Display_Event_Analyzer_Token Statements
     |;


                
           
Statements : Statements Statement
	{	
     if $2.Stats/=null then
		if (Root_Statement_Pointer(Current_Section)=null)
                then
                	Root_Statement_Pointer(Current_Section):=$2.Stats;
                        Current_Statement_Pointer(Current_Section):=Root_Statement_Pointer(Current_Section);
                else
                        Current_Statement_Pointer(Current_Section).Next_Statement:=$2.Stats;
                        Current_Statement_Pointer(Current_Section):=$2.Stats;
          end if;
      end if;
      }
      |;


Then_Included_Statements :  Then_Included_Statements Statement
      {
                $2.Stats.Next_Statement:=Then_Inner_Statement_Ptr(Then_Inner_Statement_Index);
                Then_Inner_Statement_Ptr(Then_Inner_Statement_Index):=$2.Stats;
      }
      | Statement
	 {
                Then_Inner_Statement_Index:=Then_Inner_Statement_Index+1;
                Then_Inner_Statement_Ptr(Then_Inner_Statement_Index):=$1.Stats;
                Then_Inner_Statement_ptr(Then_Inner_Statement_Index).Next_Statement:=null;
      }; 
                    

Loop_Included_Statements :  Loop_Included_Statements Statement
       {
                $2.Stats.Next_Statement:=Loop_Inner_Statement_Ptr(Loop_Inner_Statement_Index);
                Loop_Inner_Statement_Ptr(Loop_Inner_Statement_Index):=$2.Stats;
       }
       | Statement
	  {
                Loop_Inner_Statement_Index:=Loop_Inner_Statement_Index+1;
                Loop_Inner_Statement_Ptr(Loop_Inner_Statement_Index):=$1.Stats;
                Loop_Inner_Statement_ptr(Loop_Inner_Statement_Index).Next_Statement:=null;
       }; 
          
        

Else_Included_Statements :  Else_Included_Statements Statement
       {
                $2.Stats.Next_Statement:=Else_Inner_Statement_Ptr(Else_Inner_Statement_Index);
                Else_Inner_Statement_Ptr(Else_Inner_Statement_Index):=$2.Stats;
       }
       | Statement
	  {
                Else_Inner_Statement_Index:=Else_Inner_Statement_Index+1;
                Else_Inner_Statement_Ptr(Else_Inner_Statement_Index):=$1.Stats;
                Else_Inner_Statement_ptr(Else_Inner_Statement_Index).Next_Statement:=null;
       }; 
          


Statement  : 
       Put_Token '(' Expr Put_Parameter Put_Parameter ')' ';' 
	  {
	  declare
		New_Put : Put_Statement_Ptr;
	  begin

		-- Firstly, the Put_Parameter
		--
          if (($4.Expr/=null) and ($5.Expr=null))
              or (($4.Expr=null) and ($5.Expr/=null))
          then
             Raise_Exception(Expressions.Syntax_Error'Identity, " Line " & Scheduler_Lex.Lines'Img &  
                 To_String(Lb_Comma & "0 or 2 parameters should be given" ));
          end if;
          
          New_Put:= NEW Put_Statement;
          New_Put.Expression_To_Be_Displayed:=$3.Expr;
          New_Put.Put_From:=$4.Expr;
          New_Put.Put_To:=$5.Expr;
          New_Put.Line_Number:=Scheduler_Lex.Lines;
          $$.Stats:=Statement_Ptr(New_Put);
	  end;
	  }

	  | Identifier_Data ':' User_Type Affectation ';'	
	  { 
	  declare
         New_Affectation : Affectation_Statement_Ptr;
         New_Var      	: Variable_Record_Ptr;
         New_Array_Expr 	: Array_Variable_Expr_Ptr;
         New_Var_Expr    : Variable_Expr_Ptr;
                       
	  begin

          -- Is the Variable already exists ?
		--
		for I in 0..Variables_Table.Nb_Entries-1 loop
			if (Variables_Table.Entries(i).Variable.Name=$1.String_Value) 
				then
					Raise_Exception(Expressions.Syntax_Error'Identity, " Line " & Scheduler_Lex.Lines'Img & 
                           To_String(Lb_Comma & $1.String_Value & Lb_Comma & Lb_Identifier_Already_Declared(Current_Language) ));
			end if;
		end loop;



		-- Create the Variable (Array or not)
		--
		New_Var:= NEW Variable_Record;

          if ($3.Variable_Type_Value=Simulation_Array_Double)
                	or ($3.Variable_Type_Value=Simulation_Array_Integer)
                	or ($3.Variable_Type_Value=Simulation_Array_Boolean) 
                	or ($3.Variable_Type_Value=Simulation_Array_String)
                        or ($3.Variable_Type_Value=Simulation_Array_Random) 
                        or ($3.Variable_Type_Value=Simulation_Time_Unit_Array_Random)
                	or ($3.Variable_Type_Value=Simulation_Time_Unit_Array_Double)
                	or ($3.Variable_Type_Value=Simulation_Time_Unit_Array_Integer)
                	or ($3.Variable_Type_Value=Simulation_Time_Unit_Array_Boolean) 
                	or ($3.Variable_Type_Value=Simulation_Time_Unit_Array_String) 
			then

                		New_Array_Expr:= NEW Array_Variable_Expr;
                        New_Array_Expr.Variable_Type:=$3.Variable_Type_Value;
                		New_Array_Expr.Name:=$1.String_Value;
				     New_Var.Variable:=Variable_Expr_Ptr(New_Array_Expr);
               else
				     New_Var_Expr := NEW Variable_Expr;
                        New_Var_Expr.Variable_Type:=$3.Variable_Type_Value;
				     New_Var_Expr.Name:=$1.String_Value;
				     New_Var.Variable:=New_Var_Expr;
		end if;



		Add(Variables_Table, New_Var);

          -- Create simulation data
          --
          Variables_Table.Entries(Variables_Table.Nb_Entries-1).Simulation
                       :=new Simulation_Value(
          Variable_Expr_Ptr(Variables_Table.Entries(Variables_Table.Nb_Entries-1).
                       Variable).Variable_Type);


		-- If Affectation is not null, we should register
		-- a new Affectation
		--
		if $4.Expr /= null 
			then
				New_Affectation:= NEW Affectation_Statement;
				New_Affectation.Lvalue:=Expression_Ptr(New_Var_Expr);
                   New_Affectation.Rvalue:=$4.Expr;
				New_Affectation.Line_Number:=Scheduler_Lex.Lines;
				$$.Stats:=Statement_Ptr(New_Affectation);
              else $$.Stats:=null;
		end if;
	end;
	}

	| 

     Expr Affectation_Token Expr ';'
	{
          New_Affectation:= NEW Affectation_Statement;
          New_Affectation.Line_Number:=Scheduler_Lex.Lines;
          New_Affectation.Lvalue:=$1.Expr;
          New_Affectation.Rvalue:=$3.Expr;
		$$.Stats:=Statement_Ptr(New_Affectation);
	}

     | If_Token Expr Then_Token Then_Included_Statements      
	{
	    New_If:= NEW If_Statement;
	    New_If.Line_Number:=Scheduler_Lex.Lines;
         New_If.Bool_Expression:=$2.Expr;
         New_If.Then_Statement:=Then_Inner_Statement_Ptr(Then_Inner_Statement_Index);
         Then_Inner_Statement_Index:=Then_Inner_Statement_Index-1;

		$$.Stats:=Statement_Ptr(New_If);

          New_Stat:=Copy(New_If.Then_Statement);
          New_Stat.Next_Statement:=null;
          while  New_If.Then_Statement.Next_Statement/=null loop
                	Old_Stat:=Copy(New_If.Then_Statement.Next_Statement);
                	Old_Stat.Next_Statement:=New_Stat;
                	New_Stat:=Old_Stat;
                	New_If.Then_Statement:=New_If.Then_Statement.Next_Statement;
           end loop;
           New_If.Then_Statement:=New_Stat;


     }    
       Else_Rule End_If_Token ';'
     {

                New_If.Else_Statement:=Else_Inner_Statement_Ptr(Else_Inner_Statement_Index);
                Else_Inner_Statement_Index:=Else_Inner_Statement_Index-1;
		      $$.Stats:=Statement_Ptr(New_If);

                New_Stat:=Copy(New_If.Else_Statement);
                New_Stat.Next_Statement:=null;
                while  New_If.Else_Statement.Next_Statement/=null loop
                	Old_Stat:=Copy(New_If.Else_Statement.Next_Statement);
                	Old_Stat.Next_Statement:=New_Stat;
                	New_Stat:=Old_Stat;
                	New_If.Else_Statement:=New_If.Else_Statement.Next_Statement;
                end loop;
                New_If.Else_Statement:=New_Stat;
     }
  

	| Return_Token Return_Expr 
	{
	declare
		New_Return : Return_Statement_Ptr;
		
	begin
          New_Return:= NEW Return_Statement;
          New_Return.Return_Value:=$2.Expr;
          New_Return.Line_Number:=Scheduler_Lex.Lines;

          $$.Stats:=Statement_Ptr(New_Return);
	end;
	}


     | For_Token Expr In_Token Table_Type Loop_Token Loop_Included_Statements End_Loop_Token ';'
	{
                New_For:= NEW For_Statement;
                New_For.For_Type:=$4.Table_Type_Value;
                New_For.For_Index:=Variable_Expr_Ptr($2.Expr);
                New_For.Line_Number:=Scheduler_Lex.Lines;
                 
                New_For.Included_Statement:=Loop_Inner_Statement_Ptr(Loop_Inner_Statement_Index);
                Loop_Inner_Statement_Index:=Loop_Inner_Statement_Index-1;

                New_Stat:=Copy(New_For.Included_Statement);
                New_Stat.Next_Statement:=null;
                while  New_For.Included_Statement.Next_Statement/=null loop
                   Old_Stat:=Copy(New_For.Included_Statement.Next_Statement);
                	Old_Stat.Next_Statement:=New_Stat;
                	New_Stat:=Old_Stat;
                   New_For.Included_Statement:=New_For.Included_Statement.Next_Statement;
                end loop;
                New_For.Included_Statement:=New_Stat;

                $$.Stats:=Statement_Ptr(New_For);
      }


      | While_Token Expr Loop_Token Loop_Included_Statements End_Loop_Token ';'
      {
                New_While:= NEW While_Statement;
                New_While.Condition:=$2.Expr;
                New_While.Line_Number:=Scheduler_Lex.Lines;
                 
                New_While.Included_Statement:=Loop_Inner_Statement_Ptr(Loop_Inner_Statement_Index);
                Loop_Inner_Statement_Index:=Loop_Inner_Statement_Index-1;

                New_Stat:=Copy(New_While.Included_Statement);
                New_Stat.Next_Statement:=null;
                while  New_While.Included_Statement.Next_Statement/=null loop
                	Old_Stat:=Copy(New_While.Included_Statement.Next_Statement);
                	Old_Stat.Next_Statement:=New_Stat;
                	New_Stat:=Old_Stat;
                	New_While.Included_Statement:=New_While.Included_Statement.Next_Statement;
                end loop;
                New_While.Included_Statement:=New_Stat;

                $$.Stats:=Statement_Ptr(New_While);

    }


    | Exponential_Token  '(' Identifier_Data ',' Expr ')' ';'
    {
    declare 
                New_Random_Statement : Random_Initialize_Statement_Ptr;
	begin
                New_Random_Statement:=NEW Random_Initialize_Statement;
                New_Random_Statement.Lvalue:=$3.String_Value;
                New_Random_Statement.Law:=Exponential_Law;
                New_Random_Statement.Parameter1:=$5.Expr;   
                $$.Stats:=Statement_Ptr(New_Random_Statement);
	end;
     }


     | Uniform_Token '(' Identifier_Data ',' Expr ',' Expr ')' ';' 
	{
	declare 
                New_Random_Statement : Random_Initialize_Statement_Ptr;
	begin
                New_Random_Statement:=NEW Random_Initialize_Statement;
                New_Random_Statement.Lvalue:=$3.String_Value;
                New_Random_Statement.Law:=Uniform_Law;
                New_Random_Statement.Parameter1:=$5.Expr;
                New_Random_Statement.Parameter2:=$7.Expr; 
                $$.Stats:=Statement_Ptr(New_Random_Statement);
	end;
	}


     | Set_Token Identifier_Data Expr  ';'
	{
     declare
                New_Set : Set_Statement_Ptr;
     begin
                -- Is the Variable already exists ?
                --
                for I in 0..Sets_Table.Nb_Entries-1 loop
                        if (Sets_Table.Entries(i).Set_Id=$2.String_Value)
                                then
                                     Raise_Exception(Expressions.Syntax_Error'Identity, " Line " & Scheduler_Lex.Lines'Img & 
                                       To_String(Lb_Comma & $2.String_Value & Lb_Comma & Lb_Identifier_Already_Declared(Current_Language) ));
                        end if;
                end loop;

                New_Set := NEW Set_Statement;
                New_Set.Set_Value:= $3.Expr;
                New_Set.Set_Id := $2.String_Value;
                New_Set.Line_Number := Scheduler_Lex.Lines;
                Add(Sets_Table, New_Set);
	
                $$.Stats:= Statement_Ptr(New_Set);
      end;
      };



Else_Rule :     Else_Token Else_Included_Statements 
	{
		null;
	}
	|
	{
        declare
                New_Nop : Nop_Statement_Ptr;

        begin
                New_Nop:= NEW Nop_Statement;
                New_Nop.Line_Number:=Scheduler_Lex.Lines;

                Else_Inner_Statement_Index:=Else_Inner_Statement_Index+1;
                Else_Inner_Statement_Ptr(Else_Inner_Statement_Index):=Statement_Ptr(New_Nop);

        end;

	};


Put_Parameter :   ',' Expr
	{
          $$.Expr:=$2.Expr;
	}
     |
     {
          $$.Expr:=null;
     };
		
             


Return_Expr : Expr ';'
	{
		$$.Expr:=$1.Expr;
	}
	| ';'
	{
		$$.Expr:=null;
	};




User_Type : Double_Type_Token 
	{
		$$.Variable_Type_Value:=Simulation_Double;
	}
     | Array_Token '(' Table_Type ')' Of_Token Double_Type_Token
	{
          if $3.Table_Type_Value=Time_Unit_Table_Type
               then 
                  $$.Variable_Type_Value:=Simulation_Time_Unit_Array_Double;
               else
                  $$.Variable_Type_Value:=Simulation_Array_Double;
          end if;
	}
 	| Integer_Type_Token  
	{
          $$.Variable_Type_Value:=Simulation_Integer;
	}
     | Array_Token '(' Table_Type ')' Of_Token Integer_Type_Token
	{
           if $3.Table_Type_Value=Time_Unit_Table_Type
                then 
                   $$.Variable_Type_Value:=Simulation_Time_Unit_Array_Integer;
                else
                   $$.Variable_Type_Value:=Simulation_Array_Integer;
           end if; 
	}
     | Random_Type_Token  
	{
           $$.Variable_Type_Value:=Simulation_Random;
	}
     | Array_Token '(' Table_Type ')' Of_Token Random_Type_Token
	{
            if $3.Table_Type_Value=Time_Unit_Table_Type
                 then 
                    $$.Variable_Type_Value:=Simulation_Time_Unit_Array_Random;
                 else
                    $$.Variable_Type_Value:=Simulation_Array_Random;
             end if; 
	}
     | Boolean_Type_Token 
	{
            $$.Variable_Type_Value:=Simulation_Boolean;
	}
     | Array_Token '(' Table_Type ')' Of_Token Boolean_Type_Token
	{
             if $3.Table_Type_Value=Time_Unit_Table_Type
                 then 
                    $$.Variable_Type_Value:=Simulation_Time_Unit_Array_Boolean;
                 else
                    $$.Variable_Type_Value:=Simulation_Array_Boolean;
             end if; 
	}
     | Array_Token '(' Table_Type ')' Of_Token String_Type_Token
	{
              if $3.Table_Type_Value=Time_Unit_Table_Type
                   then 
                      $$.Variable_Type_Value:=Simulation_Time_Unit_Array_String;
                   else
                      $$.Variable_Type_Value:=Simulation_Array_String;
              end if; 
	}
     | String_Type_Token 
	{
               $$.Variable_Type_Value:=Simulation_String;
	}
	;


                             
Table_Type :  Tasks_Range_Token
	{
                $$.Table_Type_Value:=Task_Table_Type;
	}
     | Buffers_Range_Token
	{
                $$.Table_Type_Value:=Buffer_Table_Type;
	}
     | Messages_Range_Token
	{
                $$.Table_Type_Value:=Message_Table_Type;
	}
     | Time_Units_Range_Token
	{
                $$.Table_Type_Value:=Time_Unit_Table_Type;
	}
     | Processors_Range_Token
	{
                $$.Table_Type_Value:=Processor_Table_Type;
	}
     | Resources_Range_Token
	{
                $$.Table_Type_Value:=Resource_Table_Type;
	}
	;



Affectation : Affectation_Token Expr 
	{
        	$$.Expr:=$2.Expr;
	}
	|
	{
		$$.Expr:=null;
	};


Expr  :
	Expr And_Token Expr
	{
	declare
          New_Expr : Binary_Expr_Ptr; 
	begin
          New_Expr:=NEW Binary_Expr;
          New_Expr.Ope:=Logic_And;
          New_Expr.Lvalue:=$1.Expr;
          New_Expr.Rvalue:=$3.Expr;
          $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

        | Expr Mod_Token Expr 
	{
	declare
           New_Expr : Binary_Expr_Ptr; 
	begin
           New_Expr:=NEW Binary_Expr;
           New_Expr.Ope:=Modulo;
           New_Expr.Lvalue:=$1.Expr;            
           New_Expr.Rvalue:=$3.Expr;
           $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Expr Inf_Token Expr 
	{
	declare
           New_Expr : Binary_Expr_Ptr; 
	begin
           New_Expr:=NEW Binary_Expr;
           New_Expr.Ope:=Inf;
           New_Expr.Lvalue:=$1.Expr;            
           New_Expr.Rvalue:=$3.Expr;
           $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Expr Sup_Token Expr 
	{
	declare
            New_Expr : Binary_Expr_Ptr; 
	begin
            New_Expr:=NEW Binary_Expr;
            New_Expr.Ope:=Sup;
            New_Expr.Lvalue:=$1.Expr;
            New_Expr.Rvalue:=$3.Expr;
            $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Expr Inf_Equal_Token Expr 
	{
	declare
            New_Expr : Binary_Expr_Ptr; 
	begin
            New_Expr:=NEW Binary_Expr;
            New_Expr.Ope:=Equal_Less;
            New_Expr.Lvalue:=$1.Expr;
            New_Expr.Rvalue:=$3.Expr;
            $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Expr Sup_Equal_Token Expr 
	{
	declare
            New_Expr : Binary_Expr_Ptr; 
	begin
            New_Expr:=NEW Binary_Expr;
            New_Expr.Ope:=Equal_Greater;
            New_Expr.Lvalue:=$1.Expr;
            New_Expr.Rvalue:=$3.Expr;
            $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Expr Not_Equal_Token Expr 
	{
	declare
            New_Expr : Binary_Expr_Ptr; 
	begin
            New_Expr:=NEW Binary_Expr;
            New_Expr.Ope:=Not_Equal;
            New_Expr.Lvalue:=$1.Expr;
            New_Expr.Rvalue:=$3.Expr;
            $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Expr Equal_Token Expr 
	{
	declare
            New_Expr : Binary_Expr_Ptr; 
	begin
            New_Expr:=NEW Binary_Expr;
            New_Expr.Ope:=Equal;
            New_Expr.Lvalue:=$1.Expr;
            New_Expr.Rvalue:=$3.Expr;
            $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Expr Or_Token Expr
	{
	declare
            New_Expr : Binary_Expr_Ptr; 
	begin
            New_Expr:=NEW Binary_Expr;
            New_Expr.Ope:=Logic_Or;
            New_Expr.Lvalue:=$1.Expr;
            New_Expr.Rvalue:=$3.Expr;
            $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Not_Token Expr 
	{
	declare
            New_Expr : Unary_Expr_Ptr; 
	begin
            New_Expr:=NEW Unary_Expr;
            New_Expr.Ope:=Logic_Not;
            New_Expr.Value:=$2.Expr;
            $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

     | '(' Expr ')'
	{
        	  $$.Expr:=$2.Expr;
	}

     | True_Token 
	{
	declare
		New_Expr : Constant_Expr_Ptr;	
	begin
		New_Expr:=NEW Constant_Expr;
		New_Expr.Value:= NEW Simulation_Value'(Simulation_Boolean, True);
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

     | False_Token
	{
	declare
		New_Expr : Constant_Expr_Ptr;	
	begin
		New_Expr:=NEW Constant_Expr;
		New_Expr.Value:= NEW Simulation_Value'(Simulation_Boolean, False);
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}	

     | Expr '+' Expr
	{
	declare
          New_Expr : Binary_Expr_Ptr; 
	begin
          New_Expr:=NEW Binary_Expr;
          New_Expr.Ope:=Plus;
          New_Expr.Lvalue:=$1.Expr;
          New_Expr.Rvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Expr '-' Expr
	{
	declare
          New_Expr : Binary_Expr_Ptr; 
	begin
          New_Expr:=NEW Binary_Expr;
          New_Expr.Ope:=Minus;
          New_Expr.Lvalue:=$1.Expr;
          New_Expr.Rvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Expr '*' Expr
	{
	declare
          New_Expr : Binary_Expr_Ptr; 
	begin
          New_Expr:=NEW Binary_Expr;
          New_Expr.Ope:=Mult;
          New_Expr.Lvalue:=$1.Expr;
          New_Expr.Rvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Expr '/' Expr
	{
	declare
          New_Expr : Binary_Expr_Ptr; 
	begin
          New_Expr:=NEW Binary_Expr;
          New_Expr.Ope:=Div;
          New_Expr.Lvalue:=$1.Expr;
          New_Expr.Rvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

     | Lcm_Token '(' Expr ',' Expr ')'
	{
	declare
          New_Expr : Binary_Expr_Ptr; 
	begin
          New_Expr:=NEW Binary_Expr;
          New_Expr.Ope:=Lcm;
          New_Expr.Lvalue:=$1.Expr;
          New_Expr.Rvalue:=$3.Expr;
		$$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

     | Abs_Token '(' Expr ')'
     {
     declare
           New_Expr : Unary_Expr_Ptr;
     begin
           New_Expr:=NEW Unary_Expr;
           New_Expr.Value:=$3.Expr;
           New_Expr.ope:=Abs_Operator;
           $$.Expr:=Expression_Ptr(New_Expr);
     end;
     }


     | Concat_Token '(' Expr ',' Expr ')'
     {
     declare
            New_Expr : Binary_Expr_Ptr;
     begin
            New_Expr:=NEW Binary_Expr;
            New_Expr.Ope:=Concat;
            New_Expr.Lvalue:=$1.Expr;
            New_Expr.Rvalue:=$3.Expr;
            $$.Expr:=Expression_Ptr(New_Expr);
     end;
	}

     | Max_Token '(' Expr ',' Expr ')'
	{
	declare
            New_Expr : Binary_Expr_Ptr; 
	begin
            New_Expr:=NEW Binary_Expr;
            New_Expr.Ope:=Max_Operator;
            New_Expr.Lvalue:=$3.Expr;
            New_Expr.Rvalue:=$5.Expr;
            $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

     | Min_Token '(' Expr ',' Expr  ')'
	{
	declare
            New_Expr : Binary_Expr_Ptr; 
	begin
            New_Expr:=NEW Binary_Expr;
            New_Expr.Ope:=Min_Operator;
            New_Expr.Lvalue:=$3.Expr;
            New_Expr.Rvalue:=$5.Expr;
            $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Expr Exp_Token Expr
	{
	declare
             New_Expr : Binary_Expr_Ptr; 
	begin
             New_Expr:=NEW Binary_Expr;
             New_Expr.Ope:=Exp;
             New_Expr.Lvalue:=$1.Expr;
             New_Expr.Rvalue:=$3.Expr;
             $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| '-' Expr  %prec Unary_Minus
	{
	declare
             New_Expr1 : Binary_Expr_Ptr; 
             New_Expr2 : Constant_Expr_Ptr;	

	begin
             New_Expr2:=NEW Constant_Expr;
             if Get_Type($2.Expr.all) = Simulation_Double 
                then
                        New_Expr2.Value:=NEW Simulation_Value'(Simulation_Double,
                                        -1.0);
                else
                        New_Expr2.Value:=NEW Simulation_Value'(Simulation_Integer,
                                        -1);
              end if;
              New_Expr1:=NEW Binary_Expr;
              New_Expr1.Ope:=Mult;
              New_Expr1.Lvalue:=Expression_Ptr(New_Expr2);
              New_Expr1.Rvalue:=$2.Expr;
              $$.Expr:=Expression_Ptr(New_Expr1);
	end;
	}

     | '+' Expr  %prec Unary_Plus
     {
     declare
              New_Expr1 : Binary_Expr_Ptr;
              New_Expr2 : Constant_Expr_Ptr;

     begin
              New_Expr2:=NEW Constant_Expr;
              if Get_Type($2.Expr.all) = Simulation_Double 
                then
                        New_Expr2.value:=NEW Simulation_Value'(Simulation_Double,
                                        1.0);
                else
                        New_Expr2.value:=NEW Simulation_Value'(Simulation_Integer,
                                        1);
              end if;
              New_Expr1:=NEW Binary_Expr;
              New_Expr1.Ope:=Mult;
              New_Expr1.Lvalue:=Expression_Ptr(New_Expr2);
              New_Expr1.Rvalue:=$2.Expr;
              $$.Expr:=Expression_Ptr(New_Expr1);
     end;
     }

     | Max_To_Index_Token '(' Expr ')'
	{
	declare 
              New_Expr : Unary_Expr_Ptr;
	begin

              if  (Current_Section /= Election)
                then
                   Raise_Exception(Expressions.Syntax_Error'Identity, " Line " & Scheduler_Lex.Lines'Img &
                     To_String(Lb_Comma & 
                     "min_to_index and max_to_index operators can only be used in election_section."));
              end if;

              New_Expr := NEW Unary_Expr;
              New_Expr.value:=$3.Expr;
              New_Expr.Ope:=Max_To_Index;
              $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

     | Min_To_Index_Token '(' Expr ')'
	{
	declare 
              New_Expr : Unary_Expr_Ptr;
	begin
              if  (Current_Section /= Election)
                then
                   Raise_Exception(Expressions.Syntax_Error'Identity, " Line " & Scheduler_Lex.Lines'Img &
                     To_String(Lb_Comma & 
                     "min_to_index and max_to_index operators can only be used in election_section."));
              end if;

              New_Expr := NEW Unary_Expr;
              New_Expr.value:=$3.Expr;
              New_Expr.Ope:=Min_To_Index;
              $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}


     | Get_Task_Index_Token '(' Expr ')'
	{
	declare 
              New_Expr : Unary_Expr_Ptr;
	begin
              New_Expr := NEW Unary_Expr;
              New_Expr.value:=$3.Expr;
              New_Expr.Ope:=Get_Task_Index;
              $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

     | Get_Resource_Index_Token '(' Expr ')'
	{
	declare 
              New_Expr : Unary_Expr_Ptr;
	begin
              New_Expr := NEW Unary_Expr;
              New_Expr.value:=$3.Expr;
              New_Expr.Ope:=Get_Resource_Index;
              $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

     | Get_Buffer_Index_Token '(' Expr ')'
	{
	declare 
              New_Expr : Unary_Expr_Ptr;
	begin
              New_Expr := NEW Unary_Expr;
              New_Expr.value:=$3.Expr;
              New_Expr.Ope:=Get_Buffer_Index;
              $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

     | Get_Message_Index_Token '(' Expr ')'
	{
	declare 
              New_Expr : Unary_Expr_Ptr;
	begin
              New_Expr := NEW Unary_Expr;
              New_Expr.value:=$3.Expr;
              New_Expr.Ope:=Get_Message_Index;
              $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Double_Type_Token ''' Last_Token
	{
	declare
              New_Expr : Constant_Expr_Ptr;	
	begin
              New_Expr:=NEW Constant_Expr;
              New_Expr.value:=NEW Simulation_Value'(Simulation_Double,Double'Last);
              $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}
	| Double_Type_Token ''' First_Token
	{
	declare
               New_Expr : Constant_Expr_Ptr;	
	begin
               New_Expr:=NEW Constant_Expr;
               New_Expr.value:=NEW Simulation_Value'(Simulation_Double,Double'First);
               $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}
	| Integer_Type_Token ''' Last_Token
	{
	declare
              New_Expr : Constant_Expr_Ptr;	
	begin
              New_Expr:=NEW Constant_Expr;
              New_Expr.value:=NEW Simulation_Value'(Simulation_Integer,Integer'Last);
              $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}
	| Integer_Type_Token ''' First_Token
	{
	declare
              New_Expr : Constant_Expr_Ptr;	
	begin
              New_Expr:=NEW Constant_Expr;
              New_Expr.value:=NEW Simulation_Value'(Simulation_Integer,Integer'First);
              $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}
     | Identifier_Data '(' Expr ')'
	{
	declare
             New_Expr : Array_Variable_Expr_Ptr;	
	begin
             -- Find the type of the $1 Variable
             --
             $$.Expr:=Expression_Ptr(Variables_Table.Entries(
             Find_Variable(Variables_Table, $1.String_Value, Scheduler_Lex.Lines)).Variable);
		
             New_Expr := NEW Array_Variable_Expr;
             New_Expr.Variable_Type:=Variable_Expr_Ptr($$.Expr).Variable_Type;
             New_Expr.Name:=$1.String_Value;
             New_Expr.Array_index:=$3.Expr;
             $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}
     | Identifier_Data  
	{
             -- Find the type of the Variable before
             -- instanciation
             --
             $$.Expr:=Expression_Ptr(Variables_Table.Entries(
             Find_Variable(Variables_Table, $1.String_Value, Scheduler_Lex.Lines)).Variable);
	}
     | String_Data
	{
	declare
              New_Expr : Constant_Expr_Ptr;	
	begin
              New_Expr:=NEW Constant_Expr;
              New_Expr.Value:=NEW Simulation_Value'(Simulation_String,$1.String_Value);
              $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}
	| Double_Data
	{
	declare
              New_Expr : Constant_Expr_Ptr;	
	begin
              New_Expr:=NEW Constant_Expr;
              New_Expr.Value:=NEW Simulation_Value'(Simulation_Double,$1.Double_Value);
		    $$.Expr:=Expression_Ptr(New_Expr);
	end;
	}

	| Integer_Data 
	{
	declare
              New_Expr : Constant_Expr_Ptr;	
	begin
              New_Expr:=NEW Constant_Expr;
              New_Expr.value:=NEW Simulation_Value'(Simulation_Integer,$1.Integer_Value);
              $$.Expr:=Expression_Ptr(New_Expr);
	end;
	};





Identifier_Data :  Identifier_Token
     {
		$$.String_Value := To_Unbounded_String(Scheduler_Dfa.Yytext); 
     };


Double_Data     : Double_Token
	{ 
	declare 
		Ok : boolean;

	begin
		To_Double(To_Unbounded_String(Scheduler_Dfa.Yytext),$$.Double_Value,Ok);
         	if not Ok
               then
				Raise_Exception(Expressions.Syntax_Error'Identity, " Line " & Scheduler_Lex.Lines'Img &
                     To_String(Lb_Comma & To_Unbounded_String(Scheduler_Dfa.Yytext) & Lb_Comma & Lb_Double_Conversion_Error(Current_Language) ));
         	end if;
     end;
	};


String_Data : String_Token
	{
	declare
              Tmp : Unbounded_String;
	begin
              Tmp:=To_Unbounded_String(Scheduler_Dfa.Yytext);
              Tmp:=To_Unbounded_String(Slice(Tmp, 2, Length(Tmp)-1));
              $$.String_Value:=Tmp;
	end;
     };
     
Integer_Data     : Integer_Token
	{ 
	declare 
		Ok : boolean;

	begin
		To_Integer(To_Unbounded_String(Scheduler_Dfa.Yytext),$$.Integer_Value,Ok);
		if not Ok
			then
				Raise_Exception(Expressions.Syntax_Error'Identity, " Line " & Scheduler_Lex.Lines'Img &
                     To_String(Lb_Comma & To_Unbounded_String(Scheduler_Dfa.Yytext) & Lb_Comma & Lb_Integer_Conversion_Error(Current_Language) ));
         	end if;
	end;
	};



%%
WITH Statements;
USE  Statements;
USE  Statements.Sets_Type_Package;
WITH Expressions; 
USE  Expressions;
USE  Expressions.Variables_Type_Package;


package Parser is




	-- Function to parse a scheduler behavior
	--
	procedure Yyparse;


	-- Initiliaze parser Data
	-- Do initialize Simulation Data all together
	--
     procedure Init_Parser;


     -- Stores the first statement 
     --
	Root_Statement_Pointer : Section_Pointers;



	-- Table to store the list of Variables of 
	-- a parametric scheduler
	--
	Variables_Table : Variables_Table_Type;


	-- Table to store the list of "set" statements
	-- in a parametric scheduler
	--
	Sets_Table : Sets_Table_Type;


	-- Display list of Statements contained
	-- in a section
	--
	procedure Put(S : in Section_Pointers);



end Parser;




WITH Text_Io;
USE Text_Io;
WITH Framework_Config;
USE Framework_Config;
WITH Scheduler_Lex;
USE Scheduler_Lex;
WITH Scheduler_Dfa;
USE Scheduler_Dfa;
WITH Scheduler_Goto; 
USE Scheduler_Goto;
WITH Scheduler_Tokens; 
USE Scheduler_Tokens;
WITH Scheduler_Shift_Reduce;
USE Scheduler_Shift_Reduce;
WITH Ada.Numerics.Aux;
USE Ada.Numerics.Aux;
WITH Translate;          
USE Translate;
WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Unbounded_Strings;
USE Unbounded_Strings;
WITH Ada.Exceptions;
USE Ada.Exceptions;
WITH Simulations;
USE Simulations;
WITH Parameters;
USE Parameters;
USE Parameters.User_Defined_Parameters_Table_Package;
WITH Task_Set;
USE Task_Set;
USE Task_Set.Generic_Task_Set;



package body Parser is


        -- These variables store statements of inner blocks of code
        --
        Else_Inner_Statement_Ptr   : Array (0..Max_Block_Level) of Statement_Ptr;
        Else_Inner_Statement_Index : Natural :=0;
        Then_Inner_Statement_Ptr   : Array (0..Max_Block_Level) of Statement_Ptr;
        Then_Inner_Statement_Index : Natural :=0;
        Loop_Inner_Statement_Ptr   : Array (0..Max_Block_Level) of Statement_Ptr;
        Loop_Inner_Statement_Index : Natural :=0;


        -- Variables needed to parse a "if" or a "for" statement
        --
        New_If     : If_Statement_Ptr;
        Old_Stat   : Statement_Ptr;
        New_Stat   : Statement_Ptr;
        New_for    : For_Statement_Ptr;
        New_While  : While_Statement_Ptr;
        New_Affectation : Affectation_Statement_Ptr;

        procedure Yyerror(S: String) is
        begin
                Raise_Exception(Expressions.Syntax_Error'Identity, "token """ & To_String(
                        To_Unbounded_String(Yytext) & """" & Lb_Comma & " Line" & positive'image(Lines) & Lb_Comma & To_Unbounded_String(S) ));
        end Yyerror;



        -- Store the scheduler behavior step
        -- Used to Check is some Statements can be
        -- declared in the Current_Section
        --
        Current_Section : Sections;

        -- Current Pointers to the parsed code
        -- There is one Pointers per section (table of pointers)
        --
        Current_Statement_Pointer : Section_Pointers;



        -- Procedure used to display content of each
        -- section (Statements and Expressions)
        --
        package Sections_Io is new Text_Io.Enumeration_Io(Sections);
        use Sections_Io;
	
        procedure Put(S : in Section_Pointers) is 
        begin 
		for I in Sections'range loop
			if S(I)/=null 
				then
					Put(" Section : ");
					Put(I); New_Line;	
					Recursive_Put(S(I));
					New_Line;
					New_Line;
			end if;
	    	end loop;
        end Put;


        procedure Init_Parser  is
   

        begin
                Else_Inner_Statement_Index:=0;
                Then_Inner_Statement_Index:=0;
                Loop_Inner_Statement_Index:=0;

                Initialize(Sets_Table);

		      for I in Sections'range loop
			       Root_Statement_Pointer(I):=null;
		      end loop;

                Scheduler_Lex.Lines:=1;

end Init_Parser;





##



end Parser;


