------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------




WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;
WITH Framework_Config;
USE Framework_Config;
WITH Scheduler;
USE Scheduler;
WITH Processors;
USE Processors;
WITH Sets;



PACKAGE Processor_Set IS




   ----------------------------------------------------------
   -- Definition of a set
   ----------------------------------------------------------

   PACKAGE Generic_Processor_Set IS NEW Sets (
      Max_Element => Framework_Config.Max_Processors,
      Element     => Processor_Ptr,
      Free        => Free,
      Copy        => Copy,
      Put         => Put);


   USE Generic_Processor_Set;

   TYPE Processors_Set IS NEW Generic_Processor_Set.Set WITH PRIVATE;


   SUBTYPE Processors_Range IS Generic_Processor_Set.Element_Range;
   SUBTYPE Processors_Iterator IS   Generic_Processor_Set.Iterator;




   ----------------------------------------------------------
   -- Exceptions defined in the set package
   ----------------------------------------------------------

   -- Raised when a given processor is not in a processor set
   --
   Processor_Not_Found : EXCEPTION;

   -- Raised when parameters provided to Add_processor are wrong
   --
   Invalid_Parameter : EXCEPTION;

   -- Raised when the scheduler code of a parametric scheduler
   -- does not have a right syntax
   --
   Scheduler_Syntax_Error : EXCEPTION;




   ----------------------------------------------------------
   -- I/O operations
   ----------------------------------------------------------
   FUNCTION Export_Aadl_Implementations (
         My_Processors : IN     Processors_Set)
     RETURN Unbounded_String;

   FUNCTION Export_Aadl_Declarations (
         My_Processors : IN     Processors_Set;
         Number_Of_Ht  : IN     Natural)
     RETURN Unbounded_String;

   FUNCTION Export_Xml (
         My_Processors : IN     Processors_Set)
     RETURN Unbounded_String;



   ----------------------------------------------------------
   -- Procedure to proceed modification on a set
   ----------------------------------------------------------


   PROCEDURE Check_Processor (
         My_Processors : IN     Processors_Set;
         Name          : IN     Unbounded_String;
         Is_Preemptive : IN     Preemptive_Type;
         Quantum       : IN     Integer;
         File_Name     : IN     Unbounded_String;
         A_Scheduler   : IN     Scheduler_Type;
         A_Network     : IN     Unbounded_String);

   PROCEDURE Add_Processor (
         My_Processors : IN OUT Processors_Set;
         A_Processor   : IN OUT Processor_Ptr;
         Name          : IN     Unbounded_String;
         Is_Preemptive : IN     Preemptive_Type;
         Quantum       : IN     Integer;
         File_Name     : IN     Unbounded_String;
         A_Scheduler   : IN     Scheduler_Type;
         A_Network     : IN     Unbounded_String);

   PROCEDURE Add_Processor (
         My_Processors : IN OUT Processors_Set;
         Name          : IN     Unbounded_String;
         Is_Preemptive : IN     Preemptive_Type;
         Quantum       : IN     Integer;
         File_Name     : IN     Unbounded_String;
         A_Scheduler   : IN     Scheduler_Type;
         A_Network     : IN     Unbounded_String);


   ------------------------------------------------------
   -- Read information from a set
   ------------------------------------------------------


   -- Get the task pointer of the processor named "Name"
   --
   FUNCTION Search_Processor (
         My_Processors : IN     Processors_Set;
         Name          : IN     Unbounded_String)
     RETURN Processor_Ptr;



   ----------------------------------------------------------
   -- List of parameters the user can set of get directly
   -- throught the set
   ----------------------------------------------------------



   TYPE Processor_Parameters IS
         (Scheduler);


   FUNCTION Get (
         My_Processors  : IN     Processors_Set;
         Processor_Name : IN     Unbounded_String;
         Param_Name     : IN     Processor_Parameters)
     RETURN Scheduler_Ptr;

   PROCEDURE Set (
         My_Processors  : IN OUT Processors_Set;
         Processor_Name : IN     Unbounded_String;
         Param_Name     : IN     Processor_Parameters;
         Param_Value    : IN     Scheduler_Ptr);




PRIVATE

   TYPE Processors_Set IS NEW Generic_Processor_Set.Set WITH NULL RECORD;




END Processor_Set;
