------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

package body Queueing_System.theoretical.Md1 is

   procedure Initialize
     (A_Queueing_System : in out Md1_Queueing_System_theoretical)
   is
   begin
      Initialize (Generic_Queueing_System_theoretical (A_Queueing_System));

      A_Queueing_System.Queueing_System_Type := Qs_Md1;
   end Initialize;

   procedure Qs_Average_Waiting_Time
     (A_Queueing_System : in Md1_Queueing_System_theoretical;
      Result            : in out Double)
   is

      Service_Rate : constant Double :=
        A_Queueing_System.Service_Rate.Entries (0).Data;
      Rau          : constant Double :=
         A_Queueing_System.Arrival_Rate.Entries (0).Data /
         A_Queueing_System.Service_Rate.Entries (0).Data;
   begin
      Result := (2.0 - Rau) / (2.0 * Service_Rate * (1.0 - Rau));
   end Qs_Average_Waiting_Time;

   procedure Qs_Average_Number_Customer
     (A_Queueing_System : in Md1_Queueing_System_theoretical;
      Result            : in out Double)
   is
      Rau : constant Double :=
         A_Queueing_System.Arrival_Rate.Entries (0).Data /
         A_Queueing_System.Service_Rate.Entries (0).Data;
   begin
      Result := (2.0 * Rau - Rau * Rau) / (2.0 * (1.0 - Rau));
   end Qs_Average_Number_Customer;

   procedure Qs_Maximum_Number_Customer
     (A_Queueing_System : in Md1_Queueing_System_theoretical;
      Result            : in out Double)
   is
   begin
      Result := 0.0;
      --      raise Not_Implemented;
   end Qs_Maximum_Number_Customer;

   procedure Qs_Maximum_Waiting_Time
     (A_Queueing_System : in Md1_Queueing_System_theoretical;
      Result            : in out Double)
   is
   begin
      Result := 0.0;
      --      raise Not_Implemented;
   end Qs_Maximum_Waiting_Time;

   function Get_Probability_Of_State
     (A_Queueing_System : in Md1_Queueing_System_theoretical;
      N                 : in Natural)
      return              Double
   is
   begin
      raise Not_Implemented;
      return 0.0;
   end Get_Probability_Of_State;

end Queueing_System.theoretical.Md1;
