------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

package body Queueing_System.theoretical.Mg1 is

   procedure Initialize
     (A_Queueing_System : in out Mg1_Queueing_System_theoretical)
   is
   begin
      Initialize (Generic_Queueing_System_theoretical (A_Queueing_System));

      A_Queueing_System.Queueing_System_Type := Qs_Mg1;
   end Initialize;

   procedure Qs_Average_Waiting_Time
     (A_Queueing_System : in Mg1_Queueing_System_theoretical;
      Result            : in out Double)
   is

      Arrival_Rate     : constant Double :=
        A_Queueing_System.Arrival_Rate.Entries (0).Data;
      Avg_Service_Time : constant Double := A_Queueing_System.Avg_Service;
      Var_Service_Time : constant Double := A_Queueing_System.Var_Service;
      Taw              : Double          := 0.0;
   begin
      Taw := Avg_Service_Time +
             (Arrival_Rate * Avg_Service_Time * Avg_Service_Time +
              Arrival_Rate * Var_Service_Time) /
             (2.0 * (1.0 - Arrival_Rate * Avg_Service_Time));

      Result := Taw;
   end Qs_Average_Waiting_Time;

   procedure Qs_Average_Number_Customer
     (A_Queueing_System : in Mg1_Queueing_System_theoretical;
      Result            : in out Double)
   is

      Arrival_Rate     : constant Double :=
        A_Queueing_System.Arrival_Rate.Entries (0).Data;
      Avg_Service_Time : constant Double := A_Queueing_System.Avg_Service;
      Var_Service_Time : constant Double := A_Queueing_System.Var_Service;
      Tao              : Double          := 0.0;
   begin
      Tao := Avg_Service_Time +
             (Arrival_Rate * Avg_Service_Time * Avg_Service_Time +
              Arrival_Rate * Var_Service_Time) /
             (2.0 * (1.0 - Arrival_Rate * Avg_Service_Time));
      Tao := Tao * Arrival_Rate;

      Result := Tao;
   end Qs_Average_Number_Customer;

   procedure Qs_Maximum_Number_Customer
     (A_Queueing_System : in Mg1_Queueing_System_theoretical;
      Result            : in out Double)
   is
   begin
      Result := 0.0;
      --      raise Not_Implemented;
   end Qs_Maximum_Number_Customer;

   function Get_Probability_Of_State
     (A_Queueing_System : in Mg1_Queueing_System_theoretical;
      N                 : in Natural)
      return              Double
   is
   begin
      raise Not_Implemented;
      return 0.0;
   end Get_Probability_Of_State;

   procedure Qs_Maximum_Waiting_Time
     (A_Queueing_System : in Mg1_Queueing_System_theoretical;
      Result            : in out Double)
   is
   begin
      Result := 0.0;
      --      raise Not_Implemented;
   end Qs_Maximum_Waiting_Time;

end Queueing_System.theoretical.Mg1;
