------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

package Queueing_System.theoretical.Mms is

   type Mms_Queueing_System_theoretical is new
     Generic_Queueing_System_theoretical with
      record
         Nb_Tasks : Double;
      end record;
   type Mms_Queueing_System_theoretical_Ptr is access all 
     Generic_Queueing_System_theoretical'Class;

   procedure Initialize
     (A_Queueing_System : in out Mms_Queueing_System_theoretical);

   -- return the theorec average waiting time of one customer in the queue
   --
   procedure Qs_Average_Waiting_Time
     (A_Queueing_System : in Mms_Queueing_System_theoretical;
      Result            : in out Double);

   -- return the theorec average number of customer in the queue
   --
   procedure Qs_Average_Number_Customer
     (A_Queueing_System : in Mms_Queueing_System_theoretical;
      Result            : in out Double);

   -- return the theorec maximum waiting time of one customer in the queue
   --
   procedure Qs_Maximum_Waiting_Time
     (A_Queueing_System : in Mms_Queueing_System_theoretical;
      Result            : in out Double);

   -- return the theorec maximum number of customer in the queue
   --
   procedure Qs_Maximum_Number_Customer
     (A_Queueing_System : in Mms_Queueing_System_theoretical;
      Result            : in out Double);

   -- return the theorec probability to have N customer in the qs
   --
   function Get_Probability_Of_State
     (A_Queueing_System : in Mms_Queueing_System_theoretical;
      N                 : in Natural)
      return              Double;

   -- return the number of tasks for mms queueing system
   --
   function Get_Qs_Nb_Tasks
     (A_Queueing_System : in Mms_Queueing_System_theoretical)
      return              Double;

   -- set the number of tasks for mms queueing system
   --
   procedure Set_Qs_Nb_Tasks
     (A_Queueing_System : in out Mms_Queueing_System_theoretical;
      Value             : Double);

   --P[queueing], Eq. 2.148, Robertazzi
   --
   function Get_Probability_Of_Queueing
     (A_Queueing_System : in Mms_Queueing_System_theoretical)
      return              Double;

end Queueing_System.theoretical.Mms;
