------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

package Queueing_System.theoretical.Mp1 is

   type Mp1_Queueing_System_theoretical is new
     Generic_Queueing_System_theoretical with null record;
   type Mp1_Queueing_System_theoretical_Ptr is access all 
     Generic_Queueing_System_theoretical'Class;

   procedure Initialize
     (A_Queueing_System : in out Mp1_Queueing_System_theoretical);

   -- return the theoric average waiting time of one customer in the queue
   -- based on theoric service time
   --
   procedure Qs_Average_Waiting_Time
     (A_Queueing_System : in Mp1_Queueing_System_theoretical;
      Result            : in out Double);

   -- return the theoric average number of customer in the queue
   -- based on theoric service time
   --
   procedure Qs_Average_Number_Customer
     (A_Queueing_System : in Mp1_Queueing_System_theoretical;
      Result            : in out Double);

   -- return the theoric maximum waiting time of one customer in the queue
   --
   procedure Qs_Maximum_Waiting_Time
     (A_Queueing_System : in Mp1_Queueing_System_theoretical;
      Result            : in out Double);

   -- return the theoric maximum number of customer in the queue
   --
   procedure Qs_Maximum_Number_Customer
     (A_Queueing_System : in Mp1_Queueing_System_theoretical;
      Result            : in out Double);

   -- return the theoric probability to have N customers in the qs
   --
   function Get_Probability_Of_State
     (A_Queueing_System : in Mp1_Queueing_System_theoretical;
      N                 : in Natural)
      return              Double;

   --   -- return the theoric average waiting time of one customer in the queue
   --   -- based on simulated service time
   --   --
   --   procedure Qs_Average_Waiting_Time_Simulation (
   --         A_Queueing_System : in out Mp1_Queueing_System;
   --         Result            : in out Double);

   --   -- return the theoric average number of customer in the queue
   --   -- based on simulated service time
   --   --
   --   procedure Qs_Average_Number_Customer_Simulation (
   --         A_Queueing_System : in     Mp1_Queueing_System_theoretical;
   --         Arrival_Rate      : in     Double;
   --         Service_Rate      : in     Double;
   --         Result            : in out Double               );

end Queueing_System.theoretical.Mp1;
