------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Numerics.Float_Random; use Ada.Numerics.Float_Random;
with Ada.Numerics.Aux;          use Ada.Numerics.Aux;

package Random_Tools is

   -- return a random parameter between min and max
   function Get_Rand_Parameter
     (Min  : in Natural;
      Max  : in Natural;
      Seed : in Generator)
      return Natural;

   function Get_Rand_Parameter
     (Min  : in Double;
      Max  : in Double;
      Seed : in Generator)
      return Double;

   -- generate poisson inter arrival time (different from 0)
   --
   function Get_Exponential_Time
     (Arrival_Rate : in Double;
      Seed         : in Generator)
      return         Double;

   -- tirage aleatoire d'une valeur
   -- uniformement repartie entre 0 et 1
   --
   function Drand48 return Long_Float;
   pragma Import (C, Drand48);

   -- Initialize la graine a "value"
   --
   procedure Srand48 (Value : in Integer);
   pragma Import (C, Srand48);

end Random_Tools;
