------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Xml_Tag;           use Xml_Tag;
with Double_Util;       use Double_Util;
with Translate;         use Translate;
with Unbounded_Strings; use Unbounded_Strings;

package body Scheduler.Dynamic_Priority.Muf is

   procedure Utilization_Factor_Feasibility_Test
     (My_Scheduler   : in Muf_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String;
      Result         : in out Unbounded_String)
   is

      Tmp               : Tasks_Set;
      Bound             : Double           := 0.0;
      Utildi            : Double           := 0.0;
      Msg               : Unbounded_String := Empty_String;
      Verif             : Boolean          := True;
      A_Task            : Generic_Task_Ptr;
      My_Task_Iterator  : Tasks_Iterator;
      A_Task2           : Generic_Task_Ptr;
      My_Task_Iterator2 : Tasks_Iterator;
      Calcul            : Double           := 0.0;
      L_Debut           : Natural;
      L_Dernier         : Natural;
      Ech_Sur_Req       : Boolean          := True;
      Pbconstraints     : Boolean          := False;
      Msg2              : Unbounded_String := Empty_String;
      Msg3              : Unbounded_String := Empty_String;

   begin

      case Get_Preemptive (My_Scheduler) is

         when Preemptive =>
            Bound_On_Processor_Utilization
              (My_Scheduler,
               My_Tasks,
               Processor_Name,
               Bound,
               Msg);

            Utildi :=
               Processor_Utilization_Over_Deadline (My_Tasks, Processor_Name);

            Result := Result & Lb_Utilization_Bound1 (Current_Language);
            Result := Result & Lb_Muf (Current_Language);

            if Utildi <= Bound then
               Result := Result & Lb_Sched_Explanation1 (Current_Language);
               Result := Result &
                         Start_Fact_Bound &
                         Format (Utildi) &
                         End_Fact_Bound;
               Result := Result & Lb_Sched_Explanation12 (Current_Language);
               Result := Result &
                         Start_Fact_Bound &
                         Format (Bound) &
                         End_Fact_Bound;
            else
               Result := Result & Lb_Sched_Explanation2 (Current_Language);
               Result := Result &
                         Start_Fact_Bound &
                         Format (Utildi) &
                         End_Fact_Bound;
               Result := Result & Lb_Sched_Explanation22 (Current_Language);
               Result := Result &
                         Start_Fact_Bound &
                         Format (Bound) &
                         End_Fact_Bound;
            end if;
            Result := Result & Start_Ref & Msg & End_Ref & Unbounded_Lf;

         when Not_Preemptive =>

            Current_Processor_Name := Processor_Name;
            Select_And_Copy (My_Tasks, Tmp, Select_Cpu'Access);

            Msg2 := Empty_String;

            --Check if the system is in stricts constraints
            --The test works only on stricts constraints !
            --
            Reset_Iterator (Tmp, My_Task_Iterator);
            loop
               Current_Element (Tmp, A_Task, My_Task_Iterator);
               if (A_Task.Cpu_Name = Processor_Name) then
                  if (Periodic_Task_Ptr (A_Task).Period /=
                      A_Task.Deadline)
                  then
                     Ech_Sur_Req := False;
                  end if;
               end if;
               exit when Is_Last_Element (Tmp, My_Task_Iterator);
               Next_Element (Tmp, My_Task_Iterator);
            end loop;

            Sort (Tmp, Increasing_Deadline'Access);
            Reset_Iterator (Tmp, My_Task_Iterator);
            Current_Element (Tmp, A_Task, My_Task_Iterator);
            L_Debut := Periodic_Task_Ptr (A_Task).Period + 1;
            Verif   := True;

            -- Main loop of all tasks
            --
            if Ech_Sur_Req then

               Result := Result &
                         Lb_Utilization_Bound11 (Current_Language) &
                         Lb_Muf (Current_Language);
               loop
                  Msg3 := Empty_String;
                  if (A_Task.Cpu_Name = Processor_Name) then
                     L_Dernier := Periodic_Task_Ptr (A_Task).Period - 1;
                     for L_Courant in  L_Debut .. L_Dernier loop
                        Reset_Iterator (Tmp, My_Task_Iterator2);

                        -- Second loop until the current task
                        --
                        Calcul := 0.0;
                        loop
                           Current_Element (Tmp, A_Task2, My_Task_Iterator2);
                           exit when A_Task.Name = A_Task2.Name;
                           Calcul := Calcul +
                                     Double ((Integer ((L_Courant - 1) /
                                                       Periodic_Task_Ptr (
                             A_Task2).Period)) *
                                             A_Task2.Capacity);
                           Next_Element (Tmp, My_Task_Iterator2);
                        end loop;

                        -- Checking
                        --
                        Calcul := Calcul + Double (A_Task.Capacity);
                        if Calcul > Double (L_Courant) then
                           Verif := False;
                           Msg3  := A_Task.Name & " ";
                        end if;
                     end loop;
                  end if;
                  Msg2 := Msg2 & Msg3;
                  exit when Is_Last_Element (Tmp, My_Task_Iterator);
                  Next_Element (Tmp, My_Task_Iterator);
                  Current_Element (Tmp, A_Task, My_Task_Iterator);
               end loop;
            else
               Result        := Result &
                                Lb_Pb_Strict_Constraints (Current_Language) &
                                Unbounded_Lf;
               Pbconstraints := True;
            end if;

            -- Checks the result
            --
            if Pbconstraints = False then
               if Verif then
                  Result := Result &
                            Lb_Sched_Explanation5 (Current_Language);
                  Result := Result &
                            Lb_Sched_Explanation52 (Current_Language);
                  Result := Result &
                            To_Unbounded_String (" (") &
                            Lb_See (Current_Language) &
                            "[8].)" &
                            Unbounded_Lf;
               else
                  Result := Result &
                            Lb_Sched_Explanation6 (Current_Language);
                  Result :=
                    Result &
                    Lb_Sched_Explanation62 (Current_Language) &
                    To_Unbounded_String (" (") &
                    Lb_See (Current_Language) &
                    "[8])." &
                    Unbounded_Lf &
                    Lb_Sched_Explanation7 (Current_Language) &
                    Msg2 &
                    Unbounded_Lf;

               end if;
            end if;

      end case;

   end Utilization_Factor_Feasibility_Test;

end Scheduler.Dynamic_Priority.Muf;
