------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



PACKAGE Scheduler.Dynamic_Priority IS


   TYPE Dynamic_Priority_Scheduler IS ABSTRACT NEW Generic_Scheduler WITH PRIVATE;
   TYPE Dynamic_Priority_Scheduler_Ptr IS ACCESS ALL Dynamic_Priority_Scheduler'Class;


   TYPE Dynamic_Priority_Tcb IS NEW Tcb WITH
      RECORD
         Dynamic_Deadline : Natural;
      END RECORD;
   TYPE Dynamic_Priority_Tcb_Ptr IS ACCESS ALL Dynamic_Priority_Tcb'Class;


   PROCEDURE Initialize (
         A_Tcb : IN OUT Dynamic_Priority_Tcb);


   FUNCTION Build_Tcb (
         My_Scheduler : IN     Dynamic_Priority_Scheduler;
         A_Task       :        Generic_Task_Ptr)
     RETURN Tcb_Ptr;


   PROCEDURE Check_Before_Scheduling (
         My_Scheduler   : IN     Dynamic_Priority_Scheduler;
         My_Tasks       : IN     Tasks_Set;
         Processor_Name : IN     Unbounded_String);


   PROCEDURE Specific_Scheduler_Initialization (
         My_Scheduler   : IN OUT Dynamic_Priority_Scheduler;
         Si             : IN OUT Scheduling_Information;
         Processor_Name : IN     Unbounded_String;
         My_Tasks       : IN OUT Tasks_Set;
         My_Resources   : IN OUT Resources_Set;
         My_Buffers     : IN OUT Buffers_Set;
         My_Messages    : IN     Messages_Set;
         Msg            : IN OUT Unbounded_String);


   PROCEDURE Bound_On_Processor_Utilization (
         My_Scheduler   : IN     Dynamic_Priority_Scheduler;
         My_Tasks       : IN     Tasks_Set;
         Processor_Name : IN     Unbounded_String;
         Result         : IN OUT Double;
         Msg            : IN OUT Unbounded_String);

   PROCEDURE Compute_Blocking_Time (
         My_Scheduler   : IN     Dynamic_Priority_Scheduler;
         My_Tasks       : IN     Tasks_Set;
         My_Resources   : IN     Resources_Set;
         Processor_Name : IN     Unbounded_String;
         Msg            : IN OUT Unbounded_String;
         Blocking_Time  :    OUT Blocking_Time_Table);

   PROCEDURE Compute_Response_Time (
         My_Scheduler   : IN     Dynamic_Priority_Scheduler;
         My_Tasks       : IN OUT Tasks_Set;
         Processor_Name : IN     Unbounded_String;
         Msg            : IN OUT Unbounded_String;
         Response_Time  :    OUT Response_Time_Table);




PRIVATE

   TYPE Dynamic_Priority_Scheduler IS ABSTRACT NEW Generic_Scheduler WITH NULL RECORD;

END Scheduler.Dynamic_Priority ;




