------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Fifos;



PACKAGE Scheduler.Fixed_Priority.Hpf IS


   TYPE Hpf_Scheduler IS NEW Fixed_Priority_Scheduler WITH PRIVATE;
   TYPE Hpf_Scheduler_Ptr IS ACCESS ALL Hpf_Scheduler'Class;


   TYPE Hpf_Tcb IS NEW Fixed_Priority_Tcb WITH
      RECORD
         Task_Quantum : Natural := 0;
      END RECORD;

   TYPE Hpf_Tcb_Ptr IS ACCESS ALL Hpf_Tcb'Class;


   PROCEDURE Initialize (
         A_Tcb : IN OUT Hpf_Tcb);


   PROCEDURE Initialize (
         A_Scheduler : IN OUT Hpf_Scheduler);


   FUNCTION Copy (
         A_Scheduler : IN     Hpf_Scheduler)
     RETURN Scheduler_Ptr;




   PROCEDURE Check_Before_Scheduling (
         My_Scheduler   : IN     Hpf_Scheduler;
         My_Tasks       : IN     Tasks_Set;
         Processor_Name : IN     Unbounded_String);


   PROCEDURE Compute_Response_Time (
         My_Scheduler   : IN     Hpf_Scheduler;
         My_Tasks       : IN OUT Tasks_Set;
         Processor_Name : IN     Unbounded_String;
         Msg            : IN OUT Unbounded_String;
         Response_Time  :    OUT Response_Time_Table);


   FUNCTION Build_Tcb (
         My_Scheduler : IN     Hpf_Scheduler;
         A_Task       :        Generic_Task_Ptr)
     RETURN Tcb_Ptr;


   PROCEDURE Do_Election (
         My_Scheduler      : IN OUT Hpf_Scheduler;
         Si                : IN OUT Scheduling_Information;
         Result            : IN OUT Scheduling_Sequence_Ptr;
         Msg               : IN OUT Unbounded_String;
         Current_Time      : IN     Natural;
         Processor_Name    : IN     Unbounded_String;
         My_Dependencies   : IN     Tasks_Dependencies_Ptr;
         With_Offsets      : IN     Boolean;
         With_Precedencies : IN     Boolean;
         With_Resources    : IN     Boolean;
         Event_To_Generate : IN     Time_Unit_Event_Type_Boolean_Table;
         Elected           : IN OUT Tasks_Range;
         No_Task           : IN OUT Boolean);



   PROCEDURE Specific_Scheduler_Initialization (
         My_Scheduler   : IN OUT Hpf_Scheduler;
         Si             : IN OUT Scheduling_Information;
         Processor_Name : IN     Unbounded_String;
         My_Tasks       : IN OUT Tasks_Set;
         My_Resources   : IN OUT Resources_Set;
         My_Buffers     : IN OUT Buffers_Set;
         My_Messages    : IN     Messages_Set;
         Msg            : IN OUT Unbounded_String);


   PROCEDURE Utilization_Factor_Feasibility_Test (
         My_Scheduler   : IN     Hpf_Scheduler;
         My_Tasks       : IN     Tasks_Set;
         Processor_Name : IN     Unbounded_String;
         Result         : IN OUT Unbounded_String);


   PROCEDURE Dispatched_Change_Current_Priority (
         My_Scheduler : IN OUT Hpf_Scheduler;
         A_Tcb        : IN     Fixed_Priority_Tcb_Ptr;
         New_Priority :        Priority_Range);



PRIVATE


   PROCEDURE Put_Tcb (
         E : Hpf_Tcb_Ptr);

   PACKAGE Tcb_Fifos IS NEW Fifos(Framework_Config.Max_Tasks,
      Hpf_Tcb_Ptr, Put_Tcb);
   USE Tcb_Fifos;

   TYPE Fifo_Table IS ARRAY (Priority_Range) OF Fifo;

   TYPE Hpf_Scheduler IS NEW Fixed_Priority_Scheduler WITH
      RECORD
         Priority_Fifos : Fifo_Table;
      END RECORD;




END Scheduler.Fixed_Priority.Hpf ;



