------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Unbounded_Strings; use Unbounded_Strings;
with Translate;         use Translate;

package body Scheduler.Time_Sharing_Based_On_Cpu_Usage is

   procedure Initialize
     (A_Scheduler : in out Time_Sharing_Based_On_Cpu_Usage_Scheduler)
   is
   begin
      Reset (A_Scheduler);
      A_Scheduler.Name := Time_Sharing_Based_On_Cpu_Usage_Protocol;
   end Initialize;

   function Copy
     (A_Scheduler : in Time_Sharing_Based_On_Cpu_Usage_Scheduler)
      return        Scheduler_Ptr
   is
      Ptr : Time_Sharing_Based_On_Cpu_Usage_Scheduler_Ptr;

   begin

      Ptr := new Time_Sharing_Based_On_Cpu_Usage_Scheduler;

      Ptr.Name               := A_Scheduler.Name;
      Ptr.Is_Preemptive      := A_Scheduler.Is_Preemptive;
      Ptr.Quantum            := A_Scheduler.Quantum;
      Ptr.Previously_Elected := A_Scheduler.Previously_Elected;

      return Scheduler_Ptr (Ptr);

   end Copy;

   procedure Compute_Blocking_Time
     (My_Scheduler   : in Time_Sharing_Based_On_Cpu_Usage_Scheduler;
      My_Tasks       : in Tasks_Set;
      My_Resources   : in Resources_Set;
      Processor_Name : in Unbounded_String;
      Msg            : in out Unbounded_String;
      Blocking_Time  : out Blocking_Time_Table)
   is
   begin
      raise Invalid_Scheduler;
   end Compute_Blocking_Time;

   procedure Bound_On_Processor_Utilization
     (My_Scheduler   : in Time_Sharing_Based_On_Cpu_Usage_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String;
      Result         : in out Double;
      Msg            : in out Unbounded_String)
   is

   begin
      raise Invalid_Scheduler;
   end Bound_On_Processor_Utilization;

   procedure Compute_Response_Time
     (My_Scheduler   : in Time_Sharing_Based_On_Cpu_Usage_Scheduler;
      My_Tasks       : in out Tasks_Set;
      Processor_Name : in Unbounded_String;
      Msg            : in out Unbounded_String;
      Response_Time  : out Response_Time_Table)
   is

   begin
      raise Invalid_Scheduler;
   end Compute_Response_Time;

   procedure Put
     (My_Scheduler : in Time_Sharing_Based_On_Cpu_Usage_Scheduler)
   is
   begin
      Put (Generic_Scheduler (My_Scheduler));
   end Put;

   procedure Check_Before_Scheduling
     (My_Scheduler   : in Time_Sharing_Based_On_Cpu_Usage_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String)
   is

   begin
      null;
   end Check_Before_Scheduling;

   procedure Do_Election
     (My_Scheduler      : in out Time_Sharing_Based_On_Cpu_Usage_Scheduler;
      Si                : in out Scheduling_Information;
      Result            : in out Scheduling_Sequence_Ptr;
      Msg               : in out Unbounded_String;
      Current_Time      : in Natural;
      Processor_Name    : in Unbounded_String;
      My_Dependencies   : in Tasks_Dependencies_Ptr;
      With_Offsets      : in Boolean;
      With_Precedencies : in Boolean;
      With_Resources    : in Boolean;
      Event_To_Generate : in Time_Unit_Event_Type_Boolean_Table;
      Elected           : in out Tasks_Range;
      No_Task           : in out Boolean)
   is

      Less_Executed_Task : Natural     := Natural'Last;
      I                  : Tasks_Range := 0;

   begin

      loop
         if (Si.Tcbs (I).Tsk.Cpu_Name = Processor_Name) then
            if (Si.Tcbs (I).Wake_Up_Time <= Current_Time) and
               (Si.Tcbs (I).Used_Cpu < Less_Executed_Task) and
               (Si.Tcbs (I).Rest_Of_Capacity /= 0)
            then
               if (With_Offsets = False) or
                  Check_Offset (Si.Tcbs (I), Current_Time)
               then

                  if (With_Precedencies = False) or
                     Check_Precedencies
                        (Si,
                         My_Dependencies,
                         Current_Time,
                         Si.Tcbs (I))
                  then

                     Less_Executed_Task := Si.Tcbs (I).Used_Cpu;
                     Elected            := I;
                  end if;
               end if;
            end if;
         end if;

         I := I + 1;
         exit when Si.Tcbs (I) = null;
      end loop;

      if Less_Executed_Task = Natural'Last then
         No_Task := True;
      else
         No_Task := False;
      end if;

   end Do_Election;

   procedure Specific_Scheduler_Initialization
     (My_Scheduler   : in out Time_Sharing_Based_On_Cpu_Usage_Scheduler;
      Si             : in out Scheduling_Information;
      Processor_Name : in Unbounded_String;
      My_Tasks       : in out Tasks_Set;
      My_Resources   : in out Resources_Set;
      My_Buffers     : in out Buffers_Set;
      My_Messages    : in Messages_Set;
      Msg            : in out Unbounded_String)
   is

   begin
      null;
   end Specific_Scheduler_Initialization;

   procedure Compute_Activation_Time
     (My_Scheduler : in Time_Sharing_Based_On_Cpu_Usage_Scheduler;
      Si           : in out Scheduling_Information;
      Elected      : in Tasks_Range;
      Value        : in out Natural)
   is

   begin
      raise Invalid_Scheduler;
   end Compute_Activation_Time;

   procedure Utilization_Factor_Feasibility_Test
     (My_Scheduler   : in Time_Sharing_Based_On_Cpu_Usage_Scheduler;
      My_Tasks       : in Tasks_Set;
      Processor_Name : in Unbounded_String;
      Result         : in out Unbounded_String)
   is

   begin
      Result := Result &
                Lb_Compute_Scheduling_Error_19 (Current_Language) &
                Unbounded_Lf;
   end Utilization_Factor_Feasibility_Test;

end Scheduler.Time_Sharing_Based_On_Cpu_Usage;
