------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Unbounded_Strings;
USE Unbounded_Strings;



PACKAGE Scheduler.user_defined.compiled  IS



   TYPE compiled_user_defined_Scheduler IS NEW user_defined_Scheduler WITH PRIVATE;
   TYPE compiled_user_defined_Scheduler_Ptr IS ACCESS ALL compiled_user_defined_Scheduler'Class;



   PROCEDURE Initialize (
         A_Scheduler : IN OUT compiled_user_defined_Scheduler);


   FUNCTION Copy (
         A_Scheduler : IN     compiled_user_defined_Scheduler)
     RETURN Scheduler_Ptr;


   PROCEDURE Set_Behavior_File_Name (
         My_Scheduler : IN OUT compiled_user_defined_Scheduler;
         To_Set       : IN     Unbounded_String);


   FUNCTION Get_Behavior_File_Name (
         My_Scheduler : IN     compiled_user_defined_Scheduler)
     RETURN Unbounded_String;


   PROCEDURE Set_String_Behavior (
         My_Scheduler : IN OUT compiled_user_defined_Scheduler;
         To_Set       : IN     Unbounded_String);


   FUNCTION Get_String_Behavior (
         My_Scheduler : IN     compiled_user_defined_Scheduler)
     RETURN Unbounded_String;


   FUNCTION Get_String_Behavior (
         My_Scheduler : IN     compiled_user_defined_Scheduler)
     RETURN Unbounded_String_Ptr;


   -- Procedure to load/save values of user_defined variables
   -- which can be changed bu the programmer
   --
   PROCEDURE Save_Read_Write_user_defined_Variables (
         My_Scheduler   : IN     compiled_user_defined_Scheduler;
         Si             : IN OUT Scheduling_Information;
         Processor_Name : IN     Unbounded_String);
   PROCEDURE Load_Read_Write_user_defined_Variables (
         My_Scheduler   : IN OUT compiled_user_defined_Scheduler;
         Si             : IN     Scheduling_Information;
         Processor_Name : IN     Unbounded_String);


   PROCEDURE Put (
         My_Scheduler : IN     compiled_user_defined_Scheduler);



   FUNCTION Export_Aadl_Properties (
         My_Scheduler : IN     compiled_user_defined_Scheduler;
         Number_Of_Ht : IN     Natural)
     RETURN Unbounded_String;


   FUNCTION Export_Xml (
         My_Scheduler : IN     compiled_user_defined_Scheduler)
     RETURN Unbounded_String;



   PROCEDURE Do_Election (
         My_Scheduler      : IN OUT compiled_user_defined_Scheduler;
         Si                : IN OUT Scheduling_Information;
         Result            : IN OUT Scheduling_Sequence_Ptr;
         Msg               : IN OUT Unbounded_String;
         Current_Time      : IN     Natural;
         Processor_Name    : IN     Unbounded_String;
         My_Dependencies   : IN     Tasks_Dependencies_Ptr;
         With_Offsets      : IN     Boolean;
         With_Precedencies : IN     Boolean;
         With_Resources    : IN     Boolean;
         Event_To_Generate : IN     Time_Unit_Event_Type_Boolean_Table;
         Elected           : IN OUT Tasks_Range;
         No_Task           : IN OUT Boolean);


   PROCEDURE Specific_Scheduler_Initialization (
         My_Scheduler   : IN OUT compiled_user_defined_Scheduler;
         Si             : IN OUT Scheduling_Information;
         Processor_Name : IN     Unbounded_String;
         My_Tasks       : IN OUT Tasks_Set;
         My_Resources   : IN OUT Resources_Set;
         My_Buffers     : IN OUT Buffers_Set;
         My_Messages    : IN     Messages_Set;
         Msg            : IN OUT Unbounded_String);



   PROCEDURE Compute_Activation_Time (
         My_Scheduler : IN     compiled_user_defined_Scheduler;
         Si           : IN OUT Scheduling_Information;
         Elected      : IN     Tasks_Range;
         Value        : IN OUT Natural);

PRIVATE

   TYPE compiled_user_defined_Scheduler IS NEW user_defined_Scheduler WITH
      RECORD

         -- Scheduler behavior in string format
         --
         String_Behavior : Unbounded_String_Ptr;

         -- Scheduler behavior in string format
         --
         Behavior_File_Name : Unbounded_String;

         -- Statements for each scheduling
         -- section (pointers given by the parser)
         --
         Root_Statement_Pointer : Section_Pointers;

         -- Variables table for the parsed scheduler
         -- (given by the parser)
         --
         Variables_Table : Variables_Table_Type;


         -- Table of set statements given by the parser
         --
         Sets_Table : Sets_Table_Type;
      END RECORD;



END Scheduler.user_defined.compiled;
