------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


WITH Expressions;
USE Expressions;
USE Expressions.Variables_Type_Package;
WITH Statements;
USE Statements;
USE Statements.Sets_Type_Package;





PACKAGE Scheduler.user_defined IS



   TYPE user_defined_Scheduler IS abstract NEW Generic_Scheduler WITH null record;
   TYPE user_defined_Scheduler_Ptr IS ACCESS ALL user_defined_Scheduler'Class;



   PROCEDURE Bound_On_Processor_Utilization (
         My_Scheduler   : IN     user_defined_Scheduler;
         My_Tasks       : IN     Tasks_Set;
         Processor_Name : IN     Unbounded_String;
         Result         : IN OUT Double;
         Msg            : IN OUT Unbounded_String);


   PROCEDURE Compute_Blocking_Time (
         My_Scheduler   : IN     user_defined_Scheduler;
         My_Tasks       : IN     Tasks_Set;
         My_Resources   : IN     Resources_Set;
         Processor_Name : IN     Unbounded_String;
         Msg            : IN OUT Unbounded_String;
         Blocking_Time  :    OUT Blocking_Time_Table);


   PROCEDURE Compute_Response_Time (
         My_Scheduler   : IN     user_defined_Scheduler;
         My_Tasks       : IN OUT Tasks_Set;
         Processor_Name : IN     Unbounded_String;
         Msg            : IN OUT Unbounded_String;
         Response_Time  :    OUT Response_Time_Table);


   PROCEDURE Utilization_Factor_Feasibility_Test (
         My_Scheduler   : IN     user_defined_Scheduler;
         My_Tasks       : IN     Tasks_Set;
         Processor_Name : IN     Unbounded_String;
         Result         : IN OUT Unbounded_String);




   PROCEDURE Check_Before_Scheduling (
         My_Scheduler   : IN     user_defined_Scheduler;
         My_Tasks       : IN     Tasks_Set;
         Processor_Name : IN     Unbounded_String);




END Scheduler.user_defined ;


