------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Ada.Text_Io;
USE Ada.Text_Io;
WITH Ada.Strings.Unbounded;
USE Ada.Strings.Unbounded;



PACKAGE BODY Time_Units IS



   PROCEDURE Put (
         E : IN     Time_Unit_Event_Ptr) IS
   BEGIN
      IF (E/=NULL) THEN
         Put(E.Event_Type'Img & '/');
         CASE E.Event_Type IS
            WHEN Start_Of_Task_Capacity =>
               Put(To_String(E.Start_Task.Name));
            WHEN End_Of_Task_Capacity =>
               Put(To_String(E.End_Task.Name));
            WHEN Write_To_Buffer =>
               Put(To_String(E.Write_Buffer.Name));
               Put("/");
               Put(To_String(E.Write_Task.Name));
               Put("/" & E.Write_Size'Img);
            WHEN Read_From_Buffer =>
               Put(To_String(E.Read_Buffer.Name));
               Put("/");
               Put(To_String(E.Read_Task.Name));
               Put("/" & E.Read_Size'Img);
            WHEN Running_Task =>
               Put(To_String(E.Running_Task.Name));
            WHEN Task_Activation =>
               Put(To_String(E.Activation_Task.Name));
            WHEN Allocate_Resource =>
               Put(To_String(E.Allocate_Resource.Name));
               Put("/");
               Put(To_String(E.Allocate_Task.Name));
            WHEN Release_Resource =>
               Put(To_String(E.Release_Resource.Name));
               Put("/");
               Put(To_String(E.Release_Task.Name));
            WHEN Wait_For_Resource =>
               Put(To_String(E.Wait_Resource.Name));
               Put("/");
               Put(To_String(E.Wait_Task.Name));
            WHEN Send_Message =>
               Put(To_String(E.Send_Message.Name));
            WHEN Receive_Message =>
               Put(To_String(E.Receive_Message.Name));
         END CASE;
      END IF;
   END Put;

   PROCEDURE Put_Line (
         E : IN     Time_Unit_Event_Ptr) IS
   BEGIN
      IF (E /= NULL) THEN
         Put(E);
         New_Line;
      END IF;
   END Put_Line;


   PROCEDURE Initialize (
         E : IN OUT Time_Unit_Event_Ptr) IS
   BEGIN
      E := NULL;
   END Initialize;


END Time_Units;


