------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Events;                 use Events;
with Framework_Config;       use Framework_Config;
with Natural_Util;           use Natural_Util;
with Tasks;                  use Tasks;
with Resources;              use Resources;
with Buffers;                use Buffers;
with Messages;               use Messages;
with Indexed_Tables;
with Lists;
with Unchecked_Deallocation;

package Time_Units is

   type Time_Unit_Event (Event_Type : Time_Unit_Event_Type) is record
      case Event_Type is
      when Start_Of_Task_Capacity =>
         Start_Task : Generic_Task_Ptr;

      when End_Of_Task_Capacity =>
         End_Task : Generic_Task_Ptr;

      when Write_To_Buffer =>
         Write_Buffer : Buffer_Ptr;
         Write_Task   : Generic_Task_Ptr;
         Write_Size   : Natural;

      when Read_From_Buffer =>
         Read_Buffer : Buffer_Ptr;
         Read_Task   : Generic_Task_Ptr;
         Read_Size   : Natural;

      when Running_Task =>
         Running_Task     : Generic_Task_Ptr;
         Current_Priority : Priority_Range;

      when Task_Activation =>
         Activation_Task : Generic_Task_Ptr;

      when Allocate_Resource =>
         Allocate_Resource : Generic_Resource_Ptr;
         Allocate_Task     : Generic_Task_Ptr;

      when Release_Resource =>
         Release_Resource : Generic_Resource_Ptr;
         Release_Task     : Generic_Task_Ptr;

      when Wait_For_Resource =>
         Wait_Resource : Generic_Resource_Ptr;
         Wait_Task     : Generic_Task_Ptr;

      when Send_Message =>
         Send_Message : Message_Ptr;
         Send_Task    : Generic_Task_Ptr;

      when Receive_Message =>
         Receive_Message : Message_Ptr;
         Receive_Task    : Generic_Task_Ptr;

      end case;
   end record;

   type Time_Unit_Event_Ptr is access Time_Unit_Event;

   procedure Put (E : in Time_Unit_Event_Ptr);
   procedure Put_Line (E : in Time_Unit_Event_Ptr);
   procedure Initialize (E : in out Time_Unit_Event_Ptr);

   procedure Free is new Unchecked_Deallocation (
      Time_Unit_Event,
      Time_Unit_Event_Ptr);

   package Time_Unit_List is new Lists (Time_Unit_Event_Ptr, Put, Free);
   use Time_Unit_List;

   package A_Time_Unit is new Indexed_Tables (
      Natural,
      Time_Unit_Event_Ptr,
      Framework_Config.Max_Scheduling_Period,
      0,
      Natural_Util.Put,
      Natural_Util.Initialize,
      Put);
   use A_Time_Unit;

   subtype Time_Unit_Table is Indexed_Table;
   subtype Time_Unit_Range is Indexed_Table_Range;

end Time_Units;
