------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

WITH Sax.Exceptions;
WITH Sax.Locators;
WITH Sax.Readers;
WITH Sax.Attributes;
WITH Unicode.Ces;
WITH Systems;
USE Systems;
with Processor_Set;      use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Multiprocessor_Services; use Multiprocessor_Services;
use Multiprocessor_Services.Scheduling_Result_Per_Processor;



PACKAGE Xml_Parsers IS


   --------------------------------------------------------------------------------
   --------------------------------------------------------------------------------
   ----------------    Mandatory code for all XML parser of Cheddar
   --------------------------------------------------------------------------------
   --------------------------------------------------------------------------------



   -- Exception raised by clients of Xml_Parses when something goes
   -- wrong during XML parsing
   --
   Xml_Read_Error : EXCEPTION;



   -- Generic XML parser
   --
   TYPE Xml_Generic_Parser IS NEW Sax.Readers.Reader WITH PRIVATE;


   -- Methods for all Cheddar XML parsers
   --
   PROCEDURE Characters (
         Handler : IN OUT Xml_Generic_Parser;
         Ch      :        Unicode.Ces.Byte_Sequence);
   PROCEDURE Set_Document_Locator (
         Handler : IN OUT Xml_Generic_Parser;
         Loc     : ACCESS Sax.Locators.Locator'Class);
   PROCEDURE Error (
         Handler : IN OUT Xml_Generic_Parser;
         Except  :        Sax.Exceptions.Sax_Parse_Exception'Class);
   PROCEDURE Fatal_Error (
         Handler : IN OUT Xml_Generic_Parser;
         Except  :        Sax.Exceptions.Sax_Parse_Exception'Class);
   PROCEDURE Warning (
         Handler : IN OUT Xml_Generic_Parser;
         Except  :        Sax.Exceptions.Sax_Parse_Exception'Class);



   --------------------------------------------------------------------------------
   --------------------------------------------------------------------------------
   ----------------    Necessary code to parse XML Cheddar project files
   --------------------------------------------------------------------------------
   --------------------------------------------------------------------------------


   -- This tagged type is the parser ...
   --
   TYPE Xml_Project_Parser IS NEW Xml_Generic_Parser WITH PRIVATE;


   -- Override methods of the Reader to custom the treatement the
   -- parser have to do
   --
   PROCEDURE Start_Document (
         Handler : IN OUT Xml_Project_Parser);
   PROCEDURE Start_Element (
         Handler       : IN OUT Xml_Project_Parser;
         Namespace_Uri :        Unicode.Ces.Byte_Sequence       := "";
         Local_Name    :        Unicode.Ces.Byte_Sequence       := "";
         Qname         :        Unicode.Ces.Byte_Sequence       := "";
         Atts          :        Sax.Attributes.Attributes'Class);
   PROCEDURE End_Element (
         Handler       : IN OUT Xml_Project_Parser;
         Namespace_Uri :        Unicode.Ces.Byte_Sequence := "";
         Local_Name    :        Unicode.Ces.Byte_Sequence := "";
         Qname         :        Unicode.Ces.Byte_Sequence := "");

   PROCEDURE Initialize_Project_Parser (
         Handler : IN OUT Xml_Project_Parser);
   FUNCTION Get_Parsed_System (
         Handler : IN     Xml_Project_Parser)
     RETURN System;


   --------------------------------------------------------------------------------
   --------------------------------------------------------------------------------
   ----------------    Necessary code to parse XML Cheddar project files
   --------------------------------------------------------------------------------
   --------------------------------------------------------------------------------




   -- This tagged type is the parser ...
   --
   TYPE Xml_Event_Table_Parser IS NEW Xml_Generic_Parser WITH PRIVATE;


   -- Override methods of the Reader to custom the treatement the
   -- parser have to do
   --
   PROCEDURE Start_Document (
         Handler : IN OUT Xml_Event_Table_Parser);
   PROCEDURE Start_Element (
         Handler       : IN OUT Xml_Event_Table_Parser;
         Namespace_Uri :        Unicode.Ces.Byte_Sequence       := "";
         Local_Name    :        Unicode.Ces.Byte_Sequence       := "";
         Qname         :        Unicode.Ces.Byte_Sequence       := "";
         Atts          :        Sax.Attributes.Attributes'Class);
   PROCEDURE End_Element (
         Handler       : IN OUT Xml_Event_Table_Parser;
         Namespace_Uri :        Unicode.Ces.Byte_Sequence := "";
         Local_Name    :        Unicode.Ces.Byte_Sequence := "";
         Qname         :        Unicode.Ces.Byte_Sequence := "");

   PROCEDURE Initialize_Event_Table_Parser (
         Handler : IN OUT Xml_Event_Table_Parser);
   PROCEDURE Read_Time_Value (
         Handler : IN OUT Xml_Event_Table_Parser);
   PROCEDURE Set_Scheduled_System (
         Handler          : IN OUT Xml_Event_Table_Parser;
         Scheduled_System : IN     System);
   FUNCTION Get_Parsed_Event_Table (
         Handler : IN     Xml_Event_Table_Parser)
     RETURN Scheduling_Table_Ptr;


PRIVATE

   TYPE Xml_Generic_Parser IS NEW Sax.Readers.Reader WITH
      RECORD
         Locator : Sax.Locators.Locator_Access;
      END RECORD;



   TYPE Xml_Project_Parser IS NEW Xml_Generic_Parser WITH
      RECORD
         -- At the end of the parser execution, the project
         -- is store in the objet below
         --
         Parsed_System : System;
      END RECORD;



   TYPE Xml_Event_Table_Parser IS NEW Xml_Generic_Parser WITH
      RECORD
         -- At the end of the parser execution, the event table
         -- is store in the objet below
         --
         Parsed_Event_Table : Scheduling_Table_Ptr;

         -- At the end of the parser execution, the project
         -- is store in the objet below
         --
         Scheduled_System : System;
      END RECORD;


END Xml_Parsers;
