------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------



WITH Unbounded_Strings;
USE Unbounded_Strings;



PACKAGE BODY Xml_Tag IS


   FUNCTION Get_Dtd RETURN Unbounded_String IS

      S : Unbounded_String;

   BEGIN
      S:= S & "<?xml version=""1.0"" standalone=""yes""?>" & Unbounded_Lf;
      S:= S & "<!DOCTYPE results [" & Unbounded_Lf;
      S:= S &
         "<!ELEMENT results (scheduling_feasibility | scheduling_simulation | set_priorities | blocking_time | end_to_end_response_time | buffer_bound)>" &
         Unbounded_Lf;
      S:=S & "<!ELEMENT scheduling_feasibility(title, line+)>" &
         Unbounded_Lf;
      S:=S & "<!ELEMENT scheduling_simulation(title, line+)>" &
         Unbounded_Lf;
      S:=S & "<!ELEMENT set_priorities(title, line+)>" & Unbounded_Lf;
      S:=S & "<!ELEMENT blocking_time(title, line+)>" & Unbounded_Lf;
      S:=S & "<!ELEMENT end_to_end_response_time(title, line+)>" &
         Unbounded_Lf;
      S:=S & "<!ELEMENT buffer_bound(title, line+)>" & Unbounded_Lf;
      S:=S & "<!ELEMENT title(#PCDATA, processor, #PCDATA)>" &
         Unbounded_Lf;
      S:=S &
         "<!ELEMENT line(#PCDATA | base_period | reference | deadl_factor | period_factor | task | jitter | factor_bound | number_of_preemption)>" &
         Unbounded_Lf;
      S:=S & "<!ELEMENT processor(#PCDATA)>" & Unbounded_Lf;
      S:=S & "<!ELEMENT base_period(#PCDATA)>" & Unbounded_Lf;
      S:=S & "<!ELEMENT reference(#PCDATA)>" & Unbounded_Lf;
      S:=S & "<!ELEMENT deadl_factor(#PCDATA)>" & Unbounded_Lf;
      S:=S & "<!ELEMENT period_factor(#PCDATA)>" & Unbounded_Lf;
      S:=S & "<!ELEMENT task(#PCDATA)>" & Unbounded_Lf;
      S:=S & "<!ELEMENT jitter(#PCDATA)>" & Unbounded_Lf;
      S:=S & "<!ELEMENT factor_bound(#PCDATA)>" & Unbounded_Lf;
      S:=S & "<!ELEMENT number_of_preemption(#PCDATA)>" & Unbounded_Lf;
      S:=S & "]>" & Unbounded_Lf;
      RETURN S;
   END Get_Dtd;


   PROCEDURE Set_Tag IS

   BEGIN
      Start_Result := To_Unbounded_String("<results>");
      End_Result := To_Unbounded_String("</results>");
      Start_Line := To_Unbounded_String("<line>");
      End_Line := To_Unbounded_String("</line>");
      Start_Sched_Feas := To_Unbounded_String("<scheduling_feasibility>");
      End_Sched_Feas := To_Unbounded_String("</scheduling_feasibility>");
      End_Sched_Sim := To_Unbounded_String("</scheduling_simulation>");
      Start_Sched_Sim:= To_Unbounded_String("<scheduling_simulation>");
      Start_Probability := To_Unbounded_String("<probability>");
      End_Probability  := To_Unbounded_String("</probability>");
      Start_Response_Time_Density:= To_Unbounded_String(
         "<response_time_density>");
      End_Response_Time_Density:= To_Unbounded_String(
         "</response_time_density>");
      Start_Task := To_Unbounded_String("<task>");
      End_Task := To_Unbounded_String("</task>");
      Start_Block := To_Unbounded_String("<blocking_time>");
      End_Block := To_Unbounded_String("</blocking_time>");
      Start_Buf_Bound := To_Unbounded_String("<buffer_bound>");
      End_Buf_Bound := To_Unbounded_String("</buffer_bound>");
      Start_Response_Time := To_Unbounded_String("<response_time>");
      End_Response_Time := To_Unbounded_String("</response_time>");
      Start_Title := To_Unbounded_String("<title>");
      End_Title := To_Unbounded_String("</title>");
      Start_Set_Prio := To_Unbounded_String("<set_priorities>");
      End_Set_Prio := To_Unbounded_String("</set_priorities>");
      Start_End_To_End := To_Unbounded_String(
         "<end_to_end_response_time>");
      End_End_To_End := To_Unbounded_String("</end_to_end_response_time>");
      Start_Proc := To_Unbounded_String("<processor>");
      End_Proc := To_Unbounded_String("</processor>");
      Start_Num_Of_Preemp:= To_Unbounded_String("<number_of_preemption>");
      End_Num_Of_Preemp := To_Unbounded_String("</number_of_preemption>");
      Start_Num_Of_Context_Switch  := To_Unbounded_String(
         "<number_of_context_switch>");
      End_Num_Of_Context_Switch   := To_Unbounded_String(
         "</number_of_context_switch>");
      Start_Unit := To_Unbounded_String("<unit>");
      End_Unit := To_Unbounded_String("</unit>");
      Start_Ref := To_Unbounded_String("<reference>");
      End_Ref := To_Unbounded_String("</reference>");
      Start_Fact_Bound := To_Unbounded_String("<factor_bound>");
      End_Fact_Bound := To_Unbounded_String("</factor_bound>");
      Start_Bas_Per := To_Unbounded_String("<base_period>");
      End_Bas_Per := To_Unbounded_String("</base_period>");
      Start_Per_Fact := To_Unbounded_String("<period_factor>");
      End_Per_Fact := To_Unbounded_String("</period_factor>");
      Start_Deadl_Fact := To_Unbounded_String("<deadline_factor>");
      End_Deadl_Fact := To_Unbounded_String("</deadline_factor>");
      Start_Jit := To_Unbounded_String("<jitter>");
      End_Jit := To_Unbounded_String("</jitter>");

   END Set_Tag;


   PROCEDURE Set_Empty IS

   BEGIN


      Start_Result := Empty_String;
      End_Result := Empty_String;
      Start_Title :=Empty_String;
      End_Title := Empty_String;
      Start_Line := Empty_String ;
      End_Line := Empty_String;
      Start_Sched_Feas := Empty_String ;
      End_Sched_Feas := Empty_String ;
      End_Sched_Sim := Empty_String ;
      Start_Sched_Sim:= Empty_String ;
      Start_Task := Empty_String ;
      End_Task := Empty_String ;
      Start_Block := Empty_String ;
      End_Block := Empty_String ;
      Start_Buf_Bound := Empty_String ;
      End_Buf_Bound := Empty_String ;
      Start_Title := Empty_String;
      End_Title := Empty_String ;
      Start_Set_Prio := Empty_String ;
      End_Set_Prio := Empty_String ;
      Start_End_To_End := Empty_String ;
      End_End_To_End := Empty_String ;
      Start_Proc := Empty_String ;
      End_Proc := Empty_String ;
      Start_Num_Of_Preemp:= Empty_String ;
      End_Num_Of_Preemp := Empty_String ;
      Start_Unit :=Empty_String ;
      End_Unit := Empty_String ;
      Start_Ref := Empty_String ;
      End_Ref := Empty_String ;
      Start_Fact_Bound := Empty_String ;
      End_Fact_Bound := Empty_String ;
      Start_Bas_Per := Empty_String ;
      End_Bas_Per := Empty_String ;
      Start_Per_Fact := Empty_String;
      End_Per_Fact := Empty_String;
      Start_Deadl_Fact := Empty_String;
      End_Deadl_Fact := Empty_String;
      Start_Jit := Empty_String;
      End_Jit := Empty_String;
      Start_Probability := Empty_String;
      End_Probability  := Empty_String;
      Start_Response_Time_Density:= Empty_String;
      End_Response_Time_Density:= Empty_String;
      Start_Response_Time := Empty_String;
      End_Response_Time := Empty_String;

   END Set_Empty;


END Xml_Tag;


