------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with call_framework; use call_framework;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Tasks;    use Tasks;
with Task_Set; use Task_Set;
with Task_Dependencies; use Task_Dependencies;
with Text_IO; use Text_IO;
with Offsets; use Offsets;
use Offsets.Offsets_Table_package;

procedure Acyclic is

   Tasks1                 : Tasks_Set;
   Dep1                   : Tasks_Dependencies_Ptr;
   T1, T2, T3, T4, T5, T6 : Generic_Task_Ptr;

   Offset : Offsets_Table;
   Cpu    : Unbounded_String := To_Unbounded_String ("cpu");
   Ea     : Unbounded_String := To_Unbounded_String ("ea");

begin

call_framework.initialize;


   Initialize (Offset);
   Dep1 := new Tasks_Dependencies;

   Add_Task
     (Tasks1,
      T1,
      To_Unbounded_String ("T1"),
      Cpu,
      Ea,
      Aperiodic_Type,
      0,
      1,
      0,
      5,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      T2,
      To_Unbounded_String ("T2"),
      Cpu,
      Ea,
      Aperiodic_Type,
      0,
      2,
      0,
      10,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      T3,
      To_Unbounded_String ("T3"),
      Cpu,
      Ea,
      Aperiodic_Type,
      0,
      1,
      0,
      8,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      T4,
      To_Unbounded_String ("T4"),
      Cpu,
      Ea,
      Aperiodic_Type,
      0,
      2,
      0,
      14,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      T5,
      To_Unbounded_String ("T5"),
      Cpu,
      Ea,
      Aperiodic_Type,
      0,
      2,
      0,
      14,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      T6,
      To_Unbounded_String ("T6"),
      Cpu,
      Ea,
      Aperiodic_Type,
      0,
      2,
      0,
      14,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);

   -------------------------------------------
   Reset (Dep1);

   Add_One_Task_Dependency (Dep1, T1, T2);

   Add_One_Task_Dependency (Dep1, T1, T3);

   if Is_Cyclic (Dep1) then
      Put_Line ("Test a ; ERROR : not cyclic");
      Put (Dep1);
   end if;

   Put_Line ("Test a");

   -------------------------------------------

   Reset (Dep1);

   Add_One_Task_Dependency (Dep1, T1, T2);

   Add_One_Task_Dependency (Dep1, T1, T3);

   Add_One_Task_Dependency (Dep1, T2, T3);

   Add_One_Task_Dependency (Dep1, T2, T4);

   Add_One_Task_Dependency (Dep1, T4, T2);

   if Is_Acyclic (Dep1) then
      Put_Line ("Test b ; ERROR : not acyclic ");
      Put (Dep1);
   end if;

   Put_Line ("Test b");

   -------------------------------------------

   Reset (Dep1);

   Add_One_Task_Dependency (Dep1, T2, T3);

   Add_One_Task_Dependency (Dep1, T3, T2);

   if Is_Acyclic (Dep1) then
      Put_Line ("Test c ; ERROR : not acyclic");
      Put (Dep1);
   end if;

   Put_Line ("Test c");

   -------------------------------------------

   Reset (Dep1);

   Add_One_Task_Dependency (Dep1, T2, T3);

   Add_One_Task_Dependency (Dep1, T3, T2);

   Add_One_Task_Dependency (Dep1, T3, T4);

   if Is_Acyclic (Dep1) then
      Put_Line ("Test d ; ERROR : not acyclic ");
      Put (Dep1);
   end if;

   Put_Line ("Test d");

   -------------------------------------------

   Reset (Dep1);

   if Is_Cyclic (Dep1) then
      Put_Line ("Test e ; ERROR : not cyclic ");
      Put (Dep1);
   end if;

   Put_Line ("Test e");

   -------------------------------------------

   Reset (Dep1);

   Add_One_Task_Dependency (Dep1, T1, T2);

   Add_One_Task_Dependency (Dep1, T1, T3);

   Add_One_Task_Dependency (Dep1, T2, T4);

   Add_One_Task_Dependency (Dep1, T3, T4);

   if Is_Cyclic (Dep1) then
      Put_Line ("Test f ; ERROR not cyclic");
      Put (Dep1);
   end if;

   Put_Line ("Test f");

   -------------------------------------------

   Reset (Dep1);

   Add_One_Task_Dependency (Dep1, T1, T1);

   if Is_Acyclic (Dep1) then
      Put_Line ("Test g ; ERROR acyclic");
      Put (Dep1);
   end if;

   Put_Line ("Test g");

   -------------------------------------------

   Reset (Dep1);

   Add_One_Task_Dependency (Dep1, T1, T1);
   Add_One_Task_Dependency (Dep1, T1, T2);

   if Is_Acyclic (Dep1) then
      Put_Line ("Test h ; ERROR not acyclic");
      Put (Dep1);
   end if;

   Put_Line ("Test h");

   -------------------------------------------

   Reset (Dep1);

   Add_One_Task_Dependency (Dep1, T1, T2);
   Add_One_Task_Dependency (Dep1, T2, T3);
   Add_One_Task_Dependency (Dep1, T3, T4);
   Add_One_Task_Dependency (Dep1, T4, T1);

   if Is_Acyclic (Dep1) then
      Put_Line ("Test i ; ERROR not acyclic");
      Put (Dep1);
   end if;

   Put_Line ("Test i");

   -------------------------------------------

   Reset (Dep1);

   Add_One_Task_Dependency (Dep1, T1, T2);
   Add_One_Task_Dependency (Dep1, T4, T3);

   if Is_Cyclic (Dep1) then
      Put_Line ("Test j ; ERROR not cyclic");
      Put (Dep1);
   end if;

   Put_Line ("Test j");

   -------------------------------------------

   Reset (Dep1);

   Add_One_Task_Dependency (Dep1, T1, T2);

   if Is_Cyclic (Dep1) then
      Put_Line ("Test k ; ERROR not cyclic");
      Put (Dep1);
   end if;

   Put_Line ("Test k");

   -------------------------------------------

   Reset (Dep1);

   Add_One_Task_Dependency (Dep1, T1, T2);
   Add_One_Task_Dependency (Dep1, T3, T3);

   if Is_Acyclic (Dep1) then
      Put_Line ("Test l ; ERROR not acyclic");
      Put (Dep1);
   end if;

   Put_Line ("Test l");

   -------------------------------------------

   Reset (Dep1);

   Add_One_Task_Dependency (Dep1, T1, T2);
   Add_One_Task_Dependency (Dep1, T2, T3);

   if Is_Cyclic (Dep1) then
      Put_Line ("Test m ; ERROR not cyclic");
      Put (Dep1);
   end if;

   Put_Line ("Test m");

   -------------------------------------------

   Reset (Dep1);

   Add_One_Task_Dependency (Dep1, T1, T2);
   Add_One_Task_Dependency (Dep1, T2, T3);
   Add_One_Task_Dependency (Dep1, T5, T4);
   Add_One_Task_Dependency (Dep1, T4, T5);

   if Is_Acyclic (Dep1) then
      Put_Line ("Test n ; ERROR not acyclic");
      Put (Dep1);
   end if;

   Put_Line ("Test n");

   -------------------------------------------

   Reset (Dep1);

   Add_One_Task_Dependency (Dep1, T1, T3);
   Add_One_Task_Dependency (Dep1, T2, T3);
   Add_One_Task_Dependency (Dep1, T3, T4);
   Add_One_Task_Dependency (Dep1, T3, T5);
   Add_One_Task_Dependency (Dep1, T4, T6);
   Add_One_Task_Dependency (Dep1, T5, T6);

   if Is_Cyclic (Dep1) then
      Put_Line ("Test o ; ERROR not cyclic");
      Put (Dep1);
   end if;

   Put_Line ("Test o");

   -------------------------------------------

   Reset (Dep1);

   Add_One_Task_Dependency (Dep1, T1, T2);
   Add_One_Task_Dependency (Dep1, T2, T3);
   Add_One_Task_Dependency (Dep1, T3, T4);
   Add_One_Task_Dependency (Dep1, T4, T5);
   Add_One_Task_Dependency (Dep1, T5, T1);

   if not Is_Cyclic (Dep1) then
      Put_Line ("Test p ; ERROR not cyclic");
      Put (Dep1);
   end if;

   Put_Line ("Test p");

   -------------------------------------------

   Reset (Dep1);

   Add_One_Task_Dependency (Dep1, T1, T2);
   Add_One_Task_Dependency (Dep1, T2, T3);
   Add_One_Task_Dependency (Dep1, T3, T4);
   Add_One_Task_Dependency (Dep1, T1, T4);
   Add_One_Task_Dependency (Dep1, T4, T5);
   Add_One_Task_Dependency (Dep1, T3, T5);

   if Is_Cyclic (Dep1) then
      Put_Line ("Test q ; ERROR not cyclic");
      Put (Dep1);
   end if;

   Put_Line ("Test q");

end Acyclic;
