------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Call_Framework;              use Call_Framework;
with Ada.Strings.Unbounded;       use Ada.Strings.Unbounded;
with Tasks;                       use Tasks;
with Task_Set;                    use Task_Set;
use Task_Set.Generic_Task_Set;
with Scheduler;                   use Scheduler;
use Scheduler.Double_Tasks_Parameters;
with Scheduler.Fixed_Priority.Dm; use Scheduler.Fixed_Priority.Dm;
with Text_IO;                     use Text_IO;
with Offsets;                     use Offsets;
use Offsets.Offsets_Table_package;

procedure Audsley93 is

   Offset : Offsets_Table;
   Sched1 : Dm_Scheduler;
   Tasks1 : Tasks_Set;
   Rt     : Response_Time_Table;
   Cpu    : Unbounded_String := To_Unbounded_String ("cpu");
   Dummy  : Unbounded_String;
   Ea     : Unbounded_String := To_Unbounded_String ("ea");

begin

   Call_Framework.initialize;
   Initialize (Offset);

   Add_Task
     (Tasks1,
      To_Unbounded_String ("T1"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      51,
      1000,
      1000,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("T2"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      3000,
      200000,
      5000,
      0,
      300,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("T3"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      2000,
      25000,
      25000,
      0,
      600,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("T4"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      5000,
      25000,
      25000,
      0,
      900,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("T5"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      1000,
      40000,
      40000,
      0,
      1350,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("T6"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      3000,
      50000,
      50000,
      0,
      1350,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("T7"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      5000,
      50000,
      50000,
      0,
      750,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("T8"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      8000,
      59000,
      59000,
      0,
      750,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("T9"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      9000,
      80000,
      80000,
      0,
      1350,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("T10"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      2000,
      80000,
      80000,
      0,
      450,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("T11"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      5000,
      100000,
      100000,
      0,
      1050,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("T12"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      1000,
      200000,
      200000,
      1000,
      450,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("T13"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      3000,
      200000,
      200000,
      0,
      450,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("T14"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      1000,
      200000,
      200000,
      0,
      450,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("T15"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      1000,
      200000,
      200000,
      0,
      1350,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("T16"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      3000,
      200000,
      200000,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("T17"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      1000,
      1000000,
      1000000,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("T18"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      1000,
      1000000,
      1000000,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);

   Put (Tasks1);

   Compute_Response_Time (Sched1, Tasks1, Cpu, Dummy, Rt);

   Put (Rt);

   New_Line;
   New_Line;

end Audsley93;
