------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with call_framework; use call_framework;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Time_Units; use Time_Units;
use Time_Units.A_Time_Unit;
with Scheduler;                      use Scheduler;
with Scheduler.Dynamic_Priority.Edf; use Scheduler.Dynamic_Priority.Edf;
with Scheduling_Sequences; use Scheduling_Sequences;
with Text_IO; use Text_IO;
with Ada.Numerics.Aux; use Ada.Numerics.Aux;
with Offsets; use Offsets;
use Offsets.Offsets_Table_package;
with Tasks;    use Tasks;
with Task_Set; use Task_Set;
use Task_Set.Generic_Task_Set;
with Resources;    use Resources;
use Resources.Task_Lists;
with Resource_Set; use Resource_Set;
use Resource_Set.Generic_Resource_Set;
with Buffer_Set; use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Message_Set; use Message_Set;
use Message_Set.Generic_Message_Set;
with Events;            use Events;
with Task_Dependencies; use Task_Dependencies;

procedure Check_Ct is

   My_Scheduler : Edf_Scheduler;
   Tasks1       : Tasks_Set;
   Sched1       : Scheduling_Sequence_Ptr                     :=
      new Scheduling_Sequence;
   Offset       : Offsets_Table;
   Cpu          : Unbounded_String                            :=
      To_Unbounded_String ("cpu");
   Ea           : Unbounded_String                            :=
      To_Unbounded_String ("ea");
   Msg          : Unbounded_String;
   Mes1         : Messages_Set;
   Res1         : Resources_Set;
   Buff1        : Buffers_Set;
   Deps1        : constant Tasks_Dependencies_Ptr             :=
      new Tasks_Dependencies;
   E            : constant Time_Unit_Event_Type_Boolean_Table :=
     (others => True);
   T1, T2       : Generic_Task_Ptr;

   Result1    : Natural := 0;
   Result2    : Natural := 0;
   Result3    : Double  := 0.0;
   Result_Msg : Unbounded_String;

begin

call_framework.initialize;
   Initialize (Offset);

   Add_Task
     (Tasks1,
      T1,
      To_Unbounded_String ("T1"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      5,
      10,
      10,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      T2,
      To_Unbounded_String ("T2"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      6,
      10,
      10,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);

   Put (Tasks1);

   Build_Scheduling_Sequence
     (My_Scheduler,
      Tasks1,
      Res1,
      Buff1,
      Mes1,
      Deps1,
      Cpu,
      Sched1,
      Msg,
      E,
      80);
   Put (Sched1.all);

   New_Line;
   New_Line;

   Response_Time_By_Simulation
     (T1,
      Sched1,
      Result_Msg,
      Result1,
      Result2,
      Result3);
   Put_Line (To_String (Result_Msg));
   Put_Line (Result1'Img);
   Put_Line (Result2'Img);
   Put_Line (Result3'Img);

   Response_Time_By_Simulation
     (T2,
      Sched1,
      Result_Msg,
      Result1,
      Result2,
      Result3);
   Put_Line (To_String (Result_Msg));
   Put_Line (Result1'Img);
   Put_Line (Result2'Img);
   Put_Line (Result3'Img);

end Check_Ct;
