------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Call_Framework;        use Call_Framework;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Tasks;                 use Tasks;
with Task_Set;              use Task_Set;
with Dependency_Services;   use Dependency_Services;
with Task_Dependencies;     use Task_Dependencies;
with Text_IO;               use Text_IO;
with Offsets;               use Offsets;
use Offsets.Offsets_Table_package;

procedure Chetto is

   Tasks1 : Tasks_Set;
   Dep1   : Tasks_Dependencies_Ptr := new Tasks_Dependencies;

   Offset         : Offsets_Table;
   Cpu            : Unbounded_String := To_Unbounded_String ("cpu");
   Ea             : Unbounded_String := To_Unbounded_String ("ea");
   T1, T2, T3, T4 : Generic_Task_Ptr;

begin

   Call_Framework.initialize;
   Initialize (Offset);
   Reset (Dep1);

   Add_Task
     (Tasks1,
      T1,
      To_Unbounded_String ("T1"),
      Cpu,
      Ea,
      aPeriodic_Type,
      0,
      1,
      0,
      5,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      T2,
      To_Unbounded_String ("T2"),
      Cpu,
      Ea,
      aPeriodic_Type,
      0,
      2,
      0,
      10,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      T3,
      To_Unbounded_String ("T3"),
      Cpu,
      Ea,
      aPeriodic_Type,
      0,
      1,
      0,
      8,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      T4,
      To_Unbounded_String ("T4"),
      Cpu,
      Ea,
      aPeriodic_Type,
      0,
      2,
      0,
      14,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);

   Put (Tasks1);

   Add_One_Task_Dependency (Dep1, T1, T2);

   Add_One_Task_Dependency (Dep1, T1, T3);

   Add_One_Task_Dependency (Dep1, T2, T3);

   Add_One_Task_Dependency (Dep1, T2, T4);

   Put (Dep1);

   New_Line;

   Set_Parameters_According_To_Chetto (Dep1, Tasks1, True, False);

   Put (Tasks1);

end Chetto;
