------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Call_Framework;                 use Call_Framework;
with Ada.Strings.Unbounded;          use Ada.Strings.Unbounded;
with Ada.Numerics.Aux;               use Ada.Numerics.Aux;
with Events;                         use Events;
with Tasks;                          use Tasks;
with Task_Set;                       use Task_Set;
with Message_Set;                    use Message_Set;
with Resource_Set;                   use Resource_Set;
with Buffer_Set;                     use Buffer_Set;
with Task_Dependencies;              use Task_Dependencies;
with Scheduler;                      use Scheduler;
with Scheduler.Dynamic_Priority.Edf; use Scheduler.Dynamic_Priority.Edf;
with Scheduling_Sequences;           use Scheduling_Sequences;
with Text_IO;                        use Text_IO;
with Offsets;                        use Offsets;
use Offsets.Offsets_Table_package;
with Time_Units;                     use Time_Units;
use Time_Units.A_Time_Unit;

procedure Edf is

   My_Offset    : Offsets_Table;
   My_Scheduler : Edf_Scheduler;
   A_Task       : Generic_Task_Ptr;

   Sched1 : Scheduling_Sequence_Ptr := new Scheduling_Sequence;
   Sched2 : Scheduling_Sequence_Ptr := new Scheduling_Sequence;
   Sched3 : Scheduling_Sequence_Ptr := new Scheduling_Sequence;

   Msg   : Unbounded_String;
   Mes1  : Messages_Set;
   Res1  : Resources_Set;
   Buff1 : Buffers_Set;
   Dep1  : Tasks_Dependencies_Ptr := new Tasks_Dependencies;

   Tasks1, Tasks2, Tasks3 : Tasks_Set;

   Period : Natural := 0;
   Used   : Double  := 0.0;

   Cpu : Unbounded_String := To_Unbounded_String ("cpu");
   Ea  : Unbounded_String := To_Unbounded_String ("ea");

   E : constant Time_Unit_Event_Type_Boolean_Table := (others => True);

begin

   Call_Framework.initialize;
   Initialize (My_Offset);
   Reset (Dep1);

   Add_Task
     (Tasks1,
      A_Task,
      To_Unbounded_String ("T1"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      5,
      12,
      0,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      My_Offset);
   Add_Task
     (Tasks1,
      A_Task,
      To_Unbounded_String ("T2"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      2,
      6,
      0,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      My_Offset);
   Add_Task
     (Tasks1,
      A_Task,
      To_Unbounded_String ("T3"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      5,
      24,
      0,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      My_Offset);

   Put (Tasks1);

   Used := Processor_Utilization_Over_Period (Tasks1, Cpu);
   Put_Line ("Utilization  = " & Used'Img);

   Period := Scheduling_Period (Tasks1, Cpu);
   Put_Line ("Scheduling period  = " & Period'Img);

   Build_Scheduling_Sequence
     (My_Scheduler,
      Tasks1,
      Res1,
      Buff1,
      Mes1,
      Dep1,
      Cpu,
      Sched1,
      Msg,
      E);

   Put (Sched1.all);

   New_Line;
   New_Line;

   Add_Task
     (Tasks2,
      A_Task,
      To_Unbounded_String ("T1"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      4,
      8,
      0,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      My_Offset);
   Add_Task
     (Tasks2,
      A_Task,
      To_Unbounded_String ("T2"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      5,
      10,
      0,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      My_Offset);

   Put (Tasks2);

   Used := Processor_Utilization_Over_Period (Tasks2, Cpu);
   Put_Line ("Utilization  = " & Used'Img);

   Period := Scheduling_Period (Tasks2, Cpu);
   Put_Line ("Scheduling period  = " & Period'Img);

   Build_Scheduling_Sequence
     (My_Scheduler,
      Tasks2,
      Res1,
      Buff1,
      Mes1,
      Dep1,
      Cpu,
      Sched2,
      Msg,
      E);

   Put (Sched2.all);

   New_Line;
   New_Line;

   Add_Task
     (Tasks3,
      To_Unbounded_String ("T1"),
      Cpu,
      Ea,
      Periodic_Type,
      1,
      1,
      8,
      0,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      My_Offset);
   Add_Task
     (Tasks3,
      To_Unbounded_String ("T2"),
      Cpu,
      Ea,
      Periodic_Type,
      3,
      1,
      10,
      0,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      My_Offset);

   Put (Tasks3);

   Used := Processor_Utilization_Over_Period (Tasks3, Cpu);
   Put_Line ("Utilization  = " & Used'Img);

   Period := Scheduling_Period (Tasks3, Cpu);
   Put_Line ("Scheduling period  = " & Period'Img);

   Build_Scheduling_Sequence
     (My_Scheduler,
      Tasks3,
      Res1,
      Buff1,
      Mes1,
      Dep1,
      Cpu,
      Sched3,
      Msg,
      E);

   Put (Sched3.all);

end Edf;
