------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Call_Framework;                  use Call_Framework;
with Ada.Strings.Unbounded;           use Ada.Strings.Unbounded;
with Ada.Exceptions;                  use Ada.Exceptions;
with Ada.Command_Line;                use Ada.Command_Line;
with Scheduler;                       use Scheduler;
with Scheduler.user_defined.pipeline; use Scheduler.user_defined.pipeline;
with Scheduling_Sequences;            use Scheduling_Sequences;
with Text_IO;                         use Text_IO;
with Time_Units;                      use Time_Units;
use Time_Units.A_Time_Unit;
with Tasks;                           use Tasks;
with Task_Set;                        use Task_Set;
with Message_Set;                     use Message_Set;
with Resource_Set;                    use Resource_Set;
with Buffer_Set;                      use Buffer_Set;
with Task_Dependencies;               use Task_Dependencies;
with Offsets;                         use Offsets;
use Offsets.Offsets_Table_package;
with Io_Tools;                        use Io_Tools;
with Events;                          use Events;

procedure Parametric is

   A_Scheduler      : pipeline_user_defined_Scheduler;
   Scheduler_String : Unbounded_String;

   My_Offset : Offsets_Table;
   A_Task    : Generic_Task_Ptr;
   Tasks1    : Tasks_Set;
   Sched1    : Scheduling_Sequence_Ptr         := new Scheduling_Sequence;
   Msg       : Unbounded_String;
   Mes1      : Messages_Set;
   Res1      : Resources_Set;
   Buff1     : Buffers_Set;
   Dep1      : constant Tasks_Dependencies_Ptr := new Tasks_Dependencies;

   Cpu : Unbounded_String                            :=
      To_Unbounded_String ("cpu");
   Ea  : Unbounded_String                            :=
      To_Unbounded_String ("ea");
   E   : constant Time_Unit_Event_Type_Boolean_Table := (others => True);

begin

   Call_Framework.initialize;

   if Argument_Count /= 1 then
      Raise_Exception
        (Constraint_Error'Identity,
         "usage : parametric sc-file");
   end if;

   Initialize (My_Offset);

   Scheduler_String := Read_Sequential_File (Argument (1));

   Set_String_Behavior (A_Scheduler, Scheduler_String);
   Set_Behavior_File_Name (A_Scheduler, to_unbounded_string(Argument (1)));

   Add_Task
     (Tasks1,
      A_Task,
      To_Unbounded_String ("T1"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      5,
      12,
      0,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      My_Offset);
   Add_Task
     (Tasks1,
      A_Task,
      To_Unbounded_String ("T2"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      2,
      6,
      0,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      My_Offset);
   Add_Task
     (Tasks1,
      A_Task,
      To_Unbounded_String ("T3"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      5,
      24,
      0,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      My_Offset);

   Put (Tasks1);

   Build_Scheduling_Sequence
     (A_Scheduler,
      Tasks1,
      Res1,
      Buff1,
      Mes1,
      Dep1,
      Cpu,
      Sched1,
      Msg,
      E);

   put(sched1.all);

end Parametric;
