------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Call_Framework;        use Call_Framework;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Tasks;                 use Tasks;
with Task_Set;              use Task_Set;
with Text_IO;               use Text_IO;
with Offsets;               use Offsets;
use Offsets.Offsets_Table_package;

procedure Period is

   Tasks1, Tasks2 : Tasks_Set;

   Cpu    : Unbounded_String := To_Unbounded_String ("cpu");
   Ea     : Unbounded_String := To_Unbounded_String ("ea");
   Offset : Offsets_Table;
   Period : Natural          := 0;

begin

   Call_Framework.initialize;
   Initialize (Offset);

   Add_Task
     (Tasks1,
      To_Unbounded_String ("T1"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      1,
      10,
      0,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("T2"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      1,
      5,
      0,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("T3"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      1,
      3,
      0,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);

   Put (Tasks1);

   Period := Scheduling_Period (Tasks1, Cpu);
   Put_Line ("Res = " & Period'Img);

   Add_Task
     (Tasks2,
      To_Unbounded_String ("T1"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      1,
      5,
      0,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks2,
      To_Unbounded_String ("T2"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      1,
      1,
      0,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks2,
      To_Unbounded_String ("T3"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      1,
      3,
      0,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);

   Put (Tasks2);

   Period := Scheduling_Period (Tasks2, Cpu);
   Put_Line ("Res = " & Period'Img);

end Period;
