------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Strings.Unbounded;       use Ada.Strings.Unbounded;
with Tasks;                       use Tasks;
with Task_Set;                    use Task_Set;
use Task_Set.Generic_Task_Set;
with Scheduler;                   use Scheduler;
use Scheduler.Double_Tasks_Parameters;
with Scheduler.Fixed_Priority.Rm; use Scheduler.Fixed_Priority.Rm;
with Scheduler.Fixed_Priority.Dm; use Scheduler.Fixed_Priority.Dm;
with Text_IO;                     use Text_IO;
with Offsets;                     use Offsets;
use Offsets.Offsets_Table_package;
with Call_Framework;              use Call_Framework;

procedure Response is

   Sched1 : Rm_Scheduler;
   Sched2 : Dm_Scheduler;
   Tasks1 : Tasks_Set;
   Tasks2 : Tasks_Set;
   Rt     : Response_Time_Table;
   Cpu    : Unbounded_String := To_Unbounded_String ("cpu");
   Ea     : Unbounded_String := To_Unbounded_String ("ea");
   Offset : Offsets_Table;
   Dummy  : Unbounded_String;

begin

   Call_Framework.initialize;
   Initialize (Offset);

   Add_Task
     (Tasks1,
      To_Unbounded_String ("T1"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      3,
      7,
      7,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("T2"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      3,
      12,
      12,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("T3"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      5,
      20,
      20,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);

   Put (Tasks1);

   Compute_Response_Time (Sched1, Tasks1, Cpu, Dummy, Rt);
   Put_Line ("TR should be : TR1 = 3 ; TR2 = 6 ; TR3 = 20");
   Put (Rt);

   New_Line;
   New_Line;

   Add_Task
     (Tasks2,
      To_Unbounded_String ("T2"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      60,
      150,
      160,
      0,
      20,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks2,
      To_Unbounded_String ("T3"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      60,
      350,
      350,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks2,
      To_Unbounded_String ("T1"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      40,
      100,
      100,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);

   Put (Tasks2);
   Compute_Response_Time (Sched2, Tasks2, Cpu, Dummy, Rt);
   Put_Line ("TR should be : TR1 = 40 ; TR2 = 120  ; TR3 = 300");

   Put (Rt);

end Response;
