------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Call_Framework;              use Call_Framework;
with Queueing_System;             use Queueing_System;
with Ada.Strings.Unbounded;       use Ada.Strings.Unbounded;
with Scheduler;                   use Scheduler;
with Scheduler.Fixed_Priority.Rm; use Scheduler.Fixed_Priority.Rm;
with Scheduling_Sequences;        use Scheduling_Sequences;
use Scheduling_Sequences.Buffer_Size_Package;
with Text_IO;                     use Text_IO;
with Offsets;                     use Offsets;
use Offsets.Offsets_Table_package;
with Time_Units;                  use Time_Units;
use Time_Units.A_Time_Unit;
with Tasks;                       use Tasks;
with Buffers;                     use Buffers;
with Task_Set;                    use Task_Set;
with Message_Set;                 use Message_Set;
with Resource_Set;                use Resource_Set;
with Buffer_Set;                  use Buffer_Set;
with Task_Dependencies;           use Task_Dependencies;
with Events;                      use Events;

procedure Tb1 is

   My_Buff      : constant Buffer_Ptr                         := new Buffer;
   My_Scheduler : Rm_Scheduler;
   Tasks1       : Tasks_Set;
   Offset       : Offsets_Table;
   Mes1         : Messages_Set;
   Res1         : Resources_Set;
   Buff1        : Buffers_Set;
   Dep1         : constant Tasks_Dependencies_Ptr             :=
      new Tasks_Dependencies;
   Sched1       : Scheduling_Sequence_Ptr                     :=
      new Scheduling_Sequence;
   My_Roles     : Buffer_Roles_Table;
   Cpu          : Unbounded_String                            :=
      To_Unbounded_String ("cpu");
   Ea           : Unbounded_String                            :=
      To_Unbounded_String ("ea");
   T1, T2       : Generic_Task_Ptr;
   E            : constant Time_Unit_Event_Type_Boolean_Table :=
     (others => True);

   Bound       : Natural;
   Buffer_Size : Buffer_Size_Table_Ptr := new Buffer_Size_Table;
   Msg         : Unbounded_String;

begin

   Call_Framework.initialize;
   Initialize (Offset);

   Add_Task
     (Tasks1,
      T1,
      To_Unbounded_String ("T1"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      3,
      10,
      10,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      T2,
      To_Unbounded_String ("T2"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      3,
      10,
      10,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);

   Put (Tasks1);

   My_Roles.Entries (0).Item          := T1.Name;
   My_Roles.Entries (0).Data.Size     := 1;
   My_Roles.Entries (0).Data.Time     := 1;
   My_Roles.Entries (0).Data.The_Role := Producer;
   My_Roles.Entries (1).Item          := T2.Name;
   My_Roles.Entries (1).Data.The_Role := Consumer;
   My_Roles.Entries (1).Data.Size     := 1;
   My_Roles.Entries (0).Data.Time     := 1;
   My_Roles.Nb_Entries                := 2;

   Add_Buffer
     (Buff1,
      To_Unbounded_String ("B1"),
      1,
      Cpu,
      Ea,
      Qs_Pp1,
      My_Roles);

   Build_Scheduling_Sequence
     (My_Scheduler,
      Tasks1,
      Res1,
      Buff1,
      Mes1,
      Dep1,
      Cpu,
      Sched1,
      Msg,
      E,
      20);
   Put (Sched1.all);

   Compute_Buffer_Size_From_Simulation (Sched1, My_Buff, Buffer_Size);
   Bound := Compute_Maximum_Buffer_Size_From_Simulation (Buffer_Size);

   Put_Line ("Maximum number of messages in the buffer = " & Bound'Img);
   New_Line;
   for I in  0 .. Buffer_Size.Nb_Entries - 1 loop
      Put_Line
        ("Time = " &
         Buffer_Size.Entries (I).Time'Img &
         "/Size = " &
         Buffer_Size.Entries (I).Size'Img);
   end loop;

end Tb1;
