------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Call_Framework;            use Call_Framework;
with Ada.Numerics.Aux;          use Ada.Numerics.Aux;
with Text_IO;                   use Text_IO;
with Ada.Numerics.Float_Random; use Ada.Numerics.Float_Random;
with Random_Tools;              use Random_Tools;

procedure Test_Random is

   Avg_Random  : Double := 0.0;
   Var_Random  : Double := 0.0;
   Avg_Drand48 : Double := 0.0;
   Var_Drand48 : Double := 0.0;

   Random_Value  : Double := 0.0;
   Drand48_Value : Double := 0.0;

   Avg_Diff_Random  : Double := 0.0;
   Avg_Diff_Drand48 : Double := 0.0;
   Var_Diff_Random  : Double := 0.0;
   Var_Diff_Drand48 : Double := 0.0;

   Seed_Value : constant Natural := 0;
   Seed       : Generator;

   Nb_Occurence_Loop : constant Natural := 10000000;
   Nb_Occurence      : constant Double  := Double (Nb_Occurence_Loop);
begin

   Call_Framework.initialize;
   Reset (Seed, Seed_Value);
   Srand48 (0);

   for I in  1 .. Nb_Occurence_Loop loop
      -- random number generation
      Random_Value  := Double (Random (Seed));
      Drand48_Value := Double (Drand48);

      Put_Line ("R " & I'Img & " " & Random_Value'Img);
      Put_Line ("D " & I'Img & " " & Drand48_Value'Img);

      -- compute variance and average
      Avg_Random  := Avg_Random + Random_Value;
      Avg_Drand48 := Avg_Drand48 + Drand48_Value;

      Var_Random  := Var_Random + Random_Value * Random_Value;
      Var_Drand48 := Var_Drand48 + Drand48_Value * Drand48_Value;
   end loop;

   Put_Line
     ("# " & Nb_Occurence_Loop'Img & " occurences of random and drand48");

   Avg_Random  := Avg_Random / Nb_Occurence;
   Avg_Drand48 := Avg_Drand48 / Nb_Occurence;

   Var_Random  := Var_Random / Nb_Occurence - Avg_Random * Avg_Random;
   Var_Drand48 := Var_Drand48 / Nb_Occurence - Avg_Drand48 * Avg_Drand48;

   Put_Line ("# random average " & Avg_Random'Img);
   Put_Line ("# random variance " & Var_Random'Img);

   Put_Line ("# drand48 average " & Avg_Drand48'Img);
   Put_Line ("# drand48 variance " & Var_Drand48'Img);

   -- diff between simulation and theorical results
   Avg_Diff_Random  := 100.0 - (100.0 * Avg_Random) / 0.5;
   Avg_Diff_Drand48 := 100.0 - (100.0 * Avg_Drand48) / 0.5;

   Var_Diff_Random  := 100.0 - (100.0 * Var_Random) / (1.0 / 12.0);
   Var_Diff_Drand48 := 100.0 - (100.0 * Var_Drand48) / (1.0 / 12.0);

   Put_Line ("# random average diff 0.5 (%) " & Avg_Diff_Random'Img);
   Put_Line ("# drand48 average diff 0.5 (%) " & Avg_Diff_Drand48'Img);

   Put_Line ("# random variance diff 1/12 (%) " & Var_Diff_Random'Img);
   Put_Line ("# drand48 variance diff 1/12 (%) " & Var_Diff_Drand48'Img);

end Test_Random;
