------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Call_Framework;              use Call_Framework;
with Ada.Strings.Unbounded;       use Ada.Strings.Unbounded;
with Ada.Numerics.Aux;            use Ada.Numerics.Aux;
with Tasks;                       use Tasks;
with Task_Set;                    use Task_Set;
use Task_Set.Generic_Task_Set;
with Scheduler;                   use Scheduler;
use Scheduler.Double_Tasks_Parameters;
with Scheduler.Fixed_Priority.Dm; use Scheduler.Fixed_Priority.Dm;
with Text_IO;                     use Text_IO;
with Offsets;                     use Offsets;
use Offsets.Offsets_Table_package;

procedure Tindell1 is

   Sched1                   : Dm_Scheduler;
   Tasks1                   : Tasks_Set;
   Response_M1, Response_M2 : Double;
   Initial_M1, Initial_M2   : Double;
   M1                       : Double           := 0.0;
   M2                       : Double           := 0.0;
   Rta, Rta2                : Response_Time_Table;
   Rtb, Rtb2                : Response_Time_Table;
   Offset                   : Offsets_Table;
   Cpua                     : Unbounded_String :=
      To_Unbounded_String ("cpua");
   Cpub                     : Unbounded_String :=
      To_Unbounded_String ("cpub");
   eaa                      : Unbounded_String := To_Unbounded_String ("eaa");
   eab                      : Unbounded_String := To_Unbounded_String ("eab");
   Dummy                    : Unbounded_String;

   procedure Compute is

   begin
      Compute_Response_Time (Sched1, Tasks1, Cpua, Dummy, Rta);
      Compute_Response_Time (Sched1, Tasks1, Cpub, Dummy, Rtb);

      Put (Rta);
      Put (Rtb);
      Put_Line (" m1 : " & Response_M1'Img);
      Put_Line (" m2 : " & Response_M2'Img);

      New_Line;
      New_Line;
   end Compute;

begin

   Call_Framework.initialize;
   Initialize (Offset);
   Initialize (Rta);
   Initialize (Rta2);
   Initialize (Rtb);
   Initialize (Rtb2);

   -- task's parameters
   --
   Add_Task
     (Tasks1,
      To_Unbounded_String ("Ta1"),
      Cpua,
      eaa,
      Periodic_Type,
      0,
      1,
      20,
      20,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("Tb1"),
      Cpub,
      eab,
      Periodic_Type,
      0,
      3,
      20,
      20,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("Ta2"),
      Cpua,
      eaa,
      Periodic_Type,
      0,
      1,
      10,
      10,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Tasks1,
      To_Unbounded_String ("Tb2"),
      Cpub,
      eab,
      Periodic_Type,
      0,
      3,
      10,
      10,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);

   -- message's parameters
   --
   Initial_M1  := 10.0;
   Initial_M2  := 2.0;
   Response_M1 := Initial_M1;
   Response_M2 := Initial_M2;

   Put (Tasks1);
   New_Line;
   New_Line;

   Compute;

   New_Line;
   New_Line;

   while ((Rta2 /= Rta) and
          (Rtb2 /= Rtb) and
          (M1 /= Response_M1) and
          (M2 /= Response_M2))
   loop

      M1   := Response_M1;
      M2   := Response_M2;
      Rta2 := Rta;
      Rtb2 := Rtb;

      for I in  0 .. Response_Time_Range (Get_Number_Of_Elements (Tasks1) -
                                          1)
      loop
         if (Rta.Entries (I).Item/= null) then
         if (Rta.Entries (I).Item.Name = "Ta1") then
            Response_M1 := Initial_M1 + Rta.Entries (I).Data;
            Task_Set.Set
              (Tasks1,
               To_Unbounded_String ("Tb1"),
               Jitter,
               Natural (Response_M1));
         end if;
         if (Rtb.Entries (I).Item.Name = "Tb2") then
            Response_M2 := Initial_M2 + Rtb.Entries (I).Data;
            Task_Set.Set
              (Tasks1,
               To_Unbounded_String ("Ta2"),
               Jitter,
               Natural (Response_M2));
         end if;
	 end if;
      end loop;

      Put (Tasks1);
      Compute;

   end loop;

end Tindell1;
