------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automatically check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--          
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with networks; use networks;
with networks.predefined; use networks.predefined;
with call_framework; use call_framework;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Tasks;
use Tasks;
with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;
with Messages;
use Messages;
with Message_Set;
use Message_Set;
use Message_Set.Generic_Message_Set;
with Scheduler;
use Scheduler;
use Scheduler.Double_Tasks_Parameters;
with Scheduler.Fixed_Priority.Dm;
use Scheduler.Fixed_Priority.Dm;
with Processor_Set;
use Processor_Set;
with Dependencies;
use Dependencies;
with Task_Dependencies;
use Task_Dependencies;
with Distributed_Scheduling;
use Distributed_Scheduling;
with Offsets;
use Offsets;
use Offsets.Offsets_Table_Package;



procedure Tindell2 is 


   Offset : Offsets_Table;  
   Proc1  : Processors_Set;  
   Sched1 : Scheduler_Ptr;  
   Net1   : Predefined_Network;  
   Tasks1 : Tasks_Set;  
   Msg1,  
   Msg2,  
   Msg3   : Message_Ptr;  
   Msg    : Messages_Set;  
   Dep1   : Tasks_Dependencies;  

   Rt     : Scheduler.Response_Table;  
   Rt_Msg : Network.Response_Table;  

   Cpua : Unbounded_String := To_Unbounded_String ("cpua");  
   Cpub : Unbounded_String := To_Unbounded_String ("cpub");  


begin


call_framework.initialize;
   Initialize(Offset);
   Sched1:=new Dm_Scheduler;


   -- set of processor
   --
   Add_Processor(Proc1,Cpua,Sched1);
   Add_Processor(Proc1,Cpub,Sched1);


   -- parameters of tasks
   --
   Add_Task(Tasks1, To_Unbounded_String("Ta1"), Cpua , Periodic_Type, 0, 1,
      20, 0, 0, 0,
      1, Sched_Fifo, Offset);
   Add_Task(Tasks1, To_Unbounded_String("Tb1"), Cpub , Periodic_Type, 0, 3,
      20, 0, 0, 0,
      1, Sched_Fifo, Offset);
   Add_Task(Tasks1, To_Unbounded_String("Ta2"), Cpua , Periodic_Type, 0, 1,
      10, 0, 0, 0,
      1, Sched_Fifo, Offset);
   Add_Task(Tasks1, To_Unbounded_String("Tb2"), Cpub , Periodic_Type, 0, 3,
      10, 0, 0, 0,
      1, Sched_Fifo, Offset);


   -- message's parameters
   --
   Add_Message(Msg, Msg1, To_Unbounded_String("m1"), 1000, 20, 0, 0);
   Add_Message(Msg, Msg1, To_Unbounded_String("m2"), 1000, 10, 0, 0);
   Add_Message(Msg, Msg2, To_Unbounded_String("m1"), 1000, 20, 0, 0);
   Add_Message(Msg, Msg3, To_Unbounded_String("m2"), 1000, 10, 0, 0);

   -- task's dependencies
   --
   Add_Tasks_Dependencies(Dep1, Tasks1,
      To_Unbounded_String("Ta1"), To_Unbounded_String("Tb1"),
      Dependency_Ptr(Msg2));
   Add_Tasks_Dependencies(Dep1, Tasks1,
      To_Unbounded_String("Tb2"), To_Unbounded_String("Ta2"),
      Dependency_Ptr(Msg3));


   -- network response time
   --
   Register_Message(Net1, To_Unbounded_String("m1"),10.0);
   Register_Message(Net1, To_Unbounded_String("m2"),2.0);


   -- compute end to end delay
   --
   Initialize(Rt);
   Tindell(Tasks1, Msg, Dep1, Proc1, Net1, Rt);
   Put(Rt);

   -- not necessary but used to display message end to end delay
   -- (to compare with tindell1.adb)
   --
   Compute_Transmission_Delay (Net1, Msg, Rt_Msg);
   Put(Rt_Msg);


end Tindell2;
