------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Queueing_System;       use Queueing_System;
with Call_Framework;        use Call_Framework;
with Address_Space_Set;     use Address_Space_Set;
use Address_Space_Set.Generic_Address_Space_Set;
with Scheduler;             use Scheduler;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Unbounded_Strings;     use Unbounded_Strings;
with Tasks;                 use Tasks;
with Buffers;               use Buffers;
use Buffers.Buffer_Roles;
with Buffer_Set;            use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Task_Set;              use Task_Set;
use Task_Set.Generic_Task_Set;
with Processor_Set;         use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Messages;              use Messages;
with Message_Set;           use Message_Set;
with Parameters;            use Parameters;
with Dependencies;          use Dependencies;
with Task_Dependencies;     use Task_Dependencies;
with Systems;               use Systems;
with Offsets;               use Offsets;
use Offsets.Offsets_Table_package;

procedure Write_Sys is

   Sys : System;
   Cpu : Unbounded_String := To_Unbounded_String ("cpu");
   Ea  : Unbounded_String := To_Unbounded_String ("ea");

   T1, T2, T3 : Generic_Task_Ptr;
   Msg1       : Message_Ptr;
   Offset     : Offsets_Table;
   My_Roles   : Buffer_Roles_Table;
   Param      : User_Defined_Parameters_Table;

begin

   Call_Framework.initialize;
   Initialize (My_Roles);
   Initialize (Offset);

   Initialize (Sys);

   Add_Processor
     (Sys.Processors,
      Cpu,
      Preemptive,
      0,
      Empty_String,
      Rate_Monotonic_Protocol,
      Empty_String);

   Add_Address_Space (Sys.Address_Spaces, Ea, Cpu, 0, 0, 0, 0);

   Add_Task
     (Sys.Tasks,
      T1,
      To_Unbounded_String ("T1"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      1,
      12,
      0,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Sys.Tasks,
      T2,
      To_Unbounded_String ("T2"),
      Cpu,
      Ea,
      Periodic_Type,
      1,
      3,
      6,
      0,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);
   Add_Task
     (Sys.Tasks,
      T3,
      To_Unbounded_String ("T3"),
      Cpu,
      Ea,
      Periodic_Type,
      0,
      1,
      3,
      0,
      0,
      0,
      1,
      0,
      Sched_Fifo,
      Offset);

   My_Roles.Entries (0).Item          := T1.Name;
   My_Roles.Entries (0).Data.Size     := 1;
   My_Roles.Entries (0).Data.The_Role := Producer;
   My_Roles.Entries (1).Item          := T2.Name;
   My_Roles.Entries (1).Data.The_Role := Consumer;
   My_Roles.Entries (1).Data.Size     := 1;

   My_Roles.Nb_Entries := 2;

   Add_Buffer
     (Sys.Buffers,
      To_Unbounded_String ("B1"),
      1,
      Cpu,
      Ea,
      Qs_Mm1,
      My_Roles);

   Add_Message
     (Sys.Messages,
      Msg1,
      To_Unbounded_String ("m1"),
      1000,
      20,
      0,
      0,
      Param,
      0,
      0);

   Add_One_Task_Dependency (Sys.Dependencies, T1, Msg1, From_Task_To_Object);
   Add_One_Task_Dependency (Sys.Dependencies, T2, Msg1, From_Object_To_Task);
   Add_One_Task_Dependency (Sys.Dependencies, T3, Msg1, From_Object_To_Task);

   Put_Xml (Sys);

   Write_To_Xml_File (Sys, To_Unbounded_String ("sys_cheddar.xml"));

end Write_Sys;
