------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Text_IO;               use Text_IO;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Systems;               use Systems;
with Xml_Tag;               use Xml_Tag;
with Call_Framework;        use Call_Framework;
use Call_Framework.Framework_Response_Package;
use Call_Framework.Framework_Request_Package;
with Call_Memory_Framework; use Call_Memory_Framework;
use Call_Memory_Framework.Buffer_Result_Package;
with Parameters;            use Parameters;
with Parameters.extended;
use Parameters.extended.Framework_Parameters_Table_Package;
with Unbounded_Strings;     use Unbounded_Strings;
use Unbounded_Strings.Unbounded_String_List_Package;

procedure Xml_Comprehensive_Call is

   Sys : System;

   Response_List : Framework_Response_Table;
   Request_List  : Framework_Request_Table;
   A_Request     : Framework_Request;
   A_Param       : Parameter_ptr;
   dir           : Unbounded_String_List;

begin

   Call_Framework.initialize;
   Put_Line (To_String (Get_Dtd));
   Put_Line ("<results>");

   -- Test on periodic tasks
   --
   Read_From_Xml_File (Sys, dir, To_Unbounded_String ("jitter.xml"));

   Initialize (Response_List);
   Initialize (Request_List);
   Initialize (A_Request);
   A_Request.Statement := Scheduling_Feasibility_Basics;
   Add (Request_List, A_Request);
   Sequential_Framework_Request
     (Sys,
      Request_List,
      Response_List,
      Total_Order,
      Xml_Output);
   Put (Response_List);
   Initialize (Response_List);
   Initialize (Request_List);
   Initialize (A_Request);
   A_Request.Statement   := Scheduling_Simulation_Time_Line;
   A_Param               := new Parameter (Integer_Parameter);
   A_Param.Name          := To_Unbounded_String ("period");
   A_Param.Integer_value := 100;
   Add (A_Request.Param, A_Param);
   A_Param               := new Parameter (Boolean_Parameter);
   A_Param.Name          := To_Unbounded_String ("schedule_with_offsets");
   A_Param.Boolean_value := True;
   Add (A_Request.Param, A_Param);
   A_Param               := new Parameter (Boolean_Parameter);
   A_Param.Name          :=
      To_Unbounded_String ("schedule_with_precedencies");
   A_Param.Boolean_value := True;
   Add (A_Request.Param, A_Param);
   A_Param               := new Parameter (Boolean_Parameter);
   A_Param.Name          := To_Unbounded_String ("schedule_with_resources");
   A_Param.Boolean_value := True;
   Add (A_Request.Param, A_Param);
   A_Param               := new Parameter (Integer_Parameter);
   A_Param.Name          := To_Unbounded_String ("seed_value");
   A_Param.Integer_value := 0;
   Add (A_Request.Param, A_Param);
   Add (Request_List, A_Request);
   Sequential_Framework_Request
     (Sys,
      Request_List,
      Response_List,
      Total_Order,
      Xml_Output);
   Put (Response_List);
   Initialize (Request_List);
   Initialize (Response_List);
   Initialize (A_Request);
   A_Request.Statement := Scheduling_Simulation_Basics;
   Add (Request_List, A_Request);
   Sequential_Framework_Request
     (Sys,
      Request_List,
      Response_List,
      Total_Order,
      Xml_Output);
   Put (Response_List);
  

   Put_Line ("</results>");

end Xml_Comprehensive_Call;
