------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Gtk;               use Gtk;
with Aadl_Config;       use Aadl_Config;
with Unbounded_Strings; use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;

package body Aadl_Options_Pkg.Callbacks is

   procedure Page_Switch
     (Notebook : access Gtk_Notebook_Record'Class;
      Page     : in Gtk.Gtk_Notebook_Page;
      Page_Num : in Gtk_Notebook)
   is
      pragma Warnings (Off, Page_Num);
      Old_Page : constant Gtk.Gtk_Notebook_Page := Get_Cur_Page (Notebook);
      Pixmap   : Gtk_Pixmap;
   begin
      Pixmap := Gtk_Pixmap (Get_Child (Gtk_Box (Get_Tab_Label (Page)), 0));
      Gtk.Pixmap.Set
        (Pixmap,
         Aadl_Options.Book_Opened,
         Aadl_Options.Book_Opened_Mask);
      Pixmap :=
        Gtk_Pixmap (Get_Child (Gtk_Box (Get_Menu_Label (Page)), 0));
      Gtk.Pixmap.Set
        (Pixmap,
         Aadl_Options.Book_Opened,
         Aadl_Options.Book_Opened_Mask);

      if Old_Page /= null then
         Pixmap :=
           Gtk_Pixmap (Get_Child (Gtk_Box (Get_Tab_Label (Old_Page)), 0));
         Gtk.Pixmap.Set
           (Pixmap,
            Aadl_Options.Book_Closed,
            Aadl_Options.Book_Closed_Mask);
         Pixmap :=
           Gtk_Pixmap (Get_Child (Gtk_Box (Get_Menu_Label (Old_Page)), 0));
         Gtk.Pixmap.Set
           (Pixmap,
            Aadl_Options.Book_Closed,
            Aadl_Options.Book_Closed_Mask);
      end if;
   end Page_Switch;

   procedure On_Ok_Pressed (Object : access Gtk_Button_Record'Class) is

   begin

      -- Activate AADL Debug switch
      --
      if Get_Active (Aadl_Options.Aadl_Debug) then
         Aadl_Debug := True;
      else
         Aadl_Debug := False;
      end if;

      -- Include system/sub-system name in Cheddar's features name
      --
      if Get_Active (Aadl_Options.Aadl_Import_With_System_Name) then
         Aadl_Import_With_System_Name := True;
      else
         Aadl_Import_With_System_Name := False;
      end if;

      -- Transform event and event data port to task precedencies
      --
      if Get_Active (Aadl_Options.Aadl_Import_Event_To_Precedencies) then
         Aadl_Import_Event_To_Precedencies := True;
      else
         Aadl_Import_Event_To_Precedencies := False;
      end if;
      if Get_Active
           (Aadl_Options.Aadl_Import_Event_Data_To_Precedencies)
      then
         Aadl_Import_Event_Data_To_Precedencies := True;
      else
         Aadl_Import_Event_Data_To_Precedencies := False;
      end if;

      -- Transform event port and event data port to buffers and messages
      -- (it automatically includes buffer/message task dependencies)
      --
      if Get_Active (Aadl_Options.Aadl_Import_Event_To_Buffers_Messages) then
         Aadl_Import_Event_To_Buffers_Messages := True;
      else
         Aadl_Import_Event_To_Buffers_Messages := False;
      end if;
      if Get_Active
           (Aadl_Options.Aadl_Import_Event_Data_To_Buffers_Messages)
      then
         Aadl_Import_Event_Data_To_Buffers_Messages := True;
      else
         Aadl_Import_Event_Data_To_Buffers_Messages := False;
      end if;

      -- Transform task precedencies to event port
      --
      if Get_Active (Aadl_Options.Aadl_Export_Precedencies_To_Event) then
         Aadl_Export_Precedencies_To_Event := True;
      else
         Aadl_Export_Precedencies_To_Event := False;
      end if;

      -- Transform task precedencies to event data port
      --
      if Get_Active
           (Aadl_Options.Aadl_Export_Precedencies_To_Event_Data)
      then
         Aadl_Export_Precedencies_To_Event_Data := True;
      else
         Aadl_Export_Precedencies_To_Event_Data := False;
      end if;

      -- Transform task precedencies to a Cheddar property
      --
      if Get_Active
           (Aadl_Options.Aadl_Export_Precedencies_To_Properties)
      then
         Aadl_Export_Precedencies_To_Properties := True;
      else
         Aadl_Export_Precedencies_To_Properties := False;
      end if;

      -- Process second and third AADL import passes
      --

      if Get_Active (Aadl_Options.Aadl_Process_Second_Import_Pass) then
         Aadl_Process_Second_Import_Pass := True;
      else
         Aadl_Process_Second_Import_Pass := False;
      end if;
      if Get_Active (Aadl_Options.Aadl_Process_Third_Import_Pass) then
         Aadl_Process_Third_Import_Pass := True;
      else
         Aadl_Process_Third_Import_Pass := False;
      end if;

      Destroy (Aadl_Options);

   end On_Ok_Pressed;

   procedure On_Cancel_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      Destroy (Aadl_Options);
   end On_Cancel_Pressed;

end Aadl_Options_Pkg.Callbacks;
