------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Gtk;                        use Gtk;
with Gdk.Types;                  use Gdk.Types;
with Gtk.Enums;                  use Gtk.Enums;
with Gdk.Color;                  use Gdk.Color;
with Xpm;                        use Xpm;
with Callbacks_Cheddar;          use Callbacks_Cheddar;
with Aadl_Options_Pkg.Callbacks; use Aadl_Options_Pkg.Callbacks;
with Framework_Config;           use Framework_Config;
with Translate;                  use Translate;
with Ada.Strings.Unbounded;      use Ada.Strings.Unbounded;
with Aadl_Config;                use Aadl_Config;
with Gtk.Widget;                 use Gtk.Widget;

package body Aadl_Options_Pkg is

   procedure Gtk_New (Aadl_Options : out Aadl_Options_Access) is

   begin
      Aadl_Options := new Aadl_Options_Record;
      Aadl_Options_Pkg.Initialize (Aadl_Options);
   end Gtk_New;

   procedure Initialize (Aadl_Options : access Aadl_Options_Record'Class) is

   begin

      Gtk.Window.Initialize (Aadl_Options, Window_Toplevel);
      Set_Title
        (Aadl_Options,
         To_String (Lb_Aadl_Options (Current_Language)));
      Set_Policy (Aadl_Options, True, True, True);
      Set_Position (Aadl_Options, Win_Pos_Center_Always);
      Set_Modal (Aadl_Options, True);
      Set_Extension_Events (Aadl_Options, Extension_Events_All);
      Set_Flags (Aadl_Options, Can_Default);
      Grab_Focus (Aadl_Options);

      Gtk.Frame.Gtk_New (Aadl_Options.Main_Frame, "");
      Add (Aadl_Options, Aadl_Options.Main_Frame);

      -- Create Notebook
      --
      Gtk_New (Aadl_Options.Notebook);
      Set_Tab_Pos (Aadl_Options.Notebook, Pos_Top);
      Gtk_New_Vbox (Aadl_Options.Options_Vbox);
      Add (Aadl_Options.Main_Frame, Aadl_Options.Options_Vbox);
      Add (Aadl_Options.Options_Vbox, Aadl_Options.Notebook);
      Set_Border_Width (Aadl_Options.Notebook, 10);
      Realize (Aadl_Options.Notebook);

      Create_From_Xpm_D
        (Aadl_Options.Book_Opened,
         Get_Window (Aadl_Options.Notebook),
         Aadl_Options.Book_Opened_Mask,
         Null_Color,
         Book_Opened_Xpm);
      Create_From_Xpm_D
        (Aadl_Options.Book_Closed,
         Get_Window (Aadl_Options.Notebook),
         Aadl_Options.Book_Closed_Mask,
         Null_Color,
         Book_Closed_Xpm);

      -- AADL import page
      --
      Gtk_New
        (Aadl_Options.Child_Page,
         To_String (Lb_Import_Aadl (Current_Language)));

      Gtk_New (Aadl_Options.Value_Table, 8, 2, True);
      Add (Aadl_Options.Child_Page, Aadl_Options.Value_Table);
      Set_Border_Width (Aadl_Options.Value_Table, 14);
      Set_Row_Spacings (Aadl_Options.Value_Table, 8);
      Set_Col_Spacings (Aadl_Options.Value_Table, 0);

      Gtk_New_Hbox (Aadl_Options.Label_Box, False, 0);
      Gtk_New
        (Aadl_Options.Pixmap,
         Aadl_Options.Book_Opened,
         Aadl_Options.Book_Opened_Mask);
      Pack_Start
        (Aadl_Options.Label_Box,
         Aadl_Options.Pixmap,
         False,
         True,
         0);
      Set_Padding (Aadl_Options.Pixmap, 3, 1);

      Gtk_New
        (Aadl_Options.Lab,
         To_String (Lb_Import_Aadl (Current_Language)));
      Pack_Start (Aadl_Options.Label_Box, Aadl_Options.Lab, False, True, 0);
      Show_All (Aadl_Options.Label_Box);

      Gtk_New_Vbox (Aadl_Options.Menu_Box, False, 0);
      Gtk_New
        (Aadl_Options.Pixmap,
         Aadl_Options.Book_Opened,
         Aadl_Options.Book_Opened_Mask);
      Pack_Start (Aadl_Options.Menu_Box, Aadl_Options.Pixmap, False, True, 0);
      Set_Padding (Aadl_Options.Pixmap, 3, 1);

      Gtk_New
        (Aadl_Options.Lab,
         To_String (Lb_Import_Aadl (Current_Language)));
      Pack_Start (Aadl_Options.Menu_Box, Aadl_Options.Lab, False, True, 0);
      Show_All (Aadl_Options.Menu_Box);

      Append_Page_Menu
        (Aadl_Options.Notebook,
         Aadl_Options.Child_Page,
         Aadl_Options.Label_Box,
         Aadl_Options.Menu_Box);

      -- Include system/sub-system name in Cheddar's features name
      --
      Gtk_New
        (Aadl_Options.Aadl_Import_With_System_Name,
         To_String (Lb_Import_Aadl_System_Name (Current_Language)));
      if Aadl_Import_With_System_Name then
         Set_Active (Aadl_Options.Aadl_Import_With_System_Name, True);
      else
         Set_Active (Aadl_Options.Aadl_Import_With_System_Name, False);
      end if;
      Attach
        (Aadl_Options.Value_Table,
         Aadl_Options.Aadl_Import_With_System_Name,
         0,
         2,
         0,
         1);

      -- Transform event port and event data port to buffers and messages
      -- (it automatically includes buffer/message task dependencies)
      --
      Gtk_New
        (Aadl_Options.Aadl_Import_Event_To_Buffers_Messages,
         To_String
            (Lb_Import_Aadl_Event_To_Buffers_Messages (Current_Language)));
      if Aadl_Import_Event_To_Buffers_Messages then
         Set_Active
           (Aadl_Options.Aadl_Import_Event_To_Buffers_Messages,
            True);
      else
         Set_Active
           (Aadl_Options.Aadl_Import_Event_To_Buffers_Messages,
            False);
      end if;
      Attach
        (Aadl_Options.Value_Table,
         Aadl_Options.Aadl_Import_Event_To_Buffers_Messages,
         0,
         2,
         1,
         2);
      Gtk_New
        (Aadl_Options.Aadl_Import_Event_Data_To_Buffers_Messages,
         To_String
            (Lb_Import_Aadl_Event_Data_To_Buffers_Messages (Current_Language)))
;
      if Aadl_Import_Event_Data_To_Buffers_Messages then
         Set_Active
           (Aadl_Options.Aadl_Import_Event_Data_To_Buffers_Messages,
            True);
      else
         Set_Active
           (Aadl_Options.Aadl_Import_Event_Data_To_Buffers_Messages,
            False);
      end if;
      Attach
        (Aadl_Options.Value_Table,
         Aadl_Options.Aadl_Import_Event_Data_To_Buffers_Messages,
         0,
         2,
         2,
         3);

      -- Transform event port or event data port to task precedencies
      --
      Gtk_New
        (Aadl_Options.Aadl_Import_Event_To_Precedencies,
         To_String (Lb_Import_Aadl_Event_To_Precedencies (Current_Language)));
      if Aadl_Import_Event_To_Precedencies then
         Set_Active (Aadl_Options.Aadl_Import_Event_To_Precedencies, True);
      else
         Set_Active (Aadl_Options.Aadl_Import_Event_To_Precedencies, False);
      end if;
      Attach
        (Aadl_Options.Value_Table,
         Aadl_Options.Aadl_Import_Event_To_Precedencies,
         0,
         2,
         3,
         4);
      Gtk_New
        (Aadl_Options.Aadl_Import_Event_Data_To_Precedencies,
         To_String
            (Lb_Import_Aadl_Event_Data_To_Precedencies (Current_Language)));
      if Aadl_Import_Event_Data_To_Precedencies then
         Set_Active
           (Aadl_Options.Aadl_Import_Event_Data_To_Precedencies,
            True);
      else
         Set_Active
           (Aadl_Options.Aadl_Import_Event_Data_To_Precedencies,
            False);
      end if;
      Attach
        (Aadl_Options.Value_Table,
         Aadl_Options.Aadl_Import_Event_Data_To_Precedencies,
         0,
         2,
         4,
         5);

      -- Process the second and the third pass during AADL import
      --
      Gtk_New
        (Aadl_Options.Aadl_Process_Second_Import_Pass,
         To_String (Lb_Aadl_Process_Second_Import_Pass (Current_Language)));
      if Aadl_Process_Second_Import_Pass then
         Set_Active (Aadl_Options.Aadl_Process_Second_Import_Pass, True);
      else
         Set_Active (Aadl_Options.Aadl_Process_Second_Import_Pass, False);
      end if;
      Attach
        (Aadl_Options.Value_Table,
         Aadl_Options.Aadl_Process_Second_Import_Pass,
         0,
         2,
         5,
         6);
      Gtk_New
        (Aadl_Options.Aadl_Process_Third_Import_Pass,
         To_String (Lb_Aadl_Process_Third_Import_Pass (Current_Language)));
      if Aadl_Process_Third_Import_Pass then
         Set_Active (Aadl_Options.Aadl_Process_Third_Import_Pass, True);
      else
         Set_Active (Aadl_Options.Aadl_Process_Third_Import_Pass, False);
      end if;
      Attach
        (Aadl_Options.Value_Table,
         Aadl_Options.Aadl_Process_Third_Import_Pass,
         0,
         2,
         6,
         7);

      --
      -- button box of the widget
      --

      Gtk_New_Hseparator (Aadl_Options.Hseparator);
      Pack_Start (Aadl_Options.Options_Vbox, Aadl_Options.Hseparator);

      Gtk_New (Aadl_Options.Button_Hbox);
      Set_Layout (Aadl_Options.Button_Hbox, Buttonbox_Spread);
      Set_Spacing (Aadl_Options.Button_Hbox, 10);
      Set_Child_Size (Aadl_Options.Button_Hbox, 20, 20);
      Set_Child_Ipadding (Aadl_Options.Button_Hbox, 7, 0);
      Pack_Start
        (Aadl_Options.Options_Vbox,
         Aadl_Options.Button_Hbox,
         False,
         True,
         5);

      Gtk_New (Aadl_Options.Ok_Button, "Ok");
      Add (Aadl_Options.Button_Hbox, Aadl_Options.Ok_Button);
      Button_Callback.Connect
        (Aadl_Options.Ok_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'Access));

      Gtk_New
        (Aadl_Options.Cancel_Button,
         To_String (Lb_Cancel (Current_Language)));
      Add (Aadl_Options.Button_Hbox, Aadl_Options.Cancel_Button);
      Button_Callback.Connect
        (Aadl_Options.Cancel_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'Access));

      -- AADL export page
      --
      Gtk_New
        (Aadl_Options.Child_Page,
         To_String (Lb_Export_Aadl (Current_Language)));

      Gtk_New (Aadl_Options.Value_Table, 8, 2, True);
      Add (Aadl_Options.Child_Page, Aadl_Options.Value_Table);
      Set_Border_Width (Aadl_Options.Value_Table, 14);
      Set_Row_Spacings (Aadl_Options.Value_Table, 8);
      Set_Col_Spacings (Aadl_Options.Value_Table, 0);

      Gtk_New_Hbox (Aadl_Options.Label_Box, False, 0);
      Gtk_New
        (Aadl_Options.Pixmap,
         Aadl_Options.Book_Closed,
         Aadl_Options.Book_Closed_Mask);
      Pack_Start
        (Aadl_Options.Label_Box,
         Aadl_Options.Pixmap,
         False,
         True,
         0);
      Set_Padding (Aadl_Options.Pixmap, 3, 1);

      Gtk_New
        (Aadl_Options.Lab,
         To_String (Lb_Export_Aadl (Current_Language)));
      Pack_Start (Aadl_Options.Label_Box, Aadl_Options.Lab, False, True, 0);
      Show_All (Aadl_Options.Label_Box);

      Gtk_New_Vbox (Aadl_Options.Menu_Box, False, 0);
      Gtk_New
        (Aadl_Options.Pixmap,
         Aadl_Options.Book_Closed,
         Aadl_Options.Book_Closed_Mask);
      Pack_Start (Aadl_Options.Menu_Box, Aadl_Options.Pixmap, False, True, 0);
      Set_Padding (Aadl_Options.Pixmap, 3, 1);

      Gtk_New
        (Aadl_Options.Lab,
         To_String (Lb_Generate_Events (Current_Language)));
      Pack_Start (Aadl_Options.Menu_Box, Aadl_Options.Lab, False, True, 0);
      Show_All (Aadl_Options.Menu_Box);

      Append_Page_Menu
        (Aadl_Options.Notebook,
         Aadl_Options.Child_Page,
         Aadl_Options.Label_Box,
         Aadl_Options.Menu_Box);

      -- Transform task precedencies to properties
      --
      Gtk_New
        (Aadl_Options.Aadl_Export_Precedencies_To_Properties,
         To_String
            (Lb_Export_Aadl_Precedencies_To_Properties (Current_Language)));
      if Aadl_Export_Precedencies_To_Properties then
         Set_Active
           (Aadl_Options.Aadl_Export_Precedencies_To_Properties,
            True);
      else
         Set_Active
           (Aadl_Options.Aadl_Export_Precedencies_To_Properties,
            False);
      end if;
      Attach
        (Aadl_Options.Value_Table,
         Aadl_Options.Aadl_Export_Precedencies_To_Properties,
         0,
         2,
         0,
         1);

      -- Transform task precedencies to event port
      --
      Gtk_New
        (Aadl_Options.Aadl_Export_Precedencies_To_Event,
         To_String (Lb_Export_Aadl_Precedencies_To_Event (Current_Language)));
      if Aadl_Export_Precedencies_To_Event then
         Set_Active (Aadl_Options.Aadl_Export_Precedencies_To_Event, True);
      else
         Set_Active (Aadl_Options.Aadl_Export_Precedencies_To_Event, False);
      end if;
      Attach
        (Aadl_Options.Value_Table,
         Aadl_Options.Aadl_Export_Precedencies_To_Event,
         0,
         2,
         1,
         2);

      -- Transform task precedencies to event data port
      --
      Gtk_New
        (Aadl_Options.Aadl_Export_Precedencies_To_Event_Data,
         To_String
            (Lb_Export_Aadl_Precedencies_To_Event_Data (Current_Language)));
      if Aadl_Export_Precedencies_To_Event_Data then
         Set_Active
           (Aadl_Options.Aadl_Export_Precedencies_To_Event_Data,
            True);
      else
         Set_Active
           (Aadl_Options.Aadl_Export_Precedencies_To_Event_Data,
            False);
      end if;
      Attach
        (Aadl_Options.Value_Table,
         Aadl_Options.Aadl_Export_Precedencies_To_Event_Data,
         0,
         2,
         2,
         3);

      -- AADL  other options
      --
      Gtk_New
        (Aadl_Options.Child_Page,
         To_String (Lb_Other_Aadl_Options (Current_Language)));

      Gtk_New (Aadl_Options.Value_Table, 8, 2, True);
      Add (Aadl_Options.Child_Page, Aadl_Options.Value_Table);
      Set_Border_Width (Aadl_Options.Value_Table, 14);
      Set_Row_Spacings (Aadl_Options.Value_Table, 8);
      Set_Col_Spacings (Aadl_Options.Value_Table, 0);

      Gtk_New_Hbox (Aadl_Options.Label_Box, False, 0);
      Gtk_New
        (Aadl_Options.Pixmap,
         Aadl_Options.Book_Closed,
         Aadl_Options.Book_Closed_Mask);
      Pack_Start
        (Aadl_Options.Label_Box,
         Aadl_Options.Pixmap,
         False,
         True,
         0);
      Set_Padding (Aadl_Options.Pixmap, 3, 1);

      Gtk_New
        (Aadl_Options.Lab,
         To_String (Lb_Other_Aadl_Options (Current_Language)));
      Pack_Start (Aadl_Options.Label_Box, Aadl_Options.Lab, False, True, 0);
      Show_All (Aadl_Options.Label_Box);

      Gtk_New_Vbox (Aadl_Options.Menu_Box, False, 0);
      Gtk_New
        (Aadl_Options.Pixmap,
         Aadl_Options.Book_Closed,
         Aadl_Options.Book_Closed_Mask);
      Pack_Start (Aadl_Options.Menu_Box, Aadl_Options.Pixmap, False, True, 0);
      Set_Padding (Aadl_Options.Pixmap, 3, 1);

      Append_Page_Menu
        (Aadl_Options.Notebook,
         Aadl_Options.Child_Page,
         Aadl_Options.Label_Box,
         Aadl_Options.Menu_Box);

      -- Activate AADL Debug switch
      --
      Gtk_New
        (Aadl_Options.Aadl_Debug,
         To_String (Lb_Aadl_Debug (Current_Language)));
      if Aadl_Debug then
         Set_Active (Aadl_Options.Aadl_Debug, True);
      else
         Set_Active (Aadl_Options.Aadl_Debug, False);
      end if;
      Attach (Aadl_Options.Value_Table, Aadl_Options.Aadl_Debug, 0, 2, 0, 1);

      -- Notebook page switching activation
      --
      Two_Notebook_Callback.Connect
        (Aadl_Options.Notebook,
         "switch_page",
         Two_Notebook_Callback.To_Marshaller (Page_Switch'Access),
         Aadl_Options.Notebook);

   end Initialize;

end Aadl_Options_Pkg;
