------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff
-- Cheddar is developed by the LIMI Team/LISYC, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Message_Set;             use Message_Set;
use Message_Set.Generic_Message_Set;
with Graph_Item_Pkg;          use Graph_Item_Pkg;
with Graph_Window_Pkg;        use Graph_Window_Pkg;
with My_Canvas;               use My_Canvas;
with Update_Messages_Set_Pkg; use Update_Messages_Set_Pkg;
with Update_Buffers_Set_Pkg;  use Update_Buffers_Set_Pkg;
with Update_Tasks_Set_Pkg;    use Update_Tasks_Set_Pkg;
with Gtk.GEntry;              use Gtk.GEntry;
with Translate;               use Translate;
with Project;                 use Project;
with Framework_Config;        use Framework_Config;

package body Graph_Add_Pkg.Callbacks is

   procedure On_Ok_Pressed (Object : access Gtk_Button_Record'Class) is
      Name : Unbounded_String;
      Item : Item_Object_Access;
   begin

      Is_Saved := False;

      Name :=
         To_Unbounded_String
           (Gtk.GEntry.Get_Text (Get_Entry (Add_New_Item.Combo)));
      if (Add_New_Item.Item_Type = "message") then
         if (Name = To_String (Lb_New (Current_Language))) then
            Gtk_New
              (Update_Messages_Set,
               To_Unbounded_String ("graph_window"));
            Show_Advanced (Update_Messages_Set, False);

         else
            Item :=
              Item_Object_Access (Search_Item_By_Name
                                     (Graph_Window.Buffer,
                                      To_Unbounded_String ("message"),
                                      Name));
            Set_Visibility (Item, True);
            My_Canvas.Remove
              (Graph_Window.Buffer,
               Items_List (Last_Item - 1));
         end if;
      end if;

      if (Add_New_Item.Item_Type = "buffer") then
         if (Name = To_String (Lb_New (Current_Language))) then
            Gtk_New
              (Update_Buffers_Set,
               Lb_Add_Buffer (Current_Language),
               To_Unbounded_String ("graph_window"));
            Show_Advanced (Update_Buffers_Set, False);
         else
            Item :=
              Item_Object_Access (Search_Item_By_Name
                                     (Graph_Window.Buffer,
                                      To_Unbounded_String ("buffer"),
                                      Name));
            Set_Visibility (Item, True);
            My_Canvas.Remove
              (Graph_Window.Buffer,
               Items_List (Last_Item - 1));
         end if;
      end if;

      if (Add_New_Item.Item_Type = "task") then
         if (Name = To_String (Lb_New (Current_Language))) then
            Gtk_New (Update_Tasks_Set, Lb_Add_Task (Current_Language));
            Show_Advanced (Update_Tasks_Set, False);
         else
            Item :=
              Item_Object_Access (Search_Item_By_Name
                                     (Graph_Window.Buffer,
                                      To_Unbounded_String ("task"),
                                      Name));
            Set_Visibility (Item, True);
            My_Canvas.Remove
              (Graph_Window.Buffer,
               Items_List (Last_Item - 1));
         end if;
      end if;

      Hide (Add_New_Item);

   end On_Ok_Pressed;

   procedure On_Cancel_Pressed (Object : access Gtk_Button_Record'Class) is
   begin

      My_Canvas.Remove (Graph_Window.Buffer, Items_List (Last_Item - 1));
      Hide (Add_New_Item);
   end On_Cancel_Pressed;

end Graph_Add_Pkg.Callbacks;
