------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Gtk;                     use Gtk;
with Gdk.Types;               use Gdk.Types;
with Gtk.Widget;              use Gtk.Widget;
with Callbacks_Cheddar;       use Callbacks_Cheddar;
with Graph_Add_Pkg.Callbacks; use Graph_Add_Pkg.Callbacks;
with Gtk.GEntry;              use Gtk.GEntry;
with Graph_Window_Pkg;        use Graph_Window_Pkg;
with My_Canvas;               use My_Canvas;
with Translate;               use Translate;
with Framework_Config;        use Framework_Config;

package body Graph_Add_Pkg is

   procedure Gtk_New (Add_New_Item : out Add_Item_Access; Item : in String) is
   begin
      pragma Warnings (Off);

      Add_New_Item           := new Add_Item_Record;
      Add_New_Item.Item_Type := To_Unbounded_String (Item);
      Graph_Add_Pkg.Initialize (Add_New_Item);
   end Gtk_New;

   procedure Initialize (Add_New_Item : access Add_Item_Record'Class) is

      function Internal_Append
        (Canva : access Interactive_Canvas_Record'Class;
         Item  : access Canvas_Item_Record'Class)
         return  Boolean
      is
      begin

         if (Add_New_Item.Item_Type = "message") then
            if (Get_Figure (Item) = To_Unbounded_String ("message"))
              and then not Is_Visible (Item)
            then
               String_List.Append
                 (Add_New_Item.Items,
                  To_String (Get_Name (Item)));
            end if;
         end if;

         if (Add_New_Item.Item_Type = "buffer") then
            if (Get_Figure (Item) = To_Unbounded_String ("buffer"))
              and then not Is_Visible (Item)
            then
               String_List.Append
                 (Add_New_Item.Items,
                  To_String (Get_Name (Item)));
            end if;
         end if;

         if (Add_New_Item.Item_Type = "task") then
            if (Get_Figure (Item) = To_Unbounded_String ("task"))
              and then not Is_Visible (Item)
            then
               String_List.Append
                 (Add_New_Item.Items,
                  To_String (Get_Name (Item)));
            end if;
         end if;

         return True;
      end Internal_Append;

   begin
      Gtk.Window.Initialize (Add_New_Item, Window_Toplevel);
      Set_Title (Add_New_Item, To_String (Lb_New_Item (Current_Language)));
      Set_Policy (Add_New_Item, True, True, True);
      Set_Position (Add_New_Item, Win_Pos_Center_Always);
      Set_Extension_Events (Add_New_Item, Extension_Events_All);
      Set_Flags (Add_New_Item, Can_Default);
      Grab_Focus (Add_New_Item);

      Gtk_New_Vbox (Add_New_Item.Vbox, True, 0);
      Set_Border_Width (Add_New_Item.Vbox, 6);
      Add (Add_New_Item, Add_New_Item.Vbox);

      Gtk_New
        (Add_New_Item.Label,
         To_String (Lb_Choose_Item (Current_Language)));
      Add (Add_New_Item.Vbox, Add_New_Item.Label);

      Import_Task (Graph_Window);
      Import_Message (Graph_Window);
      Import_Buffer (Graph_Window);

      String_List.Append
        (Add_New_Item.Items,
         To_String (Lb_New (Current_Language)));
      For_Each_Item
        (Graph_Window.Buffer,
         Internal_Append'Unrestricted_Access);

      Gtk_New (Add_New_Item.Combo);
      Set_Popdown_Strings (Add_New_Item.Combo, Add_New_Item.Items);
      Set_Editable (Get_Entry (Add_New_Item.Combo), False);
      Add (Add_New_Item.Vbox, Add_New_Item.Combo);

      Gtk_New_Hseparator (Add_New_Item.Hseparator);
      Add (Add_New_Item.Vbox, Add_New_Item.Hseparator);

      Gtk_New_Hbox (Add_New_Item.Hbox, True, 0);
      Set_Border_Width (Add_New_Item.Hbox, 6);
      Add (Add_New_Item.Vbox, Add_New_Item.Hbox);

      Gtk_New (Add_New_Item.Ok_Button, "Ok");
      Pack_Start (Add_New_Item.Hbox, Add_New_Item.Ok_Button, True, True, 0);
      Button_Callback.Connect
        (Add_New_Item.Ok_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'Access));

      Gtk_New_Vseparator (Add_New_Item.Vseparator);
      Set_Sensitive (Add_New_Item.Vseparator, False);
      Pack_Start (Add_New_Item.Hbox, Add_New_Item.Vseparator, True, True, 0);

      Gtk_New
        (Add_New_Item.Cancel_Button,
         To_String (Lb_Cancel (Current_Language)));
      Pack_Start
        (Add_New_Item.Hbox,
         Add_New_Item.Cancel_Button,
         True,
         True,
         0);
      Button_Callback.Connect
        (Add_New_Item.Cancel_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'Access));

      Show_All (Add_New_Item);

   end Initialize;

end Graph_Add_Pkg;
