------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Gdk.Drawable;     use Gdk.Drawable;
with Gdk.GC;           use Gdk.GC;
with Gtk;              use Gtk;
with Gtk.Widget;       use Gtk.Widget;
with Processor_Set;    use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Project;          use Project;
with Translate;        use Translate;
with Framework_Config; use Framework_Config;
with Mmi_Tools;        use Mmi_Tools;
with Graph_Add_Pkg;    use Graph_Add_Pkg;
with Tasks;            use Tasks;
with Task_Set;         use Task_Set;
use Task_Set.Generic_Task_Set;
with Buffers;          use Buffers;
with Buffer_Set;       use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Messages;         use Messages;
with Message_Set;      use Message_Set;
use Message_Set.Generic_Message_Set;

package body Graph_Item_Pkg is

   procedure Item_Object_New
     (Item  : out Item_Object_Access;
      Fig   : in Unbounded_String;
      Start : in Boolean)
   is
   begin
      Item := new Item_Object_Record;
      Set_Figure (Item, Fig);
      if (To_String (Fig) = "task") then
         Intialize_Task (Item, Start);
      else
         if (To_String (Fig) = "message") then
            Intialize_Message (Item, Start);
         else
            if (To_String (Fig) = "buffer") then
               Intialize_Buffer (Item, Start);
            end if;
         end if;
      end if;
   end Item_Object_New;

   procedure Intialize_Task
     (Item  : access Item_Object_Record;
      Start : in Boolean)
   is

   begin
      if (Get_Number_Of_Elements (Sys.Processors) = 0) then
         Show_Message_Box (Lb_Define_Processors_Before (Current_Language));
      else
         if not Start then
            Gtk_New (Add_New_Item, "task");
         end if;

         Item.Color := Parse ("white");
         Item.W     := 50;
         Item.H     := 50;
         Set_Id (Item, Last_Item);
         if (Get_Id (Item) /= 0) then
            if Last_Item <= Items_List'Last then
               Items_List (Get_Id (Item))  := Canvas_Item (Item);
            end if;
         end if;
         Last_Item := Last_Item + 1;
         Set_Screen_Size (Item, Item.W, Item.H);
      end if;
   end Intialize_Task;

   procedure Intialize_Message
     (Item  : access Item_Object_Record;
      Start : in Boolean)
   is
   begin
      if not Start then
         Gtk_New (Add_New_Item, "message");
      end if;
      Item.Color := Parse ("white");
      Item.W     := 90;
      Item.H     := 50;
      Set_Id (Item, Last_Item);
      if (Get_Id (Item) /= 0) then
         if Last_Item <= Items_List'Last then
            Items_List (Get_Id (Item))  := Canvas_Item (Item);
         end if;
      end if;
      Last_Item := Last_Item + 1;
      Set_Screen_Size (Item, Item.W, Item.H);
   end Intialize_Message;

   procedure Intialize_Buffer
     (Item  : access Item_Object_Record;
      Start : in Boolean)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Processors) = 0) then
         Show_Message_Box (Lb_Define_Processors_Before (Current_Language));
      else
         Item.Color := Parse ("white");
         Item.W     := 90;
         Item.H     := 50;
         Set_Id (Item, Last_Item);
         if (Get_Id (Item) /= 0) then
            if Last_Item <= Items_List'Last then
               Items_List (Get_Id (Item))  := Canvas_Item (Item);
            end if;
         end if;
         Last_Item := Last_Item + 1;
         Set_Screen_Size (Item, Item.W, Item.H);
         if not Start then
            Gtk_New (Add_New_Item, "buffer");
         end if;
      end if;
   end Intialize_Buffer;

   procedure Draw
     (Item         : access Item_Object_Record;
      Canvas       : access Interactive_Canvas_Record'Class;
      Dest         : Gdk.Pixmap.Gdk_Pixmap;
      Xdest, Ydest : Glib.Gint)
   is
      Tmp : Unbounded_String;
   begin
      Tmp := Get_Figure (Item);
      if (To_String (Tmp) = "task") then
         Draw_Task (Item, Canvas, Dest, Xdest, Ydest);
      end if;
      if (To_String (Tmp) = "message") then
         Draw_Message (Item, Canvas, Dest, Xdest, Ydest);
      end if;
      if (To_String (Tmp) = "buffer") then
         Draw_Buffer (Item, Canvas, Dest, Xdest, Ydest);
      end if;
   end Draw;

   procedure Draw_Task
     (Item         : access Item_Object_Record;
      Canvas       : access Interactive_Canvas_Record'Class;
      Dest         : Gdk.Pixmap.Gdk_Pixmap;
      Xdest, Ydest : Glib.Gint)
   is

      Green_Gc : Gdk_GC;
   begin

      Set_Screen_Size
        (Item,
         To_Canvas_Coordinates (Canvas, Item.W),
         To_Canvas_Coordinates (Canvas, Item.H));
      Gdk_New (Green_Gc, Get_Window (Canvas));
      Set_Foreground
        (Green_Gc,
         Gdk.Color.Black (Gtk.Widget.Get_Default_Colormap));
      if Item.Selected then
         Set_Foreground (Green_Gc, Parse ("white"));
         Set_Foreground (Green_Gc, Parse ("white"));
         Draw_Arc
           (Drawable => Dest,
            Gc       => Green_Gc,
            Filled   => True,
            X        => Xdest,
            Y        => Ydest,
            Width    => To_Canvas_Coordinates (Canvas, Item.W),
            Height   => To_Canvas_Coordinates (Canvas, Item.H),
            Angle1   => 0,
            Angle2   => 360 * 64);
      end if;

      Set_Foreground
        (Green_Gc,
         Gdk.Color.Black (Gtk.Widget.Get_Default_Colormap));
      Draw_Arc
        (Drawable => Dest,
         Gc       => Green_Gc,
         Filled   => False,
         X        => Xdest,
         Y        => Ydest,
         Width    => To_Canvas_Coordinates (Canvas, Item.W),
         Height   => To_Canvas_Coordinates (Canvas, Item.H),
         Angle1   => 0,
         Angle2   => 360 * 64);

      if (Get_Name (Item) /= "") then
         Load
           (Font,
            "-adobe-courier-bold-o-normal--10-100-75-75-m-60-iso8859-1");
         Draw_Text
           (Dest,
            Font,
            Green_Gc,
            Xdest,
            Ydest - 4,
            To_String (Get_Name (Item)));
      end if;
   end Draw_Task;

   procedure Draw_Message
     (Item         : access Item_Object_Record;
      Canvas       : access Interactive_Canvas_Record'Class;
      Dest         : Gdk.Pixmap.Gdk_Pixmap;
      Xdest, Ydest : Glib.Gint)
   is

      Green_Gc : Gdk_GC;
   begin
      Set_Screen_Size
        (Item,
         To_Canvas_Coordinates (Canvas, Item.W),
         To_Canvas_Coordinates (Canvas, Item.H));
      Gdk_New (Green_Gc, Get_Window (Canvas));
      Set_Foreground
        (Green_Gc,
         Gdk.Color.Black (Gtk.Widget.Get_Default_Colormap));
      if Item.Selected then
         Set_Foreground (Green_Gc, Parse ("white"));
         Draw_Rectangle
           (Drawable => Dest,
            Gc       => Green_Gc,
            Filled   => True,
            X        => Xdest,
            Y        => Ydest,
            Width    => To_Canvas_Coordinates (Canvas, Item.W),
            Height   => To_Canvas_Coordinates (Canvas, Item.H));
      end if;
      Set_Foreground
        (Green_Gc,
         Gdk.Color.Black (Gtk.Widget.Get_Default_Colormap));
      Draw_Rectangle
        (Drawable => Dest,
         Gc       => Green_Gc,
         Filled   => False,
         X        => Xdest,
         Y        => Ydest,
         Width    => To_Canvas_Coordinates (Canvas, Item.W),
         Height   => To_Canvas_Coordinates (Canvas, Item.H));

      Draw_Line
        (Drawable => Dest,
         Gc       => Green_Gc,
         X1       => Xdest,
         Y1       => Ydest,
         X2       => Xdest + (To_Canvas_Coordinates (Canvas, Item.W) / 2),
         Y2       => Ydest + (To_Canvas_Coordinates (Canvas, Item.H) / 2));
      Draw_Line
        (Drawable => Dest,
         Gc       => Green_Gc,
         X1       => Xdest + To_Canvas_Coordinates (Canvas, Item.W),
         Y1       => Ydest,
         X2       => Xdest + (To_Canvas_Coordinates (Canvas, Item.W) / 2),
         Y2       => Ydest + (To_Canvas_Coordinates (Canvas, Item.H) / 2));

      if (Get_Name (Item) /= "") then
         Load
           (Font,
            "-adobe-courier-bold-o-normal--10-100-75-75-m-60-iso8859-1");
         Draw_Text
           (Dest,
            Font,
            Green_Gc,
            Xdest,
            Ydest - 5,
            To_String (Get_Name (Item)));

      end if;

   end Draw_Message;

   procedure Draw_Buffer
     (Item         : access Item_Object_Record;
      Canvas       : access Interactive_Canvas_Record'Class;
      Dest         : Gdk.Pixmap.Gdk_Pixmap;
      Xdest, Ydest : Glib.Gint)
   is

      Green_Gc : Gdk_GC;
   begin
      Set_Screen_Size
        (Item,
         To_Canvas_Coordinates (Canvas, Item.W),
         To_Canvas_Coordinates (Canvas, Item.H));
      Gdk_New (Green_Gc, Get_Window (Canvas));
      Set_Foreground
        (Green_Gc,
         Gdk.Color.Black (Gtk.Widget.Get_Default_Colormap));
      if Item.Selected then
         Set_Foreground (Green_Gc, Parse ("white"));
         Draw_Rectangle
           (Drawable => Dest,
            Gc       => Green_Gc,
            Filled   => True,
            X        => Xdest,
            Y        => Ydest,
            Width    => To_Canvas_Coordinates (Canvas, Item.W),
            Height   => To_Canvas_Coordinates (Canvas, Item.H));
      end if;
      Set_Foreground
        (Green_Gc,
         Gdk.Color.Black (Gtk.Widget.Get_Default_Colormap));
      Draw_Rectangle
        (Drawable => Dest,
         Gc       => Green_Gc,
         Filled   => False,
         X        => Xdest,
         Y        => Ydest,
         Width    => To_Canvas_Coordinates (Canvas, Item.W),
         Height   => To_Canvas_Coordinates (Canvas, Item.H));

      if (Get_Name (Item) /= "") then
         Load
           (Font,
            "-adobe-courier-bold-o-normal--10-100-75-75-m-60-iso8859-1");
         Draw_Text
           (Dest,
            Font,
            Green_Gc,
            Xdest,
            Ydest - 5,
            To_String (Get_Name (Item)));
      end if;
   end Draw_Buffer;

   procedure Add_Canvas_Link
     (Canvas       : access Interactive_Canvas_Record'Class;
      Item1, Item2 : access Canvas_Item_Record'Class;
      Text         : String := "")
   is
      Link : constant Canvas_Link := new Canvas_Link_Record;
   begin
      Configure (Link, Item1, Item2, End_Arrow, Text);
      Add_Link (Canvas, Link);
      Last_Link := Last_Link + 1;
   end Add_Canvas_Link;

   procedure Remove_Link_Between
     (Canvas       : access Interactive_Canvas_Record'Class;
      Item1, Item2 : access Item_Object_Record'Class)
   is
      function Remove_Internal
        (Canvas : access Interactive_Canvas_Record'Class;
         Link   : access Canvas_Link_Record'Class)
         return   Boolean
      is
      begin
         if (Get_Src (Link) = Canvas_Item (Item1)
            and then Get_Dest (Link) = Canvas_Item (Item2))
         then
            Remove_Link (Canvas, Link);
            return False;
         end if;
         return True;
      end Remove_Internal;
   begin
      For_Each_Link (Canvas, Remove_Internal'Unrestricted_Access);
      Refresh_Canvas (Canvas);
   end Remove_Link_Between;

   procedure Remove_Item
     (Canvas : access Interactive_Canvas_Record'Class;
      Item   : access Item_Object_Record'Class)
   is
      Mes_Iterator : Messages_Iterator;
      Tas_Iterator : Tasks_Iterator;
      Buf_Iterator : Buffers_Iterator;
      A_Message    : Message_Ptr;
      A_Task       : Generic_Task_Ptr;
      A_Buffer     : Buffer_Ptr;
   begin
      Remove (Canvas, Item);
      if Get_Figure (Item) = "message" then
         loop
            Current_Element (Sys.Messages, A_Message, Mes_Iterator);

            if A_Message.Name = Get_Name (Item) then
               Delete (Sys.Messages, A_Message);
               Is_Saved := False;
               exit;
            end if;

            exit when Is_Last_Element (Sys.Messages, Mes_Iterator);
            Next_Element (Sys.Messages, Mes_Iterator);

         end loop;
      end if;
      if Get_Figure (Item) = "task" then
         loop
            Current_Element (Sys.Tasks, A_Task, Tas_Iterator);

            if A_Task.Name = Get_Name (Item) then
               Delete (Sys.Tasks, A_Task);
               Is_Saved := False;
               exit;
            end if;

            exit when Is_Last_Element (Sys.Tasks, Tas_Iterator);
            Next_Element (Sys.Tasks, Tas_Iterator);

         end loop;
      end if;
      if Get_Figure (Item) = "buffer" then
         loop
            Current_Element (Sys.Buffers, A_Buffer, Buf_Iterator);

            if A_Buffer.Name = Get_Name (Item) then
               Delete (Sys.Buffers, A_Buffer);
               Is_Saved := False;
               exit;
            end if;

            exit when Is_Last_Element (Sys.Buffers, Buf_Iterator);
            Next_Element (Sys.Buffers, Buf_Iterator);

         end loop;
      end if;
   end Remove_Item;

   procedure Remove_Item_Link
     (Canvas : access Interactive_Canvas_Record'Class;
      Item   : access Item_Object_Record'Class)
   is
      function Iternal_Remove
        (Can  : access Interactive_Canvas_Record'Class;
         Link : access Canvas_Link_Record'Class)
         return Boolean
      is
      begin
         if (Get_Src (Link) = Canvas_Item (Item)
            or else Get_Dest (Link) = Canvas_Item (Item))
         then
            Remove_Link (Can, Link);
         end if;
         return True;
      end Iternal_Remove;
   begin
      For_Each_Link (Canvas, Iternal_Remove'Unrestricted_Access);
   end Remove_Item_Link;

end Graph_Item_Pkg;
