------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Gtk.GEntry;            use Gtk.GEntry;
with Translate;             use Translate;
with Framework_Config;      use Framework_Config;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Xpm;                   use Xpm;

package body Graph_Toolbar_Pkg is

   procedure New_Tool_Bar
     (Tool : in out Graph_Toolbar_Access;
      App  : access Gtk_Window_Record'Class)
   is

   begin
      Tool := new Graph_Toolbar;

      -- Create the Action bar, add it the the box
      --
      Gtk_New (Tool.Action_Toolbar, Orientation_Horizontal, Toolbar_Both);

      Append_Space (Tool.Action_Toolbar);

      String_List.Append
        (Tool.Items,
         To_String (Lb_Select (Current_Language)));
      String_List.Append
        (Tool.Items,
         To_String (Lb_Create (Current_Language)));
      String_List.Append
        (Tool.Items,
         To_String (Lb_Delete (Current_Language)));

      Gtk_New (Tool.Combo);
      Set_Popdown_Strings (Tool.Combo, Tool.Items);
      Set_Editable (Get_Entry (Tool.Combo), False);
      Add (Tool.Action_Toolbar, Tool.Combo);

      Append_Space (Tool.Action_Toolbar);

      -- Create and add to the bar the Zoom in button
      --
      Tool.Pixmap_Close := Create_Pixmap (Exit_Xpm, App);
      Set_Alignment (Tool.Pixmap_Close, 0.5, 0.5);
      Set_Padding (Tool.Pixmap_Close, 0, 0);

      Tool.Pixmap_Zoom_In := Create_Pixmap (Zoom_In_Xpm, App);
      Set_Alignment (Tool.Pixmap_Zoom_In, 0.5, 0.5);
      Set_Padding (Tool.Pixmap_Zoom_In, 0, 0);

      Tool.Pixmap_Zoom_Out := Create_Pixmap (Zoom_Out_Xpm, App);
      Set_Alignment (Tool.Pixmap_Zoom_Out, 0.5, 0.5);
      Set_Padding (Tool.Pixmap_Zoom_Out, 0, 0);

      Tool.Pixmap_Help := Create_Pixmap (Help_Xpm, App);
      Set_Alignment (Tool.Pixmap_Help, 0.5, 0.5);
      Set_Padding (Tool.Pixmap_Help, 0, 0);

      Tool.Pixmap_Disp1 := Create_Pixmap (Disp1_Xpm, App);
      Set_Alignment (Tool.Pixmap_Disp1, 0.5, 0.5);
      Set_Padding (Tool.Pixmap_Disp1, 0, 0);

      Tool.Pixmap_Disp2 := Create_Pixmap (Disp2_Xpm, App);
      Set_Alignment (Tool.Pixmap_Disp2, 0.5, 0.5);
      Set_Padding (Tool.Pixmap_Disp2, 0, 0);

      Tool.Pixmap_Disp3 := Create_Pixmap (Disp3_Xpm, App);
      Set_Alignment (Tool.Pixmap_Disp3, 0.5, 0.5);
      Set_Padding (Tool.Pixmap_Disp3, 0, 0);

      Append_Space (Tool.Action_Toolbar);

      -- Create the disposition bar, add it the the box
      --
      Gtk_New (Tool.Disp_Toolbar, Orientation_Horizontal, Toolbar_Both);

      Gtk_New
        (Tool.Task_Radio_Button,
         Label => To_String (Lb_Task (Current_Language)));
      Gtk_New
        (Tool.Mesg_Radio_Button,
         Group => Tool.Task_Radio_Button,
         Label => To_String (Lb_Message (Current_Language)));
      Gtk_New
        (Tool.Buffer_Radio_Button,
         Group => Tool.Task_Radio_Button,
         Label => To_String (Lb_Buffer (Current_Language)));
      Gtk_New
        (Tool.Arrow_Radio_Button,
         Group => Tool.Task_Radio_Button,
         Label => To_String (Lb_Arrow (Current_Language)));
      Add (Tool.Disp_Toolbar, Tool.Arrow_Radio_Button);
      Add (Tool.Disp_Toolbar, Tool.Task_Radio_Button);
      Add (Tool.Disp_Toolbar, Tool.Mesg_Radio_Button);
      Add (Tool.Disp_Toolbar, Tool.Buffer_Radio_Button);

      Append_Space (Tool.Disp_Toolbar);

      Set_Button_Relief (Tool.Action_Toolbar, Relief_None);
      Set_Space_Size (Tool.Action_Toolbar, 5);

      Set_Space_Style (Tool.Disp_Toolbar, Toolbar_Space_Line);
      Set_Button_Relief (Tool.Disp_Toolbar, Relief_None);
      Set_Space_Size (Tool.Disp_Toolbar, 30);

   end New_Tool_Bar;

end Graph_Toolbar_Pkg;
