------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Buffers;    use Buffers;
use Buffers.Buffer_Roles;
with Buffer_Set; use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;

with Project;   use Project;
with Mmi_Tools; use Mmi_Tools;

with Translate; use Translate;

with Framework_Config; use Framework_Config;

with Unbounded_Strings; use Unbounded_Strings;

package body New_Buffer_Role_Pkg.Callbacks is

   procedure On_Ok_Pressed (Object : access Gtk_Button_Record'Class) is

      Ok          : Boolean;
      Role        : Unbounded_String;
      Task_Name   : Unbounded_String;
      Size_String : Unbounded_String;
      Size        : Integer;
      My_Iterator : Buffers_Iterator;
      A_Buffer    : Buffer_Ptr;

   begin
      Task_Name   :=
         To_Unbounded_String (Get_Text (New_Buffer_Role.Task_Name_Entry));
      Role        :=
         To_Unbounded_String (Get_Text (New_Buffer_Role.Role_Entry));
      Size_String :=
         To_Unbounded_String (Get_Text (New_Buffer_Role.Size_Entry));

      if (Size_String = To_Unbounded_String ("")) then
         Size := 0;
      else
         To_Integer (Size_String, Size, Ok);
         if not Ok then
            Show_Message_Box
              (Lb_Size (Current_Language) &
               Lb_Must_Be_Numeric (Current_Language));
            return;
         end if;
      end if;

      if (Size <= 0) then
         Show_Message_Box
           (Lb_Size (Current_Language) &
            Lb_Must_Be (Current_Language) &
            To_Unbounded_String (" > 0"));
         return;
      end if;

      Reset_Iterator (Sys.Buffers, My_Iterator);
      loop
         Current_Element (Sys.Buffers, A_Buffer, My_Iterator);
         if (New_Buffer_Role.Buffer_Name = A_Buffer.Name) then

            A_Buffer.Roles.Entries (A_Buffer.Roles.Nb_Entries).Data.Size :=
              Natural (Size);
            A_Buffer.Roles.Entries (A_Buffer.Roles.Nb_Entries).Item      :=
              Task_Name;
            if (Role = "Producer") then
               A_Buffer.Roles.Entries (A_Buffer.Roles.Nb_Entries).Data.
                 The_Role := Producer;
            else
               A_Buffer.Roles.Entries (A_Buffer.Roles.Nb_Entries).Data.
                 The_Role := Consumer;
            end if;
            A_Buffer.Roles.Nb_Entries := A_Buffer.Roles.Nb_Entries + 1;

         end if;
         exit when Is_Last_Element (Sys.Buffers, My_Iterator);
         Next_Element (Sys.Buffers, My_Iterator);
      end loop;

      Hide (New_Buffer_Role);
   end On_Ok_Pressed;

end New_Buffer_Role_Pkg.Callbacks;
