------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Framework_Config; use Framework_Config;

with Translate; use Translate;

with Gtk;       use Gtk;
with Gtk.Enums; use Gtk.Enums;

with Callbacks_Cheddar;             use Callbacks_Cheddar;
with New_Buffer_Role_Pkg.Callbacks; use New_Buffer_Role_Pkg.Callbacks;

with Unbounded_Strings; use Unbounded_Strings;

package body New_Buffer_Role_Pkg is

   procedure Gtk_New
     (New_Buffer_Role : out New_Buffer_Role_Access;
      Task_Name       : in Unbounded_String;
      Buffer_Name     : in Unbounded_String;
      Role            : in String)
   is
   begin
      New_Buffer_Role := new New_Buffer_Role_Record;
      New_Buffer_Role_Pkg.Initialize
        (New_Buffer_Role,
         Task_Name,
         Buffer_Name,
         Role);
   end Gtk_New;

   procedure Initialize
     (New_Buffer_Role : access New_Buffer_Role_Record'Class;
      Task_Name       : in Unbounded_String;
      Buffer_Name     : in Unbounded_String;
      Role            : in String)
   is
   begin

      New_Buffer_Role.Buffer_Name := Buffer_Name;

      Gtk.Window.Initialize (New_Buffer_Role, Window_Dialog);
      Set_Title
        (New_Buffer_Role,
         To_String (Lb_New_Buffer_Role (Current_Language)));
      Set_Policy (New_Buffer_Role, False, True, False);
      Set_Position (New_Buffer_Role, Win_Pos_Center_Always);
      Set_Modal (New_Buffer_Role, True);

      Gtk_New_Vbox (New_Buffer_Role.Vbox, False, 0);
      Add (New_Buffer_Role, New_Buffer_Role.Vbox);

      Gtk_New_Hbox (New_Buffer_Role.Hbox, False, 0);
      Set_Border_Width (New_Buffer_Role.Hbox, 16);
      Pack_Start (New_Buffer_Role.Vbox, New_Buffer_Role.Hbox, True, True, 0);
      Gtk_New
        (New_Buffer_Role.Task_Name_Label,
         To_String (Lb_Task_Name (Current_Language) & Lb_Colon));
      Set_Alignment (New_Buffer_Role.Task_Name_Label, 0.5, 0.5);
      Set_Padding (New_Buffer_Role.Task_Name_Label, 0, 0);
      Set_Justify (New_Buffer_Role.Task_Name_Label, Justify_Left);
      Set_Line_Wrap (New_Buffer_Role.Task_Name_Label, False);
      Pack_Start
        (New_Buffer_Role.Hbox,
         New_Buffer_Role.Task_Name_Label,
         False,
         False,
         0);
      Gtk_New (New_Buffer_Role.Task_Name_Entry);
      Set_Editable (New_Buffer_Role.Task_Name_Entry, False);
      Set_Max_Length (New_Buffer_Role.Task_Name_Entry, 0);
      Set_Text (New_Buffer_Role.Task_Name_Entry, To_String (Task_Name));
      Set_Visibility (New_Buffer_Role.Task_Name_Entry, True);
      Pack_Start
        (New_Buffer_Role.Hbox,
         New_Buffer_Role.Task_Name_Entry,
         True,
         True,
         0);

      Gtk_New_Hbox (New_Buffer_Role.Hbox, False, 0);
      Set_Border_Width (New_Buffer_Role.Hbox, 16);
      Pack_Start (New_Buffer_Role.Vbox, New_Buffer_Role.Hbox, True, True, 0);
      Gtk_New
        (New_Buffer_Role.Role_Label,
         To_String (Lb_Role (Current_Language) & Unbounded_Sp & Lb_Colon));
      Set_Alignment (New_Buffer_Role.Role_Label, 0.5, 0.5);
      Set_Padding (New_Buffer_Role.Role_Label, 0, 0);
      Set_Justify (New_Buffer_Role.Role_Label, Justify_Left);
      Set_Line_Wrap (New_Buffer_Role.Role_Label, False);
      Pack_Start
        (New_Buffer_Role.Hbox,
         New_Buffer_Role.Role_Label,
         False,
         False,
         0);
      Gtk_New (New_Buffer_Role.Role_Entry);
      Set_Editable (New_Buffer_Role.Role_Entry, False);
      Set_Max_Length (New_Buffer_Role.Role_Entry, 0);
      Set_Text (New_Buffer_Role.Role_Entry, Role);
      Set_Visibility (New_Buffer_Role.Role_Entry, True);
      Pack_Start
        (New_Buffer_Role.Hbox,
         New_Buffer_Role.Role_Entry,
         True,
         True,
         0);

      Gtk_New_Hbox (New_Buffer_Role.Hbox, False, 0);
      Set_Border_Width (New_Buffer_Role.Hbox, 16);
      Pack_Start (New_Buffer_Role.Vbox, New_Buffer_Role.Hbox, True, True, 0);
      Gtk_New
        (New_Buffer_Role.Size_Label,
         To_String (Lb_Size (Current_Language) & Unbounded_Sp & Lb_Colon));
      Set_Alignment (New_Buffer_Role.Size_Label, 0.5, 0.5);
      Set_Padding (New_Buffer_Role.Size_Label, 0, 0);
      Set_Justify (New_Buffer_Role.Size_Label, Justify_Left);
      Set_Line_Wrap (New_Buffer_Role.Size_Label, False);
      Pack_Start
        (New_Buffer_Role.Hbox,
         New_Buffer_Role.Size_Label,
         False,
         False,
         0);
      Gtk_New (New_Buffer_Role.Size_Entry);
      Set_Editable (New_Buffer_Role.Size_Entry, True);
      Set_Max_Length (New_Buffer_Role.Size_Entry, 0);
      Set_Text (New_Buffer_Role.Size_Entry, "");
      Set_Visibility (New_Buffer_Role.Size_Entry, True);
      Pack_Start
        (New_Buffer_Role.Hbox,
         New_Buffer_Role.Size_Entry,
         True,
         True,
         0);

      Gtk_New_Hseparator (New_Buffer_Role.Separator);
      Pack_Start
        (New_Buffer_Role.Vbox,
         New_Buffer_Role.Separator,
         True,
         True,
         0);

      Gtk_New_Hbox (New_Buffer_Role.Hbox, True, 0);
      Set_Border_Width (New_Buffer_Role.Hbox, 9);
      Pack_Start (New_Buffer_Role.Vbox, New_Buffer_Role.Hbox, True, True, 0);

      Gtk_New (New_Buffer_Role.Ok_Button, "Ok");
      Set_Usize (New_Buffer_Role.Ok_Button, 70, 25);
      Button_Callback.Connect
        (New_Buffer_Role.Ok_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'Access));
      Pack_Start
        (New_Buffer_Role.Hbox,
         New_Buffer_Role.Ok_Button,
         False,
         False,
         0);

   end Initialize;

end New_Buffer_Role_Pkg;
