------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Glib;    use Glib;
with Project; use Project;

package body Display_Buffers_Set_Pkg.Callbacks is

   procedure On_Ok_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      Destroy (Display_Buffers_Set);
   end On_Ok_Pressed;

   procedure Show_Roles (A_Buffer_Name : Unbounded_String) is

      A_Buffer    : Buffer_Ptr;
      My_Iterator : Iterator;
      Index       : Integer := 0;
      Line        : Gint;
      Role_Share  : Chars_Ptr_Array (1 .. 4);
      Role_Type   : Unbounded_String;

   begin
      Reset_Iterator (Sys.Buffers, My_Iterator);
      Clear (Display_Buffers_Set.Role_List);

      loop
         Current_Element (Sys.Buffers, A_Buffer, My_Iterator);
         if (Natural (A_Buffer.Roles.Nb_Entries) >= 1) then
            if A_Buffer.Name = A_Buffer_Name then
               for I in  0 .. Buffer_Roles_Range (Natural (
                    A_Buffer.Roles.Nb_Entries) -
                                                  1)
               loop
                  if (A_Buffer.Roles.Entries (I).Data.The_Role =
                      Producer)
                  then
                     Role_Type := To_Unbounded_String ("Producer");
                  else
                     Role_Type := To_Unbounded_String ("Consumer");
                  end if;
                  Role_Share                             :=
                    To_String (A_Buffer.Roles.Entries (I).Item) +
                    To_String
                       (To_Unbounded_String
                           (Natural'Image
                               (A_Buffer.Roles.Entries (I).Data.Size))) +
                    To_String (Role_Type) +
                    To_String
                       (To_Unbounded_String
                           (Natural'Image
                               (A_Buffer.Roles.Entries (I).Data.Time)));
                  Display_Buffers_Set.Current_Role_Index :=
                    Indexed_Table_Range (Index);
                  Index                                  := Index + 1;
                  Line                                   :=
                     Append (Display_Buffers_Set.Role_List, Role_Share);
               end loop;
            end if;
         end if;
         exit when Is_Last_Element (Sys.Buffers, My_Iterator);
         Next_Element (Sys.Buffers, My_Iterator);
      end loop;
   end Show_Roles;

   procedure On_Click_List (List : access Gtk_Clist_Record'Class) is

      Buffer_Name : Unbounded_String;
      Row         : Gint;

   begin
      Row         := Get_Focus_Row (Display_Buffers_Set.Buffer_List);
      Buffer_Name :=
         To_Unbounded_String
           (Get_Text (Display_Buffers_Set.Buffer_List, Row, 0));

      Show_Roles (Buffer_Name);
   end On_Click_List;

end Display_Buffers_Set_Pkg.Callbacks;
