------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Glib;                              use Glib;
with Translate;                         use Translate;
with Framework_Config;                  use Framework_Config;
with Mmi_Tools;                         use Mmi_Tools;
with Project;                           use Project;
with Unbounded_Strings;                 use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;
with Ada.Strings.Unbounded;             use Ada.Strings.Unbounded;
with Gtk.Enums;                         use Gtk.Enums;
with Callbacks_Cheddar;                 use Callbacks_Cheddar;
with Display_Buffers_Set_Pkg.Callbacks; use Display_Buffers_Set_Pkg.Callbacks;

package body Display_Buffers_Set_Pkg is
   procedure Gtk_New (Display_Buffers_Set : out Display_Buffers_Set_Access) is

   begin
      Display_Buffers_Set := new Display_Buffers_Set_Record;
      Display_Buffers_Set_Pkg.Initialize (Display_Buffers_Set);
   end Gtk_New;

   procedure Initialize
     (Display_Buffers_Set : access Display_Buffers_Set_Record'Class)
   is
      A_Buffer    : Buffer_Ptr;
      My_Iterator : Iterator;
      Line        : Gint;

      Buffer_Share : Chars_Ptr_Array (1 .. 3);
   begin
      Gtk.Window.Initialize (Display_Buffers_Set, Window_Toplevel);
      Set_Policy (Display_Buffers_Set, True, True, True);
      Set_Title
        (Display_Buffers_Set,
         To_String (Lb_Buffers_List (Current_Language)));
      Set_Position (Display_Buffers_Set, Win_Pos_Center_Always);
      Set_Modal (Display_Buffers_Set, True);
      Grab_Focus (Display_Buffers_Set);
      Set_Usize (Display_Buffers_Set, 400, 600);
      Gtk.Box.Gtk_New_Vbox (Display_Buffers_Set.Vbox_Main, False, 10);
      Add (Display_Buffers_Set, Display_Buffers_Set.Vbox_Main);

      Gtk.Scrolled_Window.Gtk_New
        (Display_Buffers_Set.Scroll_Buffer_List,
         null,
         null);
      Display_Buffers_Set.Buffer_List_Titles :=
        To_String (Lb_Buffer_Name (Current_Language)) +
        To_String (Lb_Processor_Name (Current_Language)) +
        To_String (Lb_Size ((Current_Language)));
      Gtk.Clist.Gtk_New
        (Display_Buffers_Set.Buffer_List,
         3,
         Display_Buffers_Set.Buffer_List_Titles);
      Set_Column_Width (Display_Buffers_Set.Buffer_List, 0, 90);

      Set_Usize (Display_Buffers_Set.Buffer_List, 200, 20);
      Add
        (Display_Buffers_Set.Scroll_Buffer_List,
         Display_Buffers_Set.Buffer_List);

      Gtk.Frame.Gtk_New
        (Display_Buffers_Set.Frame_Buffer,
         To_String (Lb_Buffer_List (Current_Language)));
      Add
        (Display_Buffers_Set.Frame_Buffer,
         Display_Buffers_Set.Scroll_Buffer_List);
      Add (Display_Buffers_Set.Vbox_Main, Display_Buffers_Set.Frame_Buffer);

      Gtk.Scrolled_Window.Gtk_New
        (Display_Buffers_Set.Scroll_Role_List,
         null,
         null);
      Display_Buffers_Set.Role_List_Titles :=
        To_String (Lb_Task (Current_Language)) +
        To_String (Lb_Size (Current_Language)) +
        To_String (Lb_Role_Type (Current_Language)) +
        To_String (Lb_Time (Current_Language));
      Gtk.Clist.Gtk_New
        (Display_Buffers_Set.Role_List,
         4,
         Display_Buffers_Set.Role_List_Titles);
      Set_Column_Width (Display_Buffers_Set.Role_List, 0, 90);
      Set_Column_Width (Display_Buffers_Set.Role_List, 1, 90);
      Set_Column_Width (Display_Buffers_Set.Role_List, 2, 90);
      Set_Column_Width (Display_Buffers_Set.Role_List, 3, 90);

      Set_Usize (Display_Buffers_Set.Role_List, 200, 20);
      Add
        (Display_Buffers_Set.Scroll_Role_List,
         Display_Buffers_Set.Role_List);

      Gtk.Frame.Gtk_New
        (Display_Buffers_Set.Frame_Role,
         To_String (Lb_Role_List (Current_Language)));
      Add
        (Display_Buffers_Set.Frame_Role,
         Display_Buffers_Set.Scroll_Role_List);
      Add (Display_Buffers_Set.Vbox_Main, Display_Buffers_Set.Frame_Role);

      Gtk.Button.Gtk_New
        (Display_Buffers_Set.Button_Ok,
         To_String (Lb_Close (Current_Language)));
      Add (Display_Buffers_Set.Vbox_Main, Display_Buffers_Set.Button_Ok);
      Set_Usize (Display_Buffers_Set.Button_Ok, 60, 5);

      Reset_Iterator (Sys.Buffers, My_Iterator);
      loop
         Current_Element (Sys.Buffers, A_Buffer, My_Iterator);
         Buffer_Share := To_String (A_Buffer.Name) +
                         To_String (A_Buffer.Cpu_Name) +
                         Natural'Image (A_Buffer.Size);

         Line := Append (Display_Buffers_Set.Buffer_List, Buffer_Share);

         exit when Is_Last_Element (Sys.Buffers, My_Iterator);
         Next_Element (Sys.Buffers, My_Iterator);

      end loop;

      Button_Callback.Connect
        (Display_Buffers_Set.Button_Ok,
         "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'Access));
      List_Callback.Connect
        (Display_Buffers_Set.Buffer_List,
         "select_row",
         List_Callback.To_Marshaller (On_Click_List'Access));

      Show_All (Display_Buffers_Set);
   exception
      when Buffer_Set.Generic_Buffer_Set.Empty_Set =>
         begin
            Show_Message_Box (Lb_Define_Buffers_Before (Current_Language));
         end;
   end Initialize;
end Display_Buffers_Set_Pkg;
