------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Glib;                                      use Glib;
with Gtk;                                       use Gtk;
with Gdk.Types;                                 use Gdk.Types;
with Gtk.Widget;                                use Gtk.Widget;
with Gtk.Enums;                                 use Gtk.Enums;
with Callbacks_Cheddar;                         use Callbacks_Cheddar;
with Display_Event_Analyzers_Set_Pkg.Callbacks;
use Display_Event_Analyzers_Set_Pkg.Callbacks;
with Project;                                   use Project;
with Event_Analyzers;                           use Event_Analyzers;
with Event_Analyzer_Set;                        use Event_Analyzer_Set;
use Event_Analyzer_Set.Generic_Event_Analyzer_Set;
with Ada.Strings.Unbounded;                     use Ada.Strings.Unbounded;
with Framework_Config;                          use Framework_Config;
with Translate;                                 use Translate;

package body Display_Event_Analyzers_Set_Pkg is

   procedure Gtk_New
     (Display_Event_Analyzers_Set : out Display_Event_Analyzers_Set_Access)
   is
   begin
      Display_Event_Analyzers_Set := new Display_Event_Analyzers_Set_Record;
      Display_Event_Analyzers_Set_Pkg.Initialize
        (Display_Event_Analyzers_Set);
   end Gtk_New;

   procedure Initialize
     (Display_Event_Analyzers_Set : access Display_Event_Analyzers_Set_Record'
        Class)
   is

      An_Event_Analyzer : Event_Analyzer_Ptr;
      My_Iterator       : Iterator;
      Ligne             : Guint := 1;
      Heigth            : Gint;

   begin
      Gtk.Window.Initialize (Display_Event_Analyzers_Set, Window_Toplevel);
      Set_Title
        (Display_Event_Analyzers_Set,
         To_String (Lb_Event_Analyzers_List (Current_Language)));
      Set_Policy (Display_Event_Analyzers_Set, True, True, True);
      Heigth := Gint (3 + Get_Number_Of_Elements (Sys.Event_Analyzers)) *
                50;
      if Heigth > 400 then
         Heigth := 400;
      end if;
      Set_Default_Size (Display_Event_Analyzers_Set, 500, Heigth);
      Set_Position (Display_Event_Analyzers_Set, Win_Pos_Center_Always);
      Set_Modal (Display_Event_Analyzers_Set, True);
      Set_Extension_Events
        (Display_Event_Analyzers_Set,
         Extension_Events_All);
      Set_Flags (Display_Event_Analyzers_Set, Can_Default);

      Grab_Focus (Display_Event_Analyzers_Set);

      Gtk_New (Display_Event_Analyzers_Set.Scroll);
      Set_Border_Width
        (Display_Event_Analyzers_Set.Scroll,
         Border_Width => 10);
      Set_Policy
        (Display_Event_Analyzers_Set.Scroll,
         H_Scrollbar_Policy => Policy_Automatic,
         V_Scrollbar_Policy => Policy_Automatic);
      Add (Display_Event_Analyzers_Set, Display_Event_Analyzers_Set.Scroll);

      Gtk.Box.Gtk_New_Vbox (Display_Event_Analyzers_Set.Vbox, False, 10);
      Add_With_Viewport
        (Display_Event_Analyzers_Set.Scroll,
         Display_Event_Analyzers_Set.Vbox);
      Set_Focus_Hadjustment
        (Display_Event_Analyzers_Set.Vbox,
         Get_Hadjustment (Display_Event_Analyzers_Set.Scroll));
      Set_Focus_Vadjustment
        (Display_Event_Analyzers_Set.Vbox,
         Get_Vadjustment (Display_Event_Analyzers_Set.Scroll));

      Gtk_New
        (Display_Event_Analyzers_Set.Lab,
         To_String (Lb_Event_Analyzer_Name (Current_Language)));
      Set_Alignment (Display_Event_Analyzers_Set.Lab, 0.5, 0.5);
      Set_Padding (Display_Event_Analyzers_Set.Lab, 0, 0);
      Set_Justify (Display_Event_Analyzers_Set.Lab, Justify_Center);
      Set_Line_Wrap (Display_Event_Analyzers_Set.Lab, False);
      Add (Display_Event_Analyzers_Set.Vbox, Display_Event_Analyzers_Set.Lab);

      Reset_Iterator (Sys.Event_Analyzers, My_Iterator);

      loop
         Current_Element
           (Sys.Event_Analyzers,
            An_Event_Analyzer,
            My_Iterator);

         Gtk_New (Display_Event_Analyzers_Set.Entry_Name);
         Set_Editable (Display_Event_Analyzers_Set.Entry_Name, False);
         Set_Max_Length (Display_Event_Analyzers_Set.Entry_Name, 0);
         Set_Text
           (Display_Event_Analyzers_Set.Entry_Name,
            To_String (An_Event_Analyzer.Behavior_File_Name));
         Set_Visibility (Display_Event_Analyzers_Set.Entry_Name, True);
         Add
           (Display_Event_Analyzers_Set.Vbox,
            Display_Event_Analyzers_Set.Entry_Name);

         Ligne := Ligne + 1;
         exit when Is_Last_Element (Sys.Event_Analyzers, My_Iterator);

         Next_Element (Sys.Event_Analyzers, My_Iterator);

      end loop;

      Gtk_New_Hseparator (Display_Event_Analyzers_Set.Separator);
      Add
        (Display_Event_Analyzers_Set.Vbox,
         Display_Event_Analyzers_Set.Separator);

      Gtk_New
        (Display_Event_Analyzers_Set.Close_Button,
         To_String (Lb_Close (Current_Language)));
      Set_Usize (Display_Event_Analyzers_Set.Close_Button, 70, 25);
      Add
        (Display_Event_Analyzers_Set.Vbox,
         Display_Event_Analyzers_Set.Close_Button);
      Button_Callback.Connect
        (Display_Event_Analyzers_Set.Close_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Close_Pressed'Access));

   end Initialize;

end Display_Event_Analyzers_Set_Pkg;
