------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Strings.Unbounded;              use Ada.Strings.Unbounded;
with Translate;                          use Translate;
with Framework_Config;                   use Framework_Config;
with Gtk;                                use Gtk;
with Gtk.Enums;                          use Gtk.Enums;
with Callbacks_Cheddar;                  use Callbacks_Cheddar;
with Messages;                           use Messages;
with Message_Set;                        use Message_Set;
use Message_Set.Generic_Message_Set;
with Display_Messages_Set_Pkg.Callbacks;
use Display_Messages_Set_Pkg.Callbacks;
with Project;                            use Project;
with Glib;                               use Glib;

package body Display_Messages_Set_Pkg is

   procedure Gtk_New
     (Display_Messages_Set : out Display_Messages_Set_Access)
   is
   begin

      Display_Messages_Set := new Display_Messages_Set_Record;
      Display_Messages_Set_Pkg.Initialize (Display_Messages_Set);

   end Gtk_New;

   procedure Initialize
     (Display_Messages_Set : access Display_Messages_Set_Record'Class)
   is

      A_Message          : Message_Ptr;
      My_Iterator        : Iterator;
      The_Name           : Unbounded_String;
      The_Jitter         : Natural;
      The_Deadline       : Natural;
      The_Period         : Natural;
      The_Size           : Natural;
      The_Delay          : Natural;
      A_Line             : Chars_Ptr_Array (1 .. 6);
      Line               : Gint;
      Tmp_Jitter         : Unbounded_String;
      Tmp_Deadline       : Unbounded_String;
      Tmp_Period         : Unbounded_String;
      Tmp_Size           : Unbounded_String;
      Tmp_Delay          : Unbounded_String;
      A_Periodic_Message : Periodic_Message_Ptr;

   begin

      Gtk.Window.Initialize (Display_Messages_Set, Window_Toplevel);
      Set_Policy (Display_Messages_Set, True, True, True);
      Set_Title
        (Display_Messages_Set,
         To_String (Lb_Messages_List (Current_Language)));
      Set_Position (Display_Messages_Set, Win_Pos_Center_Always);
      Set_Modal (Display_Messages_Set, True);
      Grab_Focus (Display_Messages_Set);
      Gtk.Box.Gtk_New_Vbox (Display_Messages_Set.Vbox_Main, False, 10);
      Gtk.Box.Gtk_New_Hbox (Display_Messages_Set.Hbox_Up, False, 10);
      Gtk.Box.Gtk_New_Hbox (Display_Messages_Set.Hbox_Down, False, 10);

      Add (Display_Messages_Set, Display_Messages_Set.Vbox_Main);
      Add (Display_Messages_Set.Vbox_Main, Display_Messages_Set.Hbox_Up);
      Add (Display_Messages_Set.Vbox_Main, Display_Messages_Set.Hbox_Down);

      Gtk.Scrolled_Window.Gtk_New
        (Display_Messages_Set.Scroll_List,
         null,
         null);

      Add (Display_Messages_Set.Hbox_Up, Display_Messages_Set.Scroll_List);

      Display_Messages_Set.List_Titles :=
        To_String (Lb_Name (Current_Language)) +
        To_String (Lb_Jitter (Current_Language)) +
        To_String (Lb_Deadline (Current_Language)) +
        To_String (Lb_Period (Current_Language)) +
        To_String (Lb_Size (Current_Language)) +
        To_String (Lb_Response_Time_Message (Current_Language));

      Gtk.Clist.Gtk_New
        (Display_Messages_Set.List,
         6,
         Display_Messages_Set.List_Titles);
      Set_Usize (Display_Messages_Set.List, 300, 200);
      Add (Display_Messages_Set.Scroll_List, Display_Messages_Set.List);

      Gtk.Button.Gtk_New
        (Display_Messages_Set.Button_Close,
         To_String (Lb_Close (Current_Language)));

      Button_Callback.Connect
        (Display_Messages_Set.Button_Close,
         "clicked",
         Button_Callback.To_Marshaller (On_Close_Pressed'Access));

      Add (Display_Messages_Set.Hbox_Down, Display_Messages_Set.Button_Close);

      -- Put a Line in the Clist
      --
      Reset_Iterator (Sys.Messages, My_Iterator);
      loop

         Current_Element (Sys.Messages, A_Message, My_Iterator);

         The_Name     := A_Message.Name;
         The_Deadline := A_Message.Deadline;
         The_Size     := A_Message.Size;
         The_Delay    := A_Message.Response_Time;

         if (A_Message.Message_Type = Periodic_Type) then
            A_Periodic_Message := Periodic_Message_Ptr (A_Message);
            The_Jitter         := A_Periodic_Message.Jitter;
            The_Period         := A_Periodic_Message.Period;
         else
            The_Jitter := 0;
            The_Period := 0;
         end if;
         Tmp_Jitter   := To_Unbounded_String (Natural'Image (The_Jitter));
         Tmp_Deadline := To_Unbounded_String (Natural'Image (The_Deadline));
         Tmp_Period   := To_Unbounded_String (Natural'Image (The_Period));
         Tmp_Size     := To_Unbounded_String (Natural'Image (The_Size));
         Tmp_Delay    := To_Unbounded_String (Natural'Image (The_Delay));

         -- Create the line
         --
         A_Line := To_String (The_Name) +
                   To_String (Tmp_Jitter) +
                   To_String (Tmp_Deadline) +
                   To_String (Tmp_Period) +
                   To_String (Tmp_Size) +
                   To_String (Tmp_Delay);

         -- Display the Line
         --
         Line := Append (Display_Messages_Set.List, A_Line);

         exit when Is_Last_Element (Sys.Messages, My_Iterator);

         Next_Element (Sys.Messages, My_Iterator);

      end loop;

   end Initialize;

end Display_Messages_Set_Pkg;
