------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Glib;                                 use Glib;
with Gtk;                                  use Gtk;
with Gdk.Types;                            use Gdk.Types;
with Gtk.Widget;                           use Gtk.Widget;
with Gtk.Enums;                            use Gtk.Enums;
with Callbacks_Cheddar;                    use Callbacks_Cheddar;
with Display_Processors_Set_Pkg.Callbacks;
use Display_Processors_Set_Pkg.Callbacks;
with Ada.Strings.Unbounded;                use Ada.Strings.Unbounded;
with Scheduler;                            use Scheduler;
with Processors;                           use Processors;
with Processor_Set;                        use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Project;                              use Project;
with Framework_Config;                     use Framework_Config;
with Translate;                            use Translate;

package body Display_Processors_Set_Pkg is

   procedure Gtk_New
     (Display_Processors_Set : out Display_Processors_Set_Access)
   is
   begin
      Display_Processors_Set := new Display_Processors_Set_Record;
      Display_Processors_Set_Pkg.Initialize (Display_Processors_Set);
   end Gtk_New;

   procedure Initialize
     (Display_Processors_Set : access Display_Processors_Set_Record'Class)
   is

      A_Processor  : Processor_Ptr;
      My_Scheduler : Scheduler_Ptr;
      My_Iterator  : Iterator;
      Ligne        : Guint := 1;

   begin

      Gtk.Window.Initialize (Display_Processors_Set, Window_Toplevel);
      Set_Title
        (Display_Processors_Set,
         To_String (Lb_Processors_List (Current_Language)));
      Set_Policy (Display_Processors_Set, True, True, True);
      Set_Position (Display_Processors_Set, Win_Pos_Center_Always);
      Set_Modal (Display_Processors_Set, True);
      Set_Extension_Events (Display_Processors_Set, Extension_Events_All);
      Set_Flags (Display_Processors_Set, Can_Default);
      Grab_Focus (Display_Processors_Set);

      Gtk_New
        (Display_Processors_Set.Table,
         Guint (3 + Get_Number_Of_Elements (Sys.Processors)),
         3,
         True);
      Set_Row_Spacings (Display_Processors_Set.Table, 8);
      Set_Col_Spacings (Display_Processors_Set.Table, 7);
      Add (Display_Processors_Set, Display_Processors_Set.Table);

      Gtk_New
        (Display_Processors_Set.Processor_Name,
         To_String (Lb_Processor_Name (Current_Language)));
      Set_Alignment (Display_Processors_Set.Processor_Name, 0.5, 0.5);
      Set_Padding (Display_Processors_Set.Processor_Name, 0, 0);
      Set_Justify (Display_Processors_Set.Processor_Name, Justify_Center);
      Set_Line_Wrap (Display_Processors_Set.Processor_Name, False);
      Attach
        (Display_Processors_Set.Table,
         Display_Processors_Set.Processor_Name,
         0,
         1,
         0,
         1,
         0,
         0,
         0,
         0);

      Gtk_New
        (Display_Processors_Set.Scheduler_Name,
         To_String (Lb_Scheduler (Current_Language)));
      Set_Alignment (Display_Processors_Set.Scheduler_Name, 0.5, 0.5);
      Set_Padding (Display_Processors_Set.Scheduler_Name, 0, 0);
      Set_Justify (Display_Processors_Set.Scheduler_Name, Justify_Center);
      Set_Line_Wrap (Display_Processors_Set.Scheduler_Name, False);
      Attach
        (Display_Processors_Set.Table,
         Display_Processors_Set.Scheduler_Name,
         1,
         2,
         0,
         1,
         0,
         0,
         0,
         0);

      Gtk_New (Display_Processors_Set.Quantum, "Quantum");
      Set_Alignment (Display_Processors_Set.Quantum, 0.5, 0.5);
      Set_Padding (Display_Processors_Set.Quantum, 0, 0);
      Set_Justify (Display_Processors_Set.Quantum, Justify_Center);
      Set_Line_Wrap (Display_Processors_Set.Quantum, False);
      Attach
        (Display_Processors_Set.Table,
         Display_Processors_Set.Quantum,
         2,
         3,
         0,
         1,
         0,
         0,
         0,
         0);

      Gtk_New
        (Display_Processors_Set.Scheduler_Option,
         To_String (Lb_Option (Current_Language)));
      Set_Alignment (Display_Processors_Set.Scheduler_Option, 0.5, 0.5);
      Set_Padding (Display_Processors_Set.Scheduler_Option, 0, 0);
      Set_Justify (Display_Processors_Set.Scheduler_Option, Justify_Center);
      Set_Line_Wrap (Display_Processors_Set.Scheduler_Option, False);
      Attach
        (Display_Processors_Set.Table,
         Display_Processors_Set.Scheduler_Option,
         3,
         4,
         0,
         1,
         0,
         0,
         0,
         0);

      Reset_Iterator (Sys.Processors, My_Iterator);

      loop
         Current_Element (Sys.Processors, A_Processor, My_Iterator);

         Gtk_New (Display_Processors_Set.Entry_Processor_Name);
         Set_Editable (Display_Processors_Set.Entry_Processor_Name, False);
         Set_Max_Length (Display_Processors_Set.Entry_Processor_Name, 0);
         Set_Text
           (Display_Processors_Set.Entry_Processor_Name,
            To_String ((A_Processor.all.Name)));
         Set_Visibility (Display_Processors_Set.Entry_Processor_Name, True);
         Attach
           (Display_Processors_Set.Table,
            Display_Processors_Set.Entry_Processor_Name,
            0,
            1,
            Ligne,
            Ligne + 1,
            Expand or Fill,
            0,
            0,
            0);

         My_Scheduler := A_Processor.all.Scheduler;

         Gtk_New (Display_Processors_Set.Entry_Scheduler_Name);
         Set_Editable (Display_Processors_Set.Entry_Scheduler_Name, False);
         Set_Max_Length (Display_Processors_Set.Entry_Scheduler_Name, 0);
         Set_Text
           (Display_Processors_Set.Entry_Scheduler_Name,
            To_String (Get_Name (My_Scheduler.all)));
         Set_Visibility (Display_Processors_Set.Entry_Scheduler_Name, True);
         Attach
           (Display_Processors_Set.Table,
            Display_Processors_Set.Entry_Scheduler_Name,
            1,
            2,
            Ligne,
            Ligne + 1,
            Expand or Fill,
            0,
            0,
            0);

         Gtk_New (Display_Processors_Set.Entry_Quantum);
         Set_Editable (Display_Processors_Set.Entry_Quantum, False);
         Set_Max_Length (Display_Processors_Set.Entry_Quantum, 0);
         Set_Text
           (Display_Processors_Set.Entry_Quantum,
            To_String (Get_Quantum (My_Scheduler.all)));
         Set_Visibility (Display_Processors_Set.Entry_Quantum, True);
         Attach
           (Display_Processors_Set.Table,
            Display_Processors_Set.Entry_Quantum,
            2,
            3,
            Ligne,
            Ligne + 1,
            Expand or Fill,
            0,
            0,
            0);

         Gtk_New (Display_Processors_Set.Entry_Scheduler_Option);
         Set_Editable (Display_Processors_Set.Entry_Scheduler_Option, False);
         Set_Max_Length (Display_Processors_Set.Entry_Scheduler_Option, 0);
         if (Get_Preemptive (My_Scheduler.all) = Preemptive) then
            Set_Text
              (Display_Processors_Set.Entry_Scheduler_Option,
               To_String (Lb_Preemptive (Current_Language)));
         else
            Set_Text
              (Display_Processors_Set.Entry_Scheduler_Option,
               To_String (Lb_Not_Preemptive (Current_Language)));
         end if;
         Set_Visibility (Display_Processors_Set.Entry_Scheduler_Option, True);
         Attach
           (Display_Processors_Set.Table,
            Display_Processors_Set.Entry_Scheduler_Option,
            3,
            4,
            Ligne,
            Ligne + 1,
            Expand or Fill,
            0,
            0,
            0);

         Ligne := Ligne + 1;

         exit when Is_Last_Element (Sys.Processors, My_Iterator);

         Next_Element (Sys.Processors, My_Iterator);

      end loop;

      Gtk_New_Hseparator (Display_Processors_Set.Separator);
      Attach
        (Display_Processors_Set.Table,
         Display_Processors_Set.Separator,
         0,
         4,
         Ligne,
         Ligne + 1,
         Fill,
         Expand or Fill,
         0,
         0);

      Gtk_New
        (Display_Processors_Set.Close_Button,
         To_String (Lb_Close (Current_Language)));
      Set_Usize (Display_Processors_Set.Close_Button, 70, 25);
      Attach
        (Display_Processors_Set.Table,
         Display_Processors_Set.Close_Button,
         0,
         4,
         Ligne + 1,
         Ligne + 2,
         0,
         0,
         0,
         0);

      Button_Callback.Connect
        (Display_Processors_Set.Close_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Close_Pressed'Access));

   end Initialize;

end Display_Processors_Set_Pkg;
