------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Strings.Unbounded;               use Ada.Strings.Unbounded;
with Translate;                           use Translate;
with Framework_Config;                    use Framework_Config;
with Gtk;                                 use Gtk;
with Gtk.Enums;                           use Gtk.Enums;
with Callbacks_Cheddar;                   use Callbacks_Cheddar;
with Resources;                           use Resources;
with Resource_Set;                        use Resource_Set;
use Resource_Set.Generic_Resource_Set;
with Display_Resources_Set_Pkg.Callbacks;
use Display_Resources_Set_Pkg.Callbacks;
with Project;                             use Project;
with Glib;                                use Glib;

package body Display_Resources_Set_Pkg is

   procedure Gtk_New
     (Display_Resources_Set : out Display_Resources_Set_Access)
   is
   begin

      Display_Resources_Set := new Display_Resources_Set_Record;
      Display_Resources_Set_Pkg.Initialize (Display_Resources_Set);

   end Gtk_New;

   procedure Initialize
     (Display_Resources_Set : access Display_Resources_Set_Record'Class)
   is

      A_Resource   : Generic_Resource_Ptr;
      My_Iterator  : Iterator;
      The_Cpu      : Unbounded_String;
      The_Name     : Unbounded_String;
      The_State    : Integer;
      The_Protocol : Unbounded_String;
      A_Line       : Chars_Ptr_Array (1 .. 4);
      Line         : Gint;

   begin

      Gtk.Window.Initialize (Display_Resources_Set, Window_Toplevel);

      Set_Policy (Display_Resources_Set, True, True, True);
      Set_Title
        (Display_Resources_Set,
         To_String (Lb_Resources_List (Current_Language)));
      Set_Position (Display_Resources_Set, Win_Pos_Center_Always);
      Set_Modal (Display_Resources_Set, True);
      Grab_Focus (Display_Resources_Set);
      Gtk.Box.Gtk_New_Vbox (Display_Resources_Set.Vbox_Main, False, 10);
      Gtk.Box.Gtk_New_Hbox (Display_Resources_Set.Hbox_Up, False, 10);
      Gtk.Box.Gtk_New_Hbox (Display_Resources_Set.Hbox_Down, False, 10);

      Add (Display_Resources_Set, Display_Resources_Set.Vbox_Main);
      Add (Display_Resources_Set.Vbox_Main, Display_Resources_Set.Hbox_Up);
      Add (Display_Resources_Set.Vbox_Main, Display_Resources_Set.Hbox_Down);

      Gtk.Scrolled_Window.Gtk_New
        (Display_Resources_Set.Scroll_List,
         null,
         null);

      Add (Display_Resources_Set.Hbox_Up, Display_Resources_Set.Scroll_List);

      Display_Resources_Set.List_Titles :=
        To_String (Lb_Resource_Name (Current_Language)) +
        To_String (Lb_State (Current_Language)) +
        To_String (Lb_Protocol (Current_Language)) +
        To_String (Lb_Processor_Name (Current_Language));
      Gtk.Clist.Gtk_New
        (Display_Resources_Set.List,
         4,
         Display_Resources_Set.List_Titles);
      Set_Column_Width (Display_Resources_Set.List, 0, 150);
      Set_Usize (Display_Resources_Set.List, 300, 200);
      Add (Display_Resources_Set.Scroll_List, Display_Resources_Set.List);

      Gtk.Button.Gtk_New
        (Display_Resources_Set.Button_Close,
         To_String (Lb_Close (Current_Language)));

      Button_Callback.Connect
        (Display_Resources_Set.Button_Close,
         "clicked",
         Button_Callback.To_Marshaller (On_Close_Pressed'Access));

      Add
        (Display_Resources_Set.Hbox_Down,
         Display_Resources_Set.Button_Close);

      -- Put a Line in the Clist
      --
      Reset_Iterator (Sys.Resources, My_Iterator);
      loop

         Current_Element (Sys.Resources, A_Resource, My_Iterator);

         The_Cpu   := A_Resource.cpu_name;
         The_Name  := A_Resource.Name;
         The_State := A_Resource.state;

         -- Name of the protocol
         --
         The_Protocol := To_Unbounded_String (A_Resource.protocol'Img);

         -- Create the line
         --
         A_Line := To_String (The_Name) +
                   To_String
                      (To_Unbounded_String (Integer'Image (The_State))) +
                   To_String (The_Protocol) +
                   To_String (The_Cpu);

         -- Display the Line
         --
         Line := Append (Display_Resources_Set.List, A_Line);

         exit when Is_Last_Element (Sys.Resources, My_Iterator);

         Next_Element (Sys.Resources, My_Iterator);

      end loop;

   end Initialize;

end Display_Resources_Set_Pkg;
