------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Glib;                            use Glib;
with Gtk;                             use Gtk;
with Gdk.Types;                       use Gdk.Types;
with Gtk.Widget;                      use Gtk.Widget;
with Gtk.Enums;                       use Gtk.Enums;
with Callbacks_Cheddar;               use Callbacks_Cheddar;
with Display_Tasks_Set_Pkg.Callbacks; use Display_Tasks_Set_Pkg.Callbacks;
with Project;                         use Project;
with Tasks;                           use Tasks;
with Task_Set;                        use Task_Set;
use Task_Set.Generic_Task_Set;
with Ada.Strings.Unbounded;           use Ada.Strings.Unbounded;
with Framework_Config;                use Framework_Config;
with Translate;                       use Translate;
with Offsets;                         use Offsets;
use Offsets.Offsets_Table_package;

package body Display_Tasks_Set_Pkg is

   procedure Gtk_New (Display_Tasks_Set : out Display_Tasks_Set_Access) is
   begin
      Display_Tasks_Set := new Display_Tasks_Set_Record;
      Display_Tasks_Set_Pkg.Initialize (Display_Tasks_Set);
   end Gtk_New;

   procedure Initialize
     (Display_Tasks_Set : access Display_Tasks_Set_Record'Class)
   is

      A_Task      : Generic_Task_Ptr;
      My_Iterator : Iterator;
      Ligne       : Guint := 1;
      Heigth      : Gint;

   begin
      Gtk.Window.Initialize (Display_Tasks_Set, Window_Toplevel);
      Set_Title
        (Display_Tasks_Set,
         To_String (Lb_Tasks_List (Current_Language)));
      Set_Policy (Display_Tasks_Set, True, True, True);
      Heigth := Gint (3 + Get_Number_Of_Elements (Sys.Tasks)) * 50;
      if Heigth > 400 then
         Heigth := 400;
      end if;
      Set_Default_Size (Display_Tasks_Set, 510, Heigth);
      Set_Position (Display_Tasks_Set, Win_Pos_Center_Always);
      Set_Modal (Display_Tasks_Set, True);
      Set_Extension_Events (Display_Tasks_Set, Extension_Events_All);
      Set_Flags (Display_Tasks_Set, Can_Default);

      Grab_Focus (Display_Tasks_Set);

      Gtk_New (Display_Tasks_Set.Scroll);
      Set_Border_Width (Display_Tasks_Set.Scroll, Border_Width => 10);
      Set_Policy
        (Display_Tasks_Set.Scroll,
         H_Scrollbar_Policy => Policy_Automatic,
         V_Scrollbar_Policy => Policy_Automatic);
      Add (Display_Tasks_Set, Display_Tasks_Set.Scroll);

      Gtk_New
        (Display_Tasks_Set.Table,
         Guint (3 + Get_Number_Of_Elements (Sys.Tasks)),
         2,
         True);
      Set_Row_Spacings (Display_Tasks_Set.Table, 8);
      Set_Col_Spacings (Display_Tasks_Set.Table, 7);

      Add_With_Viewport (Display_Tasks_Set.Scroll, Display_Tasks_Set.Table);
      Set_Focus_Hadjustment
        (Display_Tasks_Set.Table,
         Get_Hadjustment (Display_Tasks_Set.Scroll));
      Set_Focus_Vadjustment
        (Display_Tasks_Set.Table,
         Get_Vadjustment (Display_Tasks_Set.Scroll));

      Gtk_New
        (Display_Tasks_Set.Lab,
         To_String (Lb_Task_Name (Current_Language)));
      Set_Alignment (Display_Tasks_Set.Lab, 0.5, 0.5);
      Set_Padding (Display_Tasks_Set.Lab, 0, 0);
      Set_Justify (Display_Tasks_Set.Lab, Justify_Center);
      Set_Line_Wrap (Display_Tasks_Set.Lab, False);
      Attach
        (Display_Tasks_Set.Table,
         Display_Tasks_Set.Lab,
         0,
         1,
         0,
         1,
         0,
         0,
         0,
         0);

      Gtk_New
        (Display_Tasks_Set.Lab,
         To_String (Lb_Processor (Current_Language)));
      Set_Alignment (Display_Tasks_Set.Lab, 0.5, 0.5);
      Set_Padding (Display_Tasks_Set.Lab, 0, 0);
      Set_Justify (Display_Tasks_Set.Lab, Justify_Center);
      Set_Line_Wrap (Display_Tasks_Set.Lab, False);
      Attach
        (Display_Tasks_Set.Table,
         Display_Tasks_Set.Lab,
         1,
         2,
         0,
         1,
         0,
         0,
         0,
         0);

      Gtk_New
        (Display_Tasks_Set.Lab,
         To_String (Lb_Task_Type (Current_Language)));
      Set_Alignment (Display_Tasks_Set.Lab, 0.5, 0.5);
      Set_Padding (Display_Tasks_Set.Lab, 0, 0);
      Set_Justify (Display_Tasks_Set.Lab, Justify_Center);
      Set_Line_Wrap (Display_Tasks_Set.Lab, False);
      Attach
        (Display_Tasks_Set.Table,
         Display_Tasks_Set.Lab,
         2,
         3,
         0,
         1,
         0,
         0,
         0,
         0);

      Reset_Iterator (Sys.Tasks, My_Iterator);

      loop
         Current_Element (Sys.Tasks, A_Task, My_Iterator);

         Gtk_New (Display_Tasks_Set.Entry_Name);
         Set_Editable (Display_Tasks_Set.Entry_Name, False);
         Set_Max_Length (Display_Tasks_Set.Entry_Name, 0);
         Set_Text (Display_Tasks_Set.Entry_Name, To_String (A_Task.Name));
         Set_Visibility (Display_Tasks_Set.Entry_Name, True);
         Attach
           (Display_Tasks_Set.Table,
            Display_Tasks_Set.Entry_Name,
            0,
            1,
            Ligne,
            Ligne + 1,
            Expand or Fill,
            0,
            0,
            0);

         Gtk_New (Display_Tasks_Set.Entry_Processor);
         Set_Editable (Display_Tasks_Set.Entry_Processor, False);
         Set_Max_Length (Display_Tasks_Set.Entry_Processor, 0);
         Set_Text
           (Display_Tasks_Set.Entry_Processor,
            To_String (A_Task.Cpu_Name));
         Set_Visibility (Display_Tasks_Set.Entry_Processor, True);
         Attach
           (Display_Tasks_Set.Table,
            Display_Tasks_Set.Entry_Processor,
            1,
            2,
            Ligne,
            Ligne + 1,
            Expand or Fill,
            0,
            0,
            0);

         Gtk_New (Display_Tasks_Set.Entry_Type);
         Set_Editable (Display_Tasks_Set.Entry_Type, False);
         Set_Max_Length (Display_Tasks_Set.Entry_Type, 0);
         Set_Text (Display_Tasks_Set.Entry_Type, A_Task.Task_Type'Img);
         Set_Visibility (Display_Tasks_Set.Entry_Type, True);
         Attach
           (Display_Tasks_Set.Table,
            Display_Tasks_Set.Entry_Type,
            2,
            3,
            Ligne,
            Ligne + 1,
            Expand or Fill,
            0,
            0,
            0);

         Ligne := Ligne + 1;
         exit when Is_Last_Element (Sys.Tasks, My_Iterator);

         Next_Element (Sys.Tasks, My_Iterator);

      end loop;

      Gtk_New_Hseparator (Display_Tasks_Set.Separator);
      Attach
        (Display_Tasks_Set.Table,
         Display_Tasks_Set.Separator,
         0,
         3,
         Ligne,
         Ligne + 1,
         Fill,
         Expand or Fill,
         0,
         0);

      Ligne := Ligne + 1;
      Gtk_New
        (Display_Tasks_Set.Close_Button,
         To_String (Lb_Close (Current_Language)));
      Set_Usize (Display_Tasks_Set.Close_Button, 70, 25);
      Attach
        (Display_Tasks_Set.Table,
         Display_Tasks_Set.Close_Button,
         1,
         2,
         Ligne,
         Ligne + 1,
         0,
         0,
         0,
         0);
      Button_Callback.Connect
        (Display_Tasks_Set.Close_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Close_Pressed'Access));

   end Initialize;

end Display_Tasks_Set_Pkg;
