------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Framework_Config;      use Framework_Config;
with Mmi_Tools;             use Mmi_Tools;
with Translate;             use Translate;
with Project;               use Project;
with Unbounded_Strings;     use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;
with Processors;            use Processors;
with Glib;                  use Glib;
with Processor_Set;         use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Text_Tools;            use Text_Tools;
with Call_Framework;        use Call_Framework;
use Call_Framework.Framework_Request_Package;
use Call_Framework.Framework_Response_Package;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Gtk.Radio_Menu_Item;   use Gtk.Radio_Menu_Item;

package body Do_Scheduling_Feasibility_Pkg.Callbacks is

   procedure On_Ok_Pressed (Object : access Gtk_Button_Record'Class) is

      Test1       : Boolean := False;
      Test2       : Boolean := False;
      A_Processor : Processor_Ptr;
      Cpu_Index   : Gint;
      My_Iterator : Iterator;

      procedure Call_Feasibility (Processor_Name : in Unbounded_String) is

         Response_List : Framework_Response_Table;
         Request_List  : Framework_Request_Table;
         A_Request     : Framework_Request;

      begin
         Initialize (Response_List);
         Initialize (Request_List);

         if Test1 then
            Initialize (A_Request);
            A_Request.Target    := Processor_Name;
            A_Request.Statement := Scheduling_Feasibility_Cpu_Utilization;
            Add (Request_List, A_Request);
         end if;

         if Test2 then
            Initialize (A_Request);
            A_Request.Target    := Processor_Name;
            A_Request.Statement := Scheduling_Feasibility_Response_Time;
            Add (Request_List, A_Request);
         end if;

         Sequential_Framework_Request (Sys, Request_List, Response_List);

         Write_Title
           (Do_Scheduling_Feasibility.Text,
            Response_List.Entries (0).Title);
         for J in  0 .. Response_List.Nb_Entries - 1 loop
            Write_Text
              (Do_Scheduling_Feasibility.Text,
               Response_List.Entries (J).Text);
         end loop;

      end Call_Feasibility;

   begin

      if Get_Active (Do_Scheduling_Feasibility.Test1) then
         Test1 := True;
      end if;

      if Get_Active (Do_Scheduling_Feasibility.Test2) then
         Test2 := True;
      end if;

      if (Test1 = False) and (Test2 = False) then
         Show_Message_Box
           (Lb_Select_Almost_One_Feasilibity_Test (Current_Language));
         return;
      end if;

      Cpu_Index :=
         Gint'Val
           (Gint (Get_Number_Of_Elements (Sys.Processors)) -
            Gint (Selected_Button
                     (Do_Scheduling_Feasibility.Clist_Processor_Menu)));
      Cpu_Index := Cpu_Index - 1;

      Get_Element_Number
        (Sys.Processors,
         A_Processor,
         Processors_Range (Cpu_Index));

      if not Get_Active
               (Do_Scheduling_Feasibility.Schedule_All_Processeurs)
      then
         Call_Feasibility (A_Processor.Name);
      else
         Reset_Iterator (Sys.Processors, My_Iterator);

         loop
            Current_Element (Sys.Processors, A_Processor, My_Iterator);

            Call_Feasibility (A_Processor.Name);

            exit when Is_Last_Element (Sys.Processors, My_Iterator);

            Next_Element (Sys.Processors, My_Iterator);

         end loop;

      end if;

      Destroy (Do_Scheduling_Feasibility);

   end On_Ok_Pressed;

   procedure On_Cancel_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      Destroy (Do_Scheduling_Feasibility);
   end On_Cancel_Pressed;

end Do_Scheduling_Feasibility_Pkg.Callbacks;
