------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Gtk;                                     use Gtk;
with Gdk.Types;                               use Gdk.Types;
with Gtk.Enums;                               use Gtk.Enums;
with Do_Scheduling_Feasibility_Pkg.Callbacks;
use Do_Scheduling_Feasibility_Pkg.Callbacks;
with Project;                                 use Project;
with Processors;                              use Processors;
with Processor_Set;                           use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Mmi_Tools;                               use Mmi_Tools;
with Glib;                                    use Glib;
with Callbacks_Cheddar;                       use Callbacks_Cheddar;
with Framework_Config;                        use Framework_Config;
with Translate;                               use Translate;
with Ada.Strings.Unbounded;                   use Ada.Strings.Unbounded;
with Unbounded_Strings;                       use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;

package body Do_Scheduling_Feasibility_Pkg is

   procedure Gtk_New
     (Do_Scheduling_Feasibility : out Do_Scheduling_Feasibility_Access;
      Text                      : in Gtk_Text)
   is

   begin
      Do_Scheduling_Feasibility := new Do_Scheduling_Feasibility_Record;
      Do_Scheduling_Feasibility_Pkg.Initialize
        (Do_Scheduling_Feasibility,
         Text);
   end Gtk_New;

   procedure Initialize
     (Do_Scheduling_Feasibility : access Do_Scheduling_Feasibility_Record'
        Class;
      Text                      : in Gtk_Text)
   is

      Processor_Iterator : Processors_Iterator;
      Processor_Items    : Strings_Table;
      A_Processor        : Processor_Ptr;

   begin

      Do_Scheduling_Feasibility.Text := Text;

      Gtk.Window.Initialize (Do_Scheduling_Feasibility, Window_Toplevel);
      Set_Title
        (Do_Scheduling_Feasibility,
         To_String (Lb_Customized_Feasibility_Tests (Current_Language)));
      Set_Policy (Do_Scheduling_Feasibility, True, True, True);
      Set_Position (Do_Scheduling_Feasibility, Win_Pos_Center_Always);
      Set_Modal (Do_Scheduling_Feasibility, True);
      Set_Extension_Events (Do_Scheduling_Feasibility, Extension_Events_All);
      Set_Flags (Do_Scheduling_Feasibility, Can_Default);
      Grab_Focus (Do_Scheduling_Feasibility);

      Gtk_New_Vbox (Do_Scheduling_Feasibility.Do_Scheduling_Feasibility_Vbox);
      Add
        (Do_Scheduling_Feasibility,
         Do_Scheduling_Feasibility.Do_Scheduling_Feasibility_Vbox);

      Gtk_New (Do_Scheduling_Feasibility.Value_Table, 2, 2, True);
      Set_Border_Width (Do_Scheduling_Feasibility.Value_Table, 14);
      Set_Row_Spacings (Do_Scheduling_Feasibility.Value_Table, 8);
      Set_Col_Spacings (Do_Scheduling_Feasibility.Value_Table, 0);
      Pack_Start
        (Do_Scheduling_Feasibility.Do_Scheduling_Feasibility_Vbox,
         Do_Scheduling_Feasibility.Value_Table);

      ----------------------------------------------------------------
      -- List of processors on which simulations have to be run
      ----------------------------------------------------------------

      Initialize (Processor_Items);
      Reset_Iterator (Sys.Processors, Processor_Iterator);

      loop
         Current_Element (Sys.Processors, A_Processor, Processor_Iterator);
         Add (Processor_Items, A_Processor.Name);

         exit when Is_Last_Element (Sys.Processors, Processor_Iterator);

         Next_Element (Sys.Processors, Processor_Iterator);
      end loop;

      Do_Scheduling_Feasibility.Clist_Processor_Menu :=
        Widget_SList.Null_List;
      Build_Option_Menu
        (Do_Scheduling_Feasibility.Processor_Menu,
         Do_Scheduling_Feasibility.Clist_Processor_Menu,
         Processor_Items,
         -1,
         null);

      Attach
        (Do_Scheduling_Feasibility.Value_Table,
         Do_Scheduling_Feasibility.Processor_Menu,
         3,
         4,
         0,
         1);

      Gtk_New
        (Do_Scheduling_Feasibility.Lab,
         To_String (Lb_Processor (Current_Language) & Lb_Colon));

      Attach
        (Do_Scheduling_Feasibility.Value_Table,
         Do_Scheduling_Feasibility.Lab,
         2,
         3,
         0,
         1);

      Gtk_New
        (Do_Scheduling_Feasibility.Lab,
         To_String
            (Lb_Run_Feasibility_Tests_On_One_Processor_Only (Current_Language)
             &
             Lb_Colon));
      Attach
        (Do_Scheduling_Feasibility.Value_Table,
         Do_Scheduling_Feasibility.Lab,
         0,
         1,
         0,
         1);

      Gtk_New (Do_Scheduling_Feasibility.Schedule_All_Processeurs, "");
      Attach
        (Do_Scheduling_Feasibility.Value_Table,
         Do_Scheduling_Feasibility.Schedule_All_Processeurs,
         1,
         2,
         0,
         1);

      Gtk_New_Hseparator (Do_Scheduling_Feasibility.Hseparator);
      Attach
        (Do_Scheduling_Feasibility.Value_Table,
         Do_Scheduling_Feasibility.Hseparator,
         0,
         5,
         1,
         2);

      -----------------------------------------------
      -- List of feasibility tests to be selected
      -----------------------------------------------

      Gtk_New
        (Do_Scheduling_Feasibility.Lab,
         To_String (Lb_Feasibility_Number1 (Current_Language)));
      Attach
        (Do_Scheduling_Feasibility.Value_Table,
         Do_Scheduling_Feasibility.Lab,
         0,
         3,
         2,
         3);

      Gtk_New (Do_Scheduling_Feasibility.Test1, "");
      Attach
        (Do_Scheduling_Feasibility.Value_Table,
         Do_Scheduling_Feasibility.Test1,
         3,
         4,
         2,
         3);

      Gtk_New
        (Do_Scheduling_Feasibility.Lab,
         To_String (Lb_Feasibility_Number2 (Current_Language)));
      Attach
        (Do_Scheduling_Feasibility.Value_Table,
         Do_Scheduling_Feasibility.Lab,
         0,
         3,
         3,
         4);

      Gtk_New (Do_Scheduling_Feasibility.Test2, "");
      Attach
        (Do_Scheduling_Feasibility.Value_Table,
         Do_Scheduling_Feasibility.Test2,
         3,
         4,
         3,
         4);

      Gtk_New_Hseparator (Do_Scheduling_Feasibility.Hseparator);
      Pack_Start
        (Do_Scheduling_Feasibility.Do_Scheduling_Feasibility_Vbox,
         Do_Scheduling_Feasibility.Hseparator);

      -----------------------------------------------
      -- List of buttons
      -----------------------------------------------

      Gtk_New (Do_Scheduling_Feasibility.Button_Hbox);
      Set_Layout (Do_Scheduling_Feasibility.Button_Hbox, Buttonbox_Spread);
      Set_Spacing (Do_Scheduling_Feasibility.Button_Hbox, 10);
      Set_Child_Size (Do_Scheduling_Feasibility.Button_Hbox, 20, 20);
      Set_Child_Ipadding (Do_Scheduling_Feasibility.Button_Hbox, 7, 0);
      Pack_Start
        (Do_Scheduling_Feasibility.Do_Scheduling_Feasibility_Vbox,
         Do_Scheduling_Feasibility.Button_Hbox,
         False,
         True,
         5);

      Gtk_New (Do_Scheduling_Feasibility.Ok_Button, "Ok");
      Add
        (Do_Scheduling_Feasibility.Button_Hbox,
         Do_Scheduling_Feasibility.Ok_Button);
      Button_Callback.Connect
        (Do_Scheduling_Feasibility.Ok_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'Access));

      Gtk_New
        (Do_Scheduling_Feasibility.Cancel_Button,
         To_String (Lb_Cancel (Current_Language)));
      Add
        (Do_Scheduling_Feasibility.Button_Hbox,
         Do_Scheduling_Feasibility.Cancel_Button);
      Button_Callback.Connect
        (Do_Scheduling_Feasibility.Cancel_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'Access));

   end Initialize;

end Do_Scheduling_Feasibility_Pkg;
