------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Processors;            use Processors;
with Processor_Set;         use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Gtk.Radio_Menu_Item;   use Gtk.Radio_Menu_Item;
with Project;               use Project;
with Glib;                  use Glib;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Unbounded_Strings;     use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;
with read_sc_file_Pkg;      use read_sc_file_Pkg;

package body Do_Scheduling_Simulation_Pkg.Callbacks is

   procedure update_file_name (File_Name : in Unbounded_String) is

   begin
      Set_Text
        (Do_Scheduling_Simulation.Event_Table_File_Name,
         To_String (File_Name));
   end update_file_name;

   procedure On_Read_From_File_Pressed
     (Object : access Gtk_Button_Record'Class)
   is

   begin

      Gtk_New (read_sc_file, update_file_name'Access);
      Show_All (read_sc_file);

   end On_Read_From_File_Pressed;

   procedure On_Ok_Pressed (Object : access Gtk_Button_Record'Class) is

      A_Processor : Processor_Ptr;
      Cpu_Index   : Gint;

   begin

      -- Get customization provided by the user
      --
      Context_Switch_From_Simulation     :=
         Get_Active (Do_Scheduling_Simulation.Context_Switch);
      Preemption_From_Simulation         :=
         Get_Active (Do_Scheduling_Simulation.Number_Preemption);
      Run_Event_Analyzer_From_Simulation :=
         Get_Active (Do_Scheduling_Simulation.Event_Analyzer);
      Schedule_And_Display_Event_Table   :=
         Get_Active
           (Do_Scheduling_Simulation.Schedule_And_Display_Event_Table);
      Schedule_And_Export_Event_Table    :=
         Get_Active
           (Do_Scheduling_Simulation.Schedule_And_Export_Event_Table);
      Event_Table_File_Name              :=
         To_Unbounded_String
           (Get_Text (Do_Scheduling_Simulation.Event_Table_File_Name));
      Schedule_All_Processeurs           :=
         Get_Active (Do_Scheduling_Simulation.Schedule_All_Processeurs);
      Xml_Event_Table_File_Name          :=
         To_Unbounded_String
           (Get_Text (Do_Scheduling_Simulation.Event_Table_File_Name));

      for I in  Data_From_Simulation_Table'Range loop
         Blocking_Time_From_Simulation (I) :=
            Get_Active (Do_Scheduling_Simulation.Blocking_Time (I));
         Response_Time_From_Simulation (I) :=
            Get_Active (Do_Scheduling_Simulation.Response_Time (I));
      end loop;

      Cpu_Index :=
         Gint'Val
           (Gint (Get_Number_Of_Elements (Sys.Processors)) -
            Gint (Selected_Button
                     (Do_Scheduling_Simulation.Clist_Processor_Menu)));
      Cpu_Index := Cpu_Index - 1;

      Get_Element_Number
        (Sys.Processors,
         A_Processor,
         Processors_Range (Cpu_Index));
      Last_Selected_Processor := A_Processor.Name;

      Do_Scheduling_Simulation.Call.all;

      Hide (Do_Scheduling_Simulation);

   end On_Ok_Pressed;

   procedure On_Cancel_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      Hide (Do_Scheduling_Simulation);
   end On_Cancel_Pressed;

end Do_Scheduling_Simulation_Pkg.Callbacks;
