------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Glib;                                   use Glib;
with Gdk.Types;                              use Gdk.Types;
with Gtk.Enums;                              use Gtk.Enums;
with Callbacks_Cheddar;                      use Callbacks_Cheddar;
with Do_Scheduling_Simulation_Pkg.Callbacks;
use Do_Scheduling_Simulation_Pkg.Callbacks;
with Framework_Config;                       use Framework_Config;
with Translate;                              use Translate;
with Project;                                use Project;
with Processors;                             use Processors;
with Processor_Set;                          use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Mmi_Tools;                              use Mmi_Tools;
with Ada.Strings.Unbounded;                  use Ada.Strings.Unbounded;
with Unbounded_Strings;                      use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;

package body Do_Scheduling_Simulation_Pkg is

   procedure Gtk_New
     (Do_Scheduling_Simulation : out Do_Scheduling_Simulation_Access;
      Call                     : Do_Scheduling_Callback_Type)
   is

   begin
      Do_Scheduling_Simulation := new Do_Scheduling_Simulation_Record;
      Do_Scheduling_Simulation_Pkg.Initialize
        (Do_Scheduling_Simulation,
         Call);
   end Gtk_New;

   procedure Initialize
     (Do_Scheduling_Simulation : access Do_Scheduling_Simulation_Record'Class;
      Call                     : Do_Scheduling_Callback_Type)
   is

      Processor_Iterator : Processors_Iterator;
      Processor_Items    : Strings_Table;
      A_Processor        : Processor_Ptr;

   begin

      Do_Scheduling_Simulation.Call := Call;

      Gtk.Window.Initialize (Do_Scheduling_Simulation, Window_Toplevel);
      Set_Title
        (Do_Scheduling_Simulation,
         To_String (Lb_Scheduling_Options (Current_Language)));
      Set_Policy (Do_Scheduling_Simulation, True, True, True);
      Set_Position (Do_Scheduling_Simulation, Win_Pos_Center_Always);
      Set_Modal (Do_Scheduling_Simulation, True);
      Set_Extension_Events (Do_Scheduling_Simulation, Extension_Events_All);
      Set_Flags (Do_Scheduling_Simulation, Can_Default);
      Grab_Focus (Do_Scheduling_Simulation);

      Gtk_New_Vbox (Do_Scheduling_Simulation.Do_Scheduling_Simulation_Vbox);
      Add
        (Do_Scheduling_Simulation,
         Do_Scheduling_Simulation.Do_Scheduling_Simulation_Vbox);

      -- Table of options choosen by the users before simulation
      --
      Gtk_New (Do_Scheduling_Simulation.Value_Table, 2, 2, True);
      Set_Border_Width (Do_Scheduling_Simulation.Value_Table, 14);
      Set_Row_Spacings (Do_Scheduling_Simulation.Value_Table, 8);
      Set_Col_Spacings (Do_Scheduling_Simulation.Value_Table, 0);
      Pack_Start
        (Do_Scheduling_Simulation.Do_Scheduling_Simulation_Vbox,
         Do_Scheduling_Simulation.Value_Table);

      ----------------------------------------------------------------
      -- List of processors on which simulations have to be run
      ----------------------------------------------------------------

      Initialize (Processor_Items);
      Reset_Iterator (Sys.Processors, Processor_Iterator);

      loop
         Current_Element (Sys.Processors, A_Processor, Processor_Iterator);
         Add (Processor_Items, A_Processor.Name);

         exit when Is_Last_Element (Sys.Processors, Processor_Iterator);

         Next_Element (Sys.Processors, Processor_Iterator);
      end loop;

      Do_Scheduling_Simulation.Clist_Processor_Menu := Widget_SList.Null_List;
      Build_Option_Menu
        (Do_Scheduling_Simulation.Processor_Menu,
         Do_Scheduling_Simulation.Clist_Processor_Menu,
         Processor_Items,
         -1,
         null);

      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Processor_Menu,
         3,
         4,
         0,
         1);

      Gtk_New
        (Do_Scheduling_Simulation.Label,
         To_String (Lb_Processor (Current_Language) & Lb_Colon));

      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Label,
         2,
         3,
         0,
         1);

      Gtk_New
        (Do_Scheduling_Simulation.Label,
         To_String
            (Lb_Run_Simulation_On_One_Processor_Only (Current_Language) &
             Lb_Colon));
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Label,
         0,
         1,
         0,
         1);

      Gtk_New (Do_Scheduling_Simulation.Schedule_All_Processeurs, "");
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Schedule_All_Processeurs,
         1,
         2,
         0,
         1);

      Gtk_New_Hseparator (Do_Scheduling_Simulation.Hseparator);
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Hseparator,
         0,
         5,
         1,
         2);

      ----------------------------------------------------------------
      -- Event analyzers, number of context switch or preemption
      ----------------------------------------------------------------

      Gtk_New
        (Do_Scheduling_Simulation.Label,
         To_String
            (Lb_Number_Of_Context_Switch (Current_Language) & Lb_Colon));

      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Label,
         0,
         1,
         2,
         3);

      Gtk_New (Do_Scheduling_Simulation.Context_Switch, "");
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Context_Switch,
         1,
         2,
         2,
         3);

      Gtk_New
        (Do_Scheduling_Simulation.Label,
         To_String (Lb_Number_Of_Preemption (Current_Language) & Lb_Colon));
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Label,
         0,
         1,
         3,
         4);

      Gtk_New (Do_Scheduling_Simulation.Number_Preemption, "");
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Number_Preemption,
         1,
         2,
         3,
         4);

      Gtk_New
        (Do_Scheduling_Simulation.Label,
         To_String (Lb_Run_Event_Analyzers (Current_Language) & Lb_Colon));
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Label,
         0,
         1,
         4,
         5);

      Gtk_New (Do_Scheduling_Simulation.Event_Analyzer, "");
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Event_Analyzer,
         1,
         2,
         4,
         5);

      Gtk_New
        (Do_Scheduling_Simulation.Label,
         To_String (Lb_Display_Event_Table (Current_Language) & Lb_Colon));
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Label,
         2,
         4,
         2,
         3);

      Gtk_New (Do_Scheduling_Simulation.Schedule_And_Display_Event_Table, "");
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Schedule_And_Display_Event_Table,
         4,
         5,
         2,
         3);

      Gtk_New
        (Do_Scheduling_Simulation.Label,
         To_String
            (Lb_Automatically_Export_Event_Table (Current_Language) &
             Lb_Colon));
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Label,
         2,
         4,
         3,
         4);

      Gtk_New (Do_Scheduling_Simulation.Schedule_And_Export_Event_Table, "");
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Schedule_And_Export_Event_Table,
         4,
         5,
         3,
         4);

      Gtk_New
        (Do_Scheduling_Simulation.Label,
         To_String (Lb_Event_Table_File_Name (Current_Language) & Lb_Colon));
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Label,
         2,
         4,
         4,
         5);

      Gtk_New
        (Do_Scheduling_Simulation.Read_From_File,
         To_String (Lb_Read (Current_Language)));
      Attach_Defaults
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Read_From_File,
         4,
         5,
         4,
         5);
      Button_Callback.Connect
        (Do_Scheduling_Simulation.Read_From_File,
         "clicked",
         Button_Callback.To_Marshaller (On_Read_From_File_Pressed'Access));

      Gtk.GEntry.Gtk_New (Do_Scheduling_Simulation.Event_Table_File_Name, 50);
      Set_Editable (Do_Scheduling_Simulation.Event_Table_File_Name, True);
      Attach_Defaults
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Event_Table_File_Name,
         4,
         5,
         5,
         6);

      ----------------------------------------------------------------
      -- Response time and blocking time
      ----------------------------------------------------------------

      Gtk_New_Hseparator (Do_Scheduling_Simulation.Hseparator);
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Hseparator,
         0,
         5,
         6,
         7);

      Gtk_New
        (Do_Scheduling_Simulation.Label,
         To_String (Lb_Worst_Case (Current_Language)));
      Set_Alignment (Do_Scheduling_Simulation.Label, 0.5, 0.5);
      Set_Padding (Do_Scheduling_Simulation.Label, 0, 0);
      Set_Justify (Do_Scheduling_Simulation.Label, Justify_Left);
      Set_Line_Wrap (Do_Scheduling_Simulation.Label, False);
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Label,
         1,
         2,
         7,
         8);

      Gtk_New
        (Do_Scheduling_Simulation.Label,
         To_String (Lb_Best_Case (Current_Language)));
      Set_Alignment (Do_Scheduling_Simulation.Label, 0.5, 0.5);
      Set_Padding (Do_Scheduling_Simulation.Label, 0, 0);
      Set_Justify (Do_Scheduling_Simulation.Label, Justify_Left);
      Set_Line_Wrap (Do_Scheduling_Simulation.Label, False);
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Label,
         2,
         3,
         7,
         8);

      Gtk_New
        (Do_Scheduling_Simulation.Label,
         To_String (Lb_Average_Case (Current_Language)));
      Set_Alignment (Do_Scheduling_Simulation.Label, 0.5, 0.5);
      Set_Padding (Do_Scheduling_Simulation.Label, 0, 0);
      Set_Justify (Do_Scheduling_Simulation.Label, Justify_Left);
      Set_Line_Wrap (Do_Scheduling_Simulation.Label, False);
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Label,
         3,
         4,
         7,
         8);

      Gtk_New
        (Do_Scheduling_Simulation.Label,
         To_String (Lb_Curve (Current_Language)));
      Set_Alignment (Do_Scheduling_Simulation.Label, 0.5, 0.5);
      Set_Padding (Do_Scheduling_Simulation.Label, 0, 0);
      Set_Justify (Do_Scheduling_Simulation.Label, Justify_Left);
      Set_Line_Wrap (Do_Scheduling_Simulation.Label, False);
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Label,
         4,
         5,
         7,
         8);

      Gtk_New
        (Do_Scheduling_Simulation.Label,
         To_String (Lb_Task_Response_Time (Current_Language) & Lb_Colon));
      Set_Alignment (Do_Scheduling_Simulation.Label, 0.5, 0.5);
      Set_Padding (Do_Scheduling_Simulation.Label, 0, 0);
      Set_Justify (Do_Scheduling_Simulation.Label, Justify_Left);
      Set_Line_Wrap (Do_Scheduling_Simulation.Label, False);
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Label,
         0,
         1,
         9,
         10);

      Gtk_New
        (Do_Scheduling_Simulation.Label,
         To_String (Lb_Blocking_Time (Current_Language) & Lb_Colon));
      Set_Alignment (Do_Scheduling_Simulation.Label, 0.5, 0.5);
      Set_Padding (Do_Scheduling_Simulation.Label, 0, 0);
      Set_Justify (Do_Scheduling_Simulation.Label, Justify_Left);
      Set_Line_Wrap (Do_Scheduling_Simulation.Label, False);
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Label,
         0,
         1,
         10,
         11);

      -- A table of Check buttons  (worst/best/average/curve cases)
      --

      Gtk_New
        (Do_Scheduling_Simulation.Align,
         Xalign => 0.5,
         Yalign => 0.5,
         Xscale => 0.1,
         Yscale => 0.1);
      Gtk_New (Do_Scheduling_Simulation.Response_Time (Worst_Case), "");
      Add
        (Do_Scheduling_Simulation.Align,
         Do_Scheduling_Simulation.Response_Time (Worst_Case));
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Align,
         1,
         2,
         9,
         10);
      Gtk_New
        (Do_Scheduling_Simulation.Align,
         Xalign => 0.5,
         Yalign => 0.5,
         Xscale => 0.1,
         Yscale => 0.1);
      Gtk_New (Do_Scheduling_Simulation.Response_Time (Best_Case), "");
      Add
        (Do_Scheduling_Simulation.Align,
         Do_Scheduling_Simulation.Response_Time (Best_Case));
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Align,
         2,
         3,
         9,
         10);
      Gtk_New
        (Do_Scheduling_Simulation.Align,
         Xalign => 0.5,
         Yalign => 0.5,
         Xscale => 0.1,
         Yscale => 0.1);
      Gtk_New (Do_Scheduling_Simulation.Response_Time (Average_Case), "");
      Add
        (Do_Scheduling_Simulation.Align,
         Do_Scheduling_Simulation.Response_Time (Average_Case));
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Align,
         3,
         4,
         9,
         10);
      Gtk_New
        (Do_Scheduling_Simulation.Align,
         Xalign => 0.5,
         Yalign => 0.5,
         Xscale => 0.1,
         Yscale => 0.1);
      Gtk_New (Do_Scheduling_Simulation.Response_Time (Plot_Case), "");
      Add
        (Do_Scheduling_Simulation.Align,
         Do_Scheduling_Simulation.Response_Time (Plot_Case));
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Align,
         4,
         5,
         9,
         10);

      Gtk_New
        (Do_Scheduling_Simulation.Align,
         Xalign => 0.5,
         Yalign => 0.5,
         Xscale => 0.1,
         Yscale => 0.1);
      Gtk_New (Do_Scheduling_Simulation.Blocking_Time (Worst_Case), "");
      Add
        (Do_Scheduling_Simulation.Align,
         Do_Scheduling_Simulation.Blocking_Time (Worst_Case));
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Align,
         1,
         2,
         10,
         11);
      Gtk_New
        (Do_Scheduling_Simulation.Align,
         Xalign => 0.5,
         Yalign => 0.5,
         Xscale => 0.1,
         Yscale => 0.1);
      Gtk_New (Do_Scheduling_Simulation.Blocking_Time (Best_Case), "");
      Add
        (Do_Scheduling_Simulation.Align,
         Do_Scheduling_Simulation.Blocking_Time (Best_Case));
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Align,
         2,
         3,
         10,
         11);
      Gtk_New
        (Do_Scheduling_Simulation.Align,
         Xalign => 0.5,
         Yalign => 0.5,
         Xscale => 0.1,
         Yscale => 0.1);
      Gtk_New (Do_Scheduling_Simulation.Blocking_Time (Average_Case), "");
      Add
        (Do_Scheduling_Simulation.Align,
         Do_Scheduling_Simulation.Blocking_Time (Average_Case));
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Align,
         3,
         4,
         10,
         11);
      Gtk_New
        (Do_Scheduling_Simulation.Align,
         Xalign => 0.5,
         Yalign => 0.5,
         Xscale => 0.1,
         Yscale => 0.1);
      Gtk_New (Do_Scheduling_Simulation.Blocking_Time (Plot_Case), "");
      Add
        (Do_Scheduling_Simulation.Align,
         Do_Scheduling_Simulation.Blocking_Time (Plot_Case));
      Attach
        (Do_Scheduling_Simulation.Value_Table,
         Do_Scheduling_Simulation.Align,
         4,
         5,
         10,
         11);

      ----------------------------------------------
      -- Initiliaze the widet of the window
      ----------------------------------------------

      Set_Active
        (Do_Scheduling_Simulation.Context_Switch,
         Context_Switch_From_Simulation);
      Set_Active
        (Do_Scheduling_Simulation.Number_Preemption,
         Preemption_From_Simulation);
      Set_Active
        (Do_Scheduling_Simulation.Event_Analyzer,
         Run_Event_Analyzer_From_Simulation);
      Set_Active
        (Do_Scheduling_Simulation.Schedule_And_Display_Event_Table,
         Schedule_And_Display_Event_Table);
      Set_Active
        (Do_Scheduling_Simulation.Schedule_And_Export_Event_Table,
         Schedule_And_Export_Event_Table);
      Set_Active
        (Do_Scheduling_Simulation.Schedule_All_Processeurs,
         Schedule_All_Processeurs);

      for I in  Data_From_Simulation'Range loop
         Set_Active
           (Do_Scheduling_Simulation.Response_Time (I),
            Response_Time_From_Simulation (I));
         Set_Active
           (Do_Scheduling_Simulation.Blocking_Time (I),
            Blocking_Time_From_Simulation (I));
      end loop;

      Set_Text
        (Do_Scheduling_Simulation.Event_Table_File_Name,
         To_String (Xml_Event_Table_File_Name));

      ---------------------------------------------------------
      -- The bottom part of the widget (ok and cancel buttons)
      ---------------------------------------------------------

      Gtk_New_Hseparator (Do_Scheduling_Simulation.Hseparator);
      Pack_Start
        (Do_Scheduling_Simulation.Do_Scheduling_Simulation_Vbox,
         Do_Scheduling_Simulation.Hseparator);

      Gtk_New_Hbox (Do_Scheduling_Simulation.Button_Hbox);
      Pack_Start
        (Do_Scheduling_Simulation.Do_Scheduling_Simulation_Vbox,
         Do_Scheduling_Simulation.Button_Hbox,
         False,
         True,
         5);

      Gtk_New (Do_Scheduling_Simulation.Ok_Button, "Ok");
      Add
        (Do_Scheduling_Simulation.Button_Hbox,
         Do_Scheduling_Simulation.Ok_Button);
      Button_Callback.Connect
        (Do_Scheduling_Simulation.Ok_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'Access));

      Gtk_New
        (Do_Scheduling_Simulation.Cancel_Button,
         To_String (Lb_Cancel (Current_Language)));
      Add
        (Do_Scheduling_Simulation.Button_Hbox,
         Do_Scheduling_Simulation.Cancel_Button);
      Button_Callback.Connect
        (Do_Scheduling_Simulation.Cancel_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'Access));

   end Initialize;

end Do_Scheduling_Simulation_Pkg;
