------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Gdk;
with Gdk.Event;            use Gdk.Event;
with Gtk;                  use Gtk;
with Double_Buffer;        use Double_Buffer;
with Gtk.Widget;           use Gtk.Widget;
with Gdk.Color;            use Gdk.Color;
with Gdk.Font;             use Gdk.Font;
with Glib;                 use Glib;
with Gdk.GC;               use Gdk.GC;
with Buffers;              use Buffers;
with Scheduling_Sequences; use Scheduling_Sequences;
with Time_Units;           use Time_Units;
use Time_Units.A_Time_Unit;
use Time_Units.Time_Unit_List;

package Draw_Buffer_Pkg is

   type Draw_Buffer_Record is new Gtk_Double_Buffer_Record with record
      Font : Gdk_Font;

      Buff             : Buffer_Size_Table_Ptr;
      Last_Time_Unit   : Natural;
      Time_Unit_Number : Time_Unit_Range;
      Max_Buff         : Natural;

      White_Gc  : Gdk.GC.Gdk_GC;
      Black_Gc  : Gdk.GC.Gdk_GC;
      Red_Color : Gdk_Color;

      Left_Margin     : Gint := 60;
      High_Margin     : Gint := 20;
      Buffer_Margin   : Gint := 20;
      Timeunit_Margin : Gint := 20;

      First_Width    : Gint := 800;
      First_Height   : Gint := 300;
      Current_Width  : Gint;
      Current_Height : Gint;

      Have_Colors : Boolean := False;
      Initialized : Boolean := False;

      Buff_Ptr : Buffer_Ptr;
   end record;

   type Draw_Buffer_Access is access all Draw_Buffer_Record'Class;

   procedure Gtk_New (Draw_Buffer : out Draw_Buffer_Access);
   procedure Initialize (Draw_Buffer : access Draw_Buffer_Record'Class);

   procedure Redraw (Draw_Buffer : access Draw_Buffer_Record'Class);

   procedure Draw
     (Draw_Buffer       : access Draw_Buffer_Record'Class;
      Buff              : Buffer_Ptr;
      Buff1             : Buffer_Size_Table;
      Last_Time_Unit1   : Natural;
      Time_Unit_Number1 : Time_Unit_Range);

   procedure Clear (Draw_Buffer : access Draw_Buffer_Record'Class);

   function Configure
     (Draw_Buffer : access Draw_Buffer_Record'Class;
      Event       : Gdk.Event.Gdk_Event)
      return        Boolean;

   procedure Color_Allocation
     (Draw_Buffer : access Draw_Buffer_Record'Class);

   procedure Size_Allocate
     (Draw_Buffer : access Draw_Buffer_Record'Class;
      Allocation  : Gtk_Allocation_Access);

   Draw_Buffer : Draw_Buffer_Access;

   pragma Inline (Redraw);
   pragma Inline (Color_Allocation);
   pragma Inline (Clear);

end Draw_Buffer_Pkg;
