------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Glib;                    use Glib;
with Gdk;                     use Gdk;
with Gdk.Event;               use Gdk.Event;
with Gdk.Font;                use Gdk.Font;
with Gdk.GC;                  use Gdk.GC;
with Gdk.Color;               use Gdk.Color;
with Gtk;                     use Gtk;
with Double_Buffer;           use Double_Buffer;
with Gtk.Widget;              use Gtk.Widget;
with Buffer_Set;              use Buffer_Set;
with Task_Set;                use Task_Set;
with Resource_Set;            use Resource_Set;
with Message_Set;             use Message_Set;
with Multiprocessor_Services; use Multiprocessor_Services;
use Multiprocessor_Services.Scheduling_Result_Per_Processor;

package Draw_Scheduling_Pkg is

   type Draw_Scheduling_Record is tagged record
      Font   : Gdk_Font;
      Buffer : Gtk_Double_Buffer;

      Tasks      : Tasks_Set;
      Resources  : Resources_Set;
      Messages   : Messages_Set;
      Buffers    : Buffers_Set;
      Period     : Natural;
      Start_Draw : Natural;
      Sched      : Scheduling_Table_Ptr;

      White_Gc   : Gdk.GC.Gdk_GC;
      Black_Gc   : Gdk.GC.Gdk_GC;
      Red_Color  : Gdk_Color;
      Blue_Color : Gdk_Color;

      Left_Margin     : Gint := 0;
      Right_Margin    : Gint := 20;
      High_Margin     : Gint := 20;
      Interligne      : Gint := 50;
      Timeunit_Margin : Gint := 20;

      First_Width    : Gint := 800;
      First_Height   : Gint := 300;
      Current_Width  : Gint;
      Current_Height : Gint;

      Have_Colors : Boolean := False;
      Initialized : Boolean := False;
   end record;

   type Draw_Scheduling_Access is access all Draw_Scheduling_Record'Class;

   procedure Gtk_New (Draw_Scheduling : out Draw_Scheduling_Access);
   procedure Initialize
     (Draw_Scheduling : access Draw_Scheduling_Record'Class);

   procedure Redraw_Time_Line
     (Draw_Scheduling : access Draw_Scheduling_Record'Class);

   procedure Draw_Time_Line
     (Draw_Scheduling : access Draw_Scheduling_Record;
      Task1           : Tasks_Set;
      Resource1       : Resources_Set;
      Message1        : Messages_Set;
      Buffer1         : Buffers_Set;
      Period1         : Natural;
      Start_Draw1     : Natural;
      Sched1          : Scheduling_Table_Ptr);

   procedure Clear_Time_Line
     (Draw_Scheduling : access Draw_Scheduling_Record);

   procedure Color_Allocation
     (Draw_Scheduling : access Draw_Scheduling_Record);

   function Configure
     (Buffer : access Gtk_Double_Buffer_Record'Class;
      Event  : Gdk.Event.Gdk_Event)
      return   Boolean;
   procedure Size_Allocate
     (Buffer     : access Gtk_Double_Buffer_Record'Class;
      Allocation : Gtk_Allocation_Access);

   Draw_Scheduling : Draw_Scheduling_Access;

end Draw_Scheduling_Pkg;
