------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Framework_Config;                     use Framework_Config;
with Translate;                            use Translate;
with Task_Set;                             use Task_Set;
use Task_Set.Generic_Task_Set;
with Processor_Set;                        use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Ada.Strings.Unbounded;                use Ada.Strings.Unbounded;
with Gtk;                                  use Gtk;
with Gtk.Enums;                            use Gtk.Enums;
with Callbacks_Cheddar;                    use Callbacks_Cheddar;
with Draw_Time_Line_From_To_Pkg.Callbacks;
use Draw_Time_Line_From_To_Pkg.Callbacks;

package body Draw_Time_Line_From_To_Pkg is

   procedure Gtk_New
     (Draw_Time_Line_From_To : out Draw_Time_Line_From_To_Access;
      Draw_Scheduling        : Draw_Scheduling_Access;
      Sys                    : in System)
   is

   begin
      Draw_Time_Line_From_To := new Draw_Time_Line_From_To_Record;
      Draw_Time_Line_From_To_Pkg.Initialize
        (Draw_Time_Line_From_To,
         Draw_Scheduling,
         Sys);
   end Gtk_New;

   procedure Initialize
     (Draw_Time_Line_From_To : access Draw_Time_Line_From_To_Record'Class;
      Draw_Scheduling        : Draw_Scheduling_Access;
      Sys                    : in System)
   is

   begin

      Draw_Time_Line_From_To.Draw_Scheduling := Draw_Scheduling;

      Gtk.Window.Initialize (Draw_Time_Line_From_To, Window_Dialog);
      Set_Title
        (Draw_Time_Line_From_To,
         To_String (Lb_Draw_Time_Line_From_Event_Table (Current_Language)));
      Set_Policy (Draw_Time_Line_From_To, False, True, False);
      Set_Position (Draw_Time_Line_From_To, Win_Pos_Center_Always);
      Set_Modal (Draw_Time_Line_From_To, True);

      Gtk_New_Vbox (Draw_Time_Line_From_To.Vbox, False, 0);
      Add (Draw_Time_Line_From_To, Draw_Time_Line_From_To.Vbox);

      Gtk_New_Hbox (Draw_Time_Line_From_To.Hbox, False, 0);
      Set_Border_Width (Draw_Time_Line_From_To.Hbox, 16);
      Pack_Start
        (Draw_Time_Line_From_To.Vbox,
         Draw_Time_Line_From_To.Hbox,
         True,
         True,
         0);

      Gtk_New
        (Draw_Time_Line_From_To.Lab,
         To_String (Lb_Draw_From (Current_Language) & Lb_Colon));
      Set_Alignment (Draw_Time_Line_From_To.Lab, 0.5, 0.5);
      Set_Padding (Draw_Time_Line_From_To.Lab, 0, 0);
      Set_Justify (Draw_Time_Line_From_To.Lab, Justify_Left);
      Set_Line_Wrap (Draw_Time_Line_From_To.Lab, False);
      Pack_Start
        (Draw_Time_Line_From_To.Hbox,
         Draw_Time_Line_From_To.Lab,
         False,
         False,
         0);

      Gtk_New (Draw_Time_Line_From_To.Draw_From_Entry);
      Set_Editable (Draw_Time_Line_From_To.Draw_From_Entry, True);
      Set_Max_Length (Draw_Time_Line_From_To.Draw_From_Entry, 0);
      Set_Text (Draw_Time_Line_From_To.Draw_From_Entry, "0");
      Set_Visibility (Draw_Time_Line_From_To.Draw_From_Entry, True);
      Pack_Start
        (Draw_Time_Line_From_To.Hbox,
         Draw_Time_Line_From_To.Draw_From_Entry,
         True,
         True,
         0);

      Gtk_New_Hbox (Draw_Time_Line_From_To.Hbox, False, 0);
      Set_Border_Width (Draw_Time_Line_From_To.Hbox, 16);
      Pack_Start
        (Draw_Time_Line_From_To.Vbox,
         Draw_Time_Line_From_To.Hbox,
         True,
         True,
         0);

      Gtk_New
        (Draw_Time_Line_From_To.Lab,
         To_String (Lb_Draw_To (Current_Language) & Lb_Colon));
      Set_Alignment (Draw_Time_Line_From_To.Lab, 0.5, 0.5);
      Set_Padding (Draw_Time_Line_From_To.Lab, 0, 0);
      Set_Justify (Draw_Time_Line_From_To.Lab, Justify_Left);
      Set_Line_Wrap (Draw_Time_Line_From_To.Lab, False);
      Pack_Start
        (Draw_Time_Line_From_To.Hbox,
         Draw_Time_Line_From_To.Lab,
         False,
         False,
         0);

      Gtk_New (Draw_Time_Line_From_To.Draw_To_Entry);
      Set_Editable (Draw_Time_Line_From_To.Draw_To_Entry, True);
      Set_Max_Length (Draw_Time_Line_From_To.Draw_To_Entry, 0);
      Set_Text (Draw_Time_Line_From_To.Draw_To_Entry, "0");
      Set_Visibility (Draw_Time_Line_From_To.Draw_To_Entry, True);
      Pack_Start
        (Draw_Time_Line_From_To.Hbox,
         Draw_Time_Line_From_To.Draw_To_Entry,
         True,
         True,
         0);

      Gtk_New_Hseparator (Draw_Time_Line_From_To.Separator);
      Pack_Start
        (Draw_Time_Line_From_To.Vbox,
         Draw_Time_Line_From_To.Separator,
         True,
         True,
         0);

      Gtk_New_Hbox (Draw_Time_Line_From_To.Hbox, True, 0);
      Set_Border_Width (Draw_Time_Line_From_To.Hbox, 9);
      Pack_Start
        (Draw_Time_Line_From_To.Vbox,
         Draw_Time_Line_From_To.Hbox,
         True,
         True,
         0);

      Gtk_New (Draw_Time_Line_From_To.Ok_Button, "Ok");
      Set_Usize (Draw_Time_Line_From_To.Ok_Button, 70, 25);
      Button_Callback.Connect
        (Draw_Time_Line_From_To.Ok_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'Access));
      Pack_Start
        (Draw_Time_Line_From_To.Hbox,
         Draw_Time_Line_From_To.Ok_Button,
         False,
         False,
         0);

      Gtk_New
        (Draw_Time_Line_From_To.Cancel_Button,
         To_String (Lb_Cancel (Current_Language)));
      Set_Usize (Draw_Time_Line_From_To.Cancel_Button, 70, 25);
      Button_Callback.Connect
        (Draw_Time_Line_From_To.Cancel_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'Access));

      Pack_Start
        (Draw_Time_Line_From_To.Hbox,
         Draw_Time_Line_From_To.Cancel_Button,
         False,
         False,
         0);

   end Initialize;

end Draw_Time_Line_From_To_Pkg;
