------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Project;               use Project;
with Framework_Config;      use Framework_Config;
with Translate;             use Translate;
with Glib;                  use Glib;
with Mmi_Tools;             use Mmi_Tools;
with Gtk.Radio_Menu_Item;   use Gtk.Radio_Menu_Item;
with Tasks;                 use Tasks;
with Task_Set;              use Task_Set;
use Task_Set.Generic_Task_Set;
with Buffers;               use Buffers;
with Buffer_Set;            use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Messages;              use Messages;
with Message_Set;           use Message_Set;
use Message_Set.Generic_Message_Set;
with Processors;            use Processors;
with Processor_Set;         use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Resources;             use Resources;
with Resource_Set;          use Resource_Set;
use Resource_Set.Generic_Resource_Set;
with Networks;              use Networks;
with Network_Set;           use Network_Set;
use Network_Set.Generic_Network_Set;
with Address_Spaces;        use Address_Spaces;
with Address_Space_Set;     use Address_Space_Set;
use Address_Space_Set.Generic_Address_Space_Set;
with Task_Dependencies;     use Task_Dependencies;

package body Duplicate_Pkg.Callbacks is

   procedure On_Cancel_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      Destroy (Duplicate);
   end On_Cancel_Pressed;

   procedure On_Duplicate_Pressed (Object : access Gtk_Button_Record'Class) is

      Object_Index           : Gint;
      Ok, Already            : Boolean;
      Old_Object, New_Object : Unbounded_String;

      A_Resource, A_New_Resource            : Generic_Resource_Ptr;
      Resource_Ite                          : Resources_Iterator;
      A_Task, A_New_Task                    : Generic_Task_Ptr;
      Task_Ite                              : Tasks_Iterator;
      A_Processor, A_New_Processor          : Processor_Ptr;
      Processor_Ite                         : Processors_Iterator;
      A_Buffer, A_New_Buffer                : Buffer_Ptr;
      Buffer_Ite                            : Buffers_Iterator;
      A_Message, A_New_Message              : Message_Ptr;
      Message_Ite                           : Messages_Iterator;
      A_Network, A_New_Network              : Network_Ptr;
      Network_Ite                           : Network_Iterator;
      An_Address_Space, A_New_Address_Space : Address_Space_Ptr;
      Address_Space_Ite                     : Address_Spaces_Iterator;

   begin

      Old_Object := To_Unbounded_String (Get_Text (Duplicate.Old_Entry));
      New_Object := To_Unbounded_String (Get_Text (Duplicate.New_Entry));

      Object_Index := Gint (Selected_Button (Duplicate.Clist_Object_Menu));

      -- First of all : check if sets are empty
      --
      Ok := False;

      if Object_Index = 0 then
         if (Get_Number_Of_Elements (Duplicate.Sys.Networks) > 0) then
            Ok := True;
         else
            Show_Message_Box (Lb_Define_Networks_Before (Current_Language));
         end if;
      end if;

      if Object_Index = 1 then
         if (Get_Number_Of_Elements (Duplicate.Sys.Messages) > 0) then
            Ok := True;
         else
            Show_Message_Box (Lb_Define_Messages_Before (Current_Language));
         end if;
      end if;

      if Object_Index = 2 then
         if (Get_Number_Of_Elements (Duplicate.Sys.Buffers) > 0) then
            Ok := True;
         else
            Show_Message_Box (Lb_Define_Buffers_Before (Current_Language));
         end if;
      end if;

      if Object_Index = 3 then
         if (Get_Number_Of_Elements (Duplicate.Sys.Resources) > 0) then
            Ok := True;
         else
            Show_Message_Box (Lb_Define_Resources_Before (Current_Language));
         end if;
      end if;

      if Object_Index = 4 then
         if (Get_Number_Of_Elements (Duplicate.Sys.Tasks) > 0) then
            Ok := True;
         else
            Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
         end if;
      end if;

      if Object_Index = 5 then
         if (Get_Number_Of_Elements (Duplicate.Sys.Address_Spaces) > 0) then
            Ok := True;
         else
            Show_Message_Box
              (Lb_Define_Address_Spaces_Before (Current_Language));
         end if;
      end if;

      if Object_Index = 6 then
         if (Get_Number_Of_Elements (Duplicate.Sys.Processors) > 0) then
            Ok := True;
         else
            Show_Message_Box (Lb_Define_Processors_Before (Current_Language));
         end if;
      end if;

      -- Now, looking for the object and copy itself
      --
      if Ok then

         Ok      := False;
         Already := False;

         if Object_Index = 0 then
            Reset_Iterator (Duplicate.Sys.Networks, Network_Ite);

            loop
               Current_Element
                 (Duplicate.Sys.Networks,
                  A_Network,
                  Network_Ite);

               if A_Network.Name = New_Object then
                  Already := True;
               end if;

               if A_Network.Name = Old_Object then
                  A_New_Network      := Copy (A_Network);
                  A_New_Network.Name := New_Object;
                  Ok                 := True;
               end if;

               exit when Is_Last_Element
                            (Duplicate.Sys.Networks,
                             Network_Ite);

               Next_Element (Duplicate.Sys.Networks, Network_Ite);
            end loop;
            if not Ok then
               Show_Message_Box (Lb_Network_Not_Found (Current_Language));

            else
               if Already then
                  Show_Message_Box
                    (Lb_Network (Current_Language) &
                     Lb_Already_Defined (Current_Language));
               else
                  Add (Duplicate.Sys.Networks, A_New_Network);
               end if;
            end if;
         end if;

         if Object_Index = 1 then
            Reset_Iterator (Duplicate.Sys.Messages, Message_Ite);

            loop
               Current_Element
                 (Duplicate.Sys.Messages,
                  A_Message,
                  Message_Ite);

               if A_Message.Name = New_Object then
                  Already := True;
               end if;

               if A_Message.Name = Old_Object then
                  A_New_Message      := Copy (A_Message);
                  A_New_Message.Name := New_Object;
                  Ok                 := True;
               end if;

               exit when Is_Last_Element
                            (Duplicate.Sys.Messages,
                             Message_Ite);

               Next_Element (Duplicate.Sys.Messages, Message_Ite);
            end loop;

            if not Ok then
               Show_Message_Box (Lb_Message_Not_Found (Current_Language));

            else
               if Already then
                  Show_Message_Box
                    (Lb_Message (Current_Language) &
                     Lb_Already_Defined (Current_Language));
               else
                  Add (Duplicate.Sys.Messages, A_New_Message);
               end if;
            end if;
         end if;

         if Object_Index = 2 then
            Reset_Iterator (Duplicate.Sys.Buffers, Buffer_Ite);

            loop
               Current_Element (Duplicate.Sys.Buffers, A_Buffer, Buffer_Ite);

               if A_Buffer.Name = New_Object then
                  Already := True;
               end if;

               if A_Buffer.Name = Old_Object then
                  A_New_Buffer      := Copy (A_Buffer);
                  A_New_Buffer.Name := New_Object;
                  Ok                := True;
               end if;

               exit when Is_Last_Element (Duplicate.Sys.Buffers, Buffer_Ite);

               Next_Element (Duplicate.Sys.Buffers, Buffer_Ite);
            end loop;

            if not Ok then
               Show_Message_Box (Lb_Buffer_Not_Found (Current_Language));

            else
               if Already then
                  Show_Message_Box
                    (Lb_Buffer (Current_Language) &
                     Lb_Already_Defined (Current_Language));
               else
                  Add (Duplicate.Sys.Buffers, A_New_Buffer);

                  -- Update dependencies :
                  --
                  Add_All_Task_Dependencies
                    (Duplicate.Sys.Dependencies,
                     Duplicate.Sys.Tasks,
                     A_New_Buffer);
               end if;
            end if;
         end if;

         if Object_Index = 3 then
            Reset_Iterator (Duplicate.Sys.Resources, Resource_Ite);

            loop
               Current_Element
                 (Duplicate.Sys.Resources,
                  A_Resource,
                  Resource_Ite);

               if A_Resource.Name = New_Object then
                  Already := True;
               end if;

               if A_Resource.Name = Old_Object then
                  A_New_Resource      := Copy (A_Resource);
                  A_New_Resource.Name := New_Object;
                  Ok                  := True;
               end if;

               exit when Is_Last_Element
                            (Duplicate.Sys.Resources,
                             Resource_Ite);

               Next_Element (Duplicate.Sys.Resources, Resource_Ite);
            end loop;

            if not Ok then
               Show_Message_Box (Lb_Resource_Not_Found (Current_Language));

            else
               if Already then
                  Show_Message_Box
                    (Lb_Resource (Current_Language) &
                     Lb_Already_Defined (Current_Language));
               else
                  Add (Duplicate.Sys.Resources, A_New_Resource);
               end if;
            end if;
         end if;

         if Object_Index = 4 then
            Reset_Iterator (Duplicate.Sys.Tasks, Task_Ite);

            loop
               Current_Element (Duplicate.Sys.Tasks, A_Task, Task_Ite);

               if A_Task.Name = New_Object then
                  Already := True;
               end if;

               if A_Task.Name = Old_Object then
                  A_New_Task      := Copy (A_Task);
                  A_New_Task.Name := New_Object;
                  Ok              := True;
               end if;

               exit when Is_Last_Element (Duplicate.Sys.Tasks, Task_Ite);

               Next_Element (Duplicate.Sys.Tasks, Task_Ite);
            end loop;

            if not Ok then
               Show_Message_Box (Lb_Task_Not_Found (Current_Language));

            else
               if Already then
                  Show_Message_Box
                    (Lb_Task (Current_Language) &
                     Lb_Already_Defined (Current_Language));
               else
                  Add (Duplicate.Sys.Tasks, A_New_Task);
               end if;
            end if;
         end if;

         if Object_Index = 5 then
            Reset_Iterator (Duplicate.Sys.Address_Spaces, Address_Space_Ite);

            loop
               Current_Element
                 (Duplicate.Sys.Address_Spaces,
                  An_Address_Space,
                  Address_Space_Ite);

               if An_Address_Space.Name = New_Object then
                  Already := True;
               end if;

               if An_Address_Space.Name = Old_Object then
                  A_New_Address_Space      := Copy (An_Address_Space);
                  A_New_Address_Space.Name := New_Object;
                  Ok                       := True;
               end if;

               exit when Is_Last_Element
                            (Duplicate.Sys.Address_Spaces,
                             Address_Space_Ite);

               Next_Element (Duplicate.Sys.Address_Spaces, Address_Space_Ite);
            end loop;
            if not Ok then
               Show_Message_Box
                 (Lb_Address_Space_Not_Found (Current_Language));

            else
               if Already then
                  Show_Message_Box
                    (Lb_Address_Space (Current_Language) &
                     Lb_Already_Defined (Current_Language));
               else
                  Add (Duplicate.Sys.Address_Spaces, A_New_Address_Space);
               end if;
            end if;
         end if;

         if Object_Index = 6 then
            Reset_Iterator (Duplicate.Sys.Processors, Processor_Ite);

            loop
               Current_Element
                 (Duplicate.Sys.Processors,
                  A_Processor,
                  Processor_Ite);

               if A_Processor.Name = New_Object then
                  Already := True;
               end if;

               if A_Processor.Name = Old_Object then
                  A_New_Processor      := Copy (A_Processor);
                  A_New_Processor.Name := New_Object;
                  Ok                   := True;
               end if;

               exit when Is_Last_Element
                            (Duplicate.Sys.Processors,
                             Processor_Ite);

               Next_Element (Duplicate.Sys.Processors, Processor_Ite);
            end loop;
            if not Ok then
               Show_Message_Box (Lb_Processor_Not_Found (Current_Language));

            else
               if Already then
                  Show_Message_Box
                    (Lb_Processor (Current_Language) &
                     Lb_Already_Defined (Current_Language));
               else
                  Add (Duplicate.Sys.Processors, A_New_Processor);
               end if;
            end if;
         end if;

      end if;

   exception

      when Task_Set.Generic_Task_Set.Full_Set =>
         begin
            Show_Message_Box
              (Lb_Can_Not_Define_More_Tasks (Current_Language));
            return;
         end;
      when Processor_Set.Generic_Processor_Set.Full_Set =>
         begin
            Show_Message_Box
              (Lb_Can_Not_Define_More_Processors (Current_Language));
            return;
         end;
      when Address_Space_Set.Generic_Address_Space_Set.Full_Set =>
         begin
            Show_Message_Box
              (Lb_Can_Not_Define_More_Address_Spaces (Current_Language));
            return;
         end;
      when Message_Set.Generic_Message_Set.Full_Set =>
         begin
            Show_Message_Box
              (Lb_Can_Not_Define_More_Messages (Current_Language));
            return;
         end;
      when Buffer_Set.Generic_Buffer_Set.Full_Set =>
         begin
            Show_Message_Box
              (Lb_Can_Not_Define_More_Buffers (Current_Language));
            return;
         end;
      when Resource_Set.Generic_Resource_Set.Full_Set =>
         begin
            Show_Message_Box
              (Lb_Can_Not_Define_More_Resources (Current_Language));
            return;
         end;

      when Network_Set.Generic_Network_Set.Full_Set =>
         begin
            Show_Message_Box
              (Lb_Can_Not_Define_More_Networks (Current_Language));
            return;
         end;
   end On_Duplicate_Pressed;

   procedure On_Ok_Pressed (Object : access Gtk_Button_Record'Class) is

   begin
      Sys := Duplicate.Sys;

      Is_New   := False;
      Is_Saved := False;

      Destroy (Duplicate);
   end On_Ok_Pressed;

end Duplicate_Pkg.Callbacks;
