------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Glib;                    use Glib;
with Framework_Config;        use Framework_Config;
with Translate;               use Translate;
with Mmi_Tools;               use Mmi_Tools;
with Ada.Strings.Unbounded;   use Ada.Strings.Unbounded;
with Unbounded_Strings;       use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;
with Gtk;                     use Gtk;
with Gtk.Enums;               use Gtk.Enums;
with Project;                 use Project;
with Callbacks_Cheddar;       use Callbacks_Cheddar;
with Duplicate_Pkg.Callbacks; use Duplicate_Pkg.Callbacks;

package body Duplicate_Pkg is

   procedure Gtk_New (Duplicate : out Duplicate_Access) is
   begin
      Duplicate := new Duplicate_Record;
      Duplicate_Pkg.Initialize (Duplicate);
   end Gtk_New;

   procedure Initialize (Duplicate : access Duplicate_Record'Class) is

      Items : Strings_Table;

   begin

      Duplicate.Sys := Sys;

      Gtk.Window.Initialize (Duplicate, Window_Dialog);
      Set_Title (Duplicate, To_String (Lb_Duplicate (Current_Language)));
      Set_Policy (Duplicate, False, True, False);
      Set_Position (Duplicate, Win_Pos_Center_Always);
      Set_Modal (Duplicate, True);

      Gtk_New (Duplicate.Table, 5, 2, True);
      Set_Border_Width (Duplicate.Table, 14);
      Set_Row_Spacings (Duplicate.Table, 8);
      Set_Col_Spacings (Duplicate.Table, 0);
      Add (Duplicate, Duplicate.Table);

      Gtk_New
        (Duplicate.Object_Label,
         To_String (Lb_Object (Current_Language) & Lb_Colon));
      Set_Alignment (Duplicate.Object_Label, 0.5, 0.5);
      Set_Padding (Duplicate.Object_Label, 0, 0);
      Set_Justify (Duplicate.Object_Label, Justify_Left);
      Set_Line_Wrap (Duplicate.Object_Label, False);
      Attach
        (Duplicate.Table,
         Duplicate.Object_Label,
         0,
         1,
         0,
         1,
         0,
         0,
         0,
         0);

      Gtk_New
        (Duplicate.Old_Label,
         To_String (Lb_Old (Current_Language) & Lb_Colon));
      Set_Alignment (Duplicate.Old_Label, 0.5, 0.5);
      Set_Padding (Duplicate.Old_Label, 0, 0);
      Set_Justify (Duplicate.Old_Label, Justify_Left);
      Set_Line_Wrap (Duplicate.Old_Label, False);
      Attach (Duplicate.Table, Duplicate.Old_Label, 0, 1, 1, 2, 0, 0, 0, 0);

      Gtk_New
        (Duplicate.New_Label,
         To_String (Lb_New (Current_Language) & Lb_Colon));
      Set_Alignment (Duplicate.New_Label, 0.5, 0.5);
      Set_Padding (Duplicate.New_Label, 0, 0);
      Set_Justify (Duplicate.New_Label, Justify_Left);
      Set_Line_Wrap (Duplicate.New_Label, False);
      Attach (Duplicate.Table, Duplicate.New_Label, 0, 1, 2, 3, 0, 0, 0, 0);

      Initialize (Items);
      Add (Items, Lb_Processor (Current_Language));
      Add (Items, Lb_Address_Space (Current_Language));
      Add (Items, Lb_Task (Current_Language));
      Add (Items, Lb_Resource (Current_Language));
      Add (Items, Lb_Buffer (Current_Language));
      Add (Items, Lb_Message (Current_Language));
      Add (Items, Lb_Network (Current_Language));
      Duplicate.Clist_Object_Menu := Widget_SList.Null_List;
      Build_Option_Menu
        (Duplicate.Object_Menu,
         Duplicate.Clist_Object_Menu,
         Items,
         -1,
         null);
      Attach (Duplicate.Table, Duplicate.Object_Menu, 1, 2, 0, 1, 0, 0, 0, 0);

      Gtk_New (Duplicate.Old_Entry);
      Set_Editable (Duplicate.Old_Entry, True);
      Set_Max_Length (Duplicate.Old_Entry, 0);
      Set_Text (Duplicate.Old_Entry, "");
      Set_Visibility (Duplicate.Old_Entry, True);
      Attach (Duplicate.Table, Duplicate.Old_Entry, 1, 2, 1, 2, 0, 0, 0, 0);

      Gtk_New (Duplicate.New_Entry);
      Set_Editable (Duplicate.New_Entry, True);
      Set_Max_Length (Duplicate.New_Entry, 0);
      Set_Text (Duplicate.New_Entry, "");
      Set_Visibility (Duplicate.New_Entry, True);
      Attach (Duplicate.Table, Duplicate.New_Entry, 1, 2, 2, 3, 0, 0, 0, 0);

      Gtk_New_Hseparator (Duplicate.Hseparator);
      Attach
        (Duplicate.Table,
         Duplicate.Hseparator,
         0,
         2,
         3,
         4,
         Fill,
         Expand or Fill,
         0,
         0);

      Gtk_New_Hbox (Duplicate.Hbox, True, 0);
      Set_Border_Width (Duplicate.Hbox, 9);
      Attach (Duplicate.Table, Duplicate.Hbox, 0, 2, 4, 5, 0, 0, 0, 0);

      Gtk_New (Duplicate.Ok_Button, To_String (Lb_Close (Current_Language)));
      Set_Usize (Duplicate.Ok_Button, 90, 25);
      Button_Callback.Connect
        (Duplicate.Ok_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'Access));
      Pack_Start (Duplicate.Hbox, Duplicate.Ok_Button, False, False, 0);

      Gtk_New_Vseparator (Duplicate.Vseparator);
      Pack_Start (Duplicate.Hbox, Duplicate.Vseparator, False, False, 0);

      Gtk_New
        (Duplicate.Duplicate_Button,
         To_String (Lb_Duplicate (Current_Language)));
      Set_Usize (Duplicate.Duplicate_Button, 90, 25);
      Button_Callback.Connect
        (Duplicate.Duplicate_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Duplicate_Pressed'Access));
      Pack_Start
        (Duplicate.Hbox,
         Duplicate.Duplicate_Button,
         False,
         False,
         0);

      Gtk_New_Vseparator (Duplicate.Vseparator);
      Pack_Start (Duplicate.Hbox, Duplicate.Vseparator, False, False, 0);

      Gtk_New
        (Duplicate.Cancel_Button,
         To_String (Lb_Cancel (Current_Language)));
      Set_Usize (Duplicate.Cancel_Button, 90, 25);
      Button_Callback.Connect
        (Duplicate.Cancel_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'Access));
      Pack_Start (Duplicate.Hbox, Duplicate.Cancel_Button, False, False, 0);

   end Initialize;

end Duplicate_Pkg;
