------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Translate;              use Translate;
with Gtk;                    use Gtk;
with Gtk.Widget;             use Gtk.Widget;
with Gtk.Enums;              use Gtk.Enums;
with Callbacks_Cheddar;      use Callbacks_Cheddar;
with Project;                use Project;
with Systems;                use Systems;
with Ada.IO_Exceptions;      use Ada.IO_Exceptions;
with Ada.Strings.Unbounded;  use Ada.Strings.Unbounded;
with Mmi_Tools;              use Mmi_Tools;
with Framework_Config;       use Framework_Config;
with GNAT.Current_Exception; use GNAT.Current_Exception;

package body Export_Aadl_Selection_Pkg is

   procedure On_Ok_Pressed (Object : access Gtk_Button_Record'Class) is
      File : Unbounded_String;
   begin
      File := To_Unbounded_String (Get_Filename (export_aadl_selection));

      begin

         Systems.Write_To_Aadl_File (Sys, File);

      exception
         when Ada.IO_Exceptions.Name_Error =>
            Show_Message_Box
              (Lb_Can_Not_Save_File (Current_Language) &
               To_Unbounded_String (", Name_Error"));
         when Ada.IO_Exceptions.Status_Error =>
            Show_Message_Box
              (Lb_Can_Not_Save_File (Current_Language) &
               To_Unbounded_String (", Status_Error"));
         when Ada.IO_Exceptions.Mode_Error =>
            Show_Message_Box
              (Lb_Can_Not_Save_File (Current_Language) &
               To_Unbounded_String (", Mode_Error"));
         when Ada.IO_Exceptions.Use_Error =>
            Show_Message_Box
              (Lb_Can_Not_Save_File (Current_Language) &
               To_Unbounded_String (", Use_Error"));
         when Ada.IO_Exceptions.Device_Error =>
            Show_Message_Box
              (Lb_Can_Not_Save_File (Current_Language) &
               To_Unbounded_String (", Device_Error"));
         when Ada.IO_Exceptions.End_Error =>
            Show_Message_Box
              (Lb_Can_Not_Save_File (Current_Language) &
               To_Unbounded_String (", End_Error"));
         when Ada.IO_Exceptions.Data_Error =>
            Show_Message_Box
              (Lb_Can_Not_Save_File (Current_Language) &
               To_Unbounded_String (", Data_Error"));
         when Ada.IO_Exceptions.Layout_Error =>
            Show_Message_Box
              (Lb_Can_Not_Save_File (Current_Language) &
               To_Unbounded_String (", Layout_Error"));
         when others =>
            Show_Message_Box
              (Lb_Can_Not_Save_File (Current_Language) &
               " ; Exception name    : " &
               Exception_Name &
               " ; " &
               "Exception message : " &
               Exception_Message);

      end;

      Destroy (export_aadl_selection);
   end On_Ok_Pressed;

   procedure On_cancel_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      Destroy (Get_Toplevel (Gtk_Widget (Object)));
   end On_cancel_Pressed;

   procedure Gtk_New
     (export_aadl_selection : out export_aadl_selection_Access)
   is

   begin
      export_aadl_selection := new export_aadl_selection_Record;
      Export_Aadl_Selection_Pkg.Initialize (export_aadl_selection);
   end Gtk_New;

   procedure Initialize
     (export_aadl_selection : access export_aadl_selection_Record'Class)
   is

   begin
      Gtk.File_Selection.Initialize
        (export_aadl_selection,
         To_String
            (Lb_Save (Current_Language) &
             To_Unbounded_String (" ") &
             Lb_File (Current_Language)));
      Set_Show_File_Op_Buttons (export_aadl_selection, True);
      Set_Border_Width (export_aadl_selection, 10);
      Set_Title
        (export_aadl_selection,
         To_String
            (Lb_Save (Current_Language) &
             To_Unbounded_String (" ") &
             Lb_File (Current_Language)));
      Set_Policy (export_aadl_selection, False, True, False);
      Set_Position (export_aadl_selection, Win_Pos_Center_Always);
      Set_Modal (export_aadl_selection, True);

      export_aadl_selection.Ok_Button :=
         Get_Ok_Button (export_aadl_selection);
      Set_Flags (export_aadl_selection.Ok_Button, Can_Default);
      Button_Callback.Connect
        (export_aadl_selection.Ok_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'Access));

      export_aadl_selection.Cancel_Button :=
         Get_Cancel_Button (export_aadl_selection);
      Set_Flags (export_aadl_selection.Cancel_Button, Can_Default);
      Button_Callback.Connect
        (export_aadl_selection.Cancel_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_cancel_Pressed'Access));

   end Initialize;

end Export_Aadl_Selection_Pkg;
