------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Gtk;                   use Gtk;
with Gtk.Widget;            use Gtk.Widget;
with Gtk.Enums;             use Gtk.Enums;
with Callbacks_Cheddar;     use Callbacks_Cheddar;
with Message_Pkg.Callbacks; use Message_Pkg.Callbacks;

package body Message_Pkg is

   procedure Gtk_New
     (Message : out Message_Access;
      Msg     : in Unbounded_String)
   is

   begin
      Message := new Message_Record;
      Message_Pkg.Initialize (Message, Msg);
   end Gtk_New;

   procedure Initialize
     (Message : access Message_Record'Class;
      Msg     : in Unbounded_String)
   is

   begin
      Gtk.Window.Initialize (Message, Window_Dialog);
      Set_Title (Message, "Message");
      Set_Policy (Message, True, True, True);
      Set_Position (Message, Win_Pos_Center_Always);
      Set_Modal (Message, True);
      Set_Flags (Message, Can_Default);
      Grab_Focus (Message);

      Gtk_New (Message.Alignment, 0.5, 0.5, 1.0, 1.0);
      Set_Sensitive (Message.Alignment, True);
      Add (Message, Message.Alignment);

      Gtk_New_Vbox (Message.Vbox, False, 0);
      Add (Message.Alignment, Message.Vbox);

      Gtk_New (Message.Msg, To_String (Msg));
      Set_Alignment (Message.Msg, 0.5, 0.5);
      Set_Padding (Message.Msg, 10, 10);
      Set_Justify (Message.Msg, Justify_Center);
      Set_Line_Wrap (Message.Msg, True);
      Pack_Start (Message.Vbox, Message.Msg, True, True, 0);

      Gtk_New_Hseparator (Message.Separator);
      Pack_Start (Message.Vbox, Message.Separator, True, True, 0);

      Gtk_New_Hbox (Message.Hbox, True, 6);
      Set_Border_Width (Message.Hbox, 15);
      Pack_Start (Message.Vbox, Message.Hbox, True, True, 0);

      Gtk_New (Message.Button, "Ok");
      Set_Usize (Message.Button, 70, 25);
      Pack_Start (Message.Hbox, Message.Button, False, False, 0);

      Button_Callback.Connect
        (Message.Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'Access));

   end Initialize;

end Message_Pkg;
