------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Message_Pkg;         use Message_Pkg;
with Gtk.Menu;            use Gtk.Menu;
with Gtk.Radio_Menu_Item; use Gtk.Radio_Menu_Item;

package body Mmi_Tools is

   procedure Inner_Scheduler_Table_Name (Items : in out Strings_Table) is

   begin
      Add (Items, To_Unbounded_String ("Earliest Deadline First"));
      Add (Items, To_Unbounded_String ("Least Laxity First"));
      Add (Items, To_Unbounded_String ("Rate Monotonic"));
      Add (Items, To_Unbounded_String ("Deadline Monotonic"));
      Add
        (Items,
         To_Unbounded_String ("POSIX 1003.1b/Highest Priority First"));
      Add
        (Items,
         To_Unbounded_String
            ("Time sharing based on waiting time (Linux scheduler)"));
      Add (Items, To_Unbounded_String ("Round robin (with quantum)"));
      Add
        (Items,
         To_Unbounded_String ("Maximum Urgency First based on laxity"));
      Add
        (Items,
         To_Unbounded_String ("Maximum Urgency First based on deadline"));
      Add (Items, To_Unbounded_String ("D-Over/Stable EDF"));
      Add (Items, To_Unbounded_String ("Time sharing based on cpu usage"));
      Add (Items, To_Unbounded_String ("Pipeline user-defined scheduler"));
      Add (Items, To_Unbounded_String ("Automata user-defined scheduler"));
      Add (Items, To_Unbounded_String ("Compiled user-defined scheduler"));
   end Inner_Scheduler_Table_Name;

   procedure Build_Local_Scheduler_Table_Name
     (Items : in out Strings_Table)
   is
   begin
      Add (Items, To_Unbounded_String ("No local scheduling protocol"));
      Inner_Scheduler_Table_Name (Items);
   end Build_Local_Scheduler_Table_Name;

   procedure Build_Global_Scheduler_Table_Name
     (Items : in out Strings_Table)
   is

   begin
      Inner_Scheduler_Table_Name (Items);
      Add (Items, To_Unbounded_String ("Hierarchical scheduling : cyclic"));
      Add
        (Items,
         To_Unbounded_String ("Hierarchical scheduling : round robin"));
      Add
        (Items,
         To_Unbounded_String ("Hierarchical scheduling : fixed priority"));
   end Build_Global_Scheduler_Table_Name;

   procedure Show_Message_Box (Msg : Unbounded_String) is
   begin
      Gtk_New (Message, Unbounded_Lf & Msg & Unbounded_Lf);
      Show_All (Message);
   end Show_Message_Box;

   procedure Show_Message_Box (Msg : String) is
   begin
      Show_Message_Box (To_Unbounded_String (Msg));
   end Show_Message_Box;

   procedure Build_Option_Menu
     (Omenu   : out Gtk.Option_Menu.Gtk_Option_Menu;
      Gr      : out Widget_SList.GSlist;
      Items   : Strings_Table;
      History : Gint;
      Cb      : Widget_Handler.Marshallers.Void_Marshaller.Handler)
   is
      Menu      : Gtk_Menu;
      Menu_Item : Gtk_Radio_Menu_Item;

   begin
      Gtk.Option_Menu.Gtk_New (Omenu);
      Gtk_New (Menu);

      for I in  0 .. Items.Nb_Entries - 1 loop
         if (Items.Entries (I) /= Empty_String) then
            Gtk_New (Menu_Item, Gr, To_String (Items.Entries (I)));
            Widget_Handler.Object_Connect
              (Menu_Item,
               "activate",
               Widget_Handler.To_Marshaller (Cb),
               Slot_Object => Menu_Item);
            Gr := Group (Menu_Item);
            Append (Menu, Menu_Item);
            if Gint (I) = History then
               Set_Active (Menu_Item, True);
            end if;
            Show (Menu_Item);
         end if;
      end loop;
      Gtk.Option_Menu.Set_Menu (Omenu, Menu);
      Gtk.Option_Menu.Set_History (Omenu, History);
   end Build_Option_Menu;

end Mmi_Tools;
