------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Translate;                         use Translate;
with Framework_Config;                  use Framework_Config;
with Networks_Parameters_Pkg.Callbacks; use Networks_Parameters_Pkg.Callbacks;
with Callbacks_Cheddar;                 use Callbacks_Cheddar;
with Gtk;                               use Gtk;
with Unbounded_Strings;                 use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;
with Project;                           use Project;

package body Networks_Parameters_Pkg is

   procedure Gtk_New (Networks_Parameters : out Networks_Parameters_Access) is

   begin

      Networks_Parameters := new Networks_Parameters_Record;
      Networks_Parameters_Pkg.Initialize (Networks_Parameters);

   end Gtk_New;

   procedure Initialize
     (Networks_Parameters : access Networks_Parameters_Record'Class)
   is

   begin

      Networks_Parameters.Undo_Networks := Sys.Networks;

      Gtk.Window.Initialize (Networks_Parameters, Window_Toplevel);
      Set_Policy (Networks_Parameters, True, True, True);
      Set_Title
        (Networks_Parameters,
         To_String (Lb_Add_Network (Current_Language)));
      Set_Position (Networks_Parameters, Win_Pos_Center_Always);
      Set_Modal (Networks_Parameters, True);
      Grab_Focus (Networks_Parameters);

      Gtk.Box.Gtk_New_Vbox (Networks_Parameters.Vbox_Main, False, 10);
      Gtk.Box.Gtk_New_Hbox (Networks_Parameters.Hbox_Name_Type, False, 10);
      Gtk.Box.Gtk_New_Hbox (Networks_Parameters.Hbox_Add_Cancel, False, 10);

      Add (Networks_Parameters, Networks_Parameters.Vbox_Main);
      Add (Networks_Parameters.Vbox_Main, Networks_Parameters.Hbox_Name_Type);

      ------------------------------------------------------------------------
      --Name and Type Fields
      ------------------------------------------------------------------------

      -- Using a Table of 2 columns and 2 lines for name / type
      Gtk.Table.Gtk_New (Networks_Parameters.Table_Name_Type, 3, 2, False);
      --(lines,columns)
      Add
        (Networks_Parameters.Hbox_Name_Type,
         Networks_Parameters.Table_Name_Type);

      -- Widgets in Table Name_Type

      Gtk.GEntry.Gtk_New (Networks_Parameters.Networks_Name, 50);
      --Name limited to 50 characters
      Gtk.Label.Gtk_New
        (Networks_Parameters.Name_Label,
         To_String (Lb_Networks_Name (Current_Language) & Lb_Colon));
      Gtk.Label.Gtk_New
        (Networks_Parameters.Type_Label,
         To_String (Lb_Networks_Type (Current_Language) & Lb_Colon));
      Gtk.Combo.Gtk_New (Networks_Parameters.Networks_Type);
      --Combo Box for the networks type

      Gtk.Label.Gtk_New
        (Networks_Parameters.File_Label,
         To_String (Lb_File_Name (Current_Language) & Lb_Colon));
      Gtk.GEntry.Gtk_New (Networks_Parameters.Entry_File_Name, 50);
      Gtk.Box.Gtk_New_Hbox (Networks_Parameters.Hbox_File, False, 10);
      Add (Networks_Parameters.Hbox_File, Networks_Parameters.File_Label);
      Add
        (Networks_Parameters.Hbox_File,
         Networks_Parameters.Entry_File_Name);
      Add (Networks_Parameters.Vbox_Main, Networks_Parameters.Hbox_File);

      Gtk.Label.Gtk_New (Networks_Parameters.File_Label, "");
      Gtk.Button.Gtk_New
        (Networks_Parameters.Read_Button,
         To_String (Lb_Read (Current_Language)));
      Gtk.Box.Gtk_New_Hbox (Networks_Parameters.Hbox_File, False, 10);
      Add (Networks_Parameters.Hbox_File, Networks_Parameters.File_Label);
      Add (Networks_Parameters.Hbox_File, Networks_Parameters.Read_Button);
      Add (Networks_Parameters.Vbox_Main, Networks_Parameters.Hbox_File);

      --Network type , list

      Set_Editable (Get_Entry (Networks_Parameters.Networks_Type), False);
      String_List.Append (Networks_Parameters.Type_Items, "Bounded delay");
      String_List.Append (Networks_Parameters.Type_Items, "Parametric delay");
      Set_Popdown_Strings
        (Networks_Parameters.Networks_Type,
         Networks_Parameters.Type_Items);

      -- Attach Table name / type

      Attach_Defaults
        (Networks_Parameters.Table_Name_Type,
         Networks_Parameters.Name_Label,
         0,
         1,
         0,
         1);
      Attach_Defaults
        (Networks_Parameters.Table_Name_Type,
         Networks_Parameters.Networks_Name,
         1,
         2,
         0,
         1);

      Attach_Defaults
        (Networks_Parameters.Table_Name_Type,
         Networks_Parameters.Type_Label,
         0,
         1,
         1,
         2);
      Attach_Defaults
        (Networks_Parameters.Table_Name_Type,
         Networks_Parameters.Networks_Type,
         1,
         2,
         1,
         2);

      -- Add, Close and Cancel

      Add
        (Networks_Parameters.Vbox_Main,
         Networks_Parameters.Hbox_Add_Cancel);

      Gtk.Button.Gtk_New
        (Networks_Parameters.Add_Button,
         To_String (Lb_Add (Current_Language)));
      Gtk.Button.Gtk_New
        (Networks_Parameters.Cancel_Button,
         To_String (Lb_Cancel (Current_Language)));
      Gtk.Button.Gtk_New
        (Networks_Parameters.Close_Button,
         To_String (Lb_Close (Current_Language)));

      Button_Callback.Connect
        (Networks_Parameters.Cancel_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'Access));
      Button_Callback.Connect
        (Networks_Parameters.Add_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Add_Pressed'Access));
      Button_Callback.Connect
        (Networks_Parameters.Close_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Close_Pressed'Access));

      Button_Callback.Connect
        (Networks_Parameters.Read_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Read_Pressed'Access));

      Add
        (Networks_Parameters.Hbox_Add_Cancel,
         Networks_Parameters.Close_Button);
      Add
        (Networks_Parameters.Hbox_Add_Cancel,
         Networks_Parameters.Add_Button);
      Add
        (Networks_Parameters.Hbox_Add_Cancel,
         Networks_Parameters.Cancel_Button);

   end Initialize;

end Networks_Parameters_Pkg;
