------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Translate;              use Translate;
with Ada.Strings.Unbounded;  use Ada.Strings.Unbounded;
with Gtk;                    use Gtk;
with Gtk.Widget;             use Gtk.Widget;
with Gtk.Enums;              use Gtk.Enums;
with Callbacks_Cheddar;      use Callbacks_Cheddar;
with Project;                use Project;
with Xml_Parsers;            use Xml_Parsers;
with Systems;                use Systems;
with Ada.IO_Exceptions;      use Ada.IO_Exceptions;
with Mmi_Tools;              use Mmi_Tools;
with Graph_Window_Pkg;       use Graph_Window_Pkg;
with My_Canvas;              use My_Canvas;
with Framework_Config;       use Framework_Config;
with GNAT.Current_Exception; use GNAT.Current_Exception;

package body Open_File_Selection_Pkg is

   procedure On_Ok_Pressed (Object : access Gtk_Button_Record'Class) is

   begin
      Xml_Project_File_Name :=
         To_Unbounded_String (Get_Filename (Open_File_Selection));

      begin

         Systems.Read_From_Xml_File
           (Sys,
            Project_File_Dir_List,
            Xml_Project_File_Name);

         -- Load Item x/y positions
         --
         if Graph_Window /= null then
            Rebuild_Canvas (Graph_Window);
            Set_Coordinates (Graph_Window.Buffer, Sys);
         end if;

         Is_Saved := True;

      exception
         when Xml_Read_Error =>
            Show_Message_Box
              (Lb_Can_Not_Read_File (Current_Language) &
               To_Unbounded_String (" ; Xml Fatal Error ; ") &
               Exception_Message);
         when Ada.IO_Exceptions.Name_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (", Name_Error"));
         when Ada.IO_Exceptions.Status_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (", Status_Error"));
         when Ada.IO_Exceptions.Mode_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (", Mode_Error"));
         when Ada.IO_Exceptions.Use_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (", Use_Error"));
         when Ada.IO_Exceptions.Device_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (", Device_Error"));
         when Ada.IO_Exceptions.End_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (", End_Error"));
         when Ada.IO_Exceptions.Data_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (", Data_Error"));
         when Ada.IO_Exceptions.Layout_Error =>
            Show_Message_Box
              (Lb_Can_Not_Open_File (Current_Language) &
               To_Unbounded_String (", Layout_Error"));
         when others =>
            Show_Message_Box
              (Lb_Can_Not_Read_File (Current_Language) &
               " ; " &
               Exception_Name &
               " ; " &
               Exception_Message);

      end;

      Destroy (Open_File_Selection);
   end On_Ok_Pressed;

   procedure On_cancel_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      Destroy (Get_Toplevel (Gtk_Widget (Object)));
   end On_cancel_Pressed;

   procedure Gtk_New (Open_File_Selection : out Open_File_Selection_Access) is

   begin
      Open_File_Selection := new Open_File_Selection_Record;
      Open_File_Selection_Pkg.Initialize (Open_File_Selection);
   end Gtk_New;

   procedure Initialize
     (Open_File_Selection : access Open_File_Selection_Record'Class)
   is

   begin
      Gtk.File_Selection.Initialize
        (Open_File_Selection,
         To_String
            (Lb_Open_Xml (Current_Language) &
             To_Unbounded_String (" ") &
             Lb_File (Current_Language)));
      Set_Show_File_Op_Buttons (Open_File_Selection, True);
      Set_Border_Width (Open_File_Selection, 10);
      Set_Title
        (Open_File_Selection,
         To_String
            (Lb_Open_Xml (Current_Language) &
             To_Unbounded_String (" ") &
             Lb_File (Current_Language)));
      Set_Policy (Open_File_Selection, False, True, False);
      Set_Position (Open_File_Selection, Win_Pos_Center_Always);
      Set_Modal (Open_File_Selection, True);

      Open_File_Selection.Ok_Button := Get_Ok_Button (Open_File_Selection);
      Set_Flags (Open_File_Selection.Ok_Button, Can_Default);
      Button_Callback.Connect
        (Open_File_Selection.Ok_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'Access));

      Open_File_Selection.Cancel_Button :=
         Get_Cancel_Button (Open_File_Selection);
      Set_Flags (Open_File_Selection.Cancel_Button, Can_Default);
      Button_Callback.Connect
        (Open_File_Selection.Cancel_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_cancel_Pressed'Access));

   end Initialize;

end Open_File_Selection_Pkg;
