------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Gtk;                       use Gtk;
with Gtk.Main;                  use Gtk.Main;
with Gtkada.Dialogs;            use Gtkada.Dialogs;
with Open_File_Selection_Pkg;   use Open_File_Selection_Pkg;
with Save_File_Selection_Pkg;   use Save_File_Selection_Pkg;
with Import_Aadl_Selection_Pkg; use Import_Aadl_Selection_Pkg;
with Export_Aadl_Selection_Pkg; use Export_Aadl_Selection_Pkg;
with Root_Window_Pkg;           use Root_Window_Pkg;
with Root_Window_Pkg.Callbacks; use Root_Window_Pkg.Callbacks;
with Call_Scheduling_Framework; use Call_Scheduling_Framework;
with My_Canvas;                 use My_Canvas;
with Graph_Window_Pkg;          use Graph_Window_Pkg;
with Framework_Config;          use Framework_Config;
with Translate;                 use Translate;
with Multiprocessor_Services;   use Multiprocessor_Services;
use Multiprocessor_Services.Scheduling_Result_Per_Processor;

package body Project is

   procedure Do_New is
   begin

      if Root_Window /= null then

         if (not Is_Saved) then
            if (not Is_New) then
               if Message_Dialog
                     (To_String
                         (Lb_Not_Saved_Project (Current_Language) &
                          ASCII.LF &
                          Lb_Continue (Current_Language)),
                      Confirmation,
                      Button_Yes or Button_Cancel,
                      Button_Cancel, -- default
                      "", -- help message
                      "") /=
                  Button_Yes
               then
                  return;
               end if;
            end if;
         end if;
         Clear_Work_Space;

      end if;

      Initialize (Sys);
      Xml_Project_File_Name := To_Unbounded_String ("");
      Initialize (Aadl_Project_File_Name);
      Xml_Event_Table_File_Name :=
         To_Unbounded_String ("default_event_table.xml");

      Is_Saved := False;
      Is_New   := True;

      Event_Table_File_Name := Empty_String;
      Initialize (Call_Scheduling_Framework.Sched.all);

   end Do_New;

   procedure Do_Open is
   begin

      if (not Is_Saved) then
         if (not Is_New) then
            if Message_Dialog
                  (To_String
                      (Lb_Not_Saved_Project (Current_Language) &
                       ASCII.LF &
                       Lb_Continue (Current_Language)),
                   Confirmation,
                   Button_Yes or Button_Cancel,
                   Button_Cancel, -- default
                   "", -- help message
                   "") /=
               Button_Yes
            then
               return;
            end if;
         end if;
      end if;

      Clear_Work_Space;

      Gtk_New (Open_File_Selection);
      Show_All (Open_File_Selection);

   end Do_Open;

   procedure Do_Save is
   begin

      if Graph_Window /= null then
         Get_Coordinates (Graph_Window.Buffer, Sys);
      end if;

      if (Xml_Project_File_Name /= To_Unbounded_String ("")) then
         Systems.Write_To_Xml_File (Sys, Xml_Project_File_Name);

         Is_Saved := True;
      else
         Gtk_New (save_File_Selection);
         Show_All (save_File_Selection);
      end if;

   end Do_Save;

   procedure Do_Aadl_Export is
   begin
      Gtk_New (export_aadl_selection);
      Show_All (export_aadl_selection);

   end Do_Aadl_Export;

   procedure Do_Aadl_Import is
   begin

      if (not Is_Saved) then
         if (not Is_New) then
            if Message_Dialog
                  (To_String
                      (Lb_Not_Saved_Project (Current_Language) &
                       ASCII.LF &
                       Lb_Continue (Current_Language)),
                   Confirmation,
                   Button_Yes or Button_Cancel,
                   Button_Cancel, -- default
                   "", -- help message
                   "") /=
               Button_Yes
            then
               return;
            end if;
         end if;
      end if;

      Clear_Work_Space;

      Gtk_New (import_aadl_selection);
      Show_All (import_aadl_selection);

   end Do_Aadl_Import;

   procedure Do_Save_As is
   begin
      Gtk_New (save_file_selection);
      Show_All (save_file_selection);
   end Do_Save_As;

   procedure Do_Exit is
   begin
      if (not Is_Saved) then
         if (not Is_New) then
            if Message_Dialog
                  (To_String
                      (Lb_Not_Saved_Project (Current_Language) &
                       ASCII.LF &
                       Lb_Continue (Current_Language)),
                   Confirmation,
                   Button_Yes or Button_Cancel,
                   Button_Cancel, -- default
                   "", -- help message
                   "") =
               Button_Yes
            then
               Gtk.Main.Gtk_Exit (0);
            else
               return;
            end if;
         end if;
      end if;
      Gtk.Main.Gtk_Exit (0);
   end Do_Exit;

end Project;
