------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Systems;               use Systems;
with Unbounded_Strings;     use Unbounded_Strings;
use Unbounded_Strings.Unbounded_String_List_Package;

package Project is

   -- The working project
   --
   Sys : System;

   -- List of directory where to read AADL or XML files
   --
   Project_File_Dir_List : Unbounded_String_List;

   -- File name of the working XML/AADL project files
   --
   Xml_Project_File_Name  : Unbounded_String;
   Aadl_Project_File_Name : Unbounded_String_List;

   -- File name of the working XML event table file
   --
   Xml_Event_Table_File_Name : Unbounded_String;

   -- Is the project modified ? should we have to save it ?
   --
   Is_Saved : Boolean;
   Is_New   : Boolean;

   -- Processor name on which we should compute something
   --
   Last_Selected_Processor : Unbounded_String;

   -- The name of the file in which the event table has to be saved
   --
   Event_Table_File_Name : Unbounded_String;

   -- Sub programs do project management
   --
   procedure Do_New;
   procedure Do_Open;
   procedure Do_Save;
   procedure Do_Save_As;
   procedure Do_Exit;
   procedure Do_Aadl_Export;
   procedure Do_Aadl_Import;

end Project;
