------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Framework_Config;       use Framework_Config;
with Translate;              use Translate;
with Gtk;                    use Gtk;
with Gtk.Enums;              use Gtk.Enums;
with Callbacks_Cheddar;      use Callbacks_Cheddar;
with Question_Pkg.Callbacks; use Question_Pkg.Callbacks;

package body Question_Pkg is

   procedure Gtk_New
     (Question      : out Question_Access;
      Title         : Unbounded_String;
      The_Question  : Unbounded_String;
      Initial_Value : Unbounded_String;
      Call          : Question_Callback_Type)
   is
   begin
      Question := new Question_Record;
      Question_Pkg.Initialize
        (Question,
         Title,
         The_Question,
         Initial_Value,
         Call);
   end Gtk_New;

   procedure Initialize
     (Question      : access Question_Record'Class;
      Title         : Unbounded_String;
      The_Question  : Unbounded_String;
      Initial_Value : Unbounded_String;
      Call          : Question_Callback_Type)
   is
   begin

      Question.Calling_Function := Call;

      Gtk.Window.Initialize (Question, Window_Dialog);
      Set_Title (Question, To_String (Title));
      Set_Policy (Question, False, True, False);
      Set_Position (Question, Win_Pos_Center_Always);
      Set_Modal (Question, True);

      Gtk_New_Vbox (Question.Vbox, False, 0);
      Add (Question, Question.Vbox);

      Gtk_New_Hbox (Question.Hbox1, False, 0);
      Set_Border_Width (Question.Hbox1, 16);
      Pack_Start (Question.Vbox, Question.Hbox1, True, True, 0);

      Gtk_New (Question.Question_Label, To_String (The_Question));
      Set_Alignment (Question.Question_Label, 0.5, 0.5);
      Set_Padding (Question.Question_Label, 0, 0);
      Set_Justify (Question.Question_Label, Justify_Left);
      Set_Line_Wrap (Question.Question_Label, False);
      Pack_Start (Question.Hbox1, Question.Question_Label, False, False, 0);

      Gtk_New (Question.Response_Entry);
      Set_Editable (Question.Response_Entry, True);
      Set_Max_Length (Question.Response_Entry, 0);
      Set_Text (Question.Response_Entry, To_String (Initial_Value));
      Set_Visibility (Question.Response_Entry, True);
      Pack_Start (Question.Hbox1, Question.Response_Entry, True, True, 0);

      Gtk_New_Hseparator (Question.Separator);
      Pack_Start (Question.Vbox, Question.Separator, True, True, 0);

      Gtk_New_Hbox (Question.Hbox2, True, 0);
      Set_Border_Width (Question.Hbox2, 9);
      Pack_Start (Question.Vbox, Question.Hbox2, True, True, 0);

      Gtk_New (Question.Ok_Button, "Ok");
      Set_Usize (Question.Ok_Button, 70, 25);
      Button_Callback.Connect
        (Question.Ok_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'Access));
      Pack_Start (Question.Hbox2, Question.Ok_Button, False, False, 0);

      Gtk_New
        (Question.Cancel_Button,
         To_String (Lb_Cancel (Current_Language)));
      Set_Usize (Question.Cancel_Button, 70, 25);
      Button_Callback.Connect
        (Question.Cancel_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'Access));

      Pack_Start (Question.Hbox2, Question.Cancel_Button, False, False, 0);

   end Initialize;

end Question_Pkg;
