------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Gtk.Window;            use Gtk.Window;
with Gtk.Box;               use Gtk.Box;
with Gtk.Label;             use Gtk.Label;
with Gtk.GEntry;            use Gtk.GEntry;
with Gtk.Separator;         use Gtk.Separator;
with Gtk.Button;            use Gtk.Button;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;

package Question_Pkg is

   type Question_Callback_Type is access function
     (Value : in Unbounded_String)
   return     Boolean;

   type Question_Record is new Gtk_Window_Record with private;
   type Question_Access is access all Question_Record'Class;

   procedure Gtk_New
     (Question      : out Question_Access;
      Title         : Unbounded_String;
      The_Question  : Unbounded_String;
      Initial_Value : Unbounded_String;
      Call          : Question_Callback_Type);

   procedure Initialize
     (Question      : access Question_Record'Class;
      Title         : Unbounded_String;
      The_Question  : Unbounded_String;
      Initial_Value : Unbounded_String;
      Call          : Question_Callback_Type);

   Question : Question_Access;

private

   type Question_Record is new Gtk_Window_Record with record
      Vbox      : Gtk_Vbox;
      Hbox1     : Gtk_Hbox;
      Hbox2     : Gtk_Hbox;
      Separator : Gtk_Hseparator;

      Question_Label : Gtk_Label;
      Response_Entry : Gtk_Entry;

      Cancel_Button : Gtk_Button;
      Ok_Button     : Gtk_Button;

      Calling_Function : Question_Callback_Type;
   end record;

end Question_Pkg;
