------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Framework_Config;                use Framework_Config;
with Translate;                       use Translate;
with Project;                         use Project;
with References;
with Manual;
with Address_Spaces;                  use Address_Spaces;
with Address_Space_Set;               use Address_Space_Set;
use Address_Space_Set.Generic_Address_Space_Set;
with Processors;                      use Processors;
with Processor_Set;                   use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Resources;                       use Resources;
with Resource_Set;                    use Resource_Set;
use Resource_Set.Generic_Resource_Set;
with Networks;                        use Networks;
with Network_Set;                     use Network_Set;
use Network_Set.Generic_Network_Set;
with Time_Units;                      use Time_Units;
with Tasks;                           use Tasks;
with Task_Set;                        use Task_Set;
use Task_Set.Generic_Task_Set;
with Scheduling_Sequences;            use Scheduling_Sequences;
use Scheduling_Sequences.Density_Table_Package;
use Scheduling_Sequences.Densities_Table_Package;
with Buffers;                         use Buffers;
with Buffer_Set;                      use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Ada.Strings.Unbounded;           use Ada.Strings.Unbounded;
with Messages;                        use Messages;
with Message_Set;                     use Message_Set;
use Message_Set.Generic_Message_Set;
with Event_Analyzers;                 use Event_Analyzers;
with Event_Analyzer_Set;              use Event_Analyzer_Set;
use Event_Analyzer_Set.Generic_Event_Analyzer_Set;
with Call_Framework;                  use Call_Framework;
use Call_Framework.Framework_Request_Package;
use Call_Framework.Framework_Response_Package;
with Call_Scheduling_Framework;       use Call_Scheduling_Framework;
with Call_Memory_Framework;           use Call_Memory_Framework;
use Call_Memory_Framework.Buffer_Result_Package;
with Parameters;                      use Parameters;
with Parameters.extended;             use Parameters.extended;
use Parameters.extended.Framework_Parameters_Table_Package;
with Glib;                            use Glib;
with Mmi_Tools;                       use Mmi_Tools;
with Display_Resources_Set_Pkg;       use Display_Resources_Set_Pkg;
with Display_Messages_Set_Pkg;        use Display_Messages_Set_Pkg;
with Display_Networks_Set_Pkg;        use Display_Networks_Set_Pkg;
with Display_Tasks_Set_Pkg;           use Display_Tasks_Set_Pkg;
with Display_Address_Spaces_Set_Pkg;  use Display_Address_Spaces_Set_Pkg;
with Display_Buffers_Set_Pkg;         use Display_Buffers_Set_Pkg;
with Display_Processors_Set_Pkg;      use Display_Processors_Set_Pkg;
with Display_Event_Analyzers_Set_Pkg; use Display_Event_Analyzers_Set_Pkg;
with Update_Event_Analyzers_Set_Pkg;  use Update_Event_Analyzers_Set_Pkg;
with Update_Buffers_Set_Pkg;          use Update_Buffers_Set_Pkg;
with Update_Tasks_Set_Pkg;            use Update_Tasks_Set_Pkg;
with Update_Address_Spaces_Set_Pkg;   use Update_Address_Spaces_Set_Pkg;
with Update_Networks_Set_Pkg;         use Update_Networks_Set_Pkg;
with Update_Processors_Set_Pkg;       use Update_Processors_Set_Pkg;
with Update_Resources_Set_Pkg;        use Update_Resources_Set_Pkg;
with Update_Messages_Set_Pkg;         use Update_Messages_Set_Pkg;
with Draw_Scheduling_Pkg;             use Draw_Scheduling_Pkg;
with Scrolling_Draw_Buffer_Pkg;       use Scrolling_Draw_Buffer_Pkg;
with Scrolling_Text_Pkg;              use Scrolling_Text_Pkg;
with Question_Pkg;                    use Question_Pkg;
with Do_Scheduling_Simulation_Pkg;    use Do_Scheduling_Simulation_Pkg;
with Do_Scheduling_Feasibility_Pkg;   use Do_Scheduling_Feasibility_Pkg;
with Networks_Parameters_Pkg;         use Networks_Parameters_Pkg;
with Graph_Window_Pkg;                use Graph_Window_Pkg;
with Scheduling_Options_Pkg;          use Scheduling_Options_Pkg;
with Aadl_Options_Pkg;                use Aadl_Options_Pkg;
with Duplicate_Pkg;                   use Duplicate_Pkg;
with Schedule_From_To_Pkg;            use Schedule_From_To_Pkg;
with Draw_Time_Line_From_To_Pkg;      use Draw_Time_Line_From_To_Pkg;
with Event_Table_Analysis_Pkg;        use Event_Table_Analysis_Pkg;
with Systems;                         use Systems;
with open_Event_Table_Pkg;            use open_Event_Table_Pkg;
with Save_Event_Table_Pkg;            use Save_Event_Table_Pkg;
with Text_Tools;                      use Text_Tools;
with Multiprocessor_Services;         use Multiprocessor_Services;
use Multiprocessor_Services.Scheduling_Result_Per_Processor;

package body Root_Window_Pkg.Callbacks is

   use Gtk.Arguments;

   ----------------------------------------------------------------------
   --   Callbacks which call the framework
   ----------------------------------------------------------------------

   procedure Check_Buffer is

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin

      if (Get_Number_Of_Elements (Sys.Buffers) = 0) then
         Show_Message_Box (Lb_Define_Buffers_Before (Current_Language));
         return;
      end if;

      Initialize (A_Request);
      A_Request.Statement := Buffer_Feasibility_Tests;
      Add (Request_List, A_Request);

      Sequential_Framework_Request (Sys, Request_List, Response_List);

      Write_Title_And_Text (Root_Window.Text, Response_List);

   end Check_Buffer;

   procedure On_Draw_Buffer_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin

      if (Get_Number_Of_Elements (Sys.Buffers) = 0) then
         Show_Message_Box (Lb_Define_Buffers_Before (Current_Language));
      else

         -- Check if scheduling is computed or not
         --
         if Call_Scheduling_Framework.Sched.Nb_Entries = 0 then
            Show_Message_Box (Lb_Schedule_Before (Current_Language));
            return;
         end if;

         -- For each buffer, compute and draw its size from
         -- 0 to the base period
         --
         Initialize (A_Request);
         A_Request.Statement := Buffer_Scheduling_Simulation;
         Add (Request_List, A_Request);

         Sequential_Framework_Request (Sys, Request_List, Response_List);

         Write_Title_And_Text (Root_Window.Text, Response_List);

         -- Now, draw the result on buffer utilization
         --
         for I in  0 .. Buff.Nb_Entries - 1 loop

            Gtk_New
              (Scrolling_Draw_Buffer,
               To_String (Buff.Entries (I).Item.Name));
            Show_All (Scrolling_Draw_Buffer);
            Clear (Scrolling_Draw_Buffer);
            Draw
              (Scrolling_Draw_Buffer,
               Buff.Entries (I).Item,
               Buff.Entries (I).Data,
               Call_Scheduling_Framework.Scheduling_Base_Period,
               Time_Unit_Range (Buff.Entries (I).Data.Nb_Entries));
         end loop;

      end if;
   end On_Draw_Buffer_Activate;

   -- Resource tools : blocking time evaluation
   --

   procedure On_Blocking_Time_Display_Callback
     (Object : access Gtk_Menu_Item_Record'Class)
   is

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin
      if (Get_Number_Of_Elements (Sys.Resources) = 0) then
         Show_Message_Box (Lb_Define_Resources_Before (Current_Language));
      else
         Initialize (A_Request);
         A_Request.Statement := Scheduling_Feasibility_Compute_Blocking_Time;
         Add (Request_List, A_Request);

         Sequential_Framework_Request (Sys, Request_List, Response_List);

         Write_Title_And_Text (Root_Window.Text, Response_List);

      end if;
   end On_Blocking_Time_Display_Callback;

   procedure On_Blocking_Time_Update_Callback
     (Object : access Gtk_Menu_Item_Record'Class)
   is

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin
      if (Get_Number_Of_Elements (Sys.Resources) = 0) then
         Show_Message_Box (Lb_Define_Resources_Before (Current_Language));
      else
         Initialize (A_Request);
         A_Request.Statement :=
           Scheduling_Feasibility_Compute_And_Set_Blocking_Time;
         Add (Request_List, A_Request);

         Sequential_Framework_Request (Sys, Request_List, Response_List);

         Write_Title_And_Text (Root_Window.Text, Response_List);

         -- Do not forget to save project
         -- since tasks are modified !!
         --
         Is_New   := False;
         Is_Saved := False;

      end if;
   end On_Blocking_Time_Update_Callback;

   -- Precedency tools : deadline, jitter and priority modifications
   --
   procedure On_Chetto_Priority_Callback
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      null;
   end On_Chetto_Priority_Callback;

   procedure On_Chetto_Priority_Display_Callback
     (Object : access Gtk_Menu_Item_Record'Class)
   is

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin
      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));

      else
         Initialize (A_Request);
         A_Request.Statement := Dependency_Compute_Chetto_Blazewicz_Priority;
         Add (Request_List, A_Request);

         Sequential_Framework_Request (Sys, Request_List, Response_List);

         Write_Title_And_Text (Root_Window.Text, Response_List);

      end if;
   end On_Chetto_Priority_Display_Callback;

   procedure On_Chetto_Priority_Update_Callback
     (Object : access Gtk_Menu_Item_Record'Class)
   is

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin
      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));

      else
         Initialize (A_Request);
         A_Request.Statement := Dependency_Set_Chetto_Blazewicz_Priority;
         Add (Request_List, A_Request);

         Sequential_Framework_Request (Sys, Request_List, Response_List);

         Write_Title_And_Text (Root_Window.Text, Response_List);

         -- Do not forget to save project
         -- since tasks are modified !!
         --
         Is_New   := False;
         Is_Saved := False;

      end if;
   end On_Chetto_Priority_Update_Callback;

   procedure On_Chetto_Deadline_Callback
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      null;
   end On_Chetto_Deadline_Callback;

   procedure On_Chetto_Deadline_Display_Callback
     (Object : access Gtk_Menu_Item_Record'Class)
   is

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin
      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));

      else
         Initialize (A_Request);
         A_Request.Statement := Dependency_Compute_Chetto_Blazewicz_Deadline;
         Add (Request_List, A_Request);

         Sequential_Framework_Request (Sys, Request_List, Response_List);

         Write_Title_And_Text (Root_Window.Text, Response_List);

      end if;
   end On_Chetto_Deadline_Display_Callback;

   procedure On_Chetto_Deadline_Update_Callback
     (Object : access Gtk_Menu_Item_Record'Class)
   is

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin
      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));

      else
         Initialize (A_Request);
         A_Request.Statement := Dependency_Set_Chetto_Blazewicz_Deadline;
         Add (Request_List, A_Request);

         Sequential_Framework_Request (Sys, Request_List, Response_List);

         Write_Title_And_Text (Root_Window.Text, Response_List);

         -- Do not forget to save project
         -- since tasks are modified !!
         --
         Is_New   := False;
         Is_Saved := False;

      end if;
   end On_Chetto_Deadline_Update_Callback;

   procedure On_Jitter_From_Response_Time_Callback
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      null;
   end On_Jitter_From_Response_Time_Callback;

   procedure On_Jitter_From_Response_Time_Display_Callback
     (Object : access Gtk_Menu_Item_Record'Class)
   is

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin
      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));

      else
         Initialize (A_Request);
         A_Request.Statement :=
           Dependency_Compute_End_To_End_Response_Time_All_Steps;
         Add (Request_List, A_Request);

         Sequential_Framework_Request (Sys, Request_List, Response_List);

         Write_Title_And_Text (Root_Window.Text, Response_List);

      end if;
   end On_Jitter_From_Response_Time_Display_Callback;

   procedure On_Jitter_From_Response_Time_Update_Callback
     (Object : access Gtk_Menu_Item_Record'Class)
   is

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin
      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));

      else
         Initialize (A_Request);
         A_Request.Statement :=
           Dependency_Set_End_To_End_Response_Time_All_Steps;
         Add (Request_List, A_Request);

         Sequential_Framework_Request (Sys, Request_List, Response_List);

         Write_Title_And_Text (Root_Window.Text, Response_List);

         -- Do not forget to save project
         -- since tasks are modified !!
         --
         Is_New   := False;
         Is_Saved := False;

      end if;
   end On_Jitter_From_Response_Time_Update_Callback;

   procedure On_Jitter_From_Response_Time_Display_One_Step_Callback
     (Object : access Gtk_Menu_Item_Record'Class)
   is

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin
      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));

      else
         Initialize (A_Request);
         A_Request.Statement :=
           Dependency_Compute_End_To_End_Response_Time_One_Step;
         Add (Request_List, A_Request);

         Sequential_Framework_Request (Sys, Request_List, Response_List);

         Write_Title_And_Text (Root_Window.Text, Response_List);

      end if;
   end On_Jitter_From_Response_Time_Display_One_Step_Callback;

   procedure On_Jitter_From_Response_Time_Update_One_Step_Callback
     (Object : access Gtk_Menu_Item_Record'Class)
   is

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin

      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));

      else
         Initialize (A_Request);
         A_Request.Statement :=
           Dependency_Set_End_To_End_Response_Time_One_Step;
         Add (Request_List, A_Request);

         Sequential_Framework_Request (Sys, Request_List, Response_List);

         Write_Title_And_Text (Root_Window.Text, Response_List);

         -- Do not forget to save project
         -- since tasks are modified !!
         --
         Is_New   := False;
         Is_Saved := False;

      end if;
   end On_Jitter_From_Response_Time_Update_One_Step_Callback;

   -- Set priorities according to a fixed priority scheduler
   --

   function Rm_Priorities_Callback
     (Value : Unbounded_String)
      return  Boolean
   is
      My_Iterator   : Processors_Iterator;
      A_Processor   : Processor_Ptr;
      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin

      loop

         Current_Element (Sys.Processors, A_Processor, My_Iterator);

         if A_Processor.Name = Value then

            Initialize (A_Request);
            A_Request.Target    := Value;
            A_Request.Statement :=
              Scheduling_Set_Priorities_According_To_Rate_Monotonic;
            Add (Request_List, A_Request);

            Sequential_Framework_Request (Sys, Request_List, Response_List);

            Write_Title_And_Text (Root_Window.Text, Response_List);

            -- Do not forget to save project
            -- since tasks are modified !!
            --
            Is_New   := False;
            Is_Saved := False;

            return True;
         end if;

         exit when Is_Last_Element (Sys.Processors, My_Iterator);
         Next_Element (Sys.Processors, My_Iterator);

      end loop;

      Show_Message_Box (Lb_Processor_Not_Found (Current_Language));
      return False;

   end Rm_Priorities_Callback;

   function Dm_Priorities_Callback
     (Value : Unbounded_String)
      return  Boolean
   is
      My_Iterator   : Processors_Iterator;
      A_Processor   : Processor_Ptr;
      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin

      loop

         Current_Element (Sys.Processors, A_Processor, My_Iterator);

         if A_Processor.Name = Value then

            Initialize (A_Request);
            A_Request.Target    := Value;
            A_Request.Statement :=
              Scheduling_Set_Priorities_According_To_Deadline_Monotonic;
            Add (Request_List, A_Request);

            Sequential_Framework_Request (Sys, Request_List, Response_List);

            Write_Title_And_Text (Root_Window.Text, Response_List);

            -- Do not forget to save project
            -- since tasks are modified !!
            --
            Is_New   := False;
            Is_Saved := False;

            return True;
         end if;

         exit when Is_Last_Element (Sys.Processors, My_Iterator);
         Next_Element (Sys.Processors, My_Iterator);

      end loop;

      Show_Message_Box (Lb_Processor_Not_Found (Current_Language));
      return False;

   end Dm_Priorities_Callback;

   procedure On_Dm_Priorities_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is

   begin

      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
      else
         Gtk_New
           (Question,
            Lb_Set_Priorities_Dm (Current_Language),
            Lb_Processor_Name (Current_Language) & " : ",
            To_Unbounded_String (""),
            Dm_Priorities_Callback'Access);
         Show_All (Question);
      end if;

   end On_Dm_Priorities_Activate;

   procedure On_Rm_Priorities_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin

      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
      else
         Gtk_New
           (Question,
            Lb_Set_Priorities_Rm (Current_Language),
            Lb_Processor_Name (Current_Language) & " : ",
            To_Unbounded_String (""),
            Rm_Priorities_Callback'Access);
         Show_All (Question);
      end if;

   end On_Rm_Priorities_Activate;

   -- Random services
   --

   procedure On_Compute_Response_Time_Density_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin

      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
         return;
      end if;

      -- Check if scheduling is computed or not
      --
      if Call_Scheduling_Framework.Sched.Nb_Entries = 0 then
         Show_Message_Box (Lb_Schedule_Before (Current_Language));
         return;
      end if;

      Initialize (A_Request);
      A_Request.Statement := Random_Response_Time_Density;
      Add (Request_List, A_Request);

      Sequential_Framework_Request (Sys, Request_List, Response_List);

      Write_Title_And_Text (Root_Window.Text, Response_List);

   end On_Compute_Response_Time_Density_Activate;

   ----------------------------------------------------------------------
   --   Menu callbacks without calls to the framework
   ----------------------------------------------------------------------

   procedure Clear_Work_Space is
      Size  : Guint;
      Dummy : Boolean;
   begin
      Size := Get_Length (Root_Window.Text);
      Set_Point (Root_Window.Text, 0);
      Dummy := Forward_Delete (Root_Window.Text, Size);
      Clear_Time_Line (Draw_Scheduling);
   end Clear_Work_Space;

   --  Delete functions : called by delete callbacks
   --

   function Delete_Resource_Callback
     (Value : Unbounded_String)
      return  Boolean
   is
      My_Iterator : Resources_Iterator;
      A_Resource  : Generic_Resource_Ptr;
   begin
      loop

         Current_Element (Sys.Resources, A_Resource, My_Iterator);

         if A_Resource.Name = Value then

            Delete (Sys.Resources, A_Resource);

            Is_New   := False;
            Is_Saved := False;

            return True;
         end if;

         exit when Is_Last_Element (Sys.Resources, My_Iterator);
         Next_Element (Sys.Resources, My_Iterator);

      end loop;

      Show_Message_Box (Lb_Resource_Not_Found (Current_Language));
      return False;
   end Delete_Resource_Callback;

   function Delete_Network_Callback
     (Value : Unbounded_String)
      return  Boolean
   is
      My_Iterator : Network_Iterator;
      A_Network   : Network_Ptr;
   begin
      loop

         Current_Element (Sys.Networks, A_Network, My_Iterator);

         if A_Network.Name = Value then

            Delete (Sys.Networks, A_Network);

            Is_New   := False;
            Is_Saved := False;

            return True;
         end if;

         exit when Is_Last_Element (Sys.Networks, My_Iterator);
         Next_Element (Sys.Networks, My_Iterator);

      end loop;

      Show_Message_Box (Lb_Network_Not_Found (Current_Language));
      return False;
   end Delete_Network_Callback;

   function Delete_Buffer_Callback
     (Value : Unbounded_String)
      return  Boolean
   is
      My_Iterator : Buffers_Iterator;
      A_Buffer    : Buffer_Ptr;

   begin

      loop

         Current_Element (Sys.Buffers, A_Buffer, My_Iterator);

         if A_Buffer.Name = Value then

            Delete_Buffer (Sys, A_Buffer);
            Rebuild_Canvas (Graph_Window);

            Is_New   := False;
            Is_Saved := False;

            return True;
         end if;

         exit when Is_Last_Element (Sys.Buffers, My_Iterator);
         Next_Element (Sys.Buffers, My_Iterator);

      end loop;

      Show_Message_Box (Lb_Buffer_Not_Found (Current_Language));
      return False;
   end Delete_Buffer_Callback;

   function Delete_Message_Callback
     (Value : Unbounded_String)
      return  Boolean
   is
      My_Iterator : Messages_Iterator;
      A_Message   : Message_Ptr;
   begin
      loop

         Current_Element (Sys.Messages, A_Message, My_Iterator);

         if A_Message.Name = Value then

            Delete_Message (Sys, A_Message);
            Rebuild_Canvas (Graph_Window);

            Is_New   := False;
            Is_Saved := False;

            return True;
         end if;

         exit when Is_Last_Element (Sys.Messages, My_Iterator);
         Next_Element (Sys.Messages, My_Iterator);

      end loop;

      Show_Message_Box (Lb_Buffer_Not_Found (Current_Language));
      return False;
   end Delete_Message_Callback;

   function Delete_Task_Callback (Value : Unbounded_String) return Boolean is
      My_Iterator : Tasks_Iterator;
      A_Task      : Generic_Task_Ptr;
   begin

      loop
         Current_Element (Sys.Tasks, A_Task, My_Iterator);

         if A_Task.Name = Value then

            Delete_Task (Sys, A_Task);
            Rebuild_Canvas (Graph_Window);

            Is_New   := False;
            Is_Saved := False;

            return True;
         end if;

         exit when Is_Last_Element (Sys.Tasks, My_Iterator);
         Next_Element (Sys.Tasks, My_Iterator);

      end loop;

      Show_Message_Box (Lb_Task_Not_Found (Current_Language));
      return False;
   end Delete_Task_Callback;

   function Delete_Processor_Callback
     (Value : Unbounded_String)
      return  Boolean
   is
      Iterator1   : Processors_Iterator;
      A_Processor : Processor_Ptr;

   begin
      Reset_Iterator (Sys.Processors, Iterator1);

      loop

         Current_Element (Sys.Processors, A_Processor, Iterator1);

         if A_Processor.Name = Value then

            Delete_Processor (Sys, A_Processor);
            Rebuild_Canvas (Graph_Window);

            Is_New   := False;
            Is_Saved := False;

            return True;
         end if;

         exit when Is_Last_Element (Sys.Processors, Iterator1);
         Next_Element (Sys.Processors, Iterator1);

      end loop;

      Show_Message_Box (Lb_Processor_Not_Found (Current_Language));
      return False;
   end Delete_Processor_Callback;

   function On_Root_Window_Delete_Event
     (Object : access Gtk_Widget_Record'Class;
      Params : Gtk.Arguments.Gtk_Args)
      return   Boolean
   is
   begin
      Do_Exit;
      return True;
   end On_Root_Window_Delete_Event;

   procedure On_File_Activate (Object : access Gtk_Menu_Item_Record'Class) is
   begin
      null;
   end On_File_Activate;

   procedure On_New_Activate (Object : access Gtk_Menu_Item_Record'Class) is
   begin
      Do_New;
      Clear_Work_Space;
      Rebuild_Canvas (Graph_Window);
   end On_New_Activate;

   procedure On_Open_Activate (Object : access Gtk_Menu_Item_Record'Class) is
   begin
      Do_Open;
   end On_Open_Activate;

   procedure On_Save_Activate (Object : access Gtk_Menu_Item_Record'Class) is
   begin
      Do_Save;
   end On_Save_Activate;

   procedure On_Save_As_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      Do_Save_As;
   end On_Save_As_Activate;

   procedure On_Exit_Activate (Object : access Gtk_Menu_Item_Record'Class) is
   begin
      Do_Exit;
   end On_Exit_Activate;

   procedure On_Add_Task_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Address_Spaces) = 0) then
         Show_Message_Box
           (Lb_Define_Address_Spaces_Before (Current_Language));
      else
         Gtk_New (Update_Tasks_Set, Lb_Add_Task (Current_Language));
         Show_Advanced (Update_Tasks_Set, False);
      end if;
   end On_Add_Task_Activate;

   procedure On_Update_Task_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Address_Spaces) = 0) then
         Show_Message_Box
           (Lb_Define_Address_Spaces_Before (Current_Language));
      else
         Gtk_New (Update_Tasks_Set, Lb_Update_Task (Current_Language));
         Show_Advanced (Update_Tasks_Set, True);
      end if;
   end On_Update_Task_Activate;

   procedure On_Add_Processor_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      Gtk_New (Update_Processors_Set, Lb_Add_Processor (Current_Language));
      Show_Advanced (Update_Processors_Set, False);
   end On_Add_Processor_Activate;

   procedure On_Update_Processor_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Processors) = 0) then
         Show_Message_Box (Lb_Define_Processors_Before (Current_Language));
      else
         Gtk_New
           (Update_Processors_Set,
            Lb_Update_Processor (Current_Language));
         Show_Advanced (Update_Processors_Set, True);
      end if;
   end On_Update_Processor_Activate;

   procedure On_Update_Network_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Networks) = 0) then
         Show_Message_Box (Lb_Define_Networks_Before (Current_Language));
      else
         Gtk_New (Update_Networks_Set);
         Show_All (Update_Networks_Set);
      end if;
   end On_Update_Network_Activate;

   procedure On_Add_Network_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      Gtk_New (Networks_Parameters);
      Show_All (Networks_Parameters);
   end On_Add_Network_Activate;

   procedure On_Tasks_List_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin

      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
      else
         Gtk_New (Display_Tasks_Set);
         Show_All (Display_Tasks_Set);
      end if;
   end On_Tasks_List_Activate;

   procedure On_Processors_List_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Processors) = 0) then
         Show_Message_Box (Lb_Define_Processors_Before (Current_Language));
      else
         Gtk_New (Display_Processors_Set);
         Show_All (Display_Processors_Set);
      end if;
   end On_Processors_List_Activate;

   procedure On_Resource_List_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Resources) = 0) then
         Show_Message_Box (Lb_Define_Resources_Before (Current_Language));
      else
         Gtk_New (Display_Resources_Set);
         Show_All (Display_Resources_Set);
      end if;
   end On_Resource_List_Activate;

   procedure On_Network_List_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Networks) = 0) then
         Show_Message_Box (Lb_Define_Networks_Before (Current_Language));
      else
         Gtk_New (Display_Networks_Set);
         Show_All (Display_Networks_Set);
      end if;
   end On_Network_List_Activate;

   procedure On_Message_List_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Messages) = 0) then
         Show_Message_Box (Lb_Define_Messages_Before (Current_Language));
      else
         Gtk_New (Display_Messages_Set);
         Show_All (Display_Messages_Set);
      end if;
   end On_Message_List_Activate;

   procedure On_Buffer_List_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Buffers) = 0) then
         Show_Message_Box (Lb_Define_Buffers_Before (Current_Language));
      else
         Gtk_New (Display_Buffers_Set);
         Show_All (Display_Buffers_Set);
      end if;
   end On_Buffer_List_Activate;

   procedure On_Tools_Activate (Object : access Gtk_Menu_Item_Record'Class) is
   begin
      null;
   end On_Tools_Activate;

   procedure On_Clear_Work_Space_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      Clear_Work_Space;
   end On_Clear_Work_Space_Activate;

   procedure Call_Schedule_From_To_After_Do_Scheduling is
   begin

      Gtk_New
        (Schedule_From_To,
         Root_Window.Text,
         Draw_Scheduling,
         Sys,
         True);
      Show_All (Schedule_From_To);

   end Call_Schedule_From_To_After_Do_Scheduling;

   procedure On_Draw_Scheduling_Graph_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is

   begin

      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
         return;
      end if;

      Gtk_New
        (Do_Scheduling_Simulation,
         Call_Schedule_From_To_After_Do_Scheduling'Access);
      Show_All (Do_Scheduling_Simulation);
   end On_Draw_Scheduling_Graph_Activate;

   procedure On_Options_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      Gtk_New (Scheduling_Options);
      Show_All (Scheduling_Options);
   end On_Options_Activate;

   procedure On_Check_Scheduling_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is

   begin

      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
         return;
      end if;

      Gtk_New (Do_Scheduling_Feasibility, Root_Window.Text);
      Show_All (Do_Scheduling_Feasibility);
   end On_Check_Scheduling_Activate;

   procedure On_Help_Activate (Object : access Gtk_Menu_Item_Record'Class) is
   begin
      null;
   end On_Help_Activate;

   procedure On_Update_Resource_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Resources) = 0) then
         Show_Message_Box (Lb_Define_Resources_Before (Current_Language));
      else
         Gtk_New
           (Update_Resources_Set,
            Lb_Update_Resource (Current_Language));
         Show_Advanced (Update_Resources_Set, True);
      end if;
   end On_Update_Resource_Activate;

   procedure On_Add_Resource_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Address_Spaces) = 0) then
         Show_Message_Box
           (Lb_Define_Address_Spaces_Before (Current_Language));
      else
         if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
            Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
         else
            Gtk_New
              (Update_Resources_Set,
               Lb_Add_Resource (Current_Language));
            Show_Advanced (Update_Resources_Set, False);
         end if;
      end if;
   end On_Add_Resource_Activate;

   procedure On_About_Cheddar_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is

   begin
      Show_Message_Box (Lb_About_Text (Current_Language));
   end On_About_Cheddar_Activate;

   procedure On_Manual_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      Gtk_New (Scrolling_Text, Lb_Manual (Current_Language));
      Set_Usize (Scrolling_Text, 690, 425);

      Write_Title_Small (Scrolling_Text, Manual.Ustr_File_Menu);
      Write_Text (Scrolling_Text, Manual.Ustr_File_Menu_Content);

      Write_Title_Small (Scrolling_Text, Manual.Ustr_Edit_Menu);
      Write_Text (Scrolling_Text, Manual.Ustr_Edit_Menu_Content);

      Write_Title_Small (Scrolling_Text, Manual.Ustr_View_Menu);
      Write_Text (Scrolling_Text, Manual.Ustr_View_Menu_Content);

      Write_Title_Small (Scrolling_Text, Manual.Ustr_Tools_Menu);
      Write_Text (Scrolling_Text, Manual.Ustr_Tools_Menu_Content);

      Write_Title_Small (Scrolling_Text, Manual.Ustr_Help_Menu);
      Write_Text (Scrolling_Text, Manual.Ustr_Help_Menu_Content);

      Show_All (Scrolling_Text);
   end On_Manual_Activate;

   procedure On_Bibliography_References_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      Gtk_New (Scrolling_Text, Lb_Ref (Current_Language));
      Set_Usize (Scrolling_Text, 535, 440);
      Write_Text (Scrolling_Text, References.Ref_Text);
      Show_All (Scrolling_Text);
   end On_Bibliography_References_Activate;

   procedure On_Delete_Task_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
      else
         Gtk_New
           (Question,
            Lb_Delete_Task (Current_Language),
            Lb_Task_Name (Current_Language) & " : ",
            To_Unbounded_String (""),
            Delete_Task_Callback'Access);
         Show_All (Question);
      end if;
   end On_Delete_Task_Activate;

   procedure On_Delete_Resource_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Resources) = 0) then
         Show_Message_Box (Lb_Define_Resources_Before (Current_Language));
      else
         Gtk_New
           (Question,
            Lb_Delete_Resource (Current_Language),
            Lb_Resource_Name (Current_Language) & " : ",
            To_Unbounded_String (""),
            Delete_Resource_Callback'Access);
         Show_All (Question);
      end if;
   end On_Delete_Resource_Activate;

   procedure On_Delete_Network_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Networks) = 0) then
         Show_Message_Box (Lb_Define_Networks_Before (Current_Language));
      else
         Gtk_New
           (Question,
            Lb_Delete_Network (Current_Language),
            Lb_Networks_Name (Current_Language) & " : ",
            To_Unbounded_String (""),
            Delete_Network_Callback'Access);
         Show_All (Question);
      end if;
   end On_Delete_Network_Activate;

   procedure On_Delete_Processor_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Processors) = 0) then
         Show_Message_Box (Lb_Define_Processors_Before (Current_Language));
      else
         Gtk_New
           (Question,
            Lb_Delete_Processor (Current_Language),
            Lb_Processor_Name (Current_Language) & " : ",
            To_Unbounded_String (""),
            Delete_Processor_Callback'Access);
         Show_All (Question);
      end if;
   end On_Delete_Processor_Activate;

   procedure On_Add_Message_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      Gtk_New (Update_Messages_Set, To_Unbounded_String ("cheddar_window"));
      Show_Advanced (Update_Messages_Set, False);
   end On_Add_Message_Activate;

   procedure On_Delete_Message_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Messages) = 0) then
         Show_Message_Box (Lb_Define_Messages_Before (Current_Language));
      else
         Gtk_New
           (Question,
            Lb_Delete_Message (Current_Language),
            Lb_Message_Name (Current_Language) & " : ",
            To_Unbounded_String (""),
            Delete_Message_Callback'Access);
         Show_All (Question);
      end if;
   end On_Delete_Message_Activate;

   procedure On_Update_Message_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Messages) = 0) then
         Show_Message_Box (Lb_Define_Messages_Before (Current_Language));
      else
         Gtk_New
           (Update_Messages_Set,
            To_Unbounded_String ("cheddar_window"));
         Show_All (Update_Messages_Set);
      end if;
   end On_Update_Message_Activate;

   procedure On_Add_Buffer_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
      else
         Gtk_New
           (Update_Buffers_Set,
            Lb_Add_Buffer (Current_Language),
            To_Unbounded_String ("cheddar_window"));
         Show_Advanced (Update_Buffers_Set, False);
      end if;
   end On_Add_Buffer_Activate;

   procedure On_Delete_Buffer_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Buffers) = 0) then
         Show_Message_Box (Lb_Define_Buffers_Before (Current_Language));
      else
         Gtk_New
           (Question,
            Lb_Delete_Buffer (Current_Language),
            Lb_Buffer_Name (Current_Language) & " : ",
            To_Unbounded_String (""),
            Delete_Buffer_Callback'Access);
         Show_All (Question);
      end if;
   end On_Delete_Buffer_Activate;

   procedure On_Update_Buffer_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
      else
         Gtk_New
           (Update_Buffers_Set,
            Lb_Update_Buffer (Current_Language),
            To_Unbounded_String ("cheddar_window"));
         Show_Advanced (Update_Buffers_Set, True);
      end if;

   end On_Update_Buffer_Activate;

   procedure On_Check_Buffer_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      Check_Buffer;
   end On_Check_Buffer_Activate;

   procedure On_Blocking_Time_Callback
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      null;
   end On_Blocking_Time_Callback;

   procedure On_Duplicate_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is

   begin
      Gtk_New (Duplicate_Pkg.Duplicate);
      Show_All (Duplicate_Pkg.Duplicate);
   end On_Duplicate_Activate;

   ----------------------------------------------------------------------
   --   Pixmap callbacks without calls to the framework
   ----------------------------------------------------------------------

   procedure On_Pixmap_Draw_Scheduling_Clicked
     (Object : access Gtk_Toolbar_Record'Class)
   is

   begin

      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
         return;
      end if;

      Gtk_New (Schedule_From_To, Root_Window.Text, Draw_Scheduling, Sys);
      Show_All (Schedule_From_To);

   end On_Pixmap_Draw_Scheduling_Clicked;

   procedure On_Pixmap_New_Clicked
     (Object : access Gtk_Toolbar_Record'Class)
   is
   begin
      Do_New;
      Clear_Work_Space;
      Rebuild_Canvas (Graph_Window);
   end On_Pixmap_New_Clicked;

   procedure On_Pixmap_Open_Clicked
     (Object : access Gtk_Toolbar_Record'Class)
   is
   begin
      Do_Open;
   end On_Pixmap_Open_Clicked;

   procedure On_Pixmap_Save_Clicked
     (Object : access Gtk_Toolbar_Record'Class)
   is
   begin
      Do_Save;
   end On_Pixmap_Save_Clicked;

   procedure On_Pixmap_Save_As_Clicked
     (Object : access Gtk_Toolbar_Record'Class)
   is
   begin
      Do_Save_As;
   end On_Pixmap_Save_As_Clicked;

   procedure On_Pixmap_Exit_Clicked
     (Object : access Gtk_Toolbar_Record'Class)
   is
   begin
      Do_Exit;
   end On_Pixmap_Exit_Clicked;

   procedure On_Pixmap_Clear_Work_Space_Clicked
     (Object : access Gtk_Toolbar_Record'Class)
   is
   begin
      Clear_Work_Space;
   end On_Pixmap_Clear_Work_Space_Clicked;

   procedure On_Pixmap_Dependencies_Clicked
     (Object : access Gtk_Toolbar_Record'Class)
   is
   begin
      Gtk_New (Graph_Window);
      Show (Graph_Window);
   end On_Pixmap_Dependencies_Clicked;

   procedure On_Pixmap_Check_Scheduling_Clicked
     (Object : access Gtk_Toolbar_Record'Class)
   is

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin

      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
         return;
      end if;

      Initialize (A_Request);
      A_Request.Statement := Scheduling_Feasibility_Basics;
      A_Request.Target    := To_Unbounded_String ("");
      Add (Request_List, A_Request);

      Sequential_Framework_Request (Sys, Request_List, Response_List);

      Write_Title_And_Text (Root_Window.Text, Response_List);

   end On_Pixmap_Check_Scheduling_Clicked;

   procedure On_Deadlock_Callback
     (Object : access Gtk_Menu_Item_Record'Class)
   is

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin

      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
         return;
      end if;

      -- Check if scheduling is computed or not
      --
      if Call_Scheduling_Framework.Sched.Nb_Entries = 0 then
         Show_Message_Box (Lb_Schedule_Before (Current_Language));
         return;
      end if;

      Initialize (A_Request);
      A_Request.Statement := Scheduling_Simulation_Deadlock;
      Add (Request_List, A_Request);

      Sequential_Framework_Request (Sys, Request_List, Response_List);

      Write_Title_And_Text (Root_Window.Text, Response_List);

   end On_Deadlock_Callback;

   procedure On_Priority_Inversion_Callback
     (Object : access Gtk_Menu_Item_Record'Class)
   is

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin

      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
         return;
      end if;

      -- Check if scheduling is computed or not
      --
      if Call_Scheduling_Framework.Sched.Nb_Entries = 0 then
         Show_Message_Box (Lb_Schedule_Before (Current_Language));
         return;
      end if;

      Initialize (A_Request);
      A_Request.Statement := Scheduling_Simulation_Priority_Inversion;
      Add (Request_List, A_Request);

      Sequential_Framework_Request (Sys, Request_List, Response_List);

      Write_Title_And_Text (Root_Window.Text, Response_List);

   end On_Priority_Inversion_Callback;

   procedure On_Add_Event_Analyzer_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      Gtk_New
        (Update_Event_Analyzers_Set,
         Lb_Add_Event_Analyzer (Current_Language));
      Show_Advanced (Update_Event_Analyzers_Set, False);
   end On_Add_Event_Analyzer_Activate;

   function Delete_Event_Analyzer_Callback
     (Value : Unbounded_String)
      return  Boolean
   is
      My_Iterator       : Event_Analyzers_Iterator;
      An_Event_Analyzer : Event_Analyzer_Ptr;
   begin
      loop

         Current_Element
           (Sys.Event_Analyzers,
            An_Event_Analyzer,
            My_Iterator);

         if An_Event_Analyzer.Behavior_File_Name = Value then

            Delete (Sys.Event_Analyzers, An_Event_Analyzer);

            Is_New   := False;
            Is_Saved := False;

            return True;
         end if;

         exit when Is_Last_Element (Sys.Event_Analyzers, My_Iterator);
         Next_Element (Sys.Event_Analyzers, My_Iterator);

      end loop;

      Show_Message_Box (Lb_Event_Analyzer_Not_Found (Current_Language));
      return False;
   end Delete_Event_Analyzer_Callback;

   procedure On_Update_Event_Analyzer_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      Gtk_New
        (Update_Event_Analyzers_Set,
         Lb_Update_Event_Analyzer (Current_Language));
      Show_Advanced (Update_Event_Analyzers_Set, True);
   end On_Update_Event_Analyzer_Activate;

   procedure On_Delete_Event_Analyzer_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Event_Analyzers) = 0) then
         Show_Message_Box
           (Lb_Define_Event_Analyzers_Before (Current_Language));
      else
         Gtk_New
           (Question,
            Lb_Delete_Event_Analyzer (Current_Language),
            Lb_Event_Analyzer_Name (Current_Language) & " : ",
            To_Unbounded_String (""),
            Delete_Event_Analyzer_Callback'Access);
         Show_All (Question);
      end if;
   end On_Delete_Event_Analyzer_Activate;

   procedure On_Event_Analyzer_List_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Event_Analyzers) = 0) then
         Show_Message_Box
           (Lb_Define_Event_Analyzers_Before (Current_Language));
      else
         Gtk_New (Display_Event_Analyzers_Set);
         Show_All (Display_Event_Analyzers_Set);
      end if;
   end On_Event_Analyzer_List_Activate;

   procedure On_Partition_Small_Task_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin

      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
         return;
      end if;
      if (Get_Number_Of_Elements (Sys.Processors) = 0) then
         Show_Message_Box (Lb_Define_Processors_Before (Current_Language));
         return;
      end if;
      if (Get_Number_Of_Elements (Sys.Address_Spaces) = 0) then
         Show_Message_Box
           (Lb_Define_Address_Spaces_Before (Current_Language));
         return;
      end if;

      Initialize (A_Request);
      A_Request.Statement := Scheduling_Feasibility_Small_Task;
      Add (Request_List, A_Request);

      Sequential_Framework_Request (Sys, Request_List, Response_List);

      Write_Title_And_Text (Root_Window.Text, Response_List);

   end On_Partition_Small_Task_Activate;

   procedure On_Partition_Best_Fit_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin

      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
         return;
      end if;
      if (Get_Number_Of_Elements (Sys.Processors) = 0) then
         Show_Message_Box (Lb_Define_Processors_Before (Current_Language));
         return;
      end if;
      if (Get_Number_Of_Elements (Sys.Address_Spaces) = 0) then
         Show_Message_Box
           (Lb_Define_Address_Spaces_Before (Current_Language));
         return;
      end if;

      Initialize (A_Request);
      A_Request.Statement := Scheduling_Feasibility_Best_Fit;
      Add (Request_List, A_Request);

      Sequential_Framework_Request (Sys, Request_List, Response_List);

      Write_Title_And_Text (Root_Window.Text, Response_List);

   end On_Partition_Best_Fit_Activate;

   procedure On_Partition_Next_Fit_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin

      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
         return;
      end if;
      if (Get_Number_Of_Elements (Sys.Processors) = 0) then
         Show_Message_Box (Lb_Define_Processors_Before (Current_Language));
         return;
      end if;
      if (Get_Number_Of_Elements (Sys.Address_Spaces) = 0) then
         Show_Message_Box
           (Lb_Define_Address_Spaces_Before (Current_Language));
         return;
      end if;

      Initialize (A_Request);
      A_Request.Statement := Scheduling_Feasibility_Next_Fit;
      Add (Request_List, A_Request);

      Sequential_Framework_Request (Sys, Request_List, Response_List);

      Write_Title_And_Text (Root_Window.Text, Response_List);

   end On_Partition_Next_Fit_Activate;

   procedure On_Partition_General_Task_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin

      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
         return;
      end if;
      if (Get_Number_Of_Elements (Sys.Processors) = 0) then
         Show_Message_Box (Lb_Define_Processors_Before (Current_Language));
         return;
      end if;
      if (Get_Number_Of_Elements (Sys.Address_Spaces) = 0) then
         Show_Message_Box
           (Lb_Define_Address_Spaces_Before (Current_Language));
         return;
      end if;

      Initialize (A_Request);
      A_Request.Statement := Scheduling_Feasibility_General_Task;
      Add (Request_List, A_Request);

      Sequential_Framework_Request (Sys, Request_List, Response_List);

      Write_Title_And_Text (Root_Window.Text, Response_List);

   end On_Partition_General_Task_Activate;

   procedure On_Partition_First_Fit_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin

      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
         return;
      end if;
      if (Get_Number_Of_Elements (Sys.Processors) = 0) then
         Show_Message_Box (Lb_Define_Processors_Before (Current_Language));
         return;
      end if;
      if (Get_Number_Of_Elements (Sys.Address_Spaces) = 0) then
         Show_Message_Box
           (Lb_Define_Address_Spaces_Before (Current_Language));
         return;
      end if;

      Initialize (A_Request);
      A_Request.Statement := Scheduling_Feasibility_First_Fit;
      Add (Request_List, A_Request);

      Sequential_Framework_Request (Sys, Request_List, Response_List);

      Write_Title_And_Text (Root_Window.Text, Response_List);

   end On_Partition_First_Fit_Activate;

   procedure On_Load_Event_Table_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is

   begin

      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
         return;
      end if;

      Gtk_New (open_event_table);
      Show_All (open_event_table);
   end On_Load_Event_Table_Activate;

   procedure On_Save_Event_Table_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin

      -- Check if scheduling is computed or not
      --
      if Call_Scheduling_Framework.Sched.Nb_Entries = 0 then
         Show_Message_Box (Lb_Schedule_Before (Current_Language));
         return;
      end if;

      Gtk_New (save_event_table);
      Show_All (save_event_table);

   end On_Save_Event_Table_Activate;

   procedure On_Event_Table_Services_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      null;
   end On_Event_Table_Services_Activate;

   procedure On_Draw_Time_Line_From_Event_Table_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin

      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
         return;
      end if;

      -- Check if scheduling is computed or not
      --
      if Call_Scheduling_Framework.Sched.Nb_Entries = 0 then
         Show_Message_Box (Lb_Schedule_Before (Current_Language));
         return;
      end if;

      Gtk_New (Draw_Time_Line_From_To, Draw_Scheduling, Sys);
      Show_All (Draw_Time_Line_From_To);
   end On_Draw_Time_Line_From_Event_Table_Activate;

   procedure On_Compute_Scheduling_And_Generate_Event_Table_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin

      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
         return;
      end if;

   end On_Compute_Scheduling_And_Generate_Event_Table_Activate;

   procedure On_Run_Analysis_On_Event_Table_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin

      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
         return;
      end if;

      -- Check if scheduling is computed or not
      --
      if Call_Scheduling_Framework.Sched.Nb_Entries = 0 then
         Show_Message_Box (Lb_Schedule_Before (Current_Language));
         return;
      end if;

      Gtk_New (Event_Table_Analysis, Root_Window.Text, Sys);
      Show_All (Event_Table_Analysis);

   end On_Run_Analysis_On_Event_Table_Activate;

   procedure On_Add_Address_Space_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Processors) = 0) then
         Show_Message_Box (Lb_Define_Processors_Before (Current_Language));
      else
         Gtk_New
           (Update_Address_Spaces_Set,
            Lb_Add_Address_Space (Current_Language));
         Show_Advanced (Update_Address_Spaces_Set, False);
      end if;
   end On_Add_Address_Space_Activate;

   procedure On_Update_Address_Space_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Processors) = 0) then
         Show_Message_Box (Lb_Define_Processors_Before (Current_Language));
      else
         Gtk_New
           (Update_Address_Spaces_Set,
            Lb_Update_Address_Space (Current_Language));
         Show_Advanced (Update_Address_Spaces_Set, True);
      end if;
   end On_Update_Address_Space_Activate;

   function Delete_Address_Space_Callback
     (Value : Unbounded_String)
      return  Boolean
   is
      My_Iterator      : Address_Spaces_Iterator;
      An_Address_Space : Address_Space_Ptr;
   begin

      loop
         Current_Element (Sys.Address_Spaces, An_Address_Space, My_Iterator);

         if An_Address_Space.Name = Value then

            Delete_Address_Space (Sys, An_Address_Space);
            Rebuild_Canvas (Graph_Window);

            Is_New   := False;
            Is_Saved := False;

            return True;
         end if;

         exit when Is_Last_Element (Sys.Address_Spaces, My_Iterator);
         Next_Element (Sys.Address_Spaces, My_Iterator);
      end loop;

      Show_Message_Box (Lb_Address_Space_Not_Found (Current_Language));
      return False;
   end Delete_Address_Space_Callback;

   procedure On_Delete_Address_Space_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Address_Spaces) = 0) then
         Show_Message_Box
           (Lb_Define_Address_Spaces_Before (Current_Language));
      else
         Gtk_New
           (Question,
            Lb_Delete_Address_Space (Current_Language),
            Lb_Address_Space_Name (Current_Language) & " : ",
            To_Unbounded_String (""),
            Delete_Address_Space_Callback'Access);
         Show_All (Question);
      end if;
   end On_Delete_Address_Space_Activate;

   procedure On_Address_Spaces_List_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      if (Get_Number_Of_Elements (Sys.Address_Spaces) = 0) then
         Show_Message_Box
           (Lb_Define_Address_Spaces_Before (Current_Language));
      else
         Gtk_New (Display_Address_Spaces_Set);
         Show_All (Display_Address_Spaces_Set);
      end if;
   end On_Address_Spaces_List_Activate;

   procedure On_Aadl_Activate (Object : access Gtk_Menu_Item_Record'Class) is
   begin
      null;
   end On_Aadl_Activate;

   procedure On_Export_Aadl_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      Do_Aadl_Export;
   end On_Export_Aadl_Activate;

   procedure On_Import_Aadl_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      Do_Aadl_Import;
   end On_Import_Aadl_Activate;

   procedure On_Check_Address_Space_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin

      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
         return;
      end if;
      if (Get_Number_Of_Elements (Sys.Address_Spaces) = 0) then
         Show_Message_Box
           (Lb_Define_Address_Spaces_Before (Current_Language));
         return;
      end if;

      Initialize (A_Request);
      A_Request.Statement := Global_Memory_Set_Requirement_Analysis;
      Add (Request_List, A_Request);

      Sequential_Framework_Request (Sys, Request_List, Response_List);

      Write_Title_And_Text (Root_Window.Text, Response_List);

   end On_Check_Address_Space_Activate;

   procedure On_Check_And_Update_Address_Space_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is

      Response_List : Framework_Response_Table;
      Request_List  : Framework_Request_Table;
      A_Request     : Framework_Request;

   begin

      if (Get_Number_Of_Elements (Sys.Tasks) = 0) then
         Show_Message_Box (Lb_Define_Tasks_Before (Current_Language));
         return;
      end if;
      if (Get_Number_Of_Elements (Sys.Address_Spaces) = 0) then
         Show_Message_Box
           (Lb_Define_Address_Spaces_Before (Current_Language));
         return;
      end if;

      Initialize (A_Request);
      A_Request.Statement := Global_Memory_Compute_Requirement_Analysis;
      Add (Request_List, A_Request);

      Sequential_Framework_Request (Sys, Request_List, Response_List);

      Write_Title_And_Text (Root_Window.Text, Response_List);

   end On_Check_And_Update_Address_Space_Activate;

   procedure On_Export_Aadl_Standard_Properties_Set_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      Write_Aadl_Standard_Properties_Set_To_File (Sys);
   end On_Export_Aadl_Standard_Properties_Set_Activate;

   procedure On_Export_Cheddar_Property_Sets_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      Write_Cheddar_Property_Sets_To_File (Sys);
   end On_Export_Cheddar_Property_Sets_Activate;

   procedure On_Aadl_Options_Activate
     (Object : access Gtk_Menu_Item_Record'Class)
   is
   begin
      Gtk_New (Aadl_Options);
      Show_All (Aadl_Options);
   end On_Aadl_Options_Activate;

end Root_Window_Pkg.Callbacks;
