------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Gtk;                       use Gtk;
with Gdk.Types;                 use Gdk.Types;
with Gtk.Widget;                use Gtk.Widget;
with Gtk.Enums;                 use Gtk.Enums;
with Gtkada.Handlers;           use Gtkada.Handlers;
with Callbacks_Cheddar;         use Callbacks_Cheddar;
with Root_Window_Pkg.Callbacks; use Root_Window_Pkg.Callbacks;
with Draw_Scheduling_Pkg;       use Draw_Scheduling_Pkg;
with Xpm;                       use Xpm;
with Framework_Config;          use Framework_Config;
with Translate;                 use Translate;
with Ada.Strings.Unbounded;     use Ada.Strings.Unbounded;
with Text_Tools;                use Text_Tools;

package body Root_Window_Pkg is

   procedure Gtk_New (Root_Window : out Root_Window_Access) is
   begin
      Root_Window := new Root_Window_Record;
      Root_Window_Pkg.Initialize (Root_Window);
   end Gtk_New;

   procedure Initialize (Root_Window : access Root_Window_Record'Class) is
   begin

      Text_Tools.Initialize;

      Gtk.Window.Initialize (Root_Window, Window_Toplevel);
      Set_Title (Root_Window, To_String (Lb_Root_Title (Current_Language)));
      Set_Policy (Root_Window, True, True, True);
      Set_Position (Root_Window, Win_Pos_Center_Always);
      Set_Default_Size (Root_Window, 800, 600);
      Set_Extension_Events (Root_Window, Extension_Events_All);
      Set_Flags (Root_Window, Can_Default);
      Grab_Focus (Root_Window);
      Return_Callback.Connect
        (Root_Window,
         "delete_event",
         On_Root_Window_Delete_Event'Access);

      Gtk_New_Vbox (Root_Window.Vbox, False, 0);
      Add (Root_Window, Root_Window.Vbox);

      Gtk_New (Root_Window.Menubar);
      Set_Border_Width (Root_Window.Menubar, 3);
      Set_Shadow_Type (Root_Window.Menubar, Shadow_In);
      Pack_Start (Root_Window.Vbox, Root_Window.Menubar, False, False, 0);

      Gtk_New (Root_Window.File, To_String (Lb_File (Current_Language)));
      Set_Right_Justify (Root_Window.File, False);
      Menu_Item_Callback.Connect
        (Root_Window.File,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_File_Activate'Access));
      Add (Root_Window.Menubar, Root_Window.File);

      Gtk_New (Root_Window.File_Menu);
      Set_Submenu (Root_Window.File, Root_Window.File_Menu);

      Gtk_New (Root_Window.New_File, To_String (Lb_New (Current_Language)));
      Set_Right_Justify (Root_Window.New_File, False);
      Menu_Item_Callback.Connect
        (Root_Window.New_File,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_New_Activate'Access));
      Add (Root_Window.File_Menu, Root_Window.New_File);

      Gtk_New
        (Root_Window.Open_File,
         To_String (Lb_Open_Xml (Current_Language)));
      Set_Right_Justify (Root_Window.Open_File, False);
      Menu_Item_Callback.Connect
        (Root_Window.Open_File,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Open_Activate'Access));
      Add (Root_Window.File_Menu, Root_Window.Open_File);

      Gtk_New
        (Root_Window.Save_File,
         To_String (Lb_Save_Xml (Current_Language)));
      Set_Right_Justify (Root_Window.Save_File, False);
      Menu_Item_Callback.Connect
        (Root_Window.Save_File,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Save_Activate'Access));
      Add (Root_Window.File_Menu, Root_Window.Save_File);

      Gtk_New
        (Root_Window.Save_As_File,
         To_String (Lb_Save_As_Xml (Current_Language)));
      Set_Right_Justify (Root_Window.Save_As_File, False);
      Menu_Item_Callback.Connect
        (Root_Window.Save_As_File,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Save_As_Activate'Access));
      Add (Root_Window.File_Menu, Root_Window.Save_As_File);

      Gtk_New (Root_Window.Separator1);
      Set_Right_Justify (Root_Window.Separator1, False);
      Add (Root_Window.File_Menu, Root_Window.Separator1);

      --
      -- AADL menu and sub-menus
      --

      Gtk_New (Root_Window.Aadl, To_String (Lb_Aadl (Current_Language)));
      Set_Right_Justify (Root_Window.Aadl, False);
      Menu_Item_Callback.Connect
        (Root_Window.Aadl,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Aadl_Activate'Access));
      Add (Root_Window.File_Menu, Root_Window.Aadl);

      Gtk_New (Root_Window.Aadl_Menu);
      Set_Submenu (Root_Window.Aadl, Root_Window.Aadl_Menu);

      Gtk_New
        (Root_Window.Import_Aadl_File,
         To_String (Lb_Import_Aadl (Current_Language)));
      Set_Right_Justify (Root_Window.Import_Aadl_File, False);
      Menu_Item_Callback.Connect
        (Root_Window.Import_Aadl_File,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Import_Aadl_Activate'Access));
      Add (Root_Window.Aadl_Menu, Root_Window.Import_Aadl_File);

      Gtk_New
        (Root_Window.Export_Aadl_File,
         To_String (Lb_Export_Aadl (Current_Language)));
      Set_Right_Justify (Root_Window.Export_Aadl_File, False);
      Menu_Item_Callback.Connect
        (Root_Window.Export_Aadl_File,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Export_Aadl_Activate'Access));
      Add (Root_Window.Aadl_Menu, Root_Window.Export_Aadl_File);

      Gtk_New
        (Root_Window.Export_Cheddar_Property_Sets,
         To_String (Lb_Export_Cheddar_Property_Sets (Current_Language)));
      Set_Right_Justify (Root_Window.Export_Cheddar_Property_Sets, False);
      Menu_Item_Callback.Connect
        (Root_Window.Export_Cheddar_Property_Sets,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Export_Cheddar_Property_Sets_Activate'Access));
      Add (Root_Window.Aadl_Menu, Root_Window.Export_Cheddar_Property_Sets);

      Gtk_New
        (Root_Window.Export_Aadl_Standard_Properties_Set,
         To_String
            (Lb_Export_Aadl_Standard_Properties_Set (Current_Language)));
      Set_Right_Justify
        (Root_Window.Export_Aadl_Standard_Properties_Set,
         False);
      Menu_Item_Callback.Connect
        (Root_Window.Export_Aadl_Standard_Properties_Set,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Export_Aadl_Standard_Properties_Set_Activate'Access));
      Add
        (Root_Window.Aadl_Menu,
         Root_Window.Export_Aadl_Standard_Properties_Set);

      Gtk_New
        (Root_Window.Aadl_Options,
         To_String (Lb_Aadl_Options (Current_Language)));
      Set_Right_Justify (Root_Window.Aadl_Options, False);
      Menu_Item_Callback.Connect
        (Root_Window.Aadl_Options,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Aadl_Options_Activate'Access));
      Add (Root_Window.Aadl_Menu, Root_Window.Aadl_Options);

      Gtk_New (Root_Window.Separator1);
      Set_Right_Justify (Root_Window.Separator1, False);
      Add (Root_Window.File_Menu, Root_Window.Separator1);

      Gtk_New
        (Root_Window.Exit_Program,
         To_String (Lb_Exit (Current_Language)));
      Set_Right_Justify (Root_Window.Exit_Program, False);
      Menu_Item_Callback.Connect
        (Root_Window.Exit_Program,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Exit_Activate'Access));
      Add (Root_Window.File_Menu, Root_Window.Exit_Program);

      Gtk_New (Root_Window.Edit, To_String (Lb_Edit (Current_Language)));
      Set_Right_Justify (Root_Window.Edit, False);
      Add (Root_Window.Menubar, Root_Window.Edit);

      Gtk_New (Root_Window.Edit_Menu);
      Set_Submenu (Root_Window.Edit, Root_Window.Edit_Menu);

      -- Create Add SubMenu
      --
      Gtk_New (Root_Window.Add, To_String (Lb_Add (Current_Language)));
      Set_Right_Justify (Root_Window.Add, False);
      Add (Root_Window.Edit_Menu, Root_Window.Add);

      Gtk_New (Root_Window.Add_Menu);
      Set_Submenu (Root_Window.Add, Root_Window.Add_Menu);

      -- Add add_Processor to Add Submenu
      --
      Gtk_New
        (Root_Window.Add_Processor,
         To_String (Lb_Add_Processor (Current_Language)));
      Set_Right_Justify (Root_Window.Add_Processor, False);
      Menu_Item_Callback.Connect
        (Root_Window.Add_Processor,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Add_Processor_Activate'Access));
      Add (Root_Window.Add_Menu, Root_Window.Add_Processor);

      -- Add add_address_space to Add Submenu
      --
      Gtk_New
        (Root_Window.Add_Address_Space,
         To_String (Lb_Add_Address_Space (Current_Language)));
      Set_Right_Justify (Root_Window.Add_Address_Space, False);
      Menu_Item_Callback.Connect
        (Root_Window.Add_Address_Space,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Add_Address_Space_Activate'Access));
      Add (Root_Window.Add_Menu, Root_Window.Add_Address_Space);

      -- Add add_Task to Add Submenu
      --
      Gtk_New
        (Root_Window.Add_Task,
         To_String (Lb_Add_Task (Current_Language)));
      Set_Right_Justify (Root_Window.Add_Task, False);
      Menu_Item_Callback.Connect
        (Root_Window.Add_Task,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Add_Task_Activate'Access));
      Add (Root_Window.Add_Menu, Root_Window.Add_Task);

      -- Add add_Resource to Add Submenu
      --
      Gtk_New
        (Root_Window.Add_Resource,
         To_String (Lb_Add_Resource (Current_Language)));
      Set_Right_Justify (Root_Window.Add_Resource, False);
      Menu_Item_Callback.Connect
        (Root_Window.Add_Resource,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Add_Resource_Activate'Access));
      Add (Root_Window.Add_Menu, Root_Window.Add_Resource);

      -- Add add_Message to Add Submenu
      --
      Gtk_New
        (Root_Window.Add_Message,
         To_String (Lb_Add_Message (Current_Language)));
      Set_Right_Justify (Root_Window.Add_Message, False);
      Menu_Item_Callback.Connect
        (Root_Window.Add_Message,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Add_Message_Activate'Access));
      Add (Root_Window.Add_Menu, Root_Window.Add_Message);

      -- Add add_Buffer to Add Submenu
      --
      Gtk_New
        (Root_Window.Add_Buffer,
         To_String (Lb_Add_Buffer (Current_Language)));
      Set_Right_Justify (Root_Window.Add_Buffer, False);
      Menu_Item_Callback.Connect
        (Root_Window.Add_Buffer,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Add_Buffer_Activate'Access));
      Add (Root_Window.Add_Menu, Root_Window.Add_Buffer);

      -- Add add_Network to Add Submenu
      --
      Gtk_New
        (Root_Window.Add_Network,
         To_String (Lb_Add_Network (Current_Language)));
      Set_Right_Justify (Root_Window.Add_Network, False);
      Menu_Item_Callback.Connect
        (Root_Window.Add_Network,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Add_Network_Activate'Access));
      Add (Root_Window.Add_Menu, Root_Window.Add_Network);

      -- Add add_event analyzer Submenu
      --
      Gtk_New
        (Root_Window.Add_Event_Analyzer,
         To_String (Lb_Add_Event_Analyzer (Current_Language)));
      Set_Right_Justify (Root_Window.Add_Event_Analyzer, False);
      Menu_Item_Callback.Connect
        (Root_Window.Add_Event_Analyzer,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Add_Event_Analyzer_Activate'Access));
      Add (Root_Window.Add_Menu, Root_Window.Add_Event_Analyzer);

      -- Create Delete Submenu
      --
      Gtk_New (Root_Window.Delete, To_String (Lb_Delete (Current_Language)));
      Set_Right_Justify (Root_Window.Delete, False);
      Add (Root_Window.Edit_Menu, Root_Window.Delete);

      Gtk_New (Root_Window.Delete_Menu);
      Set_Submenu (Root_Window.Delete, Root_Window.Delete_Menu);

      -- Add Delete_Processor to Delete_Menu
      --
      Gtk_New
        (Root_Window.Delete_Processor,
         To_String (Lb_Delete_Processor (Current_Language)));
      Set_Right_Justify (Root_Window.Delete_Processor, False);
      Menu_Item_Callback.Connect
        (Root_Window.Delete_Processor,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Delete_Processor_Activate'Access));
      Add (Root_Window.Delete_Menu, Root_Window.Delete_Processor);

      -- Add Delete_address_space to Delete_Menu
      --
      Gtk_New
        (Root_Window.Delete_Address_Space,
         To_String (Lb_Delete_Address_Space (Current_Language)));
      Set_Right_Justify (Root_Window.Delete_Address_Space, False);
      Menu_Item_Callback.Connect
        (Root_Window.Delete_Address_Space,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Delete_Address_Space_Activate'Access));
      Add (Root_Window.Delete_Menu, Root_Window.Delete_Address_Space);

      -- Add Delete_Task to Delete_Menu
      --
      Gtk_New
        (Root_Window.Delete_Task,
         To_String (Lb_Delete_Task (Current_Language)));
      Set_Right_Justify (Root_Window.Delete_Task, False);
      Menu_Item_Callback.Connect
        (Root_Window.Delete_Task,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Delete_Task_Activate'Access));
      Add (Root_Window.Delete_Menu, Root_Window.Delete_Task);

      -- Add Delete_Resource to Delete_Menu
      --
      Gtk_New
        (Root_Window.Delete_Resource,
         To_String (Lb_Delete_Resource (Current_Language)));
      Set_Right_Justify (Root_Window.Delete_Resource, False);
      Menu_Item_Callback.Connect
        (Root_Window.Delete_Resource,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Delete_Resource_Activate'Access));
      Add (Root_Window.Delete_Menu, Root_Window.Delete_Resource);

      -- Add Delete_Message to Delete_Menu
      --
      Gtk_New
        (Root_Window.Delete_Message,
         To_String (Lb_Delete_Message (Current_Language)));
      Set_Right_Justify (Root_Window.Delete_Message, False);
      Menu_Item_Callback.Connect
        (Root_Window.Delete_Message,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Delete_Message_Activate'Access));
      Add (Root_Window.Delete_Menu, Root_Window.Delete_Message);

      -- Add Delete_Buffer to Delete_Menu
      --
      Gtk_New
        (Root_Window.Delete_Buffer,
         To_String (Lb_Delete_Buffer (Current_Language)));
      Set_Right_Justify (Root_Window.Delete_Buffer, False);
      Menu_Item_Callback.Connect
        (Root_Window.Delete_Buffer,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Delete_Buffer_Activate'Access));
      Add (Root_Window.Delete_Menu, Root_Window.Delete_Buffer);

      -- Add Delete_event_analyzer sub menu
      --
      Gtk_New
        (Root_Window.Delete_Event_Analyzer,
         To_String (Lb_Delete_Event_Analyzer (Current_Language)));
      Set_Right_Justify (Root_Window.Delete_Event_Analyzer, False);
      Menu_Item_Callback.Connect
        (Root_Window.Delete_Event_Analyzer,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Delete_Event_Analyzer_Activate'Access));
      Add (Root_Window.Delete_Menu, Root_Window.Delete_Event_Analyzer);

      -- Add Delete_Network Processor to Delete_Menu
      --
      Gtk_New
        (Root_Window.Delete_Network,
         To_String (Lb_Delete_Network (Current_Language)));
      Set_Right_Justify (Root_Window.Delete_Network, False);
      Menu_Item_Callback.Connect
        (Root_Window.Delete_Network,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Delete_Network_Activate'Access));
      Add (Root_Window.Delete_Menu, Root_Window.Delete_Network);

      -- Create Update Submenu
      --
      Gtk_New (Root_Window.Update, To_String (Lb_Update (Current_Language)));
      Set_Right_Justify (Root_Window.Update, False);
      Add (Root_Window.Edit_Menu, Root_Window.Update);

      Gtk_New (Root_Window.Update_Menu);
      Set_Submenu (Root_Window.Update, Root_Window.Update_Menu);

      -- Add Update_Processor to Update Submenu
      --
      Gtk_New
        (Root_Window.Update_Processor,
         To_String (Lb_Update_Processor (Current_Language)));
      Set_Right_Justify (Root_Window.Update_Processor, False);
      Menu_Item_Callback.Connect
        (Root_Window.Update_Processor,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Update_Processor_Activate'Access));
      Add (Root_Window.Update_Menu, Root_Window.Update_Processor);

      -- Add Update_address_space to Update Submenu
      --
      Gtk_New
        (Root_Window.Update_Address_Space,
         To_String (Lb_Update_Address_Space (Current_Language)));
      Set_Right_Justify (Root_Window.Update_Address_Space, False);
      Menu_Item_Callback.Connect
        (Root_Window.Update_Address_Space,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Update_Address_Space_Activate'Access));
      Add (Root_Window.Update_Menu, Root_Window.Update_Address_Space);

      -- Add Update_Task to Update Submenu
      --
      Gtk_New
        (Root_Window.Update_Task,
         To_String (Lb_Update_Task (Current_Language)));
      Set_Right_Justify (Root_Window.Update_Task, False);
      Menu_Item_Callback.Connect
        (Root_Window.Update_Task,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Update_Task_Activate'Access));
      Add (Root_Window.Update_Menu, Root_Window.Update_Task);

      -- Add Update_Resource to Update_Menu
      --
      Gtk_New
        (Root_Window.Update_Resource,
         To_String (Lb_Update_Resource (Current_Language)));
      Set_Right_Justify (Root_Window.Update_Resource, False);
      Menu_Item_Callback.Connect
        (Root_Window.Update_Resource,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Update_Resource_Activate'Access));
      Add (Root_Window.Update_Menu, Root_Window.Update_Resource);

      -- Add Update_Message to Update_Menu
      --
      Gtk_New
        (Root_Window.Update_Message,
         To_String (Lb_Update_Message (Current_Language)));
      Set_Right_Justify (Root_Window.Update_Message, False);
      Menu_Item_Callback.Connect
        (Root_Window.Update_Message,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Update_Message_Activate'Access));
      Add (Root_Window.Update_Menu, Root_Window.Update_Message);

      -- Add Update_Buffer to Update_Menu
      --
      Gtk_New
        (Root_Window.Update_Buffer,
         To_String (Lb_Update_Buffer (Current_Language)));
      Set_Right_Justify (Root_Window.Update_Buffer, False);
      Menu_Item_Callback.Connect
        (Root_Window.Update_Buffer,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Update_Buffer_Activate'Access));
      Add (Root_Window.Update_Menu, Root_Window.Update_Buffer);

      -- Add Update_Event_analyzer sub menu
      --
      Gtk_New
        (Root_Window.Update_Event_Analyzer,
         To_String (Lb_Update_Event_Analyzer (Current_Language)));
      Set_Right_Justify (Root_Window.Update_Event_Analyzer, False);
      Menu_Item_Callback.Connect
        (Root_Window.Update_Event_Analyzer,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Update_Event_Analyzer_Activate'Access));
      Add (Root_Window.Update_Menu, Root_Window.Update_Event_Analyzer);

      -- Add Update_Network to Update_Menu
      --
      Gtk_New
        (Root_Window.Update_Network,
         To_String (Lb_Update_Network (Current_Language)));
      Set_Right_Justify (Root_Window.Update_Network, False);
      Menu_Item_Callback.Connect
        (Root_Window.Update_Network,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Update_Network_Activate'Access));
      Add (Root_Window.Update_Menu, Root_Window.Update_Network);

      -- Create Duplicate Submenu
      --
      Gtk_New (Root_Window.Separator1);
      Set_Right_Justify (Root_Window.Separator1, False);
      Add (Root_Window.Edit_Menu, Root_Window.Separator1);

      Gtk_New
        (Root_Window.Duplicate_Object,
         To_String (Lb_Duplicate (Current_Language)));
      Set_Right_Justify (Root_Window.Duplicate_Object, False);
      Menu_Item_Callback.Connect
        (Root_Window.Duplicate_Object,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Duplicate_Activate'Access));
      Add (Root_Window.Edit_Menu, Root_Window.Duplicate_Object);

      -- View sub menu
      --
      Gtk_New (Root_Window.Separator3);
      Gtk_New (Root_Window.Show, To_String (Lb_View (Current_Language)));
      Set_Right_Justify (Root_Window.Show, False);
      Add (Root_Window.Menubar, Root_Window.Show);

      Gtk_New (Root_Window.Show_Menu);
      Set_Submenu (Root_Window.Show, Root_Window.Show_Menu);

      --  Add List to Show_Menu
      --
      Gtk_New (Root_Window.List, To_String (Lb_List (Current_Language)));
      Set_Right_Justify (Root_Window.List, False);
      Add (Root_Window.Show_Menu, Root_Window.List);

      Gtk_New (Root_Window.List_Menu);
      Set_Submenu (Root_Window.List, Root_Window.List_Menu);

      --  Add Processors_List to List_Menu
      --
      Gtk_New
        (Root_Window.Processors_List,
         To_String (Lb_Processors_List (Current_Language)));
      Set_Right_Justify (Root_Window.Processors_List, False);
      Menu_Item_Callback.Connect
        (Root_Window.Processors_List,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Processors_List_Activate'Access));
      Add (Root_Window.List_Menu, Root_Window.Processors_List);

      --  Add Address_Space_List to List_Menu
      --
      Gtk_New
        (Root_Window.Address_Spaces_List,
         To_String (Lb_Address_Spaces_List (Current_Language)));
      Set_Right_Justify (Root_Window.Address_Spaces_List, False);
      Menu_Item_Callback.Connect
        (Root_Window.Address_Spaces_List,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Address_Spaces_List_Activate'Access));
      Add (Root_Window.List_Menu, Root_Window.Address_Spaces_List);

      --  Add Tasks_List to List_Menu
      --
      Gtk_New
        (Root_Window.Tasks_List,
         To_String (Lb_Tasks_List (Current_Language)));
      Set_Right_Justify (Root_Window.Tasks_List, False);
      Menu_Item_Callback.Connect
        (Root_Window.Tasks_List,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Tasks_List_Activate'Access));
      Add (Root_Window.List_Menu, Root_Window.Tasks_List);

      --  Add Resources_List to List_Menu
      --
      Gtk_New
        (Root_Window.Resources_List,
         To_String (Lb_Resources_List (Current_Language)));
      Set_Right_Justify (Root_Window.Resources_List, False);
      Menu_Item_Callback.Connect
        (Root_Window.Resources_List,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Resource_List_Activate'Access));
      Add (Root_Window.List_Menu, Root_Window.Resources_List);

      --  Add Networks_List to List_Menu
      --
      Gtk_New
        (Root_Window.Networks_List,
         To_String (Lb_Networks_List (Current_Language)));
      Set_Right_Justify (Root_Window.Networks_List, False);
      Menu_Item_Callback.Connect
        (Root_Window.Networks_List,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Network_List_Activate'Access));
      Add (Root_Window.List_Menu, Root_Window.Networks_List);

      --  Add Messages_List to List_Menu
      --
      Gtk_New
        (Root_Window.Messages_List,
         To_String (Lb_Messages_List (Current_Language)));
      Set_Right_Justify (Root_Window.Messages_List, False);
      Menu_Item_Callback.Connect
        (Root_Window.Messages_List,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Message_List_Activate'Access));
      Add (Root_Window.List_Menu, Root_Window.Messages_List);

      --  Add Buffers_List to List_Menu
      --
      Gtk_New
        (Root_Window.Buffers_List,
         To_String (Lb_Buffers_List (Current_Language)));
      Set_Right_Justify (Root_Window.Buffers_List, False);
      Menu_Item_Callback.Connect
        (Root_Window.Buffers_List,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Buffer_List_Activate'Access));
      Add (Root_Window.List_Menu, Root_Window.Buffers_List);

      --  Add Event_analyzers_List to List_Menu
      --
      Gtk_New
        (Root_Window.Event_Analyzers_List,
         To_String (Lb_Event_Analyzers_List (Current_Language)));
      Set_Right_Justify (Root_Window.Event_Analyzers_List, False);
      Menu_Item_Callback.Connect
        (Root_Window.Event_Analyzers_List,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Event_Analyzer_List_Activate'Access));
      Add (Root_Window.List_Menu, Root_Window.Event_Analyzers_List);

      ------------------------------------------------------
      --  Tool sub menu
      ------------------------------------------------------

      Gtk_New (Root_Window.Tools, To_String (Lb_Tools (Current_Language)));
      Set_Right_Justify (Root_Window.Tools, False);
      Menu_Item_Callback.Connect
        (Root_Window.Tools,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Tools_Activate'Access));
      Add (Root_Window.Menubar, Root_Window.Tools);

      Gtk_New (Root_Window.Tools_Menu);
      Set_Submenu (Root_Window.Tools, Root_Window.Tools_Menu);

      Gtk_New
        (Root_Window.Clear_Work_Space,
         To_String (Lb_Clear (Current_Language)));
      Set_Right_Justify (Root_Window.Clear_Work_Space, False);
      Menu_Item_Callback.Connect
        (Root_Window.Clear_Work_Space,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Clear_Work_Space_Activate'Access));
      Add (Root_Window.Tools_Menu, Root_Window.Clear_Work_Space);

      Gtk_New
        (Root_Window.Scheduling,
         To_String (Lb_Scheduling (Current_Language)));
      Set_Right_Justify (Root_Window.Scheduling, False);
      Add (Root_Window.Tools_Menu, Root_Window.Scheduling);

      -- Add Submenu Scheduling
      --
      Gtk_New (Root_Window.Scheduling_Menu);
      Set_Submenu (Root_Window.Scheduling, Root_Window.Scheduling_Menu);

      Gtk_New
        (Root_Window.Draw_Scheduling_Graph,
         To_String (Lb_Customized_Scheduling_Simulation (Current_Language)));
      Set_Right_Justify (Root_Window.Draw_Scheduling_Graph, False);
      Menu_Item_Callback.Connect
        (Root_Window.Draw_Scheduling_Graph,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Draw_Scheduling_Graph_Activate'Access));
      Add (Root_Window.Scheduling_Menu, Root_Window.Draw_Scheduling_Graph);

      Gtk_New
        (Root_Window.Check_Scheduling,
         To_String (Lb_Customized_Feasibility_Tests (Current_Language)));
      Set_Right_Justify (Root_Window.Check_Scheduling, False);
      Menu_Item_Callback.Connect
        (Root_Window.Check_Scheduling,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Check_Scheduling_Activate'Access));
      Add (Root_Window.Scheduling_Menu, Root_Window.Check_Scheduling);

      Gtk_New
        (Root_Window.Rm_Priorities,
         To_String (Lb_Set_Priorities_Rm (Current_Language)));
      Set_Right_Justify (Root_Window.Rm_Priorities, False);
      Menu_Item_Callback.Connect
        (Root_Window.Rm_Priorities,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Rm_Priorities_Activate'Access));
      Add (Root_Window.Scheduling_Menu, Root_Window.Rm_Priorities);

      Gtk_New
        (Root_Window.Dm_Priorities,
         To_String (Lb_Set_Priorities_Dm (Current_Language)));
      Set_Right_Justify (Root_Window.Dm_Priorities, False);
      Menu_Item_Callback.Connect
        (Root_Window.Dm_Priorities,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Dm_Priorities_Activate'Access));
      Add (Root_Window.Scheduling_Menu, Root_Window.Dm_Priorities);

      ----------------------------------
      -- Event table sub menu
      ----------------------------------

      Gtk_New
        (Root_Window.Event_Table_Services,
         To_String (Lb_Event_Table_Services (Current_Language)));
      Set_Right_Justify (Root_Window.Event_Table_Services, False);
      Add (Root_Window.Scheduling_Menu, Root_Window.Event_Table_Services);

      Gtk_New (Root_Window.Event_Table_Services_Menu);
      Set_Submenu
        (Root_Window.Event_Table_Services,
         Root_Window.Event_Table_Services_Menu);

      Gtk_New
        (Root_Window.Compute_Scheduling_And_Generate_Event_Table,
         To_String
            (Lb_Compute_Scheduling_And_Generate_Event_Table (Current_Language))
);
      Set_Right_Justify
        (Root_Window.Compute_Scheduling_And_Generate_Event_Table,
         False);
      Menu_Item_Callback.Connect
        (Root_Window.Compute_Scheduling_And_Generate_Event_Table,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Compute_Scheduling_And_Generate_Event_Table_Activate'Access));
      Add
        (Root_Window.Event_Table_Services_Menu,
         Root_Window.Compute_Scheduling_And_Generate_Event_Table);

      Gtk_New
        (Root_Window.Draw_Time_Line_From_Event_Table,
         To_String (Lb_Draw_Time_Line_From_Event_Table (Current_Language)));
      Set_Right_Justify (Root_Window.Draw_Time_Line_From_Event_Table, False);
      Menu_Item_Callback.Connect
        (Root_Window.Draw_Time_Line_From_Event_Table,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Draw_Time_Line_From_Event_Table_Activate'Access));
      Add
        (Root_Window.Event_Table_Services_Menu,
         Root_Window.Draw_Time_Line_From_Event_Table);

      Gtk_New
        (Root_Window.Run_Analysis_On_Event_Table,
         To_String (Lb_Run_Analysis_On_Event_Table (Current_Language)));
      Set_Right_Justify (Root_Window.Run_Analysis_On_Event_Table, False);
      Menu_Item_Callback.Connect
        (Root_Window.Run_Analysis_On_Event_Table,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Run_Analysis_On_Event_Table_Activate'Access));
      Add
        (Root_Window.Event_Table_Services_Menu,
         Root_Window.Run_Analysis_On_Event_Table);

      Gtk_New
        (Root_Window.Save_Event_Table,
         To_String (Lb_Export_Event_Table (Current_Language)));
      Set_Right_Justify (Root_Window.Save_Event_Table, False);
      Menu_Item_Callback.Connect
        (Root_Window.Save_Event_Table,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Save_Event_Table_Activate'Access));
      Add
        (Root_Window.Event_Table_Services_Menu,
         Root_Window.Save_Event_Table);

      Gtk_New
        (Root_Window.Load_Event_Table,
         To_String (Lb_Import_Event_Table (Current_Language)));
      Set_Right_Justify (Root_Window.Load_Event_Table, False);
      Menu_Item_Callback.Connect
        (Root_Window.Load_Event_Table,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Load_Event_Table_Activate'Access));
      Add
        (Root_Window.Event_Table_Services_Menu,
         Root_Window.Load_Event_Table);

      -- Add Submenu Partition
      --

      Gtk_New
        (Root_Window.Partition,
         To_String (Lb_Partition (Current_Language)));
      Set_Right_Justify (Root_Window.Partition, False);
      Add (Root_Window.Scheduling_Menu, Root_Window.Partition);

      Gtk_New (Root_Window.Partition_Menu);
      Set_Submenu (Root_Window.Partition, Root_Window.Partition_Menu);

      Gtk_New
        (Root_Window.Partition_Best_Fit,
         To_String (Lb_Partition_Best_Fit (Current_Language)));
      Set_Right_Justify (Root_Window.Partition_Best_Fit, False);
      Menu_Item_Callback.Connect
        (Root_Window.Partition_Best_Fit,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Partition_Best_Fit_Activate'Access));
      Add (Root_Window.Partition_Menu, Root_Window.Partition_Best_Fit);

      Gtk_New
        (Root_Window.Partition_General_Task,
         To_String (Lb_Partition_General_Task (Current_Language)));
      Set_Right_Justify (Root_Window.Partition_General_Task, False);
      Menu_Item_Callback.Connect
        (Root_Window.Partition_General_Task,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Partition_General_Task_Activate'Access));
      Add (Root_Window.Partition_Menu, Root_Window.Partition_General_Task);

      Gtk_New
        (Root_Window.Partition_Next_Fit,
         To_String (Lb_Partition_Next_Fit (Current_Language)));
      Set_Right_Justify (Root_Window.Partition_Next_Fit, False);
      Menu_Item_Callback.Connect
        (Root_Window.Partition_Next_Fit,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Partition_Next_Fit_Activate'Access));
      Add (Root_Window.Partition_Menu, Root_Window.Partition_Next_Fit);

      Gtk_New
        (Root_Window.Partition_First_Fit,
         To_String (Lb_Partition_First_Fit (Current_Language)));
      Set_Right_Justify (Root_Window.Partition_First_Fit, False);
      Menu_Item_Callback.Connect
        (Root_Window.Partition_First_Fit,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Partition_First_Fit_Activate'Access));
      Add (Root_Window.Partition_Menu, Root_Window.Partition_First_Fit);

      Gtk_New
        (Root_Window.Partition_Small_Task,
         To_String (Lb_Partition_Small_Task (Current_Language)));
      Set_Right_Justify (Root_Window.Partition_Small_Task, False);
      Menu_Item_Callback.Connect
        (Root_Window.Partition_Small_Task,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Partition_Small_Task_Activate'Access));
      Add (Root_Window.Partition_Menu, Root_Window.Partition_Small_Task);

      -- Add scheduling options sub menu
      --

      Gtk_New (Root_Window.Separator1);
      Set_Right_Justify (Root_Window.Separator1, False);
      Add (Root_Window.Scheduling_Menu, Root_Window.Separator1);

      Gtk_New (Root_Window.Options, "Options");
      Set_Right_Justify (Root_Window.Options, False);
      Menu_Item_Callback.Connect
        (Root_Window.Options,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Options_Activate'Access));
      Add (Root_Window.Scheduling_Menu, Root_Window.Options);

      -- Add Resources submenu
      --
      Gtk_New
        (Root_Window.Tools_Resource,
         To_String (Lb_Resource (Current_Language)));
      Set_Right_Justify (Root_Window.Tools_Resource, False);
      Add (Root_Window.Tools_Menu, Root_Window.Tools_Resource);

      Gtk_New (Root_Window.Tools_Resource_Menu);
      Set_Submenu
        (Root_Window.Tools_Resource,
         Root_Window.Tools_Resource_Menu);

      Gtk_New
        (Root_Window.Blocking_Time,
         To_String (Lb_Bound_Blocking_Time (Current_Language)));
      Set_Right_Justify (Root_Window.Blocking_Time, False);
      Add (Root_Window.Tools_Resource_Menu, Root_Window.Blocking_Time);

      Gtk_New (Root_Window.Blocking_Time_Menu);
      Set_Submenu (Root_Window.Blocking_Time, Root_Window.Blocking_Time_Menu);

      Gtk_New
        (Root_Window.Blocking_Time_Display,
         To_String (Lb_Compute_And_Display (Current_Language)));
      Set_Right_Justify (Root_Window.Blocking_Time_Display, False);
      Menu_Item_Callback.Connect
        (Root_Window.Blocking_Time_Display,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Blocking_Time_Display_Callback'Access));
      Add (Root_Window.Blocking_Time_Menu, Root_Window.Blocking_Time_Display);

      Gtk_New
        (Root_Window.Blocking_Time_Update,
         To_String (Lb_Compute_And_Update (Current_Language)));
      Set_Right_Justify (Root_Window.Blocking_Time_Update, False);
      Menu_Item_Callback.Connect
        (Root_Window.Blocking_Time_Update,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Blocking_Time_Update_Callback'Access));
      Add (Root_Window.Blocking_Time_Menu, Root_Window.Blocking_Time_Update);

      Gtk_New
        (Root_Window.Priority_Inversion,
         To_String (Lb_Priority_Inversion_Detection (Current_Language)));
      Set_Right_Justify (Root_Window.Priority_Inversion, False);
      Menu_Item_Callback.Connect
        (Root_Window.Priority_Inversion,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Priority_Inversion_Callback'Access));
      Add (Root_Window.Tools_Resource_Menu, Root_Window.Priority_Inversion);

      Gtk_New
        (Root_Window.Deadlock,
         To_String (Lb_Deadlock_Detection (Current_Language)));
      Set_Right_Justify (Root_Window.Deadlock, False);
      Menu_Item_Callback.Connect
        (Root_Window.Deadlock,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Deadlock_Callback'Access));
      Add (Root_Window.Tools_Resource_Menu, Root_Window.Deadlock);

      -- Add buffers submenu
      --
      Gtk_New
        (Root_Window.Tools_Buffer,
         To_String (Lb_Buffer (Current_Language)));
      Set_Right_Justify (Root_Window.Tools_Buffer, False);
      Add (Root_Window.Tools_Menu, Root_Window.Tools_Buffer);

      Gtk_New (Root_Window.Tools_Buffer_Menu);
      Set_Submenu (Root_Window.Tools_Buffer, Root_Window.Tools_Buffer_Menu);

      Gtk_New
        (Root_Window.Compute_Buffer,
         To_String (Lb_Compute_Buffer (Current_Language)));
      Set_Right_Justify (Root_Window.Compute_Buffer, False);
      Menu_Item_Callback.Connect
        (Root_Window.Compute_Buffer,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Check_Buffer_Activate'Access));
      Add (Root_Window.Tools_Buffer_Menu, Root_Window.Compute_Buffer);

      Gtk_New
        (Root_Window.Draw_Buffer,
         To_String (Lb_Draw_Buffer (Current_Language)));
      Set_Right_Justify (Root_Window.Draw_Buffer, False);
      Menu_Item_Callback.Connect
        (Root_Window.Draw_Buffer,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Draw_Buffer_Activate'Access));
      Add (Root_Window.Tools_Buffer_Menu, Root_Window.Draw_Buffer);

      -- Add precedency submenu
      --
      Gtk_New
        (Root_Window.Tools_Dependencies,
         To_String (Lb_Precedencies (Current_Language)));
      Set_Right_Justify (Root_Window.Tools_Dependencies, False);
      Add (Root_Window.Tools_Menu, Root_Window.Tools_Dependencies);

      Gtk_New (Root_Window.Tools_Dependencies_Menu);
      Set_Submenu
        (Root_Window.Tools_Dependencies,
         Root_Window.Tools_Dependencies_Menu);

      Gtk_New
        (Root_Window.Chetto_Priority,
         To_String (Lb_Chetto_Priority (Current_Language)));
      Set_Right_Justify (Root_Window.Chetto_Priority, False);
      Add (Root_Window.Tools_Dependencies_Menu, Root_Window.Chetto_Priority);

      Gtk_New (Root_Window.Chetto_Priority_Menu);
      Set_Submenu
        (Root_Window.Chetto_Priority,
         Root_Window.Chetto_Priority_Menu);

      Gtk_New
        (Root_Window.Chetto_Priority_Display,
         To_String (Lb_Compute_And_Display (Current_Language)));
      Set_Right_Justify (Root_Window.Chetto_Priority_Display, False);
      Menu_Item_Callback.Connect
        (Root_Window.Chetto_Priority_Display,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Chetto_Priority_Display_Callback'Access));
      Add
        (Root_Window.Chetto_Priority_Menu,
         Root_Window.Chetto_Priority_Display);

      Gtk_New
        (Root_Window.Chetto_Priority_Update,
         To_String (Lb_Compute_And_Update (Current_Language)));
      Set_Right_Justify (Root_Window.Chetto_Priority_Update, False);
      Menu_Item_Callback.Connect
        (Root_Window.Chetto_Priority_Update,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Chetto_Priority_Update_Callback'Access));
      Add
        (Root_Window.Chetto_Priority_Menu,
         Root_Window.Chetto_Priority_Update);

      Gtk_New
        (Root_Window.Chetto_Deadline,
         To_String (Lb_Chetto_Deadline (Current_Language)));
      Set_Right_Justify (Root_Window.Chetto_Deadline, False);
      Add (Root_Window.Tools_Dependencies_Menu, Root_Window.Chetto_Deadline);

      Gtk_New (Root_Window.Chetto_Deadline_Menu);
      Set_Submenu
        (Root_Window.Chetto_Deadline,
         Root_Window.Chetto_Deadline_Menu);

      Gtk_New
        (Root_Window.Chetto_Deadline_Display,
         To_String (Lb_Compute_And_Display (Current_Language)));
      Set_Right_Justify (Root_Window.Chetto_Deadline_Display, False);
      Menu_Item_Callback.Connect
        (Root_Window.Chetto_Deadline_Display,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Chetto_Deadline_Display_Callback'Access));
      Add
        (Root_Window.Chetto_Deadline_Menu,
         Root_Window.Chetto_Deadline_Display);

      Gtk_New
        (Root_Window.Chetto_Deadline_Update,
         To_String (Lb_Compute_And_Update (Current_Language)));
      Set_Right_Justify (Root_Window.Chetto_Deadline_Update, False);
      Menu_Item_Callback.Connect
        (Root_Window.Chetto_Deadline_Update,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Chetto_Deadline_Update_Callback'Access));
      Add
        (Root_Window.Chetto_Deadline_Menu,
         Root_Window.Chetto_Deadline_Update);

      Gtk_New
        (Root_Window.Jitter_From_Response_Time,
         To_String (Lb_Jitter_From_Response_Time (Current_Language)));
      Set_Right_Justify (Root_Window.Jitter_From_Response_Time, False);
      Add
        (Root_Window.Tools_Dependencies_Menu,
         Root_Window.Jitter_From_Response_Time);

      Gtk_New (Root_Window.Jitter_From_Response_Time_Menu);
      Set_Submenu
        (Root_Window.Jitter_From_Response_Time,
         Root_Window.Jitter_From_Response_Time_Menu);

      Gtk_New
        (Root_Window.Jitter_From_Response_Time_Display_One_Step,
         To_String (Lb_Compute_And_Display_One_Step (Current_Language)));
      Set_Right_Justify
        (Root_Window.Jitter_From_Response_Time_Display_One_Step,
         False);
      Menu_Item_Callback.Connect
        (Root_Window.Jitter_From_Response_Time_Display_One_Step,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Jitter_From_Response_Time_Display_One_Step_Callback'Access));
      Add
        (Root_Window.Jitter_From_Response_Time_Menu,
         Root_Window.Jitter_From_Response_Time_Display_One_Step);

      Gtk_New
        (Root_Window.Jitter_From_Response_Time_Update_One_Step,
         To_String (Lb_Compute_And_Update_One_Step (Current_Language)));
      Set_Right_Justify
        (Root_Window.Jitter_From_Response_Time_Update_One_Step,
         False);
      Menu_Item_Callback.Connect
        (Root_Window.Jitter_From_Response_Time_Update_One_Step,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Jitter_From_Response_Time_Update_One_Step_Callback'Access));
      Add
        (Root_Window.Jitter_From_Response_Time_Menu,
         Root_Window.Jitter_From_Response_Time_Update_One_Step);

      Gtk_New
        (Root_Window.Jitter_From_Response_Time_Display,
         To_String (Lb_Compute_And_Display (Current_Language)));
      Set_Right_Justify
        (Root_Window.Jitter_From_Response_Time_Display,
         False);
      Menu_Item_Callback.Connect
        (Root_Window.Jitter_From_Response_Time_Display,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Jitter_From_Response_Time_Display_Callback'Access));
      Add
        (Root_Window.Jitter_From_Response_Time_Menu,
         Root_Window.Jitter_From_Response_Time_Display);

      Gtk_New
        (Root_Window.Jitter_From_Response_Time_Update,
         To_String (Lb_Compute_And_Update (Current_Language)));
      Set_Right_Justify (Root_Window.Jitter_From_Response_Time_Update, False);
      Menu_Item_Callback.Connect
        (Root_Window.Jitter_From_Response_Time_Update,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Jitter_From_Response_Time_Update_Callback'Access));
      Add
        (Root_Window.Jitter_From_Response_Time_Menu,
         Root_Window.Jitter_From_Response_Time_Update);

      -- Add memory requirement analysis sub menu
      --

      Gtk_New
        (Root_Window.Memory_Requirement,
         To_String (Lb_Memory_Requirement_Analysis (Current_Language)));
      Set_Right_Justify (Root_Window.Memory_Requirement, False);
      Add (Root_Window.Tools_Menu, Root_Window.Memory_Requirement);

      Gtk_New (Root_Window.Memory_Requirement_Menu);
      Set_Submenu
        (Root_Window.Memory_Requirement,
         Root_Window.Memory_Requirement_Menu);

      Gtk_New
        (Root_Window.Check_Address_Space,
         To_String (Lb_Check_Address_Space (Current_Language)));
      Set_Right_Justify (Root_Window.Check_Address_Space, False);
      Menu_Item_Callback.Connect
        (Root_Window.Check_Address_Space,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Check_Address_Space_Activate'Access));
      Add
        (Root_Window.Memory_Requirement_Menu,
         Root_Window.Check_Address_Space);

      Gtk_New
        (Root_Window.Check_And_Update_Address_Space,
         To_String (Lb_Check_And_Update_Address_Space (Current_Language)));
      Set_Right_Justify (Root_Window.Check_And_Update_Address_Space, False);
      Menu_Item_Callback.Connect
        (Root_Window.Check_And_Update_Address_Space,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Check_And_Update_Address_Space_Activate'Access));
      Add
        (Root_Window.Memory_Requirement_Menu,
         Root_Window.Check_And_Update_Address_Space);

      -- Add random submenu
      --
      Gtk_New
        (Root_Window.Tools_Random,
         To_String (Lb_Random (Current_Language)));
      Set_Right_Justify (Root_Window.Tools_Random, False);
      Add (Root_Window.Tools_Menu, Root_Window.Tools_Random);

      Gtk_New (Root_Window.Tools_Random_Menu);
      Set_Submenu (Root_Window.Tools_Random, Root_Window.Tools_Random_Menu);

      Gtk_New
        (Root_Window.Compute_Response_Time_Density,
         To_String (Lb_Compute_Response_Time_Density (Current_Language)));
      Set_Right_Justify (Root_Window.Compute_Response_Time_Density, False);
      Menu_Item_Callback.Connect
        (Root_Window.Compute_Response_Time_Density,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Compute_Response_Time_Density_Activate'Access));
      Add
        (Root_Window.Tools_Random_Menu,
         Root_Window.Compute_Response_Time_Density);

      -- Add Help Menu
      --
      Gtk_New (Root_Window.Help, To_String (Lb_Help (Current_Language)));
      Set_Right_Justify (Root_Window.Help, False);
      Menu_Item_Callback.Connect
        (Root_Window.Help,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Help_Activate'Access));
      Add (Root_Window.Menubar, Root_Window.Help);

      Gtk_New (Root_Window.Help_Menu);
      Set_Submenu (Root_Window.Help, Root_Window.Help_Menu);

      Gtk_New
        (Root_Window.About_Cheddar,
         To_String (Lb_About (Current_Language)));
      Set_Right_Justify (Root_Window.About_Cheddar, False);
      Menu_Item_Callback.Connect
        (Root_Window.About_Cheddar,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_About_Cheddar_Activate'Access));
      Add (Root_Window.Help_Menu, Root_Window.About_Cheddar);

      Gtk_New (Root_Window.Manual, To_String (Lb_Manual (Current_Language)));
      Set_Right_Justify (Root_Window.Manual, False);
      Menu_Item_Callback.Connect
        (Root_Window.Manual,
         "activate",
         Menu_Item_Callback.To_Marshaller (On_Manual_Activate'Access));
      Add (Root_Window.Help_Menu, Root_Window.Manual);

      Gtk_New
        (Root_Window.Bibliography_References,
         To_String (Lb_Ref (Current_Language)));
      Set_Right_Justify (Root_Window.Bibliography_References, False);
      Menu_Item_Callback.Connect
        (Root_Window.Bibliography_References,
         "activate",
         Menu_Item_Callback.To_Marshaller
            (On_Bibliography_References_Activate'Access));
      Add (Root_Window.Help_Menu, Root_Window.Bibliography_References);

      Gtk_New (Root_Window.Toolbar, Orientation_Horizontal, Toolbar_Icons);
      Set_Border_Width (Root_Window.Toolbar, 3);
      Set_Space_Size (Root_Window.Toolbar, 8);
      Set_Space_Style (Root_Window.Toolbar, Toolbar_Space_Line);
      Set_Tooltips (Root_Window.Toolbar, True);
      Set_Button_Relief (Root_Window.Toolbar, Relief_Normal);
      Pack_Start (Root_Window.Vbox, Root_Window.Toolbar, False, True, 0);

      Root_Window.Pixmap_New := Create_Pixmap (New_Xpm, Root_Window);
      Set_Alignment (Root_Window.Pixmap_New, 0.5, 0.5);
      Set_Padding (Root_Window.Pixmap_New, 0, 0);

      Toolbar_Callback.Object_Connect
        (Append_Item
            (Root_Window.Toolbar,
             Tooltip_Text => To_String (Lb_New (Current_Language)),
             Icon         => Gtk_Widget (Root_Window.Pixmap_New)),
         "clicked",
         Toolbar_Callback.To_Marshaller (On_Pixmap_New_Clicked'Access),
         Slot_Object => Root_Window.Toolbar);

      Root_Window.Pixmap_Open := Create_Pixmap (Open_Xpm, Root_Window);
      Set_Alignment (Root_Window.Pixmap_Open, 0.5, 0.5);
      Set_Padding (Root_Window.Pixmap_Open, 0, 0);

      Toolbar_Callback.Object_Connect
        (Append_Item
            (Root_Window.Toolbar,
             Tooltip_Text => To_String (Lb_Open_Xml (Current_Language)),
             Icon         => Gtk_Widget (Root_Window.Pixmap_Open)),
         "clicked",
         Toolbar_Callback.To_Marshaller (On_Pixmap_Open_Clicked'Access),
         Slot_Object => Root_Window.Toolbar);

      Root_Window.Pixmap_Save := Create_Pixmap (Save_Xpm, Root_Window);
      Set_Alignment (Root_Window.Pixmap_Save, 0.5, 0.5);
      Set_Padding (Root_Window.Pixmap_Save, 0, 0);

      Toolbar_Callback.Object_Connect
        (Append_Item
            (Root_Window.Toolbar,
             Tooltip_Text => To_String (Lb_Save_Xml (Current_Language)),
             Icon         => Gtk_Widget (Root_Window.Pixmap_Save)),
         "clicked",
         Toolbar_Callback.To_Marshaller (On_Pixmap_Save_Clicked'Access),
         Slot_Object => Root_Window.Toolbar);

      Root_Window.Pixmap_Save_As := Create_Pixmap (Save_As_Xpm, Root_Window);
      Set_Alignment (Root_Window.Pixmap_Save_As, 0.5, 0.5);
      Set_Padding (Root_Window.Pixmap_Save_As, 0, 0);

      Toolbar_Callback.Object_Connect
        (Append_Item
            (Root_Window.Toolbar,
             Tooltip_Text => To_String (Lb_Save_As_Xml (Current_Language)),
             Icon         => Gtk_Widget (Root_Window.Pixmap_Save_As)),
         "clicked",
         Toolbar_Callback.To_Marshaller (On_Pixmap_Save_As_Clicked'Access),
         Slot_Object => Root_Window.Toolbar);

      Root_Window.Pixmap_Exit := Create_Pixmap (Exit_Xpm, Root_Window);
      Set_Alignment (Root_Window.Pixmap_Exit, 0.5, 0.5);
      Set_Padding (Root_Window.Pixmap_Exit, 0, 0);

      Toolbar_Callback.Object_Connect
        (Append_Item
            (Root_Window.Toolbar,
             Tooltip_Text => To_String (Lb_Exit (Current_Language)),
             Icon         => Gtk_Widget (Root_Window.Pixmap_Exit)),
         "clicked",
         Toolbar_Callback.To_Marshaller (On_Pixmap_Exit_Clicked'Access),
         Slot_Object => Root_Window.Toolbar);

      Root_Window.Pixmap_Empty := Create_Pixmap (Empty_Xpm, Root_Window);
      Set_Alignment (Root_Window.Pixmap_Empty, 0.5, 0.5);
      Set_Padding (Root_Window.Pixmap_Empty, 0, 0);
      Add (Root_Window.Toolbar, Root_Window.Pixmap_Empty);

      Root_Window.Pixmap_Clear_Work_Space :=
         Create_Pixmap (Preferences_Xpm, Root_Window);
      Set_Alignment (Root_Window.Pixmap_Clear_Work_Space, 0.5, 0.5);
      Set_Padding (Root_Window.Pixmap_Clear_Work_Space, 0, 0);

      Toolbar_Callback.Object_Connect
        (Append_Item
            (Root_Window.Toolbar,
             Tooltip_Text => To_String (Lb_Clear (Current_Language)),
             Icon         => Gtk_Widget (Root_Window.Pixmap_Clear_Work_Space)),
         "clicked",
         Toolbar_Callback.To_Marshaller
            (On_Pixmap_Clear_Work_Space_Clicked'Access),
         Slot_Object => Root_Window.Toolbar);

      Root_Window.Pixmap_Dependencies :=
         Create_Pixmap (Rezo_Xpm, Root_Window);
      Set_Alignment (Root_Window.Pixmap_Dependencies, 0.5, 0.5);
      Set_Padding (Root_Window.Pixmap_Dependencies, 0, 0);

      Toolbar_Callback.Object_Connect
        (Append_Item
            (Root_Window.Toolbar,
             Tooltip_Text => To_String (Lb_Precedencies (Current_Language)),
             Icon         => Gtk_Widget (Root_Window.Pixmap_Dependencies)),
         "clicked",
         Toolbar_Callback.To_Marshaller
            (On_Pixmap_Dependencies_Clicked'Access),
         Slot_Object => Root_Window.Toolbar);

      Root_Window.Pixmap_Check_Scheduling :=
         Create_Pixmap (Clock_Xpm, Root_Window);
      Set_Alignment (Root_Window.Pixmap_Check_Scheduling, 0.5, 0.5);
      Set_Padding (Root_Window.Pixmap_Check_Scheduling, 0, 0);

      Toolbar_Callback.Object_Connect
        (Append_Item
            (Root_Window.Toolbar,
             Tooltip_Text =>
                To_String (Lb_Scheduling_Feasibility (Current_Language)),
             Icon         => Gtk_Widget (Root_Window.Pixmap_Check_Scheduling)),
         "clicked",
         Toolbar_Callback.To_Marshaller
            (On_Pixmap_Check_Scheduling_Clicked'Access),
         Slot_Object => Root_Window.Toolbar);

      Root_Window.Pixmap_Draw_Scheduling :=
         Create_Pixmap (Ordo_Xpm, Root_Window);
      Set_Alignment (Root_Window.Pixmap_Draw_Scheduling, 0.5, 0.5);
      Set_Padding (Root_Window.Pixmap_Draw_Scheduling, 0, 0);

      Toolbar_Callback.Object_Connect
        (Append_Item
            (Root_Window.Toolbar,
             Tooltip_Text =>
                To_String (Lb_Scheduling_Simulation (Current_Language)),
             Icon         => Gtk_Widget (Root_Window.Pixmap_Draw_Scheduling)),
         "clicked",
         Toolbar_Callback.To_Marshaller
            (On_Pixmap_Draw_Scheduling_Clicked'Access),
         Slot_Object => Root_Window.Toolbar);

      Gtk_New_Vpaned (Root_Window.Vpaned);
      Set_Border_Width (Root_Window.Vpaned, 5);
      Pack_Start (Root_Window.Vbox, Root_Window.Vpaned, True, True, 0);

      Gtk_New (Root_Window.Scrolledwindow1);
      Set_Policy
        (Root_Window.Scrolledwindow1,
         Policy_Automatic,
         Policy_Automatic);
      Show (Root_Window.Scrolledwindow1);
      Add1 (Root_Window.Vpaned, Root_Window.Scrolledwindow1);

      Gtk_New (Root_Window.Scrolledwindow2);
      Set_Policy
        (Root_Window.Scrolledwindow2,
         Policy_Automatic,
         Policy_Automatic);
      Add2 (Root_Window.Vpaned, Root_Window.Scrolledwindow2);
      Set_Position (Root_Window.Vpaned, 200);

      Gtk_New (Draw_Scheduling);
      Add_With_Viewport (Root_Window.Scrolledwindow1, Draw_Scheduling.Buffer);

      Gtk_New (Root_Window.Text);
      Set_Editable (Root_Window.Text, False);
      Set_Line_Wrap (Root_Window.Text);
      Set_Word_Wrap (Root_Window.Text);
      Add (Root_Window.Scrolledwindow2, Root_Window.Text);

      Show_All (Root_Window);
      Clear_Time_Line (Draw_Scheduling);

   end Initialize;

end Root_Window_Pkg;
