------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Gtk.Box;             use Gtk.Box;
with Gtk.Menu_Bar;        use Gtk.Menu_Bar;
with Gtk.Menu_Item;       use Gtk.Menu_Item;
with Gtk.Menu;            use Gtk.Menu;
with Gtk.Toolbar;         use Gtk.Toolbar;
with Gtk.Scrolled_Window; use Gtk.Scrolled_Window;
with Gtk.Text;            use Gtk.Text;
with Gtk.Window;          use Gtk.Window;
with Gtk.Pixmap;          use Gtk.Pixmap;
with Gtk.Paned;           use Gtk.Paned;

package Root_Window_Pkg is

   type Root_Window_Record is new Gtk_Window_Record with private;
   type Root_Window_Access is access all Root_Window_Record'Class;

   procedure Gtk_New (Root_Window : out Root_Window_Access);
   procedure Initialize (Root_Window : access Root_Window_Record'Class);

   Root_Window : Root_Window_Access;

private

   type Root_Window_Record is new Gtk_Window_Record with record
      Vbox            : Gtk_Vbox;
      Vpaned          : Gtk_Vpaned;
      Scrolledwindow1 : Gtk_Scrolled_Window;
      Scrolledwindow2 : Gtk_Scrolled_Window;
      Text            : Gtk_Text;

      Separator1 : Gtk_Menu_Item;
      Separator2 : Gtk_Menu_Item;
      Separator3 : Gtk_Menu_Item;

      Menubar : Gtk_Menu_Bar;

      File         : Gtk_Menu_Item;
      File_Menu    : Gtk_Menu;
      New_File     : Gtk_Menu_Item;
      Open_File    : Gtk_Menu_Item;
      Save_File    : Gtk_Menu_Item;
      Save_As_File : Gtk_Menu_Item;
      Exit_Program : Gtk_Menu_Item;

      Aadl                                : Gtk_Menu_Item;
      Aadl_Menu                           : Gtk_Menu;
      Import_Aadl_File                    : Gtk_Menu_Item;
      Export_Aadl_File                    : Gtk_Menu_Item;
      Aadl_Options                        : Gtk_Menu_Item;
      Export_Cheddar_Property_Sets        : Gtk_Menu_Item;
      Export_Aadl_Standard_Properties_Set : Gtk_Menu_Item;

      Edit      : Gtk_Menu_Item;
      Edit_Menu : Gtk_Menu;

      Add                : Gtk_Menu_Item;
      Add_Menu           : Gtk_Menu;
      Add_Task           : Gtk_Menu_Item;
      Add_Processor      : Gtk_Menu_Item;
      Add_Resource       : Gtk_Menu_Item;
      Add_Message        : Gtk_Menu_Item;
      Add_Buffer         : Gtk_Menu_Item;
      Add_Network        : Gtk_Menu_Item;
      Add_Event_Analyzer : Gtk_Menu_Item;
      Add_Address_Space  : Gtk_Menu_Item;

      Update                : Gtk_Menu_Item;
      Update_Menu           : Gtk_Menu;
      Update_Task           : Gtk_Menu_Item;
      Update_Processor      : Gtk_Menu_Item;
      Update_Resource       : Gtk_Menu_Item;
      Update_Message        : Gtk_Menu_Item;
      Update_Buffer         : Gtk_Menu_Item;
      Update_Network        : Gtk_Menu_Item;
      Update_Event_Analyzer : Gtk_Menu_Item;
      Update_Address_Space  : Gtk_Menu_Item;

      Delete                : Gtk_Menu_Item;
      Delete_Menu           : Gtk_Menu;
      Delete_Task           : Gtk_Menu_Item;
      Delete_Processor      : Gtk_Menu_Item;
      Delete_Resource       : Gtk_Menu_Item;
      Delete_Message        : Gtk_Menu_Item;
      Delete_Buffer         : Gtk_Menu_Item;
      Delete_Network        : Gtk_Menu_Item;
      Delete_Event_Analyzer : Gtk_Menu_Item;
      Delete_Address_Space  : Gtk_Menu_Item;

      Duplicate_Object : Gtk_Menu_Item;
      Edit_Scheduler   : Gtk_Menu_Item;

      Tools      : Gtk_Menu_Item;
      Tools_Menu : Gtk_Menu;

      Parametric      : Gtk_Menu_Item;
      Parametric_Menu : Gtk_Menu;

      Scheduling                     : Gtk_Menu_Item;
      Scheduling_Menu                : Gtk_Menu;
      Draw_Scheduling_Graph          : Gtk_Menu_Item;
      Options                        : Gtk_Menu_Item;
      Memory_Requirement             : Gtk_Menu_Item;
      Memory_Requirement_Menu        : Gtk_Menu;
      Check_And_Update_Address_Space : Gtk_Menu_Item;
      Check_Address_Space            : Gtk_Menu_Item;
      Check_Scheduling               : Gtk_Menu_Item;
      Rm_Priorities                  : Gtk_Menu_Item;
      Dm_Priorities                  : Gtk_Menu_Item;
      Partition                      : Gtk_Menu_Item;
      Partition_Menu                 : Gtk_Menu;
      Partition_Best_Fit             : Gtk_Menu_Item;
      Partition_General_Task         : Gtk_Menu_Item;
      Partition_Next_Fit             : Gtk_Menu_Item;
      Partition_First_Fit            : Gtk_Menu_Item;
      Partition_Small_Task           : Gtk_Menu_Item;

      Clear_Work_Space : Gtk_Menu_Item;

      Tools_Resource        : Gtk_Menu_Item;
      Tools_Resource_Menu   : Gtk_Menu;
      Blocking_Time         : Gtk_Menu_Item;
      Blocking_Time_Menu    : Gtk_Menu;
      Blocking_Time_Update  : Gtk_Menu_Item;
      Blocking_Time_Display : Gtk_Menu_Item;
      Priority_Inversion    : Gtk_Menu_Item;
      Deadlock              : Gtk_Menu_Item;

      Event_Table_Services                        : Gtk_Menu_Item;
      Event_Table_Services_Menu                   : Gtk_Menu;
      Compute_Scheduling_And_Generate_Event_Table : Gtk_Menu_Item;
      Run_Analysis_On_Event_Table                 : Gtk_Menu_Item;
      Draw_Time_Line_From_Event_Table             : Gtk_Menu_Item;
      Load_Event_Table                            : Gtk_Menu_Item;
      Save_Event_Table                            : Gtk_Menu_Item;

      Tools_Dependencies                         : Gtk_Menu_Item;
      Tools_Dependencies_Menu                    : Gtk_Menu;
      Chetto_Priority                            : Gtk_Menu_Item;
      Chetto_Priority_Menu                       : Gtk_Menu;
      Chetto_Priority_Update                     : Gtk_Menu_Item;
      Chetto_Priority_Display                    : Gtk_Menu_Item;
      Chetto_Deadline                            : Gtk_Menu_Item;
      Chetto_Deadline_Menu                       : Gtk_Menu;
      Chetto_Deadline_Update                     : Gtk_Menu_Item;
      Chetto_Deadline_Display                    : Gtk_Menu_Item;
      Jitter_From_Response_Time                  : Gtk_Menu_Item;
      Jitter_From_Response_Time_Menu             : Gtk_Menu;
      Jitter_From_Response_Time_Update           : Gtk_Menu_Item;
      Jitter_From_Response_Time_Display          : Gtk_Menu_Item;
      Jitter_From_Response_Time_Update_One_Step  : Gtk_Menu_Item;
      Jitter_From_Response_Time_Display_One_Step : Gtk_Menu_Item;
      Tools_Buffer                               : Gtk_Menu_Item;
      Tools_Buffer_Menu                          : Gtk_Menu;
      Compute_Buffer                             : Gtk_Menu_Item;
      Draw_Buffer                                : Gtk_Menu_Item;

      Tools_Random                  : Gtk_Menu_Item;
      Tools_Random_Menu             : Gtk_Menu;
      Compute_Response_Time_Density : Gtk_Menu_Item;

      Show      : Gtk_Menu_Item;
      Show_Menu : Gtk_Menu;

      List                 : Gtk_Menu_Item;
      List_Menu            : Gtk_Menu;
      Tasks_List           : Gtk_Menu_Item;
      Processors_List      : Gtk_Menu_Item;
      Resources_List       : Gtk_Menu_Item;
      Messages_List        : Gtk_Menu_Item;
      Buffers_List         : Gtk_Menu_Item;
      Networks_List        : Gtk_Menu_Item;
      Event_Analyzers_List : Gtk_Menu_Item;
      Address_Spaces_List  : Gtk_Menu_Item;

      Help                    : Gtk_Menu_Item;
      Help_Menu               : Gtk_Menu;
      About_Cheddar           : Gtk_Menu_Item;
      Manual                  : Gtk_Menu_Item;
      Bibliography_References : Gtk_Menu_Item;

      Toolbar : Gtk_Toolbar;

      Pixmap_Empty            : Gtk_Pixmap;
      Pixmap_New              : Gtk_Pixmap;
      Pixmap_Open             : Gtk_Pixmap;
      Pixmap_Save             : Gtk_Pixmap;
      Pixmap_Save_As          : Gtk_Pixmap;
      Pixmap_Exit             : Gtk_Pixmap;
      Pixmap_Clear_Work_Space : Gtk_Pixmap;
      Pixmap_Dependencies     : Gtk_Pixmap;
      Pixmap_Check_Scheduling : Gtk_Pixmap;
      Pixmap_Draw_Scheduling  : Gtk_Pixmap;

      Canvas_Is_Created : Boolean;
   end record;

end Root_Window_Pkg;
