------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Project;                   use Project;
with Multiprocessor_Services;   use Multiprocessor_Services;
with Translate;                 use Translate;
with Call_Scheduling_Framework; use Call_Scheduling_Framework;
with Gtk;                       use Gtk;
with Gtk.Widget;                use Gtk.Widget;
with Gtk.Enums;                 use Gtk.Enums;
with Callbacks_Cheddar;         use Callbacks_Cheddar;
with Ada.IO_Exceptions;         use Ada.IO_Exceptions;
with Ada.Strings.Unbounded;     use Ada.Strings.Unbounded;
with Mmi_Tools;                 use Mmi_Tools;
with Framework_Config;          use Framework_Config;
with GNAT.Current_Exception;    use GNAT.Current_Exception;

package body Save_Event_Table_Pkg is

   procedure On_Ok_Pressed (Object : access Gtk_Button_Record'Class) is
   begin

      Xml_Event_Table_File_Name :=
         To_Unbounded_String (Get_Filename (save_event_table));

      Write_To_Xml_File
        (Call_Scheduling_Framework.Sched,
         Sys,
         Xml_Event_Table_File_Name);

   exception
      when Ada.IO_Exceptions.Name_Error =>
         Show_Message_Box
           (Lb_Can_Not_Save_File (Current_Language) &
            To_Unbounded_String (", Name_Error"));
      when Ada.IO_Exceptions.Status_Error =>
         Show_Message_Box
           (Lb_Can_Not_Save_File (Current_Language) &
            To_Unbounded_String (", Status_Error"));
      when Ada.IO_Exceptions.Mode_Error =>
         Show_Message_Box
           (Lb_Can_Not_Save_File (Current_Language) &
            To_Unbounded_String (", Mode_Error"));
      when Ada.IO_Exceptions.Use_Error =>
         Show_Message_Box
           (Lb_Can_Not_Save_File (Current_Language) &
            To_Unbounded_String (", Use_Error"));
      when Ada.IO_Exceptions.Device_Error =>
         Show_Message_Box
           (Lb_Can_Not_Save_File (Current_Language) &
            To_Unbounded_String (", Device_Error"));
      when Ada.IO_Exceptions.End_Error =>
         Show_Message_Box
           (Lb_Can_Not_Save_File (Current_Language) &
            To_Unbounded_String (", End_Error"));
      when Ada.IO_Exceptions.Data_Error =>
         Show_Message_Box
           (Lb_Can_Not_Save_File (Current_Language) &
            To_Unbounded_String (", Data_Error"));
      when Ada.IO_Exceptions.Layout_Error =>
         Show_Message_Box
           (Lb_Can_Not_Save_File (Current_Language) &
            To_Unbounded_String (", Layout_Error"));
      when others =>
         Show_Message_Box
           (Lb_Can_Not_Open_File (Current_Language) &
            " ; Exception name    : " &
            Exception_Name &
            " ; " &
            "Exception message : " &
            Exception_Message);

         Destroy (save_event_table);
   end On_Ok_Pressed;

   procedure On_cancel_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      Destroy (Get_Toplevel (Gtk_Widget (Object)));
   end On_cancel_Pressed;

   procedure Gtk_New (save_event_table : out save_event_table_Access) is

   begin
      save_event_table := new save_event_table_Record;
      Save_Event_Table_Pkg.Initialize (save_event_table);
   end Gtk_New;

   procedure Initialize
     (save_event_table : access save_event_table_Record'Class)
   is

   begin
      Gtk.File_Selection.Initialize
        (save_event_table,
         To_String
            (Lb_Save_Xml (Current_Language) &
             To_Unbounded_String (" ") &
             Lb_File (Current_Language)));
      Set_Show_File_Op_Buttons (save_event_table, True);
      Set_Border_Width (save_event_table, 10);
      Set_Title
        (save_event_table,
         To_String
            (Lb_Save_Xml (Current_Language) &
             To_Unbounded_String (" ") &
             Lb_File (Current_Language)));
      Set_Policy (save_event_table, False, True, False);
      Set_Position (save_event_table, Win_Pos_Center_Always);
      Set_Modal (save_event_table, True);

      save_event_table.Ok_Button := Get_Ok_Button (save_event_table);
      Set_Flags (save_event_table.Ok_Button, Can_Default);
      Button_Callback.Connect
        (save_event_table.Ok_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'Access));

      save_event_table.Cancel_Button := Get_Cancel_Button (save_event_table);
      Set_Flags (save_event_table.Cancel_Button, Can_Default);
      Button_Callback.Connect
        (save_event_table.Cancel_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_cancel_Pressed'Access));

   end Initialize;

end Save_Event_Table_Pkg;
