------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Translate;              use Translate;
with Gtk;                    use Gtk;
with Gtk.Widget;             use Gtk.Widget;
with Gtk.Enums;              use Gtk.Enums;
with Callbacks_Cheddar;      use Callbacks_Cheddar;
with Project;                use Project;
with Systems;                use Systems;
with Ada.IO_Exceptions;      use Ada.IO_Exceptions;
with Ada.Strings.Unbounded;  use Ada.Strings.Unbounded;
with Mmi_Tools;              use Mmi_Tools;
with Graph_Window_Pkg;       use Graph_Window_Pkg;
with My_Canvas;              use My_Canvas;
with Framework_Config;       use Framework_Config;
with GNAT.Current_Exception; use GNAT.Current_Exception;

package body Save_File_Selection_Pkg is

   procedure On_Ok_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      begin

         Xml_Project_File_Name :=
            To_Unbounded_String (Get_Filename (save_file_selection));

         if Graph_Window /= null then
            Get_Coordinates (Graph_Window.Buffer, Sys);
         end if;

         Systems.Write_To_Xml_File (Sys, Xml_Project_File_Name);

         Is_Saved := True;
         Is_New   := False;

      exception
         when Ada.IO_Exceptions.Name_Error =>
            Show_Message_Box
              (Lb_Can_Not_Save_File (Current_Language) &
               To_Unbounded_String (", Name_Error"));
         when Ada.IO_Exceptions.Status_Error =>
            Show_Message_Box
              (Lb_Can_Not_Save_File (Current_Language) &
               To_Unbounded_String (", Status_Error"));
         when Ada.IO_Exceptions.Mode_Error =>
            Show_Message_Box
              (Lb_Can_Not_Save_File (Current_Language) &
               To_Unbounded_String (", Mode_Error"));
         when Ada.IO_Exceptions.Use_Error =>
            Show_Message_Box
              (Lb_Can_Not_Save_File (Current_Language) &
               To_Unbounded_String (", Use_Error"));
         when Ada.IO_Exceptions.Device_Error =>
            Show_Message_Box
              (Lb_Can_Not_Save_File (Current_Language) &
               To_Unbounded_String (", Device_Error"));
         when Ada.IO_Exceptions.End_Error =>
            Show_Message_Box
              (Lb_Can_Not_Save_File (Current_Language) &
               To_Unbounded_String (", End_Error"));
         when Ada.IO_Exceptions.Data_Error =>
            Show_Message_Box
              (Lb_Can_Not_Save_File (Current_Language) &
               To_Unbounded_String (", Data_Error"));
         when Ada.IO_Exceptions.Layout_Error =>
            Show_Message_Box
              (Lb_Can_Not_Save_File (Current_Language) &
               To_Unbounded_String (", Layout_Error"));
         when others =>
            Show_Message_Box
              (Lb_Can_Not_Save_File (Current_Language) &
               " ; Exception name    : " &
               Exception_Name &
               " ; " &
               "Exception message : " &
               Exception_Message);
      end;

      Destroy (save_file_selection);
   end On_Ok_Pressed;

   procedure On_cancel_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      Destroy (Get_Toplevel (Gtk_Widget (Object)));
   end On_cancel_Pressed;

   procedure Gtk_New (save_file_selection : out save_file_selection_Access) is

   begin
      save_file_selection := new save_file_selection_Record;
      Save_File_Selection_Pkg.Initialize (save_file_selection);
   end Gtk_New;

   procedure Initialize
     (save_file_selection : access save_file_selection_Record'Class)
   is

   begin
      Gtk.File_Selection.Initialize
        (save_file_selection,
         To_String
            (Lb_Save_Xml (Current_Language) &
             To_Unbounded_String (" ") &
             Lb_File (Current_Language)));
      Set_Show_File_Op_Buttons (save_file_selection, True);
      Set_Border_Width (save_file_selection, 10);
      Set_Title
        (save_file_selection,
         To_String
            (Lb_Save_Xml (Current_Language) &
             To_Unbounded_String (" ") &
             Lb_File (Current_Language)));
      Set_Policy (save_file_selection, False, True, False);
      Set_Position (save_file_selection, Win_Pos_Center_Always);
      Set_Modal (save_file_selection, True);

      save_file_selection.Ok_Button := Get_Ok_Button (save_file_selection);
      Set_Flags (save_file_selection.Ok_Button, Can_Default);
      Button_Callback.Connect
        (save_file_selection.Ok_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'Access));

      save_file_selection.Cancel_Button :=
         Get_Cancel_Button (save_file_selection);
      Set_Flags (save_file_selection.Cancel_Button, Can_Default);
      Button_Callback.Connect
        (save_file_selection.Cancel_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_cancel_Pressed'Access));

   end Initialize;

end Save_File_Selection_Pkg;
