------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Gtk;                       use Gtk;
with Mmi_Tools;                 use Mmi_Tools;
with Translate;                 use Translate;
with Framework_Config;          use Framework_Config;
with Editor_Config;             use Editor_Config;
with Unbounded_Strings;         use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;
with Ada.Strings.Unbounded;     use Ada.Strings.Unbounded;
with Strings;                   use Strings;
with Ada.Numerics.Float_Random; use Ada.Numerics.Float_Random;
with Ada.Numerics.Aux;          use Ada.Numerics.Aux;

package body Scheduling_Options_Pkg.Callbacks is

   procedure Page_Switch
     (Notebook : access Gtk_Notebook_Record'Class;
      Page     : in Gtk.Gtk_Notebook_Page;
      Page_Num : in Gtk_Notebook)
   is
      pragma Warnings (Off, Page_Num);
      Old_Page : constant Gtk.Gtk_Notebook_Page := Get_Cur_Page (Notebook);
      Pixmap   : Gtk_Pixmap;
   begin
      Pixmap := Gtk_Pixmap (Get_Child (Gtk_Box (Get_Tab_Label (Page)), 0));
      Gtk.Pixmap.Set
        (Pixmap,
         Scheduling_Options.Book_Opened,
         Scheduling_Options.Book_Opened_Mask);
      Pixmap :=
        Gtk_Pixmap (Get_Child (Gtk_Box (Get_Menu_Label (Page)), 0));
      Gtk.Pixmap.Set
        (Pixmap,
         Scheduling_Options.Book_Opened,
         Scheduling_Options.Book_Opened_Mask);

      if Old_Page /= null then
         Pixmap :=
           Gtk_Pixmap (Get_Child (Gtk_Box (Get_Tab_Label (Old_Page)), 0));
         Gtk.Pixmap.Set
           (Pixmap,
            Scheduling_Options.Book_Closed,
            Scheduling_Options.Book_Closed_Mask);
         Pixmap :=
           Gtk_Pixmap (Get_Child (Gtk_Box (Get_Menu_Label (Old_Page)), 0));
         Gtk.Pixmap.Set
           (Pixmap,
            Scheduling_Options.Book_Closed,
            Scheduling_Options.Book_Closed_Mask);
      end if;
   end Page_Switch;

   procedure On_Ok_Pressed (Object : access Gtk_Button_Record'Class) is

      Ok : Boolean;

   begin

      if Get_Active (Scheduling_Options.With_Offsets) then
         Schedule_With_Offsets := True;
      else
         Schedule_With_Offsets := False;
      end if;

      if Get_Active (Scheduling_Options.With_Precedencies) then
         Schedule_With_Precedencies := True;
      else
         Schedule_With_Precedencies := False;
      end if;

      if Get_Active (Scheduling_Options.With_Resources) then
         Schedule_With_Resources := True;
      else
         Schedule_With_Resources := False;
      end if;

      if Get_Active (Scheduling_Options.Seed_Task) then
         Has_Global_Seed := False;
      else
         Has_Global_Seed := True;
      end if;

      if Get_Active (Scheduling_Options.Seed_Predictable) then
         Is_Global_Predictable_Seed := True;
      else
         Is_Global_Predictable_Seed := False;
      end if;

      if Get_Active (Scheduling_Options.Seed_Unpredictable) then
         Is_Global_Predictable_Seed := False;
      else
         Is_Global_Predictable_Seed := True;
      end if;

      To_Integer
        (Get_Text (Scheduling_Options.Seed_Entry),
         Global_Seed_Value,
         Ok);
      if not Ok then
         if Get_Text (Scheduling_Options.Seed_Entry) /= "" then
            Show_Message_Box
              (Lb_Seed (Current_Language) &
               Lb_Must_Be_Numeric (Current_Language));
            return;
         end if;
      end if;

      for I in  Time_Unit_Event_Type'Range loop
         if (I /= Task_Activation) and (I /= Running_Task) then
            if Get_Active (Scheduling_Options.Event_Check_Button (I)) then
               Generate_Events (I) := True;
            else
               Generate_Events (I) := False;
            end if;
         end if;
      end loop;

      Destroy (Scheduling_Options);

   end On_Ok_Pressed;

   procedure On_Cancel_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      Destroy (Scheduling_Options);
   end On_Cancel_Pressed;

   procedure On_Seed_Pressed (Object : access Gtk_Button_Record'Class) is

      Seed : Generator;
      Nmb  : Integer;
   begin
      -- Give random value for the schedulers seed
      --
      Reset (Seed);
      -- range from integer'first .. integer'last
      Nmb :=
        Integer (Double (Random (Seed)) *
                 Double (Integer'Last - Integer'First) +
                 Double (Integer'First));
      Gtk.GEntry.Set_Text (Scheduling_Options.Seed_Entry, Nmb'Img);

   end On_Seed_Pressed;

end Scheduling_Options_Pkg.Callbacks;
