------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Glib;                             use Glib;
with Gtk;                              use Gtk;
with Gdk.Types;                        use Gdk.Types;
with Gtk.Enums;                        use Gtk.Enums;
with Gdk.Color;                        use Gdk.Color;
with Xpm;                              use Xpm;
with Callbacks_Cheddar;                use Callbacks_Cheddar;
with Scheduling_Options_Pkg.Callbacks; use Scheduling_Options_Pkg.Callbacks;
with Translate;                        use Translate;
with Ada.Strings.Unbounded;            use Ada.Strings.Unbounded;
with Strings;                          use Strings;
with Framework_Config;                 use Framework_Config;
with Editor_Config;                    use Editor_Config;

package body Scheduling_Options_Pkg is

   procedure Gtk_New (Scheduling_Options : out Scheduling_Options_Access) is

   begin
      Scheduling_Options := new Scheduling_Options_Record;
      Scheduling_Options_Pkg.Initialize (Scheduling_Options);
   end Gtk_New;

   procedure Initialize
     (Scheduling_Options : access Scheduling_Options_Record'Class)
   is

      Index : Guint;

   begin

      Gtk.Window.Initialize (Scheduling_Options, Window_Toplevel);
      Set_Title
        (Scheduling_Options,
         To_String (Lb_Scheduling_Options (Current_Language)));
      Set_Policy (Scheduling_Options, True, True, True);
      Set_Position (Scheduling_Options, Win_Pos_Center_Always);
      Set_Modal (Scheduling_Options, True);
      Set_Extension_Events (Scheduling_Options, Extension_Events_All);
      Set_Flags (Scheduling_Options, Can_Default);
      Grab_Focus (Scheduling_Options);

      Gtk.Frame.Gtk_New (Scheduling_Options.Main_Frame, "");
      Add (Scheduling_Options, Scheduling_Options.Main_Frame);

      -- Create Notebook
      --
      Gtk_New (Scheduling_Options.Notebook);
      Set_Tab_Pos (Scheduling_Options.Notebook, Pos_Top);
      Gtk_New_Vbox (Scheduling_Options.Options_Vbox);
      Add (Scheduling_Options.Main_Frame, Scheduling_Options.Options_Vbox);
      Add (Scheduling_Options.Options_Vbox, Scheduling_Options.Notebook);
      Set_Border_Width (Scheduling_Options.Notebook, 10);
      Realize (Scheduling_Options.Notebook);

      Create_From_Xpm_D
        (Scheduling_Options.Book_Opened,
         Get_Window (Scheduling_Options.Notebook),
         Scheduling_Options.Book_Opened_Mask,
         Null_Color,
         Book_Opened_Xpm);
      Create_From_Xpm_D
        (Scheduling_Options.Book_Closed,
         Get_Window (Scheduling_Options.Notebook),
         Scheduling_Options.Book_Closed_Mask,
         Null_Color,
         Book_Closed_Xpm);

      -- Main page
      --
      Gtk_New
        (Scheduling_Options.Child_Page,
         To_String (Lb_Compute_Scheduling (Current_Language)));

      Gtk_New (Scheduling_Options.Value_Table, 8, 2, True);
      Add (Scheduling_Options.Child_Page, Scheduling_Options.Value_Table);
      Set_Border_Width (Scheduling_Options.Value_Table, 14);
      Set_Row_Spacings (Scheduling_Options.Value_Table, 8);
      Set_Col_Spacings (Scheduling_Options.Value_Table, 0);

      Gtk_New_Hbox (Scheduling_Options.Label_Box, False, 0);
      Gtk_New
        (Scheduling_Options.Pixmap,
         Scheduling_Options.Book_Opened,
         Scheduling_Options.Book_Opened_Mask);
      Pack_Start
        (Scheduling_Options.Label_Box,
         Scheduling_Options.Pixmap,
         False,
         True,
         0);
      Set_Padding (Scheduling_Options.Pixmap, 3, 1);

      Gtk_New
        (Scheduling_Options.Lab,
         To_String (Lb_Compute_Scheduling (Current_Language)));
      Pack_Start
        (Scheduling_Options.Label_Box,
         Scheduling_Options.Lab,
         False,
         True,
         0);
      Show_All (Scheduling_Options.Label_Box);

      Gtk_New_Vbox (Scheduling_Options.Menu_Box, False, 0);
      Gtk_New
        (Scheduling_Options.Pixmap,
         Scheduling_Options.Book_Opened,
         Scheduling_Options.Book_Opened_Mask);
      Pack_Start
        (Scheduling_Options.Menu_Box,
         Scheduling_Options.Pixmap,
         False,
         True,
         0);
      Set_Padding (Scheduling_Options.Pixmap, 3, 1);

      Gtk_New
        (Scheduling_Options.Lab,
         To_String (Lb_Compute_Scheduling (Current_Language)));
      Pack_Start
        (Scheduling_Options.Menu_Box,
         Scheduling_Options.Lab,
         False,
         True,
         0);
      Show_All (Scheduling_Options.Menu_Box);

      Append_Page_Menu
        (Scheduling_Options.Notebook,
         Scheduling_Options.Child_Page,
         Scheduling_Options.Label_Box,
         Scheduling_Options.Menu_Box);

      Gtk.Box.Gtk_New_Hbox (Scheduling_Options.Hbox_Seed, False, 10);
      Attach_Defaults
        (Scheduling_Options.Value_Table,
         Scheduling_Options.Hbox_Seed,
         1,
         2,
         8,
         9);

      Gtk_New
        (Scheduling_Options.Label,
         To_String (Lb_Schedule_According_To (Current_Language)));
      Set_Alignment (Scheduling_Options.Label, 0.5, 0.5);
      Set_Padding (Scheduling_Options.Label, 0, 0);
      Set_Justify (Scheduling_Options.Label, Justify_Left);
      Set_Line_Wrap (Scheduling_Options.Label, False);
      Attach
        (Scheduling_Options.Value_Table,
         Scheduling_Options.Label,
         0,
         1,
         0,
         1);

      Gtk_New
        (Scheduling_Options.With_Offsets,
         To_String (Lb_Offsets (Current_Language)));
      if Schedule_With_Offsets then
         Set_Active (Scheduling_Options.With_Offsets, True);
      else
         Set_Active (Scheduling_Options.With_Offsets, False);
      end if;
      Attach
        (Scheduling_Options.Value_Table,
         Scheduling_Options.With_Offsets,
         1,
         2,
         1,
         2);

      Gtk_New
        (Scheduling_Options.With_Precedencies,
         To_String (Lb_Precedencies (Current_Language)));
      if Schedule_With_Precedencies then
         Set_Active (Scheduling_Options.With_Precedencies, True);
      else
         Set_Active (Scheduling_Options.With_Precedencies, False);
      end if;
      Attach
        (Scheduling_Options.Value_Table,
         Scheduling_Options.With_Precedencies,
         1,
         2,
         2,
         3);

      Gtk_New
        (Scheduling_Options.With_Resources,
         To_String (Lb_Resources (Current_Language)));
      if Schedule_With_Resources then
         Set_Active (Scheduling_Options.With_Resources, True);
      else
         Set_Active (Scheduling_Options.With_Resources, False);
      end if;
      Attach
        (Scheduling_Options.Value_Table,
         Scheduling_Options.With_Resources,
         1,
         2,
         3,
         4);

      Gtk_New
        (Scheduling_Options.Label_Seed,
         To_String (Lb_Seed_Options (Current_Language)));
      Set_Alignment (Scheduling_Options.Label_Seed, 0.5, 0.5);
      Set_Padding (Scheduling_Options.Label_Seed, 0, 0);
      Set_Justify (Scheduling_Options.Label_Seed, Justify_Left);
      Set_Line_Wrap (Scheduling_Options.Label_Seed, False);
      Attach
        (Scheduling_Options.Value_Table,
         Scheduling_Options.Label_Seed,
         0,
         1,
         4,
         5);

      Gtk_New
        (Scheduling_Options.Seed_Task,
         Scheduling_Options.Seed_Group,
         To_String (Lb_Task_Specific (Current_Language)));
      Scheduling_Options.Seed_Group := Group (Scheduling_Options.Seed_Task);

      Attach
        (Scheduling_Options.Value_Table,
         Scheduling_Options.Seed_Task,
         1,
         2,
         5,
         6);

      Gtk_New
        (Scheduling_Options.Seed_Predictable,
         Scheduling_Options.Seed_Group,
         To_String (Lb_All_Predictable (Current_Language)));
      Scheduling_Options.Seed_Group :=
         Group (Scheduling_Options.Seed_Predictable);
      Attach
        (Scheduling_Options.Value_Table,
         Scheduling_Options.Seed_Predictable,
         1,
         2,
         7,
         8);

      Gtk_New
        (Scheduling_Options.Seed_Unpredictable,
         Scheduling_Options.Seed_Group,
         To_String (Lb_All_Unpredictable (Current_Language)));
      Scheduling_Options.Seed_Group :=
         Group (Scheduling_Options.Seed_Unpredictable);

      Attach
        (Scheduling_Options.Value_Table,
         Scheduling_Options.Seed_Unpredictable,
         1,
         2,
         6,
         7);

      Set_Active (Scheduling_Options.Seed_Predictable, True);
      if not Has_Global_Seed then
         Set_Active (Scheduling_Options.Seed_Task, True);
      end if;
      if not Is_Global_Predictable_Seed then
         Set_Active (Scheduling_Options.Seed_Unpredictable, True);
      end if;

      Gtk.GEntry.Gtk_New (Scheduling_Options.Seed_Entry, 50);
      Add (Scheduling_Options.Hbox_Seed, Scheduling_Options.Seed_Entry);
      Set_Text (Scheduling_Options.Seed_Entry, Global_Seed_Value'Img);

      Gtk.Button.Gtk_New
        (Scheduling_Options.Seed_Button,
         To_String (Lb_Seed (Current_Language)));
      Add (Scheduling_Options.Hbox_Seed, Scheduling_Options.Seed_Button);

      Gtk_New_Hseparator (Scheduling_Options.Hseparator);
      Pack_Start
        (Scheduling_Options.Options_Vbox,
         Scheduling_Options.Hseparator);

      Gtk_New (Scheduling_Options.Button_Hbox);
      Set_Layout (Scheduling_Options.Button_Hbox, Buttonbox_Spread);
      Set_Spacing (Scheduling_Options.Button_Hbox, 10);
      Set_Child_Size (Scheduling_Options.Button_Hbox, 20, 20);
      Set_Child_Ipadding (Scheduling_Options.Button_Hbox, 7, 0);
      Pack_Start
        (Scheduling_Options.Options_Vbox,
         Scheduling_Options.Button_Hbox,
         False,
         True,
         5);

      Gtk_New (Scheduling_Options.Ok_Button, "Ok");
      Add (Scheduling_Options.Button_Hbox, Scheduling_Options.Ok_Button);
      Button_Callback.Connect
        (Scheduling_Options.Ok_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'Access));

      Gtk_New
        (Scheduling_Options.Cancel_Button,
         To_String (Lb_Cancel (Current_Language)));
      Add (Scheduling_Options.Button_Hbox, Scheduling_Options.Cancel_Button);
      Button_Callback.Connect
        (Scheduling_Options.Cancel_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'Access));
      Button_Callback.Connect
        (Scheduling_Options.Seed_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Seed_Pressed'Access));

      -- Events page
      --
      Gtk_New
        (Scheduling_Options.Child_Page,
         To_String (Lb_Generate_Events (Current_Language)));

      Gtk_New (Scheduling_Options.Value_Table, 8, 2, True);
      Add (Scheduling_Options.Child_Page, Scheduling_Options.Value_Table);
      Set_Border_Width (Scheduling_Options.Value_Table, 14);
      Set_Row_Spacings (Scheduling_Options.Value_Table, 8);
      Set_Col_Spacings (Scheduling_Options.Value_Table, 0);

      Gtk_New_Hbox (Scheduling_Options.Label_Box, False, 0);
      Gtk_New
        (Scheduling_Options.Pixmap,
         Scheduling_Options.Book_Closed,
         Scheduling_Options.Book_Closed_Mask);
      Pack_Start
        (Scheduling_Options.Label_Box,
         Scheduling_Options.Pixmap,
         False,
         True,
         0);
      Set_Padding (Scheduling_Options.Pixmap, 3, 1);

      Gtk_New
        (Scheduling_Options.Lab,
         To_String (Lb_Generate_Events (Current_Language)));
      Pack_Start
        (Scheduling_Options.Label_Box,
         Scheduling_Options.Lab,
         False,
         True,
         0);
      Show_All (Scheduling_Options.Label_Box);

      Gtk_New_Vbox (Scheduling_Options.Menu_Box, False, 0);
      Gtk_New
        (Scheduling_Options.Pixmap,
         Scheduling_Options.Book_Closed,
         Scheduling_Options.Book_Closed_Mask);
      Pack_Start
        (Scheduling_Options.Menu_Box,
         Scheduling_Options.Pixmap,
         False,
         True,
         0);
      Set_Padding (Scheduling_Options.Pixmap, 3, 1);

      Gtk_New
        (Scheduling_Options.Lab,
         To_String (Lb_Generate_Events (Current_Language)));
      Pack_Start
        (Scheduling_Options.Menu_Box,
         Scheduling_Options.Lab,
         False,
         True,
         0);
      Show_All (Scheduling_Options.Menu_Box);

      Append_Page_Menu
        (Scheduling_Options.Notebook,
         Scheduling_Options.Child_Page,
         Scheduling_Options.Label_Box,
         Scheduling_Options.Menu_Box);

      Index := 0;
      for I in  Time_Unit_Event_Type'Range loop

         if (I /= Task_Activation) and (I /= Running_Task) then
            Gtk_New
              (Scheduling_Options.Event_Check_Button (I),
               To_Lower (I'Img));
            if Generate_Events (I) then
               Set_Active (Scheduling_Options.Event_Check_Button (I), True);
            else
               Set_Active (Scheduling_Options.Event_Check_Button (I), False);
            end if;
            if Index mod 2 = 1 then
               Attach
                 (Scheduling_Options.Value_Table,
                  Scheduling_Options.Event_Check_Button (I),
                  0,
                  1,
                  Index,
                  Index + 1);
               Attach
                 (Scheduling_Options.Value_Table,
                  Scheduling_Options.Event_Check_Button (
Time_Unit_Event_Type'Pred (I)),
                  1,
                  2,
                  Index,
                  Index + 1);
            end if;
            Index := Index + 1;
         end if;
      end loop;

      -- Notebook page switching activation
      --
      Two_Notebook_Callback.Connect
        (Scheduling_Options.Notebook,
         "switch_page",
         Two_Notebook_Callback.To_Marshaller (Page_Switch'Access),
         Scheduling_Options.Notebook);

   end Initialize;

end Scheduling_Options_Pkg;
