------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Gdk.Bitmap;       use Gdk.Bitmap;
with Gdk.Pixmap;       use Gdk.Pixmap;
with Gtk.Table;        use Gtk.Table;
with Gtk.Label;        use Gtk.Label;
with Gtk.Separator;    use Gtk.Separator;
with Gtk.Button;       use Gtk.Button;
with Gtk.Check_Button; use Gtk.Check_Button;
with Gtk.Radio_Button; use Gtk.Radio_Button;
with Gtk.Hbutton_Box;  use Gtk.Hbutton_Box;
with Gtk.Window;       use Gtk.Window;
with Gtk.Widget;       use Gtk.Widget;
with Gtk.Box;          use Gtk.Box;
with Gtk.GEntry;       use Gtk.GEntry;
with Gtk.Frame;        use Gtk.Frame;
with Gtk.Notebook;     use Gtk.Notebook;
with Gtk.Pixmap;       use Gtk.Pixmap;
with Events;           use Events;

package Scheduling_Options_Pkg is

   type Scheduling_Options_Record is new Gtk_Window_Record with private;
   type Scheduling_Options_Access is access all Scheduling_Options_Record'
     Class;

   procedure Gtk_New (Scheduling_Options : out Scheduling_Options_Access);
   procedure Initialize
     (Scheduling_Options : access Scheduling_Options_Record'Class);

   Scheduling_Options : Scheduling_Options_Access;

   type Event_Check_Button_Table is
     array (Time_Unit_Event_Type'Range) of Gtk_Check_Button;

private

   type Scheduling_Options_Record is new Gtk_Window_Record with record
      Options_Vbox : Gtk_Vbox;

      Value_Table       : Gtk_Table;
      Label             : Gtk_Label;
      Label_Seed        : Gtk_Label;
      With_Resources    : Gtk_Check_Button;
      With_Precedencies : Gtk_Check_Button;
      With_Offsets      : Gtk_Check_Button;

      Event_Check_Button : Event_Check_Button_Table;

      Seed_Group         : Widget_SList.GSlist;
      Seed_Task          : Gtk_Radio_Button;
      Seed_Predictable   : Gtk_Radio_Button;
      Seed_Entry         : Gtk_GEntry;
      Seed_Button        : Gtk_Button;
      Seed_Unpredictable : Gtk_Radio_Button;
      Hbox_Seed          : Gtk_Hbox;

      Button_Hbox   : Gtk_Hbutton_Box;
      Ok_Button     : Gtk_Button;
      Cancel_Button : Gtk_Button;
      Hseparator    : Gtk_Hseparator;

      Main_Frame : Gtk_Frame;
      Child_Page : Gtk_Frame;
      Label_Box  : Gtk_Box;
      Menu_Box   : Gtk_Box;
      Pixmap     : Gtk_Pixmap;
      Lab        : Gtk_Label;

      Book_Opened      : Gdk_Pixmap;
      Book_Opened_Mask : Gdk_Bitmap;
      Book_Closed      : Gdk_Pixmap;
      Book_Closed_Mask : Gdk_Bitmap;
      Notebook         : Gtk_Notebook;
   end record;

end Scheduling_Options_Pkg;
