------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Gtk.Enums;                           use Gtk.Enums;
with Callbacks_Cheddar;                   use Callbacks_Cheddar;
with Scrolling_Draw_Buffer_Pkg.Callbacks;
use Scrolling_Draw_Buffer_Pkg.Callbacks;
with Ada.Strings.Unbounded;               use Ada.Strings.Unbounded;
with Framework_Config;                    use Framework_Config;
with Translate;                           use Translate;

package body Scrolling_Draw_Buffer_Pkg is

   procedure Gtk_New
     (Scrolling_Draw_Buffer : out Scrolling_Draw_Buffer_Access;
      Name                  : String)
   is

   begin
      Scrolling_Draw_Buffer := new Scrolling_Draw_Buffer_Record;
      Scrolling_Draw_Buffer_Pkg.Initialize (Scrolling_Draw_Buffer, Name);
   end Gtk_New;

   procedure Initialize
     (Scrolling_Draw_Buffer : access Scrolling_Draw_Buffer_Record'Class;
      Name                  : String)
   is

   begin

      Gtk.Window.Initialize (Scrolling_Draw_Buffer, Window_Toplevel);
      Set_Title
        (Scrolling_Draw_Buffer,
         To_String (Lb_Buffer_Utilization (Current_Language)) & Name);
      Set_Policy
        (Scrolling_Draw_Buffer,
         Allow_Shrink => False,
         Allow_Grow   => True,
         Auto_Shrink  => True);
      Set_Position (Scrolling_Draw_Buffer, Win_Pos_Center_Always);
      Set_Default_Size (Scrolling_Draw_Buffer, 600, 430);

      Gtk_New (Scrolling_Draw_Buffer.Alignment, 0.5, 0.5, 1.0, 1.0);
      Set_Sensitive (Scrolling_Draw_Buffer.Alignment, True);
      Add (Scrolling_Draw_Buffer, Scrolling_Draw_Buffer.Alignment);

      Gtk_New_Vbox (Scrolling_Draw_Buffer.Vbox, False, 0);
      Add (Scrolling_Draw_Buffer.Alignment, Scrolling_Draw_Buffer.Vbox);

      Gtk_New (Scrolling_Draw_Buffer.Scrolledwindow);
      Set_Usize (Scrolling_Draw_Buffer.Scrolledwindow, 300, 370);
      Set_Policy
        (Scrolling_Draw_Buffer.Scrolledwindow,
         Policy_Automatic,
         Policy_Automatic);
      Pack_Start
        (Scrolling_Draw_Buffer.Vbox,
         Scrolling_Draw_Buffer.Scrolledwindow,
         True,
         True,
         0);

      Gtk_New_Hseparator (Scrolling_Draw_Buffer.Separator);
      Pack_Start
        (Scrolling_Draw_Buffer.Vbox,
         Scrolling_Draw_Buffer.Separator,
         True,
         True,
         0);

      Gtk_New_Vbox (Scrolling_Draw_Buffer.Vbox2, False, 0);
      Pack_Start
        (Scrolling_Draw_Buffer.Vbox,
         Scrolling_Draw_Buffer.Vbox2,
         True,
         True,
         0);
      Gtk_New_Hbox (Scrolling_Draw_Buffer.Hbox, True, 6);
      Set_Border_Width (Scrolling_Draw_Buffer.Hbox, 15);
      Pack_Start
        (Scrolling_Draw_Buffer.Vbox2,
         Scrolling_Draw_Buffer.Hbox,
         True,
         True,
         0);

      Gtk_New (Scrolling_Draw_Buffer.Button, "Ok");
      Set_Usize (Scrolling_Draw_Buffer.Button, 70, 25);
      Pack_Start
        (Scrolling_Draw_Buffer.Hbox,
         Scrolling_Draw_Buffer.Button,
         False,
         False,
         0);

      Button_Callback.Connect
        (Scrolling_Draw_Buffer.Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'Access));

      Gtk_New (Scrolling_Draw_Buffer.Dbuffer);

      Add_With_Viewport
        (Scrolling_Draw_Buffer.Scrolledwindow,
         Scrolling_Draw_Buffer.Dbuffer);

   end Initialize;

   procedure Clear
     (Scrolling_Draw_Buffer : access Scrolling_Draw_Buffer_Record'Class)
   is
   begin
      Clear (Scrolling_Draw_Buffer.Dbuffer);
   end Clear;

   procedure Draw
     (Scrolling_Draw_Buffer : access Scrolling_Draw_Buffer_Record'Class;
      Buff                  : Buffer_Ptr;
      Buff1                 : Buffer_Size_Table;
      Last_Time_Unit1       : Natural;
      Time_Unit_Number1     : Time_Unit_Range)
   is

   begin
      Draw
        (Scrolling_Draw_Buffer.Dbuffer,
         Buff,
         Buff1,
         Last_Time_Unit1,
         Time_Unit_Number1);
   end Draw;

end Scrolling_Draw_Buffer_Pkg;
