------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Unbounded_Strings;     use Unbounded_Strings;
with Gtk.Window;            use Gtk.Window;
with Gtk.Alignment;         use Gtk.Alignment;
with Gtk.Box;               use Gtk.Box;
with Gtk.Separator;         use Gtk.Separator;
with Gtk.Button;            use Gtk.Button;
with Gtk.Scrolled_Window;   use Gtk.Scrolled_Window;
with Gtk.Text;              use Gtk.Text;
with Gdk.Color;             use Gdk.Color;
with Gdk.Font;              use Gdk.Font;

package Scrolling_Editor_Pkg is

   -- Modify callback type
   --  The procedure is called each time the modify button is pressed
   --
   type Modify_Callback_Function is access procedure
  (Edited_Text : in Unbounded_String);

   type Scrolling_Editor_Record is new Gtk_Window_Record with record
      Is_Displayed   : Boolean := False;
      Alignment      : Gtk_Alignment;
      Vbox           : Gtk_Vbox;
      Separator      : Gtk_Hseparator;
      Hbox           : Gtk_Hbox;
      Scrolledwindow : Gtk_Scrolled_Window;
      Text           : Gtk_Text;
      Vseparator1    : Gtk_Vseparator;

      Ok_Button     : Gtk_Button;
      Cancel_Button : Gtk_Button;
      Close_Button  : Gtk_Button;
      Modify_Button : Gtk_Button;

      Font_R  : Gdk_Font;
      Font_B  : Gdk_Font;
      Font_Bs : Gdk_Font;
      Red     : Gdk_Color;
      Black   : Gdk_Color;

      Read_Only   : Boolean;
      Stay_Opened : Boolean;
      Edited_Text : Unbounded_String_Ptr;

      Modify_Callback : Modify_Callback_Function;
   end record;

   type Scrolling_Editor_Access is access all Scrolling_Editor_Record'Class;

   -- Read only text : the text can not be edited
   --
   procedure Gtk_New
     (Scrolling_Editor : out Scrolling_Editor_Access;
      Title            : in Unbounded_String);

   -- Write text : the text can be edited
   -- The result is stored in "edited_result"
   --
   procedure Gtk_New
     (Scrolling_Editor : out Scrolling_Editor_Access;
      Title            : in Unbounded_String;
      Edited_Text      : in Unbounded_String_Ptr);

   -- Write text : the text can be edited and the window
   -- stays opened.
   -- The result is stored in "edited_result"
   -- and the procedure "Modify_Callback" is called
   --
   procedure Gtk_New
     (Scrolling_Editor : out Scrolling_Editor_Access;
      Title            : in Unbounded_String;
      Edited_Text      : in Unbounded_String_Ptr;
      Modify_Callback  : in Modify_Callback_Function);

   procedure Initialize
     (Scrolling_Editor : access Scrolling_Editor_Record'Class;
      Title            : in Unbounded_String;
      Edited_Text      : in Unbounded_String_Ptr;
      Read_Only        : in Boolean;
      Stay_Opened      : in Boolean;
      Modify_Callback  : in Modify_Callback_Function := null);

   procedure Write_Text
     (Scrolling_Editor : in Scrolling_Editor_Access;
      Msg              : in Unbounded_String);
   procedure Write_Title
     (Scrolling_Editor : in Scrolling_Editor_Access;
      Msg              : in Unbounded_String);
   procedure Write_Title_Small
     (Scrolling_Editor : in Scrolling_Editor_Access;
      Msg              : in Unbounded_String);

   Scrolling_Editor : Scrolling_Editor_Access;

   pragma Inline (Write_Text);
   pragma Inline (Write_Title);
   pragma Inline (Write_Title_Small);

end Scrolling_Editor_Pkg;
