------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Gtk;                          use Gtk;
with Gtk.Widget;                   use Gtk.Widget;
with Gtk.Enums;                    use Gtk.Enums;
with Callbacks_Cheddar;            use Callbacks_Cheddar;
with Scrolling_Text_Pkg.Callbacks; use Scrolling_Text_Pkg.Callbacks;
with Mmi_Tools;                    use Mmi_Tools;

package body Scrolling_Text_Pkg is

   procedure Gtk_New
     (Scrolling_Text : out Scrolling_Text_Access;
      Title          : Unbounded_String)
   is

   begin
      Scrolling_Text := new Scrolling_Text_Record;
      Scrolling_Text_Pkg.Initialize (Scrolling_Text, Title);
   end Gtk_New;

   procedure Initialize
     (Scrolling_Text : access Scrolling_Text_Record'Class;
      Title          : in Unbounded_String)
   is

   begin

      Gtk.Window.Initialize (Scrolling_Text, Window_Toplevel);
      Set_Title (Scrolling_Text, To_String (Title));
      Set_Policy (Scrolling_Text, True, True, True);
      Set_Position (Scrolling_Text, Win_Pos_Center_Always);

      Set_Modal (Scrolling_Text, True);
      Set_Flags (Scrolling_Text, Can_Default);
      Grab_Focus (Scrolling_Text);

      Gtk_New (Scrolling_Text.Alignment, 0.5, 0.5, 1.0, 1.0);
      Set_Sensitive (Scrolling_Text.Alignment, True);
      Add (Scrolling_Text, Scrolling_Text.Alignment);

      Gtk_New_Vbox (Scrolling_Text.Vbox, False, 0);
      Add (Scrolling_Text.Alignment, Scrolling_Text.Vbox);

      Gtk_New (Scrolling_Text.Scrolledwindow);
      Set_Usize (Scrolling_Text.Scrolledwindow, 300, 370);
      Set_Policy
        (Scrolling_Text.Scrolledwindow,
         Policy_Automatic,
         Policy_Automatic);
      Pack_Start
        (Scrolling_Text.Vbox,
         Scrolling_Text.Scrolledwindow,
         True,
         True,
         0);

      Gtk_New (Scrolling_Text.Text);
      Set_Editable (Scrolling_Text.Text, False);
      Add (Scrolling_Text.Scrolledwindow, Scrolling_Text.Text);

      Gtk_New_Hseparator (Scrolling_Text.Separator);
      Pack_Start
        (Scrolling_Text.Vbox,
         Scrolling_Text.Separator,
         True,
         True,
         0);

      Gtk_New_Hbox (Scrolling_Text.Hbox, True, 6);
      Set_Border_Width (Scrolling_Text.Hbox, 15);
      Pack_Start (Scrolling_Text.Vbox, Scrolling_Text.Hbox, True, True, 0);

      Gtk_New (Scrolling_Text.Ok_Button, "Ok");
      Set_Usize (Scrolling_Text.Ok_Button, 70, 25);
      Pack_Start
        (Scrolling_Text.Hbox,
         Scrolling_Text.Ok_Button,
         False,
         False,
         0);

      Button_Callback.Connect
        (Scrolling_Text.Ok_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'Access));

      Load
        (Scrolling_Text.Font_R,
         "-adobe-courier-medium-r-normal--*-120-*-*-*-*-*-*");
      Load
        (Scrolling_Text.Font_Bs,
         "-adobe-courier-bold-r-*-*-*-160-*-*-*-*-*-*");
      Load
        (Scrolling_Text.Font_B,
         "-adobe-courier-bold-r-*-*-*-200-*-*-*-*-*-*");

      Set_Rgb
        (Scrolling_Text.Red,
         Text_Colors (2).Red,
         Text_Colors (2).Green,
         Text_Colors (2).Blue);
      Set_Rgb
        (Scrolling_Text.Black,
         Text_Colors (0).Red,
         Text_Colors (0).Green,
         Text_Colors (0).Blue);

   end Initialize;

   procedure Write_Text
     (Scrolling_Text : Scrolling_Text_Access;
      Msg            : Unbounded_String)
   is
   begin
      Insert
        (Scrolling_Text.Text,
         Scrolling_Text.Font_R,
         Scrolling_Text.Black,
         Null_Color,
         To_String (Msg));
   end Write_Text;

   procedure Write_Title
     (Scrolling_Text : Scrolling_Text_Access;
      Msg            : Unbounded_String)
   is
   begin
      Insert
        (Scrolling_Text.Text,
         Scrolling_Text.Font_B,
         Scrolling_Text.Red,
         Null_Color,
         To_String (Msg));
   end Write_Title;

   procedure Write_Title_Small
     (Scrolling_Text : Scrolling_Text_Access;
      Msg            : Unbounded_String)
   is
   begin
      Insert
        (Scrolling_Text.Text,
         Scrolling_Text.Font_Bs,
         Scrolling_Text.Red,
         Null_Color,
         To_String (Msg));
   end Write_Title_Small;

end Scrolling_Text_Pkg;
