------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Gtk.Window;            use Gtk.Window;
with Gtk.Alignment;         use Gtk.Alignment;
with Gtk.Box;               use Gtk.Box;
with Gtk.Separator;         use Gtk.Separator;
with Gtk.Button;            use Gtk.Button;
with Gtk.Scrolled_Window;   use Gtk.Scrolled_Window;
with Gtk.Text;              use Gtk.Text;
with Gdk.Color;             use Gdk.Color;
with Gdk.Font;              use Gdk.Font;

package Scrolling_Text_Pkg is

   type Scrolling_Text_Record is new Gtk_Window_Record with private;
   type Scrolling_Text_Access is access all Scrolling_Text_Record'Class;

   procedure Gtk_New
     (Scrolling_Text : out Scrolling_Text_Access;
      Title          : in Unbounded_String);

   procedure Initialize
     (Scrolling_Text : access Scrolling_Text_Record'Class;
      Title          : in Unbounded_String);

   procedure Write_Text
     (Scrolling_Text : in Scrolling_Text_Access;
      Msg            : in Unbounded_String);
   procedure Write_Title
     (Scrolling_Text : in Scrolling_Text_Access;
      Msg            : in Unbounded_String);
   procedure Write_Title_Small
     (Scrolling_Text : in Scrolling_Text_Access;
      Msg            : in Unbounded_String);

   Scrolling_Text : Scrolling_Text_Access;

private

   type Scrolling_Text_Record is new Gtk_Window_Record with record
      Alignment      : Gtk_Alignment;
      Vbox           : Gtk_Vbox;
      Separator      : Gtk_Hseparator;
      Hbox           : Gtk_Hbox;
      Scrolledwindow : Gtk_Scrolled_Window;
      Text           : Gtk_Text;
      Vseparator1    : Gtk_Vseparator;

      Ok_Button : Gtk_Button;

      Font_R  : Gdk_Font;
      Font_B  : Gdk_Font;
      Font_Bs : Gdk_Font;
      Red     : Gdk_Color;
      Black   : Gdk_Color;
   end record;

end Scrolling_Text_Pkg;
