------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Processor_Set;    use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Project;          use Project;
with Task_Set;         use Task_Set;
use Task_Set.Generic_Task_Set;
with Resource_Set;     use Resource_Set;
use Resource_Set.Generic_Resource_Set;
with Message_Set;      use Message_Set;
use Message_Set.Generic_Message_Set;
with Buffer_Set;       use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Mmi_Tools;        use Mmi_Tools;
with Translate;        use Translate;
with Framework_Config; use Framework_Config;

package body Select_Time_Line_Pkg.Callbacks is

   procedure On_Ok_Pressed (Object : access Gtk_Button_Record'Class) is

      An_Object                   : Generic_Object_Ptr;
      A_Name                      : Unbounded_String;
      A_Type                      : Unbounded_String;
      Number_Of_Object_To_Display : Natural := 0;

   begin

      Reset (Time_Line_To_Display);

      -- First, check we require not too much object to be displayed
      --
      for I in  0 .. Select_Time_Line.Nb_Entries - 1 loop
         if Get_Active (Select_Time_Line.Entries (I).Display_Time_Line) then
            Number_Of_Object_To_Display := Number_Of_Object_To_Display + 1;
         end if;
      end loop;

      if Number_Of_Object_To_Display >
         Editor_Config.Max_Time_Line_To_Display
      then
         Show_Message_Box
           (Lb_Can_Not_Display_So_Much_Time_Line (Current_Language));
         return;
      end if;

      if Number_Of_Object_To_Display = 0 then
         Show_Message_Box
           (Lb_Select_At_Least_One_Time_Line (Current_Language));
         return;
      end if;

      -- Scan the widget to store each task/message/buffer/resource to display
      --
      for I in  0 .. Select_Time_Line.Nb_Entries - 1 loop
         if Get_Active (Select_Time_Line.Entries (I).Display_Time_Line) then
            A_Name :=
               To_Unbounded_String
                 (Get_Text (Select_Time_Line.Entries (I).Object_Name));
            A_Type :=
               To_Unbounded_String
                 (Get_Text (Select_Time_Line.Entries (I).Object_Type));

            if (A_Type = Lb_Task (Current_Language)) then
               An_Object :=
                 Generic_Object_Ptr (Search_Task (Sys.Tasks, A_Name));
            else
               if (A_Type = Lb_Buffer (Current_Language)) then
                  An_Object :=
                    Generic_Object_Ptr (Search_Buffer (Sys.Buffers, A_Name));
               else
                  if (A_Type = Lb_Message (Current_Language)) then
                     An_Object :=
                       Generic_Object_Ptr (Search_Message
                                              (Sys.Messages,
                                               A_Name));
                  else
                     if (A_Type = Lb_Resource (Current_Language)) then
                        An_Object :=
                          Generic_Object_Ptr (Search_Resource
                                                 (Sys.Resources,
                                                  A_Name));
                     end if;
                  end if;
               end if;
            end if;
            Add (Time_Line_To_Display, An_Object);
         end if;
      end loop;

      -- Display Time line
      --
      Select_Time_Line.To_Be_Called.all;

      Destroy (Select_Time_Line);

   end On_Ok_Pressed;

end Select_Time_Line_Pkg.Callbacks;
