------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Glib;                           use Glib;
with Gdk.Types;                      use Gdk.Types;
with Gtk.Enums;                      use Gtk.Enums;
with Callbacks_Cheddar;              use Callbacks_Cheddar;
with Gtk.Widget;                     use Gtk.Widget;
with Project;                        use Project;
with Select_Time_Line_Pkg.Callbacks; use Select_Time_Line_Pkg.Callbacks;
with Framework_Config;               use Framework_Config;
with Translate;                      use Translate;
with Unbounded_Strings;              use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;
with Task_Set;                       use Task_Set;
use Task_Set.Generic_Task_Set;
with Tasks;                          use Tasks;
with Resource_Set;                   use Resource_Set;
use Resource_Set.Generic_Resource_Set;
with Resources;                      use Resources;
with Message_Set;                    use Message_Set;
use Message_Set.Generic_Message_Set;
with Messages;                       use Messages;
with Buffer_Set;                     use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Buffers;                        use Buffers;

package body Select_Time_Line_Pkg is

   procedure Gtk_New
     (Select_Time_Line : out Select_Time_Line_Access;
      A_System         : in System;
      Call             : in Select_Time_Line_Callback_Type)
   is
   begin
      Select_Time_Line := new Select_Time_Line_Record;
      Select_Time_Line_Pkg.Initialize (Select_Time_Line, Sys, Call);
   end Gtk_New;

   procedure Initialize
     (Select_Time_Line : access Select_Time_Line_Record'Class;
      A_System         : in System;
      Call             : in Select_Time_Line_Callback_Type)
   is

      My_Task_Iterator     : Tasks_Iterator;
      My_Resource_Iterator : Resources_Iterator;
      My_Buffer_Iterator   : Buffers_Iterator;
      My_Message_Iterator  : Messages_Iterator;
      A_Task               : Generic_Task_Ptr;
      A_Message            : Message_Ptr;
      A_Resource           : Generic_Resource_Ptr;
      A_Buffer             : Buffer_Ptr;
      Line                 : Guint := 1;

      procedure New_Object (An_Object : Generic_Object_Ptr) is
      begin

         Gtk_New
           (Select_Time_Line.Entries (Select_Time_Line.Nb_Entries).Object_Name)
;
         Set_Editable
           (Select_Time_Line.Entries (Select_Time_Line.Nb_Entries).Object_Name,
            False);
         Set_Max_Length
           (Select_Time_Line.Entries (Select_Time_Line.Nb_Entries).Object_Name,
            0);
         Set_Text
           (Select_Time_Line.Entries (Select_Time_Line.Nb_Entries).Object_Name,
            To_String ((An_Object.Name)));
         Set_Visibility
           (Select_Time_Line.Entries (Select_Time_Line.Nb_Entries).Object_Name,
            True);
         Attach
           (Select_Time_Line.Table,
            Select_Time_Line.Entries (Select_Time_Line.Nb_Entries).Object_Name,
            0,
            1,
            Line,
            Line + 1,
            Expand or Fill,
            0,
            0,
            0);

         Gtk_New
           (Select_Time_Line.Entries (Select_Time_Line.Nb_Entries).Object_Type)
;
         Set_Editable
           (Select_Time_Line.Entries (Select_Time_Line.Nb_Entries).Object_Type,
            False);
         Set_Max_Length
           (Select_Time_Line.Entries (Select_Time_Line.Nb_Entries).Object_Type,
            0);

         case An_Object.Object_Type is
            when Address_Space_Object_Type =>
               null;
            when Processor_Object_Type =>
               null;
            when Network_Object_Type =>
               null;
            when Message_Object_Type =>
               Set_Text
                 (Select_Time_Line.Entries (Select_Time_Line.Nb_Entries).
                 Object_Type,
                  To_String (Lb_Message (Current_Language)));
            when Task_Object_Type =>
               Set_Text
                 (Select_Time_Line.Entries (Select_Time_Line.Nb_Entries).
                 Object_Type,
                  To_String (Lb_Task (Current_Language)));
            when Buffer_Object_Type =>
               Set_Text
                 (Select_Time_Line.Entries (Select_Time_Line.Nb_Entries).
                 Object_Type,
                  To_String (Lb_Buffer (Current_Language)));
            when Resource_Object_Type =>
               Set_Text
                 (Select_Time_Line.Entries (Select_Time_Line.Nb_Entries).
                 Object_Type,
                  To_String (Lb_Resource (Current_Language)));
         end case;

         Set_Visibility
           (Select_Time_Line.Entries (Select_Time_Line.Nb_Entries).Object_Type,
            True);
         Attach
           (Select_Time_Line.Table,
            Select_Time_Line.Entries (Select_Time_Line.Nb_Entries).Object_Type,
            1,
            2,
            Line,
            Line + 1,
            Expand or Fill,
            0,
            0,
            0);

         Gtk_New
           (Select_Time_Line.Entries (Select_Time_Line.Nb_Entries).
           Display_Time_Line,
            "");
         Set_Active
           (Select_Time_Line.Entries (Select_Time_Line.Nb_Entries).
           Display_Time_Line,
            False);
         Attach
           (Select_Time_Line.Table,
            Select_Time_Line.Entries (Select_Time_Line.Nb_Entries).
           Display_Time_Line,
            2,
            3,
            Line,
            Line + 1,
            Expand or Fill,
            0,
            0,
            0);

         Line                        := Line + 1;
         Select_Time_Line.Nb_Entries := Select_Time_Line.Nb_Entries + 1;
      end New_Object;

   begin

      Reset (Time_Line_To_Display);
      Select_Time_Line.Nb_Entries   := 0;
      Select_Time_Line.To_Be_Called := Call;

      Gtk.Window.Initialize (Select_Time_Line, Window_Toplevel);
      Set_Title
        (Select_Time_Line,
         To_String (Lb_Select_Time_Line (Current_Language)));
      Set_Policy (Select_Time_Line, True, True, True);
      Set_Position (Select_Time_Line, Win_Pos_Center_Always);
      Set_Modal (Select_Time_Line, True);
      Set_Default_Size (Select_Time_Line, 530, 550);
      Set_Extension_Events (Select_Time_Line, Extension_Events_All);
      Set_Flags (Select_Time_Line, Can_Default);
      Grab_Focus (Select_Time_Line);

      Gtk_New (Select_Time_Line.Scroll);
      Set_Border_Width (Select_Time_Line.Scroll, Border_Width => 10);
      Set_Policy
        (Select_Time_Line.Scroll,
         H_Scrollbar_Policy => Policy_Automatic,
         V_Scrollbar_Policy => Policy_Automatic);
      Add (Select_Time_Line, Select_Time_Line.Scroll);

      Gtk_New (Select_Time_Line.Table, 0, 0, True);
      Set_Row_Spacings (Select_Time_Line.Table, 8);
      Set_Col_Spacings (Select_Time_Line.Table, 7);

      Add_With_Viewport (Select_Time_Line.Scroll, Select_Time_Line.Table);
      Set_Focus_Hadjustment
        (Select_Time_Line.Table,
         Get_Hadjustment (Select_Time_Line.Scroll));
      Set_Focus_Vadjustment
        (Select_Time_Line.Table,
         Get_Vadjustment (Select_Time_Line.Scroll));

      Gtk_New (Select_Time_Line.Lab, To_String (Lb_Name (Current_Language)));
      Set_Alignment (Select_Time_Line.Lab, 0.5, 0.5);
      Set_Padding (Select_Time_Line.Lab, 0, 0);
      Set_Justify (Select_Time_Line.Lab, Justify_Center);
      Set_Line_Wrap (Select_Time_Line.Lab, False);
      Attach
        (Select_Time_Line.Table,
         Select_Time_Line.Lab,
         0,
         1,
         0,
         1,
         0,
         0,
         0,
         0);

      Gtk_New (Select_Time_Line.Lab, To_String (Lb_Type (Current_Language)));
      Set_Alignment (Select_Time_Line.Lab, 0.5, 0.5);
      Set_Padding (Select_Time_Line.Lab, 0, 0);
      Set_Justify (Select_Time_Line.Lab, Justify_Center);
      Set_Line_Wrap (Select_Time_Line.Lab, False);
      Attach
        (Select_Time_Line.Table,
         Select_Time_Line.Lab,
         1,
         2,
         0,
         1,
         0,
         0,
         0,
         0);

      Gtk_New
        (Select_Time_Line.Lab,
         To_String (Lb_To_Display (Current_Language)));
      Set_Alignment (Select_Time_Line.Lab, 0.5, 0.5);
      Set_Padding (Select_Time_Line.Lab, 0, 0);
      Set_Justify (Select_Time_Line.Lab, Justify_Center);
      Set_Line_Wrap (Select_Time_Line.Lab, False);
      Attach
        (Select_Time_Line.Table,
         Select_Time_Line.Lab,
         2,
         3,
         0,
         1,
         0,
         0,
         0,
         0);

      Reset_Iterator (Sys.Tasks, My_Task_Iterator);
      loop
         Current_Element (Sys.Tasks, A_Task, My_Task_Iterator);
         New_Object (Generic_Object_Ptr (A_Task));
         exit when Is_Last_Element (Sys.Tasks, My_Task_Iterator);
         Next_Element (Sys.Tasks, My_Task_Iterator);
      end loop;
      if not Is_Empty (Sys.Buffers) then
         Reset_Iterator (Sys.Buffers, My_Buffer_Iterator);
         loop
            Current_Element (Sys.Buffers, A_Buffer, My_Buffer_Iterator);
            New_Object (Generic_Object_Ptr (A_Buffer));
            exit when Is_Last_Element (Sys.Buffers, My_Buffer_Iterator);
            Next_Element (Sys.Buffers, My_Buffer_Iterator);
         end loop;
      end if;
      if not Is_Empty (Sys.Messages) then
         Reset_Iterator (Sys.Messages, My_Message_Iterator);
         loop
            Current_Element (Sys.Messages, A_Message, My_Message_Iterator);
            New_Object (Generic_Object_Ptr (A_Message));
            exit when Is_Last_Element (Sys.Messages, My_Message_Iterator);
            Next_Element (Sys.Messages, My_Message_Iterator);
         end loop;
      end if;
      if not Is_Empty (Sys.Resources) then
         Reset_Iterator (Sys.Resources, My_Resource_Iterator);
         loop
            Current_Element (Sys.Resources, A_Resource, My_Resource_Iterator);
            New_Object (Generic_Object_Ptr (A_Resource));
            exit when Is_Last_Element (Sys.Resources, My_Resource_Iterator);
            Next_Element (Sys.Resources, My_Resource_Iterator);
         end loop;
      end if;

      Gtk_New_Hseparator (Select_Time_Line.Hseparator);
      Attach
        (Select_Time_Line.Table,
         Select_Time_Line.Hseparator,
         0,
         3,
         Line,
         Line + 1,
         Fill,
         Expand or Fill,
         0,
         0);

      Gtk_New_Hbox (Select_Time_Line.Hbox, True, 0);
      Set_Border_Width (Select_Time_Line.Hbox, 6);
      Attach
        (Select_Time_Line.Table,
         Select_Time_Line.Hbox,
         0,
         3,
         Line + 1,
         Line + 2,
         Fill,
         Expand or Fill,
         0,
         0);

      Gtk_New (Select_Time_Line.Ok_Button, "Ok");
      Pack_Start
        (Select_Time_Line.Hbox,
         Select_Time_Line.Ok_Button,
         True,
         True,
         0);

      Button_Callback.Connect
        (Select_Time_Line.Ok_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'Access));

   end Initialize;

   function To_Display
     (Searched_Name : in Unbounded_String;
      Searched_Type : in Objects_Type)
      return          Boolean
   is
      Ite       : Time_Line_Object_Iterator;
      An_Object : Generic_Object_Ptr;

   begin
      if Is_Empty (Time_Line_To_Display) then
         return True;
      end if;

      Reset_Iterator (Time_Line_To_Display, Ite);
      loop
         Current_Element (Time_Line_To_Display, An_Object, Ite);

         if (An_Object.Name = Searched_Name) and
            (An_Object.Object_Type = Searched_Type)
         then
            return True;
         end if;

         exit when Is_Last_Element (Time_Line_To_Display, Ite);
         Next_Element (Time_Line_To_Display, Ite);
      end loop;

      return False;
   end To_Display;

end Select_Time_Line_Pkg;
