------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Glib;      use Glib;
with Mmi_Tools; use Mmi_Tools;

package body Text_Tools is

   procedure Write_Text (A_Text : in Gtk_Text; Msg : Unbounded_String) is
   begin
      Freeze (A_Text);
      Insert (A_Text, Font_R, Black, Null_Color, To_String (Msg));
      Thaw (A_Text);

      Set_Position (A_Text, Gint (Get_Length (A_Text)));
   end Write_Text;

   procedure Write_Title (A_Text : in Gtk_Text; Msg : Unbounded_String) is
   begin
      Freeze (A_Text);
      Insert (A_Text, Font_B, Red, Null_Color, To_String (Msg));
      Thaw (A_Text);

      Set_Position (A_Text, Gint (Get_Length (A_Text)));
   end Write_Title;

   procedure Write_Title_And_Text
     (A_Text        : in Gtk_Text;
      Response_List : in Framework_Response_Table)
   is

   begin
      for J in  0 .. Response_List.Nb_Entries - 1 loop

         Write_Title (A_Text, Response_List.Entries (J).Title);
         Write_Text (A_Text, Response_List.Entries (J).Text);
      end loop;
   end Write_Title_And_Text;

   procedure Initialize is

   begin
      Load (Font_R, "-adobe-courier-medium-r-normal--*-120-*-*-*-*-*-*");
      Load (Font_B, "-adobe-courier-bold-r-*-*-*-200-*-*-*-*-*-*");

      Set_Rgb
        (Red,
         Text_Colors (2).Red,
         Text_Colors (2).Green,
         Text_Colors (2).Blue);
      Set_Rgb
        (Black,
         Text_Colors (0).Red,
         Text_Colors (0).Green,
         Text_Colors (0).Blue);
   end Initialize;

end Text_Tools;
