------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Glib;                                    use Glib;
with Mmi_Tools;                               use Mmi_Tools;
with Callbacks_Cheddar;                       use Callbacks_Cheddar;
with Update_Address_Spaces_Set_Pkg.Callbacks;
use Update_Address_Spaces_Set_Pkg.Callbacks;
with Processors;                              use Processors;
with Unbounded_Strings;                       use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;
with Project;                                 use Project;
with Translate;                               use Translate;
with Framework_Config;                        use Framework_Config;

package body Update_Address_Spaces_Set_Pkg is

   procedure Gtk_New
     (Update_Address_Spaces_Set : out Update_Address_Spaces_Set_Access;
      Title                     : in Unbounded_String)
   is
   begin
      Update_Address_Spaces_Set := new Update_Address_Spaces_Set_Record;
      Update_Address_Spaces_Set_Pkg.Initialize
        (Update_Address_Spaces_Set,
         Title);
   end Gtk_New;

   procedure Show_Advanced
     (Update_Address_Spaces_Set : in Update_Address_Spaces_Set_Access;
      Advanced                  : in Boolean)
   is

   begin

      if not Advanced then
         Hide (Update_Address_Spaces_Set.Vbox_List);
         Hide (Update_Address_Spaces_Set.Delete_Button);
         Hide (Update_Address_Spaces_Set.Modify_Button);
         Show (Update_Address_Spaces_Set.Close_Button);
         Show (Update_Address_Spaces_Set.Cancel_Button);
         Show (Update_Address_Spaces_Set.Advanced_Button);

         Show (Update_Address_Spaces_Set.Vbox_Main);
         Show (Update_Address_Spaces_Set.Hbox_Address_Space);
         Show_All (Update_Address_Spaces_Set.Vbox_Update);
         Show (Update_Address_Spaces_Set.Hbutton_Box_Modify);
         Show (Update_Address_Spaces_Set.Frame_Address_Space);

      else
         Show_All (Update_Address_Spaces_Set.Vbox_Main);
      end if;
      Show (Update_Address_Spaces_Set);

   end Show_Advanced;

   procedure Initialize
     (Update_Address_Spaces_Set : access Update_Address_Spaces_Set_Record'
        Class;
      Title                     : in Unbounded_String)
   is

      Processor_Iterator : Processors_Iterator;
      Processor_Items    : Strings_Table;
      A_Processor        : Processor_Ptr;

      Scheduler_Items : Strings_Table;

   begin

      Initialize (Update_Address_Spaces_Set.Undo);
      Duplicate (Sys, Update_Address_Spaces_Set.Undo);
      Update_Address_Spaces_Set.Current_Address_Space :=
         To_Unbounded_String ("");

      -- List of scheduler
      --
      Initialize (Scheduler_Items);
      Build_Local_Scheduler_Table_Name (Scheduler_Items);

      Gtk.Window.Initialize (Update_Address_Spaces_Set, Window_Toplevel);
      Set_Policy (Update_Address_Spaces_Set, True, True, True);
      Set_Title (Update_Address_Spaces_Set, To_String (Title));
      Set_Position (Update_Address_Spaces_Set, Win_Pos_Center_Always);
      Set_Modal (Update_Address_Spaces_Set, True);
      Grab_Focus (Update_Address_Spaces_Set);

      Gtk.Box.Gtk_New_Vbox (Update_Address_Spaces_Set.Vbox_Main, False, 10);
      Set_Border_Width (Update_Address_Spaces_Set.Vbox_Main, 10);
      Add (Update_Address_Spaces_Set, Update_Address_Spaces_Set.Vbox_Main);

      Gtk.Box.Gtk_New_Hbox
        (Update_Address_Spaces_Set.Hbox_Address_Space,
         False,
         10);
      Add
        (Update_Address_Spaces_Set.Vbox_Main,
         Update_Address_Spaces_Set.Hbox_Address_Space);

      Gtk.Box.Gtk_New_Vbox (Update_Address_Spaces_Set.Vbox_Update, False, 10);
      Add
        (Update_Address_Spaces_Set.Hbox_Address_Space,
         Update_Address_Spaces_Set.Vbox_Update);

      Gtk.Box.Gtk_New_Vbox (Update_Address_Spaces_Set.Vbox_List, False, 10);
      Add
        (Update_Address_Spaces_Set.Hbox_Address_Space,
         Update_Address_Spaces_Set.Vbox_List);

      Gtk.Frame.Gtk_New (Update_Address_Spaces_Set.Frame_Address_Space, "");
      Add
        (Update_Address_Spaces_Set.Vbox_Update,
         Update_Address_Spaces_Set.Frame_Address_Space);

      Gtk_New (Update_Address_Spaces_Set.Hbutton_Box_Modify);
      Set_Spacing (Update_Address_Spaces_Set.Hbutton_Box_Modify, 10);
      Set_Layout
        (Update_Address_Spaces_Set.Hbutton_Box_Modify,
         Buttonbox_Default_Style);
      Set_Child_Size (Update_Address_Spaces_Set.Hbutton_Box_Modify, 20, 20);
      Set_Child_Ipadding (Update_Address_Spaces_Set.Hbutton_Box_Modify, 7, 0);
      Pack_Start
        (Update_Address_Spaces_Set.Vbox_Update,
         Update_Address_Spaces_Set.Hbutton_Box_Modify,
         False,
         True,
         5);

      Gtk.Table.Gtk_New
        (Update_Address_Spaces_Set.Address_Space_Table,
         2,
         15,
         False);
      Set_Border_Width (Update_Address_Spaces_Set.Address_Space_Table, 5);
      Add
        (Update_Address_Spaces_Set.Frame_Address_Space,
         Update_Address_Spaces_Set.Address_Space_Table);

      Gtk.Label.Gtk_New
        (Update_Address_Spaces_Set.Lab,
         To_String (Lb_Address_Space_Name (Current_Language)) &
         To_String (Lb_Colon));
      Attach_Defaults
        (Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.Lab,
         0,
         1,
         0,
         1);

      Gtk.Label.Gtk_New
        (Update_Address_Spaces_Set.Lab,
         To_String (Lb_Processor (Current_Language)) &
         To_String (Lb_Colon));
      Attach_Defaults
        (Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.Lab,
         0,
         1,
         1,
         2);

      Gtk.Label.Gtk_New
        (Update_Address_Spaces_Set.Lab,
         To_String (Lb_Scheduler (Current_Language)) &
         To_String (Lb_Colon));
      Attach_Defaults
        (Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.Lab,
         0,
         1,
         2,
         3);

      Gtk.Label.Gtk_New (Update_Address_Spaces_Set.Lab, "Quantum :");
      Attach_Defaults
        (Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.Lab,
         0,
         1,
         3,
         4);

      Gtk.Label.Gtk_New
        (Update_Address_Spaces_Set.Lab,
         To_String (Lb_Option (Current_Language)) & To_String (Lb_Colon));
      Attach_Defaults
        (Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.Lab,
         0,
         1,
         4,
         5);

      Gtk.Label.Gtk_New
        (Update_Address_Spaces_Set.Lab,
         To_String (Lb_File_Name (Current_Language)) &
         To_String (Lb_Colon));
      Attach_Defaults
        (Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.Lab,
         0,
         1,
         5,
         6);

      Gtk.Label.Gtk_New
        (Update_Address_Spaces_Set.Lab,
         To_String (Lb_Text_Memory_Size (Current_Language)) &
         To_String (Lb_Colon));
      Attach_Defaults
        (Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.Lab,
         0,
         1,
         7,
         8);

      Gtk.Label.Gtk_New
        (Update_Address_Spaces_Set.Lab,
         To_String (Lb_Heap_Memory_Size (Current_Language)) &
         To_String (Lb_Colon));
      Attach_Defaults
        (Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.Lab,
         0,
         1,
         8,
         9);

      Gtk.Label.Gtk_New
        (Update_Address_Spaces_Set.Lab,
         To_String (Lb_Stack_Memory_Size (Current_Language)) &
         To_String (Lb_Colon));
      Attach_Defaults
        (Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.Lab,
         0,
         1,
         9,
         10);

      Gtk.Label.Gtk_New
        (Update_Address_Spaces_Set.Lab,
         To_String (Lb_Data_Memory_Size (Current_Language)) &
         To_String (Lb_Colon));
      Attach_Defaults
        (Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.Lab,
         0,
         1,
         10,
         11);

      Initialize (Processor_Items);
      Reset_Iterator (Sys.Processors, Processor_Iterator);

      loop
         Current_Element (Sys.Processors, A_Processor, Processor_Iterator);
         Add (Processor_Items, A_Processor.Name);

         exit when Is_Last_Element (Sys.Processors, Processor_Iterator);

         Next_Element (Sys.Processors, Processor_Iterator);
      end loop;

      Update_Address_Spaces_Set.Clist_Processor_Menu :=
        Widget_SList.Null_List;
      Build_Option_Menu
        (Update_Address_Spaces_Set.Processor_Menu,
         Update_Address_Spaces_Set.Clist_Processor_Menu,
         Processor_Items,
         -1,
         null);

      Gtk.GEntry.Gtk_New (Update_Address_Spaces_Set.Name, 50);
      Set_Editable (Update_Address_Spaces_Set.Name, True);
      Attach_Defaults
        (Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.Name,
         1,
         2,
         0,
         1);

      Gtk.Option_Menu.Gtk_New (Update_Address_Spaces_Set.Processor_Menu);
      Build_Option_Menu
        (Update_Address_Spaces_Set.Processor_Menu,
         Update_Address_Spaces_Set.Clist_Processor_Menu,
         Processor_Items,
         0,
         null);
      Attach_Defaults
        (Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.Processor_Menu,
         1,
         2,
         1,
         2);

      Gtk.Option_Menu.Gtk_New (Update_Address_Spaces_Set.Scheduler_Menu);
      Build_Option_Menu
        (Update_Address_Spaces_Set.Scheduler_Menu,
         Update_Address_Spaces_Set.Clist_Scheduler_Menu,
         Scheduler_Items,
         -1,
         null);
      Attach_Defaults
        (Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.Scheduler_Menu,
         1,
         2,
         2,
         3);

      Gtk.GEntry.Gtk_New (Update_Address_Spaces_Set.Quantum, 50);
      Set_Editable (Update_Address_Spaces_Set.Quantum, True);
      Attach_Defaults
        (Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.Quantum,
         1,
         2,
         3,
         4);
      Gtk.GEntry.Set_Text (Update_Address_Spaces_Set.Quantum, "");

      Gtk_New
        (Update_Address_Spaces_Set.Option,
         To_String (Lb_Preemptive (Current_Language)));
      Set_Active (Update_Address_Spaces_Set.Option, True);
      Attach_Defaults
        (Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.Option,
         1,
         2,
         4,
         5);

      Gtk.GEntry.Gtk_New (Update_Address_Spaces_Set.File_Name, 50);
      Attach_Defaults
        (Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.File_Name,
         1,
         2,
         5,
         6);

      Gtk.Button.Gtk_New
        (Update_Address_Spaces_Set.Read_Button,
         To_String (Lb_Read (Current_Language)));
      Attach_Defaults
        (Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.Read_Button,
         1,
         2,
         6,
         7);

      Gtk.GEntry.Gtk_New (Update_Address_Spaces_Set.Text_Memory_Size, 50);
      Attach_Defaults
        (Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.Text_Memory_Size,
         1,
         2,
         7,
         8);
      Gtk.GEntry.Set_Text (Update_Address_Spaces_Set.Text_Memory_Size, "");

      Gtk.GEntry.Gtk_New (Update_Address_Spaces_Set.Heap_Memory_Size, 50);
      Attach_Defaults
        (Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.Heap_Memory_Size,
         1,
         2,
         8,
         9);
      Gtk.GEntry.Set_Text (Update_Address_Spaces_Set.Heap_Memory_Size, "");

      Gtk.GEntry.Gtk_New (Update_Address_Spaces_Set.Stack_Memory_Size, 50);
      Attach_Defaults
        (Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.Stack_Memory_Size,
         1,
         2,
         9,
         10);
      Gtk.GEntry.Set_Text (Update_Address_Spaces_Set.Stack_Memory_Size, "");

      Gtk.GEntry.Gtk_New (Update_Address_Spaces_Set.Data_Memory_Size, 50);
      Attach_Defaults
        (Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.Data_Memory_Size,
         1,
         2,
         10,
         11);
      Gtk.GEntry.Set_Text (Update_Address_Spaces_Set.Data_Memory_Size, "");

      Gtk.Button.Gtk_New
        (Update_Address_Spaces_Set.Close_Button,
         To_String (Lb_Close (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_Address_Spaces_Set.Cancel_Button,
         To_String (Lb_Cancel (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_Address_Spaces_Set.Advanced_Button,
         To_String (Lb_Advanced (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_Address_Spaces_Set.Delete_Button,
         To_String (Lb_Delete (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_Address_Spaces_Set.Modify_Button,
         To_String (Lb_Modify (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_Address_Spaces_Set.Add_Button,
         To_String (Lb_Add (Current_Language)));

      Add
        (Update_Address_Spaces_Set.Hbutton_Box_Modify,
         Update_Address_Spaces_Set.Close_Button);
      Add
        (Update_Address_Spaces_Set.Hbutton_Box_Modify,
         Update_Address_Spaces_Set.Cancel_Button);
      Add
        (Update_Address_Spaces_Set.Hbutton_Box_Modify,
         Update_Address_Spaces_Set.Advanced_Button);
      Add
        (Update_Address_Spaces_Set.Hbutton_Box_Modify,
         Update_Address_Spaces_Set.Delete_Button);
      Add
        (Update_Address_Spaces_Set.Hbutton_Box_Modify,
         Update_Address_Spaces_Set.Modify_Button);
      Add
        (Update_Address_Spaces_Set.Hbutton_Box_Modify,
         Update_Address_Spaces_Set.Add_Button);

      -- Spreadsheet for address_space information
      --
      Gtk.Scrolled_Window.Gtk_New
        (Update_Address_Spaces_Set.Scroll_List,
         null,
         null);
      Add
        (Update_Address_Spaces_Set.Vbox_List,
         Update_Address_Spaces_Set.Scroll_List);

      Update_Address_Spaces_Set.List_Titles :=
        To_String (Lb_Name (Current_Language)) +
        To_String (Lb_Processor (Current_Language)) +
        To_String (Lb_Stack (Current_Language)) +
        To_String (Lb_Data (Current_Language)) +
        To_String (Lb_Heap (Current_Language)) +
        To_String (Lb_Text (Current_Language));

      Gtk.Clist.Gtk_New
        (Update_Address_Spaces_Set.Address_Spaces_List,
         6,
         Update_Address_Spaces_Set.List_Titles);
      Set_Usize (Update_Address_Spaces_Set.Address_Spaces_List, 475, 200);
      Set_Sort_Column (Update_Address_Spaces_Set.Address_Spaces_List, 0);
      Set_Sort_Type
        (Update_Address_Spaces_Set.Address_Spaces_List,
         Ascending);
      Set_Auto_Sort (Update_Address_Spaces_Set.Address_Spaces_List, True);
      Add
        (Update_Address_Spaces_Set.Scroll_List,
         Update_Address_Spaces_Set.Address_Spaces_List);

      Show_Address_Spaces (Update_Address_Spaces_Set);

      -- Select address space from spreadsheet activation
      --
      List_Callback.Connect
        (Update_Address_Spaces_Set.Address_Spaces_List,
         "select_row",
         List_Callback.To_Marshaller (On_Click_List'Access));

      Button_Callback.Connect
        (Update_Address_Spaces_Set.Modify_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Modify_Pressed'Access));
      Button_Callback.Connect
        (Update_Address_Spaces_Set.Cancel_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'Access));
      Button_Callback.Connect
        (Update_Address_Spaces_Set.Advanced_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Advanced_Pressed'Access));
      Button_Callback.Connect
        (Update_Address_Spaces_Set.Close_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Close_Pressed'Access));
      Button_Callback.Connect
        (Update_Address_Spaces_Set.Add_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Add_Pressed'Access));
      Button_Callback.Connect
        (Update_Address_Spaces_Set.Delete_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Delete_Pressed'Access));
      Button_Callback.Connect
        (Update_Address_Spaces_Set.Read_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Read_Pressed'Access));

   end Initialize;

end Update_Address_Spaces_Set_Pkg;
