------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Task_Dependencies;                                 use Task_Dependencies;
with Graph_Window_Pkg;                                  use Graph_Window_Pkg;
with GNAT.Current_Exception;                            use 
  GNAT.Current_Exception;
with Glib;                                              use Glib;
with Mmi_Tools;                                         use Mmi_Tools;
with Translate;                                         use Translate;
with Unbounded_Strings;                                 use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;
with Project;                                           use Project;
with Framework_Config;                                  use Framework_Config;
with Address_Spaces;                                    use Address_Spaces;
with Address_Space_Set;                                 use Address_Space_Set;
use Address_Space_Set.Generic_Address_Space_Set;
with Tasks;                                             use Tasks;
with Task_Set;                                          use Task_Set;
use Task_Set.Generic_Task_Set;
with Resource_Set;                                      use Resource_Set;
use Resource_Set.Generic_Resource_Set;
with Queueing_System;                                   use Queueing_System;
with Update_Buffers_Set_Pkg.Callbacks.Integrity_Checks;

package body Update_Buffers_Set_Pkg.Callbacks is

   procedure Show_Buffers
     (Update_Buffers_Set : access Update_Buffers_Set_Record'Class)
   is

      A_Buffer     : Buffer_Ptr;
      My_Iterator  : Buffers_Iterator;
      Line         : Gint;
      Buffer_Share : Chars_Ptr_Array (1 .. 3);
      Size         : Unbounded_String;

   begin
      Clear (Update_Buffers_Set.Buffers_List);

      Reset_Iterator (Sys.Buffers, My_Iterator);
      if not Is_Empty (Sys.Buffers) then
         loop

            Current_Element (Sys.Buffers, A_Buffer, My_Iterator);

            Size := To_Unbounded_String (Natural'Image (A_Buffer.Size));

            -- Create the line
            --
            Buffer_Share := To_String (A_Buffer.Name) +
                            To_String (A_Buffer.Cpu_Name) +
                            To_String (Size);

            -- Display the Line
            --
            Line := Append (Update_Buffers_Set.Buffers_List, Buffer_Share);

            exit when Is_Last_Element (Sys.Buffers, My_Iterator);

            Next_Element (Sys.Buffers, My_Iterator);

         end loop;
      end if;
   end Show_Buffers;

   procedure Show_Task_Role_List
     (Update_Buffers_Set : access Update_Buffers_Set_Record'Class)
   is

      Task_Name   : Unbounded_String;
      Role_Time   : Natural;
      Role_Size   : Natural;
      A_Role_Type : Role_type;
      Line        : Gint;
      Role_Share  : Chars_Ptr_Array (1 .. 4);

   begin
      Clear (Update_Buffers_Set.Affected_Role_List);

      for I in  0 .. Update_Buffers_Set.My_Roles.Nb_Entries - 1 loop

         Task_Name   := Update_Buffers_Set.My_Roles.Entries (I).Item;
         Role_Size   := Update_Buffers_Set.My_Roles.Entries (I).Data.Size;
         Role_Time   := Update_Buffers_Set.My_Roles.Entries (I).Data.Time;
         A_Role_Type := Update_Buffers_Set.My_Roles.Entries (I).Data.The_Role;

         Role_Share := To_String (Task_Name) +
                       Role_Size'Img +
                       A_Role_Type'Img +
                       Role_Time'Img;

         -- Show Task on Clist
         --
         Line := Append (Update_Buffers_Set.Affected_Role_List, Role_Share);

      end loop;

   end Show_Task_Role_List;

   procedure On_Click_List (List : access Gtk_Clist_Record'Class) is

      Name : Unbounded_String;
      Size : Unbounded_String;
      Row  : Gint;

      A_Buffer : Buffer_Ptr;

      A_Address_Space        : Address_Space_Ptr;
      Address_Space_Iterator :
        Address_Space_Set.Generic_Address_Space_Set.Iterator;
      Empty_Menu             : String_List.Glist;

   begin

      Row                               :=
         Get_Focus_Row (Update_Buffers_Set.Buffers_List);
      Name                              :=
         To_Unbounded_String
           (Get_Text (Update_Buffers_Set.Buffers_List, Row, 0));
      Update_Buffers_Set.Current_Buffer := Name;
      Size                              :=
         To_Unbounded_String
           (Get_Text (Update_Buffers_Set.Buffers_List, Row, 2));

      A_Buffer :=
         Search_Buffer (Sys.Buffers, Update_Buffers_Set.Current_Buffer);

      -- Show buffer name
      --
      Gtk.GEntry.Set_Text (Update_Buffers_Set.Name, To_String (Name));

      -- Show buffer size
      --
      Gtk.GEntry.Set_Text (Update_Buffers_Set.Size, To_String (Size));

      -- Rebuild The CPU Option Menu & select the right item
      --
      A_Address_Space :=
         Search_Address_Space
           (Sys.Address_Spaces,
            A_Buffer.Address_Space_Name);
      Set_Text
        (Update_Buffers_Set.Cpu_Name,
         To_String (A_Address_Space.Cpu_Name));

      Update_Buffers_Set.Clist_Address_Space_Menu := Empty_Menu;
      Reset_Iterator (Sys.Address_Spaces, Address_Space_Iterator);
      loop
         Current_Element
           (Sys.Address_Spaces,
            A_Address_Space,
            Address_Space_Iterator);
         if A_Address_Space.Name = A_Buffer.Address_Space_Name then
            String_List.Append
              (Update_Buffers_Set.Clist_Address_Space_Menu,
               To_String (A_Address_Space.Name));
         end if;
         exit when Is_Last_Element
                      (Sys.Address_Spaces,
                       Address_Space_Iterator);

         Next_Element (Sys.Address_Spaces, Address_Space_Iterator);
      end loop;
      Reset_Iterator (Sys.Address_Spaces, Address_Space_Iterator);
      loop
         Current_Element
           (Sys.Address_Spaces,
            A_Address_Space,
            Address_Space_Iterator);
         if A_Address_Space.Name /= A_Buffer.Address_Space_Name then
            String_List.Append
              (Update_Buffers_Set.Clist_Address_Space_Menu,
               To_String (A_Address_Space.Name));
         end if;
         exit when Is_Last_Element
                      (Sys.Address_Spaces,
                       Address_Space_Iterator);

         Next_Element (Sys.Address_Spaces, Address_Space_Iterator);
      end loop;

      Set_Popdown_Strings
        (Update_Buffers_Set.Address_Space_Menu,
         Update_Buffers_Set.Clist_Address_Space_Menu);

      -- Show queueing system
      --
      Update_Buffers_Set.Clist_Qs_Menu := Empty_Menu;
      String_List.Append
        (Update_Buffers_Set.Clist_Qs_Menu,
         To_String (Qs_To_Display_String (A_Buffer.Queueing_System_Type)));

      for I in  Queueing_Systems_Type'Range loop
         if I /= A_Buffer.Queueing_System_Type then
            String_List.Append
              (Update_Buffers_Set.Clist_Qs_Menu,
               To_String (Qs_To_Display_String (I)));
         end if;
      end loop;
      Set_Popdown_Strings
        (Update_Buffers_Set.Qs_Menu,
         Update_Buffers_Set.Clist_Qs_Menu);

      --  Show task using the buffer
      --
      Initialize (Update_Buffers_Set.My_Roles);
      for I in  0 .. A_Buffer.Roles.Nb_Entries - 1 loop
         Update_Buffers_Set.My_Roles.Entries (I).Item :=
           A_Buffer.Roles.Entries (I).Item;
         Update_Buffers_Set.My_Roles.Entries (I).Data :=
           A_Buffer.Roles.Entries (I).Data;
         Update_Buffers_Set.My_Roles.Nb_Entries       :=
           Update_Buffers_Set.My_Roles.Nb_Entries + 1;
      end loop;

      Show_Task_Role_List (Update_Buffers_Set);
      Show_All (Update_Buffers_Set);

   end On_Click_List;

   procedure On_Change_Cpu (Widget : access Gtk_Editable_Record'Class) is

      Buffer_Address_Space : Unbounded_String;
      A_Address_Space      : Address_Space_Ptr;

   begin
      Buffer_Address_Space :=
         To_Unbounded_String
           (Get_Text (Get_Entry (Update_Buffers_Set.Address_Space_Menu)));
      A_Address_Space      :=
         Search_Address_Space (Sys.Address_Spaces, Buffer_Address_Space);
      Set_Text
        (Update_Buffers_Set.Cpu_Name,
         To_String (A_Address_Space.Cpu_Name));
   end On_Change_Cpu;

   procedure On_Cancel_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      Duplicate (Update_Buffers_Set.Undo, Sys);
      Rebuild_Canvas (Graph_Window);
      Destroy (Update_Buffers_Set);
   end On_Cancel_Pressed;

   procedure On_Advanced_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      if Visible_Is_Set (Update_Buffers_Set.Vbox_List) then
         Show_Advanced (Update_Buffers_Set, False);
      else
         Show_Advanced (Update_Buffers_Set, True);
      end if;
   end On_Advanced_Pressed;

   procedure On_Close_Pressed (Object : access Gtk_Button_Record'Class) is

      A_Buff      : Buffer_Ptr;
      My_Iterator : Buffers_Iterator;

   begin

      Is_New   := False;
      Is_Saved := False;

      Reset_Iterator (Sys.Buffers, My_Iterator);
      loop
         Current_Element (Sys.Buffers, A_Buff, My_Iterator);

         -- For all buffers, update dependencies :
         -- Delete and then add all dependencies
         --
         Add_All_Task_Dependencies (Sys.Dependencies, Sys.Tasks, A_Buff);

         exit when Is_Last_Element (Sys.Buffers, My_Iterator);
         Next_Element (Sys.Buffers, My_Iterator);
      end loop;

      Rebuild_Canvas (Graph_Window);
      Destroy (Update_Buffers_Set);
   end On_Close_Pressed;

   procedure On_Delete_Pressed (Object : access Gtk_Button_Record'Class) is

      My_Iterator : Buffer_Set.Generic_Buffer_Set.Iterator;
      A_Buffer    : Buffer_Ptr;

   begin

      if Update_Buffers_Set.Current_Buffer = "" then
         Show_Message_Box (Lb_Select_Buffer_First (Current_Language));
         return;
      end if;

      if Is_Empty (Sys.Buffers) then
         Show_Message_Box (Lb_Can_Not_Modify (Current_Language));
         return;
      end if;

      Reset_Iterator (Sys.Buffers, My_Iterator);
      if not Is_Empty (Sys.Buffers) then
         loop

            Current_Element (Sys.Buffers, A_Buffer, My_Iterator);
            if A_Buffer.Name = Update_Buffers_Set.Current_Buffer then

               Delete_Buffer (Sys, A_Buffer);
               Rebuild_Canvas (Graph_Window);

               -- Clear  entries
               --
               Update_Buffers_Set.Current_Buffer := To_Unbounded_String ("");
               Gtk.GEntry.Set_Text (Update_Buffers_Set.Name, "");
               Gtk.GEntry.Set_Text (Update_Buffers_Set.Size, "");

               exit;
            end if;

            exit when Is_Last_Element (Sys.Buffers, My_Iterator);
            Next_Element (Sys.Buffers, My_Iterator);

         end loop;
      end if;

      Show_Buffers (Update_Buffers_Set);

   end On_Delete_Pressed;

   procedure On_Add_Pressed (Object : access Gtk_Button_Record'Class) is

      Ok            : Boolean := True;
      Error_Message : Unbounded_String;

   begin

      Integrity_Checks.Check_Integrity_Constraints
        (Update_Buffers_Set,
         Error_Message,
         Ok);
      if not Ok then
         Show_Message_Box (Error_Message);
         return;
      end if;

      -- Add the new item in the set
      --
      Add_Buffer
        (Sys.Buffers,
         Integrity_Checks.Buffer_Name,
         Integrity_Checks.Buffer_Size,
         Integrity_Checks.Buffer_Processor,
         Integrity_Checks.Buffer_Address_Space,
         Integrity_Checks.Buffer_Qs,
         Update_Buffers_Set.My_Roles);

      Update_Buffers_Set.Current_Buffer := Integrity_Checks.Buffer_Name;

      Show_Buffers (Update_Buffers_Set);

   exception
      when Buffer_Set.Invalid_Parameter =>
         begin
            Show_Message_Box (Exception_Message);
            return;
         end;
      when others =>
         begin
            Show_Message_Box (Exception_Name & Exception_Message);
            return;
         end;

   end On_Add_Pressed;

   procedure On_Modify_Pressed (Object : access Gtk_Button_Record'Class) is

      A_Buffer    : Buffer_Ptr;
      My_Iterator : Buffer_Set.Generic_Buffer_Set.Iterator;

      Ok            : Boolean := True;
      Error_Message : Unbounded_String;

   begin

      if Update_Buffers_Set.Current_Buffer = "" then
         Show_Message_Box (Lb_Select_Buffer_First (Current_Language));
         return;
      end if;

      if Is_Empty (Sys.Buffers) then
         Show_Message_Box (Lb_Can_Not_Modify (Current_Language));
         return;
      end if;

      Integrity_Checks.Check_Integrity_Constraints
        (Update_Buffers_Set,
         Error_Message,
         Ok);
      if not Ok then
         Show_Message_Box (Error_Message);
         return;
      end if;

      --
      -- Update set
      --

      Reset_Iterator (Sys.Buffers, My_Iterator);
      loop

         Current_Element (Sys.Buffers, A_Buffer, My_Iterator);
         if A_Buffer.Name = Update_Buffers_Set.Current_Buffer then

            Delete (Sys.Buffers, A_Buffer);
            Update_Buffers_Set.Current_Buffer := Integrity_Checks.Buffer_Name;
            exit;
         end if;

         exit when Is_Last_Element (Sys.Buffers, My_Iterator);
         Next_Element (Sys.Buffers, My_Iterator);

      end loop;

      Add_Buffer
        (Sys.Buffers,
         Integrity_Checks.Buffer_Name,
         Integrity_Checks.Buffer_Size,
         Integrity_Checks.Buffer_Processor,
         Integrity_Checks.Buffer_Address_Space,
         Integrity_Checks.Buffer_Qs,
         Update_Buffers_Set.My_Roles);

      Show_Buffers (Update_Buffers_Set);

   exception
      when Buffer_Set.Invalid_Parameter =>

         begin
            Show_Message_Box (Exception_Message);
            return;
         end;
      when others =>
         begin
            Show_Message_Box (Exception_Name & Exception_Message);
            return;
         end;

   end On_Modify_Pressed;

   procedure On_Role_Click (List : access Gtk_Clist_Record'Class) is

      Name        : Unbounded_String;
      Size_String : Unbounded_String;
      Time_String : Unbounded_String;
      Type_String : Unbounded_String;
      Row         : Gint;

      A_Task     : Generic_Task_Ptr;
      Ite        : Tasks_Iterator;
      Task_Items : String_List.Glist;
      Type_Items : String_List.Glist;

   begin

      Row         := Get_Focus_Row (Update_Buffers_Set.Affected_Role_List);
      Name        :=
         To_Unbounded_String
           (Get_Text (Update_Buffers_Set.Affected_Role_List, Row, 0));
      A_Task      := Search_Task (Sys.Tasks, Name);
      Size_String :=
         To_Unbounded_String
           (Get_Text (Update_Buffers_Set.Affected_Role_List, Row, 1));
      Type_String :=
         To_Unbounded_String
           (Get_Text (Update_Buffers_Set.Affected_Role_List, Row, 2));
      Time_String :=
         To_Unbounded_String
           (Get_Text (Update_Buffers_Set.Affected_Role_List, Row, 3));

      -- Set task name
      --
      String_List.Append (Task_Items, To_String (Name));
      Reset_Iterator (Sys.Tasks, Ite);
      loop
         Current_Element (Sys.Tasks, A_Task, Ite);

         if A_Task.Name /= Name then
            String_List.Append (Task_Items, To_String (A_Task.Name));
         end if;

         exit when Is_Last_Element (Sys.Tasks, Ite);

         Next_Element (Sys.Tasks, Ite);
      end loop;

      Set_Popdown_Strings (Update_Buffers_Set.Affect_Task, Task_Items);

      -- Set size and time
      --
      Set_Text (Update_Buffers_Set.Affect_Size, To_String (Size_String));
      Set_Text (Update_Buffers_Set.Affect_Roletime, To_String (Time_String));

      -- Set type
      --
      String_List.Append (Type_Items, To_String (Type_String));
      if Type_String = "CONSUMER" then
         String_List.Append (Type_Items, "PRODUCER");
      else
         String_List.Append (Type_Items, "CONSUMER");
      end if;

      Set_Popdown_Strings (Update_Buffers_Set.Affect_Roletype, Type_Items);

   end On_Role_Click;

   procedure On_Modify_Role_Pressed
     (Object : access Gtk_Button_Record'Class)
   is

      Ok                    : Boolean := True;
      Error_Message         : Unbounded_String;
      Affect_Task_Share     : Buffer_Role;
      Old_Affect_Task_Share : Buffer_Role;
      Name                  : Unbounded_String;
      Natural_Ok            : Boolean;
      Role_Ok               : Boolean;
      Size_String           : Unbounded_String;
      Time_String           : Unbounded_String;
      Type_String           : Unbounded_String;
      Old_Time              : Natural;
      Old_Size              : Natural;
      Old_Type              : Role_type;
      Row                   : Gint;

   begin

      Row         := Get_Focus_Row (Update_Buffers_Set.Affected_Role_List);
      Name        :=
         To_Unbounded_String
           (Get_Text (Update_Buffers_Set.Affected_Role_List, Row, 0));
      Size_String :=
         To_Unbounded_String
           (Get_Text (Update_Buffers_Set.Affected_Role_List, Row, 1));
      Type_String :=
         To_Unbounded_String
           (Get_Text (Update_Buffers_Set.Affected_Role_List, Row, 2));
      Time_String :=
         To_Unbounded_String
           (Get_Text (Update_Buffers_Set.Affected_Role_List, Row, 3));

      To_Natural (Size_String, Old_Size, Natural_Ok);
      To_Natural (Time_String, Old_Time, Natural_Ok);
      To_Role_type (Type_String, Old_Type, Role_Ok);

      Old_Affect_Task_Share.Time     := Old_Time;
      Old_Affect_Task_Share.Size     := Old_Size;
      Old_Affect_Task_Share.The_Role := Old_Type;

      Integrity_Checks.Check_Task_Role_Integrity_Constraints
        (Update_Buffers_Set,
         Error_Message,
         Ok);
      if not Ok then
         Show_Message_Box (Error_Message);
         return;
      end if;

      Affect_Task_Share.Time     := Integrity_Checks.Buffer_Role_Time;
      Affect_Task_Share.Size     := Integrity_Checks.Buffer_Role_Size;
      Affect_Task_Share.The_Role := Integrity_Checks.Buffer_Role_Type;

      -- Find old values and replace them
      --
      for I in  0 .. Update_Buffers_Set.My_Roles.Nb_Entries - 1 loop
         if (Update_Buffers_Set.My_Roles.Entries (I).Item = Name) and
            (Update_Buffers_Set.My_Roles.Entries (I).Data =
             Old_Affect_Task_Share)
         then
            Update_Buffers_Set.My_Roles.Entries (I).Item :=
              Integrity_Checks.Buffer_Role_Task;
            Update_Buffers_Set.My_Roles.Entries (I).Data := Affect_Task_Share;
            exit;
         end if;
      end loop;

      -- Show Task on Clist
      --
      Show_Task_Role_List (Update_Buffers_Set);

   end On_Modify_Role_Pressed;

   procedure On_Delete_Role_Pressed
     (Object : access Gtk_Button_Record'Class)
   is

      Ok                : Boolean := True;
      Error_Message     : Unbounded_String;
      Affect_Task_Share : Buffer_Role;

   begin

      Integrity_Checks.Check_Task_Role_Integrity_Constraints
        (Update_Buffers_Set,
         Error_Message,
         Ok);
      if not Ok then
         Show_Message_Box (Error_Message);
         return;
      end if;

      Affect_Task_Share.Time     := Integrity_Checks.Buffer_Role_Time;
      Affect_Task_Share.Size     := Integrity_Checks.Buffer_Role_Size;
      Affect_Task_Share.The_Role := Integrity_Checks.Buffer_Role_Type;

      -- Find item and delete it
      --
      for I in  0 .. Update_Buffers_Set.My_Roles.Nb_Entries - 1 loop
         if (Update_Buffers_Set.My_Roles.Entries (I).Item =
             Integrity_Checks.Buffer_Role_Task) and
            (Update_Buffers_Set.My_Roles.Entries (I).Data =
             Affect_Task_Share)
         then
            Update_Buffers_Set.My_Roles.Entries (I) :=
              Update_Buffers_Set.My_Roles.Entries (Update_Buffers_Set.My_Roles.
              Nb_Entries -
                                                   1);
            Update_Buffers_Set.My_Roles.Nb_Entries  :=
              Update_Buffers_Set.My_Roles.Nb_Entries - 1;
            exit;
         end if;
      end loop;

      -- Show Task on Clist
      --
      Show_Task_Role_List (Update_Buffers_Set);

   end On_Delete_Role_Pressed;

   procedure On_Add_Role_Pressed (Object : access Gtk_Button_Record'Class) is

      Affect_Task_Share : Buffer_Role;
      Ok                : Boolean := True;
      Error_Message     : Unbounded_String;

   begin

      Integrity_Checks.Check_Task_Role_Integrity_Constraints
        (Update_Buffers_Set,
         Error_Message,
         Ok);
      if not Ok then
         Show_Message_Box (Error_Message);
         return;
      end if;

      Affect_Task_Share.Time     := Integrity_Checks.Buffer_Role_Time;
      Affect_Task_Share.Size     := Integrity_Checks.Buffer_Role_Size;
      Affect_Task_Share.The_Role := Integrity_Checks.Buffer_Role_Type;

      -- Add Task on list
      --
      Update_Buffers_Set.My_Roles.Entries (
        Update_Buffers_Set.My_Roles.Nb_Entries).Item :=
        Integrity_Checks.Buffer_Role_Task;
      Update_Buffers_Set.My_Roles.Entries (
        Update_Buffers_Set.My_Roles.Nb_Entries).Data := Affect_Task_Share;
      Update_Buffers_Set.My_Roles.Nb_Entries :=
        Update_Buffers_Set.My_Roles.Nb_Entries + 1;

      -- Show Task on Clist
      --
      Show_Task_Role_List (Update_Buffers_Set);

   end On_Add_Role_Pressed;

end Update_Buffers_Set_Pkg.Callbacks;
