------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Callbacks_Cheddar;                use Callbacks_Cheddar;
with Update_Buffers_Set_Pkg.Callbacks; use Update_Buffers_Set_Pkg.Callbacks;
with Tasks;                            use Tasks;
with Task_Set;                         use Task_Set;
use Task_Set.Generic_Task_Set;
with Address_Spaces;                   use Address_Spaces;
with Address_Space_Set;                use Address_Space_Set;
use Address_Space_Set.Generic_Address_Space_Set;
with Queueing_System;                  use Queueing_System;
with Unbounded_Strings;                use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;
with Project;                          use Project;
with Translate;                        use Translate;
with Framework_Config;                 use Framework_Config;

package body Update_Buffers_Set_Pkg is

   procedure Gtk_New
     (Update_Buffers_Set : out Update_Buffers_Set_Access;
      Title              : in Unbounded_String;
      From               : in Unbounded_String)
   is
   begin
      Update_Buffers_Set := new Update_Buffers_Set_Record;
      Update_Buffers_Set_Pkg.Initialize (Update_Buffers_Set, Title);
      Update_Buffers_Set_Pkg.Parent := From;
   end Gtk_New;

   procedure Show_Advanced
     (Update_Buffers_Set : in Update_Buffers_Set_Access;
      Advanced           : in Boolean)
   is

   begin

      if not Advanced then
         Hide (Update_Buffers_Set.Vbox_List);
         Hide (Update_Buffers_Set.Delete_Button);
         Hide (Update_Buffers_Set.Modify_Button);
         Show (Update_Buffers_Set.Add_Button);
         Show (Update_Buffers_Set.Close_Button);
         Show (Update_Buffers_Set.Cancel_Button);
         Show (Update_Buffers_Set.Advanced_Button);

         Show (Update_Buffers_Set.Vbox_Main);
         Show (Update_Buffers_Set.Hbox_Buffer);
         Show_All (Update_Buffers_Set.Vbox_Update);
         Show (Update_Buffers_Set.Hbutton_Box_Modify);
         Show (Update_Buffers_Set.Frame_Buffer);

      else
         Show_All (Update_Buffers_Set.Vbox_Main);
      end if;
      Show (Update_Buffers_Set);

   end Show_Advanced;

   procedure Initialize
     (Update_Buffers_Set : access Update_Buffers_Set_Record'Class;
      Title              : in Unbounded_String)
   is

      A_Task                 : Generic_Task_Ptr;
      Task_Items             : String_List.Glist;
      Type_Items             : String_List.Glist;
      Change_Cpu             : Gtk_Entry;
      A_Address_Space        : Address_Space_Ptr;
      First_Address_Space    : Address_Space_Ptr;
      Task_Iterator          : Tasks_Iterator;
      Address_Space_Iterator : Address_Spaces_Iterator;
      Address_Space_Items    : Strings_Table;
      Empty_Menu             : String_List.Glist;

   begin

      Initialize (Update_Buffers_Set.Undo);
      Duplicate (Sys, Update_Buffers_Set.Undo);
      Update_Buffers_Set.Current_Buffer := To_Unbounded_String ("");

      Gtk.Window.Initialize (Update_Buffers_Set, Window_Toplevel);
      Set_Policy (Update_Buffers_Set, True, True, True);
      Set_Title (Update_Buffers_Set, To_String (Title));
      Set_Position (Update_Buffers_Set, Win_Pos_Center_Always);
      Set_Modal (Update_Buffers_Set, True);
      Grab_Focus (Update_Buffers_Set);

      Gtk.Box.Gtk_New_Vbox (Update_Buffers_Set.Vbox_Main, False, 10);
      Set_Border_Width (Update_Buffers_Set.Vbox_Main, 10);
      Add (Update_Buffers_Set, Update_Buffers_Set.Vbox_Main);

      Gtk.Box.Gtk_New_Hbox (Update_Buffers_Set.Hbox_Buffer, False, 10);
      Add (Update_Buffers_Set.Vbox_Main, Update_Buffers_Set.Hbox_Buffer);

      Gtk.Box.Gtk_New_Vbox (Update_Buffers_Set.Vbox_Update, False, 10);
      Add (Update_Buffers_Set.Hbox_Buffer, Update_Buffers_Set.Vbox_Update);

      Gtk.Box.Gtk_New_Vbox (Update_Buffers_Set.Vbox_List, False, 10);
      Add (Update_Buffers_Set.Hbox_Buffer, Update_Buffers_Set.Vbox_List);

      -----------------------------------------------------------------
      --   Main buffer information frame
      -----------------------------------------------------------------

      Gtk.Frame.Gtk_New (Update_Buffers_Set.Frame_Buffer, "");
      Add (Update_Buffers_Set.Vbox_Update, Update_Buffers_Set.Frame_Buffer);

      Gtk.Box.Gtk_New_Vbox
        (Update_Buffers_Set.Vbox_Splited_Buffer,
         False,
         10);
      Add
        (Update_Buffers_Set.Frame_Buffer,
         Update_Buffers_Set.Vbox_Splited_Buffer);

      -- Frames for name/location widgets and affect widgets
      --

      Gtk.Frame.Gtk_New
        (Update_Buffers_Set.Frame_Name_Location,
         To_String (Lb_Buffer_Information (Current_Language)));
      Add
        (Update_Buffers_Set.Vbox_Splited_Buffer,
         Update_Buffers_Set.Frame_Name_Location);

      Gtk.Frame.Gtk_New
        (Update_Buffers_Set.Frame_Affect,
         To_String (Lb_Role (Current_Language)));
      Add (Update_Buffers_Set.Vbox_Update, Update_Buffers_Set.Frame_Affect);

      --  Content of name/location frame
      --

      Gtk.Table.Gtk_New (Update_Buffers_Set.Table_Name_Location, 4, 2, False);
      Add
        (Update_Buffers_Set.Frame_Name_Location,
         Update_Buffers_Set.Table_Name_Location);

      -- Cpu and address space widgets
      --

      Initialize (Address_Space_Items);
      Reset_Iterator (Sys.Address_Spaces, Address_Space_Iterator);
      Gtk.Combo.Gtk_New (Update_Buffers_Set.Address_Space_Menu);
      Set_Editable (Get_Entry (Update_Buffers_Set.Address_Space_Menu), False);

      loop
         Current_Element
           (Sys.Address_Spaces,
            A_Address_Space,
            Address_Space_Iterator);
         String_List.Append
           (Update_Buffers_Set.Clist_Address_Space_Menu,
            To_String (A_Address_Space.Name));
         if First_Address_Space = null then
            First_Address_Space := A_Address_Space;
         end if;

         exit when Is_Last_Element
                      (Sys.Address_Spaces,
                       Address_Space_Iterator);

         Next_Element (Sys.Address_Spaces, Address_Space_Iterator);
      end loop;
      Set_Popdown_Strings
        (Update_Buffers_Set.Address_Space_Menu,
         Update_Buffers_Set.Clist_Address_Space_Menu);

      Gtk.GEntry.Gtk_New (Update_Buffers_Set.Cpu_Name, 50);
      Set_Editable (Update_Buffers_Set.Cpu_Name, False);
      Gtk.GEntry.Set_Text
        (Update_Buffers_Set.Cpu_Name,
         To_String (First_Address_Space.Cpu_Name));

      -- Queueing system widget
      --

      Gtk.Combo.Gtk_New (Update_Buffers_Set.Qs_Menu);
      Set_Editable (Get_Entry (Update_Buffers_Set.Qs_Menu), False);
      Update_Buffers_Set.Clist_Qs_Menu := Empty_Menu;

      for I in  Queueing_Systems_Type'Range loop
         String_List.Append
           (Update_Buffers_Set.Clist_Qs_Menu,
            To_String (Qs_To_Display_String (I)));
      end loop;
      Set_Popdown_Strings
        (Update_Buffers_Set.Qs_Menu,
         Update_Buffers_Set.Clist_Qs_Menu);

      -- Entries and labels (name, size ...)
      --

      Gtk.GEntry.Gtk_New (Update_Buffers_Set.Name, 50);
      Gtk.GEntry.Gtk_New (Update_Buffers_Set.Size, 50);
      Gtk.Label.Gtk_New
        (Update_Buffers_Set.Name_Label,
         To_String (Lb_Buffer_Name (Current_Language) & Lb_Colon));
      Gtk.Label.Gtk_New
        (Update_Buffers_Set.Size_Label,
         To_String (Lb_Size (Current_Language) & Lb_Colon));
      Gtk.Label.Gtk_New
        (Update_Buffers_Set.Cpu_Label,
         To_String (Lb_Processor (Current_Language) & Lb_Colon));
      Gtk.Label.Gtk_New
        (Update_Buffers_Set.Address_Space_Label,
         To_String (Lb_Address_Space (Current_Language) & Lb_Colon));
      Gtk.Label.Gtk_New
        (Update_Buffers_Set.Qs_Label,
         To_String (Lb_Queueing_System (Current_Language) & Lb_Colon));

      -- Attach widgets to name/location table
      --
      Attach_Defaults
        (Update_Buffers_Set.Table_Name_Location,
         Update_Buffers_Set.Name_Label,
         0,
         1,
         0,
         1);
      Attach_Defaults
        (Update_Buffers_Set.Table_Name_Location,
         Update_Buffers_Set.Name,
         1,
         2,
         0,
         1);
      Attach_Defaults
        (Update_Buffers_Set.Table_Name_Location,
         Update_Buffers_Set.Address_Space_Label,
         0,
         1,
         1,
         2);
      Attach_Defaults
        (Update_Buffers_Set.Table_Name_Location,
         Update_Buffers_Set.Address_Space_Menu,
         1,
         2,
         1,
         2);
      Attach_Defaults
        (Update_Buffers_Set.Table_Name_Location,
         Update_Buffers_Set.Cpu_Label,
         0,
         1,
         2,
         3);
      Attach_Defaults
        (Update_Buffers_Set.Table_Name_Location,
         Update_Buffers_Set.Cpu_Name,
         1,
         2,
         2,
         3);
      Attach_Defaults
        (Update_Buffers_Set.Table_Name_Location,
         Update_Buffers_Set.Size_Label,
         0,
         1,
         3,
         4);
      Attach_Defaults
        (Update_Buffers_Set.Table_Name_Location,
         Update_Buffers_Set.Size,
         1,
         2,
         3,
         4);
      Attach_Defaults
        (Update_Buffers_Set.Table_Name_Location,
         Update_Buffers_Set.Qs_Label,
         0,
         1,
         4,
         5);
      Attach_Defaults
        (Update_Buffers_Set.Table_Name_Location,
         Update_Buffers_Set.Qs_Menu,
         1,
         2,
         4,
         5);

      -- Content of Affect frame
      --

      Initialize (Update_Buffers_Set.My_Roles);
      Reset_Iterator (Sys.Tasks, Task_Iterator);

      loop

         Current_Element (Sys.Tasks, A_Task, Task_Iterator);
         String_List.Append (Task_Items, To_String (A_Task.Name));

         exit when Is_Last_Element (Sys.Tasks, Task_Iterator);

         Next_Element (Sys.Tasks, Task_Iterator);

      end loop;

      Gtk.Table.Gtk_New (Update_Buffers_Set.Table_Add_Affect, 2, 4, False);
      Add
        (Update_Buffers_Set.Frame_Affect,
         Update_Buffers_Set.Table_Add_Affect);

      Gtk.Combo.Gtk_New (Update_Buffers_Set.Affect_Task);
      Set_Editable (Get_Entry (Update_Buffers_Set.Affect_Task), False);
      Set_Popdown_Strings (Update_Buffers_Set.Affect_Task, Task_Items);

      Gtk.GEntry.Gtk_New (Update_Buffers_Set.Affect_Size, 10);
      Gtk.GEntry.Gtk_New (Update_Buffers_Set.Affect_Roletime, 10);

      String_List.Append (Type_Items, "CONSUMER");
      String_List.Append (Type_Items, "PRODUCER");
      Gtk.Combo.Gtk_New (Update_Buffers_Set.Affect_Roletype);
      Set_Editable (Get_Entry (Update_Buffers_Set.Affect_Roletype), False);
      Set_Popdown_Strings (Update_Buffers_Set.Affect_Roletype, Type_Items);

      Gtk.Label.Gtk_New
        (Update_Buffers_Set.Task_Label,
         To_String (Lb_Task (Current_Language)));
      Gtk.Label.Gtk_New
        (Update_Buffers_Set.Rolesize_Label,
         To_String (Lb_Size (Current_Language)));
      Gtk.Label.Gtk_New
        (Update_Buffers_Set.Roletime_Label,
         To_String (Lb_Time (Current_Language)));
      Gtk.Label.Gtk_New
        (Update_Buffers_Set.Roletype_Label,
         To_String (Lb_Role_Type (Current_Language)));

      -- Role buttons
      --
      Gtk.Button.Gtk_New
        (Update_Buffers_Set.Button_Add_Role,
         To_String (Lb_Add (Current_Language)));
      Button_Callback.Connect
        (Update_Buffers_Set.Button_Add_Role,
         "clicked",
         Button_Callback.To_Marshaller (On_Add_Role_Pressed'Access));

      Gtk.Button.Gtk_New
        (Update_Buffers_Set.Button_Delete_Role,
         To_String (Lb_Delete (Current_Language)));
      Button_Callback.Connect
        (Update_Buffers_Set.Button_Delete_Role,
         "clicked",
         Button_Callback.To_Marshaller (On_Delete_Role_Pressed'Access));

      Gtk.Button.Gtk_New
        (Update_Buffers_Set.Button_Modify_Role,
         To_String (Lb_Modify (Current_Language)));
      Button_Callback.Connect
        (Update_Buffers_Set.Button_Modify_Role,
         "clicked",
         Button_Callback.To_Marshaller (On_Modify_Role_Pressed'Access));

      -- Append all sub-widget to the affect frame
      --

      Attach_Defaults
        (Update_Buffers_Set.Table_Add_Affect,
         Update_Buffers_Set.Task_Label,
         0,
         1,
         0,
         1);
      Attach_Defaults
        (Update_Buffers_Set.Table_Add_Affect,
         Update_Buffers_Set.Rolesize_Label,
         1,
         2,
         0,
         1);
      Attach_Defaults
        (Update_Buffers_Set.Table_Add_Affect,
         Update_Buffers_Set.Roletype_Label,
         2,
         3,
         0,
         1);
      Attach_Defaults
        (Update_Buffers_Set.Table_Add_Affect,
         Update_Buffers_Set.Roletime_Label,
         3,
         4,
         0,
         1);
      Attach_Defaults
        (Update_Buffers_Set.Table_Add_Affect,
         Update_Buffers_Set.Affect_Task,
         0,
         1,
         1,
         2);
      Attach_Defaults
        (Update_Buffers_Set.Table_Add_Affect,
         Update_Buffers_Set.Affect_Size,
         1,
         2,
         1,
         2);
      Attach_Defaults
        (Update_Buffers_Set.Table_Add_Affect,
         Update_Buffers_Set.Affect_Roletype,
         2,
         3,
         1,
         2);
      Attach_Defaults
        (Update_Buffers_Set.Table_Add_Affect,
         Update_Buffers_Set.Affect_Roletime,
         3,
         4,
         1,
         2);
      Attach_Defaults
        (Update_Buffers_Set.Table_Add_Affect,
         Update_Buffers_Set.Button_Add_Role,
         1,
         2,
         2,
         3);
      Attach_Defaults
        (Update_Buffers_Set.Table_Add_Affect,
         Update_Buffers_Set.Button_Modify_Role,
         2,
         3,
         2,
         3);
      Attach_Defaults
        (Update_Buffers_Set.Table_Add_Affect,
         Update_Buffers_Set.Button_Delete_Role,
         3,
         4,
         2,
         3);

      -- The List of affected Role
      --
      Gtk.Scrolled_Window.Gtk_New
        (Update_Buffers_Set.Scroll_Affected_Role_List,
         null,
         null);
      Add
        (Update_Buffers_Set.Vbox_Main,
         Update_Buffers_Set.Scroll_Affected_Role_List);
      Update_Buffers_Set.Affect_List_Titles :=
        To_String (Lb_Task (Current_Language)) +
        To_String (Lb_Size (Current_Language)) +
        To_String (Lb_Role_Type (Current_Language)) +
        To_String (Lb_Time (Current_Language));
      Gtk.Clist.Gtk_New
        (Update_Buffers_Set.Affected_Role_List,
         4,
         Update_Buffers_Set.Affect_List_Titles);

      Set_Column_Width (Update_Buffers_Set.Affected_Role_List, 0, 125);
      Set_Column_Width (Update_Buffers_Set.Affected_Role_List, 1, 125);
      Set_Column_Width (Update_Buffers_Set.Affected_Role_List, 2, 125);
      Set_Column_Width (Update_Buffers_Set.Affected_Role_List, 3, 125);

      Set_Usize (Update_Buffers_Set.Affected_Role_List, 100, 200);
      Add
        (Update_Buffers_Set.Scroll_Affected_Role_List,
         Update_Buffers_Set.Affected_Role_List);

      -------------------------------------------------------------------
      -- Buttons box

      Gtk_New (Update_Buffers_Set.Hbutton_Box_Modify);
      Set_Spacing (Update_Buffers_Set.Hbutton_Box_Modify, 10);
      Set_Layout
        (Update_Buffers_Set.Hbutton_Box_Modify,
         Buttonbox_Default_Style);
      Set_Child_Size (Update_Buffers_Set.Hbutton_Box_Modify, 20, 20);
      Set_Child_Ipadding (Update_Buffers_Set.Hbutton_Box_Modify, 7, 0);
      Pack_Start
        (Update_Buffers_Set.Vbox_Update,
         Update_Buffers_Set.Hbutton_Box_Modify,
         False,
         True,
         5);

      Gtk.Button.Gtk_New
        (Update_Buffers_Set.Close_Button,
         To_String (Lb_Close (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_Buffers_Set.Cancel_Button,
         To_String (Lb_Cancel (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_Buffers_Set.Advanced_Button,
         To_String (Lb_Advanced (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_Buffers_Set.Delete_Button,
         To_String (Lb_Delete (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_Buffers_Set.Modify_Button,
         To_String (Lb_Modify (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_Buffers_Set.Add_Button,
         To_String (Lb_Add (Current_Language)));

      Add
        (Update_Buffers_Set.Hbutton_Box_Modify,
         Update_Buffers_Set.Close_Button);
      Add
        (Update_Buffers_Set.Hbutton_Box_Modify,
         Update_Buffers_Set.Cancel_Button);
      Add
        (Update_Buffers_Set.Hbutton_Box_Modify,
         Update_Buffers_Set.Advanced_Button);
      Add
        (Update_Buffers_Set.Hbutton_Box_Modify,
         Update_Buffers_Set.Delete_Button);
      Add
        (Update_Buffers_Set.Hbutton_Box_Modify,
         Update_Buffers_Set.Modify_Button);
      Add
        (Update_Buffers_Set.Hbutton_Box_Modify,
         Update_Buffers_Set.Add_Button);

      -- Spreadsheet for buffer information
      --
      Gtk.Scrolled_Window.Gtk_New
        (Update_Buffers_Set.Scroll_List,
         null,
         null);
      Add (Update_Buffers_Set.Vbox_List, Update_Buffers_Set.Scroll_List);

      Update_Buffers_Set.List_Titles :=
        To_String (Lb_Name (Current_Language)) +
        To_String (Lb_Processor (Current_Language)) +
        To_String (Lb_Size (Current_Language));

      Gtk.Clist.Gtk_New
        (Update_Buffers_Set.Buffers_List,
         3,
         Update_Buffers_Set.List_Titles);
      Set_Usize (Update_Buffers_Set.Buffers_List, 475, 200);
      Set_Sort_Column (Update_Buffers_Set.Buffers_List, 0);
      Set_Sort_Type (Update_Buffers_Set.Buffers_List, Ascending);
      Set_Auto_Sort (Update_Buffers_Set.Buffers_List, True);
      Add (Update_Buffers_Set.Scroll_List, Update_Buffers_Set.Buffers_List);

      Show_Buffers (Update_Buffers_Set);

      -- Select buffer from spreadsheet activation
      --
      List_Callback.Connect
        (Update_Buffers_Set.Buffers_List,
         "select_row",
         List_Callback.To_Marshaller (On_Click_List'Access));

      Button_Callback.Connect
        (Update_Buffers_Set.Modify_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Modify_Pressed'Access));
      Button_Callback.Connect
        (Update_Buffers_Set.Cancel_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'Access));
      Button_Callback.Connect
        (Update_Buffers_Set.Advanced_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Advanced_Pressed'Access));
      Button_Callback.Connect
        (Update_Buffers_Set.Close_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Close_Pressed'Access));
      Button_Callback.Connect
        (Update_Buffers_Set.Add_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Add_Pressed'Access));
      Button_Callback.Connect
        (Update_Buffers_Set.Delete_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Delete_Pressed'Access));
      List_Callback.Connect
        (Update_Buffers_Set.Affected_Role_List,
         "select_row",
         List_Callback.To_Marshaller (On_Role_Click'Access));

      Change_Cpu := Get_Entry (Update_Buffers_Set.Address_Space_Menu);
      Combo_Callback.Connect
        (Change_Cpu,
         "changed",
         Combo_Callback.To_Marshaller (On_Change_Cpu'Access));

   end Initialize;

end Update_Buffers_Set_Pkg;
