------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Gtk.Enums;             use Gtk.Enums;
with Gtk.Frame;             use Gtk.Frame;
with Gtk.Label;             use Gtk.Label;
with Gtk.Pixmap;            use Gtk.Pixmap;
with Gtk.Separator;         use Gtk.Separator;
with Gtk;                   use Gtk;
with Gtk.Window;            use Gtk.Window;
with Gtk.Table;             use Gtk.Table;
with Gtk.GEntry;            use Gtk.GEntry;
with Gtk.Button;            use Gtk.Button;
with Gtk.Box;               use Gtk.Box;
with Gtk.Hbutton_Box;       use Gtk.Hbutton_Box;
with Gtk.Scrolled_Window;   use Gtk.Scrolled_Window;
with Gtk.Option_Menu;       use Gtk.Option_Menu;
with Gtk.Widget;            use Gtk.Widget;
with Buffer_Set;            use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Processor_Set;         use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Gtk.Clist;             use Gtk.Clist;
with Gtkada.Types;          use Gtkada.Types;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Gtk.Combo;             use Gtk.Combo;
with Buffers;               use Buffers;
use Buffers.Buffer_Roles;
with Systems;               use Systems;

package Update_Buffers_Set_Pkg is

   type Update_Buffers_Set_Record is new Gtk_Window_Record with private;
   type Update_Buffers_Set_Access is access all Update_Buffers_Set_Record'
     Class;

   procedure Gtk_New
     (Update_Buffers_Set : out Update_Buffers_Set_Access;
      Title              : in Unbounded_String;
      From               : in Unbounded_String);
   procedure Initialize
     (Update_Buffers_Set : access Update_Buffers_Set_Record'Class;
      Title              : in Unbounded_String);
   procedure Show_Advanced
     (Update_Buffers_Set : in Update_Buffers_Set_Access;
      Advanced           : in Boolean);

   Update_Buffers_Set : Update_Buffers_Set_Access;
   Parent             : Unbounded_String;

private

   type Update_Buffers_Set_Record is new Gtk_Window_Record with record
      Scroll : Gtk_Scrolled_Window;

      Buffers_List           : Gtk.Clist.Gtk_Clist;
      Buffer_Type_Menu       : Gtk_Option_Menu;
      Clist_Buffer_Type_Menu : Widget_SList.GSlist;
      Buffer_Items           : String_List.Glist;

      Current_Buffer : Unbounded_String;

      Parent : Unbounded_String;

      Name     : Gtk.GEntry.Gtk_Entry;
      Size     : Gtk.GEntry.Gtk_Entry;
      My_Roles : Buffers.Buffer_Roles_Table;

      Cpu_Name                 : Gtk_Entry;
      Address_Space_Menu       : Gtk.Combo.Gtk_Combo;
      Clist_Address_Space_Menu : String_List.Glist;

      Qs_Menu       : Gtk.Combo.Gtk_Combo;
      Clist_Qs_Menu : String_List.Glist;

      Affected_Role_List        : Gtk.Clist.Gtk_Clist;
      Scroll_Affected_Role_List : Gtk_Scrolled_Window;
      Clist_Location_Menu       : Widget_SList.GSlist;

      Buffer_Role_Task : Gtk_Combo;
      Items            : String_List.Glist;

      Affect_Size     : Gtk_Entry;
      Affect_Task     : Gtk_Combo;
      Affect_Roletype : Gtk_Combo;
      Affect_Roletime : Gtk.GEntry.Gtk_Entry;
      Clist_Role_Menu : Widget_SList.GSlist;
      Clist_Task_Menu : Widget_SList.GSlist;

      Task_Label     : Gtk.Label.Gtk_Label;
      Rolesize_Label : Gtk.Label.Gtk_Label;
      Roletime_Label : Gtk.Label.Gtk_Label;
      Roletype_Label : Gtk.Label.Gtk_Label;

      Frame_Name_Location : Gtk.Frame.Gtk_Frame;
      Frame_Affect        : Gtk.Frame.Gtk_Frame;
      Affect_List_Titles  : Chars_Ptr_Array (1 .. 4);

      Table_Name_Location : Gtk.Table.Gtk_Table;
      Table_Add_Affect    : Gtk.Table.Gtk_Table;

      Name_Label          : Gtk.Label.Gtk_Label;
      Cpu_Label           : Gtk.Label.Gtk_Label;
      Size_Label          : Gtk.Label.Gtk_Label;
      Address_Space_Label : Gtk.Label.Gtk_Label;
      Qs_Label            : Gtk.Label.Gtk_Label;

      Frame_Buffer : Gtk.Frame.Gtk_Frame;
      Buffer_Table : Gtk.Table.Gtk_Table;

      Vbox_Main           : Gtk.Box.Gtk_Box;
      Hbox_Buffer         : Gtk.Box.Gtk_Box;
      Hbox_Confirm        : Gtk.Box.Gtk_Box;
      Vbox_List           : Gtk.Box.Gtk_Box;
      Hbox_Modify         : Gtk.Box.Gtk_Box;
      Vbox_Update         : Gtk.Box.Gtk_Box;
      Vbox_Splited_Buffer : Gtk.Box.Gtk_Box;
      Hbutton_Box_Modify  : Gtk_Hbutton_Box;

      Modify_Button      : Gtk_Button;
      Advanced_Button    : Gtk_Button;
      Close_Button       : Gtk_Button;
      Cancel_Button      : Gtk_Button;
      Add_Button         : Gtk_Button;
      Delete_Button      : Gtk_Button;
      Button_Add_Role    : Gtk_Button;
      Button_Delete_Role : Gtk_Button;
      Button_Modify_Role : Gtk_Button;

      Hseparator1 : Gtk_Hseparator;
      Vseparator  : Gtk_Vseparator;
      Vseparator2 : Gtk_Vseparator;

      Child_Page : Gtk_Frame;
      Label_Box  : Gtk_Box;
      Menu_Box   : Gtk_Box;
      Pixmap     : Gtk_Pixmap;
      Vbox       : Gtk_Box;
      Button     : Gtk_Button;

      Undo : System;

      Scroll_List : Gtk_Scrolled_Window;
      List_Titles : Chars_Ptr_Array (1 .. 3);

      Advanced : Boolean := False;
   end record;

end Update_Buffers_Set_Pkg;
