------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with GNAT.Current_Exception;                                    use 
  GNAT.Current_Exception;
with Mmi_Tools;                                                 use Mmi_Tools;
with Project;                                                   use Project;
with Event_Analyzers;                                           use 
  Event_Analyzers;
with read_sc_file_Pkg;                                          use 
  read_sc_file_Pkg;
with Glib;                                                      use Glib;
with Translate;                                                 use Translate;
with Framework_Config;                                          use 
  Framework_Config;
with Unbounded_Strings;                                         use 
  Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;
with Gtkada.Types;                                              use 
  Gtkada.Types;
with Gtk.Widget;                                                use Gtk.Widget;
with Update_Event_Analyzers_Set_Pkg.Callbacks.Integrity_Checks;

package body Update_Event_Analyzers_Set_Pkg.Callbacks is

   procedure Show_Event_Analyzers
     (Update_Event_Analyzers_Set : access Update_Event_Analyzers_Set_Record'
        Class)
   is

      An_Event_Analyzer : Event_Analyzer_Ptr;
      My_Iterator       : Event_Analyzers_Iterator;
      A_Line            : Chars_Ptr_Array (1 .. 2);
      Line              : Gint;

   begin
      Clear (Update_Event_Analyzers_Set.Event_Analyzers_List);

      Reset_Iterator (Sys.Event_Analyzers, My_Iterator);
      if not Is_Empty (Sys.Event_Analyzers) then
         loop

            Current_Element
              (Sys.Event_Analyzers,
               An_Event_Analyzer,
               My_Iterator);

            -- Create the line
            --
            A_Line := To_String (An_Event_Analyzer.Behavior_File_Name) + "";

            -- Display the Line
            --
            Line :=
               Append
                 (Update_Event_Analyzers_Set.Event_Analyzers_List,
                  A_Line);

            exit when Is_Last_Element (Sys.Event_Analyzers, My_Iterator);

            Next_Element (Sys.Event_Analyzers, My_Iterator);

         end loop;
      end if;
   end Show_Event_Analyzers;

   procedure On_Click_List (List : access Gtk_Clist_Record'Class) is

      Name : Unbounded_String;
      Row  : Gint;

   begin

      Row  := Get_Focus_Row (Update_Event_Analyzers_Set.Event_Analyzers_List);
      Name :=
         To_Unbounded_String
           (Get_Text
               (Update_Event_Analyzers_Set.Event_Analyzers_List,
                Row,
                0));

      Update_Event_Analyzers_Set.Current_Event_Analyzer := Name;

      Gtk.GEntry.Set_Text
        (Update_Event_Analyzers_Set.Entry_File_Name,
         To_String (Name));

   end On_Click_List;

   procedure update_file_name (File_Name : in Unbounded_String) is
   begin
      Set_Text
        (Update_Event_Analyzers_Set.Entry_File_Name,
         To_String (File_Name));
   end update_file_name;

   procedure On_Read_From_File_Pressed
     (Object : access Gtk_Button_Record'Class)
   is

   begin
      Gtk_New (read_sc_file, update_file_name'Access);
      Show_All (read_sc_file);
   end On_Read_From_File_Pressed;

   procedure On_Close_Pressed (Object : access Gtk_Button_Record'Class) is
   begin

      Is_New   := False;
      Is_Saved := False;

      Destroy (Update_Event_Analyzers_Set);
   end On_Close_Pressed;

   procedure On_Delete_Pressed (Object : access Gtk_Button_Record'Class) is

      My_Iterator : Event_Analyzer_Set.Generic_Event_Analyzer_Set.Iterator;

      An_Event_Analyzer : Event_Analyzer_Ptr;

   begin

      if Update_Event_Analyzers_Set.Current_Event_Analyzer = "" then
         Show_Message_Box (Lb_Select_Event_Analyzer_First (Current_Language));
         return;
      end if;

      if Is_Empty (Sys.Event_Analyzers) then
         Show_Message_Box (Lb_Can_Not_Modify (Current_Language));
         return;
      end if;

      Reset_Iterator (Sys.Event_Analyzers, My_Iterator);
      if not Is_Empty (Sys.Event_Analyzers) then
         loop

            Current_Element
              (Sys.Event_Analyzers,
               An_Event_Analyzer,
               My_Iterator);
            if An_Event_Analyzer.Behavior_File_Name =
               Update_Event_Analyzers_Set.Current_Event_Analyzer
            then

               Delete (Sys.Event_Analyzers, An_Event_Analyzer);

               -- Clear entry
               --
               Gtk.GEntry.Set_Text
                 (Update_Event_Analyzers_Set.Entry_File_Name,
                  "");
               Update_Event_Analyzers_Set.Current_Event_Analyzer :=
                 Empty_String;

               exit;
            end if;

            exit when Is_Last_Element (Sys.Event_Analyzers, My_Iterator);
            Next_Element (Sys.Event_Analyzers, My_Iterator);

         end loop;

      end if;

      Show_Event_Analyzers (Update_Event_Analyzers_Set);

   end On_Delete_Pressed;

   procedure On_Cancel_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      Duplicate (Update_Event_Analyzers_Set.Undo, Sys);
      Destroy (Update_Event_Analyzers_Set);
   end On_Cancel_Pressed;

   procedure On_Add_Pressed (Object : access Gtk_Button_Record'Class) is

      Ok            : Boolean := True;
      Error_Message : Unbounded_String;

   begin

      Integrity_Checks.Check_Integrity_Constraints
        (Update_Event_Analyzers_Set,
         Error_Message,
         Ok);
      if not Ok then
         Show_Message_Box (Error_Message);
         return;
      end if;

      Add_Event_Analyzer
        (Sys.Event_Analyzers,
         Integrity_Checks.Event_Analyzer_Name);

      Show_Event_Analyzers (Update_Event_Analyzers_Set);

   exception
      when Event_Analyzer_Set.Invalid_Parameter =>
         begin
            Show_Message_Box (Exception_Message);
            return;
         end;
      when others =>
         begin
            Show_Message_Box (Exception_Name & Exception_Message);
            return;
         end;

   end On_Add_Pressed;

   procedure On_Advanced_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      if Visible_Is_Set (Update_Event_Analyzers_Set.Vbox_List) then
         Show_Advanced (Update_Event_Analyzers_Set, False);
      else
         Show_Advanced (Update_Event_Analyzers_Set, True);
      end if;

   end On_Advanced_Pressed;

   procedure On_Modify_Pressed (Object : access Gtk_Button_Record'Class) is

      An_Event_Analyzer : Event_Analyzer_Ptr;
      My_Iterator       :
        Event_Analyzer_Set.Generic_Event_Analyzer_Set.Iterator;

      Ok            : Boolean := True;
      Error_Message : Unbounded_String;

   begin

      if Update_Event_Analyzers_Set.Current_Event_Analyzer = "" then
         Show_Message_Box (Lb_Select_Event_Analyzer_First (Current_Language));
         return;
      end if;

      if Is_Empty (Sys.Event_Analyzers) then
         Show_Message_Box (Lb_Can_Not_Modify (Current_Language));
         return;
      end if;

      Integrity_Checks.Check_Integrity_Constraints
        (Update_Event_Analyzers_Set,
         Error_Message,
         Ok);
      if not Ok then
         Show_Message_Box (Error_Message);
         return;
      end if;

      --
      -- Update set
      --

      Reset_Iterator (Sys.Event_Analyzers, My_Iterator);
      loop

         Current_Element
           (Sys.Event_Analyzers,
            An_Event_Analyzer,
            My_Iterator);
         if An_Event_Analyzer.Behavior_File_Name =
            Update_Event_Analyzers_Set.Current_Event_Analyzer
         then

            Delete (Sys.Event_Analyzers, An_Event_Analyzer);

            Add_Event_Analyzer
              (Sys.Event_Analyzers,
               Integrity_Checks.Event_Analyzer_Name);

            Update_Event_Analyzers_Set.Current_Event_Analyzer :=
              Integrity_Checks.Event_Analyzer_Name;
            exit;
         end if;

         exit when Is_Last_Element (Sys.Event_Analyzers, My_Iterator);
         Next_Element (Sys.Event_Analyzers, My_Iterator);

      end loop;

      Show_Event_Analyzers (Update_Event_Analyzers_Set);

   exception
      when Event_Analyzer_Set.Invalid_Parameter =>
         begin
            Show_Message_Box (Exception_Message);
            return;
         end;
      when others =>
         begin
            Show_Message_Box (Exception_Name & Exception_Message);
            return;
         end;

   end On_Modify_Pressed;

end Update_Event_Analyzers_Set_Pkg.Callbacks;
