------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Glib;                    use Glib;
with Mmi_Tools;               use Mmi_Tools;
with Objects;                 use Objects;
with Objects.extended;        use Objects.extended;
with Translate;               use Translate;
with Framework_Config;        use Framework_Config;
with Parameters;              use Parameters;
use Parameters.User_Defined_Parameters_Table_Package;
with Unbounded_Strings;       use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;
with Project;                 use Project;
with Gtk.Radio_Menu_Item;     use Gtk.Radio_Menu_Item;
with Graph_Window_Pkg;        use Graph_Window_Pkg;
with Graph_Item_Pkg;          use Graph_Item_Pkg;
with Graph_Add_Pkg;           use Graph_Add_Pkg;
with My_Canvas;               use My_Canvas;
with Double_Util;             use Double_Util;
with Message_Set;             use Message_Set;
use Message_Set.Generic_Message_Set;
with Update_Messages_Set_Pkg; use Update_Messages_Set_Pkg;
with Task_Dependencies;       use Task_Dependencies;

package body Update_Messages_Set_Pkg.Callbacks is

   procedure Page_Switch
     (Notebook : access Gtk_Notebook_Record'Class;
      Page     : in Gtk.Gtk_Notebook_Page;
      Page_Num : in Gtk_Notebook)
   is
      pragma Warnings (Off, Page_Num);
      Old_Page : constant Gtk.Gtk_Notebook_Page := Get_Cur_Page (Notebook);
      Pixmap   : Gtk_Pixmap;
   begin
      Pixmap := Gtk_Pixmap (Get_Child (Gtk_Box (Get_Tab_Label (Page)), 0));
      Gtk.Pixmap.Set
        (Pixmap,
         Update_Messages_Set.Book_Opened,
         Update_Messages_Set.Book_Opened_Mask);
      Pixmap :=
        Gtk_Pixmap (Get_Child (Gtk_Box (Get_Menu_Label (Page)), 0));
      Gtk.Pixmap.Set
        (Pixmap,
         Update_Messages_Set.Book_Opened,
         Update_Messages_Set.Book_Opened_Mask);

      if Old_Page /= null then
         Pixmap :=
           Gtk_Pixmap (Get_Child (Gtk_Box (Get_Tab_Label (Old_Page)), 0));
         Gtk.Pixmap.Set
           (Pixmap,
            Update_Messages_Set.Book_Closed,
            Update_Messages_Set.Book_Closed_Mask);
         Pixmap :=
           Gtk_Pixmap (Get_Child (Gtk_Box (Get_Menu_Label (Old_Page)), 0));
         Gtk.Pixmap.Set
           (Pixmap,
            Update_Messages_Set.Book_Closed,
            Update_Messages_Set.Book_Closed_Mask);
      end if;
   end Page_Switch;

   procedure Show_Messages (A_Message_Name : Unbounded_String) is
      A_Message          : Message_Ptr;
      My_Iterator        : Messages_Iterator;
      Line               : Gint;
      Message_Share      : Chars_Ptr_Array (1 .. 7);
      Jitter             : Unbounded_String;
      Deadline           : Unbounded_String;
      Period             : Unbounded_String;
      Size               : Unbounded_String;
      Response_Time      : Unbounded_String;
      Communication_Time : Unbounded_String;

   begin
      Clear (Update_Messages_Set.Messages_List);

      Reset_Iterator (Sys.Messages, My_Iterator);
      if not Is_Empty (Sys.Messages) then
         loop

            Current_Element (Sys.Messages, A_Message, My_Iterator);

            Deadline           :=
               To_Unbounded_String (Natural'Image (A_Message.Deadline));
            Size               :=
               To_Unbounded_String (Natural'Image (A_Message.Size));
            Response_Time      :=
               To_Unbounded_String (Natural'Image (A_Message.Response_Time));
            Communication_Time :=
               To_Unbounded_String
                 (Natural'Image (A_Message.Communication_Time));

            case A_Message.Message_Type is
               when Aperiodic_Type =>
                  Jitter := To_Unbounded_String (" 0");
                  Period := To_Unbounded_String (" 0");
               when Periodic_Type =>
                  Jitter :=
                     To_Unbounded_String
                       (Natural'Image
                           (Periodic_Message_Ptr (A_Message).Jitter));
                  Period :=
                     To_Unbounded_String
                       (Natural'Image
                           (Periodic_Message_Ptr (A_Message).Period));
               when Generic_Type =>
                  Jitter := To_Unbounded_String (" 0");
                  Period := To_Unbounded_String (" 0");
            end case;

            -- Create the line
            --
            Message_Share :=
              To_String (A_Message.Name) +
              To_String (Jitter) +
              To_String (Deadline) +
              To_String (Period) +
              To_String (Size) +
              To_String (Response_Time) +
              To_String (Communication_Time);

            -- Display the Line
            --
            Line := Append (Update_Messages_Set.Messages_List, Message_Share);

            exit when Is_Last_Element (Sys.Messages, My_Iterator);

            Next_Element (Sys.Messages, My_Iterator);

         end loop;
      end if;
   end Show_Messages;

   procedure On_Click_List (List : access Gtk_Clist_Record'Class) is

      Name               : Unbounded_String;
      Jitter             : Unbounded_String;
      Deadline           : Unbounded_String;
      Period             : Unbounded_String;
      Size               : Unbounded_String;
      Response_Time      : Unbounded_String;
      Communication_Time : Unbounded_String;
      Row                : Gint;

      A_Message   : Message_Ptr;
      Current     : Gint;
      Message_Str : Unbounded_String;

      Parameter_Items :
        array (User_Defined_Parameters_Range) of Strings_Table;

   begin

      Row  := Get_Focus_Row (Update_Messages_Set.Messages_List);
      Name :=
         To_Unbounded_String
           (Get_Text (Update_Messages_Set.Messages_List, Row, 0));

      Update_Messages_Set.Current_Message := Name;

      Jitter             :=
         To_Unbounded_String
           (Get_Text (Update_Messages_Set.Messages_List, Row, 1));
      Deadline           :=
         To_Unbounded_String
           (Get_Text (Update_Messages_Set.Messages_List, Row, 2));
      Period             :=
         To_Unbounded_String
           (Get_Text (Update_Messages_Set.Messages_List, Row, 3));
      Size               :=
         To_Unbounded_String
           (Get_Text (Update_Messages_Set.Messages_List, Row, 4));
      Response_Time      :=
         To_Unbounded_String
           (Get_Text (Update_Messages_Set.Messages_List, Row, 5));
      Communication_Time :=
         To_Unbounded_String
           (Get_Text (Update_Messages_Set.Messages_List, Row, 6));

      Gtk.GEntry.Set_Text (Update_Messages_Set.Name, To_String (Name));
      Gtk.GEntry.Set_Text (Update_Messages_Set.Jitter, To_String (Jitter));
      Gtk.GEntry.Set_Text
        (Update_Messages_Set.Deadline,
         To_String (Deadline));
      Gtk.GEntry.Set_Text (Update_Messages_Set.Period, To_String (Period));
      Gtk.GEntry.Set_Text (Update_Messages_Set.Size, To_String (Size));
      Gtk.GEntry.Set_Text
        (Update_Messages_Set.Response_Time,
         To_String (Response_Time));
      Gtk.GEntry.Set_Text
        (Update_Messages_Set.Communication_Time,
         To_String (Communication_Time));

      Message_Str := Update_Messages_Set.Current_Message;
      if (Message_Str /= To_Unbounded_String ("")) then
         A_Message                               :=
            Search_Message (Sys.Messages, Message_Str);
         Update_Messages_Set.Current_Message_Ptr := A_Message;

         Show_All (Update_Messages_Set);

         -- User defined parameters
         --

         for I in  0 .. User_Defined_Parameters_Range'Last - 1 loop
            Set_Text (Update_Messages_Set.Param (I).Entry_Name, "");
            Set_Text (Update_Messages_Set.Param (I).Entry_Value, "");
         end loop;

         for I in  A_Message.Param.Nb_Entries .. User_Defined_Parameters_Range'
              Last -
                                                 1
         loop
            Initialize (Parameter_Items (I));
            Add (Parameter_Items (I), To_Unbounded_String ("Integer"));
            Add (Parameter_Items (I), To_Unbounded_String ("Double"));
            Add (Parameter_Items (I), To_Unbounded_String ("String"));
            Add (Parameter_Items (I), To_Unbounded_String ("Boolean"));

            Destroy (Update_Messages_Set.Param (I).Parameter_Menu);
            Update_Messages_Set.Param (I).Clist_Parameter_Menu :=
              Widget_SList.Null_List;
            Gtk.Option_Menu.Gtk_New
              (Update_Messages_Set.Param (I).Parameter_Menu);

            Build_Option_Menu
              (Update_Messages_Set.Param (I).Parameter_Menu,
               Update_Messages_Set.Param (I).Clist_Parameter_Menu,
               Parameter_Items (I),
               -1,
               null);
            Show_All (Update_Messages_Set.Param (I).Parameter_Menu);
            Attach
              (Update_Messages_Set.Parameter_Table,
               Update_Messages_Set.Param (I).Parameter_Menu,
               1,
               2,
               1 + Guint (I),
               2 + Guint (I),
               Expand or Fill,
               0,
               0,
               0);
         end loop;
         Show_All (Update_Messages_Set);

         for I in  0 .. A_Message.Param.Nb_Entries - 1 loop
            Set_Text
              (Update_Messages_Set.Param (I).Entry_Name,
               To_String (A_Message.Param.Entries (I).Name));

            if A_Message.Param.Entries (I).ptype = Boolean_Parameter then
               Set_Text
                 (Update_Messages_Set.Param (I).Entry_Value,
                  A_Message.Param.Entries (I).Boolean_value'Img);
               Current := 3;
            else
               if A_Message.Param.Entries (I).ptype = Integer_Parameter then
                  Set_Text
                    (Update_Messages_Set.Param (I).Entry_Value,
                     A_Message.Param.Entries (I).Integer_value'Img);
                  Current := 0;
               else
                  if A_Message.Param.Entries (I).ptype =
                     Double_Parameter
                  then
                     Set_Text
                       (Update_Messages_Set.Param (I).Entry_Value,
                        To_String
                           (Format
                               (A_Message.Param.Entries (I).double_value)));
                     Current := 1;
                  else
                     Set_Text
                       (Update_Messages_Set.Param (I).Entry_Value,
                        To_String (A_Message.Param.Entries (I).string_value));
                     Current := 2;
                  end if;
               end if;
            end if;

            Initialize (Parameter_Items (I));
            Add (Parameter_Items (I), To_Unbounded_String ("Integer"));
            Add (Parameter_Items (I), To_Unbounded_String ("Double"));
            Add (Parameter_Items (I), To_Unbounded_String ("String"));
            Add (Parameter_Items (I), To_Unbounded_String ("Boolean"));

            Destroy (Update_Messages_Set.Param (I).Parameter_Menu);
            Update_Messages_Set.Param (I).Clist_Parameter_Menu :=
              Widget_SList.Null_List;
            Gtk.Option_Menu.Gtk_New
              (Update_Messages_Set.Param (I).Parameter_Menu);

            Build_Option_Menu
              (Update_Messages_Set.Param (I).Parameter_Menu,
               Update_Messages_Set.Param (I).Clist_Parameter_Menu,
               Parameter_Items (I),
               Current,
               null);
            Show_All (Update_Messages_Set.Param (I).Parameter_Menu);
            Attach
              (Update_Messages_Set.Parameter_Table,
               Update_Messages_Set.Param (I).Parameter_Menu,
               1,
               2,
               1 + Guint (I),
               2 + Guint (I),
               Expand or Fill,
               0,
               0,
               0);

         end loop;
         Show_All (Update_Messages_Set);

      end if;

   end On_Click_List;

   procedure On_Close_Pressed (Object : access Gtk_Button_Record'Class) is
   begin

      Is_New   := False;
      Is_Saved := False;

      Rebuild_Canvas (Graph_Window);
      Destroy (Update_Messages_Set);
   end On_Close_Pressed;

   procedure On_Add_Pressed (Object : access Gtk_Button_Record'Class) is

      Ok           : Boolean;
      Message_Name : Unbounded_String;

      Message_Jitter     : Integer;
      Message_Deadline   : Integer;
      Message_Period     : Integer;
      Message_Size       : Integer;
      Response_Time      : Integer;
      Communication_Time : Integer;

      Message_Size_String       : Unbounded_String;
      Message_Jitter_String     : Unbounded_String;
      Message_Deadline_String   : Unbounded_String;
      Message_Period_String     : Unbounded_String;
      Response_Time_String      : Unbounded_String;
      Communication_Time_String : Unbounded_String;

      A_Message : Message_Ptr;

      Message_Param : User_Defined_Parameters_Table;

      My_Iterator : Message_Set.Generic_Message_Set.Iterator;

      Current      : Unbounded_String;
      A_Param      : Parameter_ptr;
      A_Param_Type : Gint;

   begin

      --
      -- First Message page : general information
      --

      Message_Name :=
         To_Unbounded_String (Get_Text (Update_Messages_Set.Name));

      if Message_Name = "" then
         Show_Message_Box
           (Lb_Message_Name (Current_Language) &
            Lb_Mandatory (Current_Language));
         return;
      end if;

      if not Is_A_Valid_Identifier (Message_Name) then
         Show_Message_Box
           (Lb_Message_Name (Current_Language) &
            Lb_Colon &
            Lb_Invalid_Identifier (Current_Language));
         return;
      end if;

      Reset_Iterator (Sys.Messages, My_Iterator);
      if not Is_Empty (Sys.Messages) then
         loop
            Current_Element (Sys.Messages, A_Message, My_Iterator);
            if (Message_Name = A_Message.Name) then
               if (A_Message.Name /=
                   Update_Messages_Set.Current_Message)
               then
                  Show_Message_Box
                    (Lb_Message_Name (Current_Language) &
                     Lb_Already_Defined (Current_Language));
                  return;
               end if;
            end if;
            exit when Is_Last_Element (Sys.Messages, My_Iterator);
            Next_Element (Sys.Messages, My_Iterator);
         end loop;
      end if;

      Message_Jitter_String     :=
         To_Unbounded_String (Get_Text (Update_Messages_Set.Jitter));
      Message_Deadline_String   :=
         To_Unbounded_String (Get_Text (Update_Messages_Set.Deadline));
      Message_Period_String     :=
         To_Unbounded_String (Get_Text (Update_Messages_Set.Period));
      Message_Size_String       :=
         To_Unbounded_String (Get_Text (Update_Messages_Set.Size));
      Response_Time_String      :=
         To_Unbounded_String (Get_Text (Update_Messages_Set.Response_Time));
      Communication_Time_String :=
         To_Unbounded_String
           (Get_Text (Update_Messages_Set.Communication_Time));

      To_Integer (Response_Time_String, Response_Time, Ok);
      if not Ok then
         if Response_Time_String /= "" then
            Show_Message_Box
              (Lb_Priority (Current_Language) &
               Lb_Must_Be_Numeric (Current_Language));
            return;
         end if;
      end if;

      To_Integer (Communication_Time_String, Communication_Time, Ok);
      if not Ok then
         if Communication_Time_String /= "" then
            Show_Message_Box
              (Lb_Priority (Current_Language) &
               Lb_Must_Be_Numeric (Current_Language));
            return;
         end if;
      end if;

      To_Integer (Message_Jitter_String, Message_Jitter, Ok);
      if not Ok then
         if Message_Jitter_String /= "" then
            Show_Message_Box
              (Lb_Priority (Current_Language) &
               Lb_Must_Be_Numeric (Current_Language));
            return;
         end if;
      end if;

      To_Integer (Message_Deadline_String, Message_Deadline, Ok);
      if not Ok then
         if Message_Deadline_String /= "" then
            Show_Message_Box
              (Lb_Priority (Current_Language) &
               Lb_Must_Be_Numeric (Current_Language));
            return;
         end if;
      end if;

      To_Integer (Message_Size_String, Message_Size, Ok);
      if not Ok then
         if Message_Size_String /= "" then
            Show_Message_Box
              (Lb_Priority (Current_Language) &
               Lb_Must_Be_Numeric (Current_Language));
            return;
         end if;
      end if;

      To_Integer (Message_Period_String, Message_Period, Ok);
      if not Ok then
         if Message_Period_String /= "" then
            Show_Message_Box
              (Lb_Priority (Current_Language) &
               Lb_Must_Be_Numeric (Current_Language));
            return;
         end if;
      end if;

      if (Message_Size <= 0) then
         Show_Message_Box
           (Lb_Size (Current_Language) &
            Lb_Must_Be (Current_Language) &
            ">0");
         return;
      end if;

      if (Response_Time < 0) then
         Show_Message_Box
           (Lb_Response_Time_Message (Current_Language) &
            Lb_Must_Be (Current_Language) &
            "=>0");
         return;
      end if;

      if (Communication_Time < 0) then
         Show_Message_Box
           (Lb_Communication_Time_Message (Current_Language) &
            Lb_Must_Be (Current_Language) &
            "=>0");
         return;
      end if;

      if (Message_Jitter < 0) then
         Show_Message_Box
           (Lb_Jitter (Current_Language) &
            Lb_Must_Be (Current_Language) &
            "=>0");
         return;
      end if;

      if (Message_Period < 0) then
         Show_Message_Box
           (Lb_Period (Current_Language) &
            Lb_Must_Be (Current_Language) &
            "=>0");
         return;
      end if;

      if (Message_Deadline < 0) then
         Show_Message_Box
           (Lb_Deadline (Current_Language) &
            Lb_Must_Be (Current_Language) &
            ">0");
         return;
      end if;

      if (Message_Period /= 0 and Message_Jitter > Message_Deadline) then
         Show_Message_Box
           (Lb_Deadline (Current_Language) &
            Lb_Must_Be (Current_Language) &
            Lb_Superior (Current_Language) &
            Lb_Jitter (Current_Language));
         return;
      end if;

      if (Message_Period /= 0 and Message_Jitter = 0) then
         Show_Message_Box
           (Lb_Jitter (Current_Language) & Lb_Mandatory (Current_Language));
         return;
      end if;

      --
      --- Second page of the widget : user defined parameters
      --

      Initialize (Message_Param);

      for I in  0 .. User_Defined_Parameters_Range'Last - 1 loop

         Current :=
            To_Unbounded_String
              (Get_Text (Update_Messages_Set.Param (I).Entry_Name));

         if Current /= Empty_String then

            A_Param_Type :=
              Gint (Selected_Button
                       (Update_Messages_Set.Param (I).Clist_Parameter_Menu));

            if A_Param_Type = 3 then
               A_Param := new Parameter (Integer_Parameter);
            else
               if A_Param_Type = 2 then
                  A_Param := new Parameter (Double_Parameter);
               else
                  if A_Param_Type = 1 then
                     A_Param := new Parameter (String_Parameter);
                  else
                     A_Param := new Parameter (Boolean_Parameter);
                  end if;
               end if;
            end if;

            A_Param.Name := Current;

            Current :=
               To_Unbounded_String
                 (Get_Text (Update_Messages_Set.Param (I).Entry_Value));

            -- Integer case
            --
            if A_Param_Type = 3 then
               if (Current = "") then
                  A_Param.Integer_value := 0;
               else
                  To_Integer (Current, A_Param.Integer_value, Ok);
                  if not Ok then
                     Show_Message_Box
                       (To_Unbounded_String
                           ("Parameter " &
                            Integer'Image (Integer (I + 1)) &
                            " ") &
                        Lb_Must_Be (Current_Language) &
                        To_Unbounded_String (" integer"));
                     return;
                  end if;
               end if;

            -- Double case
            --
            else
               if A_Param_Type = 2 then
                  if (Current = "") then
                     A_Param.double_value := 0.0;
                  else
                     To_Double (Current, A_Param.double_value, Ok);
                     if not Ok then
                        Show_Message_Box
                          (To_Unbounded_String
                              ("Parameter " &
                               Integer'Image (Integer (I + 1)) &
                               " ") &
                           Lb_Must_Be (Current_Language) &
                           To_Unbounded_String (" double"));
                        return;
                     end if;
                  end if;

               -- String case
               --
               else
                  if A_Param_Type = 1 then
                     A_Param.string_value := Current;

                  -- Boolean case
                  --
                  else
                     if (Current = "") then
                        A_Param.Boolean_value := True;
                     else
                        To_Boolean (Current, A_Param.Boolean_value, Ok);
                        if not Ok then
                           Show_Message_Box
                             (To_Unbounded_String
                                 ("Parameter " &
                                  Integer'Image (Integer (I + 1)) &
                                  " ") &
                              Lb_Must_Be (Current_Language) &
                              To_Unbounded_String (" boolean"));
                           return;
                        end if;
                     end if;

                  end if;
               end if;
            end if;

            Add (Message_Param, A_Param);
         end if;

      end loop;

      --
      -- Update Message set
      --

      Add_Message
        (Sys.Messages,
         Message_Name,
         Natural (Message_Size),
         Natural (Message_Period),
         Natural (Message_Deadline),
         Natural (Message_Jitter),
         Message_Param,
         Natural (Response_Time),
         Natural (Communication_Time));

      Update_Messages_Set.Current_Message := Message_Name;

      Reset_Iterator (Sys.Messages, My_Iterator);
      Current_Element (Sys.Messages, A_Message, My_Iterator);
      Show_Messages (A_Message.Name);

      if Add_New_Item /= null then
         Set_Name (Items_List (Last_Item - 1), Message_Name);
         Refresh_Canvas (Graph_Window.Buffer);
      end if;

   exception
      when Message_Already_Defined =>
         begin
            Show_Message_Box
              (Lb_Message_Name (Current_Language) &
               Lb_Already_Defined (Current_Language));
            return;
         end;
      when Message_Set.Generic_Message_Set.Full_Set =>
         begin
            Show_Message_Box
              (Lb_Can_Not_Define_More_Messages (Current_Language));
            return;
         end;

   end On_Add_Pressed;

   procedure On_Delete_Pressed (Object : access Gtk_Button_Record'Class) is

      My_Iterator : Message_Set.Generic_Message_Set.Iterator;
      A_Message   : Message_Ptr;

   begin

      Reset_Iterator (Sys.Messages, My_Iterator);
      if not Is_Empty (Sys.Messages) then
         loop

            Current_Element (Sys.Messages, A_Message, My_Iterator);
            if A_Message.Name = Update_Messages_Set.Current_Message then

               Delete_Message (Sys, A_Message);
               Rebuild_Canvas (Graph_Window);

               -- Clear main entrys
               --
               Update_Messages_Set.Current_Message := To_Unbounded_String
                                                        ("");
               Gtk.GEntry.Set_Text (Update_Messages_Set.Name, "");
               Gtk.GEntry.Set_Text (Update_Messages_Set.Jitter, "");
               Gtk.GEntry.Set_Text (Update_Messages_Set.Deadline, "");
               Gtk.GEntry.Set_Text (Update_Messages_Set.Period, "");
               Gtk.GEntry.Set_Text (Update_Messages_Set.Size, "");
               Gtk.GEntry.Set_Text (Update_Messages_Set.Response_Time, "");
               Gtk.GEntry.Set_Text
                 (Update_Messages_Set.Communication_Time,
                  "");

               -- Clear User defined parameters
               --
               for I in  0 .. User_Defined_Parameters_Range'Last - 1 loop
                  Gtk.GEntry.Set_Text
                    (Update_Messages_Set.Param (I).Entry_Name,
                     "");
                  Gtk.GEntry.Set_Text
                    (Update_Messages_Set.Param (I).Entry_Value,
                     "");
               end loop;

               exit;
            end if;

            exit when Is_Last_Element (Sys.Messages, My_Iterator);
            Next_Element (Sys.Messages, My_Iterator);

         end loop;
         Show_Messages (A_Message.Name);
      end if;

   end On_Delete_Pressed;

   procedure On_Cancel_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      Duplicate (Update_Messages_Set.Undo, Sys);
      Rebuild_Canvas (Graph_Window);
      Destroy (Update_Messages_Set);
   end On_Cancel_Pressed;

   procedure On_Advanced_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      if Visible_Is_Set (Update_Messages_Set.Vbox_List) then
         Show_Advanced (Update_Messages_Set, False);
         null;
      else
         Show_Advanced (Update_Messages_Set, True);
         null;
      end if;

   end On_Advanced_Pressed;

   procedure On_Modify_Pressed (Object : access Gtk_Button_Record'Class) is

      Ok           : Boolean;
      Message_Name : Unbounded_String;

      Message_Jitter     : Integer;
      Message_Deadline   : Integer;
      Message_Period     : Integer;
      Message_Size       : Integer;
      Response_Time      : Integer;
      Communication_Time : Integer;

      Message_Param : User_Defined_Parameters_Table;

      Message_Jitter_String     : Unbounded_String;
      Message_Deadline_String   : Unbounded_String;
      Message_Period_String     : Unbounded_String;
      Message_Size_String       : Unbounded_String;
      Response_Time_String      : Unbounded_String;
      Communication_Time_String : Unbounded_String;

      A_Message       : Message_Ptr;
      Created_Message : Message_Ptr;

      My_Iterator : Message_Set.Generic_Message_Set.Iterator;

      Current : Unbounded_String;

      A_Param      : Parameter_ptr;
      A_Param_Type : Gint;

   begin

      -- First Message page : general information
      --

      Message_Name :=
         To_Unbounded_String (Get_Text (Update_Messages_Set.Name));
      if Update_Messages_Set.Current_Message = "" then
         Show_Message_Box (Lb_Select_Task_First (Current_Language));
         return;
      end if;

      Reset_Iterator (Sys.Messages, My_Iterator);
      if not Is_Empty (Sys.Messages) then
         loop
            Current_Element (Sys.Messages, A_Message, My_Iterator);
            if (Message_Name = A_Message.Name) then
               if (A_Message.Name /=
                   Update_Messages_Set.Current_Message)
               then
                  Show_Message_Box
                    (Lb_Message_Name (Current_Language) &
                     Lb_Already_Defined (Current_Language));
                  return;
               end if;
            end if;
            exit when Is_Last_Element (Sys.Messages, My_Iterator);
            Next_Element (Sys.Messages, My_Iterator);
         end loop;
      else
         Show_Message_Box (Lb_Can_Not_Modify (Current_Language));
         return;
      end if;

      Message_Jitter_String     :=
         To_Unbounded_String (Get_Text (Update_Messages_Set.Jitter));
      Message_Deadline_String   :=
         To_Unbounded_String (Get_Text (Update_Messages_Set.Deadline));
      Message_Period_String     :=
         To_Unbounded_String (Get_Text (Update_Messages_Set.Period));
      Message_Size_String       :=
         To_Unbounded_String (Get_Text (Update_Messages_Set.Size));
      Response_Time_String      :=
         To_Unbounded_String (Get_Text (Update_Messages_Set.Response_Time));
      Communication_Time_String :=
         To_Unbounded_String
           (Get_Text (Update_Messages_Set.Communication_Time));

      To_Integer (Response_Time_String, Response_Time, Ok);
      if not Ok then
         if Response_Time_String /= "" then
            Show_Message_Box
              (Lb_Response_Time_Message (Current_Language) &
               Lb_Must_Be_Numeric (Current_Language));
            return;
         end if;
      end if;

      To_Integer (Communication_Time_String, Communication_Time, Ok);
      if not Ok then
         if Communication_Time_String /= "" then
            Show_Message_Box
              (Lb_Communication_Time_Message (Current_Language) &
               Lb_Must_Be_Numeric (Current_Language));
            return;
         end if;
      end if;

      To_Integer (Message_Jitter_String, Message_Jitter, Ok);
      if not Ok then
         if Message_Jitter_String /= "" then
            Show_Message_Box
              (Lb_Jitter (Current_Language) &
               Lb_Must_Be_Numeric (Current_Language));
            return;
         end if;
      end if;

      To_Integer (Message_Deadline_String, Message_Deadline, Ok);
      if not Ok then
         if Message_Deadline_String /= "" then
            Show_Message_Box
              (Lb_Deadline (Current_Language) &
               Lb_Must_Be_Numeric (Current_Language));
            return;
         end if;
      end if;

      To_Integer (Message_Period_String, Message_Period, Ok);
      if not Ok then
         if Message_Period_String /= "" then
            Show_Message_Box
              (Lb_Period (Current_Language) &
               Lb_Must_Be_Numeric (Current_Language));
            return;
         end if;
      end if;

      To_Integer (Message_Size_String, Message_Size, Ok);
      if not Ok then
         if Message_Size_String /= "" then
            Show_Message_Box
              (Lb_Period (Current_Language) &
               Lb_Must_Be_Numeric (Current_Language));
            return;
         end if;
      end if;

      if (Message_Size < 0) then
         Show_Message_Box
           (Lb_Size (Current_Language) &
            Lb_Must_Be (Current_Language) &
            ">0");
         return;
      end if;

      if (Response_Time < 0) then
         Show_Message_Box
           (Lb_Response_Time_Message (Current_Language) &
            Lb_Must_Be (Current_Language) &
            "=>0");
         return;
      end if;

      if (Communication_Time < 0) then
         Show_Message_Box
           (Lb_Communication_Time_Message (Current_Language) &
            Lb_Must_Be (Current_Language) &
            "=>0");
         return;
      end if;

      if (Message_Jitter < 0) then
         Show_Message_Box
           (Lb_Jitter (Current_Language) &
            Lb_Must_Be (Current_Language) &
            "=>0");
         return;
      end if;

      if (Message_Period < 0) then
         Show_Message_Box
           (Lb_Period (Current_Language) &
            Lb_Must_Be (Current_Language) &
            "=>0");
         return;
      end if;

      if (Message_Deadline < 0) then
         Show_Message_Box
           (Lb_Deadline (Current_Language) &
            Lb_Must_Be (Current_Language) &
            ">0");
         return;
      end if;

      if (Message_Period /= 0 and Message_Jitter > Message_Deadline) then
         Show_Message_Box
           (Lb_Deadline (Current_Language) &
            Lb_Must_Be (Current_Language) &
            Lb_Superior (Current_Language) &
            Lb_Jitter (Current_Language));
         return;
      end if;

      if (Message_Period /= 0 and Message_Jitter = 0) then
         Show_Message_Box
           (Lb_Jitter (Current_Language) & Lb_Mandatory (Current_Language));
         return;
      end if;

      --
      --- Second page of the widget : user defined parameters
      --

      Initialize (Message_Param);

      for I in  0 .. User_Defined_Parameters_Range'Last - 1 loop

         Current :=
            To_Unbounded_String
              (Get_Text (Update_Messages_Set.Param (I).Entry_Name));

         if Current /= Empty_String then

            A_Param_Type :=
              Gint (Selected_Button
                       (Update_Messages_Set.Param (I).Clist_Parameter_Menu));

            if A_Param_Type = 3 then
               A_Param := new Parameter (Integer_Parameter);
            else
               if A_Param_Type = 2 then
                  A_Param := new Parameter (Double_Parameter);
               else
                  if A_Param_Type = 1 then
                     A_Param := new Parameter (String_Parameter);
                  else
                     A_Param := new Parameter (Boolean_Parameter);
                  end if;
               end if;
            end if;

            A_Param.Name := Current;

            Current :=
               To_Unbounded_String
                 (Get_Text (Update_Messages_Set.Param (I).Entry_Value));

            -- Integer case
            --
            if A_Param_Type = 3 then
               if (Current = "") then
                  A_Param.Integer_value := 0;
               else
                  To_Integer (Current, A_Param.Integer_value, Ok);
                  if not Ok then
                     Show_Message_Box
                       (To_Unbounded_String
                           ("Parameter " &
                            Integer'Image (Integer (I + 1)) &
                            " ") &
                        Lb_Must_Be (Current_Language) &
                        To_Unbounded_String (" integer"));
                     return;
                  end if;
               end if;

            -- Double case
            --
            else
               if A_Param_Type = 2 then
                  if (Current = "") then
                     A_Param.double_value := 0.0;
                  else
                     To_Double (Current, A_Param.double_value, Ok);
                     if not Ok then
                        Show_Message_Box
                          (To_Unbounded_String
                              ("Parameter " &
                               Integer'Image (Integer (I + 1)) &
                               " ") &
                           Lb_Must_Be (Current_Language) &
                           To_Unbounded_String (" double"));
                        return;
                     end if;
                  end if;

               -- String case
               --
               else
                  if A_Param_Type = 1 then
                     A_Param.string_value := Current;

                  -- Boolean case
                  --
                  else
                     if (Current = "") then
                        A_Param.Boolean_value := True;
                     else
                        To_Boolean (Current, A_Param.Boolean_value, Ok);
                        if not Ok then
                           Show_Message_Box
                             (To_Unbounded_String
                                 ("Parameter " &
                                  Integer'Image (Integer (I + 1)) &
                                  " ") &
                              Lb_Must_Be (Current_Language) &
                              To_Unbounded_String (" boolean"));
                           return;
                        end if;
                     end if;

                  end if;
               end if;
            end if;

            Add (Message_Param, A_Param);
         end if;

      end loop;

      --
      -- Update Message set
      --

      Reset_Iterator (Sys.Messages, My_Iterator);
      loop

         Current_Element (Sys.Messages, A_Message, My_Iterator);
         if A_Message.Name = Update_Messages_Set.Current_Message then

            Delete (Sys.Messages, A_Message);

            Add_Message
              (Sys.Messages,
               Created_Message,
               Message_Name,
               Natural (Message_Size),
               Natural (Message_Period),
               Natural (Message_Deadline),
               Natural (Message_Jitter),
               Message_Param,
               Natural (Response_Time),
               Natural (Communication_Time));

            Update_All_Task_Dependencies (Sys.Dependencies, Created_Message);

            Update_Messages_Set.Current_Message := Message_Name;
            exit;
         end if;

         exit when Is_Last_Element (Sys.Messages, My_Iterator);
         Next_Element (Sys.Messages, My_Iterator);

      end loop;

      Show_Messages (A_Message.Name);

   end On_Modify_Pressed;

end Update_Messages_Set_Pkg.Callbacks;
