------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------
with Gdk;
with Gdk.Bitmap;            use Gdk.Bitmap;
with Gdk.Pixmap;            use Gdk.Pixmap;
with Gtk.Enums;             use Gtk.Enums;
with Gtk.Frame;             use Gtk.Frame;
with Gtk.Label;             use Gtk.Label;
with Gtk.Notebook;          use Gtk.Notebook;
with Gtk.Pixmap;            use Gtk.Pixmap;
with Gtk.Separator;         use Gtk.Separator;
with Gtk;                   use Gtk;
with Gtk.Window;            use Gtk.Window;
with Gtk.Table;             use Gtk.Table;
with Gtk.GEntry;            use Gtk.GEntry;
with Gtk.Button;            use Gtk.Button;
with Gtk.Box;               use Gtk.Box;
with Gtk.Hbutton_Box;       use Gtk.Hbutton_Box;
with Gtk.Radio_Button;      use Gtk.Radio_Button;
with Gtk.Scrolled_Window;   use Gtk.Scrolled_Window;
with Gtk.Option_Menu;       use Gtk.Option_Menu;
with Gtk.Widget;            use Gtk.Widget;
with Processor_Set;         use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Gtk.Clist;             use Gtk.Clist;
with Gtkada.Types;          use Gtkada.Types;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Tasks_Widget_Pkg;      use Tasks_Widget_Pkg;
with Systems;               use Systems;
with Messages;              use Messages;

package Update_Messages_Set_Pkg is

   type Update_Messages_Set_Record is new Gtk_Window_Record with private;
   type Update_Messages_Set_Access is access all Update_Messages_Set_Record'
     Class;

   procedure Gtk_New
     (Update_Messages_Set : out Update_Messages_Set_Access;
      From                : in Unbounded_String);

   procedure Initialize
     (Update_Messages_Set : access Update_Messages_Set_Record'Class);

   procedure Show_Advanced
     (Update_Messages_Set : in Update_Messages_Set_Access;
      Advanced            : in Boolean);

   Update_Messages_Set : Update_Messages_Set_Access;

private
   type Update_Messages_Set_Record is new Gtk_Window_Record with record
      Parent : Unbounded_String;

      Scroll : Gtk_Scrolled_Window;

      Messages_List           : Gtk.Clist.Gtk_Clist;
      Message_Type_Menu       : Gtk_Option_Menu;
      Clist_Message_Type_Menu : Widget_SList.GSlist;
      Clist_Processor_Menu    : Widget_SList.GSlist;
      Clist_Policy_Menu       : Widget_SList.GSlist;
      Message_Items           : String_List.Glist;
      Current_Message         : Unbounded_String;
      Current_Message_Ptr     : Message_Ptr;

      Name               : Gtk_Entry;
      Size               : Gtk_Entry;
      Period             : Gtk_Entry;
      Deadline           : Gtk_Entry;
      Jitter             : Gtk_Entry;
      Response_Time      : Gtk_Entry;
      Communication_Time : Gtk_Entry;

      Name_Label               : Gtk_Label;
      Size_Label               : Gtk_Label;
      Period_Label             : Gtk_Label;
      Deadline_Label           : Gtk_Label;
      Jitter_Label             : Gtk_Label;
      Response_Time_Label      : Gtk_Label;
      Communication_Time_Label : Gtk_Label;

      Frame_Message   : Gtk.Frame.Gtk_Frame;
      Message_Table   : Gtk.Table.Gtk_Table;
      Offset_Table    : Gtk.Table.Gtk_Table;
      Parameter_Table : Gtk.Table.Gtk_Table;

      Vbox_Main    : Gtk.Box.Gtk_Box;
      Hbox_Message : Gtk.Box.Gtk_Box;
      Hbox_Confirm : Gtk.Box.Gtk_Box;
      Vbox_List    : Gtk.Box.Gtk_Box;
      Hbox_Modify  : Gtk.Box.Gtk_Box;
      Vbox_Update  : Gtk.Box.Gtk_Box;
      Hbox_File    : Gtk.Box.Gtk_Box;
      Hbox_Seed    : Gtk.Box.Gtk_Box;

      Seed_Group         : Widget_SList.GSlist;
      Seed_Unpredictable : Gtk_Radio_Button;
      Seed_Predictable   : Gtk_Radio_Button;

      Hbutton_Box_Modify : Gtk_Hbutton_Box;

      Modify_Button   : Gtk_Button;
      Advanced_Button : Gtk_Button;
      Close_Button    : Gtk_Button;
      Cancel_Button   : Gtk_Button;
      Add_Button      : Gtk_Button;
      Delete_Button   : Gtk_Button;
      Seed_Button     : Gtk_Button;

      Hseparator1 : Gtk_Hseparator;
      Vseparator  : Gtk_Vseparator;
      Vseparator2 : Gtk_Vseparator;

      Child_Page : Gtk_Frame;
      Label_Box  : Gtk_Box;
      Menu_Box   : Gtk_Box;
      Pixmap     : Gtk_Pixmap;
      Vbox       : Gtk_Box;
      Button     : Gtk_Button;
      Lab        : Gtk_Label;

      Offset : Widget_Offset_Table;
      Param  : Widget_Parameter_Table;

      Book_Opened      : Gdk_Pixmap;
      Book_Opened_Mask : Gdk_Bitmap;
      Book_Closed      : Gdk_Pixmap;
      Book_Closed_Mask : Gdk_Bitmap;
      Notebook         : Gtk_Notebook;

      Undo : System;

      Scroll_List : Gtk_Scrolled_Window;
      List_Titles : Chars_Ptr_Array (1 .. 7);

      Sys      : System;
      Advanced : Boolean := False;
   end record;

end Update_Messages_Set_Pkg;
