------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Project;                 use Project;
with Gtk.GEntry;              use Gtk.GEntry;
with Networks;                use Networks;
with Network_Set;             use Network_Set;
use Network_Set.Generic_Network_Set;
with Update_Networks_Set_Pkg; use Update_Networks_Set_Pkg;

package body Update_Networks_Set_Pkg.Callbacks is

   procedure On_Cancel_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      Duplicate (Update_Networks_Set.Undo, Sys);
      Destroy (Update_Networks_Set);
   end On_Cancel_Pressed;

   procedure On_Modify_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      Network_Modify;
      Destroy (Update_Networks_Set);
   end On_Modify_Pressed;

   procedure Network_Modify is
      A_Network      : Network_Ptr;
      A_Network_Type : Type_Of_Network;
      My_Iterator    : Network_Iterator;
   begin

      if Get_Active (Update_Networks_Set.Button_Bounded_Delay) then
         A_Network_Type := Bounded_Delay;
      else
         A_Network_Type := Parametric_Delay;
      end if;

      Reset_Iterator (Sys.Networks, My_Iterator);

      loop

         Current_Element (Sys.Networks, A_Network, My_Iterator);

         if A_Network.Name = Update_Networks_Set.Last_Network_Selected then
            exit;
         end if;
         if Is_Last_Element (Sys.Networks, My_Iterator) then
            raise Network_Not_Found;
         end if;
         Next_Element (Sys.Networks, My_Iterator);
      end loop;

      Update_Network
        (Sys.Networks,
         A_Network.Name,
         To_Unbounded_String (A_Network_Type'Img));

   end Network_Modify;

   procedure On_Change_Network (Widget : access Gtk_Editable_Record'Class) is

      Network_Name : Unbounded_String;
      A_Network    : Network_Ptr;
      My_Iterator  : Network_Iterator;

   begin

      Network_Modify;
      if To_Unbounded_String
            (Get_Text (Get_Entry (Update_Networks_Set.Name))) =
         To_Unbounded_String ("")
      then
         Update_Networks_Set.Last_Network_Selected := To_Unbounded_String
                                                        ("");
         return;
      end if;

      Reset_Iterator (Sys.Networks, My_Iterator);
      loop
         Current_Element (Sys.Networks, A_Network, My_Iterator);
         if A_Network.Name =
            To_Unbounded_String
               (Get_Text (Get_Entry (Update_Networks_Set.Name)))
         then
            exit;
         end if;
         exit when Is_Last_Element (Sys.Networks, My_Iterator);
         Next_Element (Sys.Networks, My_Iterator);
      end loop;

      if A_Network.Network_Type = Bounded_Delay then
         Set_Active (Update_Networks_Set.Button_Bounded_Delay, True);
      elsif A_Network.Network_Type = Parametric_Delay then
         Set_Active (Update_Networks_Set.Button_Parametric_Delay, True);
      end if;

      Network_Name                              :=
         To_Unbounded_String (To_String (A_Network.Name));
      Update_Networks_Set.Last_Network_Selected := Network_Name;

   end On_Change_Network;

end Update_Networks_Set_Pkg.Callbacks;
