------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Project;                                              use Project;
with Task_Set;                                             use Task_Set;
use Task_Set.Generic_Task_Set;
with Scheduler;                                            use Scheduler;
with Scheduler.user_defined.pipeline;                      use 
  Scheduler.user_defined.pipeline;
with read_sc_file_Pkg;                                     use 
  read_sc_file_Pkg;
with Gtk.Radio_Menu_Item;                                  use 
  Gtk.Radio_Menu_Item;
with Unbounded_Strings;                                    use 
  Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;
with Framework_Config;                                     use 
  Framework_Config;
with Translate;                                            use Translate;
with Networks;                                             use Networks;
with Glib;                                                 use Glib;
with Mmi_Tools;                                            use Mmi_Tools;
with Processors;                                           use Processors;
with Buffer_Set;                                           use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Resource_Set;                                         use Resource_Set;
use Resource_Set.Generic_Resource_Set;
with GNAT.Current_Exception;                               use 
  GNAT.Current_Exception;
with Graph_Window_Pkg;                                     use 
  Graph_Window_Pkg;
with Update_Processors_Set_Pkg.Callbacks.Integrity_Checks;

package body Update_Processors_Set_Pkg.Callbacks is

   procedure update_file_name (File_Name : in Unbounded_String) is

   begin
      Set_Text (Update_Processors_Set.Entry_File_Name, To_String (File_Name));
   end update_file_name;

   procedure Show_Processors
     (Update_Processors_Set : access Update_Processors_Set_Record'Class)
   is

      A_Processor : Processor_Ptr;
      My_Iterator : Processors_Iterator;
      Line        : Gint;

      My_Scheduler : Scheduler_Ptr;
      A_Line       : Chars_Ptr_Array (1 .. 5);

   begin
      Clear (Update_Processors_Set.Processors_List);

      Reset_Iterator (Sys.Processors, My_Iterator);
      if not Is_Empty (Sys.Processors) then
         loop

            Current_Element (Sys.Processors, A_Processor, My_Iterator);

            My_Scheduler := A_Processor.all.Scheduler;

            -- Create the line
            --
            A_Line := To_String (A_Processor.Name) +
                      To_String (Get_Name (My_Scheduler.all)) +
                      To_String (A_Processor.Network) +
                      Get_Quantum (My_Scheduler.all) +
                      Get_Preemptive (My_Scheduler.all);

            -- Display the Line
            --
            Line := Append (Update_Processors_Set.Processors_List, A_Line);

            exit when Is_Last_Element (Sys.Processors, My_Iterator);

            Next_Element (Sys.Processors, My_Iterator);

         end loop;
      end if;
   end Show_Processors;

   procedure On_Click_List (List : access Gtk_Clist_Record'Class) is

      Name                : Unbounded_String;
      Quantum             : Unbounded_String;
      Row                 : Gint;
      A_Processor         : Processor_Ptr;
      Index               : Strings_Range := Strings_Range'First;
      A_Network           : Network_Ptr;
      My_Network_Iterator : Network_Iterator;

      My_Scheduler : Scheduler_Ptr;
      Current      : Gint;

      Network_Items, Items : Strings_Table;
      Sched_Id             : Gint;

   begin

      Initialize (Items);
      Build_Global_Scheduler_Table_Name (Items);

      Row  := Get_Focus_Row (Update_Processors_Set.Processors_List);
      Name :=
         To_Unbounded_String
           (Get_Text (Update_Processors_Set.Processors_List, Row, 0));

      Update_Processors_Set.Current_Processor := Name;

      Quantum :=
         To_Unbounded_String
           (Get_Text (Update_Processors_Set.Processors_List, Row, 3));

      A_Processor := Search_Processor (Sys.Processors, Name);

      -- Preemptive push button
      --
      if Get_Preemptive (A_Processor.Scheduler.all) = Not_Preemptive then
         Set_Active (Update_Processors_Set.Entry_Option, False);
      else
         Set_Active (Update_Processors_Set.Entry_Option, True);
      end if;

      -- Quantum
      --
      Gtk.GEntry.Set_Text
        (Update_Processors_Set.Entry_Quantum,
         To_String (Quantum));

      -- Processor name
      --
      Gtk.GEntry.Set_Text
        (Update_Processors_Set.Entry_Processor_Name,
         To_String (Name));

      -- Rebuild The Scheduler Menu
      --

      My_Scheduler := A_Processor.all.Scheduler;

      if (Get_Name (My_Scheduler.all) = Earliest_Deadline_First_Protocol) then
         Sched_Id := 0;
      end if;
      if (Get_Name (My_Scheduler.all) = Least_Laxity_First_Protocol) then
         Sched_Id := 1;
      end if;
      if (Get_Name (My_Scheduler.all) = Rate_Monotonic_Protocol) then
         Sched_Id := 2;
      end if;
      if (Get_Name (My_Scheduler.all) = Deadline_Monotonic_Protocol) then
         Sched_Id := 3;
      end if;
      if (Get_Name (My_Scheduler.all) =
          Posix_1003_Highest_Priority_First_Protocol)
      then
         Sched_Id := 4;
      end if;
      if (Get_Name (My_Scheduler.all) =
          Time_Sharing_Based_On_Wait_Time_Protocol)
      then
         Sched_Id := 5;
      end if;
      if (Get_Name (My_Scheduler.all) = Round_Robin_Protocol) then
         Sched_Id := 6;
      end if;
      if (Get_Name (My_Scheduler.all) =
          Maximum_Urgency_First_Based_On_Laxity_Protocol)
      then
         Sched_Id := 7;
      end if;
      if (Get_Name (My_Scheduler.all) =
          Maximum_Urgency_First_Based_On_Deadline_Protocol)
      then
         Sched_Id := 8;
      end if;
      if (Get_Name (My_Scheduler.all) = D_Over_Protocol) then
         Sched_Id := 9;
      end if;
      if (Get_Name (My_Scheduler.all) =
          Time_Sharing_Based_On_Cpu_Usage_Protocol)
      then
         Sched_Id := 10;
      end if;
      if (Get_Name (My_Scheduler.all) = pipeline_User_Defined_Protocol) then
         Sched_Id := 11;
      end if;
      if (Get_Name (My_Scheduler.all) = Automata_User_Defined_Protocol) then
         Sched_Id := 12;
      end if;
      if (Get_Name (My_Scheduler.all) = Compiled_User_Defined_Protocol) then
         Sched_Id := 13;
      end if;
      if (Get_Name (My_Scheduler.all) = Hierarchical_Cyclic_Protocol) then
         Sched_Id := 14;
      end if;
      if (Get_Name (My_Scheduler.all) =
          Hierarchical_Round_Robin_Protocol)
      then
         Sched_Id := 15;
      end if;
      if (Get_Name (My_Scheduler.all) =
          Hierarchical_Fixed_Priority_Protocol)
      then
         Sched_Id := 16;
      end if;

      Update_Processors_Set.Clist_Scheduler_Menu := Widget_SList.Null_List;
      Gtk.Option_Menu.Gtk_New (Update_Processors_Set.Scheduler_Menu);
      Build_Option_Menu
        (Update_Processors_Set.Scheduler_Menu,
         Update_Processors_Set.Clist_Scheduler_Menu,
         Items,
         Sched_Id,
         null);
      Show_All (Update_Processors_Set.Scheduler_Menu);
      Attach_Defaults
        (Update_Processors_Set.Processors_Table,
         Update_Processors_Set.Scheduler_Menu,
         1,
         2,
         2,
         3);

      -- Rebuild network menu
      --

      Initialize (Network_Items);
      Index := 1;
      Reset_Iterator (Sys.Networks, My_Network_Iterator);
      Update_Processors_Set.Clist_Network_Menu := Widget_SList.Null_List;
      Add (Network_Items, To_Unbounded_String ("No_Network"));

      if not Is_Empty (Sys.Networks) then
         loop
            Current_Element (Sys.Networks, A_Network, My_Network_Iterator);
            Add (Network_Items, A_Network.Name);

            if A_Network.Name = A_Processor.Network then
               Current := Gint (Index);
            end if;
            Index := Index + 1;

            exit when Is_Last_Element (Sys.Networks, My_Network_Iterator);

            Next_Element (Sys.Networks, My_Network_Iterator);
         end loop;
      end if;

      Gtk.Option_Menu.Gtk_New (Update_Processors_Set.Network_Menu);
      Build_Option_Menu
        (Update_Processors_Set.Network_Menu,
         Update_Processors_Set.Clist_Network_Menu,
         Network_Items,
         Current,
         null);
      Show_All (Update_Processors_Set.Network_Menu);
      Attach_Defaults
        (Update_Processors_Set.Processors_Table,
         Update_Processors_Set.Network_Menu,
         1,
         2,
         3,
         4);

      -- Source code file name
      --
      if (Sched_Id = 11) or (Sched_Id = 12) or (Sched_Id = 13) then
         Gtk.GEntry.Set_Text
           (Update_Processors_Set.Entry_File_Name,
            To_String
               (Get_Behavior_File_Name
                   (pipeline_user_defined_Scheduler (A_Processor.Scheduler.all)
)));
      else
         Gtk.GEntry.Set_Text (Update_Processors_Set.Entry_File_Name, "");
      end if;

      Show_All (Update_Processors_Set);

   end On_Click_List;

   procedure On_Read_Pressed (Object : access Gtk_Button_Record'Class) is

      Sched : Gint;

   begin

      Sched :=
         Gint'Val
           (16 -
            Gint (Selected_Button
                     (Update_Processors_Set.Clist_Scheduler_Menu)));

      if (Sched /= 11) and (Sched /= 12) and (Sched /= 13) then
         Show_Message_Box (Lb_Only_On_Parametric (Current_Language));
         return;
      end if;

      Gtk_New (read_sc_file, update_file_name'Access);
      Show_All (read_sc_file);

   end On_Read_Pressed;

   procedure On_Close_Pressed (Object : access Gtk_Button_Record'Class) is
   begin

      Is_New   := False;
      Is_Saved := False;

      Destroy (Update_Processors_Set);
   end On_Close_Pressed;

   procedure On_Delete_Pressed (Object : access Gtk_Button_Record'Class) is

      My_Iterator : Processor_Set.Generic_Processor_Set.Iterator;
      A_Processor : Processor_Ptr;

   begin

      if Update_Processors_Set.Current_Processor = "" then
         Show_Message_Box (Lb_Select_Processor_First (Current_Language));
         return;
      end if;

      if Is_Empty (Sys.Processors) then
         Show_Message_Box (Lb_Can_Not_Modify (Current_Language));
         return;
      end if;

      Reset_Iterator (Sys.Processors, My_Iterator);
      if not Is_Empty (Sys.Processors) then
         loop

            Current_Element (Sys.Processors, A_Processor, My_Iterator);
            if A_Processor.Name =
               Update_Processors_Set.Current_Processor
            then

               Delete_Processor (Sys, A_Processor);
               Rebuild_Canvas (Graph_Window);

               -- Clear  entries
               --
               Update_Processors_Set.Current_Processor :=
                  To_Unbounded_String ("");
               Gtk.GEntry.Set_Text (Update_Processors_Set.Entry_Quantum, "");
               Set_Text (Update_Processors_Set.Entry_Processor_Name, "");
               Set_Text (Update_Processors_Set.Entry_File_Name, "");
               Set_Active (Update_Processors_Set.Entry_Option, False);

               exit;
            end if;

            exit when Is_Last_Element (Sys.Processors, My_Iterator);
            Next_Element (Sys.Processors, My_Iterator);

         end loop;

      end if;

      Show_Processors (Update_Processors_Set);

   end On_Delete_Pressed;

   procedure On_Cancel_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      Duplicate (Update_Processors_Set.Undo, Sys);
      Rebuild_Canvas (Graph_Window);
      Destroy (Update_Processors_Set);
   end On_Cancel_Pressed;

   procedure On_Advanced_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      if Visible_Is_Set (Update_Processors_Set.Vbox_List) then
         Show_Advanced (Update_Processors_Set, False);
      else
         Show_Advanced (Update_Processors_Set, True);
      end if;
   end On_Advanced_Pressed;

   procedure On_Add_Pressed (Object : access Gtk_Button_Record'Class) is

      Ok            : Boolean := True;
      Error_Message : Unbounded_String;

   begin

      Integrity_Checks.Check_Integrity_Constraints
        (Update_Processors_Set,
         Error_Message,
         Ok);
      if not Ok then
         Show_Message_Box (Error_Message);
         return;
      end if;

      Add_Processor
        (Sys.Processors,
         Integrity_Checks.Processor_Name,
         Integrity_Checks.Processor_Preemptive,
         Integrity_Checks.Processor_Quantum,
         Integrity_Checks.Processor_File_Name,
         Integrity_Checks.Processor_Scheduler,
         Integrity_Checks.Processor_Network);

      Show_Processors (Update_Processors_Set);

   exception
      when Processor_Set.Invalid_Parameter =>
         begin
            Show_Message_Box (Exception_Message);
            return;
         end;
      when others =>
         begin
            Show_Message_Box (Exception_Name & Exception_Message);
            return;
         end;

   end On_Add_Pressed;

   procedure On_Modify_Pressed (Object : access Gtk_Button_Record'Class) is

      Ok            : Boolean := True;
      Error_Message : Unbounded_String;
      My_Iterator   : Processors_Iterator;
      A_Processor   : Processor_Ptr;

   begin

      if Update_Processors_Set.Current_Processor = "" then
         Show_Message_Box (Lb_Select_Processor_First (Current_Language));
         return;
      end if;

      if Is_Empty (Sys.Processors) then
         Show_Message_Box (Lb_Can_Not_Modify (Current_Language));
         return;
      end if;

      Integrity_Checks.Check_Integrity_Constraints
        (Update_Processors_Set,
         Error_Message,
         Ok);
      if not Ok then
         Show_Message_Box (Error_Message);
         return;
      end if;

      --
      -- Update set
      --

      Reset_Iterator (Sys.Processors, My_Iterator);
      loop

         Current_Element (Sys.Processors, A_Processor, My_Iterator);
         if A_Processor.Name = Update_Processors_Set.Current_Processor then

            Delete (Sys.Processors, A_Processor);

            Add_Processor
              (Sys.Processors,
               Integrity_Checks.Processor_Name,
               Integrity_Checks.Processor_Preemptive,
               Integrity_Checks.Processor_Quantum,
               Integrity_Checks.Processor_File_Name,
               Integrity_Checks.Processor_Scheduler,
               Integrity_Checks.Processor_Network);

            exit;
         end if;

         exit when Is_Last_Element (Sys.Processors, My_Iterator);
         Next_Element (Sys.Processors, My_Iterator);

      end loop;

      Show_Processors (Update_Processors_Set);

   exception
      when Processor_Set.Invalid_Parameter =>
         begin
            Show_Message_Box (Exception_Message);
            return;
         end;
      when others =>
         begin
            Show_Message_Box (Exception_Name & Exception_Message);
            return;
         end;

   end On_Modify_Pressed;

end Update_Processors_Set_Pkg.Callbacks;
