------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Processor_Set;         use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Gtk.Frame;             use Gtk.Frame;
with Gtk.Label;             use Gtk.Label;
with Gtk.Pixmap;            use Gtk.Pixmap;
with Gtk.Separator;         use Gtk.Separator;
with Gtk;                   use Gtk;
with Gtk.Check_Button;      use Gtk.Check_Button;
with Gtk.Option_Menu;       use Gtk.Option_Menu;
with Gtk.Window;            use Gtk.Window;
with Gtk.Table;             use Gtk.Table;
with Gtk.GEntry;            use Gtk.GEntry;
with Gtk.Button;            use Gtk.Button;
with Gtk.Box;               use Gtk.Box;
with Gtk.Hbutton_Box;       use Gtk.Hbutton_Box;
with Gtk.Scrolled_Window;   use Gtk.Scrolled_Window;
with Gtk.Widget;            use Gtk.Widget;
with Address_Space_Set;     use Address_Space_Set;
use Address_Space_Set.Generic_Address_Space_Set;
with Network_Set;           use Network_Set;
use Network_Set.Generic_Network_Set;
with Gtk.Clist;             use Gtk.Clist;
with Gtkada.Types;          use Gtkada.Types;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Systems;               use Systems;

package Update_Processors_Set_Pkg is

   type Update_Processors_Set_Record is new Gtk_Window_Record with private;
   type Update_Processors_Set_Access is access all 
     Update_Processors_Set_Record'Class;

   procedure Gtk_New
     (Update_Processors_Set : out Update_Processors_Set_Access;
      Title                 : in Unbounded_String);
   procedure Initialize
     (Update_Processors_Set : access Update_Processors_Set_Record'Class;
      Title                 : in Unbounded_String);
   procedure Show_Advanced
     (Update_Processors_Set : in Update_Processors_Set_Access;
      Advanced              : in Boolean);

   Update_Processors_Set : Update_Processors_Set_Access;
   --  Parent                : Unbounded_String;
private

   type Update_Processors_Set_Record is new Gtk_Window_Record with record
      Scroll : Gtk_Scrolled_Window;

      Scheduler_Menu       : Gtk_Option_Menu;
      Clist_Scheduler_Menu : Widget_SList.GSlist;
      Network_Menu         : Gtk_Option_Menu;
      Clist_Network_Menu   : Widget_SList.GSlist;
      Processors_List      : Gtk.Clist.Gtk_Clist;

      Current_Processor : Unbounded_String;

      Entry_Processor_Name   : Gtk_Entry;
      Entry_Scheduler_Option : Gtk_Entry;
      Entry_Quantum          : Gtk_Entry;
      Entry_Network          : Gtk_Entry;
      Entry_File_Name        : Gtk_Entry;
      Entry_Option           : Gtk_Check_Button;

      Lab : Gtk_Label;

      Frame_Processors : Gtk.Frame.Gtk_Frame;
      Processors_Table : Gtk.Table.Gtk_Table;

      Vbox_Main       : Gtk.Box.Gtk_Box;
      Hbox_Processors : Gtk.Box.Gtk_Box;
      Hbox_Confirm    : Gtk.Box.Gtk_Box;
      Vbox_List       : Gtk.Box.Gtk_Box;
      Hbox_Modify     : Gtk.Box.Gtk_Box;
      Vbox_Update     : Gtk.Box.Gtk_Box;

      Hbutton_Box_Modify : Gtk_Hbutton_Box;

      Modify_Button   : Gtk_Button;
      Advanced_Button : Gtk_Button;
      Close_Button    : Gtk_Button;
      Cancel_Button   : Gtk_Button;
      Add_Button      : Gtk_Button;
      Delete_Button   : Gtk_Button;
      Read_Button     : Gtk_Button;

      Hseparator1 : Gtk_Hseparator;
      Vseparator  : Gtk_Vseparator;
      Vseparator2 : Gtk_Vseparator;

      Child_Page : Gtk_Frame;
      Label_Box  : Gtk_Box;
      Menu_Box   : Gtk_Box;
      Pixmap     : Gtk_Pixmap;
      Vbox       : Gtk_Box;
      Button     : Gtk_Button;

      Undo : System;

      Scroll_List : Gtk_Scrolled_Window;
      List_Titles : Chars_Ptr_Array (1 .. 5);

      Advanced : Boolean := False;
   end record;

end Update_Processors_Set_Pkg;
