------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with GNAT.Current_Exception;                              use 
  GNAT.Current_Exception;
with Glib;                                                use Glib;
with Mmi_Tools;                                           use Mmi_Tools;
with Translate;                                           use Translate;
with Framework_Config;                                    use Framework_Config;
with Unbounded_Strings;                                   use 
  Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;
with Project;                                             use Project;
with Tasks;                                               use Tasks;
with Task_Set;                                            use Task_Set;
use Task_Set.Generic_Task_Set;
with Buffer_Set;                                          use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Address_Spaces;                                      use Address_Spaces;
with Address_Space_Set;                                   use 
  Address_Space_Set;
use Address_Space_Set.Generic_Address_Space_Set;
with Update_Resources_Set_Pkg.Callbacks.Integrity_Checks;

package body Update_Resources_Set_Pkg.Callbacks is

   procedure Show_Resources
     (Update_Resources_Set : access Update_Resources_Set_Record'Class)
   is

      A_Resource      : Generic_Resource_Ptr;
      My_Iterator     : Resources_Iterator;
      Line            : Gint;
      Resource_Share  : Chars_Ptr_Array (1 .. 3);
      Protocol_String : Unbounded_String;

   begin
      Clear (Update_Resources_Set.Resources_List);

      Reset_Iterator (Sys.Resources, My_Iterator);
      if not Is_Empty (Sys.Resources) then
         loop

            Current_Element (Sys.Resources, A_Resource, My_Iterator);

            Protocol_String :=
               To_Unbounded_String
                 (Resources_Type'Image (A_Resource.protocol));

            -- Create the line
            --
            Resource_Share := To_String (A_Resource.Name) +
                              To_String (A_Resource.cpu_name) +
                              To_String (Protocol_String);

            -- Display the Line
            --
            Line :=
               Append (Update_Resources_Set.Resources_List, Resource_Share);

            exit when Is_Last_Element (Sys.Resources, My_Iterator);

            Next_Element (Sys.Resources, My_Iterator);

         end loop;
      end if;
   end Show_Resources;

   procedure Show_Task_Affect_List
     (Update_Resources_Set : access Update_Resources_Set_Record'Class)
   is

      Affect_Task_Share : Affected_Task_Lists;
      Task_Name         : Unbounded_String;
      Line              : Gint;
      Task_Share        : Chars_Ptr_Array (1 .. 3);

   begin
      Clear (Update_Resources_Set.Affected_Task_Lists);

      for I in  0 .. Update_Resources_Set.Task_Lists_Share.Nb_Entries - 1 loop

         Task_Name                    :=
           Update_Resources_Set.Task_Lists_Share.Entries (I).Item;
         Affect_Task_Share.Task_Begin :=
           Update_Resources_Set.Task_Lists_Share.Entries (I).Data.Task_Begin;
         Affect_Task_Share.Task_End   :=
           Update_Resources_Set.Task_Lists_Share.Entries (I).Data.Task_End;

         Task_Share := To_String (Task_Name) +
                       Affect_Task_Share.Task_Begin'Img +
                       Affect_Task_Share.Task_End'Img;

         -- Show Task on Clist
         --
         Line :=
            Append (Update_Resources_Set.Affected_Task_Lists, Task_Share);

      end loop;

   end Show_Task_Affect_List;

   procedure On_Click_List (List : access Gtk_Clist_Record'Class) is

      A_Address_Space        : Address_Space_Ptr;
      Address_Space_Iterator :
        Address_Space_Set.Generic_Address_Space_Set.Iterator;
      Empty_Menu             : String_List.Glist;

      Name     : Unbounded_String;
      Protocol : Unbounded_String;
      Row      : Gint;

      A_Resource : Generic_Resource_Ptr;

      Protocol_Items : String_List.Glist;

   begin

      Row := Get_Focus_Row (Update_Resources_Set.Resources_List);

      Name                                  :=
         To_Unbounded_String
           (Get_Text (Update_Resources_Set.Resources_List, Row, 0));
      Update_Resources_Set.Current_Resource := Name;

      Protocol :=
         To_Unbounded_String
           (Get_Text (Update_Resources_Set.Resources_List, Row, 2));

      A_Resource := Search_Resource (Sys.Resources, Name);

      -- Show name
      --
      Gtk.GEntry.Set_Text (Update_Resources_Set.Name, To_String (Name));

      -- Show state
      --
      Gtk.GEntry.Set_Text (Update_Resources_Set.State, A_Resource.state'Img);

      -- Show address space and processor names
      --
      A_Address_Space :=
         Search_Address_Space
           (Sys.Address_Spaces,
            A_Resource.Address_Space_Name);
      Set_Text
        (Update_Resources_Set.Cpu_Name,
         To_String (A_Address_Space.Cpu_Name));

      Update_Resources_Set.Clist_Address_Space_Menu := Empty_Menu;
      Reset_Iterator (Sys.Address_Spaces, Address_Space_Iterator);
      loop
         Current_Element
           (Sys.Address_Spaces,
            A_Address_Space,
            Address_Space_Iterator);
         if A_Address_Space.Name = A_Resource.Address_Space_Name then
            String_List.Append
              (Update_Resources_Set.Clist_Address_Space_Menu,
               To_String (A_Address_Space.Name));
         end if;
         exit when Is_Last_Element
                      (Sys.Address_Spaces,
                       Address_Space_Iterator);

         Next_Element (Sys.Address_Spaces, Address_Space_Iterator);
      end loop;
      Reset_Iterator (Sys.Address_Spaces, Address_Space_Iterator);
      loop
         Current_Element
           (Sys.Address_Spaces,
            A_Address_Space,
            Address_Space_Iterator);
         if A_Address_Space.Name /= A_Resource.Address_Space_Name then
            String_List.Append
              (Update_Resources_Set.Clist_Address_Space_Menu,
               To_String (A_Address_Space.Name));
         end if;
         exit when Is_Last_Element
                      (Sys.Address_Spaces,
                       Address_Space_Iterator);

         Next_Element (Sys.Address_Spaces, Address_Space_Iterator);
      end loop;

      Set_Popdown_Strings
        (Update_Resources_Set.Address_Space_Menu,
         Update_Resources_Set.Clist_Address_Space_Menu);

      -- Show protocol
      --
      if Protocol = "NO_PROTOCOL" then
         String_List.Append (Protocol_Items, "No Protocol");
         String_List.Append (Protocol_Items, "PCP");
         String_List.Append (Protocol_Items, "PIP");
         String_List.Append (Protocol_Items, "IPCP");
      else
         if Protocol = "PRIORITY_CEILING_PROTOCOL" then
            String_List.Append (Protocol_Items, "PCP");
            String_List.Append (Protocol_Items, "No Protocol");
            String_List.Append (Protocol_Items, "PIP");
            String_List.Append (Protocol_Items, "IPCP");
         else
            if Protocol = "IMMEDIATE_PRIORITY_CEILING_PROTOCOL" then
               String_List.Append (Protocol_Items, "IPCP");
               String_List.Append (Protocol_Items, "No Protocol");
               String_List.Append (Protocol_Items, "PIP");
               String_List.Append (Protocol_Items, "PCP");
            else
               if Protocol = "PRIORITY_INHERITANCE_PROTOCOL" then
                  String_List.Append (Protocol_Items, "PIP");
                  String_List.Append (Protocol_Items, "No Protocol");
                  String_List.Append (Protocol_Items, "PCP");
                  String_List.Append (Protocol_Items, "IPCP");
               end if;
            end if;
         end if;
      end if;

      Set_Popdown_Strings (Update_Resources_Set.Protocol, Protocol_Items);

      --  Show task using the resource
      --
      Initialize (Update_Resources_Set.Task_Lists_Share);
      for I in  0 .. A_Resource.task_tab.Nb_Entries - 1 loop
         Update_Resources_Set.Task_Lists_Share.Entries (I).Item :=
           A_Resource.task_tab.Entries (I).Item;
         Update_Resources_Set.Task_Lists_Share.Entries (I).Data :=
           A_Resource.task_tab.Entries (I).Data;
         Update_Resources_Set.Task_Lists_Share.Nb_Entries       :=
           Update_Resources_Set.Task_Lists_Share.Nb_Entries + 1;
      end loop;

      Show_Task_Affect_List (Update_Resources_Set);
      Show_All (Update_Resources_Set);

   end On_Click_List;

   procedure On_Close_Pressed (Object : access Gtk_Button_Record'Class) is
   begin

      Is_New   := False;
      Is_Saved := False;

      Destroy (Update_Resources_Set);
   end On_Close_Pressed;

   procedure On_Cancel_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      Duplicate (Update_Resources_Set.Undo, Sys);
      Destroy (Update_Resources_Set);
   end On_Cancel_Pressed;

   procedure On_Advanced_Pressed (Object : access Gtk_Button_Record'Class) is
   begin
      if Visible_Is_Set (Update_Resources_Set.Vbox_List) then
         Show_Advanced (Update_Resources_Set, False);
      else
         Show_Advanced (Update_Resources_Set, True);
      end if;
   end On_Advanced_Pressed;

   procedure On_Delete_Pressed (Object : access Gtk_Button_Record'Class) is

      My_Iterator : Resource_Set.Generic_Resource_Set.Iterator;
      A_Resource  : Generic_Resource_Ptr;

   begin

      if Update_Resources_Set.Current_Resource = "" then
         Show_Message_Box (Lb_Select_Resource_First (Current_Language));
         return;
      end if;

      if Is_Empty (Sys.Resources) then
         Show_Message_Box (Lb_Can_Not_Modify (Current_Language));
         return;
      end if;

      Reset_Iterator (Sys.Resources, My_Iterator);
      if not Is_Empty (Sys.Resources) then
         loop

            Current_Element (Sys.Resources, A_Resource, My_Iterator);
            if A_Resource.Name = Update_Resources_Set.Current_Resource then

               Delete (Sys.Resources, A_Resource);

               -- Clear  entries
               --
               Update_Resources_Set.Current_Resource :=
                  To_Unbounded_String ("");
               Gtk.GEntry.Set_Text (Update_Resources_Set.Name, "");
               Gtk.GEntry.Set_Text (Update_Resources_Set.State, "1");
               Clear (Update_Resources_Set.Affected_Task_Lists);

               exit;
            end if;

            exit when Is_Last_Element (Sys.Resources, My_Iterator);
            Next_Element (Sys.Resources, My_Iterator);

         end loop;

      end if;

      Show_Resources (Update_Resources_Set);

   end On_Delete_Pressed;

   procedure On_Add_Pressed (Object : access Gtk_Button_Record'Class) is

      Ok            : Boolean := True;
      Error_Message : Unbounded_String;

   begin

      Integrity_Checks.Check_Integrity_Constraints
        (Update_Resources_Set,
         Error_Message,
         Ok);
      if not Ok then
         Show_Message_Box (Error_Message);
         return;
      end if;

      -- Add the new item in the set
      --
      Add_Resource
        (Sys.Resources,
         Integrity_Checks.Resource_Name,
         Integrity_Checks.Resource_State,
         Integrity_Checks.Resource_Processor,
         Integrity_Checks.Resource_Address_Space,
         Integrity_Checks.Resource_Protocol,
         Update_Resources_Set.Task_Lists_Share);

      Update_Resources_Set.Current_Resource := Integrity_Checks.Resource_Name;

      Show_Resources (Update_Resources_Set);

   exception
      when Resource_Set.Invalid_Parameter =>
         begin
            Show_Message_Box (Exception_Message);
            return;
         end;
      when others =>
         begin
            Show_Message_Box (Exception_Name & Exception_Message);
            return;
         end;

   end On_Add_Pressed;

   procedure On_Modify_Pressed (Object : access Gtk_Button_Record'Class) is

      A_Resource    : Generic_Resource_Ptr;
      My_Iterator   : Resource_Set.Generic_Resource_Set.Iterator;
      Ok            : Boolean := True;
      Error_Message : Unbounded_String;

   begin

      if Update_Resources_Set.Current_Resource = "" then
         Show_Message_Box (Lb_Select_Resource_First (Current_Language));
         return;
      end if;

      if Is_Empty (Sys.Resources) then
         Show_Message_Box (Lb_Can_Not_Modify (Current_Language));
         return;
      end if;

      Integrity_Checks.Check_Integrity_Constraints
        (Update_Resources_Set,
         Error_Message,
         Ok);
      if not Ok then
         Show_Message_Box (Error_Message);
         return;
      end if;

      --
      -- Update resource set
      --

      Reset_Iterator (Sys.Resources, My_Iterator);
      loop

         Current_Element (Sys.Resources, A_Resource, My_Iterator);
         if A_Resource.Name = Update_Resources_Set.Current_Resource then

            Delete (Sys.Resources, A_Resource);

            Update_Resources_Set.Current_Resource :=
              Integrity_Checks.Resource_Name;
            exit;
         end if;

         exit when Is_Last_Element (Sys.Resources, My_Iterator);
         Next_Element (Sys.Resources, My_Iterator);

      end loop;

      Add_Resource
        (Sys.Resources,
         Integrity_Checks.Resource_Name,
         Integrity_Checks.Resource_State,
         Integrity_Checks.Resource_Processor,
         Integrity_Checks.Resource_Address_Space,
         Integrity_Checks.Resource_Protocol,
         Update_Resources_Set.Task_Lists_Share);

      Show_Resources (Update_Resources_Set);

   exception
      when Resource_Set.Invalid_Parameter =>
         begin
            Show_Message_Box (Exception_Message);
            return;
         end;
      when others =>
         begin
            Show_Message_Box (Exception_Name & Exception_Message);
            return;
         end;

   end On_Modify_Pressed;

   procedure On_Change_Cpu (Widget : access Gtk_Editable_Record'Class) is

      Resource_Address_Space : Unbounded_String;
      A_Address_Space        : Address_Space_Ptr;

   begin
      Resource_Address_Space :=
         To_Unbounded_String
           (Get_Text (Get_Entry (Update_Resources_Set.Address_Space_Menu)));
      A_Address_Space        :=
         Search_Address_Space (Sys.Address_Spaces, Resource_Address_Space);
      Set_Text
        (Update_Resources_Set.Cpu_Name,
         To_String (A_Address_Space.Cpu_Name));
   end On_Change_Cpu;

   procedure On_Affected_Task_Lists_Click
     (List : access Gtk_Clist_Record'Class)
   is

      Name       : Unbounded_String;
      Task_Begin : Unbounded_String;
      Task_End   : Unbounded_String;
      Row        : Gint;

      A_Task     : Generic_Task_Ptr;
      Ite        : Tasks_Iterator;
      Task_Items : String_List.Glist;

   begin

      Row        := Get_Focus_Row (Update_Resources_Set.Affected_Task_Lists);
      Name       :=
         To_Unbounded_String
           (Get_Text (Update_Resources_Set.Affected_Task_Lists, Row, 0));
      Task_Begin :=
         To_Unbounded_String
           (Get_Text (Update_Resources_Set.Affected_Task_Lists, Row, 1));
      Task_End   :=
         To_Unbounded_String
           (Get_Text (Update_Resources_Set.Affected_Task_Lists, Row, 2));

      String_List.Append (Task_Items, To_String (Name));
      Reset_Iterator (Sys.Tasks, Ite);
      loop
         Current_Element (Sys.Tasks, A_Task, Ite);

         if A_Task.Name /= Name then
            String_List.Append (Task_Items, To_String (A_Task.Name));
         end if;

         exit when Is_Last_Element (Sys.Tasks, Ite);

         Next_Element (Sys.Tasks, Ite);
      end loop;

      Set_Popdown_Strings
        (Update_Resources_Set.Resource_Affect_Task,
         Task_Items);

      Gtk.GEntry.Set_Text
        (Update_Resources_Set.Resource_Affect_Begin,
         To_String (Task_Begin));
      Gtk.GEntry.Set_Text
        (Update_Resources_Set.Resource_Affect_End,
         To_String (Task_End));

   end On_Affected_Task_Lists_Click;

   procedure On_Add_Affect_Pressed
     (Object : access Gtk_Button_Record'Class)
   is

      Affect_Task_Share : Affected_Task_Lists;
      Ok                : Boolean := True;
      Error_Message     : Unbounded_String;

   begin

      Integrity_Checks.Check_Task_Sharing_Integrity_Constraints
        (Update_Resources_Set,
         Error_Message,
         Ok);
      if not Ok then
         Show_Message_Box (Error_Message);
         return;
      end if;

      Affect_Task_Share.Task_Begin :=
        Natural (Integrity_Checks.Resource_Share_Begin);
      Affect_Task_Share.Task_End   :=
        Natural (Integrity_Checks.Resource_Share_End);

      -- Add Task on list
      --
      Update_Resources_Set.Task_Lists_Share.Entries (
        Update_Resources_Set.Task_Lists_Share.Nb_Entries).Item :=
        Integrity_Checks.Resource_Task_Name;
      Update_Resources_Set.Task_Lists_Share.Entries (
        Update_Resources_Set.Task_Lists_Share.Nb_Entries).Data :=
        Affect_Task_Share;
      Update_Resources_Set.Task_Lists_Share.Nb_Entries :=
        Update_Resources_Set.Task_Lists_Share.Nb_Entries + 1;

      -- Show Task on Clist
      --
      Show_Task_Affect_List (Update_Resources_Set);

   end On_Add_Affect_Pressed;

   procedure On_Modify_Affect_Pressed
     (Object : access Gtk_Button_Record'Class)
   is

      Ok            : Boolean := True;
      Error_Message : Unbounded_String;

      Name              : Unbounded_String;
      Task_Begin_String : Unbounded_String;
      Task_End_String   : Unbounded_String;
      Task_Begin        : Natural;
      Task_End          : Natural;
      Natural_Ok        : Boolean;
      Row               : Gint;

   begin

      Row               :=
         Get_Focus_Row (Update_Resources_Set.Affected_Task_Lists);
      Name              :=
         To_Unbounded_String
           (Get_Text (Update_Resources_Set.Affected_Task_Lists, Row, 0));
      Task_Begin_String :=
         To_Unbounded_String
           (Get_Text (Update_Resources_Set.Affected_Task_Lists, Row, 1));
      Task_End_String   :=
         To_Unbounded_String
           (Get_Text (Update_Resources_Set.Affected_Task_Lists, Row, 2));
      To_Natural (Task_Begin_String, Task_Begin, Natural_Ok);
      To_Natural (Task_End_String, Task_End, Natural_Ok);

      Integrity_Checks.Check_Task_Sharing_Integrity_Constraints
        (Update_Resources_Set,
         Error_Message,
         Ok);
      if not Ok then
         Show_Message_Box (Error_Message);
         return;
      end if;

      -- Find old values and replace them
      --
      for I in  0 .. Update_Resources_Set.Task_Lists_Share.Nb_Entries - 1 loop
         if (Update_Resources_Set.Task_Lists_Share.Entries (I).Item = Name) and
            (Update_Resources_Set.Task_Lists_Share.Entries (I).Data.Task_Begin
             =
             Task_Begin) and
            (Update_Resources_Set.Task_Lists_Share.Entries (I).Data.Task_End =
             Task_End)
         then
            Update_Resources_Set.Task_Lists_Share.Entries (I).Item :=
              Integrity_Checks.Resource_Task_Name;
            Update_Resources_Set.Task_Lists_Share.Entries (I).Data.Task_End :=
              Integrity_Checks.Resource_Share_End;
            Update_Resources_Set.Task_Lists_Share.Entries (I).Data.Task_Begin
               := Integrity_Checks.Resource_Share_Begin;
            exit;
         end if;
      end loop;

      --  Show task using the resource
      --
      Show_Task_Affect_List (Update_Resources_Set);

   end On_Modify_Affect_Pressed;

   procedure On_Delete_Affect_Pressed
     (Object : access Gtk_Button_Record'Class)
   is

      Ok            : Boolean := True;
      Error_Message : Unbounded_String;

   begin

      Integrity_Checks.Check_Task_Sharing_Integrity_Constraints
        (Update_Resources_Set,
         Error_Message,
         Ok);
      if not Ok then
         Show_Message_Box (Error_Message);
         return;
      end if;

      -- Find item and delete it
      --
      for I in  0 .. Update_Resources_Set.Task_Lists_Share.Nb_Entries - 1 loop
         if (Update_Resources_Set.Task_Lists_Share.Entries (I).Item =
             Integrity_Checks.Resource_Task_Name) and
            (Update_Resources_Set.Task_Lists_Share.Entries (I).Data.Task_Begin
             =
             Integrity_Checks.Resource_Share_Begin) and
            (Update_Resources_Set.Task_Lists_Share.Entries (I).Data.Task_End =
             Integrity_Checks.Resource_Share_End)
         then
            Update_Resources_Set.Task_Lists_Share.Entries (I) :=
              Update_Resources_Set.Task_Lists_Share.Entries (
              Update_Resources_Set.Task_Lists_Share.Nb_Entries -
                                                             1);
            Update_Resources_Set.Task_Lists_Share.Nb_Entries  :=
              Update_Resources_Set.Task_Lists_Share.Nb_Entries - 1;
            exit;
         end if;
      end loop;

      --  Show task using the resource
      --
      Show_Task_Affect_List (Update_Resources_Set);

   end On_Delete_Affect_Pressed;

end Update_Resources_Set_Pkg.Callbacks;
