------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Callbacks_Cheddar;                  use Callbacks_Cheddar;
with Update_Resources_Set_Pkg.Callbacks;
use Update_Resources_Set_Pkg.Callbacks;
with Processors;                         use Processors;
with Unbounded_Strings;                  use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;
with Project;                            use Project;
with Tasks;                              use Tasks;
with Task_Set;                           use Task_Set;
use Task_Set.Generic_Task_Set;
with Translate;                          use Translate;
with Framework_Config;                   use Framework_Config;
with Address_Spaces;                     use Address_Spaces;
with Address_Space_Set;                  use Address_Space_Set;
use Address_Space_Set.Generic_Address_Space_Set;

package body Update_Resources_Set_Pkg is

   procedure Gtk_New
     (Update_Resources_Set : out Update_Resources_Set_Access;
      Title                : in Unbounded_String)
   is
   begin
      Update_Resources_Set := new Update_Resources_Set_Record;
      Update_Resources_Set_Pkg.Initialize (Update_Resources_Set, Title);
   end Gtk_New;

   procedure Show_Advanced
     (Update_Resources_Set : in Update_Resources_Set_Access;
      Advanced             : in Boolean)
   is

   begin

      if not Advanced then
         Hide (Update_Resources_Set.Vbox_List);
         Hide (Update_Resources_Set.Delete_Button);
         Hide (Update_Resources_Set.Modify_Button);

         Show (Update_Resources_Set.Add_Button);
         Show (Update_Resources_Set.Close_Button);
         Show (Update_Resources_Set.Cancel_Button);
         Show (Update_Resources_Set.Advanced_Button);

         Show (Update_Resources_Set.Vbox_Main);
         Show (Update_Resources_Set.Hbox_Resource);
         Show_All (Update_Resources_Set.Vbox_Update);
         Show (Update_Resources_Set.Hbutton_Box_Modify);
         Show (Update_Resources_Set.Frame_Resource);

      else
         Show_All (Update_Resources_Set.Vbox_Main);
      end if;
      Show (Update_Resources_Set);

   end Show_Advanced;

   procedure Initialize
     (Update_Resources_Set : access Update_Resources_Set_Record'Class;
      Title                : in Unbounded_String)
   is

      A_Task         : Generic_Task_Ptr;
      My_Iterator2   : Tasks_Iterator;
      The_Name       : Unbounded_String;
      Protocol_Items : String_List.Glist;

      Processor_Iterator : Processors_Iterator;
      Processor_Items    : Strings_Table;
      A_Processor        : Processor_Ptr;

      Change_Cpu             : Gtk_Entry;
      Address_Space_Iterator : Address_Spaces_Iterator;
      Address_Space_Items    : Strings_Table;
      A_Address_Space        : Address_Space_Ptr;
      First_Address_Space    : Address_Space_Ptr;

   begin

      Initialize (Update_Resources_Set.Undo);
      Duplicate (Sys, Update_Resources_Set.Undo);
      Update_Resources_Set.Current_Resource := Empty_String;

      Gtk.Window.Initialize (Update_Resources_Set, Window_Toplevel);
      Set_Policy (Update_Resources_Set, True, True, True);
      Set_Title (Update_Resources_Set, To_String (Title));
      Set_Position (Update_Resources_Set, Win_Pos_Center_Always);
      Set_Modal (Update_Resources_Set, True);
      Grab_Focus (Update_Resources_Set);

      Gtk.Box.Gtk_New_Vbox (Update_Resources_Set.Vbox_Main, False, 10);
      Set_Border_Width (Update_Resources_Set.Vbox_Main, 10);
      Add (Update_Resources_Set, Update_Resources_Set.Vbox_Main);

      Gtk.Box.Gtk_New_Hbox (Update_Resources_Set.Hbox_Resource, False, 10);
      Add
        (Update_Resources_Set.Vbox_Main,
         Update_Resources_Set.Hbox_Resource);

      Gtk.Box.Gtk_New_Vbox (Update_Resources_Set.Vbox_Update, False, 10);
      Add
        (Update_Resources_Set.Hbox_Resource,
         Update_Resources_Set.Vbox_Update);

      Gtk.Box.Gtk_New_Vbox (Update_Resources_Set.Vbox_List, False, 10);
      Add
        (Update_Resources_Set.Hbox_Resource,
         Update_Resources_Set.Vbox_List);

      -----------------------------------------------------
      -- Main resource widget (basic shared resource information)
      -----------------------------------------------------

      Gtk.Frame.Gtk_New (Update_Resources_Set.Frame_Resource, "");
      Add
        (Update_Resources_Set.Vbox_Update,
         Update_Resources_Set.Frame_Resource);

      Gtk.Table.Gtk_New (Update_Resources_Set.Resource_Table, 2, 15, False);
      Set_Border_Width (Update_Resources_Set.Resource_Table, 5);
      Add
        (Update_Resources_Set.Frame_Resource,
         Update_Resources_Set.Resource_Table);

      -- Label widgets
      --
      Gtk.Label.Gtk_New
        (Update_Resources_Set.Lab,
         To_String (Lb_Resource_Name (Current_Language)) &
         To_String (Lb_Colon));
      Attach_Defaults
        (Update_Resources_Set.Resource_Table,
         Update_Resources_Set.Lab,
         0,
         1,
         0,
         1);

      Gtk.Label.Gtk_New
        (Update_Resources_Set.Lab,
         To_String (Lb_Protocol (Current_Language)) & To_String (Lb_Colon));
      Attach_Defaults
        (Update_Resources_Set.Resource_Table,
         Update_Resources_Set.Lab,
         0,
         1,
         2,
         3);

      Gtk.Label.Gtk_New
        (Update_Resources_Set.Lab,
         To_String (Lb_State (Current_Language)) & To_String (Lb_Colon));
      Attach_Defaults
        (Update_Resources_Set.Resource_Table,
         Update_Resources_Set.Lab,
         0,
         1,
         1,
         2);

      Gtk.Label.Gtk_New
        (Update_Resources_Set.Lab,
         To_String (Lb_Processor (Current_Language)) &
         To_String (Lb_Colon));
      Attach_Defaults
        (Update_Resources_Set.Resource_Table,
         Update_Resources_Set.Lab,
         0,
         1,
         3,
         4);

      Gtk.Label.Gtk_New
        (Update_Resources_Set.Lab,
         To_String (Lb_Address_Space (Current_Language)) &
         To_String (Lb_Colon));
      Attach_Defaults
        (Update_Resources_Set.Resource_Table,
         Update_Resources_Set.Lab,
         0,
         1,
         4,
         5);

      -- Entry widgets
      --

      Gtk.GEntry.Gtk_New (Update_Resources_Set.Name, 50);
      Set_Editable (Update_Resources_Set.Name, True);
      Attach_Defaults
        (Update_Resources_Set.Resource_Table,
         Update_Resources_Set.Name,
         1,
         2,
         0,
         1);

      Gtk.GEntry.Gtk_New (Update_Resources_Set.State, 50);
      Set_Editable (Update_Resources_Set.State, True);
      Gtk.GEntry.Set_Text (Update_Resources_Set.State, "1");
      Attach_Defaults
        (Update_Resources_Set.Resource_Table,
         Update_Resources_Set.State,
         1,
         2,
         1,
         2);

      Gtk.Combo.Gtk_New (Update_Resources_Set.Protocol);
      Set_Editable (Get_Entry (Update_Resources_Set.Protocol), False);
      String_List.Append (Protocol_Items, "No Protocol");
      String_List.Append (Protocol_Items, "PCP");
      String_List.Append (Protocol_Items, "PIP");
      String_List.Append (Protocol_Items, "IPCP");
      Set_Popdown_Strings (Update_Resources_Set.Protocol, Protocol_Items);
      Attach_Defaults
        (Update_Resources_Set.Resource_Table,
         Update_Resources_Set.Protocol,
         1,
         2,
         2,
         3);

      Initialize (Processor_Items);
      Reset_Iterator (Sys.Processors, Processor_Iterator);

      loop
         Current_Element (Sys.Processors, A_Processor, Processor_Iterator);
         Add (Processor_Items, A_Processor.Name);

         exit when Is_Last_Element (Sys.Processors, Processor_Iterator);

         Next_Element (Sys.Processors, Processor_Iterator);
      end loop;

      -- Cpu and address space widgets
      --

      Initialize (Address_Space_Items);
      Reset_Iterator (Sys.Address_Spaces, Address_Space_Iterator);
      Gtk.Combo.Gtk_New (Update_Resources_Set.Address_Space_Menu);
      Set_Editable
        (Get_Entry (Update_Resources_Set.Address_Space_Menu),
         False);

      loop
         Current_Element
           (Sys.Address_Spaces,
            A_Address_Space,
            Address_Space_Iterator);
         String_List.Append
           (Update_Resources_Set.Clist_Address_Space_Menu,
            To_String (A_Address_Space.Name));
         if First_Address_Space = null then
            First_Address_Space := A_Address_Space;
         end if;

         exit when Is_Last_Element
                      (Sys.Address_Spaces,
                       Address_Space_Iterator);

         Next_Element (Sys.Address_Spaces, Address_Space_Iterator);
      end loop;
      Set_Popdown_Strings
        (Update_Resources_Set.Address_Space_Menu,
         Update_Resources_Set.Clist_Address_Space_Menu);
      Attach_Defaults
        (Update_Resources_Set.Resource_Table,
         Update_Resources_Set.Address_Space_Menu,
         1,
         2,
         4,
         5);

      Gtk.GEntry.Gtk_New (Update_Resources_Set.Cpu_Name, 50);
      Set_Editable (Update_Resources_Set.Cpu_Name, False);
      Gtk.GEntry.Set_Text
        (Update_Resources_Set.Cpu_Name,
         To_String (First_Address_Space.Cpu_Name));
      Attach_Defaults
        (Update_Resources_Set.Resource_Table,
         Update_Resources_Set.Cpu_Name,
         1,
         2,
         3,
         4);

      -----------------------------------------------------
      -- Affect task sub-widget
      -----------------------------------------------------

      -- Frame Affect
      --
      Gtk.Frame.Gtk_New
        (Update_Resources_Set.Frame_Affect,
         To_String (Lb_Resource_Affect (Current_Language)));
      Add
        (Update_Resources_Set.Vbox_Update,
         Update_Resources_Set.Frame_Affect);

      -- Table Add_Affect
      --
      Gtk.Table.Gtk_New (Update_Resources_Set.Table_Add_Affect, 2, 4, False);
      Add
        (Update_Resources_Set.Frame_Affect,
         Update_Resources_Set.Table_Add_Affect);

      -- Box Add_Affect
      --
      Reset_Iterator (Sys.Tasks, My_Iterator2);

      loop

         Current_Element (Sys.Tasks, A_Task, My_Iterator2);
         The_Name := A_Task.Name;
         String_List.Append
           (Update_Resources_Set.Items,
            To_String (The_Name));

         exit when Is_Last_Element (Sys.Tasks, My_Iterator2);
         Next_Element (Sys.Tasks, My_Iterator2);

      end loop;

      Gtk_New (Update_Resources_Set.Resource_Affect_Task);
      Gtk.GEntry.Gtk_New (Update_Resources_Set.Resource_Affect_Begin, 10);
      Gtk.GEntry.Gtk_New (Update_Resources_Set.Resource_Affect_End, 10);
      Gtk.Button.Gtk_New
        (Update_Resources_Set.Button_Add_Affect,
         To_String (Lb_Add (Current_Language)));
      Button_Callback.Connect
        (Update_Resources_Set.Button_Add_Affect,
         "clicked",
         Button_Callback.To_Marshaller (On_Add_Affect_Pressed'Access));
      Gtk.Button.Gtk_New
        (Update_Resources_Set.Button_Delete_Affect,
         To_String (Lb_Delete (Current_Language)));
      Button_Callback.Connect
        (Update_Resources_Set.Button_Delete_Affect,
         "clicked",
         Button_Callback.To_Marshaller (On_Delete_Affect_Pressed'Access));
      Gtk.Button.Gtk_New
        (Update_Resources_Set.Button_Modify_Affect,
         To_String (Lb_Modify (Current_Language)));
      Button_Callback.Connect
        (Update_Resources_Set.Button_Modify_Affect,
         "clicked",
         Button_Callback.To_Marshaller (On_Modify_Affect_Pressed'Access));

      Gtk.Label.Gtk_New
        (Update_Resources_Set.Resource_Task_Label,
         To_String (Lb_Task_Name (Current_Language)));
      Gtk.Label.Gtk_New
        (Update_Resources_Set.Resource_Begin_Label,
         To_String (Lb_Task_Begin (Current_Language)));
      Gtk.Label.Gtk_New
        (Update_Resources_Set.Resource_End_Label,
         To_String (Lb_Task_End (Current_Language)));

      Set_Popdown_Strings
        (Update_Resources_Set.Resource_Affect_Task,
         Update_Resources_Set.Items);
      Set_Editable
        (Get_Entry (Update_Resources_Set.Resource_Affect_Task),
         False);

      Attach_Defaults
        (Update_Resources_Set.Table_Add_Affect,
         Update_Resources_Set.Resource_Task_Label,
         0,
         1,
         0,
         1);
      Attach_Defaults
        (Update_Resources_Set.Table_Add_Affect,
         Update_Resources_Set.Resource_Begin_Label,
         1,
         2,
         0,
         1);
      Attach_Defaults
        (Update_Resources_Set.Table_Add_Affect,
         Update_Resources_Set.Resource_End_Label,
         2,
         3,
         0,
         1);
      Attach_Defaults
        (Update_Resources_Set.Table_Add_Affect,
         Update_Resources_Set.Resource_Affect_Task,
         0,
         1,
         1,
         2);
      Attach_Defaults
        (Update_Resources_Set.Table_Add_Affect,
         Update_Resources_Set.Resource_Affect_Begin,
         1,
         2,
         1,
         2);
      Attach_Defaults
        (Update_Resources_Set.Table_Add_Affect,
         Update_Resources_Set.Resource_Affect_End,
         2,
         3,
         1,
         2);
      Attach_Defaults
        (Update_Resources_Set.Table_Add_Affect,
         Update_Resources_Set.Button_Add_Affect,
         0,
         1,
         2,
         3);
      Attach_Defaults
        (Update_Resources_Set.Table_Add_Affect,
         Update_Resources_Set.Button_Modify_Affect,
         1,
         2,
         2,
         3);
      Attach_Defaults
        (Update_Resources_Set.Table_Add_Affect,
         Update_Resources_Set.Button_Delete_Affect,
         2,
         3,
         2,
         3);

      -- The List of affected task
      --
      Gtk.Scrolled_Window.Gtk_New
        (Update_Resources_Set.Scroll_Affected_Task_Lists,
         null,
         null);
      Add
        (Update_Resources_Set.Vbox_Update,
         Update_Resources_Set.Scroll_Affected_Task_Lists);
      Update_Resources_Set.Affect_List_Titles :=
        To_String (Lb_Task (Current_Language)) +
        To_String (Lb_Task_Begin (Current_Language)) +
        To_String (Lb_Task_End (Current_Language));
      Gtk.Clist.Gtk_New
        (Update_Resources_Set.Affected_Task_Lists,
         3,
         Update_Resources_Set.Affect_List_Titles); -- 3 Columns
      Set_Column_Width (Update_Resources_Set.Affected_Task_Lists, 0, 100);
      Set_Usize (Update_Resources_Set.Affected_Task_Lists, 100, 200);
      Add
        (Update_Resources_Set.Scroll_Affected_Task_Lists,
         Update_Resources_Set.Affected_Task_Lists);

      -----------------------------------------------------
      -- Button set
      -----------------------------------------------------

      Gtk_New (Update_Resources_Set.Hbutton_Box_Modify);
      Set_Spacing (Update_Resources_Set.Hbutton_Box_Modify, 10);
      Set_Layout
        (Update_Resources_Set.Hbutton_Box_Modify,
         Buttonbox_Default_Style);
      Set_Child_Size (Update_Resources_Set.Hbutton_Box_Modify, 20, 20);
      Set_Child_Ipadding (Update_Resources_Set.Hbutton_Box_Modify, 7, 0);
      Pack_Start
        (Update_Resources_Set.Vbox_Update,
         Update_Resources_Set.Hbutton_Box_Modify,
         False,
         True,
         5);

      Gtk.Button.Gtk_New
        (Update_Resources_Set.Close_Button,
         To_String (Lb_Close (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_Resources_Set.Cancel_Button,
         To_String (Lb_Cancel (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_Resources_Set.Advanced_Button,
         To_String (Lb_Advanced (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_Resources_Set.Delete_Button,
         To_String (Lb_Delete (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_Resources_Set.Modify_Button,
         To_String (Lb_Modify (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_Resources_Set.Add_Button,
         To_String (Lb_Add (Current_Language)));

      Add
        (Update_Resources_Set.Hbutton_Box_Modify,
         Update_Resources_Set.Close_Button);
      Add
        (Update_Resources_Set.Hbutton_Box_Modify,
         Update_Resources_Set.Cancel_Button);
      Add
        (Update_Resources_Set.Hbutton_Box_Modify,
         Update_Resources_Set.Advanced_Button);
      Add
        (Update_Resources_Set.Hbutton_Box_Modify,
         Update_Resources_Set.Delete_Button);
      Add
        (Update_Resources_Set.Hbutton_Box_Modify,
         Update_Resources_Set.Modify_Button);
      Add
        (Update_Resources_Set.Hbutton_Box_Modify,
         Update_Resources_Set.Add_Button);

      -----------------------------------------------------
      -- Spreadsheet for resource information
      -----------------------------------------------------

      Gtk.Scrolled_Window.Gtk_New
        (Update_Resources_Set.Scroll_List,
         null,
         null);
      Add (Update_Resources_Set.Vbox_List, Update_Resources_Set.Scroll_List);

      Update_Resources_Set.List_Titles :=
        To_String (Lb_Name (Current_Language)) +
        To_String (Lb_Processor (Current_Language)) +
        To_String (Lb_Protocol (Current_Language));

      Gtk.Clist.Gtk_New
        (Update_Resources_Set.Resources_List,
         3,
         Update_Resources_Set.List_Titles);
      Set_Usize (Update_Resources_Set.Resources_List, 475, 200);
      Set_Sort_Column (Update_Resources_Set.Resources_List, 0);
      Set_Sort_Type (Update_Resources_Set.Resources_List, Ascending);
      Set_Auto_Sort (Update_Resources_Set.Resources_List, True);
      Add
        (Update_Resources_Set.Scroll_List,
         Update_Resources_Set.Resources_List);

      Show_Resources (Update_Resources_Set);

      -- Select resource from spreadsheet activation
      --
      List_Callback.Connect
        (Update_Resources_Set.Resources_List,
         "select_row",
         List_Callback.To_Marshaller (On_Click_List'Access));

      Button_Callback.Connect
        (Update_Resources_Set.Modify_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Modify_Pressed'Access));
      Button_Callback.Connect
        (Update_Resources_Set.Cancel_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'Access));
      Button_Callback.Connect
        (Update_Resources_Set.Advanced_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Advanced_Pressed'Access));
      Button_Callback.Connect
        (Update_Resources_Set.Close_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Close_Pressed'Access));
      Button_Callback.Connect
        (Update_Resources_Set.Add_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Add_Pressed'Access));
      Button_Callback.Connect
        (Update_Resources_Set.Delete_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Delete_Pressed'Access));
      List_Callback.Connect
        (Update_Resources_Set.Affected_Task_Lists,
         "select_row",
         List_Callback.To_Marshaller (On_Affected_Task_Lists_Click'Access));

      Change_Cpu := Get_Entry (Update_Resources_Set.Address_Space_Menu);
      Combo_Callback.Connect
        (Change_Cpu,
         "changed",
         Combo_Callback.To_Marshaller (On_Change_Cpu'Access));

   end Initialize;

end Update_Resources_Set_Pkg;
