------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Gtk.Pixmap;            use Gtk.Pixmap;
with Gtk.Separator;         use Gtk.Separator;
with Gtk;                   use Gtk;
with Gtk.Table;             use Gtk.Table;
with Gtk.GEntry;            use Gtk.GEntry;
with Gtk.Hbutton_Box;       use Gtk.Hbutton_Box;
with Gtk.Scrolled_Window;   use Gtk.Scrolled_Window;
with Gtk.Widget;            use Gtk.Widget;
with Resource_Set;          use Resource_Set;
use Resource_Set.Generic_Resource_Set;
with Processor_Set;         use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Gtk.Clist;             use Gtk.Clist;
with Gtkada.Types;          use Gtkada.Types;
with Gtk.Frame;             use Gtk.Frame;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Gtk.Window;            use Gtk.Window;
with Gtk.Combo;             use Gtk.Combo;
with Gtk.Enums;             use Gtk.Enums;
with Gtk.Button;            use Gtk.Button;
with Gtk.Label;             use Gtk.Label;
with Gtk.Box;               use Gtk.Box;
with Gtk.Option_Menu;       use Gtk.Option_Menu;
with Resources;             use Resources;
use Resources.Task_Lists;
with Systems;               use Systems;

package Update_Resources_Set_Pkg is

   type Update_Resources_Set_Record is new Gtk_Window_Record with private;
   type Update_Resources_Set_Access is access all Update_Resources_Set_Record'
     Class;

   procedure Gtk_New
     (Update_Resources_Set : out Update_Resources_Set_Access;
      Title                : in Unbounded_String);
   procedure Initialize
     (Update_Resources_Set : access Update_Resources_Set_Record'Class;
      Title                : in Unbounded_String);
   procedure Show_Advanced
     (Update_Resources_Set : in Update_Resources_Set_Access;
      Advanced             : in Boolean);

   Update_Resources_Set : Update_Resources_Set_Access;
   Parent               : Unbounded_String;

private

   type Update_Resources_Set_Record is new Gtk_Window_Record with record
      Scroll : Gtk_Scrolled_Window;

      Resources_List           : Gtk.Clist.Gtk_Clist;
      Resource_Type_Menu       : Gtk_Option_Menu;
      Clist_Resource_Type_Menu : Widget_SList.GSlist;
      Processor_Menu           : Gtk_Option_Menu;
      Clist_Processor_Menu     : Widget_SList.GSlist;
      Resource_Items           : String_List.Glist;

      Current_Resource : Unbounded_String;

      Lab : Gtk_Label;

      Frame_Resource : Gtk.Frame.Gtk_Frame;
      Resource_Table : Gtk.Table.Gtk_Table;

      Vbox_Main     : Gtk.Box.Gtk_Box;
      Hbox_Resource : Gtk.Box.Gtk_Box;
      Hbox_Confirm  : Gtk.Box.Gtk_Box;
      Vbox_List     : Gtk.Box.Gtk_Box;
      Hbox_Modify   : Gtk.Box.Gtk_Box;
      Vbox_Update   : Gtk.Box.Gtk_Box;

      Hbutton_Box_Modify : Gtk_Hbutton_Box;

      Modify_Button        : Gtk_Button;
      Advanced_Button      : Gtk_Button;
      Close_Button         : Gtk_Button;
      Cancel_Button        : Gtk_Button;
      Add_Button           : Gtk_Button;
      Delete_Button        : Gtk_Button;
      Button_Add_Affect    : Gtk.Button.Gtk_Button;
      Button_Modify_Affect : Gtk.Button.Gtk_Button;
      Button_Delete_Affect : Gtk.Button.Gtk_Button;

      Hseparator1 : Gtk_Hseparator;
      Vseparator  : Gtk_Vseparator;
      Vseparator2 : Gtk_Vseparator;

      Child_Page : Gtk_Frame;
      Label_Box  : Gtk_Box;
      Menu_Box   : Gtk_Box;
      Pixmap     : Gtk_Pixmap;
      Vbox       : Gtk_Box;
      Button     : Gtk_Button;

      Undo : System;

      Scroll_List : Gtk_Scrolled_Window;
      List_Titles : Chars_Ptr_Array (1 .. 3);

      Advanced : Boolean := False;

      Name                     : Gtk.GEntry.Gtk_Entry;
      State                    : Gtk.GEntry.Gtk_Entry;
      Protocol                 : Gtk.Combo.Gtk_Combo;
      Cpu_Name                 : Gtk_Entry;
      Address_Space_Menu       : Gtk.Combo.Gtk_Combo;
      Clist_Address_Space_Menu : String_List.Glist;

      Affected_Task_Lists        : Gtk.Clist.Gtk_Clist;
      Scroll_Affected_Task_Lists : Gtk_Scrolled_Window;

      Hbox_Add_Cancel       : Gtk.Box.Gtk_Box;
      Hbox_Add_Affect       : Gtk.Box.Gtk_Box;
      Hbox_Table_Name_State : Gtk.Box.Gtk_Box;

      Table_Name_State : Gtk.Table.Gtk_Table;
      Table_Add_Affect : Gtk.Table.Gtk_Table;

      Resource_Affect_Task  : Gtk_Combo;
      Items                 : String_List.Glist;
      Resource_Affect_Begin : Gtk.GEntry.Gtk_Entry;
      Resource_Affect_End   : Gtk.GEntry.Gtk_Entry;

      Resource_Name_Label          : Gtk.Label.Gtk_Label;
      Resource_State_Label         : Gtk.Label.Gtk_Label;
      Resource_Task_Label          : Gtk.Label.Gtk_Label;
      Resource_Begin_Label         : Gtk.Label.Gtk_Label;
      Resource_End_Label           : Gtk.Label.Gtk_Label;
      Resource_Combo_Label         : Gtk.Label.Gtk_Label;
      Resource_Cpu_Label           : Gtk.Label.Gtk_Label;
      Resource_Address_Space_Label : Gtk.Label.Gtk_Label;
      Frame_Name_State             : Gtk.Frame.Gtk_Frame;
      Frame_Affect                 : Gtk.Frame.Gtk_Frame;
      Affect_List_Titles           : Chars_Ptr_Array (1 .. 3);

      Task_Lists_Share : Resources.Task_Lists_Table;
   end record;

end Update_Resources_Set_Pkg;
