------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Glib;                           use Glib;
with Gdk.Color;                      use Gdk.Color;
with Mmi_Tools;                      use Mmi_Tools;
with Callbacks_Cheddar;              use Callbacks_Cheddar;
with Update_Tasks_Set_Pkg.Callbacks; use Update_Tasks_Set_Pkg.Callbacks;
with Address_Spaces;                 use Address_Spaces;
with Address_Space_Set;              use Address_Space_Set;
use Address_Space_Set.Generic_Address_Space_Set;
with Unbounded_Strings;              use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;
with Project;                        use Project;
with Translate;                      use Translate;
with Framework_Config;               use Framework_Config;
with Xpm;                            use Xpm;
with Parameters;                     use Parameters;
use Parameters.User_Defined_Parameters_Table_Package;
with Offsets;                        use Offsets;
use Offsets.Offsets_Table_package;

package body Update_Tasks_Set_Pkg is

   procedure Gtk_New
     (Update_Tasks_Set : out Update_Tasks_Set_Access;
      Title            : in Unbounded_String)
   is
   begin
      Update_Tasks_Set := new Update_Tasks_Set_Record;
      Update_Tasks_Set_Pkg.Initialize (Update_Tasks_Set, Title);
   end Gtk_New;

   procedure Show_Advanced
     (Update_Tasks_Set : in Update_Tasks_Set_Access;
      Advanced         : in Boolean)
   is

   begin

      if not Advanced then
         Hide (Update_Tasks_Set.Vbox_List);
         Hide (Update_Tasks_Set.Delete_Button);
         Hide (Update_Tasks_Set.Modify_Button);
         Show (Update_Tasks_Set.Add_Button);
         Show (Update_Tasks_Set.Close_Button);
         Show (Update_Tasks_Set.Cancel_Button);
         Show (Update_Tasks_Set.Advanced_Button);

         Show (Update_Tasks_Set.Vbox_Main);
         Show (Update_Tasks_Set.Hbox_Task);
         Show_All (Update_Tasks_Set.Vbox_Update);
         Show (Update_Tasks_Set.Hbutton_Box_Modify);
         Show_All (Update_Tasks_Set.Frame_Task);

      else
         Show_All (Update_Tasks_Set.Vbox_Main);
      end if;
      Show (Update_Tasks_Set);

   end Show_Advanced;

   procedure Initialize
     (Update_Tasks_Set : access Update_Tasks_Set_Record'Class;
      Title            : in Unbounded_String)
   is

      Change_Cpu                           : Gtk_Entry;
      Address_Space_Iterator               : Address_Spaces_Iterator;
      Parameter_Items                      : Strings_Table;
      Type_Items                           : Strings_Table;
      Address_Space_Items                  : Strings_Table;
      Policy_Items                         : Strings_Table;
      First_Address_Space, A_Address_Space : Address_Space_Ptr;

   begin

      Initialize (Update_Tasks_Set.Undo);
      Duplicate (Sys, Update_Tasks_Set.Undo);
      Update_Tasks_Set.Current_Task := Empty_String;

      Gtk.Window.Initialize (Update_Tasks_Set, Window_Toplevel);
      Set_Policy (Update_Tasks_Set, True, True, True);
      Set_Title (Update_Tasks_Set, To_String (Title));
      Set_Position (Update_Tasks_Set, Win_Pos_Center_Always);
      Set_Modal (Update_Tasks_Set, True);
      Grab_Focus (Update_Tasks_Set);

      Gtk.Box.Gtk_New_Vbox (Update_Tasks_Set.Vbox_Main, False, 10);
      Set_Border_Width (Update_Tasks_Set.Vbox_Main, 10);
      Add (Update_Tasks_Set, Update_Tasks_Set.Vbox_Main);

      Gtk.Box.Gtk_New_Hbox (Update_Tasks_Set.Hbox_Task, False, 10);
      Add (Update_Tasks_Set.Vbox_Main, Update_Tasks_Set.Hbox_Task);

      Gtk.Box.Gtk_New_Vbox (Update_Tasks_Set.Vbox_Update, False, 10);
      Add (Update_Tasks_Set.Hbox_Task, Update_Tasks_Set.Vbox_Update);

      Gtk.Box.Gtk_New_Vbox (Update_Tasks_Set.Vbox_List, False, 10);
      Add (Update_Tasks_Set.Hbox_Task, Update_Tasks_Set.Vbox_List);

      Gtk.Frame.Gtk_New (Update_Tasks_Set.Frame_Task, "");
      Add (Update_Tasks_Set.Vbox_Update, Update_Tasks_Set.Frame_Task);

      -- Button panel
      --
      Gtk_New (Update_Tasks_Set.Hbutton_Box_Modify);
      Set_Spacing (Update_Tasks_Set.Hbutton_Box_Modify, 10);
      Set_Layout
        (Update_Tasks_Set.Hbutton_Box_Modify,
         Buttonbox_Default_Style);
      Set_Child_Size (Update_Tasks_Set.Hbutton_Box_Modify, 20, 20);
      Set_Child_Ipadding (Update_Tasks_Set.Hbutton_Box_Modify, 7, 0);
      Pack_Start
        (Update_Tasks_Set.Vbox_Update,
         Update_Tasks_Set.Hbutton_Box_Modify,
         False,
         True,
         5);

      -- Create Notebook
      --
      Gtk_New (Update_Tasks_Set.Notebook);
      Set_Tab_Pos (Update_Tasks_Set.Notebook, Pos_Top);
      Add (Update_Tasks_Set.Frame_Task, Update_Tasks_Set.Notebook);
      Set_Border_Width (Update_Tasks_Set.Notebook, 10);
      Realize (Update_Tasks_Set.Notebook);

      Create_From_Xpm_D
        (Update_Tasks_Set.Book_Opened,
         Get_Window (Update_Tasks_Set.Notebook),
         Update_Tasks_Set.Book_Opened_Mask,
         Null_Color,
         Book_Opened_Xpm);
      Create_From_Xpm_D
        (Update_Tasks_Set.Book_Closed,
         Get_Window (Update_Tasks_Set.Notebook),
         Update_Tasks_Set.Book_Closed_Mask,
         Null_Color,
         Book_Closed_Xpm);

      -- Create Notebook pages
      --

      -- Main page
      --
      Gtk_New
        (Update_Tasks_Set.Child_Page,
         To_String (Lb_Main (Current_Language)));
      Gtk.Table.Gtk_New (Update_Tasks_Set.Task_Table, 2, 15, False);
      Set_Border_Width (Update_Tasks_Set.Task_Table, 5);
      Add (Update_Tasks_Set.Child_Page, Update_Tasks_Set.Task_Table);
      Gtk_New_Hbox (Update_Tasks_Set.Label_Box, False, 0);
      Gtk_New
        (Update_Tasks_Set.Pixmap,
         Update_Tasks_Set.Book_Opened,
         Update_Tasks_Set.Book_Opened_Mask);
      Pack_Start
        (Update_Tasks_Set.Label_Box,
         Update_Tasks_Set.Pixmap,
         False,
         True,
         0);
      Set_Padding (Update_Tasks_Set.Pixmap, 3, 1);

      Gtk_New (Update_Tasks_Set.Lab, To_String (Lb_Main (Current_Language)));
      Pack_Start
        (Update_Tasks_Set.Label_Box,
         Update_Tasks_Set.Lab,
         False,
         True,
         0);
      Show_All (Update_Tasks_Set.Label_Box);

      Gtk_New_Vbox (Update_Tasks_Set.Menu_Box, False, 0);
      Gtk_New
        (Update_Tasks_Set.Pixmap,
         Update_Tasks_Set.Book_Opened,
         Update_Tasks_Set.Book_Opened_Mask);
      Pack_Start
        (Update_Tasks_Set.Menu_Box,
         Update_Tasks_Set.Pixmap,
         False,
         True,
         0);
      Set_Padding (Update_Tasks_Set.Pixmap, 3, 1);

      Gtk_New (Update_Tasks_Set.Lab, To_String (Lb_Main (Current_Language)));
      Pack_Start
        (Update_Tasks_Set.Menu_Box,
         Update_Tasks_Set.Lab,
         False,
         True,
         0);
      Show_All (Update_Tasks_Set.Menu_Box);

      Append_Page_Menu
        (Update_Tasks_Set.Notebook,
         Update_Tasks_Set.Child_Page,
         Update_Tasks_Set.Label_Box,
         Update_Tasks_Set.Menu_Box);

      -- User defined parameter page
      --
      Gtk_New
        (Update_Tasks_Set.Child_Page,
         To_String (Lb_User_Defined_Parameter (Current_Language)));
      Gtk.Table.Gtk_New (Update_Tasks_Set.Parameter_Table, 2, 15, False);
      Set_Border_Width (Update_Tasks_Set.Parameter_Table, 5);
      Add (Update_Tasks_Set.Child_Page, Update_Tasks_Set.Parameter_Table);
      Gtk_New_Hbox (Update_Tasks_Set.Label_Box, False, 0);
      Gtk_New
        (Update_Tasks_Set.Pixmap,
         Update_Tasks_Set.Book_Closed,
         Update_Tasks_Set.Book_Closed_Mask);
      Pack_Start
        (Update_Tasks_Set.Label_Box,
         Update_Tasks_Set.Pixmap,
         False,
         True,
         0);
      Set_Padding (Update_Tasks_Set.Pixmap, 3, 1);

      Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_User_Defined_Parameter (Current_Language)));
      Pack_Start
        (Update_Tasks_Set.Label_Box,
         Update_Tasks_Set.Lab,
         False,
         True,
         0);
      Show_All (Update_Tasks_Set.Label_Box);

      Gtk_New_Vbox (Update_Tasks_Set.Menu_Box, False, 0);
      Gtk_New
        (Update_Tasks_Set.Pixmap,
         Update_Tasks_Set.Book_Closed,
         Update_Tasks_Set.Book_Closed_Mask);
      Pack_Start
        (Update_Tasks_Set.Menu_Box,
         Update_Tasks_Set.Pixmap,
         False,
         True,
         0);
      Set_Padding (Update_Tasks_Set.Pixmap, 3, 1);

      Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_User_Defined_Parameter (Current_Language)));
      Pack_Start
        (Update_Tasks_Set.Menu_Box,
         Update_Tasks_Set.Lab,
         False,
         True,
         0);
      Show_All (Update_Tasks_Set.Menu_Box);

      Append_Page_Menu
        (Update_Tasks_Set.Notebook,
         Update_Tasks_Set.Child_Page,
         Update_Tasks_Set.Label_Box,
         Update_Tasks_Set.Menu_Box);

      -- Offset page
      --
      Gtk_New
        (Update_Tasks_Set.Child_Page,
         To_String (Lb_Offsets (Current_Language)));
      Gtk.Table.Gtk_New (Update_Tasks_Set.Offset_Table, 2, 15, False);
      Set_Border_Width (Update_Tasks_Set.Offset_Table, 5);
      Add (Update_Tasks_Set.Child_Page, Update_Tasks_Set.Offset_Table);
      Gtk_New_Hbox (Update_Tasks_Set.Label_Box, False, 0);
      Gtk_New
        (Update_Tasks_Set.Pixmap,
         Update_Tasks_Set.Book_Closed,
         Update_Tasks_Set.Book_Closed_Mask);
      Pack_Start
        (Update_Tasks_Set.Label_Box,
         Update_Tasks_Set.Pixmap,
         False,
         True,
         0);
      Set_Padding (Update_Tasks_Set.Pixmap, 3, 1);

      Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Offsets (Current_Language)));
      Pack_Start
        (Update_Tasks_Set.Label_Box,
         Update_Tasks_Set.Lab,
         False,
         True,
         0);
      Show_All (Update_Tasks_Set.Label_Box);

      Gtk_New_Vbox (Update_Tasks_Set.Menu_Box, False, 0);
      Gtk_New
        (Update_Tasks_Set.Pixmap,
         Update_Tasks_Set.Book_Closed,
         Update_Tasks_Set.Book_Closed_Mask);
      Pack_Start
        (Update_Tasks_Set.Menu_Box,
         Update_Tasks_Set.Pixmap,
         False,
         True,
         0);
      Set_Padding (Update_Tasks_Set.Pixmap, 3, 1);

      Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Offsets (Current_Language)));
      Pack_Start
        (Update_Tasks_Set.Menu_Box,
         Update_Tasks_Set.Lab,
         False,
         True,
         0);
      Show_All (Update_Tasks_Set.Menu_Box);

      Append_Page_Menu
        (Update_Tasks_Set.Notebook,
         Update_Tasks_Set.Child_Page,
         Update_Tasks_Set.Label_Box,
         Update_Tasks_Set.Menu_Box);

      -- Frame listings

      -- Begin Main page listing
      --

      Gtk.Label.Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Name (Current_Language)) & To_String (Lb_Colon));
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Lab,
         0,
         1,
         0,
         1);

      Gtk.Label.Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Task_Type (Current_Language)) &
         To_String (Lb_Colon));
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Lab,
         0,
         1,
         1,
         2);

      Gtk.Label.Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Address_Space (Current_Language)) &
         To_String (Lb_Colon));
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Lab,
         0,
         1,
         2,
         3);

      Gtk.Label.Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Processor (Current_Language)) &
         To_String (Lb_Colon));
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Lab,
         0,
         1,
         3,
         4);

      Gtk.Label.Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Policy (Current_Language)) & To_String (Lb_Colon));
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Lab,
         0,
         1,
         4,
         5);

      Gtk.Label.Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Priority (Current_Language)) & To_String (Lb_Colon));
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Lab,
         0,
         1,
         5,
         6);

      Gtk.Label.Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Capacity (Current_Language)) & To_String (Lb_Colon));
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Lab,
         0,
         1,
         6,
         7);

      Gtk.Label.Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Jitter (Current_Language)) & To_String (Lb_Colon));
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Lab,
         0,
         1,
         7,
         8);

      Gtk.Label.Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Deadline (Current_Language)) & To_String (Lb_Colon));
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Lab,
         0,
         1,
         8,
         9);

      Gtk.Label.Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Period (Current_Language)) & To_String (Lb_Colon));
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Lab,
         0,
         1,
         9,
         10);

      Gtk.Label.Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Start_Time (Current_Language)) &
         To_String (Lb_Colon));
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Lab,
         0,
         1,
         10,
         11);

      Gtk.Label.Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Blocking_Time (Current_Language)) &
         To_String (Lb_Colon));
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Lab,
         0,
         1,
         11,
         12);

      Gtk.Label.Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Criticality (Current_Language)) &
         To_String (Lb_Colon));
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Lab,
         0,
         1,
         12,
         13);

      Gtk.Label.Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Activation_Rule (Current_Language)) &
         To_String (Lb_Colon));
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Lab,
         0,
         1,
         13,
         14);

      Gtk.Label.Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Stack_Memory_Size (Current_Language)) &
         To_String (Lb_Colon));
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Lab,
         0,
         1,
         14,
         15);

      Gtk.Label.Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Text_Memory_Size (Current_Language)) &
         To_String (Lb_Colon));
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Lab,
         0,
         1,
         15,
         16);

      Gtk.Label.Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Seed (Current_Language)) & To_String (Lb_Colon));
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Lab,
         0,
         1,
         16,
         17);

      ---

      Gtk.Label.Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Activations (Current_Language)) &
         To_String (Lb_Colon));
      Attach_Defaults
        (Update_Tasks_Set.Offset_Table,
         Update_Tasks_Set.Lab,
         0,
         1,
         12,
         13);

      Gtk.Label.Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Values (Current_Language)) & To_String (Lb_Colon));
      Attach_Defaults
        (Update_Tasks_Set.Offset_Table,
         Update_Tasks_Set.Lab,
         1,
         2,
         12,
         13);

      -- Type, address space,  processor & policy menu
      --

      Initialize (Type_Items);
      Add (Type_Items, To_Unbounded_String ("Periodic"));
      Add (Type_Items, To_Unbounded_String ("Aperiodic"));
      Add (Type_Items, To_Unbounded_String ("Sporadic"));
      Add (Type_Items, To_Unbounded_String ("Poisson"));
      Add (Type_Items, To_Unbounded_String ("User-defined"));
      Update_Tasks_Set.Clist_Task_Type_Menu := Widget_SList.Null_List;

      Build_Option_Menu
        (Update_Tasks_Set.Task_Type_Menu,
         Update_Tasks_Set.Clist_Task_Type_Menu,
         Type_Items,
         0,
         null);

      Initialize (Address_Space_Items);
      Reset_Iterator (Sys.Address_Spaces, Address_Space_Iterator);
      Gtk.Combo.Gtk_New (Update_Tasks_Set.Address_Space_Menu);
      Set_Editable (Get_Entry (Update_Tasks_Set.Address_Space_Menu), False);

      loop
         Current_Element
           (Sys.Address_Spaces,
            A_Address_Space,
            Address_Space_Iterator);
         String_List.Append
           (Update_Tasks_Set.Clist_Address_Space_Menu,
            To_String (A_Address_Space.Name));
         if First_Address_Space = null then
            First_Address_Space := A_Address_Space;
         end if;

         exit when Is_Last_Element
                      (Sys.Address_Spaces,
                       Address_Space_Iterator);

         Next_Element (Sys.Address_Spaces, Address_Space_Iterator);
      end loop;
      Set_Popdown_Strings
        (Update_Tasks_Set.Address_Space_Menu,
         Update_Tasks_Set.Clist_Address_Space_Menu);

      Initialize (Policy_Items);
      Add (Policy_Items, To_Unbounded_String ("SCHED_FIFO"));
      Add (Policy_Items, To_Unbounded_String ("SCHED_RR"));
      Add (Policy_Items, To_Unbounded_String ("SCHED_OTHERS"));
      Update_Tasks_Set.Clist_Policy_Menu := Widget_SList.Null_List;
      Build_Option_Menu
        (Update_Tasks_Set.Policy_Menu,
         Update_Tasks_Set.Clist_Policy_Menu,
         Policy_Items,
         -1,
         null);

      Gtk.GEntry.Gtk_New (Update_Tasks_Set.Name, 50);
      Set_Editable (Update_Tasks_Set.Name, True);
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Name,
         1,
         2,
         0,
         1);

      Gtk.Option_Menu.Gtk_New (Update_Tasks_Set.Task_Type_Menu);
      Build_Option_Menu
        (Update_Tasks_Set.Task_Type_Menu,
         Update_Tasks_Set.Clist_Task_Type_Menu,
         Type_Items,
         0,
         null);
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Task_Type_Menu,
         1,
         2,
         1,
         2);

      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Address_Space_Menu,
         1,
         2,
         2,
         3);

      Gtk.GEntry.Gtk_New (Update_Tasks_Set.Cpu_Name, 50);
      Set_Editable (Update_Tasks_Set.Cpu_Name, False);
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Cpu_Name,
         1,
         2,
         3,
         4);
      Gtk.GEntry.Set_Text
        (Update_Tasks_Set.Cpu_Name,
         To_String (First_Address_Space.Cpu_Name));

      Gtk.Option_Menu.Gtk_New (Update_Tasks_Set.Policy_Menu);
      Build_Option_Menu
        (Update_Tasks_Set.Policy_Menu,
         Update_Tasks_Set.Clist_Policy_Menu,
         Policy_Items,
         0,
         null);
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Policy_Menu,
         1,
         2,
         4,
         5);

      Gtk.GEntry.Gtk_New (Update_Tasks_Set.Priority, 50);
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Priority,
         1,
         2,
         5,
         6);
      Gtk.GEntry.Set_Text (Update_Tasks_Set.Priority, "1");

      Gtk.GEntry.Gtk_New (Update_Tasks_Set.Capacity, 50);
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Capacity,
         1,
         2,
         6,
         7);

      Gtk.GEntry.Gtk_New (Update_Tasks_Set.Jitter, 50);
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Jitter,
         1,
         2,
         7,
         8);

      Gtk.GEntry.Gtk_New (Update_Tasks_Set.Deadline, 50);
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Deadline,
         1,
         2,
         8,
         9);

      Gtk.GEntry.Gtk_New (Update_Tasks_Set.Period, 50);
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Period,
         1,
         2,
         9,
         10);

      Gtk.GEntry.Gtk_New (Update_Tasks_Set.Start_Time, 50);
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Start_Time,
         1,
         2,
         10,
         11);

      Gtk.GEntry.Gtk_New (Update_Tasks_Set.Blocking_Time, 50);
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Blocking_Time,
         1,
         2,
         11,
         12);

      Gtk.GEntry.Gtk_New (Update_Tasks_Set.Criticality, 50);
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Criticality,
         1,
         2,
         12,
         13);

      Gtk.GEntry.Gtk_New (Update_Tasks_Set.Activation_Rule, 50);
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Activation_Rule,
         1,
         2,
         13,
         14);

      Gtk.GEntry.Gtk_New (Update_Tasks_Set.Stack_Memory_Size, 50);
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Stack_Memory_Size,
         1,
         2,
         14,
         15);

      Gtk.GEntry.Gtk_New (Update_Tasks_Set.Text_Memory_Size, 50);
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Text_Memory_Size,
         1,
         2,
         15,
         16);

      -- Seed information
      --
      Gtk.Box.Gtk_New_Hbox (Update_Tasks_Set.Hbox_Seed, False, 10);
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Hbox_Seed,
         1,
         2,
         16,
         17);

      Gtk_New
        (Update_Tasks_Set.Seed_Predictable,
         Update_Tasks_Set.Seed_Group,
         To_String (Lb_Predictable (Current_Language)));
      Update_Tasks_Set.Seed_Group :=
         Group (Update_Tasks_Set.Seed_Predictable);
      Add (Update_Tasks_Set.Hbox_Seed, Update_Tasks_Set.Seed_Predictable);

      Gtk_New
        (Update_Tasks_Set.Seed_Unpredictable,
         Update_Tasks_Set.Seed_Group,
         To_String (Lb_Unpredictable (Current_Language)));
      Update_Tasks_Set.Seed_Group :=
         Group (Update_Tasks_Set.Seed_Unpredictable);
      Add (Update_Tasks_Set.Hbox_Seed, Update_Tasks_Set.Seed_Unpredictable);

      Gtk.Box.Gtk_New_Hbox (Update_Tasks_Set.Hbox_Seed, False, 10);
      Attach_Defaults
        (Update_Tasks_Set.Task_Table,
         Update_Tasks_Set.Hbox_Seed,
         1,
         2,
         17,
         18);

      Gtk.GEntry.Gtk_New (Update_Tasks_Set.Seed_Value, 50);
      Add (Update_Tasks_Set.Hbox_Seed, Update_Tasks_Set.Seed_Value);

      Gtk.Button.Gtk_New
        (Update_Tasks_Set.Seed_Button,
         To_String (Lb_Seed (Current_Language)));
      Add (Update_Tasks_Set.Hbox_Seed, Update_Tasks_Set.Seed_Button);

      --

      -- User defined parameters
      --

      Show_All (Update_Tasks_Set.Child_Page);

      Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Name (Current_Language) & Lb_Colon));
      Set_Alignment (Update_Tasks_Set.Lab, 0.5, 0.5);
      Set_Padding (Update_Tasks_Set.Lab, 0, 0);
      Set_Justify (Update_Tasks_Set.Lab, Justify_Center);
      Set_Line_Wrap (Update_Tasks_Set.Lab, False);
      Attach
        (Update_Tasks_Set.Parameter_Table,
         Update_Tasks_Set.Lab,
         0,
         1,
         0,
         1,
         0,
         0,
         0,
         0);

      Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Role_Type (Current_Language) & Lb_Colon));
      Set_Alignment (Update_Tasks_Set.Lab, 0.5, 0.5);
      Set_Padding (Update_Tasks_Set.Lab, 0, 0);
      Set_Justify (Update_Tasks_Set.Lab, Justify_Center);
      Set_Line_Wrap (Update_Tasks_Set.Lab, False);
      Attach
        (Update_Tasks_Set.Parameter_Table,
         Update_Tasks_Set.Lab,
         1,
         2,
         0,
         1,
         0,
         0,
         0,
         0);

      Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Value (Current_Language) & Lb_Colon));
      Set_Alignment (Update_Tasks_Set.Lab, 0.5, 0.5);
      Set_Padding (Update_Tasks_Set.Lab, 0, 0);
      Set_Justify (Update_Tasks_Set.Lab, Justify_Center);
      Set_Line_Wrap (Update_Tasks_Set.Lab, False);
      Attach
        (Update_Tasks_Set.Parameter_Table,
         Update_Tasks_Set.Lab,
         2,
         3,
         0,
         1,
         0,
         0,
         0,
         0);

      for I in  0 .. User_Defined_Parameters_Range'Last - 1 loop

         Gtk_New (Update_Tasks_Set.Param (I).Entry_Name);
         Set_Editable (Update_Tasks_Set.Param (I).Entry_Name, True);
         Set_Max_Length (Update_Tasks_Set.Param (I).Entry_Name, 0);
         Set_Text (Update_Tasks_Set.Param (I).Entry_Name, "");
         Set_Visibility (Update_Tasks_Set.Param (I).Entry_Name, True);
         Attach
           (Update_Tasks_Set.Parameter_Table,
            Update_Tasks_Set.Param (I).Entry_Name,
            0,
            1,
            1 + Guint (I),
            2 + Guint (I),
            Expand or Fill,
            0,
            0,
            0);

         Initialize (Parameter_Items);
         Add (Parameter_Items, To_Unbounded_String ("Integer"));
         Add (Parameter_Items, To_Unbounded_String ("Double"));
         Add (Parameter_Items, To_Unbounded_String ("String"));
         Add (Parameter_Items, To_Unbounded_String ("Boolean"));
         Update_Tasks_Set.Param (I).Clist_Parameter_Menu :=
           Widget_SList.Null_List;
         Build_Option_Menu
           (Update_Tasks_Set.Param (I).Parameter_Menu,
            Update_Tasks_Set.Param (I).Clist_Parameter_Menu,
            Parameter_Items,
            -1,
            null);
         Attach
           (Update_Tasks_Set.Parameter_Table,
            Update_Tasks_Set.Param (I).Parameter_Menu,
            1,
            2,
            1 + Guint (I),
            2 + Guint (I),
            Expand or Fill,
            0,
            0,
            0);

         Gtk_New (Update_Tasks_Set.Param (I).Entry_Value);
         Set_Editable (Update_Tasks_Set.Param (I).Entry_Value, True);
         Set_Max_Length (Update_Tasks_Set.Param (I).Entry_Value, 0);
         Set_Text (Update_Tasks_Set.Param (I).Entry_Value, "");
         Set_Visibility (Update_Tasks_Set.Param (I).Entry_Value, True);
         Attach
           (Update_Tasks_Set.Parameter_Table,
            Update_Tasks_Set.Param (I).Entry_Value,
            2,
            3,
            1 + Guint (I),
            2 + Guint (I),
            Expand or Fill,
            0,
            0,
            0);

      end loop;

      -- Beginning of Offset listing
      --

      Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Activations (Current_Language) & Lb_Colon));
      Attach_Defaults
        (Update_Tasks_Set.Offset_Table,
         Update_Tasks_Set.Lab,
         0,
         1,
         11,
         12);

      Gtk_New
        (Update_Tasks_Set.Lab,
         To_String (Lb_Values (Current_Language) & Lb_Colon));
      Attach_Defaults
        (Update_Tasks_Set.Offset_Table,
         Update_Tasks_Set.Lab,
         1,
         2,
         11,
         12);

      for I in  0 .. Offsets_Range'Last - 1 loop

         Gtk.GEntry.Gtk_New (Update_Tasks_Set.Offset (I).Entry_Act, 50);
         Attach_Defaults
           (Update_Tasks_Set.Offset_Table,
            Update_Tasks_Set.Offset (I).Entry_Act,
            0,
            1,
            12 + Guint (I),
            13 + Guint (I));

         Gtk.GEntry.Gtk_New (Update_Tasks_Set.Offset (I).Entry_Val, 50);
         Attach_Defaults
           (Update_Tasks_Set.Offset_Table,
            Update_Tasks_Set.Offset (I).Entry_Val,
            1,
            2,
            12 + Guint (I),
            13 + Guint (I));

      end loop;

      -- End of Notebook
      --

      Gtk.Button.Gtk_New
        (Update_Tasks_Set.Close_Button,
         To_String (Lb_Close (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_Tasks_Set.Cancel_Button,
         To_String (Lb_Cancel (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_Tasks_Set.Advanced_Button,
         To_String (Lb_Advanced (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_Tasks_Set.Delete_Button,
         To_String (Lb_Delete (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_Tasks_Set.Modify_Button,
         To_String (Lb_Modify (Current_Language)));
      Gtk.Button.Gtk_New
        (Update_Tasks_Set.Add_Button,
         To_String (Lb_Add (Current_Language)));

      Add
        (Update_Tasks_Set.Hbutton_Box_Modify,
         Update_Tasks_Set.Close_Button);
      Add
        (Update_Tasks_Set.Hbutton_Box_Modify,
         Update_Tasks_Set.Cancel_Button);
      Add
        (Update_Tasks_Set.Hbutton_Box_Modify,
         Update_Tasks_Set.Advanced_Button);
      Add
        (Update_Tasks_Set.Hbutton_Box_Modify,
         Update_Tasks_Set.Delete_Button);
      Add
        (Update_Tasks_Set.Hbutton_Box_Modify,
         Update_Tasks_Set.Modify_Button);
      Add (Update_Tasks_Set.Hbutton_Box_Modify, Update_Tasks_Set.Add_Button);

      -- Spreadsheet for Task information
      --
      Gtk.Scrolled_Window.Gtk_New (Update_Tasks_Set.Scroll_List, null, null);
      Add (Update_Tasks_Set.Vbox_List, Update_Tasks_Set.Scroll_List);

      Update_Tasks_Set.List_Titles :=
        To_String (Lb_Name (Current_Language)) +
        To_String (Lb_Task_Type (Current_Language)) +
        To_String (Lb_Processor (Current_Language)) +
        To_String (Lb_Policy (Current_Language)) +
        To_String (Lb_Priority (Current_Language)) +
        To_String (Lb_Capacity (Current_Language)) +
        To_String (Lb_Jitter (Current_Language)) +
        To_String (Lb_Deadline (Current_Language)) +
        To_String (Lb_Period (Current_Language)) +
        To_String (Lb_Start_Time (Current_Language)) +
        To_String (Lb_Blocking_Time (Current_Language)) +
        To_String (Lb_Activation_Rule (Current_Language));

      Gtk.Clist.Gtk_New
        (Update_Tasks_Set.Tasks_List,
         12,
         Update_Tasks_Set.List_Titles);
      Set_Usize (Update_Tasks_Set.Tasks_List, 475, 200);
      Set_Sort_Column (Update_Tasks_Set.Tasks_List, 0);
      Set_Sort_Type (Update_Tasks_Set.Tasks_List, Ascending);
      Set_Auto_Sort (Update_Tasks_Set.Tasks_List, True);
      Add (Update_Tasks_Set.Scroll_List, Update_Tasks_Set.Tasks_List);

      Show_Tasks (Update_Tasks_Set);

      -- Notebook page switching activation
      --
      Two_Notebook_Callback.Connect
        (Update_Tasks_Set.Notebook,
         "switch_page",
         Two_Notebook_Callback.To_Marshaller (Page_Switch'Access),
         Update_Tasks_Set.Notebook);

      -- Select task from spreadsheet activation
      --
      List_Callback.Connect
        (Update_Tasks_Set.Tasks_List,
         "select_row",
         List_Callback.To_Marshaller (On_Click_List'Access));

      Button_Callback.Connect
        (Update_Tasks_Set.Modify_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Modify_Pressed'Access));
      Button_Callback.Connect
        (Update_Tasks_Set.Cancel_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'Access));
      Button_Callback.Connect
        (Update_Tasks_Set.Advanced_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Advanced_Pressed'Access));
      Button_Callback.Connect
        (Update_Tasks_Set.Close_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Close_Pressed'Access));
      Button_Callback.Connect
        (Update_Tasks_Set.Add_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Add_Pressed'Access));
      Button_Callback.Connect
        (Update_Tasks_Set.Delete_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Delete_Pressed'Access));
      Button_Callback.Connect
        (Update_Tasks_Set.Seed_Button,
         "clicked",
         Button_Callback.To_Marshaller (On_Seed_Pressed'Access));

      Change_Cpu := Get_Entry (Update_Tasks_Set.Address_Space_Menu);
      Combo_Callback.Connect
        (Change_Cpu,
         "changed",
         Combo_Callback.To_Marshaller (On_Change_Cpu'Access));

   end Initialize;

end Update_Tasks_Set_Pkg;
