------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

generic
   type Element is private;
   with procedure Put (E : in Element);

package Binary_Trees is

   No_Cell : exception;

   type Cell is private;
   type Binary_Tree is access Cell;

   procedure Add_Left (L : in out Binary_Tree; E : in Element);
   procedure Add_Right (L : in out Binary_Tree; E : in Element);

   function Get_Left (L : in Binary_Tree) return Binary_Tree;
   function Get_Right (L : in Binary_Tree) return Binary_Tree;

   procedure Put (L : in Binary_Tree);

   procedure Delete (L : in out Binary_Tree; E : in Element);

   procedure Duplicate (Src : in Binary_Tree; Dest : in out Binary_Tree);

private

   type Cell is record
      Left, Right : Binary_Tree;
      Info        : Element;
   end record;

end Binary_Trees;
