------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Text_IO; use Text_IO;

package body Discrete_Util is

   package Integer_Io is new Text_IO.Integer_IO (Discrete_Element);

   function Format (F : in Discrete_Element) return Unbounded_String is
      S      : String (1 .. 10000);
      Result : Unbounded_String;
   begin
      Integer_Io.Put (S, F, 10);

      -- Suppress blank
      --
      for I in  1 .. S'Length loop
         if S (I) /= ' ' then
            Append (Result, S (I));
         end if;
      end loop;

      return (Result);
   end Format;

   function Gcd
     (Left  : Discrete_Element;
      Right : Discrete_Element)
      return  Discrete_Element
   is
      Min       : Discrete_Element := Discrete_Element'Min (Left, Right);
      Max       : Discrete_Element := Discrete_Element'Max (Left, Right);
      Remainder : Discrete_Element;
   begin
      Reduce : loop
         if Min <= 0 then
            return Max;
         end if;

         Remainder := Max rem Min;
         Max       := Min;
         Min       := Remainder;
      end loop Reduce;
   end Gcd;

   function Lcm
     (A    : in Discrete_Element;
      B    : in Discrete_Element)
      return Discrete_Element
   is
      Tmp    : Double := Double (A) * Double (B) / Double (Gcd (A, B));
      Result : Discrete_Element;
   begin
      Tmp    := Double'Floor (Tmp);
      Result := Discrete_Element (Tmp);

      return Result;
   end Lcm;

   function Lcm
     (A    : in Discrete_Element;
      B    : in Discrete_Element)
      return Double
   is
      Tmp : constant Double :=
         Double (A) * Double (B) / Double (Gcd (A, B));

   begin
      return Double'Floor (Tmp);
   end Lcm;

end Discrete_Util;
