------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Text_IO; use Text_IO;
with Real_Util;

package body Double_Util is

   package Double_Io is new Text_IO.Float_IO (Double);
   package Double_Real is new Real_Util (Double);

   function Format
     (F     : in Double;
      After : in Integer := 5)
      return  Unbounded_String
   is
   begin
      return Double_Real.Format (F, After);
   end Format;

   procedure Initialize (F : out Double) is
   begin
      F := 0.0;
   end Initialize;

   procedure Put (F : in Double) is
   begin
      Double_Io.Put (F, Exp => 5);
   end Put;

   procedure Put (F : in Double_Ptr) is
   begin
      Double_Io.Put (F.all, Exp => 5);
   end Put;

   procedure Put_Line (F : in Double) is
   begin
      Double_Io.Put (F, Exp => 5);
      New_Line;
   end Put_Line;

   function Equal
     (Num1, Num2 : in Double;
      Accuracy   : in Integer := 10000)
      return       Boolean
   is
   begin
      return Double_Real.Equal (Num1, Num2, Accuracy);
   end Equal;

   function My_Mod (A : Double; B : Double) return Double is
      Result : Double;
   begin
      Result := A - Double'Floor (A / B) * B;
      return (Result);
   end My_Mod;

end Double_Util;
