------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Text_IO; use Text_IO;

package body Fifos is

   procedure Reset (My_Fifo : in out Fifo) is
   begin
      My_Fifo.Queue := 0;
      My_Fifo.Head  := 0;
      My_Fifo.Size  := 0;
   end Reset;

   procedure Put (My_Fifo : in Fifo) is

      Index : Element_Range := My_Fifo.Queue;

   begin
      Put_Line
        ("Fifo : queue = " &
         My_Fifo.Queue'Img &
         "; head = " &
         My_Fifo.Head'Img &
         "; size = " &
         My_Fifo.Size'Img);

      while (Index /= My_Fifo.Head) loop
         Put (My_Fifo.Data (Index));
         New_Line;

         if (Index = Element_Range'Last) then
            Index := Element_Range'First;
         else
            Index := Index + 1;
         end if;
      end loop;

      New_Line;
      New_Line;

   end Put;

   procedure Insert (My_Fifo : in out Fifo; E : in Element) is

      Tmp : Element_Range;
   begin
      if (My_Fifo.Head = Element_Range'Last) then
         Tmp := Element_Range'First;
      else
         Tmp := My_Fifo.Head + 1;
      end if;
      if (My_Fifo.Queue = Tmp) then
         raise Full_Fifo;
      end if;

      My_Fifo.Data (My_Fifo.Head) := E;
      if (My_Fifo.Head = Element_Range'Last) then
         My_Fifo.Head := Element_Range'First;
      else
         My_Fifo.Head := My_Fifo.Head + 1;
      end if;

      My_Fifo.Size := My_Fifo.Size + 1;
   end Insert;

   procedure Extract (My_Fifo : in out Fifo; E : in out Element) is
   begin
      if Is_Empty (My_Fifo) then
         raise Empty_Fifo;
      end if;

      E := My_Fifo.Data (My_Fifo.Queue);

      if (My_Fifo.Queue = Element_Range'Last) then
         My_Fifo.Queue := Element_Range'First;
      else
         My_Fifo.Queue := My_Fifo.Queue + 1;
      end if;

      My_Fifo.Size := My_Fifo.Size - 1;
   end Extract;

   procedure Extract_Any_Where (My_Fifo : in out Fifo; E : in Element) is

      Index : Element_Range := My_Fifo.Queue;

   begin
      if Is_Empty (My_Fifo) then
         raise Empty_Fifo;
      end if;

      -- Looking for the element
      --
      while (Index /= My_Fifo.Head) loop

         if E = My_Fifo.Data (Index) then
            while Index /= My_Fifo.Head loop
               if Index = Element_Range'Last then
                  My_Fifo.Data (Index) := My_Fifo.Data (Element_Range'First);
               else
                  My_Fifo.Data (Index) := My_Fifo.Data (Index + 1);
               end if;

               if (Index = Element_Range'Last) then
                  Index := Element_Range'First;
               else
                  Index := Index + 1;
               end if;
            end loop;

            if (My_Fifo.Head = Element_Range'First) then
               My_Fifo.Head := Element_Range'Last;
            else
               My_Fifo.Head := My_Fifo.Head - 1;
            end if;
            My_Fifo.Size := My_Fifo.Size - 1;
            return;
         end if;

         if (Index = Element_Range'Last) then
            Index := Element_Range'First;
         else
            Index := Index + 1;
         end if;
      end loop;

      raise Element_Not_Found;

   end Extract_Any_Where;

   function Consult (My_Fifo : in Fifo) return Element is
   begin
      if Is_Empty (My_Fifo) then
         raise Empty_Fifo;
      end if;

      return (My_Fifo.Data (My_Fifo.Queue));
   end Consult;

   function Is_Empty (My_Fifo : in Fifo) return Boolean is
   begin
      return (My_Fifo.Queue = My_Fifo.Head);
   end Is_Empty;

   function Get_Size (My_Fifo : in Fifo) return Natural is
   begin
      return (My_Fifo.Size);
   end Get_Size;

end Fifos;
