------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;

generic
   Size : Positive := 100;
   type Data_Type is private;

   with function Compute_Hash (Index : in Unbounded_String) return Natural;
   with procedure Put (D : Data_Type);

package Hashs is

   type Hash is private;

   procedure Initialize (H : in out Hash);

   procedure Add
     (H     : in out Hash;
      Index : in Unbounded_String;
      D     : in Data_Type);

   function Consult
     (H     : in Hash;
      Index : in Unbounded_String)
      return  Data_Type;

   procedure Put (H : in Hash);

   Data_Not_Found : exception;

private

   type Cell;
   type Link is access Cell;

   type Cell is record
      Next   : Link;
      Index  : Unbounded_String;
      A_Data : Data_Type;
   end record;

   type Hash is array (1 .. Size) of Link;

end Hashs;
