------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Unchecked_Deallocation;

generic
   type Element is private;
   type Index is private;

   Size : Natural := 100;
   First : Natural := 0;
   with procedure Put (E : in Element);
   with procedure Initialize (E : out Element);
   with procedure Put (E : in Index);

package Indexed_Tables is

   -- This exception is raised when adding a new item
   -- in the table will conduct to have Ada check bound exception
   --
   Indexed_Table_Full : exception;

   -- Do not change range 0..first+size
   -- intro first..first+size : it's a tips to
   -- avoid constraint error with nb_entries
   -- define by a table_range (and not a natural)
   --
   type Indexed_Table_Range is new Natural range 0 .. First + Size;
   First_Range : Indexed_Table_Range := Indexed_Table_Range (First);

   type Item is record
      Data : Element;
      Item : Index;
   end record;

   type Data_Table is array (Indexed_Table_Range) of Item;

   type Indexed_Table is record
      Nb_Entries : Indexed_Table_Range := 0;
      Entries    : Data_Table;
   end record;

   type Indexed_Table_Ptr is access Indexed_Table;

   procedure Put (A_Item : in Item);

   procedure Put (A_Table : in Indexed_Table);

   procedure Initialize (A_Table : out Indexed_Table);

   procedure Delete
     (A_Table : in out Indexed_Table;
      A_Item  : Indexed_Table_Range);

   procedure Add
     (A_Table   : in out Indexed_Table;
      A_Index   : Index;
      A_Element : Element);

   procedure Free is new Unchecked_Deallocation (
      Indexed_Table,
      Indexed_Table_Ptr);

   pragma Inline (Add);
   pragma Inline (Delete);

end Indexed_Tables;
