------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

generic
   type Element is private;
   with procedure Put (E : in Element);
   with procedure Free (E : in out Element);

package Lists is

   type Cell is private;
   type List is private;

   type Iterator is private;

   Element_Not_Found : exception;

   procedure Initialize (L : in out List);

   procedure Add (L : in out List; E : in Element);

   procedure Put (L : in List);

   -- Remove from "L" the element "E"
   --
   procedure Delete (L : in out List; E : in Element);

   -- Delete from "L" all elements stored into "E"
   --
   procedure Delete (L : in out List; E : in List);

   -- Delete  all elements of the list
   --
   procedure Delete (L : in out List);

   procedure Duplicate (Src : in List; Dest : in out List);

   function Is_Empty (L : in List) return Boolean;

   function Get_Number_Of_Elements (L : in List) return Natural;

   function Get_Head (L : in List) return Element;

   function Get_Tail (L : in List) return Element;

   -- Move the current iterator to the next or previous element
   --
   procedure Next_Element (L : in List; My_Iterator : in out Iterator);
   procedure Previous_Element (L : in List; My_Iterator : in out Iterator);

   -- Get the current element pointed by the iterator
   --
   procedure Current_Element
     (L              : in List;
      Return_Element : out Element;
      My_Iterator    : in Iterator);

   -- Initiliaze an iterator the the head or the tail of a list
   --
   procedure Reset_Tail_Iterator
     (L           : in List;
      My_Iterator : in out Iterator);
   procedure Reset_Head_Iterator
     (L           : in List;
      My_Iterator : in out Iterator);

   -- the element is the tail of the head of a list ?
   --
   function Is_Tail_Element
     (L           : in List;
      My_Iterator : in Iterator)
      return        Boolean;
   function Is_Head_Element
     (L           : in List;
      My_Iterator : in Iterator)
      return        Boolean;

private

   type Cell_Ptr is access Cell;

   type Cell is record
      Next     : Cell_Ptr;
      Previous : Cell_Ptr;
      Info     : Element;
   end record;

   type List is record
      Head : Cell_Ptr;
      Tail : Cell_Ptr;
   end record;

   type Iterator is record
      Current : Cell_Ptr;
   end record;

end Lists;
