------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Text_IO; use Text_IO;
with Discrete_Util;

package body Natural_Util is

   package Natural_Io is new Text_IO.Integer_IO (Natural);
   package Natural_Discrete is new Discrete_Util (Natural);

   function Format (F : in Natural) return Unbounded_String is
   begin
      return (Natural_Discrete.Format (F));
   end Format;

   procedure Put (E : in Natural_Ptr) is
   begin
      Natural_Io.Put (E.all);
   end Put;

   procedure Put (E : in Natural) is
   begin
      Natural_Io.Put (E);
   end Put;

   procedure Put_Line (E : in Natural) is
   begin
      Natural_Io.Put (E);
      New_Line;
   end Put_Line;

   procedure Initialize (A : out Natural) is
   begin
      A := 0;
   end Initialize;

   function Gcd (A : in Natural; B : in Natural) return Natural is
   begin
      return (Natural_Discrete.Gcd (A, B));
   end Gcd;

   function Lcm (A : in Natural; B : in Natural) return Natural is
   begin
      return (Natural_Discrete.Lcm (A, B));
   end Lcm;

   function Lcm (A : in Natural; B : in Natural) return Double is
   begin
      return (Natural_Discrete.Lcm (A, B));
   end Lcm;

end Natural_Util;
