------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Strings.Unbounded;  use Ada.Strings.Unbounded;
with Ada.Numerics.Aux;       use Ada.Numerics.Aux;
with Lists;
with Unchecked_Deallocation;

package Natural_Util is

   type Natural_Ptr is access Natural;

   -- Convert and format a natural to
   -- an unbounded_string
   --
   function Format (F : in Natural) return Unbounded_String;

   -- Display a natural
   --
   procedure Put (E : in Natural);
   procedure Put (E : in Natural_Ptr);
   procedure Put_Line (E : in Natural);

   -- Initialize a natural
   --
   procedure Initialize (A : out Natural);

   -- Compute Great common Divisor
   -- of two natural
   --
   function Gcd (A : in Natural; B : in Natural) return Natural;

   -- Compute Last common multiplier
   -- of two natural
   --
   function Lcm (A : in Natural; B : in Natural) return Natural;
   function Lcm (A : in Natural; B : in Natural) return Double;

   procedure Free is new Unchecked_Deallocation (Natural, Natural_Ptr);

   package Natural_List is new Lists (Natural_Ptr, Put, Free);
   use Natural_List;

end Natural_Util;
