------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Text_IO; use Text_IO;

package body Stacks is

   procedure Reset (A_Stack : in out Stack) is

   begin
      A_Stack.First_Free := 0;
   end Reset;

   function Get_Size (My_Stack : in Stack) return Element_Range is
   begin
      return My_Stack.First_Free;
   end Get_Size;

   -- Is my_stack empty ?
   --
   function Is_Empty (My_Stack : in Stack) return Boolean is
   begin
      return My_Stack.First_Free = 0;
   end Is_Empty;

   -- Put the content of the stack in the
   -- screen
   --
   procedure Put (My_Stack : in Stack) is

   begin
      Put_Line ("Number of elements : " & My_Stack.First_Free'Img);
      for I in  0 .. My_Stack.First_Free - 1 loop
         Put (My_Stack.Data (I));
      end loop;
      New_Line;
   end Put;

   -- Get the last inserted element
   --
   function Get_Top (My_Stack : in Stack) return Element is
   begin
      if My_Stack.First_Free = 0 then
         raise Empty_Stack;
      end if;
      return My_Stack.Data (My_Stack.First_Free - 1);
   end Get_Top;

   -- Push a new element
   --
   procedure Push (My_Stack : in out Stack; A_Element : in Element) is
   begin
      if My_Stack.First_Free = Element_Range'Last then
         raise Full_Stack;
      end if;
      My_Stack.Data (My_Stack.First_Free) := A_Element;
      My_Stack.First_Free                 := My_Stack.First_Free + 1;

   end Push;

   -- Pop a element
   --
   procedure Pop (My_Stack : in out Stack; A_Element : in out Element) is

   begin
      if My_Stack.First_Free = 0 then
         raise Empty_Stack;
      end if;
      A_Element           := My_Stack.Data (My_Stack.First_Free - 1);
      My_Stack.First_Free := My_Stack.First_Free - 1;
   end Pop;

end Stacks;
