------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

generic
   Max_Element : Natural := 100;
   type Element is private;
   with procedure Put (A_Element : in Element);

package Stacks is

   type Stack is tagged private;
   type Element_Range is new Natural range 0 .. Max_Element;
   type Stack_Ptr is access all Stack'Class;

   Empty_Stack, Full_Stack : exception;

   -- Clear the stack
   --
   procedure Reset (A_Stack : in out Stack);

   -- Get the number of elements currently present
   -- in the stack
   --
   function Get_Size (My_Stack : in Stack) return Element_Range;

   -- Is my_stack empty ?
   --
   function Is_Empty (My_Stack : in Stack) return Boolean;

   -- Put the content of the stack in the
   -- screen
   --
   procedure Put (My_Stack : in Stack);

   -- Get the last inserted element
   --
   function Get_Top (My_Stack : in Stack) return Element;

   -- Push a new element
   --
   procedure Push (My_Stack : in out Stack; A_Element : in Element);

   -- Pop a element
   --
   procedure Pop (My_Stack : in out Stack; A_Element : in out Element);

private

   type Element_Table is array (Element_Range) of Element;

   type Stack is tagged record
      First_Free : Element_Range := 0;
      Data       : Element_Table;
   end record;

end Stacks;
