------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

generic
   type Element is private;

   Size : Natural := 100;
   with procedure Put (E : in Element);

package Tables is

   type Table_Range is new Natural range 0 .. Size;

   type Data_Table is array (Table_Range) of Element;

   type Table is record
      Nb_Entries : Table_Range := 0;
      Entries    : Data_Table;
   end record;

   procedure Put (A_Table : in Table);

   procedure Initialize (A_Table : out Table);

   procedure Delete (A_Table : in out Table; A_Item : Table_Range);

   procedure Add (A_Table : in out Table; A_Element : Element);

   pragma Inline (Add);
   pragma Inline (Delete);
   pragma Inline (Put);
   pragma Inline (Initialize);

end Tables;
