------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Unbounded_Strings;     use Unbounded_Strings;

package Manual is

   Ustr_File_Menu : constant Unbounded_String :=
      To_Unbounded_String ("File Menu") & Unbounded_Lf & Unbounded_Lf;

   Ustr_File_Menu_Content : constant Unbounded_String :=
      To_Unbounded_String ("1. New sub-menu : create a new XML project.") &
      Unbounded_Lf &
      To_Unbounded_String
         ("2. Open sub-menu : load a XML project file into the editor.") &
      Unbounded_Lf &
      To_Unbounded_String(
"3. Save sub-menu : save the current XML project into a file with the current XML project file name."
) &
      Unbounded_Lf &
      To_Unbounded_String(
"4. Save as sub-menu : save the current XML project into a file with a new XML project file name."
) &
      Unbounded_Lf &
      To_Unbounded_String ("5. AADL sub-menu : ") &
      Unbounded_Lf &
      To_Unbounded_String
         ("   5.1 AADL import : read an AADL specification into Cheddar") &
      Unbounded_Lf &
      To_Unbounded_String(
"   5.2 AADL export : translate a Cheddar specification towards an AADL specification"
) &
      Unbounded_Lf &
      To_Unbounded_String ("   5.3 Export property sets used by Cheddar") &
      Unbounded_Lf &
      To_Unbounded_String ("   5.4 Export standard AADL property set") &
      Unbounded_Lf &
      To_Unbounded_String ("   5.5 Customize how AADL services work") &
      Unbounded_Lf &
      To_Unbounded_String ("6. Exit sub-menu : Quit the Cheddar's toolset.") &
      Unbounded_Lf &
      Unbounded_Lf;

   Ustr_Edit_Menu : constant Unbounded_String :=
      To_Unbounded_String ("Edit Menu") & Unbounded_Lf & Unbounded_Lf;

   Ustr_Edit_Menu_Content : constant Unbounded_String :=
      To_Unbounded_String(
"1. Add sub-menu : create a new object in the current XML project. Object can be a processor, a task, a message, a buffer, a network or an event analyzer."
) &
      Unbounded_Lf &
      To_Unbounded_String(
"2. Delete sub-menu : remove an object from the current XML project. Object can be a processor, a task, a message, a buffer, a network or an event analyzer"
) &
      Unbounded_Lf &
      To_Unbounded_String(
"3. Update sub-menu : change parameters of an already defined object in the current XML project. Object can be a processor, a task, a message, a buffer, a network or an event analyzer."
) &
      Unbounded_Lf &
      To_Unbounded_String(
"4. Duplicate sub-menu : copy an objet to a new one in the current XML project. Object can be a processor, a task, a message, a buffer, a network or an event analyzer."
) &
      Unbounded_Lf &
      Unbounded_Lf;

   Ustr_View_Menu : constant Unbounded_String :=
      To_Unbounded_String ("View Menu") & Unbounded_Lf & Unbounded_Lf;

   Ustr_View_Menu_Content : constant Unbounded_String :=
      To_Unbounded_String(
"1. List sub-menu : show a set of object of the current XML project. Set of object can be the processor set, the"
) &
      To_Unbounded_String(
" task set, the message set, the buffer set, the network set or the event analyzer set."
) &
      Unbounded_Lf &
      Unbounded_Lf;

   Ustr_Tools_Menu : constant Unbounded_String :=
      To_Unbounded_String ("Tools Menu") & Unbounded_Lf & Unbounded_Lf;

   Ustr_Tools_Menu_Content : constant Unbounded_String :=
      To_Unbounded_String ("") &
      Unbounded_Lf &
      To_Unbounded_String(
"1. Clear work space sub-menu : clean the working area (main window). Do not change anything on the project itself. "
) &
      Unbounded_Lf &
      To_Unbounded_String ("2. Scheduling sub-menu : ") &
      Unbounded_Lf &
      To_Unbounded_String(
"   2.1 Scheduling simulation sub-menu : compute and draw scheduling simulation. This sub-menu allows the user to customize the way the scheduling is computed."
) &
      Unbounded_Lf &
      To_Unbounded_String(
"   2.2 Scheduling feasibility sub-menu : compute some basics feasibility tests on all processors. Feasibility test computed there are utilization factor test and response time test."
) &
      Unbounded_Lf &
      To_Unbounded_String(
"   2.3 Set priorities according to Rate Monotonic sub-menu : change priority of the task "
) &
      To_Unbounded_String(
" according to the period (Tasks with the smallest period become tasks with the highest priority)."
) &
      Unbounded_Lf &
      To_Unbounded_String(
"   2.4 Set priorities according to Deadline Monotonic sub-menu : change priority of the task"
) &
      To_Unbounded_String(
" according to the deadline (Tasks with the smallest deadline become tasks with the highest priority).  "
) &
      Unbounded_Lf &
      To_Unbounded_String(
"   2.5 Partition sub-menu : provide some services to assign tasks on a set of processor."
) &
      Unbounded_Lf &
      To_Unbounded_String(
"      2.5.1 With Best Fit sub-sub-menu : assign tasks on the set of processor according to the Best Fit algorithm. "
) &
      Unbounded_Lf &
      To_Unbounded_String(
"      2.5.2 With General Task sub-sub-menu : assign tasks on the set of processor according to the General Task algorithm."
) &
      Unbounded_Lf &
      To_Unbounded_String(
"      2.5.3 With Next Fit sub-sub-menu : assign tasks on the set of processor according to the Next Fit algorithm."
) &
      Unbounded_Lf &
      To_Unbounded_String(
"      2.5.4 With First Fit sub-sub-menu : assign tasks on the set of processor according to the First Fit algorithm."
) &
      Unbounded_Lf &
      To_Unbounded_String(
"      2.5.5 With Small Task sub-sub-menu : assign tasks on the set of processor according to the Small Task algorithm."
) &
      Unbounded_Lf &
      To_Unbounded_String(
"   2.6 Event table services sub-menu : provide some basic services on event tables."
) &
      Unbounded_Lf &
      To_Unbounded_String(
"      2.6.1 Compute scheduling and generate event table sub-sub-menu : compute the scheduling and produce the event table."
) &
      Unbounded_Lf &
      To_Unbounded_String(
"      2.6.2 Draw time line from event table sub-sub-menu : draw time line from the last computed or loaded scheduling/event table."
) &
      Unbounded_Lf &
      To_Unbounded_String(
"      2.6.3 Run analysis on event table sub-sub-menu : perform analysis on the last computed or loaded scheduling/event table."
) &
      Unbounded_Lf &
      To_Unbounded_String(
"      2.6.4 Export event table sub-sub-menu : save the last scheduling/event table on a file in a XML format."
) &
      Unbounded_Lf &
      To_Unbounded_String(
"   2.7 Options sub-menu : describe how the scheduling simulation will be done."
) &
      Unbounded_Lf &
      To_Unbounded_String ("3. Resource sub-menu : from there, you ") &
      Unbounded_Lf &
      To_Unbounded_String(
"   3.1 Bound on blocking time sub-sub-menu : compute bound on shared resources blocking time according to PCP and PIP protocols without computing the scheduling"
) &
      Unbounded_Lf &
      To_Unbounded_String(
"   3.2 Looking for priority inversion from simulation sub-sub-menu : run analysis on a previously computed scheduling to look for hight priority "
) &
      To_Unbounded_String
         (" tasks blocked by lower priority task at shared resource access.") &
      Unbounded_Lf &
      To_Unbounded_String(
"   3.3 Looking for priority inversion from simulation sub-sub-menu : run analysis on a previously computed scheduling to look for tasks blocked for ever on shared reousrces."
) &
      Unbounded_Lf &
      To_Unbounded_String(
"4. Buffer sub-menu : this submenu can help you to study buffers shared by tasks."
) &
      Unbounded_Lf &
      To_Unbounded_String(
"   4.1 Buffer simulation sub-sub-menu : compute buffer utilization factor and message waiting time from a given scheduling simulation. "
) &
      Unbounded_Lf &
      To_Unbounded_String(
"   4.2 Buffer feasibility tests sub-sub-menu : compute bound on buffer utilization factor and essage waiting time without computing scheduling"
) &
      Unbounded_Lf &
      To_Unbounded_String(
"5. Precedency sub-menu : You will find here some heuristics/algorithms that can schedule or check feasibility of a tasks set with dependencies"
) &
      Unbounded_Lf &
      To_Unbounded_String(
"   5.1 Chetto/Blazewicz modifications on priorities sub-sub-menu : This service create independent task set from"
) &
      To_Unbounded_String(
" a dependent task set by a modification oe task priorities according to precedency constraints."
) &
      Unbounded_Lf &
      To_Unbounded_String(
"   5.2 Chetto/Blazewicz modifications on deadlines sub-sub-menu : This service create independent task set from"
) &
      To_Unbounded_String(
" a dependent task set by a modification oe task deadlines according to precedency constraints."
) &
      Unbounded_Lf &
      To_Unbounded_String(
"   5.3 End to End response time : compute response time from a set of task (which have precendency relationship) with the Holistic method."
) &
      Unbounded_Lf &
      To_Unbounded_String(
"6. Random sub-menu : this submenu should provide necessary tools to do simulations with random events."
) &
      Unbounded_Lf &
      To_Unbounded_String(
"   6.1 Compute response time density sub-menu : compute statistic distribution of task response time from a scheduling simulation."
) &
      Unbounded_Lf &
      Unbounded_Lf;

   Ustr_Help_Menu : constant Unbounded_String :=
      To_Unbounded_String ("Help Menu") & Unbounded_Lf & Unbounded_Lf;

   Ustr_Help_Menu_Content : constant Unbounded_String :=
      To_Unbounded_String(
"1. About Cheddar sub-menu : provide version number of the Cheddar's binaries")
      &
      Unbounded_Lf &
      To_Unbounded_String
         ("2. Manual sub-menu : contains the text given in this section") &
      Unbounded_Lf &
      To_Unbounded_String(
"3. Scheduling references sub-menu : gives all paper references used to compute feasibility tests and simulation results."
) &
      Unbounded_Lf &
      Unbounded_Lf;

end Manual;
