------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Unbounded_Strings;     use Unbounded_Strings;

package References is

   -- Define a tabulation (7 spaces)
   --
   Tab : constant Unbounded_String := To_Unbounded_String ("       ");

   -- This variable contains the formatted text of all publicatons
   -- references
   --
   Ref_Text : constant Unbounded_String :=
      Unbounded_Lf &
      To_Unbounded_String ("[1]    Preemptive and Non-Preemptive") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("Real Time Uni-Processor Scheduling.") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("L. George, N. Rivierre and M. Spuri") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("INRIA Research Report number 2966.") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("September 1996.") &
      Unbounded_Lf &
      Unbounded_Lf &
      To_Unbounded_String ("[2]    Holistic Schedulability Analysis for ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("Distributed Hard Real-Time Systems.") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("K. Tindell and J. Clark") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("Microprocessing and Microprogramming ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("40:117-134. 1994.") &
      Unbounded_Lf &
      Unbounded_Lf &
      To_Unbounded_String ("[3]    Priority Inheritance Protocols : An ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("Approach to Real Time Synchronization.") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("L. Sha, R. Rajkumar and J.P. Lehoczky") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("IEEE Transactions on computers,") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("39(9):1175-1185. 1990.") &
      Unbounded_Lf &
      Unbounded_Lf &
      To_Unbounded_String ("[4]    About Bounds of Buffers Shared by") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("Periodic Tasks : the IRMA project.") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("J. Legrand, F. Singhoff, L Nana, ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("L. Marce, F. Dupont and H. Hafidi") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("In the 15th IEEE Euromicro, WIP") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("Real Time Systems Conference, 2003.") &
      Unbounded_Lf &
      Unbounded_Lf &
      To_Unbounded_String ("[5]    Scheduling dependant tasks with") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("Different Arrival Times to Meet Deadlines.") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("J. Blazewicz") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("In. Gelende H. Beilner (eds), ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("Modeling and Performance Evaluation") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("of Computer Systems,. Amsterdam 1976") &
      Unbounded_Lf &
      Unbounded_Lf &
      To_Unbounded_String ("[6]    Dynamic Scheduling of Real Time") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("Tasks Under Precedence Constraints.") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("H. Chetto, M. Silly and T. Bouchentouf") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("RTS International Journal of Time") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("Critical Computing Systems. 2(3):181-194,") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("September 1990.") &
      Unbounded_Lf &
      Unbounded_Lf &
      To_Unbounded_String
         ("[7]    Scheduling Algorithms for multi-Programming") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("in a hard-real-time envirronment.") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("C. L. Liu and J. W. Layland ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("Journal of the ACM, 20 (1):46--61,1973.") &
      Unbounded_Lf &
      Unbounded_Lf &
      To_Unbounded_String
         ("[8]    On non preemptive Scheduling of periodic and") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("Sporadic Tasks") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("K. Jeffay, D.Stanat, C. Martel ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("in Proc. Of IEEE Real Time Systems Symposium") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("San Antonio, Texas, December 1991") &
      Unbounded_Lf &
      Unbounded_Lf &
      To_Unbounded_String ("[9]    Assigning Real-Time Tasks to Homogeneous") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("Multiprocessor Systems .") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("A. Burchard, Y. Oh. Liebeherr, and S. H. Son ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("Technical Report CS-94-01, ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("University of Virginia, ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("1994 Department of Computer Science. ") &
      Unbounded_Lf &
      Unbounded_Lf &
      To_Unbounded_String ("[10]   Tight Performance Bounds of Heuristics ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("for a Real-Time Scheduling Problem.") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("Y. Oh and S. H. Son. ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("Technical Report CS93-24, ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("University of Virginia, ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("1993 Department of Computer Science.") &
      Unbounded_Lf &
      Unbounded_Lf &
      To_Unbounded_String ("[11]   Performance Analysis of Buffers Shared ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("by Independent Periodic Tasks.") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("J. Legrand, F. Singhoff, L Nana") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("and L. Marce") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("LYSIC Technical report number legrand-02-2004, ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("University of Brest, ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("2005 Department of Computer Science.") &
      Unbounded_Lf &
      Unbounded_Lf &
      To_Unbounded_String ("[12]   Queueing systems. Volume I : Theory ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("L. Kleinrock") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("A Wiley interscience publication.") &
      Unbounded_Lf &
      Unbounded_Lf &
      To_Unbounded_String ("[13]   Analyse de tampons partags ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("par des taches priodiques et indpendantes.") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("J. Legrand, F. Singhoff, L Nana") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("and L. Marce") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("LYSIC Technical report number legrand-01-2005, ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("University of Brest, ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("2005 Department of Computer Science.") &
      Unbounded_Lf &
      Unbounded_Lf &
      To_Unbounded_String ("[14]   Extending Rate Monotonic Analysis ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("when Tasks Share Buffers.") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("F. Singhoff, J. Legrand,  L Nana ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("and L. Marce") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("In the DAta Systems in Aerospace conference") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("(DASIA'2004), ESA Publication Division") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String (" - Nice (France), 2004.") &
      Unbounded_Lf &
      Unbounded_Lf &
      To_Unbounded_String ("[15]   Cheddar : a Flexible Real Time") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("Scheduling Framework. ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("F. Singhoff, J. Legrand,  L Nana ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("and L. Marce") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("In the ACM SIGADA Conference, Atlanta,") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("15-18 November, 2004.") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("Also published in the SIGADA Ada Letters.") &
      Unbounded_Lf &
      Unbounded_Lf &
      To_Unbounded_String ("[16]   Scheduling and Memory requirement") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("analysis with AADL.") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("F. Singhoff, J. Legrand,  L Nana ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("and L. Marce") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("In the ACM SIGADA Conference, Atlanta,") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("November, 2005.") &
      Unbounded_Lf &
      Unbounded_Lf &
      To_Unbounded_String
         ("[17]   Faisabilit d'une application de supervision : ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("le projet IRMA.") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("J. Legrand, F. Singhoff, L. Nana ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("and L. Marce") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("11th Conference RTS embedded systems 2003,") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("Paris (France). April 2003.") &
      Unbounded_Lf &
      Unbounded_Lf &
      To_Unbounded_String ("[18]   Limitation of the Hyper-Period in ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("Real-Time Periodic Task Set Generation.") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("J. Goossens and C. Macq") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("9th Conference RTS embedded systems 2001,") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("Paris (France). Page 133-148.") &
      Unbounded_Lf &
      Unbounded_Lf &
      To_Unbounded_String ("[19]   Rate Monotonic vs. EDF: Judgment Day.") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("G. Buttazzo") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("Real-Time Systems, Vol. 29, ") &
      Unbounded_Lf &
      Tab &
      To_Unbounded_String ("Issue 1, pp. 5-26, January 2005.") &
      Unbounded_Lf &
      Unbounded_Lf &
      Unbounded_Lf &
      Unbounded_Lf &
      Unbounded_Lf;

end References;
