------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2009   Frank Singhoff
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Framework_Config;      use Framework_Config;

package Translate is

   procedure Initialize;

   type Labels is array (Languages'Range) of Unbounded_String;

   ----------------------------------------------------
   -- Labels common on every languages
   ----------------------------------------------------

   Lb_Dot, Lb_Colon, Lb_Minus, Lb_Equal, Lb_Comma, Lb_Double_Quote :
     Unbounded_String;

   ----------------------------------------------------
   --   Menu and sub menu titles
   ----------------------------------------------------
   Lb_Root_Title, Lb_File, Lb_New, Lb_Open, Lb_Save, Lb_Save_As, Lb_Open_Xml, 
Lb_Save_Xml, Lb_Save_As_Xml, Lb_Edit_Scheduler, Lb_Parametric, Lb_Aadl, 
Lb_Import_Aadl, Lb_Export_Aadl, Lb_Aadl_Options, Lb_Other_Aadl_Options, 
Lb_Export_Cheddar_Property_Sets, Lb_Export_Aadl_Standard_Properties_Set, 
Lb_Exit, Lb_Edit, Lb_View, Lb_Tools, Lb_Compute_Buffer, Lb_Draw_Buffer, 
Lb_Clear, Lb_Scheduling_Simulation, Lb_Scheduling_Feasibility, 
Lb_Customized_Scheduling_Simulation, Lb_Customized_Feasibility_Tests, 
Lb_Compute_And_Display, Lb_Compute_And_Update, Lb_Compute_And_Display_One_Step,
Lb_Compute_And_Update_One_Step, Lb_Memory_Requirement_Analysis, 
Lb_Check_And_Update_Address_Space, Lb_Check_Address_Space, Lb_Help, Lb_Manual,
Lb_Ref, Lb_About, Lb_About_Text : Labels;

   ----------------------------------------------------
   --   Add, delete, display, list and update
   ----------------------------------------------------
   Lb_Update, Lb_Update_Task, Lb_Update_Address_Space, Lb_Update_Event_Analyzer
, Lb_Update_Message, Lb_Update_Buffer, Lb_Update_Processor, Lb_Update_Network,
Lb_Update_Resource : Labels;

   Lb_Add, Lb_Add_Task, Lb_Add_Address_Space, Lb_Add_Event_Analyzer, 
Lb_Add_Processor, Lb_Add_Buffer, Lb_Add_Message, Lb_Add_Resource, 
Lb_Add_Network : Labels;

   Lb_List, Lb_Tasks_List, Lb_Address_Spaces_List, Lb_Event_Analyzers_List, 
Lb_Processors_List, Lb_Resources_List, Lb_Messages_List, Lb_Buffers_List :
     Labels;

   Lb_Delete, Lb_Delete_Task, Lb_Delete_Address_Space, Lb_Delete_Event_Analyzer
, Lb_Delete_Processor, Lb_Delete_Resource, Lb_Delete_Message, Lb_Delete_Buffer:
     Labels;

   ------------------------------------------------------------
   --   Errors on messages/tasks/processors/resources/buffers/networks
   ------------------------------------------------------------
   Lb_Dependency_Not_Found, Lb_Resource_Not_Found, Lb_Message_Not_Found, 
Lb_Task_Not_Found, Lb_Address_Space_Not_Found, Lb_Processor_Not_Found, 
Lb_Pb_Strict_Constraints, Lb_Pb_Sched_Unknown, Lb_Buffer_Not_Found, 
Lb_Event_Analyzer_Not_Found : Labels;

   Lb_Define_Tasks_Before, Lb_Define_Address_Spaces_Before, 
Lb_Define_Processors_Before, Lb_Define_Messages_Before, 
Lb_Define_Resources_Before, Lb_Define_Buffers_Before, 
Lb_Define_Event_Analyzers_Before : Labels;

   Lb_Can_Not_Define_More_Networks, Lb_Can_Not_Define_More_Tasks, 
Lb_Can_Not_Define_More_Address_Spaces, Lb_Can_Not_Define_More_Processors, 
Lb_Can_Not_Define_More_Resources, Lb_Can_Not_Define_More_Buffers, 
Lb_Can_Not_Define_More_Messages, Lb_Can_Not_Define_More_Event_Analyzers :
     Labels;

   ----------------------------------------------------
   --    I/O Errors and messages
   ----------------------------------------------------
   Lb_Open_File, Lb_Save_File, Lb_Not_Saved_File, Lb_Can_Not_Open_File, 
Lb_Can_Not_Read_File, Lb_Can_Not_Save_File, Lb_File_Name : Labels;

   Lb_Open_Project, Lb_Save_Project, Lb_Not_Saved_Project, 
Lb_Can_Not_Open_Project, Lb_Can_Not_Save_Project, Lb_Select_File : Labels;

   ----------------------------------------------------
   --   Widget text fields
   ----------------------------------------------------
   --
   --  Event analyzer widgets
   --
   Lb_Event_Analyzer_Name, Lb_Event_Analyzer, Lb_Select_Event_Analyzer_First, 
Lb_Run_Event_Analyzers : Labels;

   --
   --  Processor widgets
   --
   Lb_Parametric_File_Name, Lb_File_Name_Control, Lb_Only_On_Parametric, 
Lb_Select_Processor_First, Lb_Read, Lb_Quantum_Control, Lb_Processor, 
Lb_Processor_Name, Lb_Scheduler, Lb_Option, Lb_Preemptive, Lb_Not_Preemptive, 
Lb_Invalid_Identifier, Lb_Variable_Size : Labels;

   --
   --  Tasks widgets
   --
   Lb_Task_Name, Lb_Task, Lb_Task_Type, Lb_Capacity, Lb_Criticality, 
Lb_Deadline, Lb_Period, Lb_Jitter, Lb_Priority, Lb_Policy, Lb_Policy_Control, 
Lb_Start_Time, Lb_Seed, Lb_Predictable, Lb_Unpredictable, Lb_Offset_Val, 
Lb_Offset_Act, Lb_Offsets, Lb_Activations, Lb_Activation, Lb_Values, Lb_Value,
Lb_User_Defined_Parameter, Lb_Rule_For_Parametric_Only, Lb_Main, Lb_No_Period,
Lb_Require_Period, Lb_Require_Activation_Rule, 
Lb_Have_To_Be_On_The_Same_Processor, Lb_Select_Task_First, Lb_Can_Not_Modify, 
Lb_Task_Not_Permitted, Lb_Activation_Rule : Labels;

   --
   --  Address space widgets
   --
   Lb_Address_Space_Name, Lb_Select_Address_Space_First, Lb_Address_Space, 
Lb_Invalid_Scheduler, Lb_Text, Lb_Heap, Lb_Data, Lb_Stack, Lb_Text_Memory_Size,
Lb_Heap_Memory_Size, Lb_Data_Memory_Size, Lb_Stack_Memory_Size : Labels;

   --
   --  Message widgets
   --
   Lb_Message_Type, Lb_Message, Lb_Response_Time_Message, 
Lb_Communication_Time_Message, Lb_Message_Name : Labels;

   --
   -- Network widgets
   --
   Lb_Only_On_Parametric_Network, Lb_Network, Lb_Networks_Name, 
Lb_Networks_List, Lb_View_Networks_Type, Lb_Define_Networks_Before, 
Lb_Delete_Network, Lb_Network_Not_Found, Lb_Networks_Type : Labels;

   --
   --  Buffer widgets
   --
   Lb_Buffer, Lb_Buffer_Name, Lb_Size, Lb_Time, Lb_Select_Buffer_First, 
Lb_Utilization_Level, Lb_Bound_Buffer, Lb_Role_Type, Lb_Role, Lb_Role_Consumer,
Lb_Role_Producer, Lb_Role_List, Lb_Buffer_List, Lb_Queueing_System, 
Lb_Buffer_Information : Labels;

   --
   --  Resource widgets
   --
   Lb_Resource_Name, Lb_Resource, Lb_Select_Resource_First, 
Lb_Update_Resource_Control, Lb_Used_Protocol, Lb_Resource_Filter, Lb_Cpu_Filter
, Lb_State, Lb_Protocol, Lb_Modify, Lb_Resource_State, Lb_Task_Begin, 
Lb_Task_End, Lb_Resources, Lb_Resource_Affect, Lb_Resource_Information, 
Lb_Invalid_Protocol : Labels;

   --
   --  Dependencies graph widgets
   --
   Lb_New_Buffer_Role, Lb_Disp1, Lb_Disp2, Lb_Disp3, Lb_Create, Lb_Select, 
Lb_Name, Lb_Select_Row_Before, Lb_Precedencies, Lb_Arrow, Lb_New_Item, 
Lb_Graph_View, Lb_Choose_Item : Labels;

   --
   -- AADL options
   --
   Lb_Aadl_Process_Second_Import_Pass, Lb_Aadl_Process_Third_Import_Pass, 
Lb_Import_Aadl_Event_To_Precedencies, Lb_Import_Aadl_Event_Data_To_Precedencies
, Lb_Import_Aadl_Event_To_Buffers_Messages, 
Lb_Import_Aadl_Event_Data_To_Buffers_Messages, Lb_Aadl_Debug, 
Lb_Import_Aadl_System_Name, Lb_Export_Aadl_Precedencies_To_Properties, 
Lb_Export_Aadl_Precedencies_To_Event, Lb_Export_Aadl_Precedencies_To_Event_Data
: Labels;

   --
   -- Scheduling and event analyzer options
   --
   Lb_Compute_Scheduling, Lb_Generate_Events, Lb_Scheduling_Options, 
Lb_Display_Event_Table, Lb_Automatically_Export_Event_Table, 
Lb_Event_Table_File_Name, Lb_Run_Simulation_On_One_Processor_Only, 
Lb_Run_Feasibility_Tests_On_One_Processor_Only, Lb_Schedule_According_To :
     Labels;

   --
   --  Event table services widgets
   --
   Lb_Event_Table_Services, Lb_Compute_Scheduling_And_Generate_Event_Table, 
Lb_Run_Analysis_On_Event_Table, Lb_Draw_Time_Line_From_Event_Table, 
Lb_Import_Event_Table, Lb_Export_Event_Table : Labels;

   --
   -- Seed options
   --
   Lb_Seed_Options, Lb_Task_Specific, Lb_All_Predictable, Lb_All_Unpredictable:
     Labels;

   --
   -- Duplicate widget
   --
   Lb_Duplicate, Lb_Old, Lb_Object : Labels;

   --
   -- Select time line widget
   --
   Lb_Select_Time_Line, Lb_To_Display, Lb_Can_Not_Display_So_Much_Time_Line, 
Lb_Select_At_Least_One_Time_Line, Lb_Type : Labels;

   --
   -- Others ...
   --
   Lb_Advanced, Lb_Cancel, Lb_Warning, Lb_See, Lb_Equation, Lb_Theorem, 
Lb_Inferior, Lb_Superior, Lb_Lemma, Lb_Close, Lb_And, Lb_Continue : Labels;

   ----------------------------------------------------
   --   Data controls
   ----------------------------------------------------
   Lb_Invalid_Priority, Lb_Mandatory, Lb_Already_Defined, Lb_Must_Be, 
Lb_Must_Be_Numeric, Lb_Parameter : Labels;

   ----------------------------------------------------
   --   Parametric scheduler messages
   ----------------------------------------------------
   --
   -- Parser and parametric simulation errors
   --
   Lb_Type_Error, Lb_Parametric_File_Error, Lb_Statement_Error, 
Lb_Variable_Error, Lb_Syntax_Error, Lb_Identifier_Already_Exists, 
Lb_Variable_Declaration_Have_To_Be_Done_In_Start_Section, 
Lb_Undeclared_Identifier, Lb_Identifier, Lb_Integer_Conversion_Error, 
Lb_Double_Conversion_Error, Lb_Index_Error, Lb_Uncompatible_Type_Error, 
Lb_Can_Not_Edit_To_Parametric_Object, Lb_Identifier_Already_Declared, 
Lb_Operation_Not_Permitted : Labels;

   ----------------------------------------------------
   --   Logic messages/errors (scheduling, buffer
   --           and blocking time)
   ----------------------------------------------------
   --
   -- Shared resource logic
   --
   Lb_Compute_Blocking_Error1, Lb_Compute_Blocking_Error2, 
Lb_Compute_Blocking_Error3, Lb_Deadlock_Detection, 
Lb_Priority_Inversion_Detection, Lb_No_Deadlock_Found, 
Lb_No_Priority_Inversion_Found, Lb_Deadlock, Lb_Deadlock_At_Time, 
Lb_Priority_Inversion, Lb_Bound_Blocking_Time, Lb_Has_Priority_Inversion, 
Lb_From_The_Time, Lb_To_The_Time, Lb_Blocking_Time, Lb_Simulation_Deadlock, 
Lb_Simulation_Priority_Inversion, Lb_Simulation_Blocking_Time, 
Lb_Blocking_Time_Inject_Success, Lb_Blocking_Time_Inject_Failed : Labels;

   --
   -- Scheduling logic
   --
   Lb_Select_Almost_One_Feasilibity_Test, Lb_Feasibility_Number1, 
Lb_Feasibility_Number2, Lb_Para, Lb_Edf, Lb_D_Over, Lb_Muf, Lb_Rm, Lb_Dm, 
Lb_Llf, Lb_Hpf, Lb_Worst_Case, Lb_Best_Case, Lb_Average_Case, Lb_Curve, 
Lb_Simulation_Error, Lb_Priorities_Error, Lb_Set_Priorities_Dm, 
Lb_Set_Priorities_Rm, Lb_Set_Priorities, Lb_Unit_Of_Times, Lb_Draw_From, 
Lb_Draw_To, Lb_Schedule_To, Lb_Schedule_Time_Line_Too_Large, 
Lb_Draw_Time_Line_Too_Large, Lb_Periodic_Situation, Lb_Scheduling, 
Lb_Free_Unit_Time, Lb_Utilization_With_Period, Lb_Utilization_With_Deadline, 
Lb_Utilization_Bound1, Lb_Utilization_Bound11, Lb_Sched_Explanation1, 
Lb_Sched_Explanation12, Lb_Sched_Explanation2, Lb_Sched_Explanation22, 
Lb_Sched_Explanation3, Lb_Sched_Explanation5, Lb_Sched_Explanation52, 
Lb_Sched_Explanation6, Lb_Sched_Explanation62, Lb_Sched_Explanation7, 
Lb_Sched_Explanation8, Lb_Sched_Explanation81, Lb_Sched_Explanation82, 
Lb_No_Deadline_Missed_In_The_Computed_Scheduling, 
Lb_No_Deadline_Will_Be_Missed_Task_Are_Schedulable, 
Lb_Deadline_Will_Be_Missed_Task_Are_Not_Schedulable, 
Lb_Bound_On_Task_Response_Time, Lb_Task_Response_Time, 
Lb_Task_Response_Time_From_Simulation, Lb_Scheduling_Period, 
Lb_Number_Of_Preemption, Lb_Number_Of_Context_Switch, Lb_Priorities, 
Lb_Partition_Best_Fit, Lb_Partition_Next_Fit, Lb_Partition_First_Fit, 
Lb_Partition_Small_Task, Lb_Partition_General_Task, Lb_Partition, 
Lb_Partitioning_Result, Lb_Partition_Error1, Lb_Partition_Error2, 
Lb_Partition_Error3, Lb_Compute_Scheduling_Error_1, 
Lb_Compute_Scheduling_Error_2, Lb_Compute_Scheduling_Error_3, 
Lb_Compute_Scheduling_Error_4, Lb_Compute_Scheduling_Error_5, 
Lb_Compute_Scheduling_Error_6, Lb_Compute_Scheduling_Error_7, 
Lb_Compute_Scheduling_Error_8, Lb_Compute_Scheduling_Error_9, 
Lb_Compute_Scheduling_Error_10, Lb_Compute_Scheduling_Error_11, 
Lb_Compute_Scheduling_Error_12, Lb_Compute_Scheduling_Error_13, 
Lb_Compute_Scheduling_Error_14, Lb_Compute_Scheduling_Error_15, 
Lb_Compute_Scheduling_Error_18, Lb_Compute_Scheduling_Error_19, 
Lb_Compute_Scheduling_Error_21, Lb_Compute_Scheduling_Error_22, 
Lb_Check_Deadline, Lb_Periodic, Lb_Aperiodic, Lb_Poisson_Process_Task, 
Lb_Sporadic_Task, Lb_Parametric_Task, Lb_Completion_Time : Labels;

   --
   -- Buffer logic
   --
   Lb_Harmonic, Lb_Compute_Buffer_Error_1, Lb_Compute_Buffer_Error_2, 
Lb_Schedule_Before, Lb_Buffer_Utilization, Lb_Compute_Buffer_Error_3, 
Lb_Compute_Buffer_Error_4, Lb_Maximum_Buffer_Size, Lb_Average_Buffer_Size, 
Lb_Maximum_Waiting_Time, Lb_Average_Waiting_Time : Labels;

   --
   -- Dependencies logic
   --
   Lb_Chetto, Lb_Jitter_From_Response_Time_All_Steps, Lb_Deadlines_Are_Updated,
Lb_Priorities_Are_Updated, Lb_Jitters_Are_Updated, Lb_Chetto_Error1, 
Lb_Chetto_Error2, Lb_Chetto_Error3, Lb_Chetto_Deadline, Lb_Chetto_Priority, 
Lb_Jitter_From_Response_Time_One_Step, Lb_Jitter_From_Response_Time, 
Lb_Precedencies_Period_Error : Labels;

   --
   -- Random tools
   --
   Lb_Random, Lb_Compute_Response_Time_Density, Lb_Response_Time_Density, 
Lb_Response_Time : Labels;

end Translate;
